# R classes generated from the DDI4 xmi.xml file, 2019-09-22T11:38:58.653004

#This work is licensed under a Creative Commons Attribution 4.0 International License.
#https://creativecommons.org/licenses/by/4.0/

library(R6)
library(uuid)
library(XML)
library(RCurl)
#  not needed, was part of initial exploration library(data.table)
library(stringr)


#' @title  DDI4 Expressed as R6 Classes
#' 
#' @description
#' Contains the DDI4 model generated from the XMI file: \url{https://lion.ddialliance.org/xmi.xml}.
#' Each class in the DDI4 model is implemented as an R6 class.
#' @author Larry Hoyle \email{larryhoyle@ku.edu}, Joachim Wackerow \email{joachim.wackerow@gesis.org}
#' @references \url{http://www.ddialliance.org/}
#' @import uuid 
#                   UUIDgenerate 
# - - - - - - - - - -
# preparation


# - - - - - - - - - -
# start included from ddiUtil.R
# - - - - - - - - - -

#' @section Utility package for DDI
#' 
#' @description
#' currently contains only functions for managing DDI URNs
#' 
#' @name DDIUtil
#' @docType package
#' @author Larry Hoyle \email{larryhoyle@ku.edu}, Joachim Wackerow \email{joachim.wackerow@gesis.org}
#' @references \url{http://www.ddialliance.org/}
#'
#' @import RCurl 
#                    getURL 
#' @import stringr 
#                    str_locate_all str_trim
#' @import XML 
#                      catalogResolve 
#'
#'
#
# - - - - - - - - - -
# Configuration for resolution of DDI URNs by XML Catalog file.
# 
#' @export
DefaultXmlCatalogFile <- system.file("extdata","ddiurn_catalog.xml",package="DDI4R")

# - - - - - - - - - -
# load required libraries
library( RCurl )    # for HTTP GET
library( stringr )  # for string handling
library( XML )      # for resolving URNs to URLs by XML Catalog 

# - - - - - - - - - -
# preparation

if ( !file.exists(DefaultXmlCatalogFile) ) {
  stop( "XML Catalog file \"", DefaultXmlCatalogFile, "\" doesn't exist!")
}

DefaultXmlCatalogFile <-  paste("file:///",DefaultXmlCatalogFile, sep="",  collapse=NULL)
# set OS environment variable for XML Catalog
if( !exists( "Sys.setenv" ) ) Sys.setenv = Sys.putenv
Sys.setenv( "XML_CATALOG_FILES" = DefaultXmlCatalogFile )
Sys.getenv()

# initialize parameter for id pattern
idPrefix <- "urn:ddi"

# make the master environment for DDI4R 
ddi.env <- new.env()
# make the environment that allows direct lookup of DDI4 objects from DDIURNs 
#ddi.env.urn <- new.env( parent=ddi.env, hash=TRUE )
#ddi.env.urn <- new.env(hash=TRUE)
ddi.env.urn <- new.env()
# make the environment that allows reverse lookup of DDIURNs from objects that reference them DDIURNs 
#ddi.env.references <- new.env( parent=ddi.env, hash=TRUE )
#ddi.env.references <- new.env(hash=TRUE)
ddi.env.references <- new.env()

# - - - - - - - - - -
# functions

#' @title is.anySentinel - Returns TRUE if any value in a vector is NA or NaN. Also returns TRUE if the length of the vector is 0 or the vector is NULL
#' 
#' @description
#' Returns true when a vector has length 0 or is NULL. Also returns true if any value in  a vector is  NA or NaN. These are sentinel values. 
#' Useful where checking for NA, but the vector might be NULL which would throw an error.
#' 
#' @param testValue The value to check (required)
#' 
#' 
#' @return TRUE if there is a sentinel value in the vector, FALSE otherwise.
#' 
#' @export
#' @examples is.anySentinel(c(NULL,1,2))
#' 

is.anySentinel <- function(testValue){
  (length(testValue) == 0 || any(is.na(testValue))  )
}

#' @title normalizeDdiUrn - Normalize DDI URN
#' 
#' @description
#' Puts DDI URN prefix and DDI agency id into lower case.
#' Removes leading and trailing spaces.
#' 
#' @note 
#' To do: check for valid DDI URN syntax.
#' 
#' @param DdiUrn DDI URN (required).
#' An object of class character of length 1.
#' 
#' @return An object of class character of length 1 containing the normalized DDI URN, or NULL if the URN is invalid.
#' 
#' @export
#' @examples
#' normalizeDdiUrn( "urn:ddi:de.GESIS:gesis_ZA4265:1.0.0" )
#' 
normalizeDdiUrn <- function( DdiUrn=NA ) {
  if(! is.character(DdiUrn)){
     message("A DDI URN must be a character vector", DdiUrn)
     return(NULL)
  }

  normedDdiUrn <- trimws(DdiUrn)
    # get the lower case prefix  
  prefix <- tolower(substr(normedDdiUrn,1,7))
  if( (!is.na(prefix)) && (prefix != idPrefix) ){
    # This is not a URN, it could be a UUID, return NULL 
    return(NULL)
  }

    # make sure the returned prefix is lower case 
  substr(normedDdiUrn,1,7) <- prefix

  normedDdiUrn
}

#' @title resolveDdiUrn - Resolve DDI URN to URL
#' 
#' @description
#' Uses the XML Catalog mechanism for resolving DDI URNs to URLs
#' 
#' @param DdiUrn DDI URN (required)
#' A character vector of length 1 containing a DDI URN.
#' 
#' @param xmlCatalogFile reference to the XML Catalog File (optional)
#' A character vector of length 1 containing a path or URI to the XML Catalog file to be used. Defaults to the package file.
#' 
#' @return An object of class character of length 1 containing a URL, or NULL if the URN cannot be resolved.
#' 
#' @export
#' @examples resolveDdiUrn( "urn:ddi:de.GESIS:gesis_ZA4265:1.0.0" )
#' 
resolveDdiUrn <- function( DdiUrn, xmlCatalogFile=DefaultXmlCatalogFile ) {
  if (is.null(normalizeDdiUrn( DdiUrn ))){
###    message(" Invalid DDI URN")
    return(NULL)
  }
  Sys.setenv( "XML_CATALOG_FILES" = xmlCatalogFile ) 
  ddiUrl <- XML::catalogResolve( normalizeDdiUrn( DdiUrn ), type="uri", asIs=FALSE )
  if ( is.na( ddiUrl ) ) {
###    message( "DDI URN \"", DdiUrn, "\" can't be resolved" )
    NULL
  } else {
    ddiUrl
  }
}

#' @title getObjectFromURL - Get DDI object by URL
#' 
#' @description
#' Gets a DDI object identified by an URL.
#' This can be a web resource or a local file.
#' 
#' @note
#' to do: objectClass, representationType
#' 
#' @param ddiUrl URL of DDI resource (required).
#' An object of class character of length 1.
#' @param ddiClass DDI class of URL resource (optional).
#' An object of class character of length 1.
#' @param representationType Representation type (XML or RDF) of URL resource (optional).
#' An object of class character of length 1.
#' 
#' @return The DDI object of type ddiClass.
#' 
#' @export
#' @examples  getObjectFromURL(ddiUrl=" https://snd.gu.se/catalogue/study/snd0741/export/ddi-3.2")
#' 
getObjectFromURL <- function( ddiUrl=NA, ddiClass=NA, representationType=NA ) {
  Rcurl::getURL( stringr::str_trim( ddiUrl ) )
}

#' @title ddiUrnToName - Returns a value to be used as a key to register or look up DDI objects 
#' 
#' @description
#' Returns a key to use for the  ddi.env.urn registry. Currently this should be a valid URN, either a DDI URN or a UUID URN  
#' @param DDiUrn  The URN to convert. 
#' 
#'
#' @return  character string which is a valid R name
#' 
#' @export
#' @examples   ddiUrnToName <- ddiUrnToName("URN:DDI:example.org:cd31b1a9-5617-4517-9c60-e9e8461c4e9b:1" )
#' 


ddiUrnToName <- function(DdiUrn){
   DdiUrn
}


#' @title nameToDdiUrn - Returns a vector with a  DDIURN or a UUID from a nameVector used as an objectKey in ddi.env.urn 
#' 
#' @description  
#'  Checks to see if this is either a urn:ddi: or a urn:uuid: returns NA if not 
#' @param nameVector  The nameVector to convert. 
#' 
#'
#' @return  a vector of valid Urns  or NA for invalid names 
#' 
#' 
#' @export
#' @examples   DDIname <- nameToDdiUrn(c("urn:ddi:example.org:myId3:1", "a bad name", "urn:uuid:cd31b1a9-5617-4517-9c6-e9e8461c4e9b") )
#' 


nameToDdiUrn <- function(nameVector){
  if(length(nameVector) == 0 ) return(NULL)
  returnVector <- sapply(nameVector, function(name){  
    if(substring(name, first=1, last=8) == "urn:ddi:"   ){
      returnValue <- name
    } else if (substring(name, first=1, last=9) == "urn:uuid:"){ 
      returnValue <- name
    } else {
      returnValue <- NA
    }
  })
  returnVector
}


####################################################################

#' @title addToReferencedUrn - Adds to the dictionary entry of the URNs referencing a referenced URN
#' 
#' @description
#' addToReferencedUrn adds an entry to a list of references to each referenced URN
#' Each entry in the list is a length 2 vector, having the  referencing URN and the related association name.
#' Both referencedUrnVector and referencingUrnVector may have a length greater than 1.
#' The length of referencingAssociationName must be 1.
#' These are stored in the environment ddi.env.references which is set up as a hash.
#' The key is generated by a call to ddiUrnToName.
#' This dictionary allows reverse lookup of object associations. It is maintained as objects are assigned associations.
#'
#' @param referencedUrnVector The URN vector of the objects being referenced 
#' 
#' @param referencingAssociationName The association name for the reference 
#' 
#' @param referencingUrnVector The URN vector of the objects doing the referencing 
#' 
#'
#' @return  the vector added
#' 
#' @examples addToReferencedUrn(referencedUrnVector="urn:ddi:example.org:idCat:1", referencingAssociationName="denotes", referencingUrnVector="urn:ddi:example.org:idCode:1")    
#' 
#' @export
#'

addToReferencedUrn <- function(referencedUrnVector, referencingAssociationName, referencingUrnVector){
  if(length(referencingAssociationName) == 0){
    message("Warning: referencingAssociationName must be a character vector of length 1, not 0")
  }
    if(length(referencingAssociationName) >1){
    message("Warning: referencingAssociationName must be a character vector of length 1")
  }
  
  for (referencedUrn in referencedUrnVector){
    referencedKey <- ddiUrnToName(referencedUrn)
    # the new value to be added to this key is a list of vectors (referencingAssociationName,referencingUrn )
    newValue <- lapply(referencingUrnVector, function(urn){c(referencingAssociationName,urn)})
  
     if (exists(referencedKey, inherits = FALSE, envir=ddi.env.references)){
       # append the new value to the end of the existing value, but do not allow duplicates
       existingValue <- get(referencedKey, inherits = FALSE, envir=ddi.env.references)
       assign(referencedKey, unique(c(existingValue, newValue)), inherits = FALSE, envir=ddi.env.references)
     } else {
       assign(referencedKey, unique(newValue), inherits = FALSE, envir=ddi.env.references)
     }
  
  }

  # return the complete value
  invisible(get(referencedKey, inherits = FALSE, envir=ddi.env.references))
}

####################################################################

#' @title getReferences - Retrives a list of the association name/URN of the objects referencing the referenced URN
#' 
#' @description
#' getReferences returns a list of vectors, each of which contains the association name and URN of an object referencing the given URN.
#' This is a reverse lookup of object associations.
#'
#' @param referencedUrn The URN of an object for which a list of referencing objects is desired 
#' 
#'
#' @return  a list of vectors c(associationName, referencingUrn) or NULL if no entry found
#' 
#' @examples getReferences(referencedUrn="urn:ddi:example.org:idCat:1")    
#' 
#' @export
#'

getReferences <- function(referencedUrn){
  referencedKey <- ddiUrnToName(referencedUrn)
  # the mget embeds the list in a list. Return just the original list
  mget(referencedKey, inherits = FALSE, envir=ddi.env.references, ifnotfound=list(NULL))[[1]]  
}




####################################################################

#' @title removeReferencedUrn - Removes the entry for a referenced URN 
#' 
#' @description
#' removeReferencedUrn removes the entry for the given URN. Thil removes ALL referencing association/URN vectors. 
#'
#' @param referencedUrn The URN to remove from the referencing lookup table 
#' 
#'
#' @return  a list of vectors c(associationName, referencingUrn) or NULL if no entry found
#' 
#' @examples removeReferencedUrn(referencedUrn="urn:ddi:example.org:idCat:1")    
#' 
#' @export
#'

removeReferencedUrn <- function(referencedUrn){
  for (urn in referencedUrn){
    referencedKey <- ddiUrnToName(urn)
    if(exists(referencedKey, inherits = FALSE, envir=ddi.env.references)){
      rm(list=referencedKey, inherits = FALSE, envir=ddi.env.references)
    }
  }
}



####################################################################

#' @title removeReferencingUrn - Removes one referencing vector from the entry for the referenced URN 
#' 
#' @description
#' removeReferencingUrn removes one referencing vector from the entry for the referenced URN.
#' This leaves an entry in the dictionary for the referenced URN if there are any other references to it.
#'
#' @param referencedUrn  The URN being referenced.  
#' 
#' @param referencingUrn  The URN doing the referencing. THe vector for this URN will be removed from the list indexed by referencedUrn.
#' 
#'
#' @return  a list of vectors c(associationName, referencingUrn) or NULL if no entry found
#' 
#' @examples removeReferencingUrn(referencedUrn="urn:ddi:example.org:idCategory:1", referencingUrn="urn:ddi:example.org:idCode:1" )    
#' 
#' @export
#'

removeReferencingUrn <- function(referencedUrn, referencingUrn){
  for (urn in referencedUrn){
    referencedKey <- ddiUrnToName(urn)
    # the mget will return a list of the list we want
    entryList <- mget(referencedKey, inherits = FALSE, envir=ddi.env.references, ifnotfound=list(NULL))[[1]]
    if(is.null(entryList)) next
    # find the vector that has the referencingUrn
    vectorMatches <- sapply(entryList, function(v){v[2]==referencingUrn} )
    # eliminate that vector from the list, keep the others
    newList <- entryList[! vectorMatches]
    
    if( length(newList)>0){
      # remove the old entry from the environment
      rm(list=referencedKey, inherits = FALSE, envir=ddi.env.references)
      # enter the new one
      assign(referencedKey, newList, inherits = FALSE, envir=ddi.env.references)
    }
  }
}


##################################################################################
#' @title listReferencedDdiUrns - Lists the DdiUrns of all of the referenced objects
#' 
#' @description
#' returns the list of DDI URNs 
#
#' 
#' @return a list of DDI URNs
#' 
#' @examples   listReferencedDdiUrns()
#' 
#' @export

listReferencedDdiUrns <- function(){
  nameToDdiUrn(ls(envir=ddi.env.references))
}

#' @title deRegisterObject - Deregisters a list of DDI R6 objects by DDI URN
#' 
#' @description 
#' This removes a list of DDI URNs from the hash table in the environment ddi.env.urn is used for managing DDI URNs.
#'
#' @note
#' to do: only one DDI URN should reference to one object
#' 
#' @param DdiUrns  a vector of DDI URNs or UUIDs (required)
#' An object of class character of length 1.
#' 
#' @export
#' @examples  deRegisterObject("urn:ddi:example.org:myID12:1")
#' 
deRegisterObject <- function( DdiUrns ) {
  returnVals <- vapply(DdiUrns,
    function(DdiUrn){
      if (is.null(normalizeDdiUrn( DdiUrn ))){
       # assume this is a UUID
       objectKey <- paste0("urn:uuid:", DdiUrn) 
      } else {
        DdiUrn <- normalizeDdiUrn( DdiUrn )      
        objectKey <- ddiUrnToName(DdiUrn)
      }
      if ( exists( objectKey, inherits = FALSE, envir=ddi.env.urn ) ) {
        rm( list=objectKey, inherits = FALSE, envir=ddi.env.urn )
        return(objectKey)
      } else {
        return(NULL)        
      }
    },
    character(1)      
  )
  invisible(returnVals)
}


#' @title registerObject - Registers a DDI R6 object by DDI URN
#' 
#' @description 
#' A hash table in the environment ddi.env.urn is used for managing DDI URNs.
#'
#' @note
#' Only one DDI URN should reference one object. If an object is already registered the old one will be deregistered first and a message will be printed to the error log.
#' 
#' @param DdiUrn a DDI URN  or a UUID for objects like struturedDatatypes with no DDI URN  (required)
#' An object of class character of length 1.
#' @param object DDI R6 object (required)
#' An object of class character of length 1.
#' 
#' @return invisibly returns the DdiUrn, or NULL if the URN is bad or already registered.
#' 
#' @export

#' @examples registerObject("urn:ddi:example.org:myID12:1", DDI4_InstanceVariable$new()) 
#' 
registerObject <- function( DdiUrn, object ) {
  if(class(object)[length(class(object))] != "R6"){
    stop("object is wrong class to register: ", paste0(class(object), sep=" ") )
  } 
  DdiUrnArgument <- DdiUrn
  DdiUrn <- normalizeDdiUrn(DdiUrn)
  if (is.null(DdiUrn)){
    if(grepl("[0-9a-f]{8}(?:-[0-9a-f]{4}){3}-[0-9a-f]{12}", DdiUrnArgument )){ 
      # this is a UUID, register it with that
      objectKey <- paste0("urn:uuid:", DdiUrnArgument)
    } else {
      stop("Cannot register an invalid DDI URN that is not a UUID: ", DdiUrnArgument)
    }
  } else {
    # this is a DDI URN, register it with that
    objectKey <-  ddiUrnToName(DdiUrn)
  }
    # check if the object isPersistent and it is already registered, if so it can't be reregistered
  if (exists( objectKey, inherits = FALSE, envir=ddi.env.urn ) ) {
    if ( !is.null(object$isPersistent)){
      if ( object$isPersistent) stop( "DDI URN \"", DdiUrn, "\" is persistent and is already registered! It can't be reregistered" )
    }
    deRegisterObject(DdiUrnArgument)
  } 
  assign( objectKey, object, inherits = FALSE, envir=ddi.env.urn )
  invisible(objectKey)
}

#' @title isDdiUrnRegistered - Checks if DDI URN is registered
#' 
#' @description
#' Checks if the given DDI URN is registered in the DDI URN table
#' 
#' @param DdiUrn DDI URN (required)
#' An object of class character of length 1.
#' 
#' @return Logical value, or NULL if the URN is invalid.
#' 
#' @export
#' @examples  isDdiUrnRegistered("urn:ddi:example.org:myID12:1")
#' 
isDdiUrnRegistered <- function( DdiUrn=NA ) {
  if (is.null(normalizeDdiUrn( DdiUrn ))){
    # assume this is a UUID 
    objectKey <- paste0("urn:uuid:", DdiUrn)
  } else {
    objectKey <-  ddiUrnToName(normalizeDdiUrn(DdiUrn))
  }
  exists( objectKey, inherits = FALSE, envir=ddi.env.urn )
}


#' @title registerAllDdiObjects - Register all DDI R6 objects in the listed environment
#' 
#' @description
#' \code{registerAllDdiObjects} registers all unregistered DdiUrns in the specified environment
#'   This might be used if loading a workspace containing Ddi R6 objects
#'
#' @return invisibly returns vector of DdiUrns newly registered
#' 
#' @export
#' @examples
#' registerAllDdiObjects(environment())

registerAllDdiObjects <- function(environ){

  objectList <- lapply(ls(envir=environ), get, envir=environ)
  
  isR6 <- sapply(objectList, is.DDIObject)

  names(isR6) <- ls(envir=environ)

  R6Classes <- lapply(ls(envir=environ), get)[isR6]

  names(R6Classes) <- names(ls(envir=environ)[isR6])

  foundUrns <- c()
  
  for (object in R6Classes){
    if (!is.null(object$DdiUrn)){
      if (!isDdiUrnRegistered(object$DdiUrn)){
        registerObject(object$DdiUrn,object)
        foundUrns <- c(foundUrns,object$DdiUrn)
      } 
    } 
  }
  invisible(foundUrns)
}






#' @title isSingleString - Check if a vector is a single string (Not Exported)
#' 
#' @description
#' TRUE or FALSE. Is the argument a character vector of length 1? This function is not exported. It is only available to other functions within th package. 
#' 
#' @param input An R object)
#' An R object of any class.
#' @return TRUE if the object is character and length 1, otherwise FALSE.
#' 
#' @examples
#' isSingleString(c("a","b")) returns FALSE
#' isSingleString("a") returns TRUE

isSingleString <- function( input ) {
  is.character( input ) & length( input ) == 1
}


#' @title listDdiUrn - Lists registered DDI URNs
#' 
#' @description
#' Lists by default all registered DDI URNs.
#' Lists optionally DDI URNs for a specific DDI agency (or sub-agency).
#' List optionally all versions for a specific DDI id for a given agency id.
#' 
#' @param agencyId DDI agency id (optional)
#' An object of class character of length 1.
#' @param id DDI object id (optional). An agency id is required if this is used.
#' An object of class character of length 1.
#' 
#' @return An object of class character, or NULL if the URN is invalid.
#' 
#' @export
#' @examples listDdiUrn() 
#' 
listDdiUrn <- function( agencyId=NA, id=NA ) {
  idStartPattern <- paste( "^", idPrefix, sep="" )
  if ( is.na( agencyId ) & is.na( id ) ) {                           # default no parameters
    idStartPattern <- paste( idStartPattern, ":", ".+", sep="" )
  } else if ( isSingleString( agencyId ) & is.na( id ) ) {           # only agency id
    idStartPattern <- paste( idStartPattern, ":", tolower( agencyId ), "[^:]*:", ".+", sep="" )
  } else if ( isSingleString( agencyId ) & isSingleString( id ) ) {  # agency id and id
    idStartPattern <- paste( idStartPattern, ":", tolower( agencyId ), ":", id, ":", ".+", sep="" )
  } else if ( is.na( agencyId ) & isSingleString( id ) ) {           # id but no agency id
    stop( "Id \"", id, "\" given without parameter for agency id!" )
  }
  nameVector <- ls(envir=ddi.env.urn )
  unname(sort(grep(pattern=idStartPattern, nameToDdiUrn(nameVector), value=TRUE ))) 
}

#' @title saveDdi - Save the registries of DDI objects
#' 
#' @description
#' Saves the registries of DDI objects to external files. Passes arguments to the built-in save function.
#' 
#' @param registryFile The file argument passed to save for the base DDI object registry. 
#'  A (writable binary-mode) connection or the name of the file where the data will be saved (when tilde expansion is done). 
#' @param referencesFile The file argument passed to save for the registry of referenced objects. 
#'  A (writable binary-mode) connection or the name of the file where the data will be saved (when tilde expansion is done). 
#'  	
#' @param  ascii  If TRUE, an ASCII representation of the data is written. 
#' The default value of ascii is FALSE which leads to a binary file being written.
#'  
#' @param compress logical or character string specifying whether saving to a named file is to use compression. 
#'  TRUE corresponds to gzip compression, and character strings "gzip", "bzip2" or "xz" specify the type of compression. 
#'  Ignored when file is a connection
#'  
#' @export
#' @examples saveDdi(registryFile = "DDI4.RData", referencesFile="DDI4references.RData") 
#' 

saveDdi <- function(registryFile, referencesFile, ascii=FALSE, compress = isTRUE(!ascii)){
  registryNameVector <- ls(envir=ddi.env.urn )
  save(list=registryNameVector, file=registryFile, ascii=ascii, envir=ddi.env.urn)

  referencesNameVector <- ls(envir=ddi.env.references )
  save(list=referencesNameVector, file=referencesFile, ascii=ascii, envir=ddi.env.references)
}


#' @title loadDdi - Loads saved copies of DDI objects into the DDI4 object registries
#' 
#' @description
#' loads the registry of DDI objects from an external file. passes arguments to the built-in load function.
#' 
#' @param registryFile The file argument passed to save for the base DDI object registry. 
#'  A (writable binary-mode) connection or the name of the file where the data will be saved (when tilde expansion is done). 
#' @param referencesFile The file argument passed to save for the registry of referenced objects. 
#'  A (writable binary-mode) connection or the name of the file where the data will be saved (when tilde expansion is done). 
#'
#' @param verbose If true returns a vector of the DDI URNs loaded
#'  
#' @export
#' @examples loadDdi(registryFile = "DDI4.RData", referencesFile="DDI4references.RData", verbose=FALSE) 
#' 

loadDdi <- function(registryFile, referencesFile,  verbose = FALSE){
  registryKeys <- load(file=registryFile, envir=ddi.env.urn, verbose=verbose)
  if(length(registryKeys>0)){
   ddiUrns <- sort( sapply(registryKeys, nameToDdiUrn)  )  
  } else {
   ddiUrns <- NULL
  }
  referencesKeys <- load(file=referencesFile, envir=ddi.env.references, verbose=verbose)
  ddiUrns
}

#' @title listDdiUuid - Lists registered DDI UUIDs
#' 
#' @description
#' Lists all registered DDI UUIDs.
#' @return An object of class character, or NULL if no UUIDS are registered.
#' 
#' @export
#' @examples  listDdiUuid()
#' 
listDdiUuid <- function( agencyId=NA, id=NA ) {
  UuidPattern <- "^[0-9a-f]{8}(?:-[0-9a-f]{4}){3}-[0-9a-f]{12}"
  nameVector <- ls(envir=ddi.env.urn )
  sort(grep(pattern=UuidPattern, nameToDdiUrn(nameVector), value=TRUE )) 
}

#' @title getObject - Gets DDI object searching the registry for its DDI URN
#' 
#' @description
#' Returns the reference to a DDI R6 object
#' Note that this is called a LOT, so to speed it up it does not normalize the argument. 
#' 
#' @param DdiUrn DDI URN (required)
#' An object of class character of length 1.
#' 
#' @return Reference to DDI R6 object
#' 
#' @export
#' @examples  getObject("urn:ddi:example.org:myID12:1"), getObject(normalizeDdiUrn(" URN:ddi:example.org:myID12:1  "))
#' 
getObject <- function( DdiUrn=NA ) {
                 # normedDdiUrn <- normalizeDdiUrn( DdiUrn )
  if (grepl("^urn:ddi", DdiUrn, ignore.case=TRUE)){
    objectKey <- ddiUrnToName(DdiUrn)
  } else {
    # assume this is a UUID 
    objectKey <- paste0("urn:uuid:", DdiUrn)
  }
  mget( objectKey, envir=ddi.env.urn , ifnotfound=list(NULL),	inherits = FALSE)[[1]] # response is reference to object
}


# - - - - - - - - - -
# end included from ddiUtil.R
# - - - - - - - - - -


#-------------------------------------------
# R Function Definitions 
#-------------------------------------------


#' @section Other R Function Definitions:
#' 
#' @description
#' Functions used by the R6 classes
#' 

#' @title returnR6ObjectIfValid - Return a list of objects of the specified R6 class, otherwise stops with an error message
#' 
#' @description
#' \code{returnR6ObjectIfValid} is used to check that elements of a list of objects all are of the specified R6 class. 
#' It returns the list if so, otherwise it terminates with an error message.
#' 
#' @param validClassName The valid R6 class name (required).
#' A character value containing the allowed name of the classes in the list. All objects in the list must be of this class. (required).

#' @param proposedObjectList A list of the proposed classes.
#' A list of the proposed R6 classes of a length within the minimum and maximum cardinalities (required).

#' @param minimumCardinality A number specifying the minimum number of elements in the list (required).

#' An object of class numeric of length 1.
#' @param maximumCardinality A number specifying the maximum number of elements in the list (required).

#' An object of class numeric of length 1.
#' @return Returns proposedObjectList if all list members are of the specified class
#' 
#' @export
#' @examples
#' a <- Machine$new()
#' b <- Machine$new()
#' returnR6ObjectIfValid('Machine',list(a,b),1,2)


# returnR6ObjectIfValid:  function returns a list of objects of the specified R6 class, otherwise stops with an error message.
#                        Note that R6 objects cannot be in a vector, only a list
#      Arguments
#        validClassName - the allowed name of the classes in the list (all must be the same)
#        proposedObjectList - a list of the proposed classes 
#        minimumCardinality - the minimum length of the list (minimum cardinality)
#        maximumCardinality - the maximum length of the list (maximum cardinality)

returnR6ObjectIfValid <- function(validClassName, proposedObjectList, minimumCardinality, maximumCardinality) {
  # check for constraints on the proposed class 

  if(class(proposedObjectList)[1] != "list"){
    stop(paste("the argument to proposedObjectList must be a list, not a ", class(proposedObjectList), "\n"))
  }

  for (proposedObject in proposedObjectList) {
    if( class(proposedObject)[length(class(proposedObject))] != "R6") {
      stop(paste("entered class,",class(proposedObject)[1],", is not an R6 class. It must be either:",validClassName, " or an ancestor of that class.  ")) 
    }
    if( !(validClassName %in% class(proposedObject) )) {
      stop(paste("The valid class name, '",   validClassName  ,"', does not appear in the class hierarchy of the proposed object: ", paste(class(proposedObject),collapse=' '))) 
    }
  }

  if (length(proposedObjectList) < minimumCardinality) {
      stop(length(proposedObject), " values of ",validClassName, " were entered. At least ", minimumCardinality, " are required." )
  } else if (length(proposedObjectList) > maximumCardinality) {
      stop(length(proposedObject), " values of ",validClassName, " were entered, No more than ", maximumCardinality, " are allowed." )
  }

  # proposed Class list follows constraints
  proposedObjectList
}




#' @title returnUrnVectorIfValid - Return a vector of URNs for objects of the specified R6 class, otherwise stops with an error message
#' 
#' @description
#' \code{returnUrnVectorIfValid} is used to check that elements of a vector of URNs all are of the specified R6 class, if possible. 
#' It returns the vector if so, otherwise it prints error messages and returns any valid URNs.
#' In silent mode it does not print warnings, but will stop if cardinality or datatype rules are violated. 
#' 
#' @param validClassName The valid R6 class name (required).
#' A character value containing the allowed name of the classes in the vector. All objects in the vector must be of this class. (required).

#' @param proposedUrnVector A vector of URNS for the proposed classes.
#' A character vector of the URNs to proposed R6 objects of a length within the minimum and maximum cardinalities (required).

#' @param minimumCardinality A number specifying the minimum number of elements in the vector (required).

#' An object of class numeric of length 1.
#' @param maximumCardinality A number specifying the maximum number of elements in the vector (required).

#' @param silent Prints warning messages if false.

#' An object of class numeric of length 1.
#' @return Returns proposedUrnVector if all vector members are of the specified class
#' 
#' @export
#' @examples
#' returnUrnVectorIfValid('Concept',c("urn:ddi:example.org:IDConcept1:1", "urn:ddi:example.org:IDConcept2:1" ),0,Inf)

returnUrnVectorIfValid <- function(validClassName, proposedUrnVector, minimumCardinality, maximumCardinality, silent=FALSE) {
  # check for constraints on the proposed URN vector and associated objects if registered 

  # first this must be a character vector
  if(class(proposedUrnVector)[1] != "character"){
    stop(paste("the argument to proposedUrnVector must be a character vector, not a ", class(proposedUrnVector), "\n"))
  }

  # check cardinality
  if (length(proposedUrnVector) < minimumCardinality) {
      stop(as.character(length(proposedUrnVector)), " values of DDI URNs were entered. At least ", as.character(minimumCardinality), " are required." )
  } else if (length(proposedUrnVector) > maximumCardinality) {
      stop(as.character(length(proposedUrnVector)), " values of DDI URNs were entered, No more than ", as.character(maximumCardinality), " are allowed." )
  }
  
  for (proposedURN in proposedUrnVector) {    
  
  # Find the object if the URN is registered and check its class    
    proposedObject <- getObject(DdiUrn=c(proposedURN))
    if ( !is.null(proposedObject)){
      #found an object in the registry
      if( class(proposedObject)[length(class(proposedObject))] != "R6")  {
        stop(paste("entered class,",class(proposedObject)[1],", is not an R6 class. It must be either:",validClassName, " or an ancestor of that class.  ")) 
      }
      if( !(validClassName %in% class(proposedObject) ) ) {
        stop(paste("In the DdiUrn ",  proposedURN,    "\n             the valid class name, '",   validClassName  ,"', does not appear in the class hierarchy of the proposed object: ", paste(class(proposedObject),collapse=' '))) 
      }           
    } else {
      
      # check if the URN can be resolved via the XML catalog, for now print a message if so
      URI <- resolveDdiUrn(proposedURN, DefaultXmlCatalogFile)
      if (!is.null(URI)  ){
        if(!silent) message("The URN ", proposedURN, " was found externally at: ", URI )
        # ######## eventually import and register if possible #########
      } else {
        if(!silent) message("The URN ", proposedURN, " could not be found. It must be validated manually")
      }    
    }
  }
    
  # proposed Class vector follows constraints
  proposedUrnVector
}

#' @title returnVectorIfValid - Return a vector of primitive values of the specified type, otherwise stops with an error message.
#' 
#' @description
#' \code{returnVectorIfValid} is used to check that elements of a vector all are of the specified type. 
#' It returns the vector if so, otherwise it terminates with an error message.
#' 
#' @param validVectorClass The valid  class name of the vector (required).
#' A character value containing the name of the allowed class of the vector. As a vector all elements must be the same type. (required).

#' @param proposedVector A list of the proposed classes.
#' The vector of primitive values. Note that all objects in the vector must be of the same type (required).

#' @param minimumCardinality A number specifying the minimum number of elements in the vector (required).

#' An object of class numeric of length 1.
#' @param maximumCardinality A number specifying the maximum number of elements in the vector (required).

#' An object of class numeric of length 1.
#' @return Returns proposedVector if all list members are of the specified class
#' 
#' @export
#' @examples
#' n <- 1 
#' m <- 2 
#' v <- c(n,m) 
#' returnVectorIfValid("numeric",v,1,2) 


#returnVectorIfValid : function returns a vector of primitive values of the specified type, otherwise stops with an error message
#                      Primitive values (numeric, character, boolean) can be in a vector all of one type
#      Arguments
#        validVectorClass - the allowed class of the objects in the list (all must be the same)
#        proposedVector - the vector of primitive values. Note that all objects in a vector must be of the same type
#        minimumCardinality - the minimum length of the vector
#        maximumCardinality = the maximum length of the vector

returnVectorIfValid <- function(validVectorClass, proposedVector, minimumCardinality, maximumCardinality) {
  # check for constraints on the proposed dataType 

  
  if( class(proposedVector) != validVectorClass) {
    stop(paste("entered dataType,",class(proposedVector), ", is not of the data type:",validVectorClass)) 
  }


  if (length(proposedVector) < minimumCardinality) {
      stop(length(proposedVector), " values of ",validVectorClass, " were entered. At least ", minimumCardinality, " are required." )
  } else if (length(proposedVector) > maximumCardinality) {
      stop(length(proposedVector), " values of ",validVectorClass, " were entered, No more than ", maximumCardinality, " are allowed." )
  }

  # proposed dataType list follows constraints
  proposedVector
}


#' @title returnEnumeratedValues - Return a character vector if all of the elements validate against an Enumeration Object
#' 
#' @description
#' \code{returnEnumeratedValues} is used to check that elements of a vector are all valid for an enumeration. 
#' It returns the vector if so, otherwise it terminates with an error message.
#' 
#' @param enumerationObject The R6 object containing the enumeration  (required).
#' An enumeration object (not a class). All objects in the vector must have values within this enumeration. (required).

#' @param proposedCharacterVector A vector of the proposed values.
#' A vector of the proposed character values of a length within the minimum and maximum cardinalities (required).

#' @param minimumCardinality A number specifying the minimum number of elements in the vector (required).

#' An object of class numeric of length 1.
#' @param maximumCardinality A number specifying the maximum number of elements in the vector (required).

#' An object of class numeric of length 1.
#' @return Returns proposedCharacterVector if all  members are in the enumeration
#' 
#' @export
#' @examples
#' tot<-TotalityType$new()#' returnEnumeratedValues(enumerationObject=tot,c("Total"),0,1)


# returnEnumeratedValues:  function Return a character vector if all of the elements validate against an Enumeration Object.
#                        
#      Arguments
#        enumerationObject - the allowed name of the classes in the list (all must be the same)
#        proposedCharacterVector - a character vector containing the proposed values 
#        minimumCardinality - the minimum length of the vector (minimum cardinality)
#        maximumCardinality - the maximum length of the vector (maximum cardinality)

returnEnumeratedValues <- function(enumerationObject, proposedCharacterVector, minimumCardinality, maximumCardinality) {
  # check for constraints on the proposed vector 

  if(class(proposedCharacterVector)[[1]] != "character"){
    stop(paste("the argument to proposedCharacterVector must be a character vector, not a ", paste(class(proposedCharacterVector),collapse=' '), "\n"))
  }

  for (proposedString in proposedCharacterVector) {
    if( !enumerationObject$isValidValue(proposedString)) {
      stop(paste("The value,",proposedString,", is not in the required enumeration. It must be one of: [", paste(enumerationObject$get_ValidValues(),collapse=',') , "]"))
    }
    if (length(proposedCharacterVector) < minimumCardinality) {
      stop(paste(length(proposedCharacterVector), " values were entered. At least ", minimumCardinality, " are required." ))
    }
    else if (length(proposedCharacterVector) > maximumCardinality) {
      stop(paste(length(proposedCharacterVector), " values were entered, No more than ", maximumCardinality, " are allowed." ))
    }
  }

  # proposed vector follows constraints
  proposedCharacterVector
}




#' @title returnRegularExpressionValues - Return a character vector if all of the elements validate against a regular expression Object
#' 
#' @description
#' \code{returnRegularExpressionValues} is used to check that elements of a vector are all valid for an enumeration. 
#' It returns the vector if so, otherwise it terminates with an error message.
#' 
#' @param RegularExpressionObject The R6 object defining the regular expression  (required).
#' An regular expression object (not a class). All objects in the vector must have values conforming to this regular expression. (required).

#' @param proposedCharacterVector A vector of the proposed values.
#' A vector of the proposed character values of a length within the minimum and maximum cardinalities (required).

#' @param minimumCardinality A number specifying the minimum number of elements in the vector (required).

#' An object of class numeric of length 1.
#' @param maximumCardinality A number specifying the maximum number of elements in the vector (required).

#' An object of class numeric of length 1.
#' @return Returns proposedCharacterVector if all members of the vector match the regular expression. The regular expression assumes that the whole string must match, althoughstart and end of string anchors are not included in the regular expression.
#' 
#' @export
#' @examples
#' ch<-Char$new()#' returnRegularExpressionValues(RegularExpressionObject=ch,c("X"),0,1) returns X

returnRegularExpressionValues <- function(RegularExpressionObject, proposedCharacterVector, minimumCardinality, maximumCardinality) {
  # check for constraints on the proposed vector 

  if(class(proposedCharacterVector)[[1]] != "character"){
    stop(paste("the argument to proposedCharacterVector must be a character vector, not a ", paste(class(proposedCharacterVector),collapse=' '), "\n"))
  }

  for (proposedString in proposedCharacterVector) {
    if( !RegularExpressionObject$isValidValue(proposedString)) {
      stop(paste("The value,",proposedString,", does not match the required regular expression: " , RegularExpressionObject$get_RegularExpression() )) 
    }
    if (length(proposedCharacterVector) < minimumCardinality) {
      stop(paste(length(proposedCharacterVector), " values were entered. At least ", minimumCardinality, " are required." ))
    }
    else if (length(proposedCharacterVector) > maximumCardinality) {
      stop(paste(length(proposedCharacterVector), " values were entered, No more than ", maximumCardinality, " are allowed." ))
    }
  }

  # proposed vector follows constraints
  proposedCharacterVector
}




#' @title getActiveValues - Get a named vector of the active values of a DDI4 Object
#' 
#' @description
#' Formats the print layout for a DDI4 R6 object
#' @param DdiObject a DDI4 R6 object (required).
#'
#' @param expandProperties include the content of all structured datatypes by recursion, using the top level value of expandRelations
#'
#' @param expandRelations include the content of all references by recursion, using the top level value of expandProperties
#'
#' @param numberOfLeadingBlanks the number of leading blanks to insert before each string in the vector
#'
#' @param level the depth level of the property or relation. A property of a property would be level 2.
#'
#' @param showNull if TRUE, show all properties with NULL values. If FALSE exclude them
#'
#' @param showId if TRUE show agency, id, and version. If FALSE only show DdiUrn
#'
#' 
#' 
#' @return a named character vector, one line per output line, indented as per the hierarchy of content. Some names may repeat
#' 
#' @examples  getActiveValues(myCategory, expandRelations=TRUE, expandProperties=TRUE)
#' @export


getActiveValues <- function(DdiObject, expandProperties=FALSE, expandRelations=FALSE, numberOfLeadingBlanks=2,  level=0, showNull=TRUE, showId=TRUE){
  # return a list of all public fields for the object DdiObject (treated as an environment)

  public_names <- ls(DdiObject, all.names = TRUE)
  
  newLineString <- "\n"  
  # return a list of the active bindings for the object DdiObject

  active_names <- public_names[vapply(public_names,bindingIsActive, FUN.VALUE = logical(1), env=DdiObject)]


  active_values_list <- NULL
  for (ixName in seq_along(active_names)){

    name <- active_names[ixName]
      # the list of populated fields for this active binding class
    fieldList <- .subset2(DdiObject,name)

      # the structure (datatype) for this active binding class
    structure <- DdiObject$describe_initializeArguments()[[name]]['structure']
      # DdiUrn is special. It is not a formal (DDI property or relationship), just a convenience item for the user
    if (name == "DdiUrn") structure = "vector" 


    if (length(fieldList)==0) {
      # ********* This is a NULL VALUE            ************************
      active_values_list <- append(active_values_list, paste0(newLineString, name, ": NULL"))
      names(active_values_list)[length(active_values_list)] <- name


    } else if (structure == "DdiUrn vector" ){
      # ********* This is a REFERENCE            ************************
      # one line for the whole list
      active_values_list <- append(active_values_list,  paste0(newLineString, name, 
                      " <", structure, "(", length(fieldList), ")>:"))  
      names(active_values_list)[length(active_values_list)] <- name
          
      # a header line for each referenced object
      for(ixReference in seq_along(fieldList)){   
        referencedObject <- getObject(fieldList[ixReference])
        if(is.null(referencedObject)){
          referencedHeader <- paste0( strrep(" ",numberOfLeadingBlanks),  "(external object) ", fieldList[ixReference])
          active_values_list <- append(active_values_list, referencedHeader)
          names(active_values_list)[length(active_values_list)] <- class(referencedObject)[1]
        } else {
          referencedHeader <- paste0( strrep(" ",numberOfLeadingBlanks), "(", class(referencedObject)[1], ") ", fieldList[ixReference])
         
          active_values_list <- append(active_values_list, referencedHeader)
          names(active_values_list)[length(active_values_list)] <- class(referencedObject)[1]
          
          if(expandRelations){
            #  find all of the properties of this object
            nextLevel <- level+1 
            subProperties <- getActiveValues(referencedObject, 
                                             expandProperties=expandProperties, 
                                             expandRelations=expandRelations, 
                                             numberOfLeadingBlanks=numberOfLeadingBlanks, 
                                             level=nextLevel, 
                                             showNull=showNull, 
                                             showId=showId )
            active_values_list <- append(active_values_list,  subProperties)
          }      
        }      
      } 


   	  
    } else if (R6::is.R6(fieldList[[1]]) &
            !("Identifiable" %in% class (fieldList[[1]]))){
              # ********* this is a STRUCTURED DATATYPE            ************************
              if(expandProperties){
   	       if(level==0) structuredLineStart <- paste0(newLineString, newLineString, strrep(" ",numberOfLeadingBlanks)) 
   	       else  structuredLineStart <- strrep(" ",level*numberOfLeadingBlanks) 
                  active_values_list <- append(active_values_list, paste0( structuredLineStart,  name, " <DDI R6 Datatype(", length(fieldList), ")>: "))
                  names(active_values_list)[length(active_values_list)] <- name
                for (ixField in seq_along(fieldList)){
                  nextLevel <- level+1 
                  subProperties <- getActiveValues(fieldList[[ixField]], 
                                               expandProperties=expandProperties, 
                                               expandRelations=expandRelations, 
                                               numberOfLeadingBlanks=numberOfLeadingBlanks, 
                                               level=nextLevel, 
                                               showNull=showNull, 
                                               showId=showId )
                  active_values_list <- append(active_values_list,  subProperties)      
                }
              } else {
                active_values_list <- append(active_values_list, paste0(newLineString, name, "<DDI R6 Datatype(", length(fieldList), ")> "))
                names(active_values_list)[length(active_values_list)] <- name
              }
              
              
              
    } else if (is.atomic(fieldList)){
      # ********* this is an atomic (leaf) value             ************************
      #           indent every embedded line return and add one at the end 
     active_values_list <- append(active_values_list, paste0(strrep(" ",level*numberOfLeadingBlanks),  name, 
                  " <",typeof(fieldList), "(", length(fieldList), ")>:  ",
          paste0(sapply(seq_along(fieldList),
                                   function(ix){
                                     if(is.na(fieldList[ix])) {
                                       fieldValue <- "NA"
                                     } else {
                                       fieldValue <- as.character(fieldList[ix]) 
                                     }
                                     if(length(fieldValue) == 0) fieldValue <- "empty string"
                                     firstCharacter <- substr(fieldValue,1,1)
                                     if(firstCharacter != "\n") fieldValue <- paste0(newLineString, fieldValue)
                                     fieldValue <- gsub("\n", paste0(newLineString, strrep(" ", (level+2)*numberOfLeadingBlanks)),  fieldValue)  
                                     paste0( strrep(" ",numberOfLeadingBlanks), fieldValue, newLineString )       
                                   }
                             ),
                             collapse=""
                      )
               )
            ) 
    names(active_values_list)[length(active_values_list)] <- name	  
    }             
    #    paste(retVal, collapse=newLineString) 
  }
 
  
  
  
  
  
  
  
  if(! showNull)active_values_list <- active_values_list[ ! grepl("NULL$",active_values_list)]
  if(! showId)active_values_list <- active_values_list[!(names(active_values_list) %in% c("agency", "id", "version"))]


  # replace embedded newlines with a newline followed by indetation to a level below the current level
  active_values <- sub("\n", paste0(newLineString, strrep(" ", numberOfLeadingBlanks)), as.character(active_values_list)  )
  #active_values <- paste(strrep(" ",level*numberOfLeadingBlanks), valueString, sep="") 

  names(active_values) <- names(active_values_list)
  active_values

}



#' @title ddiObjectFormat - Format a DDI4 R6 object for print.  (Not Exported)
#' 
#' @description
#' Formats the print layout for a DDI4 R6 object.  This function is not exported. It is only available to other functions within th package.
#' @param DdiObject a DDI4 R6 object (required).
#' An object of class R6 from the DDI4 package.
#' 
#' @return a vector with embedded line endings.
#' 
#' @examples
#' ddiObjectFormat(myInstanceVariable)
#'   where myInstanceVariable was created as myInstanceVariable <- InstanceVariable$new(....)
ddiObjectFormat <- function(DdiObject) {
  # return a list of all public fields for the object DdiObject

  public_names <- ls(DdiObject, all.names = TRUE)
  
  # return a list of the active bindings for the object DdiObject

  active_values <- getActiveValues(DdiObject, expandProperties=TRUE, expandRelations=FALSE, showNull=FALSE, showId=FALSE)
    
  # return a list of the functions for the object DdiObject
  function_names <- public_names[vapply(public_names,
                                        function(name){is.function(.subset2(DdiObject,name))},
                                        FUN.VALUE = logical(1))]      

  





# Start building the return vector


# class name and DDI URN
  retVal <- paste0("<",
                    class(DdiObject)[1], 
                    ">   DdiUrn=", 
                    DdiObject$DdiUrn,
	                "\n")

# Inheritance
  NumberOfAncestors <- length(class(DdiObject))
  if (NumberOfAncestors  > 2){ 
    retVal <- paste0(retVal, "\nInherits from: ", 
                as.character(NumberOfAncestors-1),
                 " classes\n    " , 
                paste0(sapply(class(DdiObject)[2:(NumberOfAncestors)],
                        function(ancestor){
                          paste0(ancestor, "\n")
                        }
                      ),
					  collapse="    "
               )
			   
    )
  }

  retVal <- paste0(retVal, "\nActive Bindings:\n", paste0(" ", active_values, collapse="\n"))

  retVal <- paste0(retVal, 
                 "\nFunctions:\n    ",
                  paste0(sapply(function_names,
				             function(functionName){
				               paste0(functionName, "\n")
                               }
                        ),
						collapse=" "
                  )
           )
  
  retVal                                        
}





#' @title printDdiObject - Prints a DDI4 Object, with various display options
#'  
#' @description
#' Prints a DDI4 R6 object with the options to expand structured datatypes and relations. Also allows hiding NULL valued properties and agency, id and version
#' @param DdiObject a DDI4 R6 object (required).
#'
#' @param expandProperties include the content of all structured datatypes by recursion, using the top level value of expandRelations
#'
#' @param expandRelations include the content of all references by recursion, using the top level value of expandProperties
#'
#' @param numberOfLeadingBlanks the number of leading blanks to insert before level in the vector
#'
#' @param showNull if TRUE, show all properties with NULL values. If FALSE exclude them
#'
#' @param showId if TRUE show agency, id, and version. If FALSE only show DdiUrn
#'
#' 
#' 
#' @return a named character vector, one line per output line, indented as per the hierarchy of content. Some names may repeat
#' 
#' @examples  printDdiObject(myCategory, expandRelations=TRUE, expandProperties=TRUE)
#' @export
#'

printDdiObject <- function(DdiObject, expandProperties=FALSE, expandRelations=FALSE, numberOfLeadingBlanks=2, showNull=TRUE, showId=TRUE){
  active_values <- getActiveValues(DdiObject, expandProperties=expandProperties, expandRelations=expandRelations, numberOfLeadingBlanks=numberOfLeadingBlanks, showNull=showNull, showId=showId)
  cat(paste0(active_values, collapse="\n"), "\n")
}



#' @title is.DDIClassName - Is a name the name of a DDI4 R6 Class?
#' 
#' @description
#' Checks if a name is the name of a DDI4 R6 object generator (from the package DDI4R)
#' 
#' @param name the name to check (required)
#' A string.
#' 
#' @return TRUE if name is the name of a DDI4 R6 class, FALSE if not
#' 
#' @examples is.DDIClassName(xNodeName) returns TRUE if xNodeName is an R6 class generator in the package DDI4R
#' 
#' @export

is.DDIClassName <- function(name){
  object <- mget(name, as.environment("package:DDI4R"), ifnotfound=list(FALSE))
  R6::is.R6Class(object[[1]]) 
}



#' @title is.DDIClass - Is an object a DDI4 R6 Class Generator?
#' 
#' @description
#' Checks if an object is a DDI4 R6 object generator (from the package DDI4R)
#' 
#' @param object the name to check (required)
#' An R object.
#' 
#' @return TRUE if teh object is a DDI4 R6 object generator, FALSE if not
#' 
#' @examples is.DDIClass(Code) returns TRUE,  is.DDIClass(foo) returns FALSE
#' 
#' @export

is.DDIClass <- function(object){
# explicitly check first for R6 objects, not everything has $parent_env
  if (R6::is.R6Class(object)){
   if (  identical(object$parent_env, asNamespace("DDI4R")) ){
      return(TRUE)
   }
  }
  FALSE
}



#' @title is.DDIObject - Is an object a DDI4 R6 Object?
#' 
#' @description
#' a function to check if an object is a DDI4 R6 object 
#' 
#' @param name the object to check (required)
#' An object .
#' 
#' @return TRUE if the object's class is a DDI4 R6 class, FALSE if not
#' 
#' @examples is.DDIObject(myInstanceVariable) returns TRUE if myInstanceVariable was generated by a R6 class generator in the package DDI4R
 
#' 
#' @export

is.DDIObject <- function(object){
  R6::is.R6(object) & is.DDIClassName(class(object)[1])
}



#' @title parseDdiUrn - Parse a DDI URN into a named vector
#' 
#' @description
#' parses a DDI URN into a named vector having agency, id, and version values 
#' 
#' @param DdiUrn DDI URN (required).
#' A character vector of length 1 containing a DDI URN.
#' 
#' @return A named character vector of length 3 containing agency, id, and version values.
#' 
#' @export
#' @examples
#' parseDdiUrn ("urn:ddi:de.GESIS:gesis_ZA4265:1.0.0" ) returns c(agency="GESIS", id="gesis_ZA4265", version="1.0.0")



parseDdiUrn <- function(DdiUrn){
  c(agency=strsplit(DdiUrn,":",fixed=TRUE)[[1]][3],
    id=strsplit(DdiUrn,":",fixed=TRUE)[[1]][4],
    version=strsplit(DdiUrn,":",fixed=TRUE)[[1]][5] 
  )
}



#' @title makeDdiUrn - Make a DDI URN from agency, id and version
#' 
#' @description
#' builds a DDI URN from agency, id, and version values 
#' 
#' @param agency an agency identifier (required).
#' A character vector of length 1 containing a agency identifier.
#' @param id a DDI object id (required).
#' A character vector of length 1 containing an id value, unique within the agency. This can be a globally unique value like a UUID as well.
#' @param version (required).
#' A character vector of length 1 containing version for the agency, id combination.
#' 
#' @return A named character vector of length 1 containing a DDI URN.
#' 
#' @export
#' @examples
#' makeDdiUrn(agency="de.GESIS", id="gesis_ZA4265", version="1.0.0") returns  "urn:ddi:de.GESIS:gesis_ZA4265:1.0.0"



makeDdiUrn <- function(agency=NULL, id=NULL, version=NULL){
  if(!is.character(agency) | !is.character(id) | !is.character(version)){
    stop("All values of agency, id and version must be character values ")
  }
  paste("ddi:urn", agency, id, version,  sep=":", collapse=NULL)
}


#' @title ddiXmlNodeToObject - Create a DDI4 R6 object from a DDI4 XML node
#' 
#' @description
#' From a DDI4 XML node, returns a DDI4 object using an R6 class generator in the package DDI4R

#' 
#' @param xNode the XML node to import (required)
#' An XML Node created using the XML package. Some DDI4 XML nodes represent clases. Others represent a property or relationship of the class and have a datatype a DDI4 R6 class (see xNodeClass) 
#' @param xNodeClass the DDI4 R6 class to be returned. If this parameter is not supplied xNode needs to represent a DDI4 class. Note that In DDI4R the class names all have the prefix "DDI4_" but the XML node names may not.  (optional)
#' A DDI4 R6ClassGenerator. If the node represents a class this will be the same as that class. If the node represents a property or relationship This will be the datatype of that property or relationiship.
#' @param silentState If TRUE some messages about relationship validation will be supressed (optional)
#'  If FALSE, messages about missing references or references to external objects will be displayed.
#' 
#' @return a DDI4 R6 object
#' 
#' @examples ddiXmlNodeToObject(xNode=myNode, xNodeClass=paste("DDI4_", myNode, collapse="", sep=""))  
 
#' 
#' @export

#   a function to create an R6 object from a DDI4 XML node
#   recurses down to leaves

ddiXmlNodeToObject <- function(xNode, xNodeClass, silentState=FALSE, verbose=FALSE, messageIndent=0){

  xNodeName <- xmlName(xNode) 

  messageIndent <- messageIndent +2
  if(verbose)message("\n", rep(" ",messageIndent),  "processing node: ",  xNodeName,  " at line: ",  getLineNumber(xNode)  )

  # the node name must match a DDI4 class name, with the prefix "DDI4_" if a class name is  not given
  if (missing(xNodeClass)){
    DDI4Name <- paste("DDI4_", xNodeName, collapse="", sep="")
    if (is.DDIClassName(DDI4Name )){
      xNodeClass <- get(DDI4Name)
    } else {
      warning(paste0(xNodeName, " is not a DDI4 R6 class name. An xNodeClass argument needs to be supplied in this case.\n"))
      return()
    }
  } else {
    # xNodeClass must be a DDI4 class generator
    if (! is.DDIClass(xNodeClass) ){
       stop("The xNodeClass parameter value is not a valid DDI4 R6 class. It is a  ", class(xNodeClass))
    }
  }

  # retrieve the named character vector of attributes for this node
  xNodeAttributes <- xmlAttrs(xNode)  
  
  if (is.null(xNodeAttributes)) {
    attributeDisplay <- "no attributes"
  } else {
    attributeDisplay <- paste(sapply(names(xNodeAttributes), function(attName){paste(attName, xNodeAttributes[attName], sep="=")}),  collapse=", ")
  }
  if(verbose)message(rep(" ",messageIndent+1),  "  attributes: ", attributeDisplay)
  
 



  # here we want to start from a named list of nodes, the children of xNode
  #  there may be multiple child nodes with the same name
  # the names are the property or relationship names from the DDI4 model

  # these child nodes will be processed into a named list of lists,
  # one list per child node name (property or relationship name), 
  #  with a corresponding list of R6 objects taken from a child node having that name
  # example: 
    # A Study might have an "overview" property with a one element list of R6 InternationalStructuredString objects
    # The Study might also have a "hasInstanceVariable" relationship with a multiple element list of R6 InstanceVariable objects

  # a for loop is used here since the input list will possibly have more elements than the output list

  # this is the list of lists that contains the arguments for the R6 object
  # to be returned for xNode. In XML multiple elements may have the same node name.
  # In R they will appear as a list of values for a single active binding.
  argumentList = list()


  # ChildNodes will always be properties or relationships. Their names all begin with a lower case letter but the XML may upcase them.
  # the DataType of a ChildNode could be a DDI4 R6 class or an atomic value 

  # an R6 object of type xNode, 
  # it needs to exist here to be able to retrieve datatype and structure
  # and attributes after all child nodes have been processed. It should not be registered.
  r6Object <- xNodeClass$new(registerThisObject=FALSE)   
  
  for (childNode in xmlChildren(xNode)){
     childNodeName <- xmlName(childNode)[1]
     xmlChildNodeName <- childNodeName
     substr(childNodeName, 1, 1) <- tolower(substr(childNodeName, 1, 1))
       if(verbose)message(rep(" ",messageIndent+1), "child node: ", xmlChildNodeName )

            
      # a special cases, when the content is just text (a leaf node)
      # this may be  a relationship where "typeOfClass" is the attribute containing the class type
      # sometimes the content property collapses and just text appears
      # for now this may just be LanguageSpecificStringType and LanguageSpecificStructuredStringType
      # and SummaryStatistic hasStatistic.
      # For proper validation, one must look up the datatype of the collapsed "content" property
      
     if (childNodeName == "text") {               
 
       # a collapsed content       
         childDataType <-  r6Object$describe_initializeArguments()[["content"]]["datatype"]
         argumentList[["content"]] <- tryCatch(as(xmlValue(childNode),childDataType) , warning=function(w){message("Node ", xNodeName, ": ", childNodeName, "=", xmlValue(childNode), " cannot be coerced to ",  childDataType)}) 
         next
     }
 

      
     childDataType <-  r6Object$describe_initializeArguments()[[childNodeName]]["datatype"]
     childStructure <- r6Object$describe_initializeArguments()[[childNodeName]]["structure"]
     if(verbose)message(rep(" ",messageIndent+1), " childDataType: ", childDataType, " childStructure: ", childStructure )

       # a structure of "DdiUrn vector" indicates a DDI relationship (association). 
       # The datatype is the target class of the relationship 
       # and the content of the active binding will be a vector of DdiUrns.
     
     if (is.null(childStructure)){
          #  skip anything without a structure, but don't bother to warn about comments. 
       if (childNodeName == "comment")   next
       #   look here for substitution of an extension of an abstract class name for a property name where the datatype is the abstract class
       childDDI4Name <- paste("DDI4_", childNodeName, sep="", collapse="")
       substring(childDDI4Name,6,6) <- toupper(substring(childDDI4Name,6,6))
       if (! is.DDIClassName(childDDI4Name)){
         
          message("missing structure for unknown element name ", childNodeName,". ",  childStructure, " ignoring ", childNodeName  )
            next
       } else { 
       #   A DDI4 Class name here. Look for  aproperty which has the parent of this class as a datatype 
            message("this is a DDI4 class name instead of a property name. there is probably a property with an abstract class as its datatype having an extension:  ", childDDI4Name )
            next
       } 
     } 
      

        # a reference, note that there is no validation 
     if(childStructure == "DdiUrn vector") {      

       if(verbose)message(rep(" ",messageIndent+1), "DdiUrn vector" )
       argumentList[[childNodeName]] <- c(argumentList[[childNodeName]], xmlValue(childNode))                
       
       # If a child node object is an R6 object: 
          # some datatypes like dates are structured as R6 classes with atomic content but constrained by enumeration or regex

     } else if (childStructure == "Enumeration vector"){   

       if(verbose)message(rep(" ",messageIndent+1), "Enumeration vector" )
       # Enumerations and regular expressions are a special case. 
         argumentList[[childNodeName]] <- c(argumentList[[childNodeName]], as(xmlValue(childNode), "character") )

     } else if (childStructure == "Regex vector"){   

       if(verbose)message(rep(" ",messageIndent+1), "Regex vector" )
       # Regular expressions are a special case. 
         argumentList[[childNodeName]] <- c(argumentList[[childNodeName]], as(xmlValue(childNode), "character") )

       # If a child node object is a structured R6 object, use recursion to get down to the leaf value (atomic) 
     } else if (childStructure == "StructuredDatatype list"){   

       if(verbose)message(rep(" ",messageIndent+1), "StructuredDatatype list" )
         argumentList[[childNodeName]] <- append(argumentList[[childNodeName]], 
                                               list(ddiXmlNodeToObject(childNode, 
                                                                          get(childDataType), 
                                                                          silentState=silentState,
                                                                          verbose=verbose,
                                                                          messageIndent=messageIndent)))

 
       # an atomic type 
     } else if (childStructure == "vector"){
       if(verbose)message(rep(" ",messageIndent+1), "atomic vector" )
       # this is the terminal (leaf) case, vector types are atomic and have no (sub)-components 
       # the argument value for this is a simple atomic vector
       argumentList[[childNodeName]] <- c(argumentList[[childNodeName]], tryCatch(as(xmlValue(childNode),childDataType) , warning=function(w){message("Node ", xNodeName, ": ",childNodeName,  "=",  xmlValue(childNode), " cannot be coerced to ",  childDataType)}))
                           
     }  else {
       stop("An unknown structure was encountered in parsing child node ", childNodeName)
     }
  }     


  # Attributes always have a cardinality of 1, a single value  

  r6Object <- xNodeClass$new(registerThisObject=FALSE) 
  for (attributeName in names(xNodeAttributes)){

    if(verbose)message(rep(" ",messageIndent+1), paste("attribute: ", attributeName) )

    #  some attributes in the DDI4 XML are XML binding specific, ignore those
    if (attributeName %in% names(r6Object$describe_initializeArguments())){
    
      attributeDataType <- r6Object$describe_initializeArguments()[[attributeName]]["datatype"]
      attributeStructure <- r6Object$describe_initializeArguments()[[attributeName]]["structure"]
        
      if (attributeStructure == "Enumeration vector"){   
         # Enumeration type 
        argumentList[[attributeName]] <- as(xNodeAttributes[attributeName],"character")

     } else if (attributeStructure == "Regex vector"){   
         # regex type 
        argumentList[[attributeName]] <- as(xNodeAttributes[attributeName],"character")

     } else if (attributeStructure == "StructuredDatatype list"){   
       stop("The attribute ",  xNodeName, "@", attributeName,  " had a datatype of StructuredDatatype,  ", attributeDataType)

     }  else {
         # atomic type
        argumentList[[attributeName]] <- tryCatch(as(xNodeAttributes[attributeName],attributeDataType), warning=function(w){message("Attribute  ", xNodeName, ": ",attributeName, "=", xNodeAttributes[attributeName], " cannot be coerced to ",  attributeDataType)})
     }
    }  
  }

  argumentList[["silentState"]] <- silentState
  returnR6Object <- do.call(xNodeClass$new,argumentList)
  
  returnR6Object 
}



#' @title quoteBackslashEscape - Quote the quote and backslash characters for inclusion if a quoted string in a script.  (Not Exported)
#' 
#' @description
#' replace " with \" and \ with \\.    This function is not exported. It is only available to other functions within th package.   
#' in a string that is to be printed inside of quotes


#' 
#' @param string the string to be modified 
#' 
#' @return a string with the quote and backslash character quoted
#' 
 
#' 


quoteBackslashEscape <- function(string) {
  gsub( "([\"\\])",  "\\\\\\1",  string)
}


#' @title describeObject - Generate a script that recreates a DDI4 R6 object
#' 
#' @description
#' Generates a script that recreates a DDI4 R6 object.
#' @param DdiObject a DDI4 R6 object (required).
#' An object of class R6 from the DDI4 package.
#' 
#' @return a vector (script) with embedded line endings.
#' 
#' @export
#' @examples
#' describeObject(myInstanceVariable)
#'   where myInstanceVariable was created as myInstanceVariable <- InstanceVariable$new(....)

describeObject <- function(DdiObject,objectNumber=1, lineStart=""){

  objectClass <- class(DdiObject)[1]

# use the DdiObject$name property as the object name if it is populated
  if (is.null(DdiObject$name)){
    objectName <- paste(objectClass, as.character(objectNumber), sep="", collapse="")
  } else {
    objectName <- make.names(DdiObject$name[[1]]$content)
  }

    # return a list of all public fields for the object DdiObject
  publicNames <- ls(DdiObject, all.names = TRUE)
  
    # return a list of the active bindings for the object DdiObject  
  activeNames <- publicNames[vapply(publicNames,bindingIsActive, FUN.VALUE = logical(1), env=DdiObject)]
  
    #  a vector indicating whether the vector has a values
  isParameterPresent <- sapply(activeNames, function(name){!is.null(DdiObject[[name]])})
  
    # the names of the populated active bindings
      # DdiUrn is created automatically from id, agency, and version when they are entered
      # never directly
      # only process parameters with values
  populatedNames <-  activeNames[isParameterPresent & activeNames != "DdiUrn"]
  
    # the structure of each populated active binding
  structures <- sapply(populatedNames, function(name){
    DdiObject$describe_initializeArguments()[[name]][['structure']]
  })
  
    # the datatype of each populated active binding
  datatypes <- sapply(populatedNames, function(name){
    DdiObject$describe_initializeArguments()[[name]][['datatype']]
  })  
  
    # the names of the atomic values, these will have vector parameters
  vectorNames <-  populatedNames[grepl("vector", structures[populatedNames])] 
  
    # the names of the list structures, these are references to DDI R6 objects
  listNames <-  populatedNames[grepl("list", structures[populatedNames])]  

# relationships and atomic parameters are vectors
# a vector parameter to the new function will look like param1 <-  c(val1, val2)

# StructuredDatatypes look like this(note that arguments are created first):
# param2 < paramClass$new()
# param3 < paramClass$new()
# list(param1, param2 )

  
  # string to precede each line
  lineStart <-  paste(lineStart," ")
       
  parameterNumber <- 0  
  script <- NULL 

  
# *** DDI4 StructuredDatatype
# All of the list names need to be processed first. 
# These initiate a recusive chain (depth first)  

  for (name in listNames){
    parameterNumber <- parameterNumber +1
    fieldList <- .subset2(DdiObject,name)
 
    if (R6::is.R6(fieldList[[1]]) &
        !("Identifiable" %in% class (fieldList[[1]]))){                 
            
      names(fieldList) <- paste0(class(fieldList[[1]])[1],1:length(fieldList))      
         
# recurse through all child classes
         
      childScript <- NULL
      
      for (ixChild in 1:length(fieldList)){
        childScript <-   describeObject(fieldList[[ixChild]],
                                           objectNumber=ixChild, 
                                           lineStart=lineStart)
        script <- c(script, childScript)                                            
      }
             
      listScript <- paste0(lineStart,
                           paste("param_",name, objectNumber, sep="", collapse="") ,
                          ' <- list(', 
                          paste0(names(fieldList), 
                                 collapse=', '),
                          ')'         
                          )                          
      script <- c(script, listScript)
    }    
  }

# *** DDI4 PROPERTIES

  for (name in vectorNames){
    parameterNumber <- parameterNumber +1
    fieldList <- .subset2(DdiObject,name)

       #   character vector with quotes 
    if (class(fieldList) %in% c("character", "complex", "double",  "integer", "logical", "numeric")){
    paramScript <- paste0(lineStart,
                           paste("param_",name, objectNumber, sep="", collapse="") ,
                          ' <- as(c("', 
                          paste0(quoteBackslashEscape(fieldList), 
                                 collapse='", "'),
                          '"), "', 
                          class(fieldList),  
                          '")'         
                          )        
    } else if  (class(fieldList) == "Date" )  {
       #   other atomic vector without quotes, e.g. logical 
    paramScript <- paste0(lineStart,
                           paste("param_",name, objectNumber, sep="", collapse="") ,
                          ' <- as.Date(c(', 
                          paste0(fieldList, 
                                 collapse=', '),
                          '))'         
                          )        
    } else if  (class(fieldList) == "POSIXlt" )  {
       #   other atomic vector without quotes, e.g. logical 
    paramScript <- paste0(lineStart,
                           paste("param_",name, objectNumber, sep="", collapse="") ,
                          ' <- as.POSIXlt(c(', 
                          paste0(fieldList, 
                                 collapse=', '),
                          '))'         
                          )        
    } else { 
    message("In describeObject unknown datatype ", class(fieldList), " for object", objectName ) 
    }  

    script <- c(script, paramScript)
  }
   

  

# *** Now create the new object, using the parameters generated above

  script <- append(script, 
                   paste0(lineStart,
                          objectName,
                          " <- ",
                          objectClass,
                          "$new(",
                          paste0(sapply(
                                  populatedNames,
                                  function(name){
                                    paste(name, "=", "param_", name, as.character(objectNumber), sep="")
                                  }                                  
                                ),
                              sep="", 
                              collapse=", "   
                          ),
                          ")\n"
                  )                  
            )
  script
}


#' @title validateObjectReferences - Validate a DDI4 Object
#' 
#' @description 
#' This validates the relationships (associations) from a DDI4 Object to other DDI4 Objects.
#' For each relationship it will check for the proper number of references and
#' if a target object is in the local registry it will check for proper class.
#' If an object is not in the local registry it will be checked against the XML catalog.
#' If it cannot be found a message will report so.
#' 
#' @param DDI R6 Object (required)
#' An object of class R6.
#' 
#' @export
#' @examples validateObjectReferences(InstanceVariable37) 
#' 
validateObjectReferences <- function( DdiObject ) {
  isValid <- TRUE
  
  # find the relationships. These will have a structure of DDiUrn vector
  for (name in names(DdiObject$describe_initializeArguments())){
    if ( DdiObject$describe_initializeArguments()[[name]]["structure"] == "DdiUrn vector" ){
      min <- as.numeric(DdiObject$describe_initializeArguments()[[name]]["min"])
      max <- as.numeric(DdiObject$describe_initializeArguments()[[name]]["max"])
      datatype <- DdiObject$describe_initializeArguments()[[name]]["datatype"]
      
      referenceVector <- DdiObject[[name]]
      
      if (length(referenceVector) > max){
        isValid <- FALSE
        message("The number of references for relationship ", name, ", ", length(referenceVector), ", exceeds the maximum allowed")
      }
      if (length(referenceVector) < min){
        isValid <- FALSE
        message("The number of references for relationship ", name, ", ", length(referenceVector), ", is less than the minimum allowed")
      }
      
      # check references
      
      for (referenceString in referenceVector){
        referencedObject <- getObject(referenceString)
        if (is.null(referencedObject)){
        # could not find the referenced object in the local registry
          URL <- resolveDdiUrn(referenceString, DefaultXmlCatalogFile)
          if (is.null(URL)){
            # could not find it through the XML catalog
            isValid <- FALSE
            message("For relationship ", name, ", the reference '", referenceString, ", could not be found locally or through the XML catalog. It will have to be validated manually." )
          } else {
            isValid <- NULL
            message("Note: for relationship ", name, " must be validated manually. It can be found at : ", URL) 
          }
        } else {
        # in the local registry, is it the right class?
          if(! datatype %in% class(referencedObject)){
            
            isValid <- NULL
            message("Note: for relationship ", name, " ", datatype, " must appear in the class hierarchy of ", referenceString, ": ", paste(class(referencedObject), sep="", collapse=", "))
          }
        }
      }
    }
  }
  isValid
}



#' @title validateRegistry - Validates the references of all objects in the Registry
#' 
#' @description
#' Calls listDdiUrn to get the URNs of all objects in the registry.
#' Then makse a list of the corresponding objects 
#' and calls validateObjectReferences on each one. Prints messages for any problems.
#'  No parameters.

#' 
#' @return a logical vector of the validity of each object in the DDI4 R6 object registry
#' 
#' @examples validateRegistry()
 
#' 
#' @export


validateRegistry <- function(){
  objectList <- lapply(listDdiUrn(), getObject)
  validVector <- sapply(objectList , validateObjectReferences)
  invisible(validVector)
}



#' @title importDdi4XmlFile - Import A DDI4 XML file into R
#' 
#' @description
#' From a DDI4 XML instance, import everything into DDI4 R6 objects
#' A DDI4 XML instance may contain a single DDI4 object or
#' a DDI element containing a number of objects 

#' 
#' @param xmlFile the name of the file from which to read the DDI XML 
#' This parameter is passed to xmlInternalTreeParse which has the following constraints:
#' "The name of the file containing the XML contents. 
#' This can contain \~ which is expanded to the user's home directory. 
#' It can also be a URL.  See isURL. 
#' Additionally, the file can be compressed (gzip) and is read directly without the user having to de-compress (gunzip) it."
#' 
#' @param rScriptFile  the name of the file to receive an R script that reproduces the imported R6 objects. 
#' 
#' @param verbose  display messages about the objects as they are imported. Potentially useful with problematic XML. 
#' 
#' @return a list of DDI4 R6 objects
#' 
#' @examples importDdi4XmlFile("C:\\ddrive\\projects\\various\\DDI\\DDI4\\modelingAndTC\\AESuseCase\\ANES2017_2018_09_12A.xml")  
 
#' 
#' @export

importDdi4XmlFile <- function(xmlFile, rScriptFile=NULL, verbose=FALSE){
  DDIdoc <- xmlInternalTreeParse(xmlFile)
  root <- xmlRoot(DDIdoc)
  
    # when verbose display all messages
    silentState <- ! verbose


  if (! is.null(rScriptFile)){
    writeScript <- TRUE
    scriptConnection <- file(rScriptFile, open="w")
  } else {
    writeScript <- FALSE
  }
  
    # if writing a script make sure the appropriate libraries are loaded. 
  if(writeScript) {
    script <- c("library(R6)", 
                "library(uuid)", 
                "library(XML)", 
                "library(RCurl)", 
                "library(stringr)", 
                "library(DDI4R)") 
    writeLines(script,  scriptConnection, sep = "\n") 
    script <- c("\n#R Script imported from:  ", rScriptFile  )
#    writeLines(script,  scriptConnection, sep = "\n#") 
  }
  
  rootName <- xmlName(root)
  returnList <- list()  

  if(verbose) message("processing root ", rootName  )      

    messageIndent <- 0

# ***  a DDI "document" with a root element of DDI and immediate child elements of DDI4 classes  
  if (rootName == "DDI"){
    # this is a DDI "document" containing several DDI4 elements
    # the first child element should be a DocumentInformation element 
    
    nodeNumber <- 0

  if(writeScript) {
    script <- c("\n# A DDI \"document\" containing several DDI4 elements" ,"  the first child element should be a DocumentInformation element"  )
    writeLines(script,  scriptConnection, sep = "\n#") 
  }

    for (childNode in xmlChildren(root)){
      nodeNumber <- nodeNumber + 1
      childNodeName <- xmlName(childNode)

      if(writeScript) {
        script <- paste("\n\n#  vvvvv ------ Beginning elment:  ", childNodeName, "------ vvvvv\n", sep=" ", collapse=" " )
        writeLines(script,  scriptConnection) 
      }
      if(verbose) message(" child node ", nodeNumber, " line ", getLineNumber(childNode), " '", childNodeName, "'" )      

        # skip the comments
      if (childNodeName == "comment") next
      r6Object <- ddiXmlNodeToObject(childNode, silentState=silentState, verbose=verbose, messageIndent=messageIndent)
      returnList[[paste(childNodeName, as.character(nodeNumber), sep="")]] <- r6Object
      if(writeScript) {
        writeLines(describeObject(r6Object, objectNumber=nodeNumber),  scriptConnection, sep = "\n") 
        script <- paste("#  ^^^^^ ------ Ending elment:  ", childNodeName, "------ ^^^^^\n\n", sep=" ", collapse=" " )
        writeLines(script,  scriptConnection) 
      }
    }
    
  } else {
        
# ***  a DDI4 class is the root 
    # this must be a single DDI4 element
  if(writeScript) {
    script <- c("\n# This must be a single DDI4 element"  )
    writeLines(script,  scriptConnection, sep = "\n#") 
  }
    singletonNode <- root
    singletonNodeName <- xmlName(singletonNode)
    returnList <- list()
    r6Object <- ddiXmlNodeToObject(root, silentState=silentState, verbose=verbose, messageIndent=messageIndent)
    returnList[[singletonNodeName]] <- r6Object
    if(writeScript) writeLines(describeObject(r6Object, objectNumber=1), scriptConnection, sep = "\n")
  }
  if(writeScript){
    writeLines("#____________________________ Validating registered objects ______________________\n", scriptConnection)
    writeLines("validateRegistry()", scriptConnection)
    close(scriptConnection)
  }
  if(!verbose){
    message("__________________ validating imported classes _______________")
    sapply(returnList, validateObjectReferences)
  }

  returnList
}

#' @title insertDdiXmlNode - Insert a Ddi4 object into an XML Document
#' 
#' @description
#' Inserts a DDI4 R6 object into an XML document file
#' @param DdiObject a DDI4 R6 object (required).
#' @param ObjectNodeName - the node name of the object in XML
#' @param XmlParent an XMLInternalDocument. Can be a sub-node in a larger XML document (required).
#' 
#' @return "an R object that points to the C-Level structure instance" (see newXMLDoc in the XML package).
#' 
#' @examples
#' insertDdiXmlNode(DdiObject, ObjectNodeName XmlDoc)
#'
#' @export

#' @examples insertDdiXmlNode(DdiObject=InstVar$unitOfMeasurement, ObjectNodeName="UnitOfMeawsureMent", XmlParent=xDoc ) 
#' 
insertDdiXmlNode <- function(DdiObject, ObjectNodeName, XmlParent){
		  
  if(!is.null(DdiObject)){
  
    # flag identifiable objects 
    ObjectIsIdentifiable <- DdiObject$get_ClassGroup() == "identifiable"
  
    DdiObjectNode <- XML::newXMLNode(name=ObjectNodeName,
                                    parent=XmlParent)

    # return a list of all public fields for the object DdiObject

    public_names <- ls(DdiObject, all.names = TRUE)
  
    # return a list of the active bindings for the object DdiObject

    active_names <- public_names[vapply(public_names,bindingIsActive, FUN.VALUE = logical(1), env=DdiObject)]
    #  drop DdiUrn from the list of active names to process
    active_names <- active_names[! active_names=="DdiUrn"]

    #DDI4 XML requires that the first elements of an identifiable element be
    #  agency, id, and version in that order
    # identifiable objects will have these properties
    
    if (ObjectIsIdentifiable){
      XML::newXMLNode(name="Agency",
                  text=.subset2(DdiObject,"agency"),
                  parent=DdiObjectNode)
      XML::newXMLNode(name="Id",
                  text=.subset2(DdiObject,"id"),
                  parent=DdiObjectNode)            
      XML::newXMLNode(name="Version",
                  text=.subset2(DdiObject,"version"),
                  parent=DdiObjectNode)
    }

    # check if this object has a collapsed content field, with all properties as attributes
    collapseContent <- TRUE
    # If any property has a cardinality greater than 1, then cannot collapse
    # a special case is translationSourceLanguage which has a cardinality of 0..n but is alloed to be an attribute. 
    # If there are relationships then cannot collapse 
    for (fieldName in active_names){
      if ( fieldName != "translationSourceLanguage" & 
         (as.numeric(DdiObject$describe_initializeArguments()[[fieldName]]['max']) > 1) ) collapseContent <- FALSE
      if ( ! (DdiObject$describe_initializeArguments()[[fieldName]]['structure'] %in% c("vector", "Regex vector", "Enumeration vector"))) collapseContent <- FALSE 
    }   
    # there must be a content field to allow compression
    if ( ! "content" %in%  active_names) collapseContent <- FALSE
    if(collapseContent){   
      #message( " Note - The Content property for   ", ObjectNodeName, " has been collapsed. All other properties become attributes. "  )
      XML::newXMLCommentNode(text=   paste0(" The content property for  ", ObjectNodeName, " has been collapsed into a text node. All other properties become attributes. " ) , 
         parent=XML::xmlParent(DdiObjectNode))
    }   

    
    #process each of the fields (properties and associations) of this object           
    for (fieldName in active_names){
    
      # The DDI4 XML has all tags upper case
      fieldNodeName <- fieldName
      substring(fieldNodeName,1,1) <- toupper(substring(fieldNodeName,1,1))
    
      # Check the structure of each field (property or relationship) of the object 
      # There are three subcases listed below
      # For Structured Datatypes use recursion to fill in the subtree

      # the structure (datatype) for this active binding class
       fieldStructure <- DdiObject$describe_initializeArguments()[[fieldName]]['structure']

       
      # the values for this field
       fieldValues <- .subset2(DdiObject,fieldName)
       
      # ignore empty fields agency, id version and DdiUrn
       if ( (length(fieldValues) == 0) |
            (fieldName == "agency") |
            (fieldName == "id") |
            (fieldName == "version") |
            (fieldName == "DdiUrn")             
          ){
       }

       
      # associations (references by DdiUrn) "DdiUrn vector"
      # DDI4 XML requires a typeofClass attribute
      # this can be found by looking up the object from the URN and capturing its class
      #  In DDI4 XML there is no "DDI4_" prefix. Delete it.
       else if(fieldStructure == "DdiUrn vector"){   
         sapply(fieldValues, 
                function(val){
                  referencedClass <- gsub("^DDI4_", "", class(DDI4R::getObject(val))[1])
                  XML::newXMLNode(name=fieldNodeName,
                    text=val,
                    attrs=c(typeOfClass=referencedClass),
                    parent=DdiObjectNode)
                }
         )          
    
       }
       
          
      # atomic structures: "vector", "Regex vector", and "Enumeration vector"
      # logical values need to be lower case in XML
      # there is also a special case for collapsing content fields 
       else if(is.atomic(fieldValues)){
         if(! collapseContent) sapply(fieldValues, 
                function(val){
                  if(is.logical(val))textVal <- tolower(as.character(val))
                  else textVal <- as.character(val)                   
                  XML::newXMLNode(name=fieldNodeName,
                  text=textVal,
                  parent=DdiObjectNode)
                }
         )       
         else {
                # a single content field value becomes the the text of the object 
           if(is.logical(fieldValues[1])) textVal <- tolower(as.character(fieldValues[1]))
           else textVal <- as.character(fieldValues[1])                   

           if (fieldName == "content") addChildren(node=DdiObjectNode, textVal)
                # All other properties become attributes 
           else { 
             # message("attribute ", fieldNodeName,  " = ", textVal  )
             # the first character of each attribute name must be lower case 
             lowerFieldNodeName <- fieldNodeName 
             substring(lowerFieldNodeName,1,1) <- tolower(substring(lowerFieldNodeName,1,1)) 
             names(textVal) <- lowerFieldNodeName 
             XML::addAttributes(node=DdiObjectNode, .attrs=textVal, append=TRUE)
           }
         }
       }
    
       
      # Non-identifiable DDI4 classes  "StructuredDatatype list" - travel down the subtree
        else if(fieldStructure == "StructuredDatatype list"){
          # Insert each of the objects in fieldValues as complex elements
          sapply(seq_along(fieldValues),
                function(ixValue){
                    # Check for a reference to an abstract StructuredDatatype, if so the XML Node Name is the class name of the extension
                  if (DdiObject$describe_initializeArguments()[[fieldName]]['isAbstractDatatype']){   
                     fieldClassName = class(.subset2(DdiObject, fieldName)[[ixValue]])[1]
                     fieldClassAncestry = paste(class(.subset2(DdiObject, fieldName)[[ixValue]]), sep=", ", collapse=" " )
                     fieldNodeName = substring(fieldClassName, 6, nchar(fieldClassName))
 #message(xmlName(DdiObjectNode), " Note - Abstract extension used for  ", fieldName, ": ", fieldNodeName  )
                     XML::newXMLCommentNode(text=   paste0(" Note - field  ", fieldName, " has been replaced by the class name of an extension of its datatype. The hierarchy for the extension class is: ", fieldClassAncestry ) , 
                       parent=DdiObjectNode)
                  }
                  insertDdiXmlNode(.subset2(DdiObject, fieldName)[[ixValue]], 
                                   fieldNodeName,
                                   XmlParent=DdiObjectNode )
                }
          )      
       }     
       
       else{
         message("unknown field type for object ",DdiObject$DdiUrn, ",  field ", fieldName )
       }
    }
  
  
  
  } else {
    message("DDI4 object not found from DDI URN")
    DdiObjectNode <- NULL
  }
  
  
  invisible(DdiObjectNode)
}






#' @title exportObjectXml - Export a list of Ddi4 objects to XML
#' 
#' @description
#' Exports a list of DDI4 R6 objects to an XML document file
#' @param DdiObjects a list of DDI4 R6 objects (required).
#' a vector of URNs such as returned by DDI4R::listDdiUrn()
#' @param XmlFile the qualified name of the XML file (required).
#' @param  docAgency the agency for the document DDI identifier.
#' @param docId the id for the document DDI identifier, defaults to a randm UUID.
#' @param docVersion the version for the document DDI identifier.
#' 
#' @return an XML document.
#' 
#' @examples
#' exportObjectXml(DdiObjects, XmlFile, docAgency="example.org", docVersion="1")
#'
#'
#' @export

exportObjectXml <- function(DdiObjects, XmlFile, docAgency="example.org", docId=NA, docVersion="example.org"){


# if ID is not specified generate a random UUID
  if(missing(docId)) docId <- uuid::UUIDgenerate(TRUE)

# note: encoding can't be specified here. Specify it in saveXML
  doc <- XML::newXMLDoc()
  topNode <- XML::newXMLNode(name="DDI",namespace=c("urn:ddi.org:4", xsi="http://www.w3.org/2001/XMLSchema-instance"), parent=doc)

  documentInfo <- XML::newXMLNode(name="DocumentInformation",  parent=topNode)
  XML::newXMLNode(name="Agency", text=docAgency, parent=documentInfo)
  XML::newXMLNode(name="Id", text=docId, parent=documentInfo)
  XML::newXMLNode(name="Version", text=docVersion, parent=documentInfo)
  XML::newXMLNode(name="OfType", text="DR0.2", parent=documentInfo)
 
#   Remove the prefix of DDI4_ from the DDI4R class name  to make the proper DDI4 class name (XML Node Name in this case).
  sapply(DdiObjects,
         function(DdiObject){
           insertDdiXmlNode(DdiObject=DdiObject,
                            ObjectNodeName=gsub("^DDI4_","", class(DdiObject)[1]), 
                            XmlParent=topNode  
           )                            
         }
        )
   
 
  XML::saveXML(doc, encoding="UTF-8", file=XmlFile, indent=TRUE)
 
}


#-------------------------------------------
# Enumerations follow
#-------------------------------------------


#' @title DDI4_CategoryRelationCode an Enumeration class.
#' 
#' @docType class 
#' @import R6   
#               R6Class is.R6Class  is.R6
#' @keywords enumeration 
#' @return Object of class \code{\link{R6Class}} with methods for exploring values in the enumeration. 
#' @format \code{\link{R6Class}} object. 
#' @export  
#' @description
#' Contains a list of valid values. 
#' This class has two functions and no initialization arguments: 
#'    isValidValue returns true if the argument is in the list
#'    get_ValidValues returns the values in the list.
#' 
#' @section Methods: 
#'    An Enumeration class contains a list of valid values and functions to validata against that list 
#'   \describe{
#'     \item{\code{new()}}{The default initialize method. An object of this enumeration class will be needed to use the class's methods.}
#'     \item{\code{get_ValidValues()}}{Returns the enumerated list.}

#'     \item{\code{isValidValue(val)}}{Returns TRUE if the contents of val is in the enumerated list, otherwise FALSE.}
#'     \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'     \item{\code{initialize()}}{This initializes an object of this class.}

#'   }


#' @examples
#' enum <- DDI4_CategoryRelationCode$new() 
#' enum$isValidValue("Bag") 
#' enum$get_ValidValues() 
#' 


#DDI4_CategoryRelationCode:
DDI4_CategoryRelationCode <- R6Class(classname="DDI4_CategoryRelationCode",
  public = list(

    isValidValue = function(val){
      val %in% private$allowedValues 
    },
    get_ClassGroup = function(){
      private$classGroup 
    },

    get_ValidValues = function(){
      private$allowedValues 
    },

    initialize = function(...){
      if (private$isAbstract){
        stop("The  abstract class, DDI4_CategoryRelationCode  may not be instantiated")
      }

    }
	
  ),
  private = list(
    isAbstract = FALSE,
    classGroup = c("enumeration"),
    allowedValues = c("Continuous","Interval","Nominal","Ordinal","Ratio")
  )
)



#' @title DDI4_CollectionType an Enumeration class.
#' 
#' @docType class 
#' @import R6   
#               R6Class is.R6Class  is.R6
#' @keywords enumeration 
#' @return Object of class \code{\link{R6Class}} with methods for exploring values in the enumeration. 
#' @format \code{\link{R6Class}} object. 
#' @export  
#' @description
#' Contains a list of valid values. 
#' This class has two functions and no initialization arguments: 
#'    isValidValue returns true if the argument is in the list
#'    get_ValidValues returns the values in the list.
#' 
#' @section Methods: 
#'    An Enumeration class contains a list of valid values and functions to validata against that list 
#'   \describe{
#'     \item{\code{new()}}{The default initialize method. An object of this enumeration class will be needed to use the class's methods.}
#'     \item{\code{get_ValidValues()}}{Returns the enumerated list.}

#'     \item{\code{isValidValue(val)}}{Returns TRUE if the contents of val is in the enumerated list, otherwise FALSE.}
#'     \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'     \item{\code{initialize()}}{This initializes an object of this class.}

#'   }


#' @examples
#' enum <- DDI4_CollectionType$new() 
#' enum$isValidValue("Bag") 
#' enum$get_ValidValues() 
#' 


#DDI4_CollectionType:
DDI4_CollectionType <- R6Class(classname="DDI4_CollectionType",
  public = list(

    isValidValue = function(val){
      val %in% private$allowedValues 
    },
    get_ClassGroup = function(){
      private$classGroup 
    },

    get_ValidValues = function(){
      private$allowedValues 
    },

    initialize = function(...){
      if (private$isAbstract){
        stop("The  abstract class, DDI4_CollectionType  may not be instantiated")
      }

    }
	
  ),
  private = list(
    isAbstract = FALSE,
    classGroup = c("enumeration"),
    allowedValues = c("Bag","Set")
  )
)



#' @title DDI4_ComputationBaseList an Enumeration class.
#' 
#' @docType class 
#' @import R6   
#               R6Class is.R6Class  is.R6
#' @keywords enumeration 
#' @return Object of class \code{\link{R6Class}} with methods for exploring values in the enumeration. 
#' @format \code{\link{R6Class}} object. 
#' @export  
#' @description
#' Contains a list of valid values. 
#' This class has two functions and no initialization arguments: 
#'    isValidValue returns true if the argument is in the list
#'    get_ValidValues returns the values in the list.
#' 
#' @section Methods: 
#'    An Enumeration class contains a list of valid values and functions to validata against that list 
#'   \describe{
#'     \item{\code{new()}}{The default initialize method. An object of this enumeration class will be needed to use the class's methods.}
#'     \item{\code{get_ValidValues()}}{Returns the enumerated list.}

#'     \item{\code{isValidValue(val)}}{Returns TRUE if the contents of val is in the enumerated list, otherwise FALSE.}
#'     \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'     \item{\code{initialize()}}{This initializes an object of this class.}

#'   }


#' @examples
#' enum <- DDI4_ComputationBaseList$new() 
#' enum$isValidValue("Bag") 
#' enum$get_ValidValues() 
#' 


#DDI4_ComputationBaseList:
DDI4_ComputationBaseList <- R6Class(classname="DDI4_ComputationBaseList",
  public = list(

    isValidValue = function(val){
      val %in% private$allowedValues 
    },
    get_ClassGroup = function(){
      private$classGroup 
    },

    get_ValidValues = function(){
      private$allowedValues 
    },

    initialize = function(...){
      if (private$isAbstract){
        stop("The  abstract class, DDI4_ComputationBaseList  may not be instantiated")
      }

    }
	
  ),
  private = list(
    isAbstract = FALSE,
    classGroup = c("enumeration"),
    allowedValues = c("MissingOnly","Total","ValidOnly")
  )
)



#' @title DDI4_MappingRelation an Enumeration class.
#' 
#' @docType class 
#' @import R6   
#               R6Class is.R6Class  is.R6
#' @keywords enumeration 
#' @return Object of class \code{\link{R6Class}} with methods for exploring values in the enumeration. 
#' @format \code{\link{R6Class}} object. 
#' @export  
#' @description
#' Contains a list of valid values. 
#' This class has two functions and no initialization arguments: 
#'    isValidValue returns true if the argument is in the list
#'    get_ValidValues returns the values in the list.
#' 
#' @section Methods: 
#'    An Enumeration class contains a list of valid values and functions to validata against that list 
#'   \describe{
#'     \item{\code{new()}}{The default initialize method. An object of this enumeration class will be needed to use the class's methods.}
#'     \item{\code{get_ValidValues()}}{Returns the enumerated list.}

#'     \item{\code{isValidValue(val)}}{Returns TRUE if the contents of val is in the enumerated list, otherwise FALSE.}
#'     \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'     \item{\code{initialize()}}{This initializes an object of this class.}

#'   }


#' @examples
#' enum <- DDI4_MappingRelation$new() 
#' enum$isValidValue("Bag") 
#' enum$get_ValidValues() 
#' 


#DDI4_MappingRelation:
DDI4_MappingRelation <- R6Class(classname="DDI4_MappingRelation",
  public = list(

    isValidValue = function(val){
      val %in% private$allowedValues 
    },
    get_ClassGroup = function(){
      private$classGroup 
    },

    get_ValidValues = function(){
      private$allowedValues 
    },

    initialize = function(...){
      if (private$isAbstract){
        stop("The  abstract class, DDI4_MappingRelation  may not be instantiated")
      }

    }
	
  ),
  private = list(
    isAbstract = FALSE,
    classGroup = c("enumeration"),
    allowedValues = c("CloseMatch","Disjoint","ExactMatch")
  )
)



#' @title DDI4_PointFormat an Enumeration class.
#' 
#' @docType class 
#' @import R6   
#               R6Class is.R6Class  is.R6
#' @keywords enumeration 
#' @return Object of class \code{\link{R6Class}} with methods for exploring values in the enumeration. 
#' @format \code{\link{R6Class}} object. 
#' @export  
#' @description
#' Contains a list of valid values. 
#' This class has two functions and no initialization arguments: 
#'    isValidValue returns true if the argument is in the list
#'    get_ValidValues returns the values in the list.
#' 
#' @section Methods: 
#'    An Enumeration class contains a list of valid values and functions to validata against that list 
#'   \describe{
#'     \item{\code{new()}}{The default initialize method. An object of this enumeration class will be needed to use the class's methods.}
#'     \item{\code{get_ValidValues()}}{Returns the enumerated list.}

#'     \item{\code{isValidValue(val)}}{Returns TRUE if the contents of val is in the enumerated list, otherwise FALSE.}
#'     \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'     \item{\code{initialize()}}{This initializes an object of this class.}

#'   }


#' @examples
#' enum <- DDI4_PointFormat$new() 
#' enum$isValidValue("Bag") 
#' enum$get_ValidValues() 
#' 


#DDI4_PointFormat:
DDI4_PointFormat <- R6Class(classname="DDI4_PointFormat",
  public = list(

    isValidValue = function(val){
      val %in% private$allowedValues 
    },
    get_ClassGroup = function(){
      private$classGroup 
    },

    get_ValidValues = function(){
      private$allowedValues 
    },

    initialize = function(...){
      if (private$isAbstract){
        stop("The  abstract class, DDI4_PointFormat  may not be instantiated")
      }

    }
	
  ),
  private = list(
    isAbstract = FALSE,
    classGroup = c("enumeration"),
    allowedValues = c("DecimalDegree","DecimalMinutes","DegreesMinutesSeconds","Feet","Meters")
  )
)



#' @title DDI4_RelationSpecification an Enumeration class.
#' 
#' @docType class 
#' @import R6   
#               R6Class is.R6Class  is.R6
#' @keywords enumeration 
#' @return Object of class \code{\link{R6Class}} with methods for exploring values in the enumeration. 
#' @format \code{\link{R6Class}} object. 
#' @export  
#' @description
#' Contains a list of valid values. 
#' This class has two functions and no initialization arguments: 
#'    isValidValue returns true if the argument is in the list
#'    get_ValidValues returns the values in the list.
#' 
#' @section Methods: 
#'    An Enumeration class contains a list of valid values and functions to validata against that list 
#'   \describe{
#'     \item{\code{new()}}{The default initialize method. An object of this enumeration class will be needed to use the class's methods.}
#'     \item{\code{get_ValidValues()}}{Returns the enumerated list.}

#'     \item{\code{isValidValue(val)}}{Returns TRUE if the contents of val is in the enumerated list, otherwise FALSE.}
#'     \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'     \item{\code{initialize()}}{This initializes an object of this class.}

#'   }


#' @examples
#' enum <- DDI4_RelationSpecification$new() 
#' enum$isValidValue("Bag") 
#' enum$get_ValidValues() 
#' 


#DDI4_RelationSpecification:
DDI4_RelationSpecification <- R6Class(classname="DDI4_RelationSpecification",
  public = list(

    isValidValue = function(val){
      val %in% private$allowedValues 
    },
    get_ClassGroup = function(){
      private$classGroup 
    },

    get_ValidValues = function(){
      private$allowedValues 
    },

    initialize = function(...){
      if (private$isAbstract){
        stop("The  abstract class, DDI4_RelationSpecification  may not be instantiated")
      }

    }
	
  ),
  private = list(
    isAbstract = FALSE,
    classGroup = c("enumeration"),
    allowedValues = c("AcyclicPrecedence","Equivalence","GeneralSpecific","List","ParentChild","Unordered","WholePart")
  )
)



#' @title DDI4_SexSpecificationType an Enumeration class.
#' 
#' @docType class 
#' @import R6   
#               R6Class is.R6Class  is.R6
#' @keywords enumeration 
#' @return Object of class \code{\link{R6Class}} with methods for exploring values in the enumeration. 
#' @format \code{\link{R6Class}} object. 
#' @export  
#' @description
#' Contains a list of valid values. 
#' This class has two functions and no initialization arguments: 
#'    isValidValue returns true if the argument is in the list
#'    get_ValidValues returns the values in the list.
#' 
#' @section Methods: 
#'    An Enumeration class contains a list of valid values and functions to validata against that list 
#'   \describe{
#'     \item{\code{new()}}{The default initialize method. An object of this enumeration class will be needed to use the class's methods.}
#'     \item{\code{get_ValidValues()}}{Returns the enumerated list.}

#'     \item{\code{isValidValue(val)}}{Returns TRUE if the contents of val is in the enumerated list, otherwise FALSE.}
#'     \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'     \item{\code{initialize()}}{This initializes an object of this class.}

#'   }


#' @examples
#' enum <- DDI4_SexSpecificationType$new() 
#' enum$isValidValue("Bag") 
#' enum$get_ValidValues() 
#' 


#DDI4_SexSpecificationType:
DDI4_SexSpecificationType <- R6Class(classname="DDI4_SexSpecificationType",
  public = list(

    isValidValue = function(val){
      val %in% private$allowedValues 
    },
    get_ClassGroup = function(){
      private$classGroup 
    },

    get_ValidValues = function(){
      private$allowedValues 
    },

    initialize = function(...){
      if (private$isAbstract){
        stop("The  abstract class, DDI4_SexSpecificationType  may not be instantiated")
      }

    }
	
  ),
  private = list(
    isAbstract = FALSE,
    classGroup = c("enumeration"),
    allowedValues = c("Feminine","GenderNeutral","Masculine")
  )
)



#' @title DDI4_ShapeCoded an Enumeration class.
#' 
#' @docType class 
#' @import R6   
#               R6Class is.R6Class  is.R6
#' @keywords enumeration 
#' @return Object of class \code{\link{R6Class}} with methods for exploring values in the enumeration. 
#' @format \code{\link{R6Class}} object. 
#' @export  
#' @description
#' Contains a list of valid values. 
#' This class has two functions and no initialization arguments: 
#'    isValidValue returns true if the argument is in the list
#'    get_ValidValues returns the values in the list.
#' 
#' @section Methods: 
#'    An Enumeration class contains a list of valid values and functions to validata against that list 
#'   \describe{
#'     \item{\code{new()}}{The default initialize method. An object of this enumeration class will be needed to use the class's methods.}
#'     \item{\code{get_ValidValues()}}{Returns the enumerated list.}

#'     \item{\code{isValidValue(val)}}{Returns TRUE if the contents of val is in the enumerated list, otherwise FALSE.}
#'     \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'     \item{\code{initialize()}}{This initializes an object of this class.}

#'   }


#' @examples
#' enum <- DDI4_ShapeCoded$new() 
#' enum$isValidValue("Bag") 
#' enum$get_ValidValues() 
#' 


#DDI4_ShapeCoded:
DDI4_ShapeCoded <- R6Class(classname="DDI4_ShapeCoded",
  public = list(

    isValidValue = function(val){
      val %in% private$allowedValues 
    },
    get_ClassGroup = function(){
      private$classGroup 
    },

    get_ValidValues = function(){
      private$allowedValues 
    },

    initialize = function(...){
      if (private$isAbstract){
        stop("The  abstract class, DDI4_ShapeCoded  may not be instantiated")
      }

    }
	
  ),
  private = list(
    isAbstract = FALSE,
    classGroup = c("enumeration"),
    allowedValues = c("Circle","LinearRing","Polygon","Rectangle")
  )
)



#' @title DDI4_SpatialObjectPairs an Enumeration class.
#' 
#' @docType class 
#' @import R6   
#               R6Class is.R6Class  is.R6
#' @keywords enumeration 
#' @return Object of class \code{\link{R6Class}} with methods for exploring values in the enumeration. 
#' @format \code{\link{R6Class}} object. 
#' @export  
#' @description
#' Contains a list of valid values. 
#' This class has two functions and no initialization arguments: 
#'    isValidValue returns true if the argument is in the list
#'    get_ValidValues returns the values in the list.
#' 
#' @section Methods: 
#'    An Enumeration class contains a list of valid values and functions to validata against that list 
#'   \describe{
#'     \item{\code{new()}}{The default initialize method. An object of this enumeration class will be needed to use the class's methods.}
#'     \item{\code{get_ValidValues()}}{Returns the enumerated list.}

#'     \item{\code{isValidValue(val)}}{Returns TRUE if the contents of val is in the enumerated list, otherwise FALSE.}
#'     \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'     \item{\code{initialize()}}{This initializes an object of this class.}

#'   }


#' @examples
#' enum <- DDI4_SpatialObjectPairs$new() 
#' enum$isValidValue("Bag") 
#' enum$get_ValidValues() 
#' 


#DDI4_SpatialObjectPairs:
DDI4_SpatialObjectPairs <- R6Class(classname="DDI4_SpatialObjectPairs",
  public = list(

    isValidValue = function(val){
      val %in% private$allowedValues 
    },
    get_ClassGroup = function(){
      private$classGroup 
    },

    get_ValidValues = function(){
      private$allowedValues 
    },

    initialize = function(...){
      if (private$isAbstract){
        stop("The  abstract class, DDI4_SpatialObjectPairs  may not be instantiated")
      }

    }
	
  ),
  private = list(
    isAbstract = FALSE,
    classGroup = c("enumeration"),
    allowedValues = c("AreaToArea","LineToArea","LineToLine","PointToArea","PointToLine","PointToPoint")
  )
)



#' @title DDI4_SpatialObjectType an Enumeration class.
#' 
#' @docType class 
#' @import R6   
#               R6Class is.R6Class  is.R6
#' @keywords enumeration 
#' @return Object of class \code{\link{R6Class}} with methods for exploring values in the enumeration. 
#' @format \code{\link{R6Class}} object. 
#' @export  
#' @description
#' Contains a list of valid values. 
#' This class has two functions and no initialization arguments: 
#'    isValidValue returns true if the argument is in the list
#'    get_ValidValues returns the values in the list.
#' 
#' @section Methods: 
#'    An Enumeration class contains a list of valid values and functions to validata against that list 
#'   \describe{
#'     \item{\code{new()}}{The default initialize method. An object of this enumeration class will be needed to use the class's methods.}
#'     \item{\code{get_ValidValues()}}{Returns the enumerated list.}

#'     \item{\code{isValidValue(val)}}{Returns TRUE if the contents of val is in the enumerated list, otherwise FALSE.}
#'     \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'     \item{\code{initialize()}}{This initializes an object of this class.}

#'   }


#' @examples
#' enum <- DDI4_SpatialObjectType$new() 
#' enum$isValidValue("Bag") 
#' enum$get_ValidValues() 
#' 


#DDI4_SpatialObjectType:
DDI4_SpatialObjectType <- R6Class(classname="DDI4_SpatialObjectType",
  public = list(

    isValidValue = function(val){
      val %in% private$allowedValues 
    },
    get_ClassGroup = function(){
      private$classGroup 
    },

    get_ValidValues = function(){
      private$allowedValues 
    },

    initialize = function(...){
      if (private$isAbstract){
        stop("The  abstract class, DDI4_SpatialObjectType  may not be instantiated")
      }

    }
	
  ),
  private = list(
    isAbstract = FALSE,
    classGroup = c("enumeration"),
    allowedValues = c("Line","LinearRing","Point","Polygon","Raster")
  )
)



#' @title DDI4_SpatialRelationSpecification an Enumeration class.
#' 
#' @docType class 
#' @import R6   
#               R6Class is.R6Class  is.R6
#' @keywords enumeration 
#' @return Object of class \code{\link{R6Class}} with methods for exploring values in the enumeration. 
#' @format \code{\link{R6Class}} object. 
#' @export  
#' @description
#' Contains a list of valid values. 
#' This class has two functions and no initialization arguments: 
#'    isValidValue returns true if the argument is in the list
#'    get_ValidValues returns the values in the list.
#' 
#' @section Methods: 
#'    An Enumeration class contains a list of valid values and functions to validata against that list 
#'   \describe{
#'     \item{\code{new()}}{The default initialize method. An object of this enumeration class will be needed to use the class's methods.}
#'     \item{\code{get_ValidValues()}}{Returns the enumerated list.}

#'     \item{\code{isValidValue(val)}}{Returns TRUE if the contents of val is in the enumerated list, otherwise FALSE.}
#'     \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'     \item{\code{initialize()}}{This initializes an object of this class.}

#'   }


#' @examples
#' enum <- DDI4_SpatialRelationSpecification$new() 
#' enum$isValidValue("Bag") 
#' enum$get_ValidValues() 
#' 


#DDI4_SpatialRelationSpecification:
DDI4_SpatialRelationSpecification <- R6Class(classname="DDI4_SpatialRelationSpecification",
  public = list(

    isValidValue = function(val){
      val %in% private$allowedValues 
    },
    get_ClassGroup = function(){
      private$classGroup 
    },

    get_ValidValues = function(){
      private$allowedValues 
    },

    initialize = function(...){
      if (private$isAbstract){
        stop("The  abstract class, DDI4_SpatialRelationSpecification  may not be instantiated")
      }

    }
	
  ),
  private = list(
    isAbstract = FALSE,
    classGroup = c("enumeration"),
    allowedValues = c("Contains","Disjoint","Equals","Intersects","Touches")
  )
)



#' @title DDI4_StringStructureType an Enumeration class.
#' 
#' @docType class 
#' @import R6   
#               R6Class is.R6Class  is.R6
#' @keywords enumeration 
#' @return Object of class \code{\link{R6Class}} with methods for exploring values in the enumeration. 
#' @format \code{\link{R6Class}} object. 
#' @export  
#' @description
#' Contains a list of valid values. 
#' This class has two functions and no initialization arguments: 
#'    isValidValue returns true if the argument is in the list
#'    get_ValidValues returns the values in the list.
#' 
#' @section Methods: 
#'    An Enumeration class contains a list of valid values and functions to validata against that list 
#'   \describe{
#'     \item{\code{new()}}{The default initialize method. An object of this enumeration class will be needed to use the class's methods.}
#'     \item{\code{get_ValidValues()}}{Returns the enumerated list.}

#'     \item{\code{isValidValue(val)}}{Returns TRUE if the contents of val is in the enumerated list, otherwise FALSE.}
#'     \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'     \item{\code{initialize()}}{This initializes an object of this class.}

#'   }


#' @examples
#' enum <- DDI4_StringStructureType$new() 
#' enum$isValidValue("Bag") 
#' enum$get_ValidValues() 
#' 


#DDI4_StringStructureType:
DDI4_StringStructureType <- R6Class(classname="DDI4_StringStructureType",
  public = list(

    isValidValue = function(val){
      val %in% private$allowedValues 
    },
    get_ClassGroup = function(){
      private$classGroup 
    },

    get_ValidValues = function(){
      private$allowedValues 
    },

    initialize = function(...){
      if (private$isAbstract){
        stop("The  abstract class, DDI4_StringStructureType  may not be instantiated")
      }

    }
	
  ),
  private = list(
    isAbstract = FALSE,
    classGroup = c("enumeration"),
    allowedValues = c("Other","RST","XHTML")
  )
)



#' @title DDI4_TableDirectionValues an Enumeration class.
#' 
#' @docType class 
#' @import R6   
#               R6Class is.R6Class  is.R6
#' @keywords enumeration 
#' @return Object of class \code{\link{R6Class}} with methods for exploring values in the enumeration. 
#' @format \code{\link{R6Class}} object. 
#' @export  
#' @description
#' Contains a list of valid values. 
#' This class has two functions and no initialization arguments: 
#'    isValidValue returns true if the argument is in the list
#'    get_ValidValues returns the values in the list.
#' 
#' @section Methods: 
#'    An Enumeration class contains a list of valid values and functions to validata against that list 
#'   \describe{
#'     \item{\code{new()}}{The default initialize method. An object of this enumeration class will be needed to use the class's methods.}
#'     \item{\code{get_ValidValues()}}{Returns the enumerated list.}

#'     \item{\code{isValidValue(val)}}{Returns TRUE if the contents of val is in the enumerated list, otherwise FALSE.}
#'     \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'     \item{\code{initialize()}}{This initializes an object of this class.}

#'   }


#' @examples
#' enum <- DDI4_TableDirectionValues$new() 
#' enum$isValidValue("Bag") 
#' enum$get_ValidValues() 
#' 


#DDI4_TableDirectionValues:
DDI4_TableDirectionValues <- R6Class(classname="DDI4_TableDirectionValues",
  public = list(

    isValidValue = function(val){
      val %in% private$allowedValues 
    },
    get_ClassGroup = function(){
      private$classGroup 
    },

    get_ValidValues = function(){
      private$allowedValues 
    },

    initialize = function(...){
      if (private$isAbstract){
        stop("The  abstract class, DDI4_TableDirectionValues  may not be instantiated")
      }

    }
	
  ),
  private = list(
    isAbstract = FALSE,
    classGroup = c("enumeration"),
    allowedValues = c("Auto","Ltr","Rtl")
  )
)



#' @title DDI4_TemporalRelationSpecification an Enumeration class.
#' 
#' @docType class 
#' @import R6   
#               R6Class is.R6Class  is.R6
#' @keywords enumeration 
#' @return Object of class \code{\link{R6Class}} with methods for exploring values in the enumeration. 
#' @format \code{\link{R6Class}} object. 
#' @export  
#' @description
#' Contains a list of valid values. 
#' This class has two functions and no initialization arguments: 
#'    isValidValue returns true if the argument is in the list
#'    get_ValidValues returns the values in the list.
#' 
#' @section Methods: 
#'    An Enumeration class contains a list of valid values and functions to validata against that list 
#'   \describe{
#'     \item{\code{new()}}{The default initialize method. An object of this enumeration class will be needed to use the class's methods.}
#'     \item{\code{get_ValidValues()}}{Returns the enumerated list.}

#'     \item{\code{isValidValue(val)}}{Returns TRUE if the contents of val is in the enumerated list, otherwise FALSE.}
#'     \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'     \item{\code{initialize()}}{This initializes an object of this class.}

#'   }


#' @examples
#' enum <- DDI4_TemporalRelationSpecification$new() 
#' enum$isValidValue("Bag") 
#' enum$get_ValidValues() 
#' 


#DDI4_TemporalRelationSpecification:
DDI4_TemporalRelationSpecification <- R6Class(classname="DDI4_TemporalRelationSpecification",
  public = list(

    isValidValue = function(val){
      val %in% private$allowedValues 
    },
    get_ClassGroup = function(){
      private$classGroup 
    },

    get_ValidValues = function(){
      private$allowedValues 
    },

    initialize = function(...){
      if (private$isAbstract){
        stop("The  abstract class, DDI4_TemporalRelationSpecification  may not be instantiated")
      }

    }
	
  ),
  private = list(
    isAbstract = FALSE,
    classGroup = c("enumeration"),
    allowedValues = c("TemporalContains","TemporalEquals","TemporalFinishes","TemporalMeets","TemporalOverlaps","TemporalPrecedes","TemporalStarts")
  )
)



#' @title DDI4_TextDirectionValues an Enumeration class.
#' 
#' @docType class 
#' @import R6   
#               R6Class is.R6Class  is.R6
#' @keywords enumeration 
#' @return Object of class \code{\link{R6Class}} with methods for exploring values in the enumeration. 
#' @format \code{\link{R6Class}} object. 
#' @export  
#' @description
#' Contains a list of valid values. 
#' This class has two functions and no initialization arguments: 
#'    isValidValue returns true if the argument is in the list
#'    get_ValidValues returns the values in the list.
#' 
#' @section Methods: 
#'    An Enumeration class contains a list of valid values and functions to validata against that list 
#'   \describe{
#'     \item{\code{new()}}{The default initialize method. An object of this enumeration class will be needed to use the class's methods.}
#'     \item{\code{get_ValidValues()}}{Returns the enumerated list.}

#'     \item{\code{isValidValue(val)}}{Returns TRUE if the contents of val is in the enumerated list, otherwise FALSE.}
#'     \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'     \item{\code{initialize()}}{This initializes an object of this class.}

#'   }


#' @examples
#' enum <- DDI4_TextDirectionValues$new() 
#' enum$isValidValue("Bag") 
#' enum$get_ValidValues() 
#' 


#DDI4_TextDirectionValues:
DDI4_TextDirectionValues <- R6Class(classname="DDI4_TextDirectionValues",
  public = list(

    isValidValue = function(val){
      val %in% private$allowedValues 
    },
    get_ClassGroup = function(){
      private$classGroup 
    },

    get_ValidValues = function(){
      private$allowedValues 
    },

    initialize = function(...){
      if (private$isAbstract){
        stop("The  abstract class, DDI4_TextDirectionValues  may not be instantiated")
      }

    }
	
  ),
  private = list(
    isAbstract = FALSE,
    classGroup = c("enumeration"),
    allowedValues = c("Auto","Inherit","Ltr","Rtl")
  )
)



#' @title DDI4_TotalityType an Enumeration class.
#' 
#' @docType class 
#' @import R6   
#               R6Class is.R6Class  is.R6
#' @keywords enumeration 
#' @return Object of class \code{\link{R6Class}} with methods for exploring values in the enumeration. 
#' @format \code{\link{R6Class}} object. 
#' @export  
#' @description
#' Contains a list of valid values. 
#' This class has two functions and no initialization arguments: 
#'    isValidValue returns true if the argument is in the list
#'    get_ValidValues returns the values in the list.
#' 
#' @section Methods: 
#'    An Enumeration class contains a list of valid values and functions to validata against that list 
#'   \describe{
#'     \item{\code{new()}}{The default initialize method. An object of this enumeration class will be needed to use the class's methods.}
#'     \item{\code{get_ValidValues()}}{Returns the enumerated list.}

#'     \item{\code{isValidValue(val)}}{Returns TRUE if the contents of val is in the enumerated list, otherwise FALSE.}
#'     \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'     \item{\code{initialize()}}{This initializes an object of this class.}

#'   }


#' @examples
#' enum <- DDI4_TotalityType$new() 
#' enum$isValidValue("Bag") 
#' enum$get_ValidValues() 
#' 


#DDI4_TotalityType:
DDI4_TotalityType <- R6Class(classname="DDI4_TotalityType",
  public = list(

    isValidValue = function(val){
      val %in% private$allowedValues 
    },
    get_ClassGroup = function(){
      private$classGroup 
    },

    get_ValidValues = function(){
      private$allowedValues 
    },

    initialize = function(...){
      if (private$isAbstract){
        stop("The  abstract class, DDI4_TotalityType  may not be instantiated")
      }

    }
	
  ),
  private = list(
    isAbstract = FALSE,
    classGroup = c("enumeration"),
    allowedValues = c("Partial","Total","Unknown")
  )
)



#' @title DDI4_TrimValues an Enumeration class.
#' 
#' @docType class 
#' @import R6   
#               R6Class is.R6Class  is.R6
#' @keywords enumeration 
#' @return Object of class \code{\link{R6Class}} with methods for exploring values in the enumeration. 
#' @format \code{\link{R6Class}} object. 
#' @export  
#' @description
#' Contains a list of valid values. 
#' This class has two functions and no initialization arguments: 
#'    isValidValue returns true if the argument is in the list
#'    get_ValidValues returns the values in the list.
#' 
#' @section Methods: 
#'    An Enumeration class contains a list of valid values and functions to validata against that list 
#'   \describe{
#'     \item{\code{new()}}{The default initialize method. An object of this enumeration class will be needed to use the class's methods.}
#'     \item{\code{get_ValidValues()}}{Returns the enumerated list.}

#'     \item{\code{isValidValue(val)}}{Returns TRUE if the contents of val is in the enumerated list, otherwise FALSE.}
#'     \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'     \item{\code{initialize()}}{This initializes an object of this class.}

#'   }


#' @examples
#' enum <- DDI4_TrimValues$new() 
#' enum$isValidValue("Bag") 
#' enum$get_ValidValues() 
#' 


#DDI4_TrimValues:
DDI4_TrimValues <- R6Class(classname="DDI4_TrimValues",
  public = list(

    isValidValue = function(val){
      val %in% private$allowedValues 
    },
    get_ClassGroup = function(){
      private$classGroup 
    },

    get_ValidValues = function(){
      private$allowedValues 
    },

    initialize = function(...){
      if (private$isAbstract){
        stop("The  abstract class, DDI4_TrimValues  may not be instantiated")
      }

    }
	
  ),
  private = list(
    isAbstract = FALSE,
    classGroup = c("enumeration"),
    allowedValues = c("Both","End","Neither","Start")
  )
)



#' @title DDI4_ValueRelationshipType an Enumeration class.
#' 
#' @docType class 
#' @import R6   
#               R6Class is.R6Class  is.R6
#' @keywords enumeration 
#' @return Object of class \code{\link{R6Class}} with methods for exploring values in the enumeration. 
#' @format \code{\link{R6Class}} object. 
#' @export  
#' @description
#' Contains a list of valid values. 
#' This class has two functions and no initialization arguments: 
#'    isValidValue returns true if the argument is in the list
#'    get_ValidValues returns the values in the list.
#' 
#' @section Methods: 
#'    An Enumeration class contains a list of valid values and functions to validata against that list 
#'   \describe{
#'     \item{\code{new()}}{The default initialize method. An object of this enumeration class will be needed to use the class's methods.}
#'     \item{\code{get_ValidValues()}}{Returns the enumerated list.}

#'     \item{\code{isValidValue(val)}}{Returns TRUE if the contents of val is in the enumerated list, otherwise FALSE.}
#'     \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'     \item{\code{initialize()}}{This initializes an object of this class.}

#'   }


#' @examples
#' enum <- DDI4_ValueRelationshipType$new() 
#' enum$isValidValue("Bag") 
#' enum$get_ValidValues() 
#' 


#DDI4_ValueRelationshipType:
DDI4_ValueRelationshipType <- R6Class(classname="DDI4_ValueRelationshipType",
  public = list(

    isValidValue = function(val){
      val %in% private$allowedValues 
    },
    get_ClassGroup = function(){
      private$classGroup 
    },

    get_ValidValues = function(){
      private$allowedValues 
    },

    initialize = function(...){
      if (private$isAbstract){
        stop("The  abstract class, DDI4_ValueRelationshipType  may not be instantiated")
      }

    }
	
  ),
  private = list(
    isAbstract = FALSE,
    classGroup = c("enumeration"),
    allowedValues = c("Equal","GreaterThan","GreaterThanOrEqualTo","LessThan","LessThanOrEqualTo","NotEqual")
  )
)



#' @title DDI4_WhiteSpaceRule an Enumeration class.
#' 
#' @docType class 
#' @import R6   
#               R6Class is.R6Class  is.R6
#' @keywords enumeration 
#' @return Object of class \code{\link{R6Class}} with methods for exploring values in the enumeration. 
#' @format \code{\link{R6Class}} object. 
#' @export  
#' @description
#' Contains a list of valid values. 
#' This class has two functions and no initialization arguments: 
#'    isValidValue returns true if the argument is in the list
#'    get_ValidValues returns the values in the list.
#' 
#' @section Methods: 
#'    An Enumeration class contains a list of valid values and functions to validata against that list 
#'   \describe{
#'     \item{\code{new()}}{The default initialize method. An object of this enumeration class will be needed to use the class's methods.}
#'     \item{\code{get_ValidValues()}}{Returns the enumerated list.}

#'     \item{\code{isValidValue(val)}}{Returns TRUE if the contents of val is in the enumerated list, otherwise FALSE.}
#'     \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'     \item{\code{initialize()}}{This initializes an object of this class.}

#'   }


#' @examples
#' enum <- DDI4_WhiteSpaceRule$new() 
#' enum$isValidValue("Bag") 
#' enum$get_ValidValues() 
#' 


#DDI4_WhiteSpaceRule:
DDI4_WhiteSpaceRule <- R6Class(classname="DDI4_WhiteSpaceRule",
  public = list(

    isValidValue = function(val){
      val %in% private$allowedValues 
    },
    get_ClassGroup = function(){
      private$classGroup 
    },

    get_ValidValues = function(){
      private$allowedValues 
    },

    initialize = function(...){
      if (private$isAbstract){
        stop("The  abstract class, DDI4_WhiteSpaceRule  may not be instantiated")
      }

    }
	
  ),
  private = list(
    isAbstract = FALSE,
    classGroup = c("enumeration"),
    allowedValues = c("Collapse","Preserve","Replace")
  )
)




#-------------------------------------------
# Regular Expression Classes follow
#-------------------------------------------


#' @title DDI4_Char a regular expression class.
#' 
#' @docType class 
#' @import R6    
#               R6Class 
#' @keywords regular expression regex 
#' @return Object of class \code{\link{R6Class}} with methods checking a character value for fitting the regular expression. 
#' @format \code{\link{R6Class}} object. 
#' @export  
#' @description
#' Contains a regular expression defining valid values. 
#' This class has two functions and no initialization arguments: 
#'    isValidValue returns true if the argument is in the list
#'    get_RegularExpression returns the values of the regular expression.
#' 
#' @section Methods:
#'    An regular Expression class contains a regular expression and functions to validata against that regular expression 
#'   \describe{
#'     \item{\code{new()}}{The default initialize method. An object of this enumeration class will be needed to use the class's methods.}
#'     \item{\code{get_RegularExpression()}}{Returns the regular expression.}

#'     \item{\code{isValidValue(val)}}{Returns TRUE if the contents of val match the regular expression, otherwise FALSE.}
#'     \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'     \item{\code{initialize()}}{This initializes an object of this class.}

#'   }


#' @examples
#' rex <- DDI4_Char$new() 
#' rex$isValidValue("Bag") 
#' rex$get_RegularExpression() 
#' 


#DDI4_Char:
DDI4_Char <- R6Class(classname="DDI4_Char",
  public = list(

    isValidValue = function(val){
      if (is.na(private$RegularExpressionString)) {
         TRUE
      } else {
        grepl(private$RegularExpressionString,val) 
      }

    },
    get_ClassGroup = function(){
      private$classGroup 
    },

    get_RegularExpression = function(){
      private$RegularExpressionString 
    },

    initialize = function(...){
      if (private$isAbstract){
        stop("The  abstract class, DDI4_Char  may not be instantiated")
      }

    }
	
  ),
  private = list(
    isAbstract = FALSE,
    classGroup = c("regex"),
    RegularExpressionString = NA
  )
)



#' @title DDI4_IsoDateType a regular expression class.
#' 
#' @docType class 
#' @import R6    
#               R6Class 
#' @keywords regular expression regex 
#' @return Object of class \code{\link{R6Class}} with methods checking a character value for fitting the regular expression. 
#' @format \code{\link{R6Class}} object. 
#' @export  
#' @description
#' Contains a regular expression defining valid values. 
#' This class has two functions and no initialization arguments: 
#'    isValidValue returns true if the argument is in the list
#'    get_RegularExpression returns the values of the regular expression.
#' 
#' @section Methods:
#'    An regular Expression class contains a regular expression and functions to validata against that regular expression 
#'   \describe{
#'     \item{\code{new()}}{The default initialize method. An object of this enumeration class will be needed to use the class's methods.}
#'     \item{\code{get_RegularExpression()}}{Returns the regular expression.}

#'     \item{\code{isValidValue(val)}}{Returns TRUE if the contents of val match the regular expression, otherwise FALSE.}
#'     \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'     \item{\code{initialize()}}{This initializes an object of this class.}

#'   }


#' @examples
#' rex <- DDI4_IsoDateType$new() 
#' rex$isValidValue("Bag") 
#' rex$get_RegularExpression() 
#' 


#DDI4_IsoDateType:
DDI4_IsoDateType <- R6Class(classname="DDI4_IsoDateType",
  public = list(

    isValidValue = function(val){
      if (is.na(private$RegularExpressionString)) {
         TRUE
      } else {
        grepl(private$RegularExpressionString,val) 
      }

    },
    get_ClassGroup = function(){
      private$classGroup 
    },

    get_RegularExpression = function(){
      private$RegularExpressionString 
    },

    initialize = function(...){
      if (private$isAbstract){
        stop("The  abstract class, DDI4_IsoDateType  may not be instantiated")
      }

    }
	
  ),
  private = list(
    isAbstract = FALSE,
    classGroup = c("regex"),
    RegularExpressionString = NA
  )
)



#---------------------------------------------
# class definitions follow for Content Classes
#---------------------------------------------


# 
# These classes carry the metadata content.
# 


#' @title DDI4_Access  Describes access to the annotated object
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Access}{Access} 

#'
#'   Definition
#'
#'   ============
#'
#'   Describes access to the annotated object. This item includes a confidentiality statement, descriptions of the access permissions required, restrictions to access, citation requirements, depositor requirements, conditions for access, a disclaimer, any time limits for access restrictions, and contact information regarding access.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A proprietary InstanceQuestion might have specific access restrictions.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   a:AccessType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Access$DdiUrn} returns the DDI URN of myDDI4_Access.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text
#'
#'                              \code{myDDI4_Access$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_Access' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Access$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{confidentialityStatement}}{ A read/write active binding.
#'        A statement regarding the confidentiality of the related data or metadata.
#'
#'                              \code{myDDI4_Access$confidentialityStatement <- list(myInternationalStructuredString)} sets the 'confidentialityStatement' property of 'myDDI4_Access' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Access$confidentialityStatement} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{accessPermission}}{ A read/write active binding.
#'        A link to a form used to provide access to the data or metadata including a statement of the purpose of the form.
#'
#'                              \code{myDDI4_Access$accessPermission <- list(myForm)} sets the 'accessPermission' property of 'myDDI4_Access' to a list of R6 Form class instances
#'
#'                              \code{myDDI4_Access$accessPermission} returns a list of of R6 : Form class instances}
#'   }
#'   \describe{
#'     \item{\code{restrictions}}{ A read/write active binding.
#'        A statement regarding restrictions to access. May be expressed in multiple languages and supports the use of structured content.
#'
#'                              \code{myDDI4_Access$restrictions <- list(myInternationalStructuredString)} sets the 'restrictions' property of 'myDDI4_Access' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Access$restrictions} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{citationRequirement}}{ A read/write active binding.
#'        A statement regarding the citation requirement. May be expressed in multiple languages and supports the use of structured content.
#'
#'                              \code{myDDI4_Access$citationRequirement <- list(myInternationalStructuredString)} sets the 'citationRequirement' property of 'myDDI4_Access' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Access$citationRequirement} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{depositRequirement}}{ A read/write active binding.
#'        A statement regarding depositor requirements. May be expressed in multiple languages and supports the use of structured content.
#'
#'                              \code{myDDI4_Access$depositRequirement <- list(myInternationalStructuredString)} sets the 'depositRequirement' property of 'myDDI4_Access' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Access$depositRequirement} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{accessConditions}}{ A read/write active binding.
#'        A statement regarding conditions for access. May be expressed in multiple languages and supports the use of structured content.
#'
#'                              \code{myDDI4_Access$accessConditions <- list(myInternationalStructuredString)} sets the 'accessConditions' property of 'myDDI4_Access' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Access$accessConditions} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{disclaimer}}{ A read/write active binding.
#'        A disclaimer regarding the liability of the data producers or providers. May be expressed in multiple languages and supports the use of structured content.
#'
#'                              \code{myDDI4_Access$disclaimer <- list(myInternationalStructuredString)} sets the 'disclaimer' property of 'myDDI4_Access' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Access$disclaimer} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{contactAgent}}{ A read/write active binding.
#'        The agent to contact regarding access including the role of the agent.
#'
#'                              \code{myDDI4_Access$contactAgent <- list(myAgentAssociation)} sets the 'contactAgent' property of 'myDDI4_Access' to a list of R6 AgentAssociation class instances
#'
#'                              \code{myDDI4_Access$contactAgent} returns a list of of R6 : AgentAssociation class instances}
#'   }
#'   \describe{
#'     \item{\code{validDates}}{ A read/write active binding.
#'        The date range or start date of the access description.
#'
#'                              \code{myDDI4_Access$validDates <- list(myDateRange)} sets the 'validDates' property of 'myDDI4_Access' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_Access$validDates} returns a list of of R6 : DateRange class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Access$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, purpose=NA, confidentialityStatement=NA, accessPermission=NA, restrictions=NA, citationRequirement=NA, depositRequirement=NA, accessConditions=NA, disclaimer=NA, contactAgent=NA, validDates=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Access <- R6Class(classname="DDI4_Access",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, purpose=NA, confidentialityStatement=NA, accessPermission=NA, restrictions=NA, citationRequirement=NA, depositRequirement=NA, accessConditions=NA, disclaimer=NA, contactAgent=NA, validDates=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Access may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(confidentialityStatement)){
        private$.confidentialityStatement <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=confidentialityStatement , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(accessPermission)){
        private$.accessPermission <- returnR6ObjectIfValid(validClass="DDI4_Form", proposedObjectList=accessPermission , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(restrictions)){
        private$.restrictions <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=restrictions , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(citationRequirement)){
        private$.citationRequirement <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=citationRequirement , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(depositRequirement)){
        private$.depositRequirement <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=depositRequirement , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(accessConditions)){
        private$.accessConditions <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=accessConditions , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(disclaimer)){
        private$.disclaimer <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=disclaimer , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(contactAgent)){
        private$.contactAgent <- returnR6ObjectIfValid(validClass="DDI4_AgentAssociation", proposedObjectList=contactAgent , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(validDates)){
        private$.validDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=validDates , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 10 properties: 
    accessConditions = function(val){
      if (!missing(val)){
        private$.accessConditions <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.accessConditions
      }
    },

    accessPermission = function(val){
      if (!missing(val)){
        private$.accessPermission <- returnR6ObjectIfValid(validClass="DDI4_Form", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.accessPermission
      }
    },

    citationRequirement = function(val){
      if (!missing(val)){
        private$.citationRequirement <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.citationRequirement
      }
    },

    confidentialityStatement = function(val){
      if (!missing(val)){
        private$.confidentialityStatement <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.confidentialityStatement
      }
    },

    contactAgent = function(val){
      if (!missing(val)){
        private$.contactAgent <- returnR6ObjectIfValid(validClass="DDI4_AgentAssociation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contactAgent
      }
    },

    depositRequirement = function(val){
      if (!missing(val)){
        private$.depositRequirement <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.depositRequirement
      }
    },

    disclaimer = function(val){
      if (!missing(val)){
        private$.disclaimer <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.disclaimer
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    restrictions = function(val){
      if (!missing(val)){
        private$.restrictions <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.restrictions
      }
    },

    validDates = function(val){
      if (!missing(val)){
        private$.validDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.validDates
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .purpose = NULL,
    .confidentialityStatement = NULL,
    .accessPermission = NULL,
    .restrictions = NULL,
    .citationRequirement = NULL,
    .depositRequirement = NULL,
    .accessConditions = NULL,
    .disclaimer = NULL,
    .contactAgent = NULL,
    .validDates = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        accessConditions = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        accessPermission = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Form", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        citationRequirement = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        confidentialityStatement = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contactAgent = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AgentAssociation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        depositRequirement = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        disclaimer = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        restrictions = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        validDates = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Access


#' @title DDI4_AccessLocation  A set of access information for a Machine including URI, mime type, and physical location
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/AccessLocation}{AccessLocation} 

#'
#'   Definition
#'
#'   ============
#'
#'   A set of access information for a Machine including URI, mime type, and physical location
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_AccessLocation$DdiUrn} returns the DDI URN of myDDI4_AccessLocation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{uri}}{ A read/write active binding.
#'        A URI for access, normally expressed as a URL
#'
#'                              \code{myDDI4_AccessLocation$uri <- mycharacter} sets the uri property of 'myDDI4_AccessLocation' to the vector of datatype:character
#'
#'                              \code{myDDI4_AccessLocation$uri} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{mimeType}}{ A read/write active binding.
#'        The mime type. Supports the use of an controlled vocabulary.
#'
#'                              \code{myDDI4_AccessLocation$mimeType <- list(myExternalControlledVocabularyEntry)} sets the 'mimeType' property of 'myDDI4_AccessLocation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_AccessLocation$mimeType} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{physicalLocation}}{ A read/write active binding.
#'        The physical location of the machine
#'
#'                              \code{myDDI4_AccessLocation$physicalLocation <- list(myInternationalString)} sets the 'physicalLocation' property of 'myDDI4_AccessLocation' to a list of R6 InternationalString class instances
#'
#'                              \code{myDDI4_AccessLocation$physicalLocation} returns a list of of R6 : InternationalString class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_AccessLocation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, uri=NA, mimeType=NA, physicalLocation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_AccessLocation <- R6Class(classname="DDI4_AccessLocation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, uri=NA, mimeType=NA, physicalLocation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_AccessLocation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(uri)){
        private$.uri <- returnVectorIfValid(validVectorClass="character", proposedVector=uri , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(mimeType)){
        private$.mimeType <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=mimeType , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(physicalLocation)){
        private$.physicalLocation <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=physicalLocation , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    mimeType = function(val){
      if (!missing(val)){
        private$.mimeType <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.mimeType
      }
    },

    physicalLocation = function(val){
      if (!missing(val)){
        private$.physicalLocation <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.physicalLocation
      }
    },

    uri = function(val){
      if (!missing(val)){
        private$.uri <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.uri
      }
    }


  ),

  private = list(
    .uri = NULL,
    .mimeType = NULL,
    .physicalLocation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        mimeType = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        physicalLocation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        uri = c(
          min="0", 
          max="Inf", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_AccessLocation


#' @title DDI4_Act  An Act is an indivisible, atomic step, i
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Act}{Act} 

#'
#'   Definition
#'
#'   ============
#'
#'   An Act is an indivisible, atomic step, i.e. not composed of other steps. An Act can also be viewed as a terminal node in a hierarchy of Workflow Steps. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   QuestionConstructType from DDI 3.x is an example of an Act.
#'
#'   GenerationInstructionType from DDI 3..x is an example of an Act extended to include a CommandCode used to create a derivation or recode for an InstanceVariable or other similar use 
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Act is named after the act in the HL7 RIM. This act can take many forms including an observation, a procedure, a referral, a prescription, a consent and so forth.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_WorkflowStep}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Act$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Act <- R6Class(classname="DDI4_Act",
  inherit = DDI4_WorkflowStep,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Act may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

    }
  ),

  active = list(
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInformationFlow = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPerformedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowService", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_Act


#' @title DDI4_Address  Location address identifying each part of the address as separate elements, identifying the type of address, the level of privacy associated with the release of the address, and a flag to identify the preferred address for contact
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Address}{Address} 

#'
#'   Definition
#'
#'   ============
#'
#'   Location address identifying each part of the address as separate elements, identifying the type of address, the level of privacy associated with the release of the address, and a flag to identify the preferred address for contact.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   For example:
#'
#'   1.  OFFICE, ABS HOUSE, 45 Benjamin Way, Belconnen, Canberra, ACT, 2617, AU
#'
#'   2.  OFFICE, Institute of Education, 20 Bedford Way, London, WC1H 0AL, UK
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   a:AddressType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Address$DdiUrn} returns the DDI URN of myDDI4_Address.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{typeOfAddress}}{ A read/write active binding.
#'        Indicates address type (i.e. home, office, mailing, etc.)
#'
#'                              \code{myDDI4_Address$typeOfAddress <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfAddress' property of 'myDDI4_Address' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_Address$typeOfAddress} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{line}}{ A read/write active binding.
#'        Number and street including office or suite number. May use multiple lines.
#'
#'                              \code{myDDI4_Address$line <- mycharacter} sets the line property of 'myDDI4_Address' to the vector of datatype:character
#'
#'                              \code{myDDI4_Address$line} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{cityPlaceLocal}}{ A read/write active binding.
#'        City, place, or local area used as part of an address.
#'
#'                              \code{myDDI4_Address$cityPlaceLocal <- mycharacter} sets the cityPlaceLocal property of 'myDDI4_Address' to the vector of datatype:character
#'
#'                              \code{myDDI4_Address$cityPlaceLocal} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{stateProvince}}{ A read/write active binding.
#'        A major subnational division such as a state or province used to identify a major region within an address.
#'
#'                              \code{myDDI4_Address$stateProvince <- mycharacter} sets the stateProvince property of 'myDDI4_Address' to the vector of datatype:character
#'
#'                              \code{myDDI4_Address$stateProvince} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{postalCode}}{ A read/write active binding.
#'        Postal or ZIP Code
#'
#'                              \code{myDDI4_Address$postalCode <- mycharacter} sets the postalCode property of 'myDDI4_Address' to the vector of datatype:character
#'
#'                              \code{myDDI4_Address$postalCode} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{countryCode}}{ A read/write active binding.
#'        Country of the location
#'
#'                              \code{myDDI4_Address$countryCode <- list(myExternalControlledVocabularyEntry)} sets the 'countryCode' property of 'myDDI4_Address' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_Address$countryCode} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{timeZone}}{ A read/write active binding.
#'        Time zone of the location expressed as code.
#'
#'                              \code{myDDI4_Address$timeZone <- list(myExternalControlledVocabularyEntry)} sets the 'timeZone' property of 'myDDI4_Address' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_Address$timeZone} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{effectiveDates}}{ A read/write active binding.
#'        Clarifies when the identification information is accurate.
#'
#'                              \code{myDDI4_Address$effectiveDates <- list(myDateRange)} sets the 'effectiveDates' property of 'myDDI4_Address' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_Address$effectiveDates} returns a list of of R6 : DateRange class instances}
#'   }
#'   \describe{
#'     \item{\code{privacy}}{ A read/write active binding.
#'        Specify the level privacy for the address as public, restricted, or private. Supports the use of an external controlled vocabulary
#'
#'                              \code{myDDI4_Address$privacy <- list(myExternalControlledVocabularyEntry)} sets the 'privacy' property of 'myDDI4_Address' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_Address$privacy} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{isPreferred}}{ A read/write active binding.
#'        Set to "true" if this is the preferred location for contacting the organization or individual.
#'
#'                              \code{myDDI4_Address$isPreferred <- mylogical} sets the isPreferred property of 'myDDI4_Address' to the vector of datatype:logical
#'
#'                              \code{myDDI4_Address$isPreferred} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{geographicPoint}}{ A read/write active binding.
#'        Geographic coordinates corresponding to the address.
#'
#'                              \code{myDDI4_Address$geographicPoint <- list(mySpatialPoint)} sets the 'geographicPoint' property of 'myDDI4_Address' to a list of R6 SpatialPoint class instances
#'
#'                              \code{myDDI4_Address$geographicPoint} returns a list of of R6 : SpatialPoint class instances}
#'   }
#'   \describe{
#'     \item{\code{regionalCoverage}}{ A read/write active binding.
#'        The region covered by the agent at this address
#'
#'                              \code{myDDI4_Address$regionalCoverage <- list(myExternalControlledVocabularyEntry)} sets the 'regionalCoverage' property of 'myDDI4_Address' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_Address$regionalCoverage} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{typeOfLocation}}{ A read/write active binding.
#'        The type or purpose of the location (i.e. regional office, distribution center, home)
#'
#'                              \code{myDDI4_Address$typeOfLocation <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfLocation' property of 'myDDI4_Address' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_Address$typeOfLocation} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{locationName}}{ A read/write active binding.
#'        Name of the location if applicable.
#'
#'                              \code{myDDI4_Address$locationName <- list(myObjectName)} sets the 'locationName' property of 'myDDI4_Address' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_Address$locationName} returns a list of of R6 : ObjectName class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Address$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, typeOfAddress=NA, line=NA, cityPlaceLocal=NA, stateProvince=NA, postalCode=NA, countryCode=NA, timeZone=NA, effectiveDates=NA, privacy=NA, isPreferred=NA, geographicPoint=NA, regionalCoverage=NA, typeOfLocation=NA, locationName=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Address <- R6Class(classname="DDI4_Address",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, typeOfAddress=NA, line=NA, cityPlaceLocal=NA, stateProvince=NA, postalCode=NA, countryCode=NA, timeZone=NA, effectiveDates=NA, privacy=NA, isPreferred=NA, geographicPoint=NA, regionalCoverage=NA, typeOfLocation=NA, locationName=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Address may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(typeOfAddress)){
        private$.typeOfAddress <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfAddress , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(line)){
        private$.line <- returnVectorIfValid(validVectorClass="character", proposedVector=line , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(cityPlaceLocal)){
        private$.cityPlaceLocal <- returnVectorIfValid(validVectorClass="character", proposedVector=cityPlaceLocal , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(stateProvince)){
        private$.stateProvince <- returnVectorIfValid(validVectorClass="character", proposedVector=stateProvince , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(postalCode)){
        private$.postalCode <- returnVectorIfValid(validVectorClass="character", proposedVector=postalCode , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(countryCode)){
        private$.countryCode <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=countryCode , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(timeZone)){
        private$.timeZone <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=timeZone , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(effectiveDates)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=effectiveDates , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(privacy)){
        private$.privacy <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=privacy , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isPreferred)){
        private$.isPreferred <- returnVectorIfValid(validVectorClass="logical", proposedVector=isPreferred , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(geographicPoint)){
        private$.geographicPoint <- returnR6ObjectIfValid(validClass="DDI4_SpatialPoint", proposedObjectList=geographicPoint , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(regionalCoverage)){
        private$.regionalCoverage <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=regionalCoverage , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(typeOfLocation)){
        private$.typeOfLocation <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfLocation , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(locationName)){
        private$.locationName <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=locationName , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 14 properties: 
    cityPlaceLocal = function(val){
      if (!missing(val)){
        private$.cityPlaceLocal <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.cityPlaceLocal
      }
    },

    countryCode = function(val){
      if (!missing(val)){
        private$.countryCode <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.countryCode
      }
    },

    effectiveDates = function(val){
      if (!missing(val)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.effectiveDates
      }
    },

    geographicPoint = function(val){
      if (!missing(val)){
        private$.geographicPoint <- returnR6ObjectIfValid(validClass="DDI4_SpatialPoint", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.geographicPoint
      }
    },

    isPreferred = function(val){
      if (!missing(val)){
        private$.isPreferred <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isPreferred
      }
    },

    line = function(val){
      if (!missing(val)){
        private$.line <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.line
      }
    },

    locationName = function(val){
      if (!missing(val)){
        private$.locationName <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.locationName
      }
    },

    postalCode = function(val){
      if (!missing(val)){
        private$.postalCode <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.postalCode
      }
    },

    privacy = function(val){
      if (!missing(val)){
        private$.privacy <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.privacy
      }
    },

    regionalCoverage = function(val){
      if (!missing(val)){
        private$.regionalCoverage <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.regionalCoverage
      }
    },

    stateProvince = function(val){
      if (!missing(val)){
        private$.stateProvince <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.stateProvince
      }
    },

    timeZone = function(val){
      if (!missing(val)){
        private$.timeZone <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.timeZone
      }
    },

    typeOfAddress = function(val){
      if (!missing(val)){
        private$.typeOfAddress <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.typeOfAddress
      }
    },

    typeOfLocation = function(val){
      if (!missing(val)){
        private$.typeOfLocation <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.typeOfLocation
      }
    }


  ),

  private = list(
    .typeOfAddress = NULL,
    .line = NULL,
    .cityPlaceLocal = NULL,
    .stateProvince = NULL,
    .postalCode = NULL,
    .countryCode = NULL,
    .timeZone = NULL,
    .effectiveDates = NULL,
    .privacy = NULL,
    .isPreferred = NULL,
    .geographicPoint = NULL,
    .regionalCoverage = NULL,
    .typeOfLocation = NULL,
    .locationName = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        cityPlaceLocal = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        countryCode = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        effectiveDates = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        geographicPoint = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SpatialPoint", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        isPreferred = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        line = c(
          min="0", 
          max="Inf", 
          datatype = "character", 
          structure = "vector"
        ),
        locationName = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        postalCode = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        privacy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        regionalCoverage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        stateProvince = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        timeZone = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        typeOfAddress = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        typeOfLocation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Address


#' @title DDI4_Agent  An actor that performs a role in relation to a process or product
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Agent}{Agent} 

#'
#'   Definition
#'
#'   ============
#'
#'   An actor that performs a role in relation to a process or product.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Analyst performing edits on data, interviewer conducting an interview, a relational database management system managing data, organization publishing data on a regular basis, creator or contributor of a publication.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   foaf:Agent is: An agent (eg. person, group, software or physical artifact)
#'
#'   prov:Agent is An agent is something that bears some form of responsibility for an activity taking place, for the existence of an entity, or for another agent's activity.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   Agent
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Agent$DdiUrn} returns the DDI URN of myDDI4_Agent.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasAgentId}}{ A read/write active binding.
#'        An identifier within a specified system for specifying an agent
#'
#'                              \code{myDDI4_Agent$hasAgentId <- list(myAgentId)} sets the 'hasAgentId' property of 'myDDI4_Agent' to a list of R6 AgentId class instances
#'
#'                              \code{myDDI4_Agent$hasAgentId} returns a list of of R6 : AgentId class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_Agent$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_Agent' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Agent$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{image}}{ A read/write active binding.
#'        References an image using the standard Image description. In addition to the standard attributes provides an effective date (period), the type of image, and a privacy ranking.
#'
#'                              \code{myDDI4_Agent$image <- list(myPrivateImage)} sets the 'image' property of 'myDDI4_Agent' to a list of R6 PrivateImage class instances
#'
#'                              \code{myDDI4_Agent$image} returns a list of of R6 : PrivateImage class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Agent$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasAgentId=NA, purpose=NA, image=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Agent <- R6Class(classname="DDI4_Agent",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasAgentId=NA, purpose=NA, image=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Agent may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(hasAgentId)){
        private$.hasAgentId <- returnR6ObjectIfValid(validClass="DDI4_AgentId", proposedObjectList=hasAgentId , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(image)){
        private$.image <- returnR6ObjectIfValid(validClass="DDI4_PrivateImage", proposedObjectList=image , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    hasAgentId = function(val){
      if (!missing(val)){
        private$.hasAgentId <- returnR6ObjectIfValid(validClass="DDI4_AgentId", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasAgentId
      }
    },

    image = function(val){
      if (!missing(val)){
        private$.image <- returnR6ObjectIfValid(validClass="DDI4_PrivateImage", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.image
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasAgentId = NULL,
    .purpose = NULL,
    .image = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAgentId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AgentId", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        image = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_PrivateImage", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_Agent


#' @title DDI4_AgentAssociation  A basic structure for declaring the name of an Agent inline, reference to an Agent, and role specification
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/AgentAssociation}{AgentAssociation} 

#'
#'   Definition
#'
#'   ============
#'
#'   A basic structure for declaring the name of an Agent inline, reference to an Agent, and role specification. This object is used primarily within Annotation.
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:ContributorType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_AgentAssociation$DdiUrn} returns the DDI URN of myDDI4_AgentAssociation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{agentName}}{ A read/write active binding.
#'        Full name of the contributor. Language equivalents should be expressed within the International String structure.
#'
#'                              \code{myDDI4_AgentAssociation$agentName <- list(myBibliographicName)} sets the 'agentName' property of 'myDDI4_AgentAssociation' to a list of R6 BibliographicName class instances
#'
#'                              \code{myDDI4_AgentAssociation$agentName} returns a list of of R6 : BibliographicName class instances}
#'   }
#'   \describe{
#'     \item{\code{role}}{ A read/write active binding.
#'        The role of the of the Agent within the context of the parent property name with information on the extent to which the role applies. Allows for use of external controlled vocabularies. Reference should be made to the vocabulary within the structure of the role. Recommended role for contributors can be found in the CASRAI Contributor Roles Vocabulary (CRediT) http://dictionary.casrai.org/Contributor_Roles, or the OPENRIF contribution ontology https://github.com/openrif/contribution-ontology
#'
#'                              \code{myDDI4_AgentAssociation$role <- list(myPairedExternalControlledVocabularyEntry)} sets the 'role' property of 'myDDI4_AgentAssociation' to a list of R6 PairedExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_AgentAssociation$role} returns a list of of R6 : PairedExternalControlledVocabularyEntry class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{associatedAgent}}{ A read/write active binding.
#'          Reference to an agent as described by any object that is a member of the abstract type Agent.
#'
#'                              \code{myDDI4_AgentAssociation$associatedAgent <- c(myAgentURN)} sets the association 'associatedAgent' from 'myDDI4_AgentAssociation' to a character vector of DdiUrns for R6 Agent class instances
#'
#'                              \code{myDDI4_AgentAssociation$associatedAgent} returns a character vector of of DdiUrns for R6  Agent class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_AgentAssociation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, agentName=NA, role=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_AgentAssociation <- R6Class(classname="DDI4_AgentAssociation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, agentName=NA, role=NA,    associatedAgent=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_AgentAssociation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(agentName)){
        private$.agentName <- returnR6ObjectIfValid(validClass="DDI4_BibliographicName", proposedObjectList=agentName , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(role)){
        private$.role <- returnR6ObjectIfValid(validClass="DDI4_PairedExternalControlledVocabularyEntry", proposedObjectList=role , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(associatedAgent)){
        private$.associatedAgent <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=associatedAgent , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.associatedAgent, referencingAssociationName="associatedAgent",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 2 properties: 
    agentName = function(val){
      if (!missing(val)){
        private$.agentName <- returnR6ObjectIfValid(validClass="DDI4_BibliographicName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.agentName
      }
    },

    role = function(val){
      if (!missing(val)){
        private$.role <- returnR6ObjectIfValid(validClass="DDI4_PairedExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.role
      }
    },


# There are 1 class association(s):
    associatedAgent = function(val){
      if (!missing(val)){
        private$.associatedAgent <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.associatedAgent, referencingAssociationName="associatedAgent",  referencingUrn=thisUrn)  
       # invisible( private$.associatedAgent)
      } else {
          private$.associatedAgent
      }
    }

  ),

  private = list(
    .agentName = NULL,
    .role = NULL,
    .associatedAgent = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agentName = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BibliographicName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        associatedAgent = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Agent", 
          structure = "DdiUrn vector"
        ),
        role = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_PairedExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_AgentAssociation


#' @title DDI4_AgentId  Persistent identifier for a researcher using a system like ORCID
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/AgentId}{AgentId} 

#'
#'   Definition
#'
#'   ============
#'
#'   Persistent identifier for a researcher using a system like ORCID
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   ORCID
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_AgentId$DdiUrn} returns the DDI URN of myDDI4_AgentId.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{agentIdValue}}{ A read/write active binding.
#'        The identifier for the agent.
#'
#'                              \code{myDDI4_AgentId$agentIdValue <- mycharacter} sets the agentIdValue property of 'myDDI4_AgentId' to the vector of datatype:character
#'
#'                              \code{myDDI4_AgentId$agentIdValue} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{agentIdType}}{ A read/write active binding.
#'        The identifier system in use.
#'
#'                              \code{myDDI4_AgentId$agentIdType <- mycharacter} sets the agentIdType property of 'myDDI4_AgentId' to the vector of datatype:character
#'
#'                              \code{myDDI4_AgentId$agentIdType} returns a vector of datatype: character}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_AgentId$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, agentIdValue=NA, agentIdType=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_AgentId <- R6Class(classname="DDI4_AgentId",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, agentIdValue=NA, agentIdType=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_AgentId may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(agentIdValue)){
        private$.agentIdValue <- returnVectorIfValid(validVectorClass="character", proposedVector=agentIdValue , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(agentIdType)){
        private$.agentIdType <- returnVectorIfValid(validVectorClass="character", proposedVector=agentIdType , minimumCardinality=1, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    agentIdType = function(val){
      if (!missing(val)){
        private$.agentIdType <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.agentIdType
      }
    },

    agentIdValue = function(val){
      if (!missing(val)){
        private$.agentIdValue <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.agentIdValue
      }
    }


  ),

  private = list(
    .agentIdValue = NULL,
    .agentIdType = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agentIdType = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        agentIdValue = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_AgentId


#' @title DDI4_AgentIndicator  Member Indicator for use with member type Agent
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/AgentIndicator}{AgentIndicator} 

#'
#'   Definition
#'
#'   ============
#'
#'   Member Indicator for use with member type Agent
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_AgentIndicator$DdiUrn} returns the DDI URN of myDDI4_AgentIndicator.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Index value within an ordered array
#'
#'                              \code{myDDI4_AgentIndicator$index <- myinteger} sets the index property of 'myDDI4_AgentIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_AgentIndicator$index} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{member}}{ A read/write active binding.
#'          Restricts member target to Agent
#'
#'                              \code{myDDI4_AgentIndicator$member <- c(myAgentURN)} sets the association 'member' from 'myDDI4_AgentIndicator' to a character vector of DdiUrns for R6 Agent class instances
#'
#'                              \code{myDDI4_AgentIndicator$member} returns a character vector of of DdiUrns for R6  Agent class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_AgentIndicator$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_AgentIndicator <- R6Class(classname="DDI4_AgentIndicator",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA,    member=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_AgentIndicator may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(member)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=member , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.index
      }
    },


# There are 1 class association(s):
    member = function(val){
      if (!missing(val)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
       # invisible( private$.member)
      } else {
          private$.member
      }
    }

  ),

  private = list(
    .index = NULL,
    .member = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        index = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        member = c(
          min="1", 
          max="1", 
          datatype = "DDI4_Agent", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_AgentIndicator


#' @title DDI4_AgentListing  A listing of Agents of any type
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/AgentListing}{AgentListing} 

#'
#'   Definition
#'
#'   ============
#'
#'   A listing of Agents of any type. The AgentList may be organized to describe relationships between members using AgentRelationStructure.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Organizations contributing to a project. Individuals within an agency. All organizations, indivduals, and machines identified within the collections of an archive.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Relationships between agents are fluid and reflect effective dates of the relationship. An agent may have multiple relationships which may be sequencial or concurrent. Relationships may or may not be hierarchical in nature. All Agents are serialized individually and brought into relationships as appropriate.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_AgentListing$DdiUrn} returns the DDI URN of myDDI4_AgentListing.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{type}}{ A read/write active binding.
#'        Whether the collection is a bag or a set: a bag is a collection with duplicates allowed, a set is a collection without duplicates.
#'
#'                              \code{myDDI4_AgentListing$type <- list(myCollectionType)} sets the 'type' property of 'myDDI4_AgentListing' to a list of R6 CollectionType class instances
#'
#'                              \code{myDDI4_AgentListing$type} returns a list of of R6 : CollectionType class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage.
#'
#'                              \code{myDDI4_AgentListing$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_AgentListing' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_AgentListing$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of the Agent Listing. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_AgentListing$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_AgentListing' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_AgentListing$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        Allows for the identification of the member and optionally provides an index for the member within an ordered array
#'
#'                              \code{myDDI4_AgentListing$contains <- list(myAgentIndicator)} sets the 'contains' property of 'myDDI4_AgentListing' to a list of R6 AgentIndicator class instances
#'
#'                              \code{myDDI4_AgentListing$contains} returns a list of of R6 : AgentIndicator class instances}
#'   }
#'   \describe{
#'     \item{\code{isOrdered}}{ A read/write active binding.
#'        If members are ordered set to true, if unordered set to false.
#'
#'                              \code{myDDI4_AgentListing$isOrdered <- mylogical} sets the isOrdered property of 'myDDI4_AgentListing' to the vector of datatype:logical
#'
#'                              \code{myDDI4_AgentListing$isOrdered} returns a vector of datatype: logical}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isStructuredBy}}{ A read/write active binding.
#'          Description of a complex structure for the Collection. A collection may be structured in more than one way for different uses.
#'
#'                              \code{myDDI4_AgentListing$isStructuredBy <- c(myAgentRelationStructureURN)} sets the association 'isStructuredBy' from 'myDDI4_AgentListing' to a character vector of DdiUrns for R6 AgentRelationStructure class instances
#'
#'                              \code{myDDI4_AgentListing$isStructuredBy} returns a character vector of of DdiUrns for R6  AgentRelationStructure class instances}
#'   }
#'   \describe{
#'     \item{\code{maintenanceUnit}}{ A read/write active binding.
#'          Agent that maintains this Agency list
#'
#'                              \code{myDDI4_AgentListing$maintenanceUnit <- c(myAgentURN)} sets the association 'maintenanceUnit' from 'myDDI4_AgentListing' to a character vector of DdiUrns for R6 Agent class instances
#'
#'                              \code{myDDI4_AgentListing$maintenanceUnit} returns a character vector of of DdiUrns for R6  Agent class instances}
#'   }
#'   \describe{
#'     \item{\code{definingConcept}}{ A read/write active binding.
#'          The conceptual basis for the collection of members.
#'
#'                              \code{myDDI4_AgentListing$definingConcept <- c(myConceptURN)} sets the association 'definingConcept' from 'myDDI4_AgentListing' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_AgentListing$definingConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_AgentListing$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_AgentListing <- R6Class(classname="DDI4_AgentListing",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA,    isStructuredBy=NA,maintenanceUnit=NA,definingConcept=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_AgentListing may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(type)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=type , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_AgentIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(isOrdered)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=isOrdered , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(isStructuredBy)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_AgentRelationStructure", proposedUrnVector=isStructuredBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
      }
      if(!missing(maintenanceUnit)){
        private$.maintenanceUnit <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=maintenanceUnit , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.maintenanceUnit, referencingAssociationName="maintenanceUnit",  referencingUrn=thisUrn)  
      }
      if(!missing(definingConcept)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=definingConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 5 properties: 
    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_AgentIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },

    isOrdered = function(val){
      if (!missing(val)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isOrdered
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    type = function(val){
      if (!missing(val)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.type
      }
    },


# There are 3 class association(s):
    definingConcept = function(val){
      if (!missing(val)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
       # invisible( private$.definingConcept)
      } else {
          private$.definingConcept
      }
    },

    isStructuredBy = function(val){
      if (!missing(val)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_AgentRelationStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
       # invisible( private$.isStructuredBy)
      } else {
          private$.isStructuredBy
      }
    },

    maintenanceUnit = function(val){
      if (!missing(val)){
        private$.maintenanceUnit <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.maintenanceUnit, referencingAssociationName="maintenanceUnit",  referencingUrn=thisUrn)  
       # invisible( private$.maintenanceUnit)
      } else {
          private$.maintenanceUnit
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .type = NULL,
    .name = NULL,
    .purpose = NULL,
    .contains = NULL,
    .isOrdered = NULL,
    .isStructuredBy = NULL,
    .maintenanceUnit = NULL,
    .definingConcept = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AgentIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isStructuredBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AgentRelationStructure", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        maintenanceUnit = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Agent", 
          structure = "DdiUrn vector"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_AgentListing


#' @title DDI4_AgentRelation  Used to define the relation agents in a hierarchical structure
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/AgentRelation}{AgentRelation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Used to define the relation agents in a hierarchical structure
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   An Organization (source/parent) employing and Individual (target/child); An Individual (source/parent) supervisory to an Individual (target/child); An Organization (source/parent) overseeing a project (Organization) (target/child). Select appropriate relationship using the controlled vocabulary available through hasRelationshipSpecification. 
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_AgentRelation$DdiUrn} returns the DDI URN of myDDI4_AgentRelation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{effectiveDates}}{ A read/write active binding.
#'        The effective dates of the relation. A structured DateRange with start and end Date (both with the structure of Date and supporting the use of ISO and non-ISO date structures); Use to relate a period with a start and end date.
#'
#'                              \code{myDDI4_AgentRelation$effectiveDates <- list(myDateRange)} sets the 'effectiveDates' property of 'myDDI4_AgentRelation' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_AgentRelation$effectiveDates} returns a list of of R6 : DateRange class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_AgentRelation$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_AgentRelation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_AgentRelation$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an ennumeration list
#'
#'                              \code{myDDI4_AgentRelation$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_AgentRelation' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_AgentRelation$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_AgentRelation$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_AgentRelation' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_AgentRelation$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          The Parent agent in the hierarchical relationship
#'
#'                              \code{myDDI4_AgentRelation$source <- c(myAgentURN)} sets the association 'source' from 'myDDI4_AgentRelation' to a character vector of DdiUrns for R6 Agent class instances
#'
#'                              \code{myDDI4_AgentRelation$source} returns a character vector of of DdiUrns for R6  Agent class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          The Child agent in the hierarchical relationship
#'
#'                              \code{myDDI4_AgentRelation$target <- c(myAgentURN)} sets the association 'target' from 'myDDI4_AgentRelation' to a character vector of DdiUrns for R6 Agent class instances
#'
#'                              \code{myDDI4_AgentRelation$target} returns a character vector of of DdiUrns for R6  Agent class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_AgentRelation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, effectiveDates=NA, semantic=NA, hasRelationSpecification=NA, totality=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_AgentRelation <- R6Class(classname="DDI4_AgentRelation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, effectiveDates=NA, semantic=NA, hasRelationSpecification=NA, totality=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_AgentRelation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(effectiveDates)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=effectiveDates , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=source , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=target , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 4 properties: 
    effectiveDates = function(val){
      if (!missing(val)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.effectiveDates
      }
    },

    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .effectiveDates = NULL,
    .semantic = NULL,
    .hasRelationSpecification = NULL,
    .totality = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        effectiveDates = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="1", 
          max="1", 
          datatype = "DDI4_Agent", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="1", 
          max="1", 
          datatype = "DDI4_Agent", 
          structure = "DdiUrn vector"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_AgentRelation


#' @title DDI4_AgentRelationStructure  Defines the relationships between Agents in a collection
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/AgentRelationStructure}{AgentRelationStructure} 

#'
#'   Definition
#'
#'   ============
#'
#'   Defines the relationships between Agents in a collection. Clarifies valid period and the purpose the relationship serve
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   An individual employed by an Organization. A unit or project (organization) within another Organization.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Describes relations between agents not roles within a project or in relationship to a product. Roles are defined by the parent class and relationship name that uses an Agent as a target.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_AgentRelationStructure$DdiUrn} returns the DDI URN of myDDI4_AgentRelationStructure.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{effectiveDates}}{ A read/write active binding.
#'        The effective start and end date of the relationship
#'
#'                              \code{myDDI4_AgentRelationStructure$effectiveDates <- list(myDateRange)} sets the 'effectiveDates' property of 'myDDI4_AgentRelationStructure' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_AgentRelationStructure$effectiveDates} returns a list of of R6 : DateRange class instances}
#'   }
#'   \describe{
#'     \item{\code{privacy}}{ A read/write active binding.
#'        Define the level of privacy regarding this relationship. Supports the use of a controlled vocabulary.
#'
#'                              \code{myDDI4_AgentRelationStructure$privacy <- list(myExternalControlledVocabularyEntry)} sets the 'privacy' property of 'myDDI4_AgentRelationStructure' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_AgentRelationStructure$privacy} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of creating the relation. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_AgentRelationStructure$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_AgentRelationStructure' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_AgentRelationStructure$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an ennumeration list. Use if all relations within this relation structure are of the same specification.
#'
#'                              \code{myDDI4_AgentRelationStructure$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_AgentRelationStructure' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_AgentRelationStructure$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relation structure
#'
#'                              \code{myDDI4_AgentRelationStructure$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_AgentRelationStructure' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_AgentRelationStructure$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_AgentRelationStructure$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_AgentRelationStructure' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_AgentRelationStructure$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMemberRelation}}{ A read/write active binding.
#'        Agent relations that comprise the relation structure
#'
#'                              \code{myDDI4_AgentRelationStructure$hasMemberRelation <- list(myAgentRelation)} sets the 'hasMemberRelation' property of 'myDDI4_AgentRelationStructure' to a list of R6 AgentRelation class instances
#'
#'                              \code{myDDI4_AgentRelationStructure$hasMemberRelation} returns a list of of R6 : AgentRelation class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_AgentRelationStructure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, effectiveDates=NA, privacy=NA, purpose=NA, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_AgentRelationStructure <- R6Class(classname="DDI4_AgentRelationStructure",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, effectiveDates=NA, privacy=NA, purpose=NA, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_AgentRelationStructure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(effectiveDates)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=effectiveDates , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(privacy)){
        private$.privacy <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=privacy , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasMemberRelation)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_AgentRelation", proposedObjectList=hasMemberRelation , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 7 properties: 
    effectiveDates = function(val){
      if (!missing(val)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.effectiveDates
      }
    },

    hasMemberRelation = function(val){
      if (!missing(val)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_AgentRelation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasMemberRelation
      }
    },

    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    privacy = function(val){
      if (!missing(val)){
        private$.privacy <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.privacy
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .effectiveDates = NULL,
    .privacy = NULL,
    .purpose = NULL,
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .hasMemberRelation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        effectiveDates = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasMemberRelation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AgentRelation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasRelationSpecification = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        privacy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_AgentRelationStructure


#' @title DDI4_Algorithm  An algorithm is an effective method that can be expressed within a finite amount of space and time and in a well-defined formal language for calculating a function
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Algorithm}{Algorithm} 

#'
#'   Definition
#'
#'   ============
#'
#'   An algorithm is an effective method that can be expressed within a finite amount of space and time and in a well-defined formal language for calculating a function. Starting from an initial state and initial input (perhaps empty), the instructions describe a computation that, when executed, proceeds through a finite number of well-defined successive states, eventually producing "output" and terminating at a final ending state. The transition from one state to the next is not necessarily deterministic; some algorithms, known as randomized algorithms, incorporate random input. [from www.wikipedia.org 15 April 2016]
#'
#'   
#'
#'   The underlying properties of the algorithm or method rather than the specifics of any particular implementation. In short a description of the method in its simplest and most general representation.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Algorithm$DdiUrn} returns the DDI URN of myDDI4_Algorithm.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{overview}}{ A read/write active binding.
#'        Provides a high level overview or summary of the class. Can be used to inform end-users or as part of an executive summary. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_Algorithm$overview <- list(myInternationalStructuredString)} sets the 'overview' property of 'myDDI4_Algorithm' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Algorithm$overview} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isDiscussedIn}}{ A read/write active binding.
#'          Identifies material discussing the algorithm. The material may be in DDI or other format.
#'
#'                              \code{myDDI4_Algorithm$isDiscussedIn <- c(myExternalMaterialURN)} sets the association 'isDiscussedIn' from 'myDDI4_Algorithm' to a character vector of DdiUrns for R6 ExternalMaterial class instances
#'
#'                              \code{myDDI4_Algorithm$isDiscussedIn} returns a character vector of of DdiUrns for R6  ExternalMaterial class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Algorithm$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, overview=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Algorithm <- R6Class(classname="DDI4_Algorithm",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, overview=NA,    isDiscussedIn=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Algorithm may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(overview)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=overview , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(isDiscussedIn)){
        private$.isDiscussedIn <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=isDiscussedIn , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isDiscussedIn, referencingAssociationName="isDiscussedIn",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    overview = function(val){
      if (!missing(val)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overview
      }
    },


# There are 1 class association(s):
    isDiscussedIn = function(val){
      if (!missing(val)){
        private$.isDiscussedIn <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isDiscussedIn, referencingAssociationName="isDiscussedIn",  referencingUrn=thisUrn)  
       # invisible( private$.isDiscussedIn)
      } else {
          private$.isDiscussedIn
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .overview = NULL,
    .isDiscussedIn = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isDiscussedIn = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_Algorithm


#' @title DDI4_AlgorithmOverview  High level, descriptive, human informative, algorithm statement used to describe the overall methodology
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/AlgorithmOverview}{AlgorithmOverview} 

#'
#'   Definition
#'
#'   ============
#'
#'   High level, descriptive, human informative, algorithm statement used to describe the overall methodology. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   This would most commonly be used in a Codebook along with a MethodologyOverview and a DesignOverview.The underlying properties of the algorithm or method rather than the specifics of any particular implementation. In short a description of the method in its simplest and most general representation.
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_AlgorithmOverview$DdiUrn} returns the DDI URN of myDDI4_AlgorithmOverview.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{subjectOfAlgorithm}}{ A read/write active binding.
#'        A term identifying the subject of the algorithm being described. Supports the use of a controlled vocabulary
#'
#'                              \code{myDDI4_AlgorithmOverview$subjectOfAlgorithm <- list(myExternalControlledVocabularyEntry)} sets the 'subjectOfAlgorithm' property of 'myDDI4_AlgorithmOverview' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_AlgorithmOverview$subjectOfAlgorithm} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{overview}}{ A read/write active binding.
#'        Short natural language account of the information obtained from the combination of properties and relationships associated with an object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_AlgorithmOverview$overview <- list(myInternationalStructuredString)} sets the 'overview' property of 'myDDI4_AlgorithmOverview' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_AlgorithmOverview$overview} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isDescribedIn}}{ A read/write active binding.
#'          An external document or other source that provides descriptive information on the object.
#'
#'                              \code{myDDI4_AlgorithmOverview$isDescribedIn <- c(myExternalMaterialURN)} sets the association 'isDescribedIn' from 'myDDI4_AlgorithmOverview' to a character vector of DdiUrns for R6 ExternalMaterial class instances
#'
#'                              \code{myDDI4_AlgorithmOverview$isDescribedIn} returns a character vector of of DdiUrns for R6  ExternalMaterial class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_AlgorithmOverview$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, subjectOfAlgorithm=NA, overview=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_AlgorithmOverview <- R6Class(classname="DDI4_AlgorithmOverview",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, subjectOfAlgorithm=NA, overview=NA,    isDescribedIn=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_AlgorithmOverview may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(subjectOfAlgorithm)){
        private$.subjectOfAlgorithm <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=subjectOfAlgorithm , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(overview)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=overview , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(isDescribedIn)){
        private$.isDescribedIn <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=isDescribedIn , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isDescribedIn, referencingAssociationName="isDescribedIn",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 2 properties: 
    overview = function(val){
      if (!missing(val)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overview
      }
    },

    subjectOfAlgorithm = function(val){
      if (!missing(val)){
        private$.subjectOfAlgorithm <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.subjectOfAlgorithm
      }
    },


# There are 1 class association(s):
    isDescribedIn = function(val){
      if (!missing(val)){
        private$.isDescribedIn <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isDescribedIn, referencingAssociationName="isDescribedIn",  referencingUrn=thisUrn)  
       # invisible( private$.isDescribedIn)
      } else {
          private$.isDescribedIn
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .subjectOfAlgorithm = NULL,
    .overview = NULL,
    .isDescribedIn = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isDescribedIn = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        subjectOfAlgorithm = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_AlgorithmOverview


#' @title DDI4_AnnotatedIdentifiable  Used to identify objects for purposes of internal and/or external referencing
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/AnnotatedIdentifiable}{AnnotatedIdentifiable} 

#'
#'   Definition
#'
#'   ============
#'
#'   Used to identify objects for purposes of internal and/or external referencing. Elements of this type are versioned. Provides identification and administrative metadata about the object. Adds optional annotation. Use this as the extension base for First Order Classes that contain intellectual content that needs to be discoverable in its own right.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_AnnotatedIdentifiable$DdiUrn} returns the DDI URN of myDDI4_AnnotatedIdentifiable.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasAnnotation}}{ A read/write active binding.
#'        Provides annotation information on the object to support citation and crediting of the creator(s) of the object.
#'
#'                              \code{myDDI4_AnnotatedIdentifiable$hasAnnotation <- list(myAnnotation)} sets the 'hasAnnotation' property of 'myDDI4_AnnotatedIdentifiable' to a list of R6 Annotation class instances
#'
#'                              \code{myDDI4_AnnotatedIdentifiable$hasAnnotation} returns a list of of R6 : Annotation class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{relatedMaterial}}{ A read/write active binding.
#'          Used to reference external resources. Should include the citation to such material, an external reference to a URI, and a statement of the relationship of the material to the object it has been associated with.
#'
#'                              \code{myDDI4_AnnotatedIdentifiable$relatedMaterial <- c(myExternalMaterialURN)} sets the association 'relatedMaterial' from 'myDDI4_AnnotatedIdentifiable' to a character vector of DdiUrns for R6 ExternalMaterial class instances
#'
#'                              \code{myDDI4_AnnotatedIdentifiable$relatedMaterial} returns a character vector of of DdiUrns for R6  ExternalMaterial class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_AnnotatedIdentifiable$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasAnnotation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_AnnotatedIdentifiable <- R6Class(classname="DDI4_AnnotatedIdentifiable",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasAnnotation=NA,    relatedMaterial=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_AnnotatedIdentifiable may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(hasAnnotation)){
        private$.hasAnnotation <- returnR6ObjectIfValid(validClass="DDI4_Annotation", proposedObjectList=hasAnnotation , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(relatedMaterial)){
        private$.relatedMaterial <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=relatedMaterial , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.relatedMaterial, referencingAssociationName="relatedMaterial",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    hasAnnotation = function(val){
      if (!missing(val)){
        private$.hasAnnotation <- returnR6ObjectIfValid(validClass="DDI4_Annotation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasAnnotation
      }
    },


# There are 1 class association(s):
    relatedMaterial = function(val){
      if (!missing(val)){
        private$.relatedMaterial <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.relatedMaterial, referencingAssociationName="relatedMaterial",  referencingUrn=thisUrn)  
       # invisible( private$.relatedMaterial)
      } else {
          private$.relatedMaterial
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasAnnotation = NULL,
    .relatedMaterial = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_AnnotatedIdentifiable


#' @title DDI4_Annotation  Provides annotation information on the object to support citation and crediting of the creator(s) of the object
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Annotation}{Annotation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Provides annotation information on the object to support citation and crediting of the creator(s) of the object.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:CitationType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Annotation$DdiUrn} returns the DDI URN of myDDI4_Annotation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{title}}{ A read/write active binding.
#'        Full authoritative title. List any additional titles for this item as AlternativeTitle.
#'
#'                              \code{myDDI4_Annotation$title <- list(myInternationalString)} sets the 'title' property of 'myDDI4_Annotation' to a list of R6 InternationalString class instances
#'
#'                              \code{myDDI4_Annotation$title} returns a list of of R6 : InternationalString class instances}
#'   }
#'   \describe{
#'     \item{\code{subTitle}}{ A read/write active binding.
#'        Secondary or explanatory title.
#'
#'                              \code{myDDI4_Annotation$subTitle <- list(myInternationalString)} sets the 'subTitle' property of 'myDDI4_Annotation' to a list of R6 InternationalString class instances
#'
#'                              \code{myDDI4_Annotation$subTitle} returns a list of of R6 : InternationalString class instances}
#'   }
#'   \describe{
#'     \item{\code{alternativeTitle}}{ A read/write active binding.
#'        An alternative title by which a data collection is commonly referred, or an abbreviation  for the title.
#'
#'                              \code{myDDI4_Annotation$alternativeTitle <- list(myInternationalString)} sets the 'alternativeTitle' property of 'myDDI4_Annotation' to a list of R6 InternationalString class instances
#'
#'                              \code{myDDI4_Annotation$alternativeTitle} returns a list of of R6 : InternationalString class instances}
#'   }
#'   \describe{
#'     \item{\code{creator}}{ A read/write active binding.
#'        Person, corporate body, or agency responsible for the substantive and intellectual content of the described object.
#'
#'                              \code{myDDI4_Annotation$creator <- list(myAgentAssociation)} sets the 'creator' property of 'myDDI4_Annotation' to a list of R6 AgentAssociation class instances
#'
#'                              \code{myDDI4_Annotation$creator} returns a list of of R6 : AgentAssociation class instances}
#'   }
#'   \describe{
#'     \item{\code{publisher}}{ A read/write active binding.
#'        Person or organization responsible for making the resource available in its present form.
#'
#'                              \code{myDDI4_Annotation$publisher <- list(myAgentAssociation)} sets the 'publisher' property of 'myDDI4_Annotation' to a list of R6 AgentAssociation class instances
#'
#'                              \code{myDDI4_Annotation$publisher} returns a list of of R6 : AgentAssociation class instances}
#'   }
#'   \describe{
#'     \item{\code{contributor}}{ A read/write active binding.
#'        The name of a contributing author or creator, who worked in support of the primary creator given above.
#'
#'                              \code{myDDI4_Annotation$contributor <- list(myAgentAssociation)} sets the 'contributor' property of 'myDDI4_Annotation' to a list of R6 AgentAssociation class instances
#'
#'                              \code{myDDI4_Annotation$contributor} returns a list of of R6 : AgentAssociation class instances}
#'   }
#'   \describe{
#'     \item{\code{date}}{ A read/write active binding.
#'        A date associated with the annotated object (not the coverage period). Use typeOfDate to specify the type of date such as Version, Publication, Submitted, Copyrighted, Accepted, etc.
#'
#'                              \code{myDDI4_Annotation$date <- list(myAnnotationDate)} sets the 'date' property of 'myDDI4_Annotation' to a list of R6 AnnotationDate class instances
#'
#'                              \code{myDDI4_Annotation$date} returns a list of of R6 : AnnotationDate class instances}
#'   }
#'   \describe{
#'     \item{\code{languageOfObject}}{ A read/write active binding.
#'        Language of the intellectual content of the described object. Multiple languages are supported by the structure itself as defined in the transformation to specific bindings. Use language codes supported by xs:language which include the 2 and 3 character and extended structures defined by RFC4646 or its successors. Supports multiple language codes.
#'
#'                              \code{myDDI4_Annotation$languageOfObject <- mycharacter} sets the languageOfObject property of 'myDDI4_Annotation' to the vector of datatype:character
#'
#'                              \code{myDDI4_Annotation$languageOfObject} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{identifier}}{ A read/write active binding.
#'        An identifier or locator. Contains identifier and Managing agency (ISBN, ISSN, DOI, local archive). Indicates if it is a URI.
#'
#'                              \code{myDDI4_Annotation$identifier <- list(myInternationalIdentifier)} sets the 'identifier' property of 'myDDI4_Annotation' to a list of R6 InternationalIdentifier class instances
#'
#'                              \code{myDDI4_Annotation$identifier} returns a list of of R6 : InternationalIdentifier class instances}
#'   }
#'   \describe{
#'     \item{\code{copyright}}{ A read/write active binding.
#'        The copyright statement.
#'
#'                              \code{myDDI4_Annotation$copyright <- list(myInternationalString)} sets the 'copyright' property of 'myDDI4_Annotation' to a list of R6 InternationalString class instances
#'
#'                              \code{myDDI4_Annotation$copyright} returns a list of of R6 : InternationalString class instances}
#'   }
#'   \describe{
#'     \item{\code{typeOfResource}}{ A read/write active binding.
#'        Provide the type of the resource. This supports the use of a controlled vocabulary. It should be appropriate to the level of the annotation.
#'
#'                              \code{myDDI4_Annotation$typeOfResource <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfResource' property of 'myDDI4_Annotation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_Annotation$typeOfResource} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{informationSource}}{ A read/write active binding.
#'        The name or identifier of source information for the annotated object.
#'
#'                              \code{myDDI4_Annotation$informationSource <- list(myInternationalString)} sets the 'informationSource' property of 'myDDI4_Annotation' to a list of R6 InternationalString class instances
#'
#'                              \code{myDDI4_Annotation$informationSource} returns a list of of R6 : InternationalString class instances}
#'   }
#'   \describe{
#'     \item{\code{versionIdentification}}{ A read/write active binding.
#'        Means of identifying the current version of the annotated object. 
#'
#'                              \code{myDDI4_Annotation$versionIdentification <- mycharacter} sets the versionIdentification property of 'myDDI4_Annotation' to the vector of datatype:character
#'
#'                              \code{myDDI4_Annotation$versionIdentification} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{versioningAgent}}{ A read/write active binding.
#'        The agent responsible for the version. May have an associated role.
#'
#'                              \code{myDDI4_Annotation$versioningAgent <- list(myAgentAssociation)} sets the 'versioningAgent' property of 'myDDI4_Annotation' to a list of R6 AgentAssociation class instances
#'
#'                              \code{myDDI4_Annotation$versioningAgent} returns a list of of R6 : AgentAssociation class instances}
#'   }
#'   \describe{
#'     \item{\code{summary}}{ A read/write active binding.
#'        A summary description (abstract) of the annotated object.
#'
#'                              \code{myDDI4_Annotation$summary <- list(myInternationalString)} sets the 'summary' property of 'myDDI4_Annotation' to a list of R6 InternationalString class instances
#'
#'                              \code{myDDI4_Annotation$summary} returns a list of of R6 : InternationalString class instances}
#'   }
#'   \describe{
#'     \item{\code{relatedResource}}{ A read/write active binding.
#'        Provide the identifier, managing agency, and type of resource related to this object. Use to specify related resources similar to Dublin Core isPartOf and hasPart to indicate collection/series membership for objects where there is an identifiable record. If not an identified object use the relationship to ExternalMaterial using a type that indicates a series description. 
#'
#'                              \code{myDDI4_Annotation$relatedResource <- list(myResourceIdentifier)} sets the 'relatedResource' property of 'myDDI4_Annotation' to a list of R6 ResourceIdentifier class instances
#'
#'                              \code{myDDI4_Annotation$relatedResource} returns a list of of R6 : ResourceIdentifier class instances}
#'   }
#'   \describe{
#'     \item{\code{provenance}}{ A read/write active binding.
#'        A statement of any changes in ownership and custody of the resource since its creation that are significant for its authenticity, integrity, and interpretation.
#'
#'                              \code{myDDI4_Annotation$provenance <- list(myInternationalString)} sets the 'provenance' property of 'myDDI4_Annotation' to a list of R6 InternationalString class instances
#'
#'                              \code{myDDI4_Annotation$provenance} returns a list of of R6 : InternationalString class instances}
#'   }
#'   \describe{
#'     \item{\code{rights}}{ A read/write active binding.
#'        Information about rights held in and over the resource. Typically, rights information includes a statement about various property rights associated with the resource, including intellectual property rights.
#'
#'                              \code{myDDI4_Annotation$rights <- list(myInternationalString)} sets the 'rights' property of 'myDDI4_Annotation' to a list of R6 InternationalString class instances
#'
#'                              \code{myDDI4_Annotation$rights} returns a list of of R6 : InternationalString class instances}
#'   }
#'   \describe{
#'     \item{\code{recordCreationDate}}{ A read/write active binding.
#'        Date the record was created
#'
#'                              \code{myDDI4_Annotation$recordCreationDate <- list(myIsoDateType)} sets the 'recordCreationDate' property of 'myDDI4_Annotation' to a list of R6 IsoDateType class instances
#'
#'                              \code{myDDI4_Annotation$recordCreationDate} returns a list of of R6 : IsoDateType class instances}
#'   }
#'   \describe{
#'     \item{\code{recordLastRevisionDate}}{ A read/write active binding.
#'        Date the record was last revised
#'
#'                              \code{myDDI4_Annotation$recordLastRevisionDate <- list(myIsoDateType)} sets the 'recordLastRevisionDate' property of 'myDDI4_Annotation' to a list of R6 IsoDateType class instances
#'
#'                              \code{myDDI4_Annotation$recordLastRevisionDate} returns a list of of R6 : IsoDateType class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Annotation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, title=NA, subTitle=NA, alternativeTitle=NA, creator=NA, publisher=NA, contributor=NA, date=NA, languageOfObject=NA, identifier=NA, copyright=NA, typeOfResource=NA, informationSource=NA, versionIdentification=NA, versioningAgent=NA, summary=NA, relatedResource=NA, provenance=NA, rights=NA, recordCreationDate=NA, recordLastRevisionDate=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Annotation <- R6Class(classname="DDI4_Annotation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, title=NA, subTitle=NA, alternativeTitle=NA, creator=NA, publisher=NA, contributor=NA, date=NA, languageOfObject=NA, identifier=NA, copyright=NA, typeOfResource=NA, informationSource=NA, versionIdentification=NA, versioningAgent=NA, summary=NA, relatedResource=NA, provenance=NA, rights=NA, recordCreationDate=NA, recordLastRevisionDate=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Annotation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(title)){
        private$.title <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=title , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(subTitle)){
        private$.subTitle <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=subTitle , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(alternativeTitle)){
        private$.alternativeTitle <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=alternativeTitle , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(creator)){
        private$.creator <- returnR6ObjectIfValid(validClass="DDI4_AgentAssociation", proposedObjectList=creator , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(publisher)){
        private$.publisher <- returnR6ObjectIfValid(validClass="DDI4_AgentAssociation", proposedObjectList=publisher , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(contributor)){
        private$.contributor <- returnR6ObjectIfValid(validClass="DDI4_AgentAssociation", proposedObjectList=contributor , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(date)){
        private$.date <- returnR6ObjectIfValid(validClass="DDI4_AnnotationDate", proposedObjectList=date , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(languageOfObject)){
        private$.languageOfObject <- returnVectorIfValid(validVectorClass="character", proposedVector=languageOfObject , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(identifier)){
        private$.identifier <- returnR6ObjectIfValid(validClass="DDI4_InternationalIdentifier", proposedObjectList=identifier , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(copyright)){
        private$.copyright <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=copyright , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(typeOfResource)){
        private$.typeOfResource <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfResource , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(informationSource)){
        private$.informationSource <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=informationSource , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(versionIdentification)){
        private$.versionIdentification <- returnVectorIfValid(validVectorClass="character", proposedVector=versionIdentification , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(versioningAgent)){
        private$.versioningAgent <- returnR6ObjectIfValid(validClass="DDI4_AgentAssociation", proposedObjectList=versioningAgent , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(summary)){
        private$.summary <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=summary , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(relatedResource)){
        private$.relatedResource <- returnR6ObjectIfValid(validClass="DDI4_ResourceIdentifier", proposedObjectList=relatedResource , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(provenance)){
        private$.provenance <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=provenance , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(rights)){
        private$.rights <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=rights , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(recordCreationDate)){
        private$.recordCreationDate <- returnRegularExpressionValues(RegularExpressionObject=DDI4_IsoDateType$new(registerThisObject=FALSE), proposedCharacterVector=recordCreationDate , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(recordLastRevisionDate)){
        private$.recordLastRevisionDate <- returnRegularExpressionValues(RegularExpressionObject=DDI4_IsoDateType$new(registerThisObject=FALSE), proposedCharacterVector=recordLastRevisionDate , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 20 properties: 
    alternativeTitle = function(val){
      if (!missing(val)){
        private$.alternativeTitle <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.alternativeTitle
      }
    },

    contributor = function(val){
      if (!missing(val)){
        private$.contributor <- returnR6ObjectIfValid(validClass="DDI4_AgentAssociation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contributor
      }
    },

    copyright = function(val){
      if (!missing(val)){
        private$.copyright <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.copyright
      }
    },

    creator = function(val){
      if (!missing(val)){
        private$.creator <- returnR6ObjectIfValid(validClass="DDI4_AgentAssociation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.creator
      }
    },

    date = function(val){
      if (!missing(val)){
        private$.date <- returnR6ObjectIfValid(validClass="DDI4_AnnotationDate", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.date
      }
    },

    identifier = function(val){
      if (!missing(val)){
        private$.identifier <- returnR6ObjectIfValid(validClass="DDI4_InternationalIdentifier", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.identifier
      }
    },

    informationSource = function(val){
      if (!missing(val)){
        private$.informationSource <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.informationSource
      }
    },

    languageOfObject = function(val){
      if (!missing(val)){
        private$.languageOfObject <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.languageOfObject
      }
    },

    provenance = function(val){
      if (!missing(val)){
        private$.provenance <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.provenance
      }
    },

    publisher = function(val){
      if (!missing(val)){
        private$.publisher <- returnR6ObjectIfValid(validClass="DDI4_AgentAssociation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.publisher
      }
    },

    recordCreationDate = function(val){
      if (!missing(val)){
        private$.recordCreationDate <- returnRegularExpressionValues(RegularExpressionObject=DDI4_IsoDateType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.recordCreationDate
      }
    },

    recordLastRevisionDate = function(val){
      if (!missing(val)){
        private$.recordLastRevisionDate <- returnRegularExpressionValues(RegularExpressionObject=DDI4_IsoDateType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.recordLastRevisionDate
      }
    },

    relatedResource = function(val){
      if (!missing(val)){
        private$.relatedResource <- returnR6ObjectIfValid(validClass="DDI4_ResourceIdentifier", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.relatedResource
      }
    },

    rights = function(val){
      if (!missing(val)){
        private$.rights <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.rights
      }
    },

    subTitle = function(val){
      if (!missing(val)){
        private$.subTitle <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.subTitle
      }
    },

    summary = function(val){
      if (!missing(val)){
        private$.summary <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.summary
      }
    },

    title = function(val){
      if (!missing(val)){
        private$.title <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.title
      }
    },

    typeOfResource = function(val){
      if (!missing(val)){
        private$.typeOfResource <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.typeOfResource
      }
    },

    versionIdentification = function(val){
      if (!missing(val)){
        private$.versionIdentification <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.versionIdentification
      }
    },

    versioningAgent = function(val){
      if (!missing(val)){
        private$.versioningAgent <- returnR6ObjectIfValid(validClass="DDI4_AgentAssociation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.versioningAgent
      }
    }


  ),

  private = list(
    .title = NULL,
    .subTitle = NULL,
    .alternativeTitle = NULL,
    .creator = NULL,
    .publisher = NULL,
    .contributor = NULL,
    .date = NULL,
    .languageOfObject = NULL,
    .identifier = NULL,
    .copyright = NULL,
    .typeOfResource = NULL,
    .informationSource = NULL,
    .versionIdentification = NULL,
    .versioningAgent = NULL,
    .summary = NULL,
    .relatedResource = NULL,
    .provenance = NULL,
    .rights = NULL,
    .recordCreationDate = NULL,
    .recordLastRevisionDate = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        alternativeTitle = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contributor = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AgentAssociation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        copyright = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        creator = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AgentAssociation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        date = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AnnotationDate", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        identifier = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalIdentifier", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        informationSource = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        languageOfObject = c(
          min="0", 
          max="Inf", 
          datatype = "character", 
          structure = "vector"
        ),
        provenance = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        publisher = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AgentAssociation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        recordCreationDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        recordLastRevisionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        relatedResource = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ResourceIdentifier", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        rights = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        subTitle = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        summary = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        title = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        typeOfResource = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionIdentification = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versioningAgent = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AgentAssociation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Annotation


#' @title DDI4_AnnotationDate  A generic date type for use in Annotation which provides the standard date structure plus a property to define the date type (Publication date,  Accepted date, Copyrighted date, Submitted date, etc
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/AnnotationDate}{AnnotationDate} 

#'
#'   Definition
#'
#'   ============
#'
#'   A generic date type for use in Annotation which provides the standard date structure plus a property to define the date type (Publication date,  Accepted date, Copyrighted date, Submitted date, etc.). 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   Equivalent of http://purl.org/dc/elements/1.1/date where the type of date may identify the Dublin Core refinement term.
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Date}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_AnnotationDate$DdiUrn} returns the DDI URN of myDDI4_AnnotationDate.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{typeOfDate}}{ A read/write active binding.
#'        Use to specify the type of date. This may reflect the refinements of dc:date such as dateAccepted, dateCopyrighted, dateSubmitted, etc.
#'
#'                              \code{myDDI4_AnnotationDate$typeOfDate <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfDate' property of 'myDDI4_AnnotationDate' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_AnnotationDate$typeOfDate} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_AnnotationDate$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, typeOfDate=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_AnnotationDate <- R6Class(classname="DDI4_AnnotationDate",
  inherit = DDI4_Date,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, typeOfDate=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_AnnotationDate may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(typeOfDate)){
        private$.typeOfDate <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfDate , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 1 properties: 
    typeOfDate = function(val){
      if (!missing(val)){
        private$.typeOfDate <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.typeOfDate
      }
    }


  ),

  private = list(
    .typeOfDate = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        isoDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        nonIsoDate = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_NonIsoDateType", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        typeOfDate = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_AnnotationDate


#' @title DDI4_AppliedUse  Links the guidance instructions to specific unit types
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/AppliedUse}{AppliedUse} 

#'
#'   Definition
#'
#'   ============
#'
#'   Links the guidance instructions to specific unit types.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Links one or more guides for the use of a sample (result) obtained in the first stage of complex sample with the unit type. A sample of Counties from which households will be selected in the next sample stage.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_AppliedUse$DdiUrn} returns the DDI URN of myDDI4_AppliedUse.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{overview}}{ A read/write active binding.
#'        Provides a high level overview or summary of the class. Can be used to inform end-users or as part of an executive summary. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_AppliedUse$overview <- list(myInternationalStructuredString)} sets the 'overview' property of 'myDDI4_AppliedUse' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_AppliedUse$overview} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasGuide}}{ A read/write active binding.
#'          The guide describing the appropriate usage of the result in the context of specified unit types and/or objects.
#'
#'                              \code{myDDI4_AppliedUse$hasGuide <- c(myGuideURN)} sets the association 'hasGuide' from 'myDDI4_AppliedUse' to a character vector of DdiUrns for R6 Guide class instances
#'
#'                              \code{myDDI4_AppliedUse$hasGuide} returns a character vector of of DdiUrns for R6  Guide class instances}
#'   }
#'   \describe{
#'     \item{\code{appliesToUnitType}}{ A read/write active binding.
#'          The unit type or types for which the guide is appropriate
#'
#'                              \code{myDDI4_AppliedUse$appliesToUnitType <- c(myUnitTypeURN)} sets the association 'appliesToUnitType' from 'myDDI4_AppliedUse' to a character vector of DdiUrns for R6 UnitType class instances
#'
#'                              \code{myDDI4_AppliedUse$appliesToUnitType} returns a character vector of of DdiUrns for R6  UnitType class instances}
#'   }
#'   \describe{
#'     \item{\code{isUsedBy}}{ A read/write active binding.
#'          Specifies the object for which the guide is appropriate
#'
#'                              \code{myDDI4_AppliedUse$isUsedBy <- c(myAnnotatedIdentifiableURN)} sets the association 'isUsedBy' from 'myDDI4_AppliedUse' to a character vector of DdiUrns for R6 AnnotatedIdentifiable class instances
#'
#'                              \code{myDDI4_AppliedUse$isUsedBy} returns a character vector of of DdiUrns for R6  AnnotatedIdentifiable class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_AppliedUse$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, overview=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_AppliedUse <- R6Class(classname="DDI4_AppliedUse",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, overview=NA,    hasGuide=NA,appliesToUnitType=NA,isUsedBy=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_AppliedUse may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(overview)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=overview , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(hasGuide)){
        private$.hasGuide <- returnUrnVectorIfValid(validClass="DDI4_Guide", proposedUrnVector=hasGuide , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasGuide, referencingAssociationName="hasGuide",  referencingUrn=thisUrn)  
      }
      if(!missing(appliesToUnitType)){
        private$.appliesToUnitType <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=appliesToUnitType , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.appliesToUnitType, referencingAssociationName="appliesToUnitType",  referencingUrn=thisUrn)  
      }
      if(!missing(isUsedBy)){
        private$.isUsedBy <- returnUrnVectorIfValid(validClass="DDI4_AnnotatedIdentifiable", proposedUrnVector=isUsedBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isUsedBy, referencingAssociationName="isUsedBy",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    overview = function(val){
      if (!missing(val)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overview
      }
    },


# There are 3 class association(s):
    appliesToUnitType = function(val){
      if (!missing(val)){
        private$.appliesToUnitType <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.appliesToUnitType, referencingAssociationName="appliesToUnitType",  referencingUrn=thisUrn)  
       # invisible( private$.appliesToUnitType)
      } else {
          private$.appliesToUnitType
      }
    },

    hasGuide = function(val){
      if (!missing(val)){
        private$.hasGuide <- returnUrnVectorIfValid(validClass="DDI4_Guide", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasGuide, referencingAssociationName="hasGuide",  referencingUrn=thisUrn)  
       # invisible( private$.hasGuide)
      } else {
          private$.hasGuide
      }
    },

    isUsedBy = function(val){
      if (!missing(val)){
        private$.isUsedBy <- returnUrnVectorIfValid(validClass="DDI4_AnnotatedIdentifiable", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isUsedBy, referencingAssociationName="isUsedBy",  referencingUrn=thisUrn)  
       # invisible( private$.isUsedBy)
      } else {
          private$.isUsedBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .overview = NULL,
    .hasGuide = NULL,
    .appliesToUnitType = NULL,
    .isUsedBy = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        appliesToUnitType = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_UnitType", 
          structure = "DdiUrn vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasGuide = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Guide", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUsedBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AnnotatedIdentifiable", 
          structure = "DdiUrn vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_AppliedUse


#' @title DDI4_AreaCoverage  Use to specify the area of land, water, total or other area coverage in terms of square miles/kilometers or other measure as part of a Geographic Extent
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/AreaCoverage}{AreaCoverage} 

#'
#'   Definition
#'
#'   ============
#'
#'   Use to specify the area of land, water, total or other area coverage in terms of square miles/kilometers or other measure as part of a Geographic Extent.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:AreaCoverageType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_AreaCoverage$DdiUrn} returns the DDI URN of myDDI4_AreaCoverage.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{typeOfArea}}{ A read/write active binding.
#'        Specify the type of area covered i.e. Total, Land, Water, etc. Supports the use of an external controlled vocabulary.
#'
#'                              \code{myDDI4_AreaCoverage$typeOfArea <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfArea' property of 'myDDI4_AreaCoverage' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_AreaCoverage$typeOfArea} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{measurementUnit}}{ A read/write active binding.
#'        Records the measurement unit, for example, Square Kilometer, Square Mile. Supports the use of an external controlled vocabulary.
#'
#'                              \code{myDDI4_AreaCoverage$measurementUnit <- list(myExternalControlledVocabularyEntry)} sets the 'measurementUnit' property of 'myDDI4_AreaCoverage' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_AreaCoverage$measurementUnit} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{areaMeasure}}{ A read/write active binding.
#'        The area measure expressed as a decimal for the measurement unit designated.
#'
#'                              \code{myDDI4_AreaCoverage$areaMeasure <- mynumeric} sets the areaMeasure property of 'myDDI4_AreaCoverage' to the vector of datatype:numeric
#'
#'                              \code{myDDI4_AreaCoverage$areaMeasure} returns a vector of datatype: numeric}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_AreaCoverage$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, typeOfArea=NA, measurementUnit=NA, areaMeasure=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_AreaCoverage <- R6Class(classname="DDI4_AreaCoverage",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, typeOfArea=NA, measurementUnit=NA, areaMeasure=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_AreaCoverage may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(typeOfArea)){
        private$.typeOfArea <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfArea , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(measurementUnit)){
        private$.measurementUnit <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=measurementUnit , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(areaMeasure)){
        private$.areaMeasure <- returnVectorIfValid(validVectorClass="numeric", proposedVector=areaMeasure , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    areaMeasure = function(val){
      if (!missing(val)){
        private$.areaMeasure <- returnVectorIfValid(validVectorClass="numeric", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.areaMeasure
      }
    },

    measurementUnit = function(val){
      if (!missing(val)){
        private$.measurementUnit <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.measurementUnit
      }
    },

    typeOfArea = function(val){
      if (!missing(val)){
        private$.typeOfArea <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.typeOfArea
      }
    }


  ),

  private = list(
    .typeOfArea = NULL,
    .measurementUnit = NULL,
    .areaMeasure = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        areaMeasure = c(
          min="0", 
          max="1", 
          datatype = "numeric", 
          structure = "vector"
        ),
        measurementUnit = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        typeOfArea = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_AreaCoverage


#' @title DDI4_AttributeRole  An AttributeRole identifies one or more InstanceVariables as being attributes within a ViewPoint
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/AttributeRole}{AttributeRole} 

#'
#'   Definition
#'
#'   ============
#'
#'   An AttributeRole identifies one or more InstanceVariables as being attributes within a ViewPoint. An AttributeRole is a SimpleCollection of InstanceVariables acting in the AttributeRole.
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A data record with four variables: "PersonId", "Systolic", "Diastolic", "Seated" might have a Viewpoint with 
#'
#'   PersonId defined as having the IdentifierRole
#'
#'   Systolic defined as having the MeasureRole
#'
#'   Diastolic defined as having the MeasureRole
#'
#'   Seated defined as having the AttributeRole
#'
#'   
#'
#'   PersonId, Systolic, Diastolic, Seated
#'
#'   123,122,20,yes
#'
#'   145,130,90,no
#'
#'   
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   See the Viewpoint documentation for an in depth discussion of the uses of ViewpointRoles: http://lion.ddialliance.org/ddiobjects/viewpoint.
#'
#'   An InstanceVariable with an AttributeRole assigned might contain data about the conditions under which the MeausreRole InstanceVariables were collected or other paradata.
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ViewpointRole}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_AttributeRole$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_AttributeRole <- R6Class(classname="DDI4_AttributeRole",
  inherit = DDI4_ViewpointRole,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_AttributeRole may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

    }
  ),

  active = list(
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InstanceVariableIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_AttributeRole


#' @title DDI4_AudioSegment  Describes the type and length of the audio segment
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/AudioSegment}{AudioSegment} 

#'
#'   Definition
#'
#'   ============
#'
#'   Describes the type and length of the audio segment.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:AudioType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_AudioSegment$DdiUrn} returns the DDI URN of myDDI4_AudioSegment.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{typeOfAudioClip}}{ A read/write active binding.
#'        The type of audio clip provided. Supports the use of a controlled vocabulary.
#'
#'                              \code{myDDI4_AudioSegment$typeOfAudioClip <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfAudioClip' property of 'myDDI4_AudioSegment' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_AudioSegment$typeOfAudioClip} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{audioClipBegin}}{ A read/write active binding.
#'        The point to begin the audio clip. If no point is provided the assumption is that the start point is the beginning of the clip provided.
#'
#'                              \code{myDDI4_AudioSegment$audioClipBegin <- mycharacter} sets the audioClipBegin property of 'myDDI4_AudioSegment' to the vector of datatype:character
#'
#'                              \code{myDDI4_AudioSegment$audioClipBegin} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{audioClipEnd}}{ A read/write active binding.
#'        The point to end the audio clip. If no point is provided the assumption is that the end point is the end of the clip provided.
#'
#'                              \code{myDDI4_AudioSegment$audioClipEnd <- mycharacter} sets the audioClipEnd property of 'myDDI4_AudioSegment' to the vector of datatype:character
#'
#'                              \code{myDDI4_AudioSegment$audioClipEnd} returns a vector of datatype: character}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_AudioSegment$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, typeOfAudioClip=NA, audioClipBegin=NA, audioClipEnd=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_AudioSegment <- R6Class(classname="DDI4_AudioSegment",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, typeOfAudioClip=NA, audioClipBegin=NA, audioClipEnd=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_AudioSegment may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(typeOfAudioClip)){
        private$.typeOfAudioClip <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfAudioClip , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(audioClipBegin)){
        private$.audioClipBegin <- returnVectorIfValid(validVectorClass="character", proposedVector=audioClipBegin , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(audioClipEnd)){
        private$.audioClipEnd <- returnVectorIfValid(validVectorClass="character", proposedVector=audioClipEnd , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    audioClipBegin = function(val){
      if (!missing(val)){
        private$.audioClipBegin <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.audioClipBegin
      }
    },

    audioClipEnd = function(val){
      if (!missing(val)){
        private$.audioClipEnd <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.audioClipEnd
      }
    },

    typeOfAudioClip = function(val){
      if (!missing(val)){
        private$.typeOfAudioClip <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.typeOfAudioClip
      }
    }


  ),

  private = list(
    .typeOfAudioClip = NULL,
    .audioClipBegin = NULL,
    .audioClipEnd = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        audioClipBegin = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        audioClipEnd = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        typeOfAudioClip = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_AudioSegment


#' @title DDI4_AuthorizationSource  Identifies the authorizing agency and allows for the full text of the authorization (law, regulation, or other form of authorization)
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/AuthorizationSource}{AuthorizationSource} 

#'
#'   Definition
#'
#'   ============
#'
#'   Identifies the authorizing agency and allows for the full text of the authorization (law, regulation, or other form of authorization).
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'    May be used to list authorizations from oversight committees and other regulatory agencies.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Supports requirements for some statistical offices to identify the agency or law authorizing the collection or management of data or metadata.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   Use for the Case Law, Case Law Description, and Case Law Date properties in ClassificationItem
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:AuthorizationSourceType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_AuthorizationSource$DdiUrn} returns the DDI URN of myDDI4_AuthorizationSource.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{statementOfAuthorization}}{ A read/write active binding.
#'        Text of the authorization (law, mandate, approved business case).
#'
#'                              \code{myDDI4_AuthorizationSource$statementOfAuthorization <- list(myInternationalStructuredString)} sets the 'statementOfAuthorization' property of 'myDDI4_AuthorizationSource' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_AuthorizationSource$statementOfAuthorization} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{legalMandate}}{ A read/write active binding.
#'        Provide a legal citation to a law authorizing the study/data collection. For example, a legal citation for a law authorizing a country's census.
#'
#'                              \code{myDDI4_AuthorizationSource$legalMandate <- list(myInternationalString)} sets the 'legalMandate' property of 'myDDI4_AuthorizationSource' to a list of R6 InternationalString class instances
#'
#'                              \code{myDDI4_AuthorizationSource$legalMandate} returns a list of of R6 : InternationalString class instances}
#'   }
#'   \describe{
#'     \item{\code{authorizationDate}}{ A read/write active binding.
#'        Identifies the date of Authorization.
#'
#'                              \code{myDDI4_AuthorizationSource$authorizationDate <- list(myDate)} sets the 'authorizationDate' property of 'myDDI4_AuthorizationSource' to a list of R6 Date class instances
#'
#'                              \code{myDDI4_AuthorizationSource$authorizationDate} returns a list of of R6 : Date class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_AuthorizationSource$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_AuthorizationSource' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_AuthorizationSource$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{authorizingAgent}}{ A read/write active binding.
#'          References the authorizing agent generally described as an organization or individual
#'
#'                              \code{myDDI4_AuthorizationSource$authorizingAgent <- c(myAgentURN)} sets the association 'authorizingAgent' from 'myDDI4_AuthorizationSource' to a character vector of DdiUrns for R6 Agent class instances
#'
#'                              \code{myDDI4_AuthorizationSource$authorizingAgent} returns a character vector of of DdiUrns for R6  Agent class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_AuthorizationSource$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, statementOfAuthorization=NA, legalMandate=NA, authorizationDate=NA, purpose=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_AuthorizationSource <- R6Class(classname="DDI4_AuthorizationSource",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, statementOfAuthorization=NA, legalMandate=NA, authorizationDate=NA, purpose=NA,    authorizingAgent=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_AuthorizationSource may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(statementOfAuthorization)){
        private$.statementOfAuthorization <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=statementOfAuthorization , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(legalMandate)){
        private$.legalMandate <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=legalMandate , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(authorizationDate)){
        private$.authorizationDate <- returnR6ObjectIfValid(validClass="DDI4_Date", proposedObjectList=authorizationDate , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(authorizingAgent)){
        private$.authorizingAgent <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=authorizingAgent , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.authorizingAgent, referencingAssociationName="authorizingAgent",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 4 properties: 
    authorizationDate = function(val){
      if (!missing(val)){
        private$.authorizationDate <- returnR6ObjectIfValid(validClass="DDI4_Date", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.authorizationDate
      }
    },

    legalMandate = function(val){
      if (!missing(val)){
        private$.legalMandate <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.legalMandate
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    statementOfAuthorization = function(val){
      if (!missing(val)){
        private$.statementOfAuthorization <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.statementOfAuthorization
      }
    },


# There are 1 class association(s):
    authorizingAgent = function(val){
      if (!missing(val)){
        private$.authorizingAgent <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.authorizingAgent, referencingAssociationName="authorizingAgent",  referencingUrn=thisUrn)  
       # invisible( private$.authorizingAgent)
      } else {
          private$.authorizingAgent
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .statementOfAuthorization = NULL,
    .legalMandate = NULL,
    .authorizationDate = NULL,
    .purpose = NULL,
    .authorizingAgent = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        authorizationDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Date", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        authorizingAgent = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Agent", 
          structure = "DdiUrn vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        legalMandate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        statementOfAuthorization = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_AuthorizationSource


#' @title DDI4_BasedOnObjectInformation  Use when creating an object that is based on an existing object or objects that are managed by a different agency or when the new object is NOT simply a version change but you wish to maintain a reference to the object that served as a basis for the new object
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/BasedOnObjectInformation}{BasedOnObjectInformation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Use when creating an object that is based on an existing object or objects that are managed by a different agency or when the new object is NOT simply a version change but you wish to maintain a reference to the object that served as a basis for the new object. BasedOnObject may contain references to any number of objects which serve as a basis for this object, a BasedOnRationaleDescription of how the content of the referenced object was incorporated or altered, and a BasedOnRationaleCode to allow for specific typing of the BasedOnReference according to an external controlled vocabulary.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:BasedOnObjectType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_BasedOnObjectInformation$DdiUrn} returns the DDI URN of myDDI4_BasedOnObjectInformation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{basedOnRationaleDescription}}{ A read/write active binding.
#'        Textual description of the rationale/purpose for the based on reference to inform users as to the extent and implication of the version change. May be expressed in multiple languages.
#'
#'                              \code{myDDI4_BasedOnObjectInformation$basedOnRationaleDescription <- list(myInternationalString)} sets the 'basedOnRationaleDescription' property of 'myDDI4_BasedOnObjectInformation' to a list of R6 InternationalString class instances
#'
#'                              \code{myDDI4_BasedOnObjectInformation$basedOnRationaleDescription} returns a list of of R6 : InternationalString class instances}
#'   }
#'   \describe{
#'     \item{\code{basedOnRationaleCode}}{ A read/write active binding.
#'        RationaleCode is primarily for internal processing flags within an organization or system. Supports the use of an external controlled vocabulary.
#'
#'                              \code{myDDI4_BasedOnObjectInformation$basedOnRationaleCode <- list(myExternalControlledVocabularyEntry)} sets the 'basedOnRationaleCode' property of 'myDDI4_BasedOnObjectInformation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_BasedOnObjectInformation$basedOnRationaleCode} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{basedOn}}{ A read/write active binding.
#'          The identification for the object upon which the current object is based.
#'
#'                              \code{myDDI4_BasedOnObjectInformation$basedOn <- c(myIdentifiableURN)} sets the association 'basedOn' from 'myDDI4_BasedOnObjectInformation' to a character vector of DdiUrns for R6 Identifiable class instances
#'
#'                              \code{myDDI4_BasedOnObjectInformation$basedOn} returns a character vector of of DdiUrns for R6  Identifiable class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_BasedOnObjectInformation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, basedOnRationaleDescription=NA, basedOnRationaleCode=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_BasedOnObjectInformation <- R6Class(classname="DDI4_BasedOnObjectInformation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, basedOnRationaleDescription=NA, basedOnRationaleCode=NA,    basedOn=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_BasedOnObjectInformation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(basedOnRationaleDescription)){
        private$.basedOnRationaleDescription <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=basedOnRationaleDescription , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(basedOnRationaleCode)){
        private$.basedOnRationaleCode <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=basedOnRationaleCode , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(basedOn)){
        private$.basedOn <- returnUrnVectorIfValid(validClass="DDI4_Identifiable", proposedUrnVector=basedOn , minimumCardinality=1, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.basedOn, referencingAssociationName="basedOn",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 2 properties: 
    basedOnRationaleCode = function(val){
      if (!missing(val)){
        private$.basedOnRationaleCode <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.basedOnRationaleCode
      }
    },

    basedOnRationaleDescription = function(val){
      if (!missing(val)){
        private$.basedOnRationaleDescription <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.basedOnRationaleDescription
      }
    },


# There are 1 class association(s):
    basedOn = function(val){
      if (!missing(val)){
        private$.basedOn <- returnUrnVectorIfValid(validClass="DDI4_Identifiable", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.basedOn, referencingAssociationName="basedOn",  referencingUrn=thisUrn)  
       # invisible( private$.basedOn)
      } else {
          private$.basedOn
      }
    }

  ),

  private = list(
    .basedOnRationaleDescription = NULL,
    .basedOnRationaleCode = NULL,
    .basedOn = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        basedOn = c(
          min="1", 
          max="Inf", 
          datatype = "DDI4_Identifiable", 
          structure = "DdiUrn vector"
        ),
        basedOnRationaleCode = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        basedOnRationaleDescription = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_BasedOnObjectInformation


#' @title DDI4_BibliographicName  Personal names should be listed surname or family name first, followed by forename or given name
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/BibliographicName}{BibliographicName} 

#'
#'   Definition
#'
#'   ============
#'
#'   Personal names should be listed surname or family name first, followed by forename or given name. When in doubt, give the name as it appears, and do not invert. In the case of organizations where there is clearly a hierarchy present, list the parts of the hierarchy from largest to smallest, separated by full stops and a space. If it is not clear whether there is a hierarchy present, or unclear which is the larger or smaller portion of the body, give the name as it appears in the item. The name may be provided in one or more languages.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:BibliographicNameType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_InternationalString}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_BibliographicName$DdiUrn} returns the DDI URN of myDDI4_BibliographicName.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{affiliation}}{ A read/write active binding.
#'        The affiliation of this person to an organization. This is generally an organization or sub-organization name and should be related to the specific role within which the individual is being listed.
#'
#'                              \code{myDDI4_BibliographicName$affiliation <- mycharacter} sets the affiliation property of 'myDDI4_BibliographicName' to the vector of datatype:character
#'
#'                              \code{myDDI4_BibliographicName$affiliation} returns a vector of datatype: character}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_BibliographicName$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, affiliation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_BibliographicName <- R6Class(classname="DDI4_BibliographicName",
  inherit = DDI4_InternationalString,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, affiliation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_BibliographicName may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(affiliation)){
        private$.affiliation <- returnVectorIfValid(validVectorClass="character", proposedVector=affiliation , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 1 properties: 
    affiliation = function(val){
      if (!missing(val)){
        private$.affiliation <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.affiliation
      }
    }


  ),

  private = list(
    .affiliation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        affiliation = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        languageSpecificString = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LanguageSpecificStringType", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_BibliographicName


#' @title DDI4_Binding  Binds two parameters together to direct flow of data through a process
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Binding}{Binding} 

#'
#'   Definition
#'
#'   ============
#'
#'   Binds two parameters together to direct flow of data through a process
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   From the output of an InstanceQuestion to the input of a ComputationAction
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Binding is used to define the flow of data into, out of, and within a process. It is separate from the flow of a process. When used in the workflow of a data capture process most data may go from a capture to an instance variable, but when needed as a check sum in a loop, a recoding process, or conditional content for DynamicText, Binding provides the means for explicitly directing the movement of data from one point to another in the process.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{fromParameter}}{ A read/write active binding.
#'          Parameter supplying the data
#'
#'                              \code{myDDI4_Binding$fromParameter <- c(myParameterURN)} sets the association 'fromParameter' from 'myDDI4_Binding' to a character vector of DdiUrns for R6 Parameter class instances
#'
#'                              \code{myDDI4_Binding$fromParameter} returns a character vector of of DdiUrns for R6  Parameter class instances}
#'   }
#'   \describe{
#'     \item{\code{toParameter}}{ A read/write active binding.
#'          Parameter accepting the data as input to the process step
#'
#'                              \code{myDDI4_Binding$toParameter <- c(myParameterURN)} sets the association 'toParameter' from 'myDDI4_Binding' to a character vector of DdiUrns for R6 Parameter class instances
#'
#'                              \code{myDDI4_Binding$toParameter} returns a character vector of of DdiUrns for R6  Parameter class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Binding$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Binding <- R6Class(classname="DDI4_Binding",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, fromParameter=NA,toParameter=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Binding may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }



       if(!missing(fromParameter)){
        private$.fromParameter <- returnUrnVectorIfValid(validClass="DDI4_Parameter", proposedUrnVector=fromParameter , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.fromParameter, referencingAssociationName="fromParameter",  referencingUrn=thisUrn)  
      }
      if(!missing(toParameter)){
        private$.toParameter <- returnUrnVectorIfValid(validClass="DDI4_Parameter", proposedUrnVector=toParameter , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.toParameter, referencingAssociationName="toParameter",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 2 class association(s):
    fromParameter = function(val){
      if (!missing(val)){
        private$.fromParameter <- returnUrnVectorIfValid(validClass="DDI4_Parameter", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.fromParameter, referencingAssociationName="fromParameter",  referencingUrn=thisUrn)  
       # invisible( private$.fromParameter)
      } else {
          private$.fromParameter
      }
    },

    toParameter = function(val){
      if (!missing(val)){
        private$.toParameter <- returnUrnVectorIfValid(validClass="DDI4_Parameter", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.toParameter, referencingAssociationName="toParameter",  referencingUrn=thisUrn)  
       # invisible( private$.toParameter)
      } else {
          private$.toParameter
      }
    }

  ),

  private = list(
    .fromParameter = NULL,
    .toParameter = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        fromParameter = c(
          min="1", 
          max="1", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        toParameter = c(
          min="1", 
          max="1", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Binding


#' @title DDI4_BooleanResponseDomain  A response domain capturing a binary response, such as selected/unselected or checked/unchecked or true/false
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/BooleanResponseDomain}{BooleanResponseDomain} 

#'
#'   Definition
#'
#'   ============
#'
#'   A response domain capturing a binary response, such as selected/unselected or checked/unchecked or true/false.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Exactly what is captured may be dependent upon the Implemented Instrument, however the response being captured is either there or not there (ON/OFF, True/False)
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ResponseDomain}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{forCategory}}{ A read/write active binding.
#'          The category to which a boolean response is requested.
#'
#'                              \code{myDDI4_BooleanResponseDomain$forCategory <- c(myCategoryURN)} sets the association 'forCategory' from 'myDDI4_BooleanResponseDomain' to a character vector of DdiUrns for R6 Category class instances
#'
#'                              \code{myDDI4_BooleanResponseDomain$forCategory} returns a character vector of of DdiUrns for R6  Category class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_BooleanResponseDomain$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_BooleanResponseDomain <- R6Class(classname="DDI4_BooleanResponseDomain",
  inherit = DDI4_ResponseDomain,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, forCategory=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_BooleanResponseDomain may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)



       if(!missing(forCategory)){
        private$.forCategory <- returnUrnVectorIfValid(validClass="DDI4_Category", proposedUrnVector=forCategory , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.forCategory, referencingAssociationName="forCategory",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 class association(s):
    forCategory = function(val){
      if (!missing(val)){
        private$.forCategory <- returnUrnVectorIfValid(validClass="DDI4_Category", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.forCategory, referencingAssociationName="forCategory",  referencingUrn=thisUrn)  
       # invisible( private$.forCategory)
      } else {
          private$.forCategory
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .forCategory = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="1", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        forCategory = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Category", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        intendedRepresentation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RepresentedVariable", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_BooleanResponseDomain


#' @title DDI4_BoundingBox  A type of Spatial coverage describing a rectangular area within which the actual range of location fits
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/BoundingBox}{BoundingBox} 

#'
#'   Definition
#'
#'   ============
#'
#'   A type of Spatial coverage describing a rectangular area within which the actual range of location fits. A BoundingBox is described by 4 numbers - the maxima of the north, south, east, and west coordinates found in the area.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Burkino Faso: (N) 15.082773; (S) 9.395691; (E) 2.397927; (W) -5.520837
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   A BoundingBox is often described by two x,y coordinates where the x coordinates are used for the North and South Latitudes and y coordinates for the West and East Longitudes
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   r:BoundingBox
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_BoundingBox$DdiUrn} returns the DDI URN of myDDI4_BoundingBox.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{eastLongitude}}{ A read/write active binding.
#'        The easternmost coordinate expressed as a decimal between the values of -180 and 180 degrees 
#'
#'                              \code{myDDI4_BoundingBox$eastLongitude <- mynumeric} sets the eastLongitude property of 'myDDI4_BoundingBox' to the vector of datatype:numeric
#'
#'                              \code{myDDI4_BoundingBox$eastLongitude} returns a vector of datatype: numeric}
#'   }
#'   \describe{
#'     \item{\code{westLongitude}}{ A read/write active binding.
#'        The westernmost coordinate expressed as a decimal between the values of -180 and 180 degrees 
#'
#'                              \code{myDDI4_BoundingBox$westLongitude <- mynumeric} sets the westLongitude property of 'myDDI4_BoundingBox' to the vector of datatype:numeric
#'
#'                              \code{myDDI4_BoundingBox$westLongitude} returns a vector of datatype: numeric}
#'   }
#'   \describe{
#'     \item{\code{northLatitude}}{ A read/write active binding.
#'        The northernmost coordinate expressed as a decimal between the values of -90 and 90 degrees.
#'
#'                              \code{myDDI4_BoundingBox$northLatitude <- mynumeric} sets the northLatitude property of 'myDDI4_BoundingBox' to the vector of datatype:numeric
#'
#'                              \code{myDDI4_BoundingBox$northLatitude} returns a vector of datatype: numeric}
#'   }
#'   \describe{
#'     \item{\code{southLatitude}}{ A read/write active binding.
#'        The southermost latitude expressed as a decimal between the values of -90 and 90 degrees
#'
#'                              \code{myDDI4_BoundingBox$southLatitude <- mynumeric} sets the southLatitude property of 'myDDI4_BoundingBox' to the vector of datatype:numeric
#'
#'                              \code{myDDI4_BoundingBox$southLatitude} returns a vector of datatype: numeric}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_BoundingBox$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, eastLongitude=NA, westLongitude=NA, northLatitude=NA, southLatitude=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_BoundingBox <- R6Class(classname="DDI4_BoundingBox",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, eastLongitude=NA, westLongitude=NA, northLatitude=NA, southLatitude=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_BoundingBox may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(eastLongitude)){
        private$.eastLongitude <- returnVectorIfValid(validVectorClass="numeric", proposedVector=eastLongitude , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(westLongitude)){
        private$.westLongitude <- returnVectorIfValid(validVectorClass="numeric", proposedVector=westLongitude , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(northLatitude)){
        private$.northLatitude <- returnVectorIfValid(validVectorClass="numeric", proposedVector=northLatitude , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(southLatitude)){
        private$.southLatitude <- returnVectorIfValid(validVectorClass="numeric", proposedVector=southLatitude , minimumCardinality=1, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    eastLongitude = function(val){
      if (!missing(val)){
        private$.eastLongitude <- returnVectorIfValid(validVectorClass="numeric", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.eastLongitude
      }
    },

    northLatitude = function(val){
      if (!missing(val)){
        private$.northLatitude <- returnVectorIfValid(validVectorClass="numeric", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.northLatitude
      }
    },

    southLatitude = function(val){
      if (!missing(val)){
        private$.southLatitude <- returnVectorIfValid(validVectorClass="numeric", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.southLatitude
      }
    },

    westLongitude = function(val){
      if (!missing(val)){
        private$.westLongitude <- returnVectorIfValid(validVectorClass="numeric", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.westLongitude
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .eastLongitude = NULL,
    .westLongitude = NULL,
    .northLatitude = NULL,
    .southLatitude = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        eastLongitude = c(
          min="1", 
          max="1", 
          datatype = "numeric", 
          structure = "vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        northLatitude = c(
          min="1", 
          max="1", 
          datatype = "numeric", 
          structure = "vector"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        southLatitude = c(
          min="1", 
          max="1", 
          datatype = "numeric", 
          structure = "vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        westLongitude = c(
          min="1", 
          max="1", 
          datatype = "numeric", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_BoundingBox


#' @title DDI4_Budget  A description of the budget for any of the main publication types that can contain a reference to an external budget document
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Budget}{Budget} 

#'
#'   Definition
#'
#'   ============
#'
#'   A description of the budget for any of the main publication types that can contain a reference to an external budget document.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:BudgetType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Budget$DdiUrn} returns the DDI URN of myDDI4_Budget.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{overview}}{ A read/write active binding.
#'        Short natural language account of the information obtained from the combination of properties and relationships associated with an object. Supports the use of multiple languages and structured text. 
#'
#'                              \code{myDDI4_Budget$overview <- list(myInternationalStructuredString)} sets the 'overview' property of 'myDDI4_Budget' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Budget$overview} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage. 
#'
#'                              \code{myDDI4_Budget$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_Budget' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_Budget$name} returns a list of of R6 : ObjectName class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{budgetDocument}}{ A read/write active binding.
#'          References to one or more external budget documents.
#'
#'                              \code{myDDI4_Budget$budgetDocument <- c(myExternalMaterialURN)} sets the association 'budgetDocument' from 'myDDI4_Budget' to a character vector of DdiUrns for R6 ExternalMaterial class instances
#'
#'                              \code{myDDI4_Budget$budgetDocument} returns a character vector of of DdiUrns for R6  ExternalMaterial class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Budget$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, overview=NA, name=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Budget <- R6Class(classname="DDI4_Budget",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, overview=NA, name=NA,    budgetDocument=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Budget may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(overview)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=overview , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(budgetDocument)){
        private$.budgetDocument <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=budgetDocument , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.budgetDocument, referencingAssociationName="budgetDocument",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 2 properties: 
    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    overview = function(val){
      if (!missing(val)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overview
      }
    },


# There are 1 class association(s):
    budgetDocument = function(val){
      if (!missing(val)){
        private$.budgetDocument <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.budgetDocument, referencingAssociationName="budgetDocument",  referencingUrn=thisUrn)  
       # invisible( private$.budgetDocument)
      } else {
          private$.budgetDocument
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .overview = NULL,
    .name = NULL,
    .budgetDocument = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        budgetDocument = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Budget


#' @title DDI4_BusinessAlgorithm  A Business Algorithm is used to express the generalized function of the Business Process
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/BusinessAlgorithm}{BusinessAlgorithm} 

#'
#'   Definition
#'
#'   ============
#'
#'   A Business Algorithm is used to express the generalized function of the Business Process.
#'
#'   The underlying properties of the algorithm or method rather than the specifics of any particular implementation. In short a description of the method in its simplest and most general representation.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   There are several well-established algorithms for performing data anonymization including "local suppression", "global attribute generalization" and "k-Anonymization with suppression" which is a specific combination of the two. An anonymizing business process might implement one of these algorithms.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AlgorithmOverview}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_BusinessAlgorithm$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_BusinessAlgorithm <- R6Class(classname="DDI4_BusinessAlgorithm",
  inherit = DDI4_AlgorithmOverview,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_BusinessAlgorithm may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

    }
  ),

  active = list(
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isDescribedIn = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        subjectOfAlgorithm = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_BusinessAlgorithm


#' @title DDI4_BusinessFunction  Something an enterprise does, or needs to do, in order to achieve its objectives
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/BusinessFunction}{BusinessFunction} 

#'
#'   Definition
#'
#'   ============
#'
#'   Something an enterprise does, or needs to do, in order to achieve its objectives.
#'
#'   
#'
#'   A Business Function delivers added value from a business point of view. It is delivered by bringing together people, processes and technology (resources), for a specific business purpose. 
#'
#'   
#'
#'   Business Functions answer in a generic sense "What business purpose does this Business Service or Process Step serve?" Through identifying the Business Function associated with each Business Service or Process Step it increases the documentation of the use of the associated Business Services and Process Steps, to enable future reuse.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A Business Function may be defined directly with descriptive text and/or through reference to an existing catalogue of Business Functions. 
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Business Function
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_BusinessFunction$DdiUrn} returns the DDI URN of myDDI4_BusinessFunction.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{overview}}{ A read/write active binding.
#'        Provides a high level overview or summary of the class. Can be used to inform end-users or as part of an executive summary. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_BusinessFunction$overview <- list(myInternationalStructuredString)} sets the 'overview' property of 'myDDI4_BusinessFunction' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_BusinessFunction$overview} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_BusinessFunction$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, overview=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_BusinessFunction <- R6Class(classname="DDI4_BusinessFunction",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, overview=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_BusinessFunction may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(overview)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=overview , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 1 properties: 
    overview = function(val){
      if (!missing(val)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overview
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .overview = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_BusinessFunction


#' @title DDI4_BusinessProcess  BusinessProcesses could be Generic Longitudinal Business Process Model (GLBPM) and/or Generic Statistical Business Process Model (GSBPM) steps and/or sub-steps
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/BusinessProcess}{BusinessProcess} 

#'
#'   Definition
#'
#'   ============
#'
#'   BusinessProcesses could be Generic Longitudinal Business Process Model (GLBPM) and/or Generic Statistical Business Process Model (GSBPM) steps and/or sub-steps. BusinessProcesses participate in a DataPipeline by virtue of their preconditions and postconditions. 
#'
#'   
#'
#'   A BusinessProcess differs from a WorkflowStep by virtue of its granularity. Higher level operations are the subject of BusinessProcesses. A BusinessProcess has a WorkflowStepSequence through which these higher levels operations may optionally be decomposed into WorkflowSteps.
#'
#'   
#'
#'   BusinessProcess preconditions and postconditions differ from input and output parameters because they don't take variables and values. Instead the subject of preconditions and postconditions are typically whole datasets.
#'
#'   
#'
#'   Given this level of granularity it is possible to view a BusinessProcess through the Open Archival Information System (OAIS) Reference Model. So a BusinessProcess may optionally be described as a component of a Submission Information Package (SIP), an Archival Information Package (AIP) or a Dissemination Information Package (DIP).
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   In a single BusinessProcess we might receive data from a data source or merge data from different data sources or create new variables and recodes or format data for data dissemination
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   One or more PairedExternalControlledVocabularyEntry objects may be used to identify and characterize controlled vocabularies like the GLBPM, GSBPM and OAIS. The PairedExternalControlledVocabularyEntry both identifies both the model (term) and the step/sub-step of the model (extent). If a BusinessProcess combines two steps from the same model (GLBPM, GSBPM, OAIS, etc.) or from different models, multiple PairedExternalControlledVocabularyEntry objects can be specified.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   Task
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_WorkflowProcess}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_BusinessProcess$DdiUrn} returns the DDI URN of myDDI4_BusinessProcess.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{standardModelUsed}}{ A read/write active binding.
#'        If a standard process model such as The Generic Statistical Business Process Model (GSBPN), the Generic Longitudinal Business Process Model (GLBPM), Open Archive Information System model (OAIS), etc. has been related to this business process, the model and step or sub-step is noted here using the Paired External Controlled Vocabulary Entry. Enter the name of the model in "term" and the step, sub-step, or specific portion of the model in "extent".
#'
#'                              \code{myDDI4_BusinessProcess$standardModelUsed <- list(myPairedExternalControlledVocabularyEntry)} sets the 'standardModelUsed' property of 'myDDI4_BusinessProcess' to a list of R6 PairedExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_BusinessProcess$standardModelUsed} returns a list of of R6 : PairedExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{preCondition}}{ A read/write active binding.
#'        A condition for beginning the Business Process which may include the specification of the data set required for beginning the Business Process.
#'
#'                              \code{myDDI4_BusinessProcess$preCondition <- list(myBusinessProcessCondition)} sets the 'preCondition' property of 'myDDI4_BusinessProcess' to a list of R6 BusinessProcessCondition class instances
#'
#'                              \code{myDDI4_BusinessProcess$preCondition} returns a list of of R6 : BusinessProcessCondition class instances}
#'   }
#'   \describe{
#'     \item{\code{postCondition}}{ A read/write active binding.
#'        A condition for completing and exiting the Business Process which may include the specification of the data set resulting from the Business Process.
#'
#'                              \code{myDDI4_BusinessProcess$postCondition <- list(myBusinessProcessCondition)} sets the 'postCondition' property of 'myDDI4_BusinessProcess' to a list of R6 BusinessProcessCondition class instances
#'
#'                              \code{myDDI4_BusinessProcess$postCondition} returns a list of of R6 : BusinessProcessCondition class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_BusinessProcess$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, standardModelUsed=NA, preCondition=NA, postCondition=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_BusinessProcess <- R6Class(classname="DDI4_BusinessProcess",
  inherit = DDI4_WorkflowProcess,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, standardModelUsed=NA, preCondition=NA, postCondition=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_BusinessProcess may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(standardModelUsed)){
        private$.standardModelUsed <- returnR6ObjectIfValid(validClass="DDI4_PairedExternalControlledVocabularyEntry", proposedObjectList=standardModelUsed , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(preCondition)){
        private$.preCondition <- returnR6ObjectIfValid(validClass="DDI4_BusinessProcessCondition", proposedObjectList=preCondition , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(postCondition)){
        private$.postCondition <- returnR6ObjectIfValid(validClass="DDI4_BusinessProcessCondition", proposedObjectList=postCondition , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    postCondition = function(val){
      if (!missing(val)){
        private$.postCondition <- returnR6ObjectIfValid(validClass="DDI4_BusinessProcessCondition", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.postCondition
      }
    },

    preCondition = function(val){
      if (!missing(val)){
        private$.preCondition <- returnR6ObjectIfValid(validClass="DDI4_BusinessProcessCondition", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.preCondition
      }
    },

    standardModelUsed = function(val){
      if (!missing(val)){
        private$.standardModelUsed <- returnR6ObjectIfValid(validClass="DDI4_PairedExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.standardModelUsed
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .standardModelUsed = NULL,
    .preCondition = NULL,
    .postCondition = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        implementsAlgorithm = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AlgorithmOverview", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isSpecifiedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowControlStep", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        postCondition = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_BusinessProcessCondition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        preCondition = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_BusinessProcessCondition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        standardModelUsed = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_PairedExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_BusinessProcess


#' @title DDI4_BusinessProcessCondition  A BusinessProcess precondition or post condition which describes the condition which must be met to begin (pre) or exit (post) a process
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/BusinessProcessCondition}{BusinessProcessCondition} 

#'
#'   Definition
#'
#'   ============
#'
#'   A BusinessProcess precondition or post condition which describes the condition which must be met to begin (pre) or exit (post) a process. It may use a specified LogicalRecord. The Logical Record has SQL that describes it, rejectionCriteria against which its adequacy may be tested and an optional annotation that describes its provenance.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_BusinessProcessCondition$DdiUrn} returns the DDI URN of myDDI4_BusinessProcessCondition.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{sql}}{ A read/write active binding.
#'        SQL SELECT statement that describes the selection of the LogicalRecord from the Dataset
#'
#'                              \code{myDDI4_BusinessProcessCondition$sql <- mycharacter} sets the sql property of 'myDDI4_BusinessProcessCondition' to the vector of datatype:character
#'
#'                              \code{myDDI4_BusinessProcessCondition$sql} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{rejectionCriteria}}{ A read/write active binding.
#'        Criteria for failing an input dataset
#'
#'                              \code{myDDI4_BusinessProcessCondition$rejectionCriteria <- list(myCommandCode)} sets the 'rejectionCriteria' property of 'myDDI4_BusinessProcessCondition' to a list of R6 CommandCode class instances
#'
#'                              \code{myDDI4_BusinessProcessCondition$rejectionCriteria} returns a list of of R6 : CommandCode class instances}
#'   }
#'   \describe{
#'     \item{\code{dataDescription}}{ A read/write active binding.
#'        Provides an alternative to or supplements the SQL data description.
#'
#'                              \code{myDDI4_BusinessProcessCondition$dataDescription <- list(myInternationalStructuredString)} sets the 'dataDescription' property of 'myDDI4_BusinessProcessCondition' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_BusinessProcessCondition$dataDescription} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{usesLogicalRecord}}{ A read/write active binding.
#'          A Logical Record used by the BusinessProcessCondition in defining the condition.
#'
#'                              \code{myDDI4_BusinessProcessCondition$usesLogicalRecord <- c(myLogicalRecordURN)} sets the association 'usesLogicalRecord' from 'myDDI4_BusinessProcessCondition' to a character vector of DdiUrns for R6 LogicalRecord class instances
#'
#'                              \code{myDDI4_BusinessProcessCondition$usesLogicalRecord} returns a character vector of of DdiUrns for R6  LogicalRecord class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_BusinessProcessCondition$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, sql=NA, rejectionCriteria=NA, dataDescription=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_BusinessProcessCondition <- R6Class(classname="DDI4_BusinessProcessCondition",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, sql=NA, rejectionCriteria=NA, dataDescription=NA,    usesLogicalRecord=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_BusinessProcessCondition may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(sql)){
        private$.sql <- returnVectorIfValid(validVectorClass="character", proposedVector=sql , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(rejectionCriteria)){
        private$.rejectionCriteria <- returnR6ObjectIfValid(validClass="DDI4_CommandCode", proposedObjectList=rejectionCriteria , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(dataDescription)){
        private$.dataDescription <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=dataDescription , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(usesLogicalRecord)){
        private$.usesLogicalRecord <- returnUrnVectorIfValid(validClass="DDI4_LogicalRecord", proposedUrnVector=usesLogicalRecord , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesLogicalRecord, referencingAssociationName="usesLogicalRecord",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    dataDescription = function(val){
      if (!missing(val)){
        private$.dataDescription <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.dataDescription
      }
    },

    rejectionCriteria = function(val){
      if (!missing(val)){
        private$.rejectionCriteria <- returnR6ObjectIfValid(validClass="DDI4_CommandCode", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.rejectionCriteria
      }
    },

    sql = function(val){
      if (!missing(val)){
        private$.sql <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.sql
      }
    },


# There are 1 class association(s):
    usesLogicalRecord = function(val){
      if (!missing(val)){
        private$.usesLogicalRecord <- returnUrnVectorIfValid(validClass="DDI4_LogicalRecord", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesLogicalRecord, referencingAssociationName="usesLogicalRecord",  referencingUrn=thisUrn)  
       # invisible( private$.usesLogicalRecord)
      } else {
          private$.usesLogicalRecord
      }
    }

  ),

  private = list(
    .sql = NULL,
    .rejectionCriteria = NULL,
    .dataDescription = NULL,
    .usesLogicalRecord = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        dataDescription = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        rejectionCriteria = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CommandCode", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        sql = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        usesLogicalRecord = c(
          min="0", 
          max="1", 
          datatype = "DDI4_LogicalRecord", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_BusinessProcessCondition


#' @title DDI4_BusinessProcessIndicator  Allows for the identification of the BusinessProcess specifically as a member and optionally provides an index for the member within an ordered array
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/BusinessProcessIndicator}{BusinessProcessIndicator} 

#'
#'   Definition
#'
#'   ============
#'
#'   Allows for the identification of the BusinessProcess specifically as a member and optionally provides an index for the member within an ordered array. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_BusinessProcessIndicator$DdiUrn} returns the DDI URN of myDDI4_BusinessProcessIndicator.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Provides an index for the member within an ordered array
#'
#'                              \code{myDDI4_BusinessProcessIndicator$index <- myinteger} sets the index property of 'myDDI4_BusinessProcessIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_BusinessProcessIndicator$index} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{member}}{ A read/write active binding.
#'          Member of the collection
#'
#'                              \code{myDDI4_BusinessProcessIndicator$member <- c(myBusinessProcessURN)} sets the association 'member' from 'myDDI4_BusinessProcessIndicator' to a character vector of DdiUrns for R6 BusinessProcess class instances
#'
#'                              \code{myDDI4_BusinessProcessIndicator$member} returns a character vector of of DdiUrns for R6  BusinessProcess class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_BusinessProcessIndicator$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_BusinessProcessIndicator <- R6Class(classname="DDI4_BusinessProcessIndicator",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA,    member=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_BusinessProcessIndicator may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(member)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_BusinessProcess", proposedUrnVector=member , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.index
      }
    },


# There are 1 class association(s):
    member = function(val){
      if (!missing(val)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_BusinessProcess", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
       # invisible( private$.member)
      } else {
          private$.member
      }
    }

  ),

  private = list(
    .index = NULL,
    .member = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        index = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        member = c(
          min="1", 
          max="1", 
          datatype = "DDI4_BusinessProcess", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_BusinessProcessIndicator


#' @title DDI4_Capture  A measurement that describes a means of capturing data
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Capture}{Capture} 

#'
#'   Definition
#'
#'   ============
#'
#'   A measurement that describes a means of capturing data. This class can be extended to account for different specific means. Use a specific instantiation of a Capture to describe a means of capturing a measurement.
#'
#'   
#'
#'   
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A survey question, blood pressure reading; MRI images; thermometer; web service; experimental observation. Classes could include InstanceQuestion, InstanceMeasurement or other class extending Capture.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Provides an abstract base so that current and future forms of data capture can use this as an extension base and be freely mixed and matched within conceptual instruments as needed such as capturing a GPS point (using a RepresentedMeasurement) when administering a questionnaire (using RepresentedQuestions).
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Capture$DdiUrn} returns the DDI URN of myDDI4_Capture.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A name for the measurement. A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage. 
#'
#'                              \code{myDDI4_Capture$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_Capture' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_Capture$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A structured display label providing a fully human readable display for the identification of the object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_Capture$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_Capture' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_Capture$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#'   \describe{
#'     \item{\code{usage}}{ A read/write active binding.
#'        Explanation of the ways in which some decision or object is employed. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_Capture$usage <- list(myInternationalStructuredString)} sets the 'usage' property of 'myDDI4_Capture' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Capture$usage} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        A description of the purpose or use of the Measurement. May be expressed in multiple languages and supports the use of structured content. 
#'
#'                              \code{myDDI4_Capture$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_Capture' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Capture$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{captureSource}}{ A read/write active binding.
#'        The source of a capture structure defined briefly; typically using an external controlled vocabulary
#'
#'                              \code{myDDI4_Capture$captureSource <- list(myExternalControlledVocabularyEntry)} sets the 'captureSource' property of 'myDDI4_Capture' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_Capture$captureSource} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{analysisUnit}}{ A read/write active binding.
#'        Identifies the unit being analyzed such as a Person, Housing Unit, Enterprise, etc. 
#'
#'                              \code{myDDI4_Capture$analysisUnit <- list(myExternalControlledVocabularyEntry)} sets the 'analysisUnit' property of 'myDDI4_Capture' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_Capture$analysisUnit} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasResponseDomain}}{ A read/write active binding.
#'          
#'
#'                              \code{myDDI4_Capture$hasResponseDomain <- c(myResponseDomainURN)} sets the association 'hasResponseDomain' from 'myDDI4_Capture' to a character vector of DdiUrns for R6 ResponseDomain class instances
#'
#'                              \code{myDDI4_Capture$hasResponseDomain} returns a character vector of of DdiUrns for R6  ResponseDomain class instances}
#'   }
#'   \describe{
#'     \item{\code{hasConcept}}{ A read/write active binding.
#'          Capture has a Concept
#'
#'                              \code{myDDI4_Capture$hasConcept <- c(myConceptURN)} sets the association 'hasConcept' from 'myDDI4_Capture' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_Capture$hasConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#'   \describe{
#'     \item{\code{hasInstruction}}{ A read/write active binding.
#'          Capture has an Instruction
#'
#'                              \code{myDDI4_Capture$hasInstruction <- c(myInstructionURN)} sets the association 'hasInstruction' from 'myDDI4_Capture' to a character vector of DdiUrns for R6 Instruction class instances
#'
#'                              \code{myDDI4_Capture$hasInstruction} returns a character vector of of DdiUrns for R6  Instruction class instances}
#'   }
#'   \describe{
#'     \item{\code{hasExternalAid}}{ A read/write active binding.
#'          Capture has an External Aid
#'
#'                              \code{myDDI4_Capture$hasExternalAid <- c(myExternalAidURN)} sets the association 'hasExternalAid' from 'myDDI4_Capture' to a character vector of DdiUrns for R6 ExternalAid class instances
#'
#'                              \code{myDDI4_Capture$hasExternalAid} returns a character vector of of DdiUrns for R6  ExternalAid class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Capture$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, name=NA, displayLabel=NA, usage=NA, purpose=NA, captureSource=NA, analysisUnit=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Capture <- R6Class(classname="DDI4_Capture",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, name=NA, displayLabel=NA, usage=NA, purpose=NA, captureSource=NA, analysisUnit=NA,    hasResponseDomain=NA,hasConcept=NA,hasInstruction=NA,hasExternalAid=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Capture may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(usage)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=usage , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(captureSource)){
        private$.captureSource <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=captureSource , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(analysisUnit)){
        private$.analysisUnit <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=analysisUnit , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(hasResponseDomain)){
        private$.hasResponseDomain <- returnUrnVectorIfValid(validClass="DDI4_ResponseDomain", proposedUrnVector=hasResponseDomain , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasResponseDomain, referencingAssociationName="hasResponseDomain",  referencingUrn=thisUrn)  
      }
      if(!missing(hasConcept)){
        private$.hasConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=hasConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasConcept, referencingAssociationName="hasConcept",  referencingUrn=thisUrn)  
      }
      if(!missing(hasInstruction)){
        private$.hasInstruction <- returnUrnVectorIfValid(validClass="DDI4_Instruction", proposedUrnVector=hasInstruction , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasInstruction, referencingAssociationName="hasInstruction",  referencingUrn=thisUrn)  
      }
      if(!missing(hasExternalAid)){
        private$.hasExternalAid <- returnUrnVectorIfValid(validClass="DDI4_ExternalAid", proposedUrnVector=hasExternalAid , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasExternalAid, referencingAssociationName="hasExternalAid",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 6 properties: 
    analysisUnit = function(val){
      if (!missing(val)){
        private$.analysisUnit <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.analysisUnit
      }
    },

    captureSource = function(val){
      if (!missing(val)){
        private$.captureSource <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.captureSource
      }
    },

    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    usage = function(val){
      if (!missing(val)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.usage
      }
    },


# There are 4 class association(s):
    hasConcept = function(val){
      if (!missing(val)){
        private$.hasConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasConcept, referencingAssociationName="hasConcept",  referencingUrn=thisUrn)  
       # invisible( private$.hasConcept)
      } else {
          private$.hasConcept
      }
    },

    hasExternalAid = function(val){
      if (!missing(val)){
        private$.hasExternalAid <- returnUrnVectorIfValid(validClass="DDI4_ExternalAid", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasExternalAid, referencingAssociationName="hasExternalAid",  referencingUrn=thisUrn)  
       # invisible( private$.hasExternalAid)
      } else {
          private$.hasExternalAid
      }
    },

    hasInstruction = function(val){
      if (!missing(val)){
        private$.hasInstruction <- returnUrnVectorIfValid(validClass="DDI4_Instruction", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasInstruction, referencingAssociationName="hasInstruction",  referencingUrn=thisUrn)  
       # invisible( private$.hasInstruction)
      } else {
          private$.hasInstruction
      }
    },

    hasResponseDomain = function(val){
      if (!missing(val)){
        private$.hasResponseDomain <- returnUrnVectorIfValid(validClass="DDI4_ResponseDomain", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasResponseDomain, referencingAssociationName="hasResponseDomain",  referencingUrn=thisUrn)  
       # invisible( private$.hasResponseDomain)
      } else {
          private$.hasResponseDomain
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .name = NULL,
    .displayLabel = NULL,
    .usage = NULL,
    .purpose = NULL,
    .captureSource = NULL,
    .analysisUnit = NULL,
    .hasResponseDomain = NULL,
    .hasConcept = NULL,
    .hasInstruction = NULL,
    .hasExternalAid = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        analysisUnit = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        captureSource = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasExternalAid = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalAid", 
          structure = "DdiUrn vector"
        ),
        hasInstruction = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Instruction", 
          structure = "DdiUrn vector"
        ),
        hasResponseDomain = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ResponseDomain", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_Capture


#' @title DDI4_Category  A Concept whose role is to define and measure a characteristic
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Category}{Category} 

#'
#'   Definition
#'
#'   ============
#'
#'   A Concept whose role is to define and measure a characteristic.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Self-identified as "Male".  "Extremely Satisfied" as a response category.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   The Category is a Concept. It can have multiple name and display label properties as well as a definition and some descriptive text. As a "Signified" class there can be one or more "Sign" classes (e.g. a Code) that denotes it with some representation. The relationship is from the Code to the Category.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   l:CategoryType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Concept}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Category$DdiUrn} returns the DDI URN of myDDI4_Category.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{descriptiveText}}{ A read/write active binding.
#'        A short natural language account of the characteristics of the object.
#'
#'                              \code{myDDI4_Category$descriptiveText <- list(myInternationalStructuredString)} sets the 'descriptiveText' property of 'myDDI4_Category' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Category$descriptiveText} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Category$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, descriptiveText=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Category <- R6Class(classname="DDI4_Category",
  inherit = DDI4_Concept,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, descriptiveText=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Category may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(descriptiveText)){
        private$.descriptiveText <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=descriptiveText , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 1 properties: 
    descriptiveText = function(val){
      if (!missing(val)){
        private$.descriptiveText <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.descriptiveText
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .descriptiveText = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definition = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        descriptiveText = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Category


#' @title DDI4_CategoryIndicator  Member Indicator for use when the member type is restricted to Category
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CategoryIndicator}{CategoryIndicator} 

#'
#'   Definition
#'
#'   ============
#'
#'   Member Indicator for use when the member type is restricted to Category.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_CategoryIndicator$DdiUrn} returns the DDI URN of myDDI4_CategoryIndicator.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Index value of member in an ordered array
#'
#'                              \code{myDDI4_CategoryIndicator$index <- myinteger} sets the index property of 'myDDI4_CategoryIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_CategoryIndicator$index} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{member}}{ A read/write active binding.
#'          Restricts member target class to Concept or subtype of Concept
#'
#'                              \code{myDDI4_CategoryIndicator$member <- c(myCategoryURN)} sets the association 'member' from 'myDDI4_CategoryIndicator' to a character vector of DdiUrns for R6 Category class instances
#'
#'                              \code{myDDI4_CategoryIndicator$member} returns a character vector of of DdiUrns for R6  Category class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CategoryIndicator$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CategoryIndicator <- R6Class(classname="DDI4_CategoryIndicator",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA,    member=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CategoryIndicator may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(member)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_Category", proposedUrnVector=member , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.index
      }
    },


# There are 1 class association(s):
    member = function(val){
      if (!missing(val)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_Category", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
       # invisible( private$.member)
      } else {
          private$.member
      }
    }

  ),

  private = list(
    .index = NULL,
    .member = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        index = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        member = c(
          min="1", 
          max="1", 
          datatype = "DDI4_Category", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_CategoryIndicator


#' @title DDI4_CategoryRelation  Source target relationship between categories in a classification relation structure
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CategoryRelation}{CategoryRelation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Source target relationship between categories in a classification relation structure
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_CategoryRelation$DdiUrn} returns the DDI URN of myDDI4_CategoryRelation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_CategoryRelation$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_CategoryRelation' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_CategoryRelation$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an ennumeration list
#'
#'                              \code{myDDI4_CategoryRelation$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_CategoryRelation' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_CategoryRelation$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_CategoryRelation$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_CategoryRelation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_CategoryRelation$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          First member in a relationship.

#'
#'                              \code{myDDI4_CategoryRelation$source <- c(myCategoryURN)} sets the association 'source' from 'myDDI4_CategoryRelation' to a character vector of DdiUrns for R6 Category class instances
#'
#'                              \code{myDDI4_CategoryRelation$source} returns a character vector of of DdiUrns for R6  Category class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          Second member in a relationship. Note that this can be realized as a collection to support tuples.

#'
#'                              \code{myDDI4_CategoryRelation$target <- c(myCategoryURN)} sets the association 'target' from 'myDDI4_CategoryRelation' to a character vector of DdiUrns for R6 Category class instances
#'
#'                              \code{myDDI4_CategoryRelation$target} returns a character vector of of DdiUrns for R6  Category class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CategoryRelation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, totality=NA, hasRelationSpecification=NA, semantic=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CategoryRelation <- R6Class(classname="DDI4_CategoryRelation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, totality=NA, hasRelationSpecification=NA, semantic=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CategoryRelation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_Category", proposedUrnVector=source , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_Category", proposedUrnVector=target , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_Category", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_Category", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .totality = NULL,
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="1", 
          max="1", 
          datatype = "DDI4_Category", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Category", 
          structure = "DdiUrn vector"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_CategoryRelation


#' @title DDI4_CategoryRelationStructure  Relation structure of categories within a collection
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CategoryRelationStructure}{CategoryRelationStructure} 

#'
#'   Definition
#'
#'   ============
#'
#'   Relation structure of categories within a collection. Allows for the specification of complex relationships among categories.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   The category of student might be described as having sub-types of primary school student and high school student.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   The CategoryRelationStructure employs a set of CategoryRelations to describe the relationship among concepts. Each CategoryRelation is a one to many description of connections between categories. Together they might commonly describe relationships as complex as a hierarchy. 
#'
#'   This is a kind of a ConceptRelationStructure restricted to categories (which are concepts).
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_CategoryRelationStructure$DdiUrn} returns the DDI URN of myDDI4_CategoryRelationStructure.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an ennumeration list. Use if all relations within this relation structure are of the same specification.
#'
#'                              \code{myDDI4_CategoryRelationStructure$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_CategoryRelationStructure' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_CategoryRelationStructure$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_CategoryRelationStructure$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_CategoryRelationStructure' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_CategoryRelationStructure$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_CategoryRelationStructure$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_CategoryRelationStructure' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_CategoryRelationStructure$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMemberRelation}}{ A read/write active binding.
#'        Restricted to CategoryRelation. CategoryRelations making up the relation structure.
#'
#'                              \code{myDDI4_CategoryRelationStructure$hasMemberRelation <- list(myCategoryRelation)} sets the 'hasMemberRelation' property of 'myDDI4_CategoryRelationStructure' to a list of R6 CategoryRelation class instances
#'
#'                              \code{myDDI4_CategoryRelationStructure$hasMemberRelation} returns a list of of R6 : CategoryRelation class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CategoryRelationStructure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CategoryRelationStructure <- R6Class(classname="DDI4_CategoryRelationStructure",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CategoryRelationStructure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(hasMemberRelation)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_CategoryRelation", proposedObjectList=hasMemberRelation , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    hasMemberRelation = function(val){
      if (!missing(val)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_CategoryRelation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasMemberRelation
      }
    },

    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .hasMemberRelation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasMemberRelation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_CategoryRelation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        totality = c(
          min="1", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_CategoryRelationStructure


#' @title DDI4_CategorySet  Specialization of a Concept System focusing on sets of categories used as specifications for a generalized concept
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CategorySet}{CategorySet} 

#'
#'   Definition
#'
#'   ============
#'
#'   Specialization of a Concept System focusing on sets of categories used as specifications for a generalized concept. Can be used directly by questions to express a set of categories for a response domain or as the related source for a CodeList or StatisticalClassification
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   "Male" and "Female" categories in a CategorySet named "Gender"
#'
#'   
#'
#'   A CategorySet for an occupational classification system like ISCO-08 could use a ClassificationRelationStructure to describe a hierarchy (Chief Executives and Administrative and Commercial Managers as subtypes of Managers) without an associated Code List.
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ConceptSystem}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_CategorySet$DdiUrn} returns the DDI URN of myDDI4_CategorySet.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        MemberIndicator Allows for the identification of the member and optionally provides an index for the member within an ordered array
#'
#'                              \code{myDDI4_CategorySet$contains <- list(myCategoryIndicator)} sets the 'contains' property of 'myDDI4_CategorySet' to a list of R6 CategoryIndicator class instances
#'
#'                              \code{myDDI4_CategorySet$contains} returns a list of of R6 : CategoryIndicator class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isStructuredBy}}{ A read/write active binding.
#'          A structure containing specific relationships between members of the collection.
#'
#'                              \code{myDDI4_CategorySet$isStructuredBy <- c(myCategoryRelationStructureURN)} sets the association 'isStructuredBy' from 'myDDI4_CategorySet' to a character vector of DdiUrns for R6 CategoryRelationStructure class instances
#'
#'                              \code{myDDI4_CategorySet$isStructuredBy} returns a character vector of of DdiUrns for R6  CategoryRelationStructure class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CategorySet$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, contains=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CategorySet <- R6Class(classname="DDI4_CategorySet",
  inherit = DDI4_ConceptSystem,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, contains=NA,    isStructuredBy=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CategorySet may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_CategoryIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(isStructuredBy)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_CategoryRelationStructure", proposedUrnVector=isStructuredBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_CategoryIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },


# There are 1 class association(s):
    isStructuredBy = function(val){
      if (!missing(val)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_CategoryRelationStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
       # invisible( private$.isStructuredBy)
      } else {
          private$.isStructuredBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .contains = NULL,
    .isStructuredBy = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_CategoryIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isStructuredBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_CategoryRelationStructure", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_CategorySet


#' @title DDI4_CategoryStatistic  Statistics related to a specific category of an InstanceVariable within a data set
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CategoryStatistic}{CategoryStatistic} 

#'
#'   Definition
#'
#'   ============
#'
#'   Statistics related to a specific category of an InstanceVariable within a data set.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   pi:CategoryStatisticType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_CategoryStatistic$DdiUrn} returns the DDI URN of myDDI4_CategoryStatistic.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{typeOfCategoryStatistic}}{ A read/write active binding.
#'        Type of category statistic. Supports the use of an external controlled vocabulary. DDI strongly recommends the use of a controlled vocabulary.
#'
#'                              \code{myDDI4_CategoryStatistic$typeOfCategoryStatistic <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfCategoryStatistic' property of 'myDDI4_CategoryStatistic' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_CategoryStatistic$typeOfCategoryStatistic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{hasStatistic}}{ A read/write active binding.
#'        The value of the identified type of statistic for the category. May be repeated to provide unweighted or weighted values and different computation bases.
#'
#'                              \code{myDDI4_CategoryStatistic$hasStatistic <- list(myStatistic)} sets the 'hasStatistic' property of 'myDDI4_CategoryStatistic' to a list of R6 Statistic class instances
#'
#'                              \code{myDDI4_CategoryStatistic$hasStatistic} returns a list of of R6 : Statistic class instances}
#'   }
#'   \describe{
#'     \item{\code{categoryValue}}{ A read/write active binding.
#'        Value of the category to which the statistics apply
#'
#'                              \code{myDDI4_CategoryStatistic$categoryValue <- list(myValueString)} sets the 'categoryValue' property of 'myDDI4_CategoryStatistic' to a list of R6 ValueString class instances
#'
#'                              \code{myDDI4_CategoryStatistic$categoryValue} returns a list of of R6 : ValueString class instances}
#'   }
#'   \describe{
#'     \item{\code{filterValue}}{ A read/write active binding.
#'        The value of the filter variable to which the category statistic is restricted
#'
#'                              \code{myDDI4_CategoryStatistic$filterValue <- list(myValueString)} sets the 'filterValue' property of 'myDDI4_CategoryStatistic' to a list of R6 ValueString class instances
#'
#'                              \code{myDDI4_CategoryStatistic$filterValue} returns a list of of R6 : ValueString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{forCodeItem}}{ A read/write active binding.
#'          For categorical representations the CodeItem describing the value (signifier) and category (signified) may be identified.
#'
#'                              \code{myDDI4_CategoryStatistic$forCodeItem <- c(myDesignationURN)} sets the association 'forCodeItem' from 'myDDI4_CategoryStatistic' to a character vector of DdiUrns for R6 Designation class instances
#'
#'                              \code{myDDI4_CategoryStatistic$forCodeItem} returns a character vector of of DdiUrns for R6  Designation class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CategoryStatistic$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, typeOfCategoryStatistic=NA, hasStatistic=NA, categoryValue=NA, filterValue=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CategoryStatistic <- R6Class(classname="DDI4_CategoryStatistic",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, typeOfCategoryStatistic=NA, hasStatistic=NA, categoryValue=NA, filterValue=NA,    forCodeItem=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CategoryStatistic may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(typeOfCategoryStatistic)){
        private$.typeOfCategoryStatistic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfCategoryStatistic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasStatistic)){
        private$.hasStatistic <- returnR6ObjectIfValid(validClass="DDI4_Statistic", proposedObjectList=hasStatistic , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(categoryValue)){
        private$.categoryValue <- returnR6ObjectIfValid(validClass="DDI4_ValueString", proposedObjectList=categoryValue , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(filterValue)){
        private$.filterValue <- returnR6ObjectIfValid(validClass="DDI4_ValueString", proposedObjectList=filterValue , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(forCodeItem)){
        private$.forCodeItem <- returnUrnVectorIfValid(validClass="DDI4_Designation", proposedUrnVector=forCodeItem , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.forCodeItem, referencingAssociationName="forCodeItem",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 4 properties: 
    categoryValue = function(val){
      if (!missing(val)){
        private$.categoryValue <- returnR6ObjectIfValid(validClass="DDI4_ValueString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.categoryValue
      }
    },

    filterValue = function(val){
      if (!missing(val)){
        private$.filterValue <- returnR6ObjectIfValid(validClass="DDI4_ValueString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.filterValue
      }
    },

    hasStatistic = function(val){
      if (!missing(val)){
        private$.hasStatistic <- returnR6ObjectIfValid(validClass="DDI4_Statistic", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasStatistic
      }
    },

    typeOfCategoryStatistic = function(val){
      if (!missing(val)){
        private$.typeOfCategoryStatistic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.typeOfCategoryStatistic
      }
    },


# There are 1 class association(s):
    forCodeItem = function(val){
      if (!missing(val)){
        private$.forCodeItem <- returnUrnVectorIfValid(validClass="DDI4_Designation", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.forCodeItem, referencingAssociationName="forCodeItem",  referencingUrn=thisUrn)  
       # invisible( private$.forCodeItem)
      } else {
          private$.forCodeItem
      }
    }

  ),

  private = list(
    .typeOfCategoryStatistic = NULL,
    .hasStatistic = NULL,
    .categoryValue = NULL,
    .filterValue = NULL,
    .forCodeItem = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        categoryValue = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ValueString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        filterValue = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ValueString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        forCodeItem = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Designation", 
          structure = "DdiUrn vector"
        ),
        hasStatistic = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Statistic", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        typeOfCategoryStatistic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_CategoryStatistic


#' @title DDI4_CharacterOffset  Specification of the character offset for the beginning and end of the segment, or beginning and length
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CharacterOffset}{CharacterOffset} 

#'
#'   Definition
#'
#'   ============
#'
#'   Specification of the character offset for the beginning and end of the segment, or beginning and length.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:CharacterParameterType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_CharacterOffset$DdiUrn} returns the DDI URN of myDDI4_CharacterOffset.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{startCharOffset}}{ A read/write active binding.
#'        Number of characters from beginning of the document, indicating the inclusive start of the text range.
#'
#'                              \code{myDDI4_CharacterOffset$startCharOffset <- myinteger} sets the startCharOffset property of 'myDDI4_CharacterOffset' to the vector of datatype:integer
#'
#'                              \code{myDDI4_CharacterOffset$startCharOffset} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{endCharOffset}}{ A read/write active binding.
#'        Number of characters from the beginning of the document, indicating the inclusive end of the text segment.
#'
#'                              \code{myDDI4_CharacterOffset$endCharOffset <- myinteger} sets the endCharOffset property of 'myDDI4_CharacterOffset' to the vector of datatype:integer
#'
#'                              \code{myDDI4_CharacterOffset$endCharOffset} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{characterLength}}{ A read/write active binding.
#'        can be used to describe a text segment as start and length
#'
#'                              \code{myDDI4_CharacterOffset$characterLength <- myinteger} sets the characterLength property of 'myDDI4_CharacterOffset' to the vector of datatype:integer
#'
#'                              \code{myDDI4_CharacterOffset$characterLength} returns a vector of datatype: integer}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CharacterOffset$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, startCharOffset=NA, endCharOffset=NA, characterLength=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CharacterOffset <- R6Class(classname="DDI4_CharacterOffset",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, startCharOffset=NA, endCharOffset=NA, characterLength=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CharacterOffset may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(startCharOffset)){
        private$.startCharOffset <- returnVectorIfValid(validVectorClass="integer", proposedVector=startCharOffset , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(endCharOffset)){
        private$.endCharOffset <- returnVectorIfValid(validVectorClass="integer", proposedVector=endCharOffset , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(characterLength)){
        private$.characterLength <- returnVectorIfValid(validVectorClass="integer", proposedVector=characterLength , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    characterLength = function(val){
      if (!missing(val)){
        private$.characterLength <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.characterLength
      }
    },

    endCharOffset = function(val){
      if (!missing(val)){
        private$.endCharOffset <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.endCharOffset
      }
    },

    startCharOffset = function(val){
      if (!missing(val)){
        private$.startCharOffset <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.startCharOffset
      }
    }


  ),

  private = list(
    .startCharOffset = NULL,
    .endCharOffset = NULL,
    .characterLength = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        characterLength = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        endCharOffset = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        startCharOffset = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_CharacterOffset


#' @title DDI4_ClassificationFamily  A Classification Family is a collection of Classification Series related from a particular point of view
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ClassificationFamily}{ClassificationFamily} 

#'
#'   Definition
#'
#'   ============
#'
#'   A Classification Family is a collection of Classification Series related from a particular point of view. The Classification Family is related by being based on a common Concept, as documented by the definingConcept relationship   (e.g. economic activity).[GSIM1.1]
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A family of industrial classifications each a separate series (i.e. U.S. Standard Industrial Classification (SIC) and North American Industrial Classification System (NAICS)
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Different classification databases may use different types of Classification Families and have different names for the families, as no standard has been agreed upon. [GSIM1.1]
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Classification Family
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ClassificationFamily$DdiUrn} returns the DDI URN of myDDI4_ClassificationFamily.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{type}}{ A read/write active binding.
#'        Whether the collection is a bag or a set: a bag is a collection with duplicates allowed, a set is a collection without duplicates.
#'
#'                              \code{myDDI4_ClassificationFamily$type <- list(myCollectionType)} sets the 'type' property of 'myDDI4_ClassificationFamily' to a list of R6 CollectionType class instances
#'
#'                              \code{myDDI4_ClassificationFamily$type} returns a list of of R6 : CollectionType class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage.
#'
#'                              \code{myDDI4_ClassificationFamily$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_ClassificationFamily' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_ClassificationFamily$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_ClassificationFamily$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_ClassificationFamily' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_ClassificationFamily$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        Allows for the identification of the member Classification Series and optionally provides an index for the member within an ordered array
#'
#'                              \code{myDDI4_ClassificationFamily$contains <- list(myClassificationSeriesIndicator)} sets the 'contains' property of 'myDDI4_ClassificationFamily' to a list of R6 ClassificationSeriesIndicator class instances
#'
#'                              \code{myDDI4_ClassificationFamily$contains} returns a list of of R6 : ClassificationSeriesIndicator class instances}
#'   }
#'   \describe{
#'     \item{\code{isOrdered}}{ A read/write active binding.
#'        If members are ordered set to true, if unordered set to false.
#'
#'                              \code{myDDI4_ClassificationFamily$isOrdered <- mylogical} sets the isOrdered property of 'myDDI4_ClassificationFamily' to the vector of datatype:logical
#'
#'                              \code{myDDI4_ClassificationFamily$isOrdered} returns a vector of datatype: logical}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasClassificationIndex}}{ A read/write active binding.
#'          ClassificationIndexes associated to the ClassificationFamily.
#'
#'                              \code{myDDI4_ClassificationFamily$hasClassificationIndex <- c(myClassificationIndexURN)} sets the association 'hasClassificationIndex' from 'myDDI4_ClassificationFamily' to a character vector of DdiUrns for R6 ClassificationIndex class instances
#'
#'                              \code{myDDI4_ClassificationFamily$hasClassificationIndex} returns a character vector of of DdiUrns for R6  ClassificationIndex class instances}
#'   }
#'   \describe{
#'     \item{\code{isStructuredBy}}{ A read/write active binding.
#'          Description of a complex structure for the Collection. A collection may be structured in more than one way for different uses.
#'
#'                              \code{myDDI4_ClassificationFamily$isStructuredBy <- c(myClassificationSeriesRelationStructureURN)} sets the association 'isStructuredBy' from 'myDDI4_ClassificationFamily' to a character vector of DdiUrns for R6 ClassificationSeriesRelationStructure class instances
#'
#'                              \code{myDDI4_ClassificationFamily$isStructuredBy} returns a character vector of of DdiUrns for R6  ClassificationSeriesRelationStructure class instances}
#'   }
#'   \describe{
#'     \item{\code{definingConcept}}{ A read/write active binding.
#'          The conceptual basis for the collection of members.
#'
#'                              \code{myDDI4_ClassificationFamily$definingConcept <- c(myConceptURN)} sets the association 'definingConcept' from 'myDDI4_ClassificationFamily' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_ClassificationFamily$definingConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ClassificationFamily$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ClassificationFamily <- R6Class(classname="DDI4_ClassificationFamily",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA,    hasClassificationIndex=NA,isStructuredBy=NA,definingConcept=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ClassificationFamily may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(type)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=type , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_ClassificationSeriesIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(isOrdered)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=isOrdered , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(hasClassificationIndex)){
        private$.hasClassificationIndex <- returnUrnVectorIfValid(validClass="DDI4_ClassificationIndex", proposedUrnVector=hasClassificationIndex , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasClassificationIndex, referencingAssociationName="hasClassificationIndex",  referencingUrn=thisUrn)  
      }
      if(!missing(isStructuredBy)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_ClassificationSeriesRelationStructure", proposedUrnVector=isStructuredBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
      }
      if(!missing(definingConcept)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=definingConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 5 properties: 
    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_ClassificationSeriesIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },

    isOrdered = function(val){
      if (!missing(val)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isOrdered
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    type = function(val){
      if (!missing(val)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.type
      }
    },


# There are 3 class association(s):
    definingConcept = function(val){
      if (!missing(val)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
       # invisible( private$.definingConcept)
      } else {
          private$.definingConcept
      }
    },

    hasClassificationIndex = function(val){
      if (!missing(val)){
        private$.hasClassificationIndex <- returnUrnVectorIfValid(validClass="DDI4_ClassificationIndex", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasClassificationIndex, referencingAssociationName="hasClassificationIndex",  referencingUrn=thisUrn)  
       # invisible( private$.hasClassificationIndex)
      } else {
          private$.hasClassificationIndex
      }
    },

    isStructuredBy = function(val){
      if (!missing(val)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_ClassificationSeriesRelationStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
       # invisible( private$.isStructuredBy)
      } else {
          private$.isStructuredBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .type = NULL,
    .name = NULL,
    .purpose = NULL,
    .contains = NULL,
    .isOrdered = NULL,
    .hasClassificationIndex = NULL,
    .isStructuredBy = NULL,
    .definingConcept = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ClassificationSeriesIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasClassificationIndex = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ClassificationIndex", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isStructuredBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ClassificationSeriesRelationStructure", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ClassificationFamily


#' @title DDI4_ClassificationIndex  A Classification Index is an ordered list (alphabetical, in code order etc) of Classification Index Entries
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ClassificationIndex}{ClassificationIndex} 

#'
#'   Definition
#'
#'   ============
#'
#'   A Classification Index is an ordered list (alphabetical, in code order etc) of Classification Index Entries. A Classification Index can relate to one particular or to several Statistical Classifications. [GSIM Statistical Classification Model]
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   An alphabetical index of a topically ordered Statistical Classification
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   A Classification Index shows the relationship between text found in statistical data sources (responses to survey questionnaires, administrative records) and one or more Statistical Classifications.  A Classification Index may be used to assign the codes for Classification Items to observations in statistical collections. 
#'
#'   A Statistical Classification is a subtype of a Structured Collection.  [GSIM Statistical Classification Model]  Note that a GSIM Node is the equivalent of a DDI Member and a GSIM Node Set is a DDI Collection.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Classification Index
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ClassificationIndex$DdiUrn} returns the DDI URN of myDDI4_ClassificationIndex.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{releaseDate}}{ A read/write active binding.
#'        Date when the current version of the Classification Index was released.
#'
#'                              \code{myDDI4_ClassificationIndex$releaseDate <- list(myDate)} sets the 'releaseDate' property of 'myDDI4_ClassificationIndex' to a list of R6 Date class instances
#'
#'                              \code{myDDI4_ClassificationIndex$releaseDate} returns a list of of R6 : Date class instances}
#'   }
#'   \describe{
#'     \item{\code{availableLanguage}}{ A read/write active binding.
#'        A Classification Index can exist in several languages. Indicates the languages available. If a Classification Index exists in several languages, the number of entries in each language may be different, as the number of terms describing the same phenomenon can change from one language to another. However, the same phenomena should be described in each language. Supports the indication of multiple languages within a single property. Supports use of codes defined by the RFC 1766.
#'
#'                              \code{myDDI4_ClassificationIndex$availableLanguage <- mycharacter} sets the availableLanguage property of 'myDDI4_ClassificationIndex' to the vector of datatype:character
#'
#'                              \code{myDDI4_ClassificationIndex$availableLanguage} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{corrections}}{ A read/write active binding.
#'        Verbal summary description of corrections, which have occurred within the Classification Index. Corrections include changing the item code associated with an Classification Index Entry.
#'
#'                              \code{myDDI4_ClassificationIndex$corrections <- list(myInternationalString)} sets the 'corrections' property of 'myDDI4_ClassificationIndex' to a list of R6 InternationalString class instances
#'
#'                              \code{myDDI4_ClassificationIndex$corrections} returns a list of of R6 : InternationalString class instances}
#'   }
#'   \describe{
#'     \item{\code{codingInstruction}}{ A read/write active binding.
#'        Additional information which drives the coding process for all entries in a Classification Index.
#'
#'                              \code{myDDI4_ClassificationIndex$codingInstruction <- list(myCommandCode)} sets the 'codingInstruction' property of 'myDDI4_ClassificationIndex' to a list of R6 CommandCode class instances
#'
#'                              \code{myDDI4_ClassificationIndex$codingInstruction} returns a list of of R6 : CommandCode class instances}
#'   }
#'   \describe{
#'     \item{\code{type}}{ A read/write active binding.
#'        Whether the collection is a bag or a set: a bag is a collection with duplicates allowed, a set is a collection without duplicates.
#'
#'                              \code{myDDI4_ClassificationIndex$type <- list(myCollectionType)} sets the 'type' property of 'myDDI4_ClassificationIndex' to a list of R6 CollectionType class instances
#'
#'                              \code{myDDI4_ClassificationIndex$type} returns a list of of R6 : CollectionType class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage.
#'
#'                              \code{myDDI4_ClassificationIndex$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_ClassificationIndex' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_ClassificationIndex$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_ClassificationIndex$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_ClassificationIndex' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_ClassificationIndex$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        Allows for the identification of the member and optionally provides an index for the member within an ordered array
#'
#'                              \code{myDDI4_ClassificationIndex$contains <- list(myClassificationIndexEntryIndicator)} sets the 'contains' property of 'myDDI4_ClassificationIndex' to a list of R6 ClassificationIndexEntryIndicator class instances
#'
#'                              \code{myDDI4_ClassificationIndex$contains} returns a list of of R6 : ClassificationIndexEntryIndicator class instances}
#'   }
#'   \describe{
#'     \item{\code{isOrdered}}{ A read/write active binding.
#'        If members are ordered set to true, if unordered set to false.
#'
#'                              \code{myDDI4_ClassificationIndex$isOrdered <- mylogical} sets the isOrdered property of 'myDDI4_ClassificationIndex' to the vector of datatype:logical
#'
#'                              \code{myDDI4_ClassificationIndex$isOrdered} returns a vector of datatype: logical}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasPublication}}{ A read/write active binding.
#'          A list of the publications in which the Classification Index has been published.
#'
#'                              \code{myDDI4_ClassificationIndex$hasPublication <- c(myExternalMaterialURN)} sets the association 'hasPublication' from 'myDDI4_ClassificationIndex' to a character vector of DdiUrns for R6 ExternalMaterial class instances
#'
#'                              \code{myDDI4_ClassificationIndex$hasPublication} returns a character vector of of DdiUrns for R6  ExternalMaterial class instances}
#'   }
#'   \describe{
#'     \item{\code{isStructuredBy}}{ A read/write active binding.
#'          Description of a complex structure for the Collection. A collection may be structured in more than one way for different uses.
#'
#'                              \code{myDDI4_ClassificationIndex$isStructuredBy <- c(myIndexEntryRelationStructureURN)} sets the association 'isStructuredBy' from 'myDDI4_ClassificationIndex' to a character vector of DdiUrns for R6 IndexEntryRelationStructure class instances
#'
#'                              \code{myDDI4_ClassificationIndex$isStructuredBy} returns a character vector of of DdiUrns for R6  IndexEntryRelationStructure class instances}
#'   }
#'   \describe{
#'     \item{\code{maintenanceUnit}}{ A read/write active binding.
#'          The unit or group of persons within the organization responsible for the Classification Index, i.e. for adding, changing or deleting Classification Index Entries.
#'
#'                              \code{myDDI4_ClassificationIndex$maintenanceUnit <- c(myAgentURN)} sets the association 'maintenanceUnit' from 'myDDI4_ClassificationIndex' to a character vector of DdiUrns for R6 Agent class instances
#'
#'                              \code{myDDI4_ClassificationIndex$maintenanceUnit} returns a character vector of of DdiUrns for R6  Agent class instances}
#'   }
#'   \describe{
#'     \item{\code{contactPerson}}{ A read/write active binding.
#'          Person(s) who may be contacted for additional information about the Classification Index.
#'
#'                              \code{myDDI4_ClassificationIndex$contactPerson <- c(myAgentURN)} sets the association 'contactPerson' from 'myDDI4_ClassificationIndex' to a character vector of DdiUrns for R6 Agent class instances
#'
#'                              \code{myDDI4_ClassificationIndex$contactPerson} returns a character vector of of DdiUrns for R6  Agent class instances}
#'   }
#'   \describe{
#'     \item{\code{definingConcept}}{ A read/write active binding.
#'          The conceptual basis for the collection of members.
#'
#'                              \code{myDDI4_ClassificationIndex$definingConcept <- c(myConceptURN)} sets the association 'definingConcept' from 'myDDI4_ClassificationIndex' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_ClassificationIndex$definingConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ClassificationIndex$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, releaseDate=NA, availableLanguage=NA, corrections=NA, codingInstruction=NA, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ClassificationIndex <- R6Class(classname="DDI4_ClassificationIndex",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, releaseDate=NA, availableLanguage=NA, corrections=NA, codingInstruction=NA, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA,    hasPublication=NA,isStructuredBy=NA,maintenanceUnit=NA,contactPerson=NA,definingConcept=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ClassificationIndex may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(releaseDate)){
        private$.releaseDate <- returnR6ObjectIfValid(validClass="DDI4_Date", proposedObjectList=releaseDate , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(availableLanguage)){
        private$.availableLanguage <- returnVectorIfValid(validVectorClass="character", proposedVector=availableLanguage , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(corrections)){
        private$.corrections <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=corrections , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(codingInstruction)){
        private$.codingInstruction <- returnR6ObjectIfValid(validClass="DDI4_CommandCode", proposedObjectList=codingInstruction , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(type)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=type , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_ClassificationIndexEntryIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(isOrdered)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=isOrdered , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(hasPublication)){
        private$.hasPublication <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=hasPublication , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasPublication, referencingAssociationName="hasPublication",  referencingUrn=thisUrn)  
      }
      if(!missing(isStructuredBy)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_IndexEntryRelationStructure", proposedUrnVector=isStructuredBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
      }
      if(!missing(maintenanceUnit)){
        private$.maintenanceUnit <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=maintenanceUnit , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.maintenanceUnit, referencingAssociationName="maintenanceUnit",  referencingUrn=thisUrn)  
      }
      if(!missing(contactPerson)){
        private$.contactPerson <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=contactPerson , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.contactPerson, referencingAssociationName="contactPerson",  referencingUrn=thisUrn)  
      }
      if(!missing(definingConcept)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=definingConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 9 properties: 
    availableLanguage = function(val){
      if (!missing(val)){
        private$.availableLanguage <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.availableLanguage
      }
    },

    codingInstruction = function(val){
      if (!missing(val)){
        private$.codingInstruction <- returnR6ObjectIfValid(validClass="DDI4_CommandCode", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.codingInstruction
      }
    },

    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_ClassificationIndexEntryIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },

    corrections = function(val){
      if (!missing(val)){
        private$.corrections <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.corrections
      }
    },

    isOrdered = function(val){
      if (!missing(val)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isOrdered
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    releaseDate = function(val){
      if (!missing(val)){
        private$.releaseDate <- returnR6ObjectIfValid(validClass="DDI4_Date", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.releaseDate
      }
    },

    type = function(val){
      if (!missing(val)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.type
      }
    },


# There are 5 class association(s):
    contactPerson = function(val){
      if (!missing(val)){
        private$.contactPerson <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.contactPerson, referencingAssociationName="contactPerson",  referencingUrn=thisUrn)  
       # invisible( private$.contactPerson)
      } else {
          private$.contactPerson
      }
    },

    definingConcept = function(val){
      if (!missing(val)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
       # invisible( private$.definingConcept)
      } else {
          private$.definingConcept
      }
    },

    hasPublication = function(val){
      if (!missing(val)){
        private$.hasPublication <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasPublication, referencingAssociationName="hasPublication",  referencingUrn=thisUrn)  
       # invisible( private$.hasPublication)
      } else {
          private$.hasPublication
      }
    },

    isStructuredBy = function(val){
      if (!missing(val)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_IndexEntryRelationStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
       # invisible( private$.isStructuredBy)
      } else {
          private$.isStructuredBy
      }
    },

    maintenanceUnit = function(val){
      if (!missing(val)){
        private$.maintenanceUnit <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.maintenanceUnit, referencingAssociationName="maintenanceUnit",  referencingUrn=thisUrn)  
       # invisible( private$.maintenanceUnit)
      } else {
          private$.maintenanceUnit
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .releaseDate = NULL,
    .availableLanguage = NULL,
    .corrections = NULL,
    .codingInstruction = NULL,
    .type = NULL,
    .name = NULL,
    .purpose = NULL,
    .contains = NULL,
    .isOrdered = NULL,
    .hasPublication = NULL,
    .isStructuredBy = NULL,
    .maintenanceUnit = NULL,
    .contactPerson = NULL,
    .definingConcept = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        availableLanguage = c(
          min="0", 
          max="Inf", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        codingInstruction = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_CommandCode", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contactPerson = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Agent", 
          structure = "DdiUrn vector"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ClassificationIndexEntryIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        corrections = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasPublication = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isStructuredBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_IndexEntryRelationStructure", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        maintenanceUnit = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Agent", 
          structure = "DdiUrn vector"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        releaseDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Date", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ClassificationIndex


#' @title DDI4_ClassificationIndexEntry  A Classification Index Entry is a word or a short text (e
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ClassificationIndexEntry}{ClassificationIndexEntry} 

#'
#'   Definition
#'
#'   ============
#'
#'   A Classification Index Entry is a word or a short text (e.g. the name of a locality, an economic activity or an occupational title) describing a type of object/unit or object property to which a Classification Item applies, together with the code of the corresponding Classification Item. Each Classification Index Entry typically refers to one item of the Statistical Classification. Although a Classification Index Entry may be associated with a Classification Item at any Level of a Statistical Classification, Classification Index Entries are normally associated with items at the lowest Level.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Classification Index Entry
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ClassificationIndexEntry$DdiUrn} returns the DDI URN of myDDI4_ClassificationIndexEntry.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{entry}}{ A read/write active binding.
#'        Text describing the type of object/unit or object property.
#'
#'                              \code{myDDI4_ClassificationIndexEntry$entry <- list(myInternationalString)} sets the 'entry' property of 'myDDI4_ClassificationIndexEntry' to a list of R6 InternationalString class instances
#'
#'                              \code{myDDI4_ClassificationIndexEntry$entry} returns a list of of R6 : InternationalString class instances}
#'   }
#'   \describe{
#'     \item{\code{validDates}}{ A read/write active binding.
#'        Date from which the Classification Index Entry became valid (startDate). The date must be defined if the Classification Index Entry belongs to a floating Classification Index. Date at which the Classification Index Entry became invalid (endDate). The date must be defined if the Classification Index Entry belongs to a floating Classification Index and is no longer valid. 
#'
#'                              \code{myDDI4_ClassificationIndexEntry$validDates <- list(myDateRange)} sets the 'validDates' property of 'myDDI4_ClassificationIndexEntry' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_ClassificationIndexEntry$validDates} returns a list of of R6 : DateRange class instances}
#'   }
#'   \describe{
#'     \item{\code{codingInstruction}}{ A read/write active binding.
#'        Additional information which drives the coding process. Required when coding is dependent upon one or many other factors.
#'
#'                              \code{myDDI4_ClassificationIndexEntry$codingInstruction <- list(myCommandCode)} sets the 'codingInstruction' property of 'myDDI4_ClassificationIndexEntry' to a list of R6 CommandCode class instances
#'
#'                              \code{myDDI4_ClassificationIndexEntry$codingInstruction} returns a list of of R6 : CommandCode class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ClassificationIndexEntry$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, entry=NA, validDates=NA, codingInstruction=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ClassificationIndexEntry <- R6Class(classname="DDI4_ClassificationIndexEntry",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, entry=NA, validDates=NA, codingInstruction=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ClassificationIndexEntry may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(entry)){
        private$.entry <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=entry , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(validDates)){
        private$.validDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=validDates , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(codingInstruction)){
        private$.codingInstruction <- returnR6ObjectIfValid(validClass="DDI4_CommandCode", proposedObjectList=codingInstruction , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    codingInstruction = function(val){
      if (!missing(val)){
        private$.codingInstruction <- returnR6ObjectIfValid(validClass="DDI4_CommandCode", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.codingInstruction
      }
    },

    entry = function(val){
      if (!missing(val)){
        private$.entry <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.entry
      }
    },

    validDates = function(val){
      if (!missing(val)){
        private$.validDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.validDates
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .entry = NULL,
    .validDates = NULL,
    .codingInstruction = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        codingInstruction = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CommandCode", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        entry = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        validDates = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ClassificationIndexEntry


#' @title DDI4_ClassificationIndexEntryIndicator  Member Indicator for use with member type ClassificationIndexEntry
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ClassificationIndexEntryIndicator}{ClassificationIndexEntryIndicator} 

#'
#'   Definition
#'
#'   ============
#'
#'   Member Indicator for use with member type ClassificationIndexEntry
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ClassificationIndexEntryIndicator$DdiUrn} returns the DDI URN of myDDI4_ClassificationIndexEntryIndicator.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Index value of member in an ordered array
#'
#'                              \code{myDDI4_ClassificationIndexEntryIndicator$index <- myinteger} sets the index property of 'myDDI4_ClassificationIndexEntryIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_ClassificationIndexEntryIndicator$index} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{member}}{ A read/write active binding.
#'          Restricts member target class to ClassificationIndexEntry
#'
#'                              \code{myDDI4_ClassificationIndexEntryIndicator$member <- c(myClassificationIndexEntryURN)} sets the association 'member' from 'myDDI4_ClassificationIndexEntryIndicator' to a character vector of DdiUrns for R6 ClassificationIndexEntry class instances
#'
#'                              \code{myDDI4_ClassificationIndexEntryIndicator$member} returns a character vector of of DdiUrns for R6  ClassificationIndexEntry class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ClassificationIndexEntryIndicator$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ClassificationIndexEntryIndicator <- R6Class(classname="DDI4_ClassificationIndexEntryIndicator",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA,    member=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ClassificationIndexEntryIndicator may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(member)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_ClassificationIndexEntry", proposedUrnVector=member , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.index
      }
    },


# There are 1 class association(s):
    member = function(val){
      if (!missing(val)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_ClassificationIndexEntry", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
       # invisible( private$.member)
      } else {
          private$.member
      }
    }

  ),

  private = list(
    .index = NULL,
    .member = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        index = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        member = c(
          min="1", 
          max="1", 
          datatype = "DDI4_ClassificationIndexEntry", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ClassificationIndexEntryIndicator


#' @title DDI4_ClassificationItem  A Classification Item represents a Category at a certain Level within a Statistical Classification
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ClassificationItem}{ClassificationItem} 

#'
#'   Definition
#'
#'   ============
#'
#'   A Classification Item represents a Category at a certain Level within a Statistical Classification.
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   In the  2012 NAICS (North American Industry Classification System) one Classification Item has the Code 23 and the Category construction.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   A Classification Item defines the content and the borders of the Category. A Unit can be classified to one and only one item at each Level of a Statistical Classification. As such a Classification Item is a placeholder for a position in a StatisitcalClassification. It contains a Designation, for which Code is a kind; a Category; and other things. 
#'
#'   
#'
#'   This differentiates it from Code which is a kind of Designation, in particular it is an alphanumeric string assigned to stand in place of a category. For example, the letter M might stand for the category Male in the CodeList called Gender.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Classification Item
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Designation}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ClassificationItem$DdiUrn} returns the DDI URN of myDDI4_ClassificationItem.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{isValid}}{ A read/write active binding.
#'        Indicates whether or not the item is currently valid. If updates are allowed in the Statistical Classification, an item may be restricted in its validity, i.e. it may become valid or invalid after the Statistical Classification has been released. 
#'
#'                              \code{myDDI4_ClassificationItem$isValid <- mylogical} sets the isValid property of 'myDDI4_ClassificationItem' to the vector of datatype:logical
#'
#'                              \code{myDDI4_ClassificationItem$isValid} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{isGenerated}}{ A read/write active binding.
#'        Indicates whether or not the item has been generated to make the level to which it belongs complete
#'
#'                              \code{myDDI4_ClassificationItem$isGenerated <- mylogical} sets the isGenerated property of 'myDDI4_ClassificationItem' to the vector of datatype:logical
#'
#'                              \code{myDDI4_ClassificationItem$isGenerated} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{explanatoryNotes}}{ A read/write active binding.
#'        A Classification Item may be associated with explanatory notes, which further describe and clarify the contents of the Category. Explanatory notes consist of: General note: Contains either additional information about the Category, or a general description of the Category, which is not structured according to the "includes", "includes also", "excludes" pattern. Includes: Specifies the contents of the Category. Includes also: A list of borderline cases, which belong to the described Category. Excludes: A list of borderline cases, which do not belong to the described Category. Excluded cases may contain a reference to the Classification Items to which the excluded cases belong.
#'
#'                              \code{myDDI4_ClassificationItem$explanatoryNotes <- list(myInternationalStructuredString)} sets the 'explanatoryNotes' property of 'myDDI4_ClassificationItem' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_ClassificationItem$explanatoryNotes} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{futureNotes}}{ A read/write active binding.
#'        The future events describe a change (or a number of changes) related to an invalid item. These changes may e.g. have turned the now invalid item into one or several successor items. This allows the possibility to follow successors of the item in the future.
#'
#'                              \code{myDDI4_ClassificationItem$futureNotes <- list(myInternationalString)} sets the 'futureNotes' property of 'myDDI4_ClassificationItem' to a list of R6 InternationalString class instances
#'
#'                              \code{myDDI4_ClassificationItem$futureNotes} returns a list of of R6 : InternationalString class instances}
#'   }
#'   \describe{
#'     \item{\code{changeLog}}{ A read/write active binding.
#'        Describes the changes, which the item has been subject to during the life time of the actual Statistical Classification.
#'
#'                              \code{myDDI4_ClassificationItem$changeLog <- list(myInternationalString)} sets the 'changeLog' property of 'myDDI4_ClassificationItem' to a list of R6 InternationalString class instances
#'
#'                              \code{myDDI4_ClassificationItem$changeLog} returns a list of of R6 : InternationalString class instances}
#'   }
#'   \describe{
#'     \item{\code{changeFromPreviousVersion}}{ A read/write active binding.
#'        Describes the changes, which the item has been subject to from the previous version to the actual Statistical Classification
#'
#'                              \code{myDDI4_ClassificationItem$changeFromPreviousVersion <- list(myInternationalString)} sets the 'changeFromPreviousVersion' property of 'myDDI4_ClassificationItem' to a list of R6 InternationalString class instances
#'
#'                              \code{myDDI4_ClassificationItem$changeFromPreviousVersion} returns a list of of R6 : InternationalString class instances}
#'   }
#'   \describe{
#'     \item{\code{validDates}}{ A read/write active binding.
#'        Dates for which the classification is valid. Date from which the item became valid. The date must be defined if the item belongs to a floating Statistical classification. Date at which the item became invalid. The date must be defined if the item belongs to a floating Statistical classification and is no longer valid
#'
#'                              \code{myDDI4_ClassificationItem$validDates <- list(myDateRange)} sets the 'validDates' property of 'myDDI4_ClassificationItem' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_ClassificationItem$validDates} returns a list of of R6 : DateRange class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A Classification Item has a name as provided by the owner or maintenance unit. The name describes the content of the category. The name is unique within the Statistical Classification to which the item belongs, except for categories that are identical at more than one level in a hierarchical classification. Use the context attribute to identify Official Name or alternate names for the Classification Item.
#'
#'                              \code{myDDI4_ClassificationItem$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_ClassificationItem' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_ClassificationItem$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{representation}}{ A read/write active binding.
#'        The value as expressed in the data file. 
#'
#'                              \code{myDDI4_ClassificationItem$representation <- list(myValueString)} sets the 'representation' property of 'myDDI4_ClassificationItem' to a list of R6 ValueString class instances
#'
#'                              \code{myDDI4_ClassificationItem$representation} returns a list of of R6 : ValueString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{caseLaw}}{ A read/write active binding.
#'          Case law rulings related to the Classification Item.
#'
#'                              \code{myDDI4_ClassificationItem$caseLaw <- c(myAuthorizationSourceURN)} sets the association 'caseLaw' from 'myDDI4_ClassificationItem' to a character vector of DdiUrns for R6 AuthorizationSource class instances
#'
#'                              \code{myDDI4_ClassificationItem$caseLaw} returns a character vector of of DdiUrns for R6  AuthorizationSource class instances}
#'   }
#'   \describe{
#'     \item{\code{exclude}}{ A read/write active binding.
#'          Classification Items to which the excluded cases belong (as described in explanatoryNotes).
#'
#'                              \code{myDDI4_ClassificationItem$exclude <- c(myClassificationItemURN)} sets the association 'exclude' from 'myDDI4_ClassificationItem' to a character vector of DdiUrns for R6 ClassificationItem class instances
#'
#'                              \code{myDDI4_ClassificationItem$exclude} returns a character vector of of DdiUrns for R6  ClassificationItem class instances}
#'   }
#'   \describe{
#'     \item{\code{denotes}}{ A read/write active binding.
#'          A definition for the code. Specialization of denotes for Categories.
#'
#'                              \code{myDDI4_ClassificationItem$denotes <- c(myCategoryURN)} sets the association 'denotes' from 'myDDI4_ClassificationItem' to a character vector of DdiUrns for R6 Category class instances
#'
#'                              \code{myDDI4_ClassificationItem$denotes} returns a character vector of of DdiUrns for R6  Category class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ClassificationItem$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, isValid=NA, isGenerated=NA, explanatoryNotes=NA, futureNotes=NA, changeLog=NA, changeFromPreviousVersion=NA, validDates=NA, name=NA, representation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ClassificationItem <- R6Class(classname="DDI4_ClassificationItem",
  inherit = DDI4_Designation,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, isValid=NA, isGenerated=NA, explanatoryNotes=NA, futureNotes=NA, changeLog=NA, changeFromPreviousVersion=NA, validDates=NA, name=NA, representation=NA,    caseLaw=NA,exclude=NA,denotes=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ClassificationItem may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(isValid)){
        private$.isValid <- returnVectorIfValid(validVectorClass="logical", proposedVector=isValid , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isGenerated)){
        private$.isGenerated <- returnVectorIfValid(validVectorClass="logical", proposedVector=isGenerated , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(explanatoryNotes)){
        private$.explanatoryNotes <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=explanatoryNotes , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(futureNotes)){
        private$.futureNotes <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=futureNotes , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(changeLog)){
        private$.changeLog <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=changeLog , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(changeFromPreviousVersion)){
        private$.changeFromPreviousVersion <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=changeFromPreviousVersion , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(validDates)){
        private$.validDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=validDates , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(representation)){
        private$.representation <- returnR6ObjectIfValid(validClass="DDI4_ValueString", proposedObjectList=representation , minimumCardinality=1, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(caseLaw)){
        private$.caseLaw <- returnUrnVectorIfValid(validClass="DDI4_AuthorizationSource", proposedUrnVector=caseLaw , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.caseLaw, referencingAssociationName="caseLaw",  referencingUrn=thisUrn)  
      }
      if(!missing(exclude)){
        private$.exclude <- returnUrnVectorIfValid(validClass="DDI4_ClassificationItem", proposedUrnVector=exclude , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.exclude, referencingAssociationName="exclude",  referencingUrn=thisUrn)  
      }
      if(!missing(denotes)){
        private$.denotes <- returnUrnVectorIfValid(validClass="DDI4_Category", proposedUrnVector=denotes , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.denotes, referencingAssociationName="denotes",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 9 properties: 
    changeFromPreviousVersion = function(val){
      if (!missing(val)){
        private$.changeFromPreviousVersion <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.changeFromPreviousVersion
      }
    },

    changeLog = function(val){
      if (!missing(val)){
        private$.changeLog <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.changeLog
      }
    },

    explanatoryNotes = function(val){
      if (!missing(val)){
        private$.explanatoryNotes <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.explanatoryNotes
      }
    },

    futureNotes = function(val){
      if (!missing(val)){
        private$.futureNotes <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.futureNotes
      }
    },

    isGenerated = function(val){
      if (!missing(val)){
        private$.isGenerated <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isGenerated
      }
    },

    isValid = function(val){
      if (!missing(val)){
        private$.isValid <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isValid
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    representation = function(val){
      if (!missing(val)){
        private$.representation <- returnR6ObjectIfValid(validClass="DDI4_ValueString", proposedObjectList=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.representation
      }
    },

    validDates = function(val){
      if (!missing(val)){
        private$.validDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.validDates
      }
    },


# There are 3 class association(s):
    caseLaw = function(val){
      if (!missing(val)){
        private$.caseLaw <- returnUrnVectorIfValid(validClass="DDI4_AuthorizationSource", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.caseLaw, referencingAssociationName="caseLaw",  referencingUrn=thisUrn)  
       # invisible( private$.caseLaw)
      } else {
          private$.caseLaw
      }
    },

    denotes = function(val){
      if (!missing(val)){
        private$.denotes <- returnUrnVectorIfValid(validClass="DDI4_Category", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.denotes, referencingAssociationName="denotes",  referencingUrn=thisUrn)  
       # invisible( private$.denotes)
      } else {
          private$.denotes
      }
    },

    exclude = function(val){
      if (!missing(val)){
        private$.exclude <- returnUrnVectorIfValid(validClass="DDI4_ClassificationItem", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.exclude, referencingAssociationName="exclude",  referencingUrn=thisUrn)  
       # invisible( private$.exclude)
      } else {
          private$.exclude
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .isValid = NULL,
    .isGenerated = NULL,
    .explanatoryNotes = NULL,
    .futureNotes = NULL,
    .changeLog = NULL,
    .changeFromPreviousVersion = NULL,
    .validDates = NULL,
    .name = NULL,
    .representation = NULL,
    .caseLaw = NULL,
    .exclude = NULL,
    .denotes = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        caseLaw = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AuthorizationSource", 
          structure = "DdiUrn vector"
        ),
        changeFromPreviousVersion = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        changeLog = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        denotes = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Category", 
          structure = "DdiUrn vector"
        ),
        exclude = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ClassificationItem", 
          structure = "DdiUrn vector"
        ),
        explanatoryNotes = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        futureNotes = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isGenerated = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isValid = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        representation = c(
          min="1", 
          max="1", 
          datatype = "DDI4_ValueString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        validDates = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ClassificationItem


#' @title DDI4_ClassificationItemIndicator  A ClassificationItemIndicator realizes and extends a MemberIndicator which provides a Classification Item with an index indicating order and a level reference providing the level location of the Classification Item within a hierarchical structure
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ClassificationItemIndicator}{ClassificationItemIndicator} 

#'
#'   Definition
#'
#'   ============
#'
#'   A ClassificationItemIndicator realizes and extends a MemberIndicator which provides a Classification Item with an index indicating order and a level reference providing the level location of the Classification Item within a hierarchical structure.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ClassificationItemIndicator$DdiUrn} returns the DDI URN of myDDI4_ClassificationItemIndicator.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Provides an index for the member within an ordered array
#'
#'                              \code{myDDI4_ClassificationItemIndicator$index <- myinteger} sets the index property of 'myDDI4_ClassificationItemIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_ClassificationItemIndicator$index} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{hasLevel}}{ A read/write active binding.
#'        Indicates the level within which the CodeItem resides
#'
#'                              \code{myDDI4_ClassificationItemIndicator$hasLevel <- myinteger} sets the hasLevel property of 'myDDI4_ClassificationItemIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_ClassificationItemIndicator$hasLevel} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{member}}{ A read/write active binding.
#'          The member containing the designation of the Classification Item
#'
#'                              \code{myDDI4_ClassificationItemIndicator$member <- c(myClassificationItemURN)} sets the association 'member' from 'myDDI4_ClassificationItemIndicator' to a character vector of DdiUrns for R6 ClassificationItem class instances
#'
#'                              \code{myDDI4_ClassificationItemIndicator$member} returns a character vector of of DdiUrns for R6  ClassificationItem class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ClassificationItemIndicator$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, hasLevel=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ClassificationItemIndicator <- R6Class(classname="DDI4_ClassificationItemIndicator",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA, hasLevel=NA,    member=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ClassificationItemIndicator may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasLevel)){
        private$.hasLevel <- returnVectorIfValid(validVectorClass="integer", proposedVector=hasLevel , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(member)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_ClassificationItem", proposedUrnVector=member , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 2 properties: 
    hasLevel = function(val){
      if (!missing(val)){
        private$.hasLevel <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasLevel
      }
    },

    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.index
      }
    },


# There are 1 class association(s):
    member = function(val){
      if (!missing(val)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_ClassificationItem", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
       # invisible( private$.member)
      } else {
          private$.member
      }
    }

  ),

  private = list(
    .index = NULL,
    .hasLevel = NULL,
    .member = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasLevel = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        index = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        member = c(
          min="1", 
          max="1", 
          datatype = "DDI4_ClassificationItem", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ClassificationItemIndicator


#' @title DDI4_ClassificationItemRelation  Source target relationship between classification items in a classification item relation structure
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ClassificationItemRelation}{ClassificationItemRelation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Source target relationship between classification items in a classification item relation structure
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ClassificationItemRelation$DdiUrn} returns the DDI URN of myDDI4_ClassificationItemRelation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection
#'
#'                              \code{myDDI4_ClassificationItemRelation$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_ClassificationItemRelation' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_ClassificationItemRelation$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an ennumeration list
#'
#'                              \code{myDDI4_ClassificationItemRelation$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_ClassificationItemRelation' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_ClassificationItemRelation$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_ClassificationItemRelation$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_ClassificationItemRelation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ClassificationItemRelation$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          First member in a relationship. 
#'
#'                              \code{myDDI4_ClassificationItemRelation$source <- c(myClassificationItemURN)} sets the association 'source' from 'myDDI4_ClassificationItemRelation' to a character vector of DdiUrns for R6 ClassificationItem class instances
#'
#'                              \code{myDDI4_ClassificationItemRelation$source} returns a character vector of of DdiUrns for R6  ClassificationItem class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          Second member in a relationship. Note that this can be realized as a collection to support tuples. 
#'
#'                              \code{myDDI4_ClassificationItemRelation$target <- c(myClassificationItemURN)} sets the association 'target' from 'myDDI4_ClassificationItemRelation' to a character vector of DdiUrns for R6 ClassificationItem class instances
#'
#'                              \code{myDDI4_ClassificationItemRelation$target} returns a character vector of of DdiUrns for R6  ClassificationItem class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ClassificationItemRelation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, totality=NA, hasRelationSpecification=NA, semantic=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ClassificationItemRelation <- R6Class(classname="DDI4_ClassificationItemRelation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, totality=NA, hasRelationSpecification=NA, semantic=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ClassificationItemRelation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_ClassificationItem", proposedUrnVector=source , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_ClassificationItem", proposedUrnVector=target , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_ClassificationItem", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_ClassificationItem", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .totality = NULL,
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="1", 
          max="1", 
          datatype = "DDI4_ClassificationItem", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ClassificationItem", 
          structure = "DdiUrn vector"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ClassificationItemRelation


#' @title DDI4_ClassificationItemRelationStructure  A complex RelationStructure for use with statistical classification
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ClassificationItemRelationStructure}{ClassificationItemRelationStructure} 

#'
#'   Definition
#'
#'   ============
#'
#'   A complex RelationStructure for use with statistical classification
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A ClassificationRelationStructure for ISCO-08 would describe each of the major classifications as a parent of its sub-classifications.   1 Managers, for example would be listed as a parent of four sub groups: 11 Chief Executives, Senior Officials and Legislators;  12 Chief Executives, Senior Officials and Legislators;  13 Production and Specialized Services Managers; and  14 Hospitality, Retail and Other Services Managers. 
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   The ClassificationItemRelationStructure has a set of ClassificationItemRelations which are basically adjacency lists. A source ClassificationItem has a described relationship to a target list of ClassificationItem. The semantic might be, for example, "parentOf", or "contains", etc.. 
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ClassificationItemRelationStructure$DdiUrn} returns the DDI URN of myDDI4_ClassificationItemRelationStructure.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an ennumeration list.  Use if all relations within this relation structure are of the same specification.
#'
#'                              \code{myDDI4_ClassificationItemRelationStructure$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_ClassificationItemRelationStructure' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_ClassificationItemRelationStructure$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_ClassificationItemRelationStructure$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_ClassificationItemRelationStructure' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ClassificationItemRelationStructure$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_ClassificationItemRelationStructure$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_ClassificationItemRelationStructure' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_ClassificationItemRelationStructure$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A display label for the OrderRelation. May be expressed in multiple languages. Repeat for labels with different content, for example, labels with differing length limitations.
#'
#'                              \code{myDDI4_ClassificationItemRelationStructure$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_ClassificationItemRelationStructure' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_ClassificationItemRelationStructure$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMemberRelation}}{ A read/write active binding.
#'        Member relations that comprise the relation structure
#'
#'                              \code{myDDI4_ClassificationItemRelationStructure$hasMemberRelation <- list(myClassificationItemRelation)} sets the 'hasMemberRelation' property of 'myDDI4_ClassificationItemRelationStructure' to a list of R6 ClassificationItemRelation class instances
#'
#'                              \code{myDDI4_ClassificationItemRelationStructure$hasMemberRelation} returns a list of of R6 : ClassificationItemRelation class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ClassificationItemRelationStructure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, displayLabel=NA, hasMemberRelation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ClassificationItemRelationStructure <- R6Class(classname="DDI4_ClassificationItemRelationStructure",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, displayLabel=NA, hasMemberRelation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ClassificationItemRelationStructure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(hasMemberRelation)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_ClassificationItemRelation", proposedObjectList=hasMemberRelation , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 5 properties: 
    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },

    hasMemberRelation = function(val){
      if (!missing(val)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_ClassificationItemRelation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasMemberRelation
      }
    },

    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .displayLabel = NULL,
    .hasMemberRelation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasMemberRelation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ClassificationItemRelation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ClassificationItemRelationStructure


#' @title DDI4_ClassificationSeries  A Classification Series is an ensemble of one or more Statistical Classifications, based on the same concept, as documented by the definingConcept relationship, and related to each other as versions or updates
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ClassificationSeries}{ClassificationSeries} 

#'
#'   Definition
#'
#'   ============
#'
#'   A Classification Series is an ensemble of one or more Statistical Classifications, based on the same concept, as documented by the definingConcept relationship, and related to each other as versions or updates. Typically, these Statistical Classifications have the same name.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   ISIC or ISCO
#'
#'   SIC (with different published versions related to the publication year)
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Classification Series
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ClassificationSeries$DdiUrn} returns the DDI URN of myDDI4_ClassificationSeries.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{context}}{ A read/write active binding.
#'        ClassificationSeries can be designed in a specific context. Supports the use of and External Controlled Vocabulary.
#'
#'                              \code{myDDI4_ClassificationSeries$context <- list(myExternalControlledVocabularyEntry)} sets the 'context' property of 'myDDI4_ClassificationSeries' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ClassificationSeries$context} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{objectsOrUnitsClassified}}{ A read/write active binding.
#'        A ClassificationSeries is designed to classify a specific type of object/unit according to a specific attribute.
#'
#'                              \code{myDDI4_ClassificationSeries$objectsOrUnitsClassified <- list(myExternalControlledVocabularyEntry)} sets the 'objectsOrUnitsClassified' property of 'myDDI4_ClassificationSeries' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ClassificationSeries$objectsOrUnitsClassified} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{subject}}{ A read/write active binding.
#'        Areas of statistics in which the ClassificationSeries is implemented.
#'
#'                              \code{myDDI4_ClassificationSeries$subject <- list(myExternalControlledVocabularyEntry)} sets the 'subject' property of 'myDDI4_ClassificationSeries' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ClassificationSeries$subject} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{keyword}}{ A read/write active binding.
#'        A ClassificationSeries can be associated with one or a number of keywords. 
#'
#'                              \code{myDDI4_ClassificationSeries$keyword <- list(myExternalControlledVocabularyEntry)} sets the 'keyword' property of 'myDDI4_ClassificationSeries' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ClassificationSeries$keyword} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{type}}{ A read/write active binding.
#'        Whether the collection is a bag or a set: a bag is a collection with duplicates allowed, a set is a collection without duplicates.
#'
#'                              \code{myDDI4_ClassificationSeries$type <- list(myCollectionType)} sets the 'type' property of 'myDDI4_ClassificationSeries' to a list of R6 CollectionType class instances
#'
#'                              \code{myDDI4_ClassificationSeries$type} returns a list of of R6 : CollectionType class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage.
#'
#'                              \code{myDDI4_ClassificationSeries$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_ClassificationSeries' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_ClassificationSeries$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_ClassificationSeries$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_ClassificationSeries' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_ClassificationSeries$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        MemberIndicator Allows for the identification of the member and optionally provides an index for the member within an ordered array
#'
#'                              \code{myDDI4_ClassificationSeries$contains <- list(myStatisticalClassificationIndicator)} sets the 'contains' property of 'myDDI4_ClassificationSeries' to a list of R6 StatisticalClassificationIndicator class instances
#'
#'                              \code{myDDI4_ClassificationSeries$contains} returns a list of of R6 : StatisticalClassificationIndicator class instances}
#'   }
#'   \describe{
#'     \item{\code{isOrdered}}{ A read/write active binding.
#'        If members are ordered set to true, if unordered set to false.
#'
#'                              \code{myDDI4_ClassificationSeries$isOrdered <- mylogical} sets the isOrdered property of 'myDDI4_ClassificationSeries' to the vector of datatype:logical
#'
#'                              \code{myDDI4_ClassificationSeries$isOrdered} returns a vector of datatype: logical}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isStructuredBy}}{ A read/write active binding.
#'          Description of a complex structure for the Collection. A collection may be structured in more than one way for different uses.
#'
#'                              \code{myDDI4_ClassificationSeries$isStructuredBy <- c(myStatisticalClassificationRelationStructureURN)} sets the association 'isStructuredBy' from 'myDDI4_ClassificationSeries' to a character vector of DdiUrns for R6 StatisticalClassificationRelationStructure class instances
#'
#'                              \code{myDDI4_ClassificationSeries$isStructuredBy} returns a character vector of of DdiUrns for R6  StatisticalClassificationRelationStructure class instances}
#'   }
#'   \describe{
#'     \item{\code{owner}}{ A read/write active binding.
#'          The statistical office or other authority, which created and maintains the StatisticalClassification (s) related to the ClassificationSeries. A ClassificationSeries may have several owners.
#'
#'                              \code{myDDI4_ClassificationSeries$owner <- c(myAgentURN)} sets the association 'owner' from 'myDDI4_ClassificationSeries' to a character vector of DdiUrns for R6 Agent class instances
#'
#'                              \code{myDDI4_ClassificationSeries$owner} returns a character vector of of DdiUrns for R6  Agent class instances}
#'   }
#'   \describe{
#'     \item{\code{definingConcept}}{ A read/write active binding.
#'          The conceptual basis for the collection of members.
#'
#'                              \code{myDDI4_ClassificationSeries$definingConcept <- c(myConceptURN)} sets the association 'definingConcept' from 'myDDI4_ClassificationSeries' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_ClassificationSeries$definingConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ClassificationSeries$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, context=NA, objectsOrUnitsClassified=NA, subject=NA, keyword=NA, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ClassificationSeries <- R6Class(classname="DDI4_ClassificationSeries",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, context=NA, objectsOrUnitsClassified=NA, subject=NA, keyword=NA, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA,    isStructuredBy=NA,owner=NA,definingConcept=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ClassificationSeries may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(context)){
        private$.context <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=context , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(objectsOrUnitsClassified)){
        private$.objectsOrUnitsClassified <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=objectsOrUnitsClassified , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(subject)){
        private$.subject <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=subject , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(keyword)){
        private$.keyword <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=keyword , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(type)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=type , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_StatisticalClassificationIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(isOrdered)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=isOrdered , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(isStructuredBy)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_StatisticalClassificationRelationStructure", proposedUrnVector=isStructuredBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
      }
      if(!missing(owner)){
        private$.owner <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=owner , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.owner, referencingAssociationName="owner",  referencingUrn=thisUrn)  
      }
      if(!missing(definingConcept)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=definingConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 9 properties: 
    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_StatisticalClassificationIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },

    context = function(val){
      if (!missing(val)){
        private$.context <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.context
      }
    },

    isOrdered = function(val){
      if (!missing(val)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isOrdered
      }
    },

    keyword = function(val){
      if (!missing(val)){
        private$.keyword <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.keyword
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    objectsOrUnitsClassified = function(val){
      if (!missing(val)){
        private$.objectsOrUnitsClassified <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.objectsOrUnitsClassified
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    subject = function(val){
      if (!missing(val)){
        private$.subject <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.subject
      }
    },

    type = function(val){
      if (!missing(val)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.type
      }
    },


# There are 3 class association(s):
    definingConcept = function(val){
      if (!missing(val)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
       # invisible( private$.definingConcept)
      } else {
          private$.definingConcept
      }
    },

    isStructuredBy = function(val){
      if (!missing(val)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_StatisticalClassificationRelationStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
       # invisible( private$.isStructuredBy)
      } else {
          private$.isStructuredBy
      }
    },

    owner = function(val){
      if (!missing(val)){
        private$.owner <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.owner, referencingAssociationName="owner",  referencingUrn=thisUrn)  
       # invisible( private$.owner)
      } else {
          private$.owner
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .context = NULL,
    .objectsOrUnitsClassified = NULL,
    .subject = NULL,
    .keyword = NULL,
    .type = NULL,
    .name = NULL,
    .purpose = NULL,
    .contains = NULL,
    .isOrdered = NULL,
    .isStructuredBy = NULL,
    .owner = NULL,
    .definingConcept = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_StatisticalClassificationIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        context = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isStructuredBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_StatisticalClassificationRelationStructure", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        keyword = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        objectsOrUnitsClassified = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        owner = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Agent", 
          structure = "DdiUrn vector"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        subject = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ClassificationSeries


#' @title DDI4_ClassificationSeriesIndicator  Member Indicator for use with member type ClassificationSeries
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ClassificationSeriesIndicator}{ClassificationSeriesIndicator} 

#'
#'   Definition
#'
#'   ============
#'
#'   Member Indicator for use with member type ClassificationSeries
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ClassificationSeriesIndicator$DdiUrn} returns the DDI URN of myDDI4_ClassificationSeriesIndicator.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Index value of member in an ordered array
#'
#'                              \code{myDDI4_ClassificationSeriesIndicator$index <- myinteger} sets the index property of 'myDDI4_ClassificationSeriesIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_ClassificationSeriesIndicator$index} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{member}}{ A read/write active binding.
#'          Restricts member target class to ClassificationSeries
#'
#'                              \code{myDDI4_ClassificationSeriesIndicator$member <- c(myClassificationSeriesURN)} sets the association 'member' from 'myDDI4_ClassificationSeriesIndicator' to a character vector of DdiUrns for R6 ClassificationSeries class instances
#'
#'                              \code{myDDI4_ClassificationSeriesIndicator$member} returns a character vector of of DdiUrns for R6  ClassificationSeries class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ClassificationSeriesIndicator$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ClassificationSeriesIndicator <- R6Class(classname="DDI4_ClassificationSeriesIndicator",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA,    member=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ClassificationSeriesIndicator may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(member)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_ClassificationSeries", proposedUrnVector=member , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.index
      }
    },


# There are 1 class association(s):
    member = function(val){
      if (!missing(val)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_ClassificationSeries", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
       # invisible( private$.member)
      } else {
          private$.member
      }
    }

  ),

  private = list(
    .index = NULL,
    .member = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        index = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        member = c(
          min="1", 
          max="1", 
          datatype = "DDI4_ClassificationSeries", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ClassificationSeriesIndicator


#' @title DDI4_ClassificationSeriesRelation  Defines the Classification Series in a complex relation and specifies the relationshi
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ClassificationSeriesRelation}{ClassificationSeriesRelation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Defines the Classification Series in a complex relation and specifies the relationshi
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ClassificationSeriesRelation$DdiUrn} returns the DDI URN of myDDI4_ClassificationSeriesRelation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an enumeration list
#'
#'                              \code{myDDI4_ClassificationSeriesRelation$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_ClassificationSeriesRelation' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_ClassificationSeriesRelation$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_ClassificationSeriesRelation$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_ClassificationSeriesRelation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ClassificationSeriesRelation$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_ClassificationSeriesRelation$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_ClassificationSeriesRelation' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_ClassificationSeriesRelation$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          First member in the relationship
#'
#'                              \code{myDDI4_ClassificationSeriesRelation$source <- c(myClassificationSeriesURN)} sets the association 'source' from 'myDDI4_ClassificationSeriesRelation' to a character vector of DdiUrns for R6 ClassificationSeries class instances
#'
#'                              \code{myDDI4_ClassificationSeriesRelation$source} returns a character vector of of DdiUrns for R6  ClassificationSeries class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          Second member in the relationship
#'
#'                              \code{myDDI4_ClassificationSeriesRelation$target <- c(myClassificationSeriesURN)} sets the association 'target' from 'myDDI4_ClassificationSeriesRelation' to a character vector of DdiUrns for R6 ClassificationSeries class instances
#'
#'                              \code{myDDI4_ClassificationSeriesRelation$target} returns a character vector of of DdiUrns for R6  ClassificationSeries class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ClassificationSeriesRelation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ClassificationSeriesRelation <- R6Class(classname="DDI4_ClassificationSeriesRelation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ClassificationSeriesRelation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_ClassificationSeries", proposedUrnVector=source , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_ClassificationSeries", proposedUrnVector=target , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_ClassificationSeries", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_ClassificationSeries", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ClassificationSeries", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ClassificationSeries", 
          structure = "DdiUrn vector"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ClassificationSeriesRelation


#' @title DDI4_ClassificationSeriesRelationStructure  Describes the complex relation structure of a classification family
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ClassificationSeriesRelationStructure}{ClassificationSeriesRelationStructure} 

#'
#'   Definition
#'
#'   ============
#'
#'   Describes the complex relation structure of a classification family.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ClassificationSeriesRelationStructure$DdiUrn} returns the DDI URN of myDDI4_ClassificationSeriesRelationStructure.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an enumeration list.  Use if all relations within this relation structure are of the same specification.
#'
#'                              \code{myDDI4_ClassificationSeriesRelationStructure$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_ClassificationSeriesRelationStructure' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_ClassificationSeriesRelationStructure$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_ClassificationSeriesRelationStructure$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_ClassificationSeriesRelationStructure' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ClassificationSeriesRelationStructure$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_ClassificationSeriesRelationStructure$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_ClassificationSeriesRelationStructure' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_ClassificationSeriesRelationStructure$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMemberRelation}}{ A read/write active binding.
#'        Member relations that comprise the relation structure
#'
#'                              \code{myDDI4_ClassificationSeriesRelationStructure$hasMemberRelation <- list(myClassificationSeriesRelation)} sets the 'hasMemberRelation' property of 'myDDI4_ClassificationSeriesRelationStructure' to a list of R6 ClassificationSeriesRelation class instances
#'
#'                              \code{myDDI4_ClassificationSeriesRelationStructure$hasMemberRelation} returns a list of of R6 : ClassificationSeriesRelation class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ClassificationSeriesRelationStructure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ClassificationSeriesRelationStructure <- R6Class(classname="DDI4_ClassificationSeriesRelationStructure",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ClassificationSeriesRelationStructure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasMemberRelation)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_ClassificationSeriesRelation", proposedObjectList=hasMemberRelation , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    hasMemberRelation = function(val){
      if (!missing(val)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_ClassificationSeriesRelation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasMemberRelation
      }
    },

    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .hasMemberRelation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasMemberRelation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ClassificationSeriesRelation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasRelationSpecification = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ClassificationSeriesRelationStructure


#' @title DDI4_Code  A type of Designation that relates a representation expressed as a string with or without meaningful white space to a specific classification
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Code}{Code} 

#'
#'   Definition
#'
#'   ============
#'
#'   A type of Designation that relates a representation expressed as a string with or without meaningful white space to a specific classification. For use in a Code List. The representation property (Value) is expressed as it would be found in a data file. Multiple representations may relate to the same Category but should be expressed as separate Codes.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   The letter M might stand for the category Male in the CodeList called Gender.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   A Code is a kind of Designation, in particular it is the assignment of an alphanumeric string to stand in place of a category. 
#'
#'   
#'
#'   It should not be confused with a ClassificationItem which is a placeholder for a position in a StatisticalClassification. It contains a Designation, for which Code is a kind; a Category; and other things. For example, in 2012 NAICS (North American Industry Classification System) one Classification Item has the Code 23 and the Category construction.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   l:CodeType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Code
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Designation}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Code$DdiUrn} returns the DDI URN of myDDI4_Code.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{representation}}{ A read/write active binding.
#'        The value as expressed in the data file. 
#'
#'                              \code{myDDI4_Code$representation <- list(myValueString)} sets the 'representation' property of 'myDDI4_Code' to a list of R6 ValueString class instances
#'
#'                              \code{myDDI4_Code$representation} returns a list of of R6 : ValueString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{denotes}}{ A read/write active binding.
#'          A definition for the code. Specialization of denotes for Categories.
#'
#'                              \code{myDDI4_Code$denotes <- c(myCategoryURN)} sets the association 'denotes' from 'myDDI4_Code' to a character vector of DdiUrns for R6 Category class instances
#'
#'                              \code{myDDI4_Code$denotes} returns a character vector of of DdiUrns for R6  Category class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Code$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, representation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Code <- R6Class(classname="DDI4_Code",
  inherit = DDI4_Designation,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, representation=NA,    denotes=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Code may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(representation)){
        private$.representation <- returnR6ObjectIfValid(validClass="DDI4_ValueString", proposedObjectList=representation , minimumCardinality=1, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(denotes)){
        private$.denotes <- returnUrnVectorIfValid(validClass="DDI4_Category", proposedUrnVector=denotes , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.denotes, referencingAssociationName="denotes",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    representation = function(val){
      if (!missing(val)){
        private$.representation <- returnR6ObjectIfValid(validClass="DDI4_ValueString", proposedObjectList=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.representation
      }
    },


# There are 1 class association(s):
    denotes = function(val){
      if (!missing(val)){
        private$.denotes <- returnUrnVectorIfValid(validClass="DDI4_Category", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.denotes, referencingAssociationName="denotes",  referencingUrn=thisUrn)  
       # invisible( private$.denotes)
      } else {
          private$.denotes
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .representation = NULL,
    .denotes = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        denotes = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Category", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        representation = c(
          min="1", 
          max="1", 
          datatype = "DDI4_ValueString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Code


#' @title DDI4_CodeIndicator  A CodeIndicator realizes and extends a MemberIndicator which provides a Code with an index indicating order and a level reference providing the level location of the Code within a hierarchical structure
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CodeIndicator}{CodeIndicator} 

#'
#'   Definition
#'
#'   ============
#'
#'   A CodeIndicator realizes and extends a MemberIndicator which provides a Code with an index indicating order and a level reference providing the level location of the Code within a hierarchical structure.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_CodeIndicator$DdiUrn} returns the DDI URN of myDDI4_CodeIndicator.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Provides an index for the member within an ordered array
#'
#'                              \code{myDDI4_CodeIndicator$index <- myinteger} sets the index property of 'myDDI4_CodeIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_CodeIndicator$index} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{isInLevel}}{ A read/write active binding.
#'        Indicates the level within which the CodeItem resides
#'
#'                              \code{myDDI4_CodeIndicator$isInLevel <- myinteger} sets the isInLevel property of 'myDDI4_CodeIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_CodeIndicator$isInLevel} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{member}}{ A read/write active binding.
#'          The member containing the designation of the CodeItem
#'
#'                              \code{myDDI4_CodeIndicator$member <- c(myCodeURN)} sets the association 'member' from 'myDDI4_CodeIndicator' to a character vector of DdiUrns for R6 Code class instances
#'
#'                              \code{myDDI4_CodeIndicator$member} returns a character vector of of DdiUrns for R6  Code class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CodeIndicator$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, isInLevel=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CodeIndicator <- R6Class(classname="DDI4_CodeIndicator",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA, isInLevel=NA,    member=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CodeIndicator may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isInLevel)){
        private$.isInLevel <- returnVectorIfValid(validVectorClass="integer", proposedVector=isInLevel , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(member)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_Code", proposedUrnVector=member , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 2 properties: 
    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.index
      }
    },

    isInLevel = function(val){
      if (!missing(val)){
        private$.isInLevel <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isInLevel
      }
    },


# There are 1 class association(s):
    member = function(val){
      if (!missing(val)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_Code", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
       # invisible( private$.member)
      } else {
          private$.member
      }
    }

  ),

  private = list(
    .index = NULL,
    .isInLevel = NULL,
    .member = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        index = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        isInLevel = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        member = c(
          min="1", 
          max="1", 
          datatype = "DDI4_Code", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_CodeIndicator


#' @title DDI4_CodeList  A list of Codes and associated Categories
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CodeList}{CodeList} 

#'
#'   Definition
#'
#'   ============
#'
#'   A list of Codes and associated Categories. May be flat or hierarchical. A hierarchical structure may have an indexed order for intended presentation even though the content within levels of the hierarchy are conceptually unordered. For hierarchical structures ClassificationRelationStructure is used to provide additional information on the structure and organization of the categories. Note that a CategorySet can be structured by a ClassificationRelationStructure without the need for associating any Codes with the Categories. This allows for the creation of a CategorySet, for example for a response domain, without an associated CodeList.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   The codes "M" and "F" could point to "Male" and "Female" categories respectively.
#'
#'   
#'
#'   A CodeList for an occupational classification system like ISCO-08 could use a ClassificationRelationStructure to describe a hierarchy (Chief Executives and Administrative and Commercial Managers as subtypes of Managers)
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   l:CodeListType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Code List
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_EnumerationDomain}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_CodeList$DdiUrn} returns the DDI URN of myDDI4_CodeList.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        Allows for the identification of the member and optionally provides an index for the member within an ordered array
#'
#'                              \code{myDDI4_CodeList$contains <- list(myCodeIndicator)} sets the 'contains' property of 'myDDI4_CodeList' to a list of R6 CodeIndicator class instances
#'
#'                              \code{myDDI4_CodeList$contains} returns a list of of R6 : CodeIndicator class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isStructuredBy}}{ A read/write active binding.
#'          A complex structure, hierarchical, for a CodeList. Restricted to single structure.
#'
#'                              \code{myDDI4_CodeList$isStructuredBy <- c(myCodeRelationStructureURN)} sets the association 'isStructuredBy' from 'myDDI4_CodeList' to a character vector of DdiUrns for R6 CodeRelationStructure class instances
#'
#'                              \code{myDDI4_CodeList$isStructuredBy} returns a character vector of of DdiUrns for R6  CodeRelationStructure class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CodeList$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, contains=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CodeList <- R6Class(classname="DDI4_CodeList",
  inherit = DDI4_EnumerationDomain,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, contains=NA,    isStructuredBy=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CodeList may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_CodeIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(isStructuredBy)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_CodeRelationStructure", proposedUrnVector=isStructuredBy , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_CodeIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },


# There are 1 class association(s):
    isStructuredBy = function(val){
      if (!missing(val)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_CodeRelationStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
       # invisible( private$.isStructuredBy)
      } else {
          private$.isStructuredBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .contains = NULL,
    .isStructuredBy = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_CodeIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isStructuredBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CodeRelationStructure", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        references = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CategorySet", 
          structure = "DdiUrn vector"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        usesLevelStructure = c(
          min="0", 
          max="1", 
          datatype = "DDI4_LevelStructure", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_CodeList


#' @title DDI4_CodeListResponseDomain  A response domain capturing a coded response (where both codes and their related category value are displayed) for a question
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CodeListResponseDomain}{CodeListResponseDomain} 

#'
#'   Definition
#'
#'   ============
#'
#'   A response domain capturing a coded response (where both codes and their related category value are displayed) for a question. This response domain allows the single selection of one coded response.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   d:CodeDomainType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ResponseDomain}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{usesCodeList}}{ A read/write active binding.
#'          The CodeList which includes the response options to list.

#'
#'                              \code{myDDI4_CodeListResponseDomain$usesCodeList <- c(myCodeListURN)} sets the association 'usesCodeList' from 'myDDI4_CodeListResponseDomain' to a character vector of DdiUrns for R6 CodeList class instances
#'
#'                              \code{myDDI4_CodeListResponseDomain$usesCodeList} returns a character vector of of DdiUrns for R6  CodeList class instances}
#'   }
#'   \describe{
#'     \item{\code{specifyOtherResponseDomain}}{ A read/write active binding.
#'          Optionally allows specifying the attached secondary response domain used by a specific item within this CodeListResponseDomain. For example, attach a TextResponseDomain to the code for "Other" using the TextDomain label (Please specify) as a label for the TextDomain. The attached secondary response domain must also be added to the Represented Question or Measure.
#'
#'                              \code{myDDI4_CodeListResponseDomain$specifyOtherResponseDomain <- c(myResponseDomainURN)} sets the association 'specifyOtherResponseDomain' from 'myDDI4_CodeListResponseDomain' to a character vector of DdiUrns for R6 ResponseDomain class instances
#'
#'                              \code{myDDI4_CodeListResponseDomain$specifyOtherResponseDomain} returns a character vector of of DdiUrns for R6  ResponseDomain class instances}
#'   }
#'   \describe{
#'     \item{\code{specifyOtherCorrespondingCode}}{ A read/write active binding.
#'          Optionally allows specifying the Code to which a secondary response domain is attached. For example, link to code which signifies "Other" within this CodeList.
#'
#'                              \code{myDDI4_CodeListResponseDomain$specifyOtherCorrespondingCode <- c(myCodeURN)} sets the association 'specifyOtherCorrespondingCode' from 'myDDI4_CodeListResponseDomain' to a character vector of DdiUrns for R6 Code class instances
#'
#'                              \code{myDDI4_CodeListResponseDomain$specifyOtherCorrespondingCode} returns a character vector of of DdiUrns for R6  Code class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CodeListResponseDomain$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CodeListResponseDomain <- R6Class(classname="DDI4_CodeListResponseDomain",
  inherit = DDI4_ResponseDomain,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, usesCodeList=NA,specifyOtherResponseDomain=NA,specifyOtherCorrespondingCode=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CodeListResponseDomain may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)



       if(!missing(usesCodeList)){
        private$.usesCodeList <- returnUrnVectorIfValid(validClass="DDI4_CodeList", proposedUrnVector=usesCodeList , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesCodeList, referencingAssociationName="usesCodeList",  referencingUrn=thisUrn)  
      }
      if(!missing(specifyOtherResponseDomain)){
        private$.specifyOtherResponseDomain <- returnUrnVectorIfValid(validClass="DDI4_ResponseDomain", proposedUrnVector=specifyOtherResponseDomain , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.specifyOtherResponseDomain, referencingAssociationName="specifyOtherResponseDomain",  referencingUrn=thisUrn)  
      }
      if(!missing(specifyOtherCorrespondingCode)){
        private$.specifyOtherCorrespondingCode <- returnUrnVectorIfValid(validClass="DDI4_Code", proposedUrnVector=specifyOtherCorrespondingCode , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.specifyOtherCorrespondingCode, referencingAssociationName="specifyOtherCorrespondingCode",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 class association(s):
    specifyOtherCorrespondingCode = function(val){
      if (!missing(val)){
        private$.specifyOtherCorrespondingCode <- returnUrnVectorIfValid(validClass="DDI4_Code", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.specifyOtherCorrespondingCode, referencingAssociationName="specifyOtherCorrespondingCode",  referencingUrn=thisUrn)  
       # invisible( private$.specifyOtherCorrespondingCode)
      } else {
          private$.specifyOtherCorrespondingCode
      }
    },

    specifyOtherResponseDomain = function(val){
      if (!missing(val)){
        private$.specifyOtherResponseDomain <- returnUrnVectorIfValid(validClass="DDI4_ResponseDomain", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.specifyOtherResponseDomain, referencingAssociationName="specifyOtherResponseDomain",  referencingUrn=thisUrn)  
       # invisible( private$.specifyOtherResponseDomain)
      } else {
          private$.specifyOtherResponseDomain
      }
    },

    usesCodeList = function(val){
      if (!missing(val)){
        private$.usesCodeList <- returnUrnVectorIfValid(validClass="DDI4_CodeList", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesCodeList, referencingAssociationName="usesCodeList",  referencingUrn=thisUrn)  
       # invisible( private$.usesCodeList)
      } else {
          private$.usesCodeList
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .usesCodeList = NULL,
    .specifyOtherResponseDomain = NULL,
    .specifyOtherCorrespondingCode = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="1", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        intendedRepresentation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RepresentedVariable", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        specifyOtherCorrespondingCode = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Code", 
          structure = "DdiUrn vector"
        ),
        specifyOtherResponseDomain = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ResponseDomain", 
          structure = "DdiUrn vector"
        ),
        usesCodeList = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CodeList", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_CodeListResponseDomain


#' @title DDI4_CodeRelation  Source target relationship between codes in a code relation structure
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CodeRelation}{CodeRelation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Source target relationship between codes in a code relation structure
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_CodeRelation$DdiUrn} returns the DDI URN of myDDI4_CodeRelation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection
#'
#'                              \code{myDDI4_CodeRelation$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_CodeRelation' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_CodeRelation$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an ennumeration list
#'
#'                              \code{myDDI4_CodeRelation$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_CodeRelation' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_CodeRelation$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_CodeRelation$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_CodeRelation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_CodeRelation$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          First member in a relationship. 
#'
#'                              \code{myDDI4_CodeRelation$source <- c(myCodeURN)} sets the association 'source' from 'myDDI4_CodeRelation' to a character vector of DdiUrns for R6 Code class instances
#'
#'                              \code{myDDI4_CodeRelation$source} returns a character vector of of DdiUrns for R6  Code class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          Second member in a relationship. Note that this can be realized as a collection to support tuples. 
#'
#'                              \code{myDDI4_CodeRelation$target <- c(myCodeURN)} sets the association 'target' from 'myDDI4_CodeRelation' to a character vector of DdiUrns for R6 Code class instances
#'
#'                              \code{myDDI4_CodeRelation$target} returns a character vector of of DdiUrns for R6  Code class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CodeRelation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, totality=NA, hasRelationSpecification=NA, semantic=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CodeRelation <- R6Class(classname="DDI4_CodeRelation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, totality=NA, hasRelationSpecification=NA, semantic=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CodeRelation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_Code", proposedUrnVector=source , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_Code", proposedUrnVector=target , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_Code", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_Code", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .totality = NULL,
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="1", 
          max="1", 
          datatype = "DDI4_Code", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Code", 
          structure = "DdiUrn vector"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_CodeRelation


#' @title DDI4_CodeRelationStructure  Relation structure of codes within a codelist
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CodeRelationStructure}{CodeRelationStructure} 

#'
#'   Definition
#'
#'   ============
#'
#'   Relation structure of codes within a codelist. Allows for the specification of complex relationships among codes.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   The CodeRelationStructure employs a set of CodeRelations to describe the relationship among concepts. Each CodeRelation is a one to many description of connections between codes. Together they might commonly describe relationships as complex as a hierarchy.
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_CodeRelationStructure$DdiUrn} returns the DDI URN of myDDI4_CodeRelationStructure.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an ennumeration list. Use if all relations within this relation structure are of the same specification.
#'
#'                              \code{myDDI4_CodeRelationStructure$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_CodeRelationStructure' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_CodeRelationStructure$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_CodeRelationStructure$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_CodeRelationStructure' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_CodeRelationStructure$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection
#'
#'                              \code{myDDI4_CodeRelationStructure$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_CodeRelationStructure' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_CodeRelationStructure$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMemberRelation}}{ A read/write active binding.
#'        CodeRelations making up the relation structure.
#'
#'                              \code{myDDI4_CodeRelationStructure$hasMemberRelation <- list(myCodeRelation)} sets the 'hasMemberRelation' property of 'myDDI4_CodeRelationStructure' to a list of R6 CodeRelation class instances
#'
#'                              \code{myDDI4_CodeRelationStructure$hasMemberRelation} returns a list of of R6 : CodeRelation class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CodeRelationStructure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CodeRelationStructure <- R6Class(classname="DDI4_CodeRelationStructure",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CodeRelationStructure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(hasMemberRelation)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_CodeRelation", proposedObjectList=hasMemberRelation , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    hasMemberRelation = function(val){
      if (!missing(val)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_CodeRelation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasMemberRelation
      }
    },

    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .hasMemberRelation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasMemberRelation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_CodeRelation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        totality = c(
          min="1", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_CodeRelationStructure


#' @title DDI4_CollectionMember  Generic class representing members of a collection
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CollectionMember}{CollectionMember} 

#'
#'   Definition
#'
#'   ============
#'
#'   Generic class representing members of a collection. 
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Classification Item, Process Step.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Members have to belong to some Collection, except in the case of nested Collections where the top level Collection is a Member that doesn't belong to any Collection.
#'
#'   
#'
#'   Member is not extended, it is realized (see Collection Pattern documentation).
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Node
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CollectionMember$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CollectionMember <- R6Class(classname="DDI4_CollectionMember",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CollectionMember may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

    }
  ),

  active = list(
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_CollectionMember


#' @title DDI4_Command  Provides the following information on the command
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Command}{Command} 

#'
#'   Definition
#'
#'   ============
#'
#'   Provides the following information on the command. The content of the command and the programming language used.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:CommandType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Command$DdiUrn} returns the DDI URN of myDDI4_Command.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{programLanguage}}{ A read/write active binding.
#'        Designates the programming language used for the command. Supports the use of a controlled vocabulary.
#'
#'                              \code{myDDI4_Command$programLanguage <- list(myExternalControlledVocabularyEntry)} sets the 'programLanguage' property of 'myDDI4_Command' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_Command$programLanguage} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{commandContent}}{ A read/write active binding.
#'        Content of the command itself expressed in the language designated in Programming Language.
#'
#'                              \code{myDDI4_Command$commandContent <- mycharacter} sets the commandContent property of 'myDDI4_Command' to the vector of datatype:character
#'
#'                              \code{myDDI4_Command$commandContent} returns a vector of datatype: character}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Command$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, programLanguage=NA, commandContent=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Command <- R6Class(classname="DDI4_Command",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, programLanguage=NA, commandContent=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Command may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(programLanguage)){
        private$.programLanguage <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=programLanguage , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(commandContent)){
        private$.commandContent <- returnVectorIfValid(validVectorClass="character", proposedVector=commandContent , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    commandContent = function(val){
      if (!missing(val)){
        private$.commandContent <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.commandContent
      }
    },

    programLanguage = function(val){
      if (!missing(val)){
        private$.programLanguage <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.programLanguage
      }
    }


  ),

  private = list(
    .programLanguage = NULL,
    .commandContent = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        commandContent = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        programLanguage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Command


#' @title DDI4_CommandCode  Contains information on the command used for processing data
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CommandCode}{CommandCode} 

#'
#'   Definition
#'
#'   ============
#'
#'   Contains information on the command used for processing data. Contains a description of the command which should clarify for the user the purpose and process of the command, an in-line provision of the command itself, and a reference to an external version of the command such as a coding script. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:CommandCodeType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_CommandCode$DdiUrn} returns the DDI URN of myDDI4_CommandCode.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{description}}{ A read/write active binding.
#'        A description of the purpose and use of the command code provided. Supports multiple languages.
#'
#'                              \code{myDDI4_CommandCode$description <- list(myInternationalStructuredString)} sets the 'description' property of 'myDDI4_CommandCode' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_CommandCode$description} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{usesCommandFile}}{ A read/write active binding.
#'        Identifies and provides a link to an external copy of the command, for example, a SAS Command Code script. Designates the programming language of the command file as well as the URI for the file.
#'
#'                              \code{myDDI4_CommandCode$usesCommandFile <- list(myCommandFile)} sets the 'usesCommandFile' property of 'myDDI4_CommandCode' to a list of R6 CommandFile class instances
#'
#'                              \code{myDDI4_CommandCode$usesCommandFile} returns a list of of R6 : CommandFile class instances}
#'   }
#'   \describe{
#'     \item{\code{usesCommand}}{ A read/write active binding.
#'        This is an in-line provision of the command itself. It provides the programming language used as well as the command.
#'
#'                              \code{myDDI4_CommandCode$usesCommand <- list(myCommand)} sets the 'usesCommand' property of 'myDDI4_CommandCode' to a list of R6 Command class instances
#'
#'                              \code{myDDI4_CommandCode$usesCommand} returns a list of of R6 : Command class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CommandCode$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, description=NA, usesCommandFile=NA, usesCommand=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CommandCode <- R6Class(classname="DDI4_CommandCode",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, description=NA, usesCommandFile=NA, usesCommand=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CommandCode may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(description)){
        private$.description <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=description , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(usesCommandFile)){
        private$.usesCommandFile <- returnR6ObjectIfValid(validClass="DDI4_CommandFile", proposedObjectList=usesCommandFile , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(usesCommand)){
        private$.usesCommand <- returnR6ObjectIfValid(validClass="DDI4_Command", proposedObjectList=usesCommand , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    description = function(val){
      if (!missing(val)){
        private$.description <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.description
      }
    },

    usesCommand = function(val){
      if (!missing(val)){
        private$.usesCommand <- returnR6ObjectIfValid(validClass="DDI4_Command", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.usesCommand
      }
    },

    usesCommandFile = function(val){
      if (!missing(val)){
        private$.usesCommandFile <- returnR6ObjectIfValid(validClass="DDI4_CommandFile", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.usesCommandFile
      }
    }


  ),

  private = list(
    .description = NULL,
    .usesCommandFile = NULL,
    .usesCommand = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        description = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usesCommand = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Command", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usesCommandFile = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_CommandFile", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_CommandCode


#' @title DDI4_CommandFile  Identifies and provides a link to an external copy of the command, for example, a SAS Command Code script
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CommandFile}{CommandFile} 

#'
#'   Definition
#'
#'   ============
#'
#'   Identifies and provides a link to an external copy of the command, for example, a SAS Command Code script. Designates the programming language of the command file, a description of the location of the file , and a URN or URL for the command file.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:CommandFileType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_CommandFile$DdiUrn} returns the DDI URN of myDDI4_CommandFile.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{programLanguage}}{ A read/write active binding.
#'        Designates the programming language used for the command. Supports the use of a controlled vocabulary.
#'
#'                              \code{myDDI4_CommandFile$programLanguage <- list(myExternalControlledVocabularyEntry)} sets the 'programLanguage' property of 'myDDI4_CommandFile' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_CommandFile$programLanguage} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{location}}{ A read/write active binding.
#'        A description of the location of the file. This may not be machine actionable. It supports a description expressed in multiple languages.
#'
#'                              \code{myDDI4_CommandFile$location <- list(myInternationalString)} sets the 'location' property of 'myDDI4_CommandFile' to a list of R6 InternationalString class instances
#'
#'                              \code{myDDI4_CommandFile$location} returns a list of of R6 : InternationalString class instances}
#'   }
#'   \describe{
#'     \item{\code{uri}}{ A read/write active binding.
#'        The URL or URN of the command file.
#'
#'                              \code{myDDI4_CommandFile$uri <- mycharacter} sets the uri property of 'myDDI4_CommandFile' to the vector of datatype:character
#'
#'                              \code{myDDI4_CommandFile$uri} returns a vector of datatype: character}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CommandFile$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, programLanguage=NA, location=NA, uri=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CommandFile <- R6Class(classname="DDI4_CommandFile",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, programLanguage=NA, location=NA, uri=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CommandFile may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(programLanguage)){
        private$.programLanguage <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=programLanguage , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(location)){
        private$.location <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=location , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(uri)){
        private$.uri <- returnVectorIfValid(validVectorClass="character", proposedVector=uri , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    location = function(val){
      if (!missing(val)){
        private$.location <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.location
      }
    },

    programLanguage = function(val){
      if (!missing(val)){
        private$.programLanguage <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.programLanguage
      }
    },

    uri = function(val){
      if (!missing(val)){
        private$.uri <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.uri
      }
    }


  ),

  private = list(
    .programLanguage = NULL,
    .location = NULL,
    .uri = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        location = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        programLanguage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        uri = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_CommandFile


#' @title DDI4_Comparison  The minimal pattern for a comparison including a mapping between members
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Comparison}{Comparison} 

#'
#'   Definition
#'
#'   ============
#'
#'   The minimal pattern for a comparison including a mapping between members.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Comparison$DdiUrn} returns the DDI URN of myDDI4_Comparison.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{correspondence}}{ A read/write active binding.
#'        A map defining the match relationship and the members in the relationship
#'
#'                              \code{myDDI4_Comparison$correspondence <- list(myComparisonMap)} sets the 'correspondence' property of 'myDDI4_Comparison' to a list of R6 ComparisonMap class instances
#'
#'                              \code{myDDI4_Comparison$correspondence} returns a list of of R6 : ComparisonMap class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{maps}}{ A read/write active binding.
#'          The collection(s) contianing the mapped members.
#'
#'                              \code{myDDI4_Comparison$maps <- c(mySimpleCollectionURN)} sets the association 'maps' from 'myDDI4_Comparison' to a character vector of DdiUrns for R6 SimpleCollection class instances
#'
#'                              \code{myDDI4_Comparison$maps} returns a character vector of of DdiUrns for R6  SimpleCollection class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Comparison$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, correspondence=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Comparison <- R6Class(classname="DDI4_Comparison",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, correspondence=NA,    maps=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Comparison may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(correspondence)){
        private$.correspondence <- returnR6ObjectIfValid(validClass="DDI4_ComparisonMap", proposedObjectList=correspondence , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(maps)){
        private$.maps <- returnUrnVectorIfValid(validClass="DDI4_SimpleCollection", proposedUrnVector=maps , minimumCardinality=1, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.maps, referencingAssociationName="maps",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    correspondence = function(val){
      if (!missing(val)){
        private$.correspondence <- returnR6ObjectIfValid(validClass="DDI4_ComparisonMap", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.correspondence
      }
    },


# There are 1 class association(s):
    maps = function(val){
      if (!missing(val)){
        private$.maps <- returnUrnVectorIfValid(validClass="DDI4_SimpleCollection", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.maps, referencingAssociationName="maps",  referencingUrn=thisUrn)  
       # invisible( private$.maps)
      } else {
          private$.maps
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .correspondence = NULL,
    .maps = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        correspondence = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ComparisonMap", 
          isAbstractDatatype = TRUE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        maps = c(
          min="1", 
          max="Inf", 
          datatype = "DDI4_SimpleCollection", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_Comparison


#' @title DDI4_ComparisonMap  Provides a basic pattern for a comparison map which identifies source and target members and details about their match
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ComparisonMap}{ComparisonMap} 

#'
#'   Definition
#'
#'   ============
#'
#'   Provides a basic pattern for a comparison map which identifies source and target members and details about their match. Source and target were retained as in describing differences and commonalities the direction of the comparison may be important.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Comparing the Classification Items between Statistical Classifications; describing the comparative relationship between InstanceVariables in two LogicalRecords for use in linking records
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ComparisonMap$DdiUrn} returns the DDI URN of myDDI4_ComparisonMap.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasCorrespondenceType}}{ A read/write active binding.
#'        Describes the relationship between the source and target members using both controlled vocabularies and descriptive text.
#'
#'                              \code{myDDI4_ComparisonMap$hasCorrespondenceType <- list(myCorrespondenceType)} sets the 'hasCorrespondenceType' property of 'myDDI4_ComparisonMap' to a list of R6 CorrespondenceType class instances
#'
#'                              \code{myDDI4_ComparisonMap$hasCorrespondenceType} returns a list of of R6 : CorrespondenceType class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          The member defined as the source in the relationship. Note that when a correspondence has been defined as disjoint only the source relationship should be used.
#'
#'                              \code{myDDI4_ComparisonMap$source <- c(myCollectionMemberURN)} sets the association 'source' from 'myDDI4_ComparisonMap' to a character vector of DdiUrns for R6 CollectionMember class instances
#'
#'                              \code{myDDI4_ComparisonMap$source} returns a character vector of of DdiUrns for R6  CollectionMember class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          The member(s) defined as the target of the mapping. If relationship is disjoint do not use target relationship.
#'
#'                              \code{myDDI4_ComparisonMap$target <- c(myCollectionMemberURN)} sets the association 'target' from 'myDDI4_ComparisonMap' to a character vector of DdiUrns for R6 CollectionMember class instances
#'
#'                              \code{myDDI4_ComparisonMap$target} returns a character vector of of DdiUrns for R6  CollectionMember class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ComparisonMap$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasCorrespondenceType=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ComparisonMap <- R6Class(classname="DDI4_ComparisonMap",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasCorrespondenceType=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ComparisonMap may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(hasCorrespondenceType)){
        private$.hasCorrespondenceType <- returnR6ObjectIfValid(validClass="DDI4_CorrespondenceType", proposedObjectList=hasCorrespondenceType , minimumCardinality=1, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_CollectionMember", proposedUrnVector=source , minimumCardinality=1, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_CollectionMember", proposedUrnVector=target , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    hasCorrespondenceType = function(val){
      if (!missing(val)){
        private$.hasCorrespondenceType <- returnR6ObjectIfValid(validClass="DDI4_CorrespondenceType", proposedObjectList=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasCorrespondenceType
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_CollectionMember", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_CollectionMember", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .hasCorrespondenceType = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasCorrespondenceType = c(
          min="1", 
          max="1", 
          datatype = "DDI4_CorrespondenceType", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="1", 
          max="Inf", 
          datatype = "DDI4_CollectionMember", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_CollectionMember", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_ComparisonMap


#' @title DDI4_ComplianceStatement  Allows for a quality statement based on frameworks to be described using itemized properties
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ComplianceStatement}{ComplianceStatement} 

#'
#'   Definition
#'
#'   ============
#'
#'   Allows for a quality statement based on frameworks to be described using itemized properties. A reference to a concept, a coded value, or both can be used to specify the property from the standard framework identified in StandardUsed. Usage can provide further details or a general description of compliance with a standard.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:ComplianceType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ComplianceStatement$DdiUrn} returns the DDI URN of myDDI4_ComplianceStatement.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{externalComplianceCode}}{ A read/write active binding.
#'        Specification of a code which relates to an area of coverage of the standard. Supports the use of an external controlled vocabulary.
#'
#'                              \code{myDDI4_ComplianceStatement$externalComplianceCode <- list(myExternalControlledVocabularyEntry)} sets the 'externalComplianceCode' property of 'myDDI4_ComplianceStatement' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ComplianceStatement$externalComplianceCode} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{usage}}{ A read/write active binding.
#'        Explanation of the ways in which some decision or object is employed. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_ComplianceStatement$usage <- list(myInternationalStructuredString)} sets the 'usage' property of 'myDDI4_ComplianceStatement' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_ComplianceStatement$usage} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{definingConcept}}{ A read/write active binding.
#'          A reference to a concept which relates to an area of coverage of the standard.
#'
#'                              \code{myDDI4_ComplianceStatement$definingConcept <- c(myConceptURN)} sets the association 'definingConcept' from 'myDDI4_ComplianceStatement' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_ComplianceStatement$definingConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ComplianceStatement$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, externalComplianceCode=NA, usage=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ComplianceStatement <- R6Class(classname="DDI4_ComplianceStatement",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, externalComplianceCode=NA, usage=NA,    definingConcept=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ComplianceStatement may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(externalComplianceCode)){
        private$.externalComplianceCode <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=externalComplianceCode , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(usage)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=usage , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(definingConcept)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=definingConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 2 properties: 
    externalComplianceCode = function(val){
      if (!missing(val)){
        private$.externalComplianceCode <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.externalComplianceCode
      }
    },

    usage = function(val){
      if (!missing(val)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.usage
      }
    },


# There are 1 class association(s):
    definingConcept = function(val){
      if (!missing(val)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
       # invisible( private$.definingConcept)
      } else {
          private$.definingConcept
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .externalComplianceCode = NULL,
    .usage = NULL,
    .definingConcept = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        externalComplianceCode = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ComplianceStatement


#' @title DDI4_ComputationAction  Provides an extensible framework for specific computation or transformation objects
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ComputationAction}{ComputationAction} 

#'
#'   Definition
#'
#'   ============
#'
#'   Provides an extensible framework for specific computation or transformation objects.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   In data processing, a ComputationAction might be a code statement in a code sequence.
#'
#'   
#'
#'   In data capture, a ComputationAction might takes the form of a question, a measurement or instrument code.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   d:ComputationItemType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Act}}}
#'    \item{\code{\link{DDI4_WorkflowStep}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ComputationAction$DdiUrn} returns the DDI URN of myDDI4_ComputationAction.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{activityDescription}}{ A read/write active binding.
#'        Describes the transformation activity
#'
#'                              \code{myDDI4_ComputationAction$activityDescription <- list(myInternationalStructuredString)} sets the 'activityDescription' property of 'myDDI4_ComputationAction' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_ComputationAction$activityDescription} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{usesCommandCode}}{ A read/write active binding.
#'        The command code performed as a part of the transformation or computation
#'
#'                              \code{myDDI4_ComputationAction$usesCommandCode <- list(myCommandCode)} sets the 'usesCommandCode' property of 'myDDI4_ComputationAction' to a list of R6 CommandCode class instances
#'
#'                              \code{myDDI4_ComputationAction$usesCommandCode} returns a list of of R6 : CommandCode class instances}
#'   }
#'   \describe{
#'     \item{\code{typeOfComputation}}{ A read/write active binding.
#'        Allows for the specification of a computation or transformation type supporting the use of an external controlled vocabulary
#'
#'                              \code{myDDI4_ComputationAction$typeOfComputation <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfComputation' property of 'myDDI4_ComputationAction' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ComputationAction$typeOfComputation} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ComputationAction$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, activityDescription=NA, usesCommandCode=NA, typeOfComputation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ComputationAction <- R6Class(classname="DDI4_ComputationAction",
  inherit = DDI4_Act,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, activityDescription=NA, usesCommandCode=NA, typeOfComputation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ComputationAction may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(activityDescription)){
        private$.activityDescription <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=activityDescription , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(usesCommandCode)){
        private$.usesCommandCode <- returnR6ObjectIfValid(validClass="DDI4_CommandCode", proposedObjectList=usesCommandCode , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(typeOfComputation)){
        private$.typeOfComputation <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfComputation , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    activityDescription = function(val){
      if (!missing(val)){
        private$.activityDescription <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.activityDescription
      }
    },

    typeOfComputation = function(val){
      if (!missing(val)){
        private$.typeOfComputation <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.typeOfComputation
      }
    },

    usesCommandCode = function(val){
      if (!missing(val)){
        private$.usesCommandCode <- returnR6ObjectIfValid(validClass="DDI4_CommandCode", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.usesCommandCode
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .activityDescription = NULL,
    .usesCommandCode = NULL,
    .typeOfComputation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        activityDescription = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInformationFlow = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPerformedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowService", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        typeOfComputation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usesCommandCode = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CommandCode", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ComputationAction


#' @title DDI4_Concept  Unit of thought differentiated by characteristics [GSIM 1
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Concept}{Concept} 

#'
#'   Definition
#'
#'   ============
#'
#'   Unit of thought differentiated by characteristics [GSIM 1.1]
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Poverty, Income, Household relationship, Family, Gender, Business Establishment, Satisfaction, etc.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Many DDI4 classes are subtypes of the Concept class including Category, Universe, UnitType, ConceptualVariable. This class realizes the pattern class Signified and as such a Concept can be denoted by a Sign.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   c:ConceptType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Concept
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Concept$DdiUrn} returns the DDI URN of myDDI4_Concept.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage. 
#'
#'                              \code{myDDI4_Concept$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_Concept' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_Concept$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{definition}}{ A read/write active binding.
#'        Natural language statement conveying the meaning of a concept, differentiating it from other concepts. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_Concept$definition <- list(myInternationalStructuredString)} sets the 'definition' property of 'myDDI4_Concept' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Concept$definition} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A structured display label providing a fully human readable display for the identification of the object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_Concept$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_Concept' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_Concept$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Concept$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, name=NA, definition=NA, displayLabel=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Concept <- R6Class(classname="DDI4_Concept",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, name=NA, definition=NA, displayLabel=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Concept may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(definition)){
        private$.definition <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=definition , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    definition = function(val){
      if (!missing(val)){
        private$.definition <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.definition
      }
    },

    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .name = NULL,
    .definition = NULL,
    .displayLabel = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definition = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Concept


#' @title DDI4_ConceptIndicator  Member Indicator for use with member type Concept and all subtypes of Concept: Category, Universe, Population, Unit Type
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ConceptIndicator}{ConceptIndicator} 

#'
#'   Definition
#'
#'   ============
#'
#'   Member Indicator for use with member type Concept and all subtypes of Concept: Category, Universe, Population, Unit Type
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ConceptIndicator$DdiUrn} returns the DDI URN of myDDI4_ConceptIndicator.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Index value of member in an ordered array
#'
#'                              \code{myDDI4_ConceptIndicator$index <- myinteger} sets the index property of 'myDDI4_ConceptIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_ConceptIndicator$index} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{member}}{ A read/write active binding.
#'          Restricts member target class to Concept or subtype of Concept
#'
#'                              \code{myDDI4_ConceptIndicator$member <- c(myConceptURN)} sets the association 'member' from 'myDDI4_ConceptIndicator' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_ConceptIndicator$member} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ConceptIndicator$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ConceptIndicator <- R6Class(classname="DDI4_ConceptIndicator",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA,    member=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ConceptIndicator may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(member)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=member , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.index
      }
    },


# There are 1 class association(s):
    member = function(val){
      if (!missing(val)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
       # invisible( private$.member)
      } else {
          private$.member
      }
    }

  ),

  private = list(
    .index = NULL,
    .member = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        index = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        member = c(
          min="1", 
          max="1", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ConceptIndicator


#' @title DDI4_ConceptRelation  Specifies the relationship between concepts in a collection of concepts
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ConceptRelation}{ConceptRelation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Specifies the relationship between concepts in a collection of concepts. Use controlled vocabulary provided in hasRelationSpecification to identify the type of relationship (e.g. ParentChild, WholePart, etc.)
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ConceptRelation$DdiUrn} returns the DDI URN of myDDI4_ConceptRelation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        RelationSpecification Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an ennumeration list
#'
#'                              \code{myDDI4_ConceptRelation$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_ConceptRelation' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_ConceptRelation$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_ConceptRelation$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_ConceptRelation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ConceptRelation$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_ConceptRelation$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_ConceptRelation' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_ConceptRelation$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          Restricts source object to Concept for relationshp
#'
#'                              \code{myDDI4_ConceptRelation$source <- c(myConceptURN)} sets the association 'source' from 'myDDI4_ConceptRelation' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_ConceptRelation$source} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          Restricts target object to Concept for relationshp
#'
#'                              \code{myDDI4_ConceptRelation$target <- c(myConceptURN)} sets the association 'target' from 'myDDI4_ConceptRelation' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_ConceptRelation$target} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ConceptRelation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ConceptRelation <- R6Class(classname="DDI4_ConceptRelation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ConceptRelation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=source , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=target , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="1", 
          max="1", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="1", 
          max="1", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ConceptRelation


#' @title DDI4_ConceptRelationStructure  Relation structure of concepts within a collection
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ConceptRelationStructure}{ConceptRelationStructure} 

#'
#'   Definition
#'
#'   ============
#'
#'   Relation structure of concepts within a collection. Allows for the specification of complex relationships among concepts.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A concept of vacation might be described as having sub-types of beach vacation and mountain vacation.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   The ConceptRelationStructure employs a set of ConceptRelations to describe the relationship among concepts. Each ConceptRelation is a one to many description of connections between concepts. Together they can describe relationships as complex as a hierarchy or even a complete cyclical network as in a concept map.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ConceptRelationStructure$DdiUrn} returns the DDI URN of myDDI4_ConceptRelationStructure.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an ennumeration list.  Use if all relations within this relation structure are of the same specification.
#'
#'                              \code{myDDI4_ConceptRelationStructure$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_ConceptRelationStructure' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_ConceptRelationStructure$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_ConceptRelationStructure$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_ConceptRelationStructure' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ConceptRelationStructure$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_ConceptRelationStructure$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_ConceptRelationStructure' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_ConceptRelationStructure$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMemberRelation}}{ A read/write active binding.
#'        Restricted to ConceptRelation. ConceptRelations making up the relation structure.
#'
#'                              \code{myDDI4_ConceptRelationStructure$hasMemberRelation <- list(myConceptRelation)} sets the 'hasMemberRelation' property of 'myDDI4_ConceptRelationStructure' to a list of R6 ConceptRelation class instances
#'
#'                              \code{myDDI4_ConceptRelationStructure$hasMemberRelation} returns a list of of R6 : ConceptRelation class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ConceptRelationStructure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ConceptRelationStructure <- R6Class(classname="DDI4_ConceptRelationStructure",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ConceptRelationStructure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasMemberRelation)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_ConceptRelation", proposedObjectList=hasMemberRelation , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    hasMemberRelation = function(val){
      if (!missing(val)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_ConceptRelation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasMemberRelation
      }
    },

    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .hasMemberRelation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasMemberRelation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ConceptRelation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ConceptRelationStructure


#' @title DDI4_ConceptSystem  A set of Concepts structured by the relations among them
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ConceptSystem}{ConceptSystem} 

#'
#'   Definition
#'
#'   ============
#'
#'   A set of Concepts structured by the relations among them. [GSIM 1.1] 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   1) Concept of Sex: Male, Female, Other 2) Concept of Household Relationship: Household Head, Spouse of Household Head, Child of Household Head, Unrelated Household Member, etc.  
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Note that this structure can be used to structure Concept, Classification, Universe, Population, Unit Type and any other class that extends from Concept.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   c:ConceptSchemeType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Concept System
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ConceptSystem$DdiUrn} returns the DDI URN of myDDI4_ConceptSystem.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{type}}{ A read/write active binding.
#'        Whether the collection is a bag or a set: a bag is a collection with duplicates allowed, a set is a collection without duplicates.
#'
#'                              \code{myDDI4_ConceptSystem$type <- list(myCollectionType)} sets the 'type' property of 'myDDI4_ConceptSystem' to a list of R6 CollectionType class instances
#'
#'                              \code{myDDI4_ConceptSystem$type} returns a list of of R6 : CollectionType class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage.
#'
#'                              \code{myDDI4_ConceptSystem$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_ConceptSystem' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_ConceptSystem$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_ConceptSystem$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_ConceptSystem' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_ConceptSystem$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        MemberIndicator Allows for the identification of the member and optionally provides an index for the member within an ordered array
#'
#'                              \code{myDDI4_ConceptSystem$contains <- list(myConceptIndicator)} sets the 'contains' property of 'myDDI4_ConceptSystem' to a list of R6 ConceptIndicator class instances
#'
#'                              \code{myDDI4_ConceptSystem$contains} returns a list of of R6 : ConceptIndicator class instances}
#'   }
#'   \describe{
#'     \item{\code{isOrdered}}{ A read/write active binding.
#'        If members are ordered set to true, if unordered set to false.
#'
#'                              \code{myDDI4_ConceptSystem$isOrdered <- mylogical} sets the isOrdered property of 'myDDI4_ConceptSystem' to the vector of datatype:logical
#'
#'                              \code{myDDI4_ConceptSystem$isOrdered} returns a vector of datatype: logical}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isStructuredBy}}{ A read/write active binding.
#'          A structure containing specific relationships between members of the collection.
#'
#'                              \code{myDDI4_ConceptSystem$isStructuredBy <- c(myConceptRelationStructureURN)} sets the association 'isStructuredBy' from 'myDDI4_ConceptSystem' to a character vector of DdiUrns for R6 ConceptRelationStructure class instances
#'
#'                              \code{myDDI4_ConceptSystem$isStructuredBy} returns a character vector of of DdiUrns for R6  ConceptRelationStructure class instances}
#'   }
#'   \describe{
#'     \item{\code{definingConcept}}{ A read/write active binding.
#'          The conceptual basis for the collection of members.
#'
#'                              \code{myDDI4_ConceptSystem$definingConcept <- c(myConceptURN)} sets the association 'definingConcept' from 'myDDI4_ConceptSystem' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_ConceptSystem$definingConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ConceptSystem$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ConceptSystem <- R6Class(classname="DDI4_ConceptSystem",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA,    isStructuredBy=NA,definingConcept=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ConceptSystem may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(type)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=type , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_ConceptIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(isOrdered)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=isOrdered , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(isStructuredBy)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_ConceptRelationStructure", proposedUrnVector=isStructuredBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
      }
      if(!missing(definingConcept)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=definingConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 5 properties: 
    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_ConceptIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },

    isOrdered = function(val){
      if (!missing(val)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isOrdered
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    type = function(val){
      if (!missing(val)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.type
      }
    },


# There are 2 class association(s):
    definingConcept = function(val){
      if (!missing(val)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
       # invisible( private$.definingConcept)
      } else {
          private$.definingConcept
      }
    },

    isStructuredBy = function(val){
      if (!missing(val)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_ConceptRelationStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
       # invisible( private$.isStructuredBy)
      } else {
          private$.isStructuredBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .type = NULL,
    .name = NULL,
    .purpose = NULL,
    .contains = NULL,
    .isOrdered = NULL,
    .isStructuredBy = NULL,
    .definingConcept = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ConceptIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isStructuredBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ConceptRelationStructure", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ConceptSystem


#' @title DDI4_ConceptSystemCorrespondence  Relationship between 2 or more ConceptSystems described through mapping similarity relationships between their member Concepts
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ConceptSystemCorrespondence}{ConceptSystemCorrespondence} 

#'
#'   Definition
#'
#'   ============
#'
#'   Relationship between 2 or more ConceptSystems described through mapping similarity relationships between their member Concepts.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Correspondence between the concepts used to define the populations in the censuses of two countries with similarity mapping of Concepts "Resident Population", "Labor Force", "Housing Unit", etc. 
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Describes correspondence with one or more Maps which identify a source and target concept and defines their commonality and difference using descriptive text and controlled vocabularies.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ConceptSystemCorrespondence$DdiUrn} returns the DDI URN of myDDI4_ConceptSystemCorrespondence.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A display label for the CollectionCorrespondence. May be expressed in multiple languages. Repeat for labels with different content, for example, labels with differing length limitations.
#'
#'                              \code{myDDI4_ConceptSystemCorrespondence$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_ConceptSystemCorrespondence' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_ConceptSystemCorrespondence$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_ConceptSystemCorrespondence$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_ConceptSystemCorrespondence' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_ConceptSystemCorrespondence$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{usage}}{ A read/write active binding.
#'        Explanation of the ways in which some decision or object is employed. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_ConceptSystemCorrespondence$usage <- list(myInternationalStructuredString)} sets the 'usage' property of 'myDDI4_ConceptSystemCorrespondence' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_ConceptSystemCorrespondence$usage} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{correspondence}}{ A read/write active binding.
#'        Realization of correspondence in Comparison for mapping similar concepts.
#'
#'                              \code{myDDI4_ConceptSystemCorrespondence$correspondence <- list(myMap)} sets the 'correspondence' property of 'myDDI4_ConceptSystemCorrespondence' to a list of R6 Map class instances
#'
#'                              \code{myDDI4_ConceptSystemCorrespondence$correspondence} returns a list of of R6 : Map class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{maps}}{ A read/write active binding.
#'          Realization of structures in Symmetric Relation. When Concepts of a single ConceptSystem are mapped, the Concept has to appear twice as target.
#'
#'                              \code{myDDI4_ConceptSystemCorrespondence$maps <- c(myConceptSystemURN)} sets the association 'maps' from 'myDDI4_ConceptSystemCorrespondence' to a character vector of DdiUrns for R6 ConceptSystem class instances
#'
#'                              \code{myDDI4_ConceptSystemCorrespondence$maps} returns a character vector of of DdiUrns for R6  ConceptSystem class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ConceptSystemCorrespondence$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, displayLabel=NA, purpose=NA, usage=NA, correspondence=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ConceptSystemCorrespondence <- R6Class(classname="DDI4_ConceptSystemCorrespondence",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, displayLabel=NA, purpose=NA, usage=NA, correspondence=NA,    maps=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ConceptSystemCorrespondence may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(usage)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=usage , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(correspondence)){
        private$.correspondence <- returnR6ObjectIfValid(validClass="DDI4_Map", proposedObjectList=correspondence , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(maps)){
        private$.maps <- returnUrnVectorIfValid(validClass="DDI4_ConceptSystem", proposedUrnVector=maps , minimumCardinality=2, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.maps, referencingAssociationName="maps",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 4 properties: 
    correspondence = function(val){
      if (!missing(val)){
        private$.correspondence <- returnR6ObjectIfValid(validClass="DDI4_Map", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.correspondence
      }
    },

    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    usage = function(val){
      if (!missing(val)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.usage
      }
    },


# There are 1 class association(s):
    maps = function(val){
      if (!missing(val)){
        private$.maps <- returnUrnVectorIfValid(validClass="DDI4_ConceptSystem", proposedUrnVector=val , minimumCardinality=2, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.maps, referencingAssociationName="maps",  referencingUrn=thisUrn)  
       # invisible( private$.maps)
      } else {
          private$.maps
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .displayLabel = NULL,
    .purpose = NULL,
    .usage = NULL,
    .correspondence = NULL,
    .maps = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        correspondence = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Map", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        maps = c(
          min="2", 
          max="Inf", 
          datatype = "DDI4_ConceptSystem", 
          structure = "DdiUrn vector"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ConceptSystemCorrespondence


#' @title DDI4_ConceptualDomain  Set of Concepts, both sentinel and substantive, that can be described by either enumeration or by an expression
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ConceptualDomain}{ConceptualDomain} 

#'
#'   Definition
#'
#'   ============
#'
#'   Set of Concepts, both sentinel and substantive, that can be described by either enumeration or by an expression.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Substantive: Housing Unit Tenure - Owned, Rented, Vacant; Sentinel: Non-response - Refused, Don't Know, Not Applicable 
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Intent of a Conceptual Domain is defining a set of concepts used to measure a broader concept. For effective use they should be discrete (non-overlapping) and provide exhaustive coverage of the broader concept.  
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ConceptualDomain$DdiUrn} returns the DDI URN of myDDI4_ConceptualDomain.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A structured display label providing a fully human readable display for the identification of the object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_ConceptualDomain$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_ConceptualDomain' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_ConceptualDomain$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{describedConceptualDomain}}{ A read/write active binding.
#'          A description of the concepts in the domain. A numeric domain might use a logical expression to be machine actionable a text domain might use a regular expression to describe strings that describe the concepts.
#'
#'                              \code{myDDI4_ConceptualDomain$describedConceptualDomain <- c(myValueAndConceptDescriptionURN)} sets the association 'describedConceptualDomain' from 'myDDI4_ConceptualDomain' to a character vector of DdiUrns for R6 ValueAndConceptDescription class instances
#'
#'                              \code{myDDI4_ConceptualDomain$describedConceptualDomain} returns a character vector of of DdiUrns for R6  ValueAndConceptDescription class instances}
#'   }
#'   \describe{
#'     \item{\code{enumeratedConceptualDomain}}{ A read/write active binding.
#'          The set of categories containing the concepts in the domain.
#'
#'                              \code{myDDI4_ConceptualDomain$enumeratedConceptualDomain <- c(myConceptSystemURN)} sets the association 'enumeratedConceptualDomain' from 'myDDI4_ConceptualDomain' to a character vector of DdiUrns for R6 ConceptSystem class instances
#'
#'                              \code{myDDI4_ConceptualDomain$enumeratedConceptualDomain} returns a character vector of of DdiUrns for R6  ConceptSystem class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ConceptualDomain$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, displayLabel=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ConceptualDomain <- R6Class(classname="DDI4_ConceptualDomain",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, displayLabel=NA,    describedConceptualDomain=NA,enumeratedConceptualDomain=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ConceptualDomain may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(describedConceptualDomain)){
        private$.describedConceptualDomain <- returnUrnVectorIfValid(validClass="DDI4_ValueAndConceptDescription", proposedUrnVector=describedConceptualDomain , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.describedConceptualDomain, referencingAssociationName="describedConceptualDomain",  referencingUrn=thisUrn)  
      }
      if(!missing(enumeratedConceptualDomain)){
        private$.enumeratedConceptualDomain <- returnUrnVectorIfValid(validClass="DDI4_ConceptSystem", proposedUrnVector=enumeratedConceptualDomain , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.enumeratedConceptualDomain, referencingAssociationName="enumeratedConceptualDomain",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },


# There are 2 class association(s):
    describedConceptualDomain = function(val){
      if (!missing(val)){
        private$.describedConceptualDomain <- returnUrnVectorIfValid(validClass="DDI4_ValueAndConceptDescription", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.describedConceptualDomain, referencingAssociationName="describedConceptualDomain",  referencingUrn=thisUrn)  
       # invisible( private$.describedConceptualDomain)
      } else {
          private$.describedConceptualDomain
      }
    },

    enumeratedConceptualDomain = function(val){
      if (!missing(val)){
        private$.enumeratedConceptualDomain <- returnUrnVectorIfValid(validClass="DDI4_ConceptSystem", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.enumeratedConceptualDomain, referencingAssociationName="enumeratedConceptualDomain",  referencingUrn=thisUrn)  
       # invisible( private$.enumeratedConceptualDomain)
      } else {
          private$.enumeratedConceptualDomain
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .displayLabel = NULL,
    .describedConceptualDomain = NULL,
    .enumeratedConceptualDomain = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        describedConceptualDomain = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ValueAndConceptDescription", 
          structure = "DdiUrn vector"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        enumeratedConceptualDomain = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ConceptSystem", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_ConceptualDomain


#' @title DDI4_ConceptualInstrument  Design plan for creating a data capture tool
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ConceptualInstrument}{ConceptualInstrument} 

#'
#'   Definition
#'
#'   ============
#'
#'   Design plan for creating a data capture tool. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Design of a questionnaire regardless of mode; Design of an experiment
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   A single ConceptualInstrument would contain workflow sequences of the intended flow logic of the instrument, which could be implemented as a web survey or paper survey, utilizing the same sequence and questions.
#'
#'   Similarly a single ConceptualInstrument would contain workflow sequences of the intended flow logic of the instrument, which could be implemented as a protocol delivered by a nurse or by self-administration.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   Idea; Intention; Theory; Design; LogicalInstrument
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_WorkflowProcess}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ConceptualInstrument$DdiUrn} returns the DDI URN of myDDI4_ConceptualInstrument.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A structured display label providing a fully human readable display for the identification of the object. Supports the use of multiple languages and structured text. 
#'
#'                              \code{myDDI4_ConceptualInstrument$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_ConceptualInstrument' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_ConceptualInstrument$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage. 
#'
#'                              \code{myDDI4_ConceptualInstrument$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_ConceptualInstrument' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_ConceptualInstrument$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{description}}{ A read/write active binding.
#'        A description of the purpose or use of the Member. May be expressed in multiple languages and supports the use of structured content. 
#'
#'                              \code{myDDI4_ConceptualInstrument$description <- list(myInternationalStructuredString)} sets the 'description' property of 'myDDI4_ConceptualInstrument' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_ConceptualInstrument$description} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{usage}}{ A read/write active binding.
#'        Explanation of the ways in which some decision or object is employed. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_ConceptualInstrument$usage <- list(myInternationalStructuredString)} sets the 'usage' property of 'myDDI4_ConceptualInstrument' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_ConceptualInstrument$usage} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{organizes}}{ A read/write active binding.
#'          Refers to the RepresentedMeasures and RepresentedQuestions used by the ConceptualInstrument.
#'
#'                              \code{myDDI4_ConceptualInstrument$organizes <- c(myCaptureURN)} sets the association 'organizes' from 'myDDI4_ConceptualInstrument' to a character vector of DdiUrns for R6 Capture class instances
#'
#'                              \code{myDDI4_ConceptualInstrument$organizes} returns a character vector of of DdiUrns for R6  Capture class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ConceptualInstrument$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, displayLabel=NA, name=NA, description=NA, usage=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ConceptualInstrument <- R6Class(classname="DDI4_ConceptualInstrument",
  inherit = DDI4_WorkflowProcess,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, displayLabel=NA, name=NA, description=NA, usage=NA,    organizes=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ConceptualInstrument may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(description)){
        private$.description <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=description , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(usage)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=usage , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(organizes)){
        private$.organizes <- returnUrnVectorIfValid(validClass="DDI4_Capture", proposedUrnVector=organizes , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.organizes, referencingAssociationName="organizes",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 4 properties: 
    description = function(val){
      if (!missing(val)){
        private$.description <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.description
      }
    },

    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    usage = function(val){
      if (!missing(val)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.usage
      }
    },


# There are 1 class association(s):
    organizes = function(val){
      if (!missing(val)){
        private$.organizes <- returnUrnVectorIfValid(validClass="DDI4_Capture", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.organizes, referencingAssociationName="organizes",  referencingUrn=thisUrn)  
       # invisible( private$.organizes)
      } else {
          private$.organizes
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .displayLabel = NULL,
    .name = NULL,
    .description = NULL,
    .usage = NULL,
    .organizes = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        description = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        implementsAlgorithm = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AlgorithmOverview", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isSpecifiedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowControlStep", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        organizes = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Capture", 
          structure = "DdiUrn vector"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ConceptualInstrument


#' @title DDI4_ConceptualVariable  The use of a Concept as a characteristic of a UnitType intended to be measured
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ConceptualVariable}{ConceptualVariable} 

#'
#'   Definition
#'
#'   ============
#'
#'   The use of a Concept as a characteristic of a UnitType intended to be measured
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   1. Sex of person
#'
#'   2. Number of employees 
#'
#'   3. Value of endowment
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Note that DDI varies from GSIM in the use of a UnitType rather than a Universe. "The use of a Concept as a characteristic of a Universe intended to be measured" [GSIM 1.1]
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   c:ConceptualVariableType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Variable
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Concept}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ConceptualVariable$DdiUrn} returns the DDI URN of myDDI4_ConceptualVariable.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{descriptiveText}}{ A read/write active binding.
#'        A short natural language account of the characteristics of the object.
#'
#'                              \code{myDDI4_ConceptualVariable$descriptiveText <- list(myInternationalStructuredString)} sets the 'descriptiveText' property of 'myDDI4_ConceptualVariable' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_ConceptualVariable$descriptiveText} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{usesConcept}}{ A read/write active binding.
#'          Reference to a Concept that is being used
#'
#'                              \code{myDDI4_ConceptualVariable$usesConcept <- c(myConceptURN)} sets the association 'usesConcept' from 'myDDI4_ConceptualVariable' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_ConceptualVariable$usesConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#'   \describe{
#'     \item{\code{takesSubstantiveConceptsFrom}}{ A read/write active binding.
#'          Identifies the ConceptualDomain containing the set of substantive concepts used to describe the  ConceptualVariable.
#'
#'                              \code{myDDI4_ConceptualVariable$takesSubstantiveConceptsFrom <- c(mySubstantiveConceptualDomainURN)} sets the association 'takesSubstantiveConceptsFrom' from 'myDDI4_ConceptualVariable' to a character vector of DdiUrns for R6 SubstantiveConceptualDomain class instances
#'
#'                              \code{myDDI4_ConceptualVariable$takesSubstantiveConceptsFrom} returns a character vector of of DdiUrns for R6  SubstantiveConceptualDomain class instances}
#'   }
#'   \describe{
#'     \item{\code{usesUnitType}}{ A read/write active binding.
#'          Identifies the UnitType associated with the ConceptualVariable
#'
#'                              \code{myDDI4_ConceptualVariable$usesUnitType <- c(myUnitTypeURN)} sets the association 'usesUnitType' from 'myDDI4_ConceptualVariable' to a character vector of DdiUrns for R6 UnitType class instances
#'
#'                              \code{myDDI4_ConceptualVariable$usesUnitType} returns a character vector of of DdiUrns for R6  UnitType class instances}
#'   }
#'   \describe{
#'     \item{\code{takesSentinelConceptsFrom}}{ A read/write active binding.
#'          Identifies the ConceptualDomain containing the set of sentinel concepts used to describe the  ConceptualVariable.
#'
#'                              \code{myDDI4_ConceptualVariable$takesSentinelConceptsFrom <- c(mySentinelConceptualDomainURN)} sets the association 'takesSentinelConceptsFrom' from 'myDDI4_ConceptualVariable' to a character vector of DdiUrns for R6 SentinelConceptualDomain class instances
#'
#'                              \code{myDDI4_ConceptualVariable$takesSentinelConceptsFrom} returns a character vector of of DdiUrns for R6  SentinelConceptualDomain class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ConceptualVariable$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, descriptiveText=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ConceptualVariable <- R6Class(classname="DDI4_ConceptualVariable",
  inherit = DDI4_Concept,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, descriptiveText=NA,    usesConcept=NA,takesSubstantiveConceptsFrom=NA,usesUnitType=NA,takesSentinelConceptsFrom=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ConceptualVariable may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(descriptiveText)){
        private$.descriptiveText <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=descriptiveText , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(usesConcept)){
        private$.usesConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=usesConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesConcept, referencingAssociationName="usesConcept",  referencingUrn=thisUrn)  
      }
      if(!missing(takesSubstantiveConceptsFrom)){
        private$.takesSubstantiveConceptsFrom <- returnUrnVectorIfValid(validClass="DDI4_SubstantiveConceptualDomain", proposedUrnVector=takesSubstantiveConceptsFrom , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.takesSubstantiveConceptsFrom, referencingAssociationName="takesSubstantiveConceptsFrom",  referencingUrn=thisUrn)  
      }
      if(!missing(usesUnitType)){
        private$.usesUnitType <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=usesUnitType , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesUnitType, referencingAssociationName="usesUnitType",  referencingUrn=thisUrn)  
      }
      if(!missing(takesSentinelConceptsFrom)){
        private$.takesSentinelConceptsFrom <- returnUrnVectorIfValid(validClass="DDI4_SentinelConceptualDomain", proposedUrnVector=takesSentinelConceptsFrom , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.takesSentinelConceptsFrom, referencingAssociationName="takesSentinelConceptsFrom",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    descriptiveText = function(val){
      if (!missing(val)){
        private$.descriptiveText <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.descriptiveText
      }
    },


# There are 4 class association(s):
    takesSentinelConceptsFrom = function(val){
      if (!missing(val)){
        private$.takesSentinelConceptsFrom <- returnUrnVectorIfValid(validClass="DDI4_SentinelConceptualDomain", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.takesSentinelConceptsFrom, referencingAssociationName="takesSentinelConceptsFrom",  referencingUrn=thisUrn)  
       # invisible( private$.takesSentinelConceptsFrom)
      } else {
          private$.takesSentinelConceptsFrom
      }
    },

    takesSubstantiveConceptsFrom = function(val){
      if (!missing(val)){
        private$.takesSubstantiveConceptsFrom <- returnUrnVectorIfValid(validClass="DDI4_SubstantiveConceptualDomain", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.takesSubstantiveConceptsFrom, referencingAssociationName="takesSubstantiveConceptsFrom",  referencingUrn=thisUrn)  
       # invisible( private$.takesSubstantiveConceptsFrom)
      } else {
          private$.takesSubstantiveConceptsFrom
      }
    },

    usesConcept = function(val){
      if (!missing(val)){
        private$.usesConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesConcept, referencingAssociationName="usesConcept",  referencingUrn=thisUrn)  
       # invisible( private$.usesConcept)
      } else {
          private$.usesConcept
      }
    },

    usesUnitType = function(val){
      if (!missing(val)){
        private$.usesUnitType <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesUnitType, referencingAssociationName="usesUnitType",  referencingUrn=thisUrn)  
       # invisible( private$.usesUnitType)
      } else {
          private$.usesUnitType
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .descriptiveText = NULL,
    .usesConcept = NULL,
    .takesSubstantiveConceptsFrom = NULL,
    .usesUnitType = NULL,
    .takesSentinelConceptsFrom = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definition = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        descriptiveText = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        takesSentinelConceptsFrom = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SentinelConceptualDomain", 
          structure = "DdiUrn vector"
        ),
        takesSubstantiveConceptsFrom = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SubstantiveConceptualDomain", 
          structure = "DdiUrn vector"
        ),
        usesConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        usesUnitType = c(
          min="0", 
          max="1", 
          datatype = "DDI4_UnitType", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ConceptualVariable


#' @title DDI4_ConditionalControlStep  Type of WorkflowControlStep in which the execution flow is determined by one or more conditions
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ConditionalControlStep}{ConditionalControlStep} 

#'
#'   Definition
#'
#'   ============
#'
#'   Type of WorkflowControlStep in which the execution flow is determined by one or more conditions.
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   If-then-else and iterative structures.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_WorkflowControlStep}}}
#'    \item{\code{\link{DDI4_WorkflowStep}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ConditionalControlStep$DdiUrn} returns the DDI URN of myDDI4_ConditionalControlStep.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{condition}}{ A read/write active binding.
#'        Condition to be evaluated to determine whether or not to execute the Workflow Sequence in contains. It can be an expression and/or programmatic code. The specialized sub-classes determine whether the Sequence is executed when the condition is true or false
#'
#'                              \code{myDDI4_ConditionalControlStep$condition <- list(myCommandCode)} sets the 'condition' property of 'myDDI4_ConditionalControlStep' to a list of R6 CommandCode class instances
#'
#'                              \code{myDDI4_ConditionalControlStep$condition} returns a list of of R6 : CommandCode class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{executes}}{ A read/write active binding.
#'          The Workflow Sequence is executed depending on the result of the condition evaluation. The specialized sub-classes determine whether the Sequence is executed when the condition is true or false. Specialization of contains in Control Construct.
#'
#'                              \code{myDDI4_ConditionalControlStep$executes <- c(myWorkflowStepSequenceURN)} sets the association 'executes' from 'myDDI4_ConditionalControlStep' to a character vector of DdiUrns for R6 WorkflowStepSequence class instances
#'
#'                              \code{myDDI4_ConditionalControlStep$executes} returns a character vector of of DdiUrns for R6  WorkflowStepSequence class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ConditionalControlStep$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, condition=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ConditionalControlStep <- R6Class(classname="DDI4_ConditionalControlStep",
  inherit = DDI4_WorkflowControlStep,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, condition=NA,    executes=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ConditionalControlStep may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(condition)){
        private$.condition <- returnR6ObjectIfValid(validClass="DDI4_CommandCode", proposedObjectList=condition , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(executes)){
        private$.executes <- returnUrnVectorIfValid(validClass="DDI4_WorkflowStepSequence", proposedUrnVector=executes , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.executes, referencingAssociationName="executes",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    condition = function(val){
      if (!missing(val)){
        private$.condition <- returnR6ObjectIfValid(validClass="DDI4_CommandCode", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.condition
      }
    },


# There are 1 class association(s):
    executes = function(val){
      if (!missing(val)){
        private$.executes <- returnUrnVectorIfValid(validClass="DDI4_WorkflowStepSequence", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.executes, referencingAssociationName="executes",  referencingUrn=thisUrn)  
       # invisible( private$.executes)
      } else {
          private$.executes
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .condition = NULL,
    .executes = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        condition = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CommandCode", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        executes = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowStepSequence", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInformationFlow = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPerformedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowService", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ConditionalControlStep


#' @title DDI4_ConditionalText  Text which has a changeable value depending on a stated condition, response to earlier questions, or as input from a set of metrics (pre-supplied data)
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ConditionalText}{ConditionalText} 

#'
#'   Definition
#'
#'   ============
#'
#'   Text which has a changeable value depending on a stated condition, response to earlier questions, or as input from a set of metrics (pre-supplied data). The command code indicates the condition and/or calculation used to determine the content of the text.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   d:ConditionalTextType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_DynamicTextContent}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ConditionalText$DdiUrn} returns the DDI URN of myDDI4_ConditionalText.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{expression}}{ A read/write active binding.
#'        The condition on which the associated text varies expressed by a command code. For example, a command that inserts an age by calculating the difference between today’s date and a previously defined date of birth.
#'
#'                              \code{myDDI4_ConditionalText$expression <- list(myCommandCode)} sets the 'expression' property of 'myDDI4_ConditionalText' to a list of R6 CommandCode class instances
#'
#'                              \code{myDDI4_ConditionalText$expression} returns a list of of R6 : CommandCode class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ConditionalText$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, expression=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ConditionalText <- R6Class(classname="DDI4_ConditionalText",
  inherit = DDI4_DynamicTextContent,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, expression=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ConditionalText may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(expression)){
        private$.expression <- returnR6ObjectIfValid(validClass="DDI4_CommandCode", proposedObjectList=expression , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 1 properties: 
    expression = function(val){
      if (!missing(val)){
        private$.expression <- returnR6ObjectIfValid(validClass="DDI4_CommandCode", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.expression
      }
    }


  ),

  private = list(
    .expression = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        expression = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CommandCode", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        orderPosition = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ConditionalText


#' @title DDI4_ContactInformation  Contact information for the individual or organization including location specification, address, web site, phone numbers, and other means of communication access
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ContactInformation}{ContactInformation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Contact information for the individual or organization including location specification, address, web site, phone numbers, and other means of communication access. Address, location, telephone, and other means of communication can be repeated to express multiple means of a single type or change over time. Each major piece of contact information contains the element EffectiveDates in order to date stamp the period for which the information is valid.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   a:ContactInformationType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ContactInformation$DdiUrn} returns the DDI URN of myDDI4_ContactInformation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{website}}{ A read/write active binding.
#'        The URL of the Agent's website
#'
#'                              \code{myDDI4_ContactInformation$website <- list(myWebLink)} sets the 'website' property of 'myDDI4_ContactInformation' to a list of R6 WebLink class instances
#'
#'                              \code{myDDI4_ContactInformation$website} returns a list of of R6 : WebLink class instances}
#'   }
#'   \describe{
#'     \item{\code{hasEmail}}{ A read/write active binding.
#'        Email contact information
#'
#'                              \code{myDDI4_ContactInformation$hasEmail <- list(myEmail)} sets the 'hasEmail' property of 'myDDI4_ContactInformation' to a list of R6 Email class instances
#'
#'                              \code{myDDI4_ContactInformation$hasEmail} returns a list of of R6 : Email class instances}
#'   }
#'   \describe{
#'     \item{\code{electronicMessaging}}{ A read/write active binding.
#'        Electronic messaging other than email
#'
#'                              \code{myDDI4_ContactInformation$electronicMessaging <- list(myElectronicMessageSystem)} sets the 'electronicMessaging' property of 'myDDI4_ContactInformation' to a list of R6 ElectronicMessageSystem class instances
#'
#'                              \code{myDDI4_ContactInformation$electronicMessaging} returns a list of of R6 : ElectronicMessageSystem class instances}
#'   }
#'   \describe{
#'     \item{\code{hasAddress}}{ A read/write active binding.
#'        The address for contact.
#'
#'                              \code{myDDI4_ContactInformation$hasAddress <- list(myAddress)} sets the 'hasAddress' property of 'myDDI4_ContactInformation' to a list of R6 Address class instances
#'
#'                              \code{myDDI4_ContactInformation$hasAddress} returns a list of of R6 : Address class instances}
#'   }
#'   \describe{
#'     \item{\code{hasTelephone}}{ A read/write active binding.
#'        Telephone for contact
#'
#'                              \code{myDDI4_ContactInformation$hasTelephone <- list(myTelephone)} sets the 'hasTelephone' property of 'myDDI4_ContactInformation' to a list of R6 Telephone class instances
#'
#'                              \code{myDDI4_ContactInformation$hasTelephone} returns a list of of R6 : Telephone class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ContactInformation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, website=NA, hasEmail=NA, electronicMessaging=NA, hasAddress=NA, hasTelephone=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ContactInformation <- R6Class(classname="DDI4_ContactInformation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, website=NA, hasEmail=NA, electronicMessaging=NA, hasAddress=NA, hasTelephone=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ContactInformation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(website)){
        private$.website <- returnR6ObjectIfValid(validClass="DDI4_WebLink", proposedObjectList=website , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(hasEmail)){
        private$.hasEmail <- returnR6ObjectIfValid(validClass="DDI4_Email", proposedObjectList=hasEmail , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(electronicMessaging)){
        private$.electronicMessaging <- returnR6ObjectIfValid(validClass="DDI4_ElectronicMessageSystem", proposedObjectList=electronicMessaging , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(hasAddress)){
        private$.hasAddress <- returnR6ObjectIfValid(validClass="DDI4_Address", proposedObjectList=hasAddress , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(hasTelephone)){
        private$.hasTelephone <- returnR6ObjectIfValid(validClass="DDI4_Telephone", proposedObjectList=hasTelephone , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 5 properties: 
    electronicMessaging = function(val){
      if (!missing(val)){
        private$.electronicMessaging <- returnR6ObjectIfValid(validClass="DDI4_ElectronicMessageSystem", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.electronicMessaging
      }
    },

    hasAddress = function(val){
      if (!missing(val)){
        private$.hasAddress <- returnR6ObjectIfValid(validClass="DDI4_Address", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasAddress
      }
    },

    hasEmail = function(val){
      if (!missing(val)){
        private$.hasEmail <- returnR6ObjectIfValid(validClass="DDI4_Email", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasEmail
      }
    },

    hasTelephone = function(val){
      if (!missing(val)){
        private$.hasTelephone <- returnR6ObjectIfValid(validClass="DDI4_Telephone", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasTelephone
      }
    },

    website = function(val){
      if (!missing(val)){
        private$.website <- returnR6ObjectIfValid(validClass="DDI4_WebLink", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.website
      }
    }


  ),

  private = list(
    .website = NULL,
    .hasEmail = NULL,
    .electronicMessaging = NULL,
    .hasAddress = NULL,
    .hasTelephone = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        electronicMessaging = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ElectronicMessageSystem", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAddress = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Address", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasEmail = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Email", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasTelephone = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Telephone", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        website = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_WebLink", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ContactInformation


#' @title DDI4_ContentDateOffset  Identifies the difference between the date applied to the data as a whole and this specific item such as previous year's income or residence 5 years ago
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ContentDateOffset}{ContentDateOffset} 

#'
#'   Definition
#'
#'   ============
#'
#'   Identifies the difference between the date applied to the data as a whole and this specific item such as previous year's income or residence 5 years ago. A value of true for the attribute isNegativeOffset indicates that the offset is the specified number of declared units prior to the date of the data as a whole and false indicates information regarding a future state.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:ContentDateOffsetType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ExternalControlledVocabularyEntry}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ContentDateOffset$DdiUrn} returns the DDI URN of myDDI4_ContentDateOffset.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{numberOfUnits}}{ A read/write active binding.
#'        The number of units to off-set the date for this item expressed as a decimal.
#'
#'                              \code{myDDI4_ContentDateOffset$numberOfUnits <- mynumeric} sets the numberOfUnits property of 'myDDI4_ContentDateOffset' to the vector of datatype:numeric
#'
#'                              \code{myDDI4_ContentDateOffset$numberOfUnits} returns a vector of datatype: numeric}
#'   }
#'   \describe{
#'     \item{\code{isNegativeOffset}}{ A read/write active binding.
#'        If set to "true" the date is offset the number of units specified PRIOR to the default date of the data. A setting of "false" indicates a date the specified number of units in the FUTURE from the default date of the data.
#'
#'                              \code{myDDI4_ContentDateOffset$isNegativeOffset <- mylogical} sets the isNegativeOffset property of 'myDDI4_ContentDateOffset' to the vector of datatype:logical
#'
#'                              \code{myDDI4_ContentDateOffset$isNegativeOffset} returns a vector of datatype: logical}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ContentDateOffset$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, numberOfUnits=NA, isNegativeOffset=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ContentDateOffset <- R6Class(classname="DDI4_ContentDateOffset",
  inherit = DDI4_ExternalControlledVocabularyEntry,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, numberOfUnits=NA, isNegativeOffset=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ContentDateOffset may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(numberOfUnits)){
        private$.numberOfUnits <- returnVectorIfValid(validVectorClass="numeric", proposedVector=numberOfUnits , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isNegativeOffset)){
        private$.isNegativeOffset <- returnVectorIfValid(validVectorClass="logical", proposedVector=isNegativeOffset , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    isNegativeOffset = function(val){
      if (!missing(val)){
        private$.isNegativeOffset <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isNegativeOffset
      }
    },

    numberOfUnits = function(val){
      if (!missing(val)){
        private$.numberOfUnits <- returnVectorIfValid(validVectorClass="numeric", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.numberOfUnits
      }
    }


  ),

  private = list(
    .numberOfUnits = NULL,
    .isNegativeOffset = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        content = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        controlledVocabularyAgencyName = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        controlledVocabularyID = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        controlledVocabularyName = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        controlledVocabularyVersionID = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isNegativeOffset = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        language = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        numberOfUnits = c(
          min="0", 
          max="1", 
          datatype = "numeric", 
          structure = "vector"
        ),
        otherValue = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        uri = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ContentDateOffset


#' @title DDI4_ControlledVocabulary  The specification of a controlled vocabulary defines a set of values and their definitions together with the order relationships among those entries
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ControlledVocabulary}{ControlledVocabulary} 

#'
#'   Definition
#'
#'   ============
#'
#'   The specification of a controlled vocabulary defines a set of values and their definitions together with the order relationships among those entries.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ControlledVocabulary$DdiUrn} returns the DDI URN of myDDI4_ControlledVocabulary.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{type}}{ A read/write active binding.
#'        Whether the collection is a bag or a set: a bag is a collection with duplicates allowed, a set is a collection without duplicates.
#'
#'                              \code{myDDI4_ControlledVocabulary$type <- list(myCollectionType)} sets the 'type' property of 'myDDI4_ControlledVocabulary' to a list of R6 CollectionType class instances
#'
#'                              \code{myDDI4_ControlledVocabulary$type} returns a list of of R6 : CollectionType class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage.
#'
#'                              \code{myDDI4_ControlledVocabulary$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_ControlledVocabulary' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_ControlledVocabulary$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_ControlledVocabulary$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_ControlledVocabulary' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_ControlledVocabulary$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        Allows for the identification of the member and optionally provides an index for the member within an ordered array 
#'
#'                              \code{myDDI4_ControlledVocabulary$contains <- list(myVocabularyEntryIndicator)} sets the 'contains' property of 'myDDI4_ControlledVocabulary' to a list of R6 VocabularyEntryIndicator class instances
#'
#'                              \code{myDDI4_ControlledVocabulary$contains} returns a list of of R6 : VocabularyEntryIndicator class instances}
#'   }
#'   \describe{
#'     \item{\code{isOrdered}}{ A read/write active binding.
#'        If members are ordered set to true, if unordered set to false.
#'
#'                              \code{myDDI4_ControlledVocabulary$isOrdered <- mylogical} sets the isOrdered property of 'myDDI4_ControlledVocabulary' to the vector of datatype:logical
#'
#'                              \code{myDDI4_ControlledVocabulary$isOrdered} returns a vector of datatype: logical}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isStructuredBy}}{ A read/write active binding.
#'          The RelationStructure defining hierarchy or specific realtionships between vocabulary entries in the vocabulary
#'
#'                              \code{myDDI4_ControlledVocabulary$isStructuredBy <- c(myVocabularyRelationStructureURN)} sets the association 'isStructuredBy' from 'myDDI4_ControlledVocabulary' to a character vector of DdiUrns for R6 VocabularyRelationStructure class instances
#'
#'                              \code{myDDI4_ControlledVocabulary$isStructuredBy} returns a character vector of of DdiUrns for R6  VocabularyRelationStructure class instances}
#'   }
#'   \describe{
#'     \item{\code{definingConcept}}{ A read/write active binding.
#'          The conceptual basis for the collection of members.
#'
#'                              \code{myDDI4_ControlledVocabulary$definingConcept <- c(myConceptURN)} sets the association 'definingConcept' from 'myDDI4_ControlledVocabulary' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_ControlledVocabulary$definingConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ControlledVocabulary$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ControlledVocabulary <- R6Class(classname="DDI4_ControlledVocabulary",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA,    isStructuredBy=NA,definingConcept=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ControlledVocabulary may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(type)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=type , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_VocabularyEntryIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(isOrdered)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=isOrdered , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(isStructuredBy)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_VocabularyRelationStructure", proposedUrnVector=isStructuredBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
      }
      if(!missing(definingConcept)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=definingConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 5 properties: 
    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_VocabularyEntryIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },

    isOrdered = function(val){
      if (!missing(val)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isOrdered
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    type = function(val){
      if (!missing(val)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.type
      }
    },


# There are 2 class association(s):
    definingConcept = function(val){
      if (!missing(val)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
       # invisible( private$.definingConcept)
      } else {
          private$.definingConcept
      }
    },

    isStructuredBy = function(val){
      if (!missing(val)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_VocabularyRelationStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
       # invisible( private$.isStructuredBy)
      } else {
          private$.isStructuredBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .type = NULL,
    .name = NULL,
    .purpose = NULL,
    .contains = NULL,
    .isOrdered = NULL,
    .isStructuredBy = NULL,
    .definingConcept = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_VocabularyEntryIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isStructuredBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_VocabularyRelationStructure", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ControlledVocabulary


#' @title DDI4_CorrespondenceTable  A Correspondence Table expresses relationships between the members within or between StatisticalClassifications
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CorrespondenceTable}{CorrespondenceTable} 

#'
#'   Definition
#'
#'   ============
#'
#'   A Correspondence Table expresses relationships between the members within or between StatisticalClassifications.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Correspondence between the U.S. Standard Industrial Classification (SIC) and North American Industrial Classification System (NAICS)
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   CorrespondenceTables are used with Statistical Classifications. For instance, it can relate two versions from the same Classification Series; Statistical Classifications from different Classification Series; a variant and the version on which it is based; or, different versions of a variant. In the first and last examples, the Correspondence Table facilitates comparability over time.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Correspondence Table
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_CorrespondenceTable$DdiUrn} returns the DDI URN of myDDI4_CorrespondenceTable.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{effectiveDates}}{ A read/write active binding.
#'        Effective period of validity of the CorrespondenceTable. The correspondence table expresses the relationships between the two NodeSets as they existed on the period specified in the table.
#'
#'                              \code{myDDI4_CorrespondenceTable$effectiveDates <- list(myDateRange)} sets the 'effectiveDates' property of 'myDDI4_CorrespondenceTable' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_CorrespondenceTable$effectiveDates} returns a list of of R6 : DateRange class instances}
#'   }
#'   \describe{
#'     \item{\code{correspondence}}{ A read/write active binding.
#'        Set of mappings between nodes that participate in the correspondence.
#'
#'                              \code{myDDI4_CorrespondenceTable$correspondence <- list(myMap)} sets the 'correspondence' property of 'myDDI4_CorrespondenceTable' to a list of R6 Map class instances
#'
#'                              \code{myDDI4_CorrespondenceTable$correspondence} returns a list of of R6 : Map class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{maps}}{ A read/write active binding.
#'          The Statistical Classification(s) from which the correspondence is made.
#'
#'                              \code{myDDI4_CorrespondenceTable$maps <- c(myStatisticalClassificationURN)} sets the association 'maps' from 'myDDI4_CorrespondenceTable' to a character vector of DdiUrns for R6 StatisticalClassification class instances
#'
#'                              \code{myDDI4_CorrespondenceTable$maps} returns a character vector of of DdiUrns for R6  StatisticalClassification class instances}
#'   }
#'   \describe{
#'     \item{\code{sourceLevel}}{ A read/write active binding.
#'          Level from which the correspondence is made. Correspondences might be restricted to a certain Level in the NodeSet. In this case, target items are assigned only to source items on the given level. If no level is indicated, source items can be assigned to any level of the target NodeSet.
#'
#'                              \code{myDDI4_CorrespondenceTable$sourceLevel <- c(myLevelStructureURN)} sets the association 'sourceLevel' from 'myDDI4_CorrespondenceTable' to a character vector of DdiUrns for R6 LevelStructure class instances
#'
#'                              \code{myDDI4_CorrespondenceTable$sourceLevel} returns a character vector of of DdiUrns for R6  LevelStructure class instances}
#'   }
#'   \describe{
#'     \item{\code{targetLevel}}{ A read/write active binding.
#'          Level to which the correspondence is made. Correspondences might be restricted to a certain Level in the NodeSet. In this case, target items are assigned only to source items on the given level. If no level is indicated, target items can be assigned to any level of the source NodeSet.
#'
#'                              \code{myDDI4_CorrespondenceTable$targetLevel <- c(myLevelStructureURN)} sets the association 'targetLevel' from 'myDDI4_CorrespondenceTable' to a character vector of DdiUrns for R6 LevelStructure class instances
#'
#'                              \code{myDDI4_CorrespondenceTable$targetLevel} returns a character vector of of DdiUrns for R6  LevelStructure class instances}
#'   }
#'   \describe{
#'     \item{\code{hasPublication}}{ A read/write active binding.
#'          A list of the publications in which the Correspondence Table has been published.
#'
#'                              \code{myDDI4_CorrespondenceTable$hasPublication <- c(myExternalMaterialURN)} sets the association 'hasPublication' from 'myDDI4_CorrespondenceTable' to a character vector of DdiUrns for R6 ExternalMaterial class instances
#'
#'                              \code{myDDI4_CorrespondenceTable$hasPublication} returns a character vector of of DdiUrns for R6  ExternalMaterial class instances}
#'   }
#'   \describe{
#'     \item{\code{owner}}{ A read/write active binding.
#'          The statistical office, other authority or section that created and maintains the Correspondence Table. A Correspondence Table may have several owners.
#'
#'                              \code{myDDI4_CorrespondenceTable$owner <- c(myAgentURN)} sets the association 'owner' from 'myDDI4_CorrespondenceTable' to a character vector of DdiUrns for R6 Agent class instances
#'
#'                              \code{myDDI4_CorrespondenceTable$owner} returns a character vector of of DdiUrns for R6  Agent class instances}
#'   }
#'   \describe{
#'     \item{\code{maintenanceUnit}}{ A read/write active binding.
#'          The unit or group of persons who are responsible for the Correspondence Table, i.e. for maintaining and updating it.
#'
#'                              \code{myDDI4_CorrespondenceTable$maintenanceUnit <- c(myAgentURN)} sets the association 'maintenanceUnit' from 'myDDI4_CorrespondenceTable' to a character vector of DdiUrns for R6 Agent class instances
#'
#'                              \code{myDDI4_CorrespondenceTable$maintenanceUnit} returns a character vector of of DdiUrns for R6  Agent class instances}
#'   }
#'   \describe{
#'     \item{\code{contactPerson}}{ A read/write active binding.
#'          The person(s) who may be contacted for additional information about the Correspondence Table. Can be an individual or organization.
#'
#'                              \code{myDDI4_CorrespondenceTable$contactPerson <- c(myAgentURN)} sets the association 'contactPerson' from 'myDDI4_CorrespondenceTable' to a character vector of DdiUrns for R6 Agent class instances
#'
#'                              \code{myDDI4_CorrespondenceTable$contactPerson} returns a character vector of of DdiUrns for R6  Agent class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CorrespondenceTable$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, effectiveDates=NA, correspondence=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CorrespondenceTable <- R6Class(classname="DDI4_CorrespondenceTable",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, effectiveDates=NA, correspondence=NA,    maps=NA,sourceLevel=NA,targetLevel=NA,hasPublication=NA,owner=NA,maintenanceUnit=NA,contactPerson=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CorrespondenceTable may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(effectiveDates)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=effectiveDates , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(correspondence)){
        private$.correspondence <- returnR6ObjectIfValid(validClass="DDI4_Map", proposedObjectList=correspondence , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(maps)){
        private$.maps <- returnUrnVectorIfValid(validClass="DDI4_StatisticalClassification", proposedUrnVector=maps , minimumCardinality=1, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.maps, referencingAssociationName="maps",  referencingUrn=thisUrn)  
      }
      if(!missing(sourceLevel)){
        private$.sourceLevel <- returnUrnVectorIfValid(validClass="DDI4_LevelStructure", proposedUrnVector=sourceLevel , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.sourceLevel, referencingAssociationName="sourceLevel",  referencingUrn=thisUrn)  
      }
      if(!missing(targetLevel)){
        private$.targetLevel <- returnUrnVectorIfValid(validClass="DDI4_LevelStructure", proposedUrnVector=targetLevel , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.targetLevel, referencingAssociationName="targetLevel",  referencingUrn=thisUrn)  
      }
      if(!missing(hasPublication)){
        private$.hasPublication <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=hasPublication , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasPublication, referencingAssociationName="hasPublication",  referencingUrn=thisUrn)  
      }
      if(!missing(owner)){
        private$.owner <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=owner , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.owner, referencingAssociationName="owner",  referencingUrn=thisUrn)  
      }
      if(!missing(maintenanceUnit)){
        private$.maintenanceUnit <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=maintenanceUnit , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.maintenanceUnit, referencingAssociationName="maintenanceUnit",  referencingUrn=thisUrn)  
      }
      if(!missing(contactPerson)){
        private$.contactPerson <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=contactPerson , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.contactPerson, referencingAssociationName="contactPerson",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 2 properties: 
    correspondence = function(val){
      if (!missing(val)){
        private$.correspondence <- returnR6ObjectIfValid(validClass="DDI4_Map", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.correspondence
      }
    },

    effectiveDates = function(val){
      if (!missing(val)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.effectiveDates
      }
    },


# There are 7 class association(s):
    contactPerson = function(val){
      if (!missing(val)){
        private$.contactPerson <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.contactPerson, referencingAssociationName="contactPerson",  referencingUrn=thisUrn)  
       # invisible( private$.contactPerson)
      } else {
          private$.contactPerson
      }
    },

    hasPublication = function(val){
      if (!missing(val)){
        private$.hasPublication <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasPublication, referencingAssociationName="hasPublication",  referencingUrn=thisUrn)  
       # invisible( private$.hasPublication)
      } else {
          private$.hasPublication
      }
    },

    maintenanceUnit = function(val){
      if (!missing(val)){
        private$.maintenanceUnit <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.maintenanceUnit, referencingAssociationName="maintenanceUnit",  referencingUrn=thisUrn)  
       # invisible( private$.maintenanceUnit)
      } else {
          private$.maintenanceUnit
      }
    },

    maps = function(val){
      if (!missing(val)){
        private$.maps <- returnUrnVectorIfValid(validClass="DDI4_StatisticalClassification", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.maps, referencingAssociationName="maps",  referencingUrn=thisUrn)  
       # invisible( private$.maps)
      } else {
          private$.maps
      }
    },

    owner = function(val){
      if (!missing(val)){
        private$.owner <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.owner, referencingAssociationName="owner",  referencingUrn=thisUrn)  
       # invisible( private$.owner)
      } else {
          private$.owner
      }
    },

    sourceLevel = function(val){
      if (!missing(val)){
        private$.sourceLevel <- returnUrnVectorIfValid(validClass="DDI4_LevelStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.sourceLevel, referencingAssociationName="sourceLevel",  referencingUrn=thisUrn)  
       # invisible( private$.sourceLevel)
      } else {
          private$.sourceLevel
      }
    },

    targetLevel = function(val){
      if (!missing(val)){
        private$.targetLevel <- returnUrnVectorIfValid(validClass="DDI4_LevelStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.targetLevel, referencingAssociationName="targetLevel",  referencingUrn=thisUrn)  
       # invisible( private$.targetLevel)
      } else {
          private$.targetLevel
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .effectiveDates = NULL,
    .correspondence = NULL,
    .maps = NULL,
    .sourceLevel = NULL,
    .targetLevel = NULL,
    .hasPublication = NULL,
    .owner = NULL,
    .maintenanceUnit = NULL,
    .contactPerson = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contactPerson = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Agent", 
          structure = "DdiUrn vector"
        ),
        correspondence = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Map", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        effectiveDates = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasPublication = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        maintenanceUnit = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Agent", 
          structure = "DdiUrn vector"
        ),
        maps = c(
          min="1", 
          max="Inf", 
          datatype = "DDI4_StatisticalClassification", 
          structure = "DdiUrn vector"
        ),
        owner = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Agent", 
          structure = "DdiUrn vector"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        sourceLevel = c(
          min="0", 
          max="1", 
          datatype = "DDI4_LevelStructure", 
          structure = "DdiUrn vector"
        ),
        targetLevel = c(
          min="0", 
          max="1", 
          datatype = "DDI4_LevelStructure", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_CorrespondenceTable


#' @title DDI4_CorrespondenceType  Describes the commonalities and differences between two members using a textual description of both commonalities and differences plus an optional coding of the type of commonality
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CorrespondenceType}{CorrespondenceType} 

#'
#'   Definition
#'
#'   ============
#'
#'   Describes the commonalities and differences between two members using a textual description of both commonalities and differences plus an optional coding of the type of commonality.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   cm:CorrespondenceType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_CorrespondenceType$DdiUrn} returns the DDI URN of myDDI4_CorrespondenceType.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{commonality}}{ A read/write active binding.
#'        A description of the common features of the two items using a StructuredString to support multiple language versions of the same content as well as optional formatting of the content.
#'
#'                              \code{myDDI4_CorrespondenceType$commonality <- list(myInternationalStructuredString)} sets the 'commonality' property of 'myDDI4_CorrespondenceType' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_CorrespondenceType$commonality} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{difference}}{ A read/write active binding.
#'        A description of the differences between the two items using a StructuredString to support multiple language versions of the same content as well as optional formatting of the content.
#'
#'                              \code{myDDI4_CorrespondenceType$difference <- list(myInternationalStructuredString)} sets the 'difference' property of 'myDDI4_CorrespondenceType' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_CorrespondenceType$difference} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{commonalityTypeCode}}{ A read/write active binding.
#'        Commonality expressed as a term or code. Supports the use of an external controlled vocabulary. If repeated, clarify each external controlled vocabulary used.
#'
#'                              \code{myDDI4_CorrespondenceType$commonalityTypeCode <- list(myExternalControlledVocabularyEntry)} sets the 'commonalityTypeCode' property of 'myDDI4_CorrespondenceType' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_CorrespondenceType$commonalityTypeCode} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMappingRelation}}{ A read/write active binding.
#'        Allows specification of exact match, close match, or disjoint. These relationships can be further defined by describing commonalities or differences or providing additional controlled vocabulary description of relationship.
#'
#'                              \code{myDDI4_CorrespondenceType$hasMappingRelation <- list(myMappingRelation)} sets the 'hasMappingRelation' property of 'myDDI4_CorrespondenceType' to a list of R6 MappingRelation class instances
#'
#'                              \code{myDDI4_CorrespondenceType$hasMappingRelation} returns a list of of R6 : MappingRelation class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CorrespondenceType$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, commonality=NA, difference=NA, commonalityTypeCode=NA, hasMappingRelation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CorrespondenceType <- R6Class(classname="DDI4_CorrespondenceType",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, commonality=NA, difference=NA, commonalityTypeCode=NA, hasMappingRelation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CorrespondenceType may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(commonality)){
        private$.commonality <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=commonality , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(difference)){
        private$.difference <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=difference , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(commonalityTypeCode)){
        private$.commonalityTypeCode <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=commonalityTypeCode , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(hasMappingRelation)){
        private$.hasMappingRelation <- returnEnumeratedValues(enumerationObject=DDI4_MappingRelation$new(registerThisObject=FALSE), proposedCharacterVector=hasMappingRelation , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    commonality = function(val){
      if (!missing(val)){
        private$.commonality <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.commonality
      }
    },

    commonalityTypeCode = function(val){
      if (!missing(val)){
        private$.commonalityTypeCode <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.commonalityTypeCode
      }
    },

    difference = function(val){
      if (!missing(val)){
        private$.difference <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.difference
      }
    },

    hasMappingRelation = function(val){
      if (!missing(val)){
        private$.hasMappingRelation <- returnEnumeratedValues(enumerationObject=DDI4_MappingRelation$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasMappingRelation
      }
    }


  ),

  private = list(
    .commonality = NULL,
    .difference = NULL,
    .commonalityTypeCode = NULL,
    .hasMappingRelation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        commonality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        commonalityTypeCode = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        difference = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasMappingRelation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_MappingRelation", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_CorrespondenceType


#' @title DDI4_Coverage  Coverage information for an annotated object
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Coverage}{Coverage} 

#'
#'   Definition
#'
#'   ============
#'
#'   Coverage information for an annotated object. Includes coverage information for temporal, topical, and spatial coverage. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A survey might have ask people about motels they stayed in (topical coverage) in the year 2015 (temporal coverage), while they were travelling in Kansas (spatial coverage). This is different than the temporal, and spatial attributes of the population studied – (international travelers to the US surveyed in 2017).
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Coverage is a container for the more specific temporal, spatial, and topical coverages to which it refers.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:CoverageType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Coverage$DdiUrn} returns the DDI URN of myDDI4_Coverage.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{overview}}{ A read/write active binding.
#'        A generic description including temporal, topical, and spatial coverage that is the equivalent of dc:coverage (the refinement base of dcterms:spatial and dcterms:temporal. Use specific coverage content for detailed information. Short natural language account of the information obtained from the combination of properties and relationships associated with an object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_Coverage$overview <- list(myInternationalStructuredString)} sets the 'overview' property of 'myDDI4_Coverage' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Coverage$overview} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasTemporalCoverage}}{ A read/write active binding.
#'          The dates and time periods described by the contents of the annotated object.
#'
#'                              \code{myDDI4_Coverage$hasTemporalCoverage <- c(myTemporalCoverageURN)} sets the association 'hasTemporalCoverage' from 'myDDI4_Coverage' to a character vector of DdiUrns for R6 TemporalCoverage class instances
#'
#'                              \code{myDDI4_Coverage$hasTemporalCoverage} returns a character vector of of DdiUrns for R6  TemporalCoverage class instances}
#'   }
#'   \describe{
#'     \item{\code{hasTopicalCoverage}}{ A read/write active binding.
#'          The topics covered by the contents of the annotated object. These may be expressed by subject classification systems and structured or unstructured keywords.
#'
#'                              \code{myDDI4_Coverage$hasTopicalCoverage <- c(myTopicalCoverageURN)} sets the association 'hasTopicalCoverage' from 'myDDI4_Coverage' to a character vector of DdiUrns for R6 TopicalCoverage class instances
#'
#'                              \code{myDDI4_Coverage$hasTopicalCoverage} returns a character vector of of DdiUrns for R6  TopicalCoverage class instances}
#'   }
#'   \describe{
#'     \item{\code{hasSpatialCoverage}}{ A read/write active binding.
#'          Description of the spatial (geographic) coverage of the contents of the annotated object.
#'
#'                              \code{myDDI4_Coverage$hasSpatialCoverage <- c(mySpatialCoverageURN)} sets the association 'hasSpatialCoverage' from 'myDDI4_Coverage' to a character vector of DdiUrns for R6 SpatialCoverage class instances
#'
#'                              \code{myDDI4_Coverage$hasSpatialCoverage} returns a character vector of of DdiUrns for R6  SpatialCoverage class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Coverage$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, overview=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Coverage <- R6Class(classname="DDI4_Coverage",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, overview=NA,    hasTemporalCoverage=NA,hasTopicalCoverage=NA,hasSpatialCoverage=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Coverage may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(overview)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=overview , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(hasTemporalCoverage)){
        private$.hasTemporalCoverage <- returnUrnVectorIfValid(validClass="DDI4_TemporalCoverage", proposedUrnVector=hasTemporalCoverage , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasTemporalCoverage, referencingAssociationName="hasTemporalCoverage",  referencingUrn=thisUrn)  
      }
      if(!missing(hasTopicalCoverage)){
        private$.hasTopicalCoverage <- returnUrnVectorIfValid(validClass="DDI4_TopicalCoverage", proposedUrnVector=hasTopicalCoverage , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasTopicalCoverage, referencingAssociationName="hasTopicalCoverage",  referencingUrn=thisUrn)  
      }
      if(!missing(hasSpatialCoverage)){
        private$.hasSpatialCoverage <- returnUrnVectorIfValid(validClass="DDI4_SpatialCoverage", proposedUrnVector=hasSpatialCoverage , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasSpatialCoverage, referencingAssociationName="hasSpatialCoverage",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    overview = function(val){
      if (!missing(val)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overview
      }
    },


# There are 3 class association(s):
    hasSpatialCoverage = function(val){
      if (!missing(val)){
        private$.hasSpatialCoverage <- returnUrnVectorIfValid(validClass="DDI4_SpatialCoverage", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasSpatialCoverage, referencingAssociationName="hasSpatialCoverage",  referencingUrn=thisUrn)  
       # invisible( private$.hasSpatialCoverage)
      } else {
          private$.hasSpatialCoverage
      }
    },

    hasTemporalCoverage = function(val){
      if (!missing(val)){
        private$.hasTemporalCoverage <- returnUrnVectorIfValid(validClass="DDI4_TemporalCoverage", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasTemporalCoverage, referencingAssociationName="hasTemporalCoverage",  referencingUrn=thisUrn)  
       # invisible( private$.hasTemporalCoverage)
      } else {
          private$.hasTemporalCoverage
      }
    },

    hasTopicalCoverage = function(val){
      if (!missing(val)){
        private$.hasTopicalCoverage <- returnUrnVectorIfValid(validClass="DDI4_TopicalCoverage", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasTopicalCoverage, referencingAssociationName="hasTopicalCoverage",  referencingUrn=thisUrn)  
       # invisible( private$.hasTopicalCoverage)
      } else {
          private$.hasTopicalCoverage
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .overview = NULL,
    .hasTemporalCoverage = NULL,
    .hasTopicalCoverage = NULL,
    .hasSpatialCoverage = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasSpatialCoverage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SpatialCoverage", 
          structure = "DdiUrn vector"
        ),
        hasTemporalCoverage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TemporalCoverage", 
          structure = "DdiUrn vector"
        ),
        hasTopicalCoverage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TopicalCoverage", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Coverage


#' @title DDI4_CustomInstance  A set of CustomValues to be attached to some object
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CustomInstance}{CustomInstance} 

#'
#'   Definition
#'
#'   ============
#'
#'   A set of CustomValues to be attached to some object.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A set of OMB required information about a question.
#'
#'   
#'
#'   The set of openEHR protocol information to be attached to a blood pressure Capture
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_CustomInstance$DdiUrn} returns the DDI URN of myDDI4_CustomInstance.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        see Collection
#'
#'                              \code{myDDI4_CustomInstance$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_CustomInstance' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_CustomInstance$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        see Collection
#'
#'                              \code{myDDI4_CustomInstance$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_CustomInstance' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_CustomInstance$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{type}}{ A read/write active binding.
#'        see Collection
#'
#'                              \code{myDDI4_CustomInstance$type <- list(myCollectionType)} sets the 'type' property of 'myDDI4_CustomInstance' to a list of R6 CollectionType class instances
#'
#'                              \code{myDDI4_CustomInstance$type} returns a list of of R6 : CollectionType class instances}
#'   }
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        Allows for the identification of the member and optionally provides an index for the member within an ordered array 
#'
#'                              \code{myDDI4_CustomInstance$contains <- list(myCustomValueIndicator)} sets the 'contains' property of 'myDDI4_CustomInstance' to a list of R6 CustomValueIndicator class instances
#'
#'                              \code{myDDI4_CustomInstance$contains} returns a list of of R6 : CustomValueIndicator class instances}
#'   }
#'   \describe{
#'     \item{\code{isOrdered}}{ A read/write active binding.
#'        If members are ordered set to true, if unordered set to false. 
#'
#'                              \code{myDDI4_CustomInstance$isOrdered <- mylogical} sets the isOrdered property of 'myDDI4_CustomInstance' to the vector of datatype:logical
#'
#'                              \code{myDDI4_CustomInstance$isOrdered} returns a vector of datatype: logical}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{correspondsTo}}{ A read/write active binding.
#'          Is realted to a CustomStructure
#'
#'                              \code{myDDI4_CustomInstance$correspondsTo <- c(myCustomStructureURN)} sets the association 'correspondsTo' from 'myDDI4_CustomInstance' to a character vector of DdiUrns for R6 CustomStructure class instances
#'
#'                              \code{myDDI4_CustomInstance$correspondsTo} returns a character vector of of DdiUrns for R6  CustomStructure class instances}
#'   }
#'   \describe{
#'     \item{\code{definingConcept}}{ A read/write active binding.
#'          The conceptual basis for the collection of members.
#'
#'                              \code{myDDI4_CustomInstance$definingConcept <- c(myConceptURN)} sets the association 'definingConcept' from 'myDDI4_CustomInstance' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_CustomInstance$definingConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CustomInstance$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, name=NA, purpose=NA, type=NA, contains=NA, isOrdered=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CustomInstance <- R6Class(classname="DDI4_CustomInstance",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, name=NA, purpose=NA, type=NA, contains=NA, isOrdered=NA,    correspondsTo=NA,definingConcept=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CustomInstance may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(type)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=type , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_CustomValueIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(isOrdered)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=isOrdered , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(correspondsTo)){
        private$.correspondsTo <- returnUrnVectorIfValid(validClass="DDI4_CustomStructure", proposedUrnVector=correspondsTo , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.correspondsTo, referencingAssociationName="correspondsTo",  referencingUrn=thisUrn)  
      }
      if(!missing(definingConcept)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=definingConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 5 properties: 
    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_CustomValueIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },

    isOrdered = function(val){
      if (!missing(val)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isOrdered
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    type = function(val){
      if (!missing(val)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.type
      }
    },


# There are 2 class association(s):
    correspondsTo = function(val){
      if (!missing(val)){
        private$.correspondsTo <- returnUrnVectorIfValid(validClass="DDI4_CustomStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.correspondsTo, referencingAssociationName="correspondsTo",  referencingUrn=thisUrn)  
       # invisible( private$.correspondsTo)
      } else {
          private$.correspondsTo
      }
    },

    definingConcept = function(val){
      if (!missing(val)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
       # invisible( private$.definingConcept)
      } else {
          private$.definingConcept
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .name = NULL,
    .purpose = NULL,
    .type = NULL,
    .contains = NULL,
    .isOrdered = NULL,
    .correspondsTo = NULL,
    .definingConcept = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_CustomValueIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        correspondsTo = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CustomStructure", 
          structure = "DdiUrn vector"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_CustomInstance


#' @title DDI4_CustomItem  A custom item description
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CustomItem}{CustomItem} 

#'
#'   Definition
#'
#'   ============
#'
#'   A custom item description. This allows the definition of an item which is a member of a CustomStructure.
#'
#'   It defines the minimum and maximum number of occurrences and representation of a CustomValue.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   DegreeOfBurden for a question as required by the U.S. OMB.
#'
#'   
#'
#'   BloodPressureCuffSize - the size of a blood pressure cuff specified for a particular protocol.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   This definition will be referenced by a CustomValue when recording a key,value instance
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_CustomItem$DdiUrn} returns the DDI URN of myDDI4_CustomItem.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{maxOccurs}}{ A read/write active binding.
#'        The maximum number of occurrences of an associated CustomValue
#'
#'                              \code{myDDI4_CustomItem$maxOccurs <- myinteger} sets the maxOccurs property of 'myDDI4_CustomItem' to the vector of datatype:integer
#'
#'                              \code{myDDI4_CustomItem$maxOccurs} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{minOccurs}}{ A read/write active binding.
#'        The minimum number of occurrences of an associated CustomValue
#'
#'                              \code{myDDI4_CustomItem$minOccurs <- myinteger} sets the minOccurs property of 'myDDI4_CustomItem' to the vector of datatype:integer
#'
#'                              \code{myDDI4_CustomItem$minOccurs} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{key}}{ A read/write active binding.
#'        The key to be used by a key,value pair
#'
#'                              \code{myDDI4_CustomItem$key <- mycharacter} sets the key property of 'myDDI4_CustomItem' to the vector of datatype:character
#'
#'                              \code{myDDI4_CustomItem$key} returns a vector of datatype: character}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{uses}}{ A read/write active binding.
#'          The type of value for the key.value pair
#'
#'                              \code{myDDI4_CustomItem$uses <- c(myValueDomainURN)} sets the association 'uses' from 'myDDI4_CustomItem' to a character vector of DdiUrns for R6 ValueDomain class instances
#'
#'                              \code{myDDI4_CustomItem$uses} returns a character vector of of DdiUrns for R6  ValueDomain class instances}
#'   }
#'   \describe{
#'     \item{\code{hasConcept}}{ A read/write active binding.
#'          A Concept associated with the key of a CustomValue
#'
#'                              \code{myDDI4_CustomItem$hasConcept <- c(myConceptURN)} sets the association 'hasConcept' from 'myDDI4_CustomItem' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_CustomItem$hasConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#'   \describe{
#'     \item{\code{comparesTo}}{ A read/write active binding.
#'          This optional relationship indicates a RepresentedVariable compatible with this key. Custom metadata might be reused as data and data might be used as metadata. This mechanism could enable these transformations.
#'
#'                              \code{myDDI4_CustomItem$comparesTo <- c(myRepresentedVariableURN)} sets the association 'comparesTo' from 'myDDI4_CustomItem' to a character vector of DdiUrns for R6 RepresentedVariable class instances
#'
#'                              \code{myDDI4_CustomItem$comparesTo} returns a character vector of of DdiUrns for R6  RepresentedVariable class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CustomItem$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, maxOccurs=NA, minOccurs=NA, key=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CustomItem <- R6Class(classname="DDI4_CustomItem",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, maxOccurs=NA, minOccurs=NA, key=NA,    uses=NA,hasConcept=NA,comparesTo=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CustomItem may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(maxOccurs)){
        private$.maxOccurs <- returnVectorIfValid(validVectorClass="integer", proposedVector=maxOccurs , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(minOccurs)){
        private$.minOccurs <- returnVectorIfValid(validVectorClass="integer", proposedVector=minOccurs , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(key)){
        private$.key <- returnVectorIfValid(validVectorClass="character", proposedVector=key , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(uses)){
        private$.uses <- returnUrnVectorIfValid(validClass="DDI4_ValueDomain", proposedUrnVector=uses , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.uses, referencingAssociationName="uses",  referencingUrn=thisUrn)  
      }
      if(!missing(hasConcept)){
        private$.hasConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=hasConcept , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasConcept, referencingAssociationName="hasConcept",  referencingUrn=thisUrn)  
      }
      if(!missing(comparesTo)){
        private$.comparesTo <- returnUrnVectorIfValid(validClass="DDI4_RepresentedVariable", proposedUrnVector=comparesTo , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.comparesTo, referencingAssociationName="comparesTo",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    key = function(val){
      if (!missing(val)){
        private$.key <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.key
      }
    },

    maxOccurs = function(val){
      if (!missing(val)){
        private$.maxOccurs <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.maxOccurs
      }
    },

    minOccurs = function(val){
      if (!missing(val)){
        private$.minOccurs <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.minOccurs
      }
    },


# There are 3 class association(s):
    comparesTo = function(val){
      if (!missing(val)){
        private$.comparesTo <- returnUrnVectorIfValid(validClass="DDI4_RepresentedVariable", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.comparesTo, referencingAssociationName="comparesTo",  referencingUrn=thisUrn)  
       # invisible( private$.comparesTo)
      } else {
          private$.comparesTo
      }
    },

    hasConcept = function(val){
      if (!missing(val)){
        private$.hasConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasConcept, referencingAssociationName="hasConcept",  referencingUrn=thisUrn)  
       # invisible( private$.hasConcept)
      } else {
          private$.hasConcept
      }
    },

    uses = function(val){
      if (!missing(val)){
        private$.uses <- returnUrnVectorIfValid(validClass="DDI4_ValueDomain", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.uses, referencingAssociationName="uses",  referencingUrn=thisUrn)  
       # invisible( private$.uses)
      } else {
          private$.uses
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .maxOccurs = NULL,
    .minOccurs = NULL,
    .key = NULL,
    .uses = NULL,
    .hasConcept = NULL,
    .comparesTo = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        comparesTo = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RepresentedVariable", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasConcept = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        key = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        maxOccurs = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        minOccurs = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        uses = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ValueDomain", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_CustomItem


#' @title DDI4_CustomItemIndicator  Member Indicator for use with member type CustomItem
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CustomItemIndicator}{CustomItemIndicator} 

#'
#'   Definition
#'
#'   ============
#'
#'   Member Indicator for use with member type CustomItem
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_CustomItemIndicator$DdiUrn} returns the DDI URN of myDDI4_CustomItemIndicator.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Index value of member in an ordered array
#'
#'                              \code{myDDI4_CustomItemIndicator$index <- myinteger} sets the index property of 'myDDI4_CustomItemIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_CustomItemIndicator$index} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{member}}{ A read/write active binding.
#'          Restricts member target class to CustomItem
#'
#'                              \code{myDDI4_CustomItemIndicator$member <- c(myCustomItemURN)} sets the association 'member' from 'myDDI4_CustomItemIndicator' to a character vector of DdiUrns for R6 CustomItem class instances
#'
#'                              \code{myDDI4_CustomItemIndicator$member} returns a character vector of of DdiUrns for R6  CustomItem class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CustomItemIndicator$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CustomItemIndicator <- R6Class(classname="DDI4_CustomItemIndicator",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA,    member=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CustomItemIndicator may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(member)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_CustomItem", proposedUrnVector=member , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.index
      }
    },


# There are 1 class association(s):
    member = function(val){
      if (!missing(val)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_CustomItem", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
       # invisible( private$.member)
      } else {
          private$.member
      }
    }

  ),

  private = list(
    .index = NULL,
    .member = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        index = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        member = c(
          min="1", 
          max="1", 
          datatype = "DDI4_CustomItem", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_CustomItemIndicator


#' @title DDI4_CustomItemRelation  Defines a relationship between CustomItems 
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CustomItemRelation}{CustomItemRelation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Defines a relationship between CustomItems .
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_CustomItemRelation$DdiUrn} returns the DDI URN of myDDI4_CustomItemRelation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        The type of relationship between the two CustomItems. The verb in the sentence describing the relationship between the source (subject) and the target (object).
#'
#'                              \code{myDDI4_CustomItemRelation$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_CustomItemRelation' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_CustomItemRelation$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_CustomItemRelation$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_CustomItemRelation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_CustomItemRelation$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_CustomItemRelation$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_CustomItemRelation' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_CustomItemRelation$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          Restricts source object to a CustomItem
#'
#'                              \code{myDDI4_CustomItemRelation$source <- c(myCustomItemURN)} sets the association 'source' from 'myDDI4_CustomItemRelation' to a character vector of DdiUrns for R6 CustomItem class instances
#'
#'                              \code{myDDI4_CustomItemRelation$source} returns a character vector of of DdiUrns for R6  CustomItem class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          Restricts target object to a CustomItem
#'
#'                              \code{myDDI4_CustomItemRelation$target <- c(myCustomItemURN)} sets the association 'target' from 'myDDI4_CustomItemRelation' to a character vector of DdiUrns for R6 CustomItem class instances
#'
#'                              \code{myDDI4_CustomItemRelation$target} returns a character vector of of DdiUrns for R6  CustomItem class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CustomItemRelation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CustomItemRelation <- R6Class(classname="DDI4_CustomItemRelation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CustomItemRelation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_CustomItem", proposedUrnVector=source , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_CustomItem", proposedUrnVector=target , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_CustomItem", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_CustomItem", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="1", 
          max="1", 
          datatype = "DDI4_CustomItem", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_CustomItem", 
          structure = "DdiUrn vector"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_CustomItemRelation


#' @title DDI4_CustomItemRelationStructure  Contains a set of CustomItemRelations which together define the relationships of Custom Structure
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CustomItemRelationStructure}{CustomItemRelationStructure} 

#'
#'   Definition
#'
#'   ============
#'
#'   Contains a set of CustomItemRelations which together define the relationships of Custom Structure.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_CustomItemRelationStructure$DdiUrn} returns the DDI URN of myDDI4_CustomItemRelationStructure.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{criteria}}{ A read/write active binding.
#'        Intentional definition of the order criteria (e.g. alphabetical, numerical, increasing, decreasing, etc.)
#'
#'                              \code{myDDI4_CustomItemRelationStructure$criteria <- list(myInternationalStructuredString)} sets the 'criteria' property of 'myDDI4_CustomItemRelationStructure' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_CustomItemRelationStructure$criteria} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A display label for the RelationStructure. May be expressed in multiple languages. Repeat for labels with different content, for example, labels with differing length limitations.
#'
#'                              \code{myDDI4_CustomItemRelationStructure$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_CustomItemRelationStructure' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_CustomItemRelationStructure$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#'   \describe{
#'     \item{\code{usage}}{ A read/write active binding.
#'        Explanation of the ways in which some decision or object is employed. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_CustomItemRelationStructure$usage <- list(myInternationalStructuredString)} sets the 'usage' property of 'myDDI4_CustomItemRelationStructure' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_CustomItemRelationStructure$usage} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an ennumeration list.  Use if all relations within this relation structure are of the same specification. .
#'
#'                              \code{myDDI4_CustomItemRelationStructure$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_CustomItemRelationStructure' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_CustomItemRelationStructure$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_CustomItemRelationStructure$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_CustomItemRelationStructure' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_CustomItemRelationStructure$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_CustomItemRelationStructure$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_CustomItemRelationStructure' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_CustomItemRelationStructure$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMemberRelation}}{ A read/write active binding.
#'        Member relationships that comprise the relationship structure
#'
#'                              \code{myDDI4_CustomItemRelationStructure$hasMemberRelation <- list(myCustomItemRelation)} sets the 'hasMemberRelation' property of 'myDDI4_CustomItemRelationStructure' to a list of R6 CustomItemRelation class instances
#'
#'                              \code{myDDI4_CustomItemRelationStructure$hasMemberRelation} returns a list of of R6 : CustomItemRelation class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CustomItemRelationStructure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, criteria=NA, displayLabel=NA, usage=NA, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CustomItemRelationStructure <- R6Class(classname="DDI4_CustomItemRelationStructure",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, criteria=NA, displayLabel=NA, usage=NA, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CustomItemRelationStructure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(criteria)){
        private$.criteria <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=criteria , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(usage)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=usage , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasMemberRelation)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_CustomItemRelation", proposedObjectList=hasMemberRelation , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 7 properties: 
    criteria = function(val){
      if (!missing(val)){
        private$.criteria <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.criteria
      }
    },

    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },

    hasMemberRelation = function(val){
      if (!missing(val)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_CustomItemRelation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasMemberRelation
      }
    },

    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },

    usage = function(val){
      if (!missing(val)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.usage
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .criteria = NULL,
    .displayLabel = NULL,
    .usage = NULL,
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .hasMemberRelation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        criteria = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasMemberRelation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_CustomItemRelation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasRelationSpecification = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        ),
        usage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_CustomItemRelationStructure


#' @title DDI4_CustomStructure  A Collection containing a set of item descriptions defining a structure for a set of key,value pairs
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CustomStructure}{CustomStructure} 

#'
#'   Definition
#'
#'   ============
#'
#'   A Collection containing a set of item descriptions defining a structure for a set of key,value pairs
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   The set of CustomItems which are to be used to document the OMB required information for a question.
#'
#'   
#'
#'   The set of CustomItems used to describe an openEHR blood pressure protocol.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_CustomStructure$DdiUrn} returns the DDI URN of myDDI4_CustomStructure.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{type}}{ A read/write active binding.
#'        Whether the collection is a bag or a set: a bag is a collection with duplicates allowed, a set is a collection without duplicates.
#'
#'                              \code{myDDI4_CustomStructure$type <- list(myCollectionType)} sets the 'type' property of 'myDDI4_CustomStructure' to a list of R6 CollectionType class instances
#'
#'                              \code{myDDI4_CustomStructure$type} returns a list of of R6 : CollectionType class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage.
#'
#'                              \code{myDDI4_CustomStructure$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_CustomStructure' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_CustomStructure$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_CustomStructure$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_CustomStructure' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_CustomStructure$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        Allows for the identification of the member and optionally provides an index for the member within an ordered array 
#'
#'                              \code{myDDI4_CustomStructure$contains <- list(myCustomItemIndicator)} sets the 'contains' property of 'myDDI4_CustomStructure' to a list of R6 CustomItemIndicator class instances
#'
#'                              \code{myDDI4_CustomStructure$contains} returns a list of of R6 : CustomItemIndicator class instances}
#'   }
#'   \describe{
#'     \item{\code{isOrdered}}{ A read/write active binding.
#'        If members are ordered set to true, if unordered set to false.
#'
#'                              \code{myDDI4_CustomStructure$isOrdered <- mylogical} sets the isOrdered property of 'myDDI4_CustomStructure' to the vector of datatype:logical
#'
#'                              \code{myDDI4_CustomStructure$isOrdered} returns a vector of datatype: logical}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isStructuredBy}}{ A read/write active binding.
#'          Description of a complex structure for the Collection. A collection may be structured in more than one way for different uses.
#'
#'                              \code{myDDI4_CustomStructure$isStructuredBy <- c(myCustomItemRelationStructureURN)} sets the association 'isStructuredBy' from 'myDDI4_CustomStructure' to a character vector of DdiUrns for R6 CustomItemRelationStructure class instances
#'
#'                              \code{myDDI4_CustomStructure$isStructuredBy} returns a character vector of of DdiUrns for R6  CustomItemRelationStructure class instances}
#'   }
#'   \describe{
#'     \item{\code{definingConcept}}{ A read/write active binding.
#'          The conceptual basis for the collection of members.
#'
#'                              \code{myDDI4_CustomStructure$definingConcept <- c(myConceptURN)} sets the association 'definingConcept' from 'myDDI4_CustomStructure' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_CustomStructure$definingConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CustomStructure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CustomStructure <- R6Class(classname="DDI4_CustomStructure",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA,    isStructuredBy=NA,definingConcept=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CustomStructure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(type)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=type , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_CustomItemIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(isOrdered)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=isOrdered , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(isStructuredBy)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_CustomItemRelationStructure", proposedUrnVector=isStructuredBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
      }
      if(!missing(definingConcept)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=definingConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 5 properties: 
    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_CustomItemIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },

    isOrdered = function(val){
      if (!missing(val)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isOrdered
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    type = function(val){
      if (!missing(val)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.type
      }
    },


# There are 2 class association(s):
    definingConcept = function(val){
      if (!missing(val)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
       # invisible( private$.definingConcept)
      } else {
          private$.definingConcept
      }
    },

    isStructuredBy = function(val){
      if (!missing(val)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_CustomItemRelationStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
       # invisible( private$.isStructuredBy)
      } else {
          private$.isStructuredBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .type = NULL,
    .name = NULL,
    .purpose = NULL,
    .contains = NULL,
    .isOrdered = NULL,
    .isStructuredBy = NULL,
    .definingConcept = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_CustomItemIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isStructuredBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_CustomItemRelationStructure", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_CustomStructure


#' @title DDI4_CustomValue  An instance of a  key, value pair for a  particular  key
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CustomValue}{CustomValue} 

#'
#'   Definition
#'
#'   ============
#'
#'   An instance of a  key, value pair for a  particular  key.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   OMBBurden,"High"
#'
#'   
#'
#'   openEHRBPposition, "upper arm"
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_CustomValue$DdiUrn} returns the DDI URN of myDDI4_CustomValue.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{value}}{ A read/write active binding.
#'        the value associated with a particular key expressed as a string. The ultimate value representation is defined by the corresponding CustomValue.
#'
#'                              \code{myDDI4_CustomValue$value <- list(myValueString)} sets the 'value' property of 'myDDI4_CustomValue' to a list of R6 ValueString class instances
#'
#'                              \code{myDDI4_CustomValue$value} returns a list of of R6 : ValueString class instances}
#'   }
#'   \describe{
#'     \item{\code{key}}{ A read/write active binding.
#'        The unique String identifying the value
#'
#'                              \code{myDDI4_CustomValue$key <- mycharacter} sets the key property of 'myDDI4_CustomValue' to the vector of datatype:character
#'
#'                              \code{myDDI4_CustomValue$key} returns a vector of datatype: character}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{correspondsTo}}{ A read/write active binding.
#'          The CustomItem which defines the key for this key,value pair, It also defines a ValueRepresentation
#'
#'                              \code{myDDI4_CustomValue$correspondsTo <- c(myCustomItemURN)} sets the association 'correspondsTo' from 'myDDI4_CustomValue' to a character vector of DdiUrns for R6 CustomItem class instances
#'
#'                              \code{myDDI4_CustomValue$correspondsTo} returns a character vector of of DdiUrns for R6  CustomItem class instances}
#'   }
#'   \describe{
#'     \item{\code{comparesTo}}{ A read/write active binding.
#'          This optional relationship indicates an InstanceVariable compatible with the value associated with this key. Custom metadata might be reused as data and data might be used as metadata. This mechanism could enable these transformations. The related InstanceVariable could also carry information about units of measurement, missing values etc.
#'
#'                              \code{myDDI4_CustomValue$comparesTo <- c(myInstanceVariableURN)} sets the association 'comparesTo' from 'myDDI4_CustomValue' to a character vector of DdiUrns for R6 InstanceVariable class instances
#'
#'                              \code{myDDI4_CustomValue$comparesTo} returns a character vector of of DdiUrns for R6  InstanceVariable class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CustomValue$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, value=NA, key=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CustomValue <- R6Class(classname="DDI4_CustomValue",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, value=NA, key=NA,    correspondsTo=NA,comparesTo=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CustomValue may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(value)){
        private$.value <- returnR6ObjectIfValid(validClass="DDI4_ValueString", proposedObjectList=value , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(key)){
        private$.key <- returnVectorIfValid(validVectorClass="character", proposedVector=key , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(correspondsTo)){
        private$.correspondsTo <- returnUrnVectorIfValid(validClass="DDI4_CustomItem", proposedUrnVector=correspondsTo , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.correspondsTo, referencingAssociationName="correspondsTo",  referencingUrn=thisUrn)  
      }
      if(!missing(comparesTo)){
        private$.comparesTo <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=comparesTo , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.comparesTo, referencingAssociationName="comparesTo",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 2 properties: 
    key = function(val){
      if (!missing(val)){
        private$.key <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.key
      }
    },

    value = function(val){
      if (!missing(val)){
        private$.value <- returnR6ObjectIfValid(validClass="DDI4_ValueString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.value
      }
    },


# There are 2 class association(s):
    comparesTo = function(val){
      if (!missing(val)){
        private$.comparesTo <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.comparesTo, referencingAssociationName="comparesTo",  referencingUrn=thisUrn)  
       # invisible( private$.comparesTo)
      } else {
          private$.comparesTo
      }
    },

    correspondsTo = function(val){
      if (!missing(val)){
        private$.correspondsTo <- returnUrnVectorIfValid(validClass="DDI4_CustomItem", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.correspondsTo, referencingAssociationName="correspondsTo",  referencingUrn=thisUrn)  
       # invisible( private$.correspondsTo)
      } else {
          private$.correspondsTo
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .value = NULL,
    .key = NULL,
    .correspondsTo = NULL,
    .comparesTo = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        comparesTo = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InstanceVariable", 
          structure = "DdiUrn vector"
        ),
        correspondsTo = c(
          min="1", 
          max="1", 
          datatype = "DDI4_CustomItem", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        key = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        value = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ValueString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_CustomValue


#' @title DDI4_CustomValueIndicator  Member Indicator for use with member type CustomValue
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/CustomValueIndicator}{CustomValueIndicator} 

#'
#'   Definition
#'
#'   ============
#'
#'   Member Indicator for use with member type CustomValue
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_CustomValueIndicator$DdiUrn} returns the DDI URN of myDDI4_CustomValueIndicator.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Index value of member in an ordered array
#'
#'                              \code{myDDI4_CustomValueIndicator$index <- myinteger} sets the index property of 'myDDI4_CustomValueIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_CustomValueIndicator$index} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{member}}{ A read/write active binding.
#'          Restricts member target class to CustomValue
#'
#'                              \code{myDDI4_CustomValueIndicator$member <- c(myCustomValueURN)} sets the association 'member' from 'myDDI4_CustomValueIndicator' to a character vector of DdiUrns for R6 CustomValue class instances
#'
#'                              \code{myDDI4_CustomValueIndicator$member} returns a character vector of of DdiUrns for R6  CustomValue class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_CustomValueIndicator$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_CustomValueIndicator <- R6Class(classname="DDI4_CustomValueIndicator",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA,    member=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_CustomValueIndicator may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(member)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_CustomValue", proposedUrnVector=member , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.index
      }
    },


# There are 1 class association(s):
    member = function(val){
      if (!missing(val)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_CustomValue", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
       # invisible( private$.member)
      } else {
          private$.member
      }
    }

  ),

  private = list(
    .index = NULL,
    .member = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        index = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        member = c(
          min="1", 
          max="1", 
          datatype = "DDI4_CustomValue", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_CustomValueIndicator


#' @title DDI4_DataPipeline  A DataPipeline is a single traversal of the Generic Longitudinal Business Model (GLBPM) and/or the Generic Statistical Business process Model (GSBPM) in the course of a study where a study is either one off or a wave in a StudySeries
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/DataPipeline}{DataPipeline} 

#'
#'   Definition
#'
#'   ============
#'
#'   A DataPipeline is a single traversal of the Generic Longitudinal Business Model (GLBPM) and/or the Generic Statistical Business process Model (GSBPM) in the course of a study where a study is either one off or a wave in a StudySeries.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   In a study where the study is a wave in a StudySeries, we do a single traversal of the Generic Longitudinal Business Process Model. From one wave to the next the traversal may be different. Each traversal is described in a DataPipeline.
#'
#'   
#'
#'   Extract Transform and Load (ETL) platforms support data pipelines to move data between systems. Using an ETL platform, data engineers create data pipelines to orchestrate the movement, transformation, validation, and loading of data, from source to final destination. The DataPipeline describes this "orchestration".
#'
#'   
#'
#'   A prospective DataPipeline gives guidance to data engineers. It is a design pattern. A retrospective DataPipeline documents an ETL data pipeline after the fact.
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   Traversal, Data Lifecycle
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_DataPipeline$DdiUrn} returns the DDI URN of myDDI4_DataPipeline.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{type}}{ A read/write active binding.
#'        Whether the collection is a bag or a set: a bag is a collection with duplicates allowed, a set is a collection without duplicates.
#'
#'                              \code{myDDI4_DataPipeline$type <- list(myCollectionType)} sets the 'type' property of 'myDDI4_DataPipeline' to a list of R6 CollectionType class instances
#'
#'                              \code{myDDI4_DataPipeline$type} returns a list of of R6 : CollectionType class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage.
#'
#'                              \code{myDDI4_DataPipeline$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_DataPipeline' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_DataPipeline$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_DataPipeline$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_DataPipeline' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_DataPipeline$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        Specifies BusinessProcess that participates in the DataPipeline
#'
#'                              \code{myDDI4_DataPipeline$contains <- list(myBusinessProcessIndicator)} sets the 'contains' property of 'myDDI4_DataPipeline' to a list of R6 BusinessProcessIndicator class instances
#'
#'                              \code{myDDI4_DataPipeline$contains} returns a list of of R6 : BusinessProcessIndicator class instances}
#'   }
#'   \describe{
#'     \item{\code{isOrdered}}{ A read/write active binding.
#'        If members are ordered set to true, if unordered set to false.
#'
#'                              \code{myDDI4_DataPipeline$isOrdered <- mylogical} sets the isOrdered property of 'myDDI4_DataPipeline' to the vector of datatype:logical
#'
#'                              \code{myDDI4_DataPipeline$isOrdered} returns a vector of datatype: logical}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{definingConcept}}{ A read/write active binding.
#'          The conceptual basis for the collection of members.
#'
#'                              \code{myDDI4_DataPipeline$definingConcept <- c(myConceptURN)} sets the association 'definingConcept' from 'myDDI4_DataPipeline' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_DataPipeline$definingConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#'   \describe{
#'     \item{\code{isInStudy}}{ A read/write active binding.
#'          A study has at most one DataPipeline. The same DataPipeline can be used in many studies.
#'
#'                              \code{myDDI4_DataPipeline$isInStudy <- c(myStudyURN)} sets the association 'isInStudy' from 'myDDI4_DataPipeline' to a character vector of DdiUrns for R6 Study class instances
#'
#'                              \code{myDDI4_DataPipeline$isInStudy} returns a character vector of of DdiUrns for R6  Study class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_DataPipeline$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_DataPipeline <- R6Class(classname="DDI4_DataPipeline",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA,    definingConcept=NA,isInStudy=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_DataPipeline may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(type)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=type , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_BusinessProcessIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(isOrdered)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=isOrdered , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(definingConcept)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=definingConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
      }
      if(!missing(isInStudy)){
        private$.isInStudy <- returnUrnVectorIfValid(validClass="DDI4_Study", proposedUrnVector=isInStudy , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isInStudy, referencingAssociationName="isInStudy",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 5 properties: 
    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_BusinessProcessIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },

    isOrdered = function(val){
      if (!missing(val)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isOrdered
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    type = function(val){
      if (!missing(val)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.type
      }
    },


# There are 2 class association(s):
    definingConcept = function(val){
      if (!missing(val)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
       # invisible( private$.definingConcept)
      } else {
          private$.definingConcept
      }
    },

    isInStudy = function(val){
      if (!missing(val)){
        private$.isInStudy <- returnUrnVectorIfValid(validClass="DDI4_Study", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isInStudy, referencingAssociationName="isInStudy",  referencingUrn=thisUrn)  
       # invisible( private$.isInStudy)
      } else {
          private$.isInStudy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .type = NULL,
    .name = NULL,
    .purpose = NULL,
    .contains = NULL,
    .isOrdered = NULL,
    .definingConcept = NULL,
    .isInStudy = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_BusinessProcessIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isInStudy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Study", 
          structure = "DdiUrn vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_DataPipeline


#' @title DDI4_DataPoint  A DataPoint is a container for a Datum
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/DataPoint}{DataPoint} 

#'
#'   Definition
#'
#'   ============
#'
#'   A DataPoint is a container for a Datum.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A cell in a spreadsheet table. Note that this could be empty. It exists independently of the value to be stored in it.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   The DataPoint is structural and distinct from the value (the Datum) that it holds. [GSIM 1.1]
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Data Point
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isDescribedBy}}{ A read/write active binding.
#'          The InstanceVariable delimits the values which can populate a DataPoint.
#'
#'                              \code{myDDI4_DataPoint$isDescribedBy <- c(myInstanceVariableURN)} sets the association 'isDescribedBy' from 'myDDI4_DataPoint' to a character vector of DdiUrns for R6 InstanceVariable class instances
#'
#'                              \code{myDDI4_DataPoint$isDescribedBy} returns a character vector of of DdiUrns for R6  InstanceVariable class instances}
#'   }
#'   \describe{
#'     \item{\code{hasDatum}}{ A read/write active binding.
#'          The Datum populating the DataPoint.
#'
#'                              \code{myDDI4_DataPoint$hasDatum <- c(myDatumURN)} sets the association 'hasDatum' from 'myDDI4_DataPoint' to a character vector of DdiUrns for R6 Datum class instances
#'
#'                              \code{myDDI4_DataPoint$hasDatum} returns a character vector of of DdiUrns for R6  Datum class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_DataPoint$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_DataPoint <- R6Class(classname="DDI4_DataPoint",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, isDescribedBy=NA,hasDatum=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_DataPoint may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)



       if(!missing(isDescribedBy)){
        private$.isDescribedBy <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=isDescribedBy , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isDescribedBy, referencingAssociationName="isDescribedBy",  referencingUrn=thisUrn)  
      }
      if(!missing(hasDatum)){
        private$.hasDatum <- returnUrnVectorIfValid(validClass="DDI4_Datum", proposedUrnVector=hasDatum , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasDatum, referencingAssociationName="hasDatum",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 2 class association(s):
    hasDatum = function(val){
      if (!missing(val)){
        private$.hasDatum <- returnUrnVectorIfValid(validClass="DDI4_Datum", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasDatum, referencingAssociationName="hasDatum",  referencingUrn=thisUrn)  
       # invisible( private$.hasDatum)
      } else {
          private$.hasDatum
      }
    },

    isDescribedBy = function(val){
      if (!missing(val)){
        private$.isDescribedBy <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isDescribedBy, referencingAssociationName="isDescribedBy",  referencingUrn=thisUrn)  
       # invisible( private$.isDescribedBy)
      } else {
          private$.isDescribedBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .isDescribedBy = NULL,
    .hasDatum = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasDatum = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Datum", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isDescribedBy = c(
          min="1", 
          max="1", 
          datatype = "DDI4_InstanceVariable", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_DataPoint


#' @title DDI4_DataPointIndicator  Member Indicator for use with member type DataPoint
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/DataPointIndicator}{DataPointIndicator} 

#'
#'   Definition
#'
#'   ============
#'
#'   Member Indicator for use with member type DataPoint
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_DataPointIndicator$DdiUrn} returns the DDI URN of myDDI4_DataPointIndicator.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Index value of member in an ordered array
#'
#'                              \code{myDDI4_DataPointIndicator$index <- myinteger} sets the index property of 'myDDI4_DataPointIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_DataPointIndicator$index} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{member}}{ A read/write active binding.
#'          Restricts member target class to DataPoint
#'
#'                              \code{myDDI4_DataPointIndicator$member <- c(myDataPointURN)} sets the association 'member' from 'myDDI4_DataPointIndicator' to a character vector of DdiUrns for R6 DataPoint class instances
#'
#'                              \code{myDDI4_DataPointIndicator$member} returns a character vector of of DdiUrns for R6  DataPoint class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_DataPointIndicator$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_DataPointIndicator <- R6Class(classname="DDI4_DataPointIndicator",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA,    member=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_DataPointIndicator may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(member)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_DataPoint", proposedUrnVector=member , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.index
      }
    },


# There are 1 class association(s):
    member = function(val){
      if (!missing(val)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_DataPoint", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
       # invisible( private$.member)
      } else {
          private$.member
      }
    }

  ),

  private = list(
    .index = NULL,
    .member = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        index = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        member = c(
          min="1", 
          max="1", 
          datatype = "DDI4_DataPoint", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_DataPointIndicator


#' @title DDI4_DataPointRelation  Data Point relations used by the Data Point Relation Structure of a Logical Record to describe specific source target Data Points and their relationship
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/DataPointRelation}{DataPointRelation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Data Point relations used by the Data Point Relation Structure of a Logical Record to describe specific source target Data Points and their relationship
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_DataPointRelation$DdiUrn} returns the DDI URN of myDDI4_DataPointRelation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an enumeration list 
#'
#'                              \code{myDDI4_DataPointRelation$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_DataPointRelation' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_DataPointRelation$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_DataPointRelation$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_DataPointRelation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_DataPointRelation$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_DataPointRelation$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_DataPointRelation' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_DataPointRelation$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          First member in a relationship.
#'
#'                              \code{myDDI4_DataPointRelation$source <- c(myDataPointURN)} sets the association 'source' from 'myDDI4_DataPointRelation' to a character vector of DdiUrns for R6 DataPoint class instances
#'
#'                              \code{myDDI4_DataPointRelation$source} returns a character vector of of DdiUrns for R6  DataPoint class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          Second member in a relationship. 
#'
#'                              \code{myDDI4_DataPointRelation$target <- c(myDataPointURN)} sets the association 'target' from 'myDDI4_DataPointRelation' to a character vector of DdiUrns for R6 DataPoint class instances
#'
#'                              \code{myDDI4_DataPointRelation$target} returns a character vector of of DdiUrns for R6  DataPoint class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_DataPointRelation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_DataPointRelation <- R6Class(classname="DDI4_DataPointRelation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_DataPointRelation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_DataPoint", proposedUrnVector=source , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_DataPoint", proposedUrnVector=target , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_DataPoint", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_DataPoint", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_DataPoint", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_DataPoint", 
          structure = "DdiUrn vector"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_DataPointRelation


#' @title DDI4_DataPointRelationStructure  A means for describing the complex relational structure of Data Points in a Logical Record 
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/DataPointRelationStructure}{DataPointRelationStructure} 

#'
#'   Definition
#'
#'   ============
#'
#'   A means for describing the complex relational structure of Data Points in a Logical Record 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_DataPointRelationStructure$DdiUrn} returns the DDI URN of myDDI4_DataPointRelationStructure.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an enumeration list.  Use if all relations within this relation structure are of the same specification.
#'
#'                              \code{myDDI4_DataPointRelationStructure$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_DataPointRelationStructure' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_DataPointRelationStructure$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_DataPointRelationStructure$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_DataPointRelationStructure' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_DataPointRelationStructure$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_DataPointRelationStructure$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_DataPointRelationStructure' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_DataPointRelationStructure$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMemberRelation}}{ A read/write active binding.
#'        Member relations that comprise the relation structure
#'
#'                              \code{myDDI4_DataPointRelationStructure$hasMemberRelation <- list(myDataPointRelation)} sets the 'hasMemberRelation' property of 'myDDI4_DataPointRelationStructure' to a list of R6 DataPointRelation class instances
#'
#'                              \code{myDDI4_DataPointRelationStructure$hasMemberRelation} returns a list of of R6 : DataPointRelation class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_DataPointRelationStructure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_DataPointRelationStructure <- R6Class(classname="DDI4_DataPointRelationStructure",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_DataPointRelationStructure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasMemberRelation)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_DataPointRelation", proposedObjectList=hasMemberRelation , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    hasMemberRelation = function(val){
      if (!missing(val)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_DataPointRelation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasMemberRelation
      }
    },

    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .hasMemberRelation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasMemberRelation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_DataPointRelation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasRelationSpecification = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_DataPointRelationStructure


#' @title DDI4_DataStore  A DataStore is either a SimpleCollection or a StructuredCollection of LogicalRecords, keeping in mind that a LogicalRecords is a definition, not a "datasets"
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/DataStore}{DataStore} 

#'
#'   Definition
#'
#'   ============
#'
#'   A DataStore is either a SimpleCollection or a StructuredCollection of LogicalRecords, keeping in mind that a LogicalRecords is a definition, not a "datasets".
#'
#'   
#'
#'   LogicalRecords organized in a StructuredCollection is called a LogicalRecordRelationStructure.
#'
#'   
#'
#'   Instances of LogicalRecords instantiated as organizations of DataPoints hosting data are described in FormatDescription.
#'
#'   
#'
#'   A DataStore is reusable across studies. Each Study has at most one DataStore.
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   The data lineage of an individual BusinessProcess or an entire DataPipeline are both examples of  a LogicalRecordRelationStructures. In a data lineage we can observe how LogicalRecords are connected within a BusinessProcess or across BusinessProcesses.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   Schema repository, data network
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Data Set
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_DataStore$DdiUrn} returns the DDI URN of myDDI4_DataStore.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{characterSet}}{ A read/write active binding.
#'        Default character set used in the Data Store.
#'
#'                              \code{myDDI4_DataStore$characterSet <- mycharacter} sets the characterSet property of 'myDDI4_DataStore' to the vector of datatype:character
#'
#'                              \code{myDDI4_DataStore$characterSet} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{dataStoreType}}{ A read/write active binding.
#'        The type of DataStore. Could be delimited file, fixed record length file, relational database, etc. Points to an external definition which can be part of a controlled vocabulary maintained by the DDI Alliance.
#'
#'                              \code{myDDI4_DataStore$dataStoreType <- list(myExternalControlledVocabularyEntry)} sets the 'dataStoreType' property of 'myDDI4_DataStore' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_DataStore$dataStoreType} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{type}}{ A read/write active binding.
#'        Whether the collection is a bag or a set: a bag is a collection with duplicates allowed, a set is a collection without duplicates.
#'
#'                              \code{myDDI4_DataStore$type <- list(myCollectionType)} sets the 'type' property of 'myDDI4_DataStore' to a list of R6 CollectionType class instances
#'
#'                              \code{myDDI4_DataStore$type} returns a list of of R6 : CollectionType class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage.
#'
#'                              \code{myDDI4_DataStore$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_DataStore' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_DataStore$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_DataStore$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_DataStore' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_DataStore$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{recordCount}}{ A read/write active binding.
#'        The number of records in the Data Store.
#'
#'                              \code{myDDI4_DataStore$recordCount <- myinteger} sets the recordCount property of 'myDDI4_DataStore' to the vector of datatype:integer
#'
#'                              \code{myDDI4_DataStore$recordCount} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{aboutMissing}}{ A read/write active binding.
#'        General information about missing data, e.g., that missing data have been standardized across the collection, missing data are present because of merging, etc.-  corresponds to DDI2.5 dataMsng.
#'
#'                              \code{myDDI4_DataStore$aboutMissing <- list(myInternationalStructuredString)} sets the 'aboutMissing' property of 'myDDI4_DataStore' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_DataStore$aboutMissing} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        Data in the form of Data Records contained in the Data Store. Allows for the identification of the member and optionally provides an index for the member within an ordered array 
#'
#'                              \code{myDDI4_DataStore$contains <- list(myLogicalRecordIndicator)} sets the 'contains' property of 'myDDI4_DataStore' to a list of R6 LogicalRecordIndicator class instances
#'
#'                              \code{myDDI4_DataStore$contains} returns a list of of R6 : LogicalRecordIndicator class instances}
#'   }
#'   \describe{
#'     \item{\code{isOrdered}}{ A read/write active binding.
#'        If members are ordered set to true, if unordered set to false.
#'
#'                              \code{myDDI4_DataStore$isOrdered <- mylogical} sets the isOrdered property of 'myDDI4_DataStore' to the vector of datatype:logical
#'
#'                              \code{myDDI4_DataStore$isOrdered} returns a vector of datatype: logical}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isStructuredBy}}{ A read/write active binding.
#'          Description of a complex structure for the Collection. A collection may be structured in more than one way for different uses.
#'
#'                              \code{myDDI4_DataStore$isStructuredBy <- c(myLogicalRecordRelationStructureURN)} sets the association 'isStructuredBy' from 'myDDI4_DataStore' to a character vector of DdiUrns for R6 LogicalRecordRelationStructure class instances
#'
#'                              \code{myDDI4_DataStore$isStructuredBy} returns a character vector of of DdiUrns for R6  LogicalRecordRelationStructure class instances}
#'   }
#'   \describe{
#'     \item{\code{usesRecordRelation}}{ A read/write active binding.
#'          The Record Relation that defines the relationship and linking requirements between LogicalRecords in the DataStore 
#'
#'                              \code{myDDI4_DataStore$usesRecordRelation <- c(myRecordRelationURN)} sets the association 'usesRecordRelation' from 'myDDI4_DataStore' to a character vector of DdiUrns for R6 RecordRelation class instances
#'
#'                              \code{myDDI4_DataStore$usesRecordRelation} returns a character vector of of DdiUrns for R6  RecordRelation class instances}
#'   }
#'   \describe{
#'     \item{\code{definingConcept}}{ A read/write active binding.
#'          The conceptual basis for the collection of members.
#'
#'                              \code{myDDI4_DataStore$definingConcept <- c(myConceptURN)} sets the association 'definingConcept' from 'myDDI4_DataStore' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_DataStore$definingConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#'   \describe{
#'     \item{\code{isInStudy}}{ A read/write active binding.
#'          A Study has at most one DataStore. Many studies can have the same set of record types.
#'
#'                              \code{myDDI4_DataStore$isInStudy <- c(myStudyURN)} sets the association 'isInStudy' from 'myDDI4_DataStore' to a character vector of DdiUrns for R6 Study class instances
#'
#'                              \code{myDDI4_DataStore$isInStudy} returns a character vector of of DdiUrns for R6  Study class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_DataStore$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, characterSet=NA, dataStoreType=NA, type=NA, name=NA, purpose=NA, recordCount=NA, aboutMissing=NA, contains=NA, isOrdered=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_DataStore <- R6Class(classname="DDI4_DataStore",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, characterSet=NA, dataStoreType=NA, type=NA, name=NA, purpose=NA, recordCount=NA, aboutMissing=NA, contains=NA, isOrdered=NA,    isStructuredBy=NA,usesRecordRelation=NA,definingConcept=NA,isInStudy=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_DataStore may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(characterSet)){
        private$.characterSet <- returnVectorIfValid(validVectorClass="character", proposedVector=characterSet , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(dataStoreType)){
        private$.dataStoreType <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=dataStoreType , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(type)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=type , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(recordCount)){
        private$.recordCount <- returnVectorIfValid(validVectorClass="integer", proposedVector=recordCount , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(aboutMissing)){
        private$.aboutMissing <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=aboutMissing , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_LogicalRecordIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(isOrdered)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=isOrdered , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(isStructuredBy)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_LogicalRecordRelationStructure", proposedUrnVector=isStructuredBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
      }
      if(!missing(usesRecordRelation)){
        private$.usesRecordRelation <- returnUrnVectorIfValid(validClass="DDI4_RecordRelation", proposedUrnVector=usesRecordRelation , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesRecordRelation, referencingAssociationName="usesRecordRelation",  referencingUrn=thisUrn)  
      }
      if(!missing(definingConcept)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=definingConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
      }
      if(!missing(isInStudy)){
        private$.isInStudy <- returnUrnVectorIfValid(validClass="DDI4_Study", proposedUrnVector=isInStudy , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isInStudy, referencingAssociationName="isInStudy",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 9 properties: 
    aboutMissing = function(val){
      if (!missing(val)){
        private$.aboutMissing <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.aboutMissing
      }
    },

    characterSet = function(val){
      if (!missing(val)){
        private$.characterSet <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.characterSet
      }
    },

    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_LogicalRecordIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },

    dataStoreType = function(val){
      if (!missing(val)){
        private$.dataStoreType <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.dataStoreType
      }
    },

    isOrdered = function(val){
      if (!missing(val)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isOrdered
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    recordCount = function(val){
      if (!missing(val)){
        private$.recordCount <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.recordCount
      }
    },

    type = function(val){
      if (!missing(val)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.type
      }
    },


# There are 4 class association(s):
    definingConcept = function(val){
      if (!missing(val)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
       # invisible( private$.definingConcept)
      } else {
          private$.definingConcept
      }
    },

    isInStudy = function(val){
      if (!missing(val)){
        private$.isInStudy <- returnUrnVectorIfValid(validClass="DDI4_Study", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isInStudy, referencingAssociationName="isInStudy",  referencingUrn=thisUrn)  
       # invisible( private$.isInStudy)
      } else {
          private$.isInStudy
      }
    },

    isStructuredBy = function(val){
      if (!missing(val)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_LogicalRecordRelationStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
       # invisible( private$.isStructuredBy)
      } else {
          private$.isStructuredBy
      }
    },

    usesRecordRelation = function(val){
      if (!missing(val)){
        private$.usesRecordRelation <- returnUrnVectorIfValid(validClass="DDI4_RecordRelation", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesRecordRelation, referencingAssociationName="usesRecordRelation",  referencingUrn=thisUrn)  
       # invisible( private$.usesRecordRelation)
      } else {
          private$.usesRecordRelation
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .characterSet = NULL,
    .dataStoreType = NULL,
    .type = NULL,
    .name = NULL,
    .purpose = NULL,
    .recordCount = NULL,
    .aboutMissing = NULL,
    .contains = NULL,
    .isOrdered = NULL,
    .isStructuredBy = NULL,
    .usesRecordRelation = NULL,
    .definingConcept = NULL,
    .isInStudy = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        aboutMissing = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        characterSet = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LogicalRecordIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        dataStoreType = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isInStudy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Study", 
          structure = "DdiUrn vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isStructuredBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LogicalRecordRelationStructure", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        recordCount = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        usesRecordRelation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RecordRelation", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_DataStore


#' @title DDI4_DataStoreIndicator  Member Indicator for use with member type DataStore
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/DataStoreIndicator}{DataStoreIndicator} 

#'
#'   Definition
#'
#'   ============
#'
#'   Member Indicator for use with member type DataStore
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_DataStoreIndicator$DdiUrn} returns the DDI URN of myDDI4_DataStoreIndicator.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Index value of member in an ordered array
#'
#'                              \code{myDDI4_DataStoreIndicator$index <- myinteger} sets the index property of 'myDDI4_DataStoreIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_DataStoreIndicator$index} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{member}}{ A read/write active binding.
#'          Restricts member target class to DataStore
#'
#'                              \code{myDDI4_DataStoreIndicator$member <- c(myDataStoreURN)} sets the association 'member' from 'myDDI4_DataStoreIndicator' to a character vector of DdiUrns for R6 DataStore class instances
#'
#'                              \code{myDDI4_DataStoreIndicator$member} returns a character vector of of DdiUrns for R6  DataStore class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_DataStoreIndicator$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_DataStoreIndicator <- R6Class(classname="DDI4_DataStoreIndicator",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA,    member=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_DataStoreIndicator may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(member)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_DataStore", proposedUrnVector=member , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.index
      }
    },


# There are 1 class association(s):
    member = function(val){
      if (!missing(val)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_DataStore", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
       # invisible( private$.member)
      } else {
          private$.member
      }
    }

  ),

  private = list(
    .index = NULL,
    .member = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        index = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        member = c(
          min="1", 
          max="1", 
          datatype = "DDI4_DataStore", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_DataStoreIndicator


#' @title DDI4_DataStoreLibrary  A DataStoreLibrary is a collection or, again, a "library" of DataStores
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/DataStoreLibrary}{DataStoreLibrary} 

#'
#'   Definition
#'
#'   ============
#'
#'   A DataStoreLibrary is a collection or, again, a "library" of DataStores.
#'
#'   
#'
#'   An individual DataStore is associated with a Study. A collection of DataStores is associated with a StudySeries.
#'
#'   
#'
#'   The relationships among the DataStores in the DataStoreLibrary is described by the DataStoreRelationStructure. Relations may be more or less complicated depending on the StudySeries type. A StudySeries may be ad hoc. A StudySeries may form a time series. The variety of these collections has been described using the <a href=https://www.ddialliance.org/Specification/DDI-Lifecycle/3.1/XMLSchema/FieldLevelDocumentation/">"Group"</a> in DDI 3.1.
#'
#'   
#'
#'   Like any RelationStructure, the DataStoreRelationStructure is able to describe both part/whole relations and generalization/specialization relations. See the controlled vocabulary at <a href="http://lion.ddialliance.org/datatypes/relationspecification">RelationSpecification</a> to review all the types of relations a RelationStructure is able to describe. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A StudySeries is a time series and a DataStoreRelationStructure is dedicated to tracking "supplements".
#'
#'   
#'
#'   There is a study at T0. Perhaps it is never executed. It is the "core" study. At T1 there is a study that includes the core and Supplement A. At T2 there is a study that includes the core, Supplement A and Supplement B. At T3 there is a study that includes the core and a one off supplement that is never asked again. T4 though T6 repeats T1 through T3.
#'
#'   
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   If we broke a study down into sub-studies where the core is a sub-study and each supplement is a sub-study, we could also use the DataStoreRelationStructure to track the generalization/specialization relationship or, again, "going deep" over time.
#'
#'   
#'
#'   Sometimes across a StudySeries we add panels. If panels were also sub-studies, we could use the DataStoreRelationStructure to track both the core and supplements by panel over time.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   Structured metadata archive
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_DataStoreLibrary$DdiUrn} returns the DDI URN of myDDI4_DataStoreLibrary.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{type}}{ A read/write active binding.
#'        Whether the collection is a bag or a set: a bag is a collection with duplicates allowed, a set is a collection without duplicates.
#'
#'                              \code{myDDI4_DataStoreLibrary$type <- list(myCollectionType)} sets the 'type' property of 'myDDI4_DataStoreLibrary' to a list of R6 CollectionType class instances
#'
#'                              \code{myDDI4_DataStoreLibrary$type} returns a list of of R6 : CollectionType class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage.
#'
#'                              \code{myDDI4_DataStoreLibrary$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_DataStoreLibrary' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_DataStoreLibrary$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_DataStoreLibrary$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_DataStoreLibrary' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_DataStoreLibrary$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        The DataStores in the Catalog. Allows for the identification of the member and optionally provides an index for the member within an ordered array 
#'
#'                              \code{myDDI4_DataStoreLibrary$contains <- list(myDataStoreIndicator)} sets the 'contains' property of 'myDDI4_DataStoreLibrary' to a list of R6 DataStoreIndicator class instances
#'
#'                              \code{myDDI4_DataStoreLibrary$contains} returns a list of of R6 : DataStoreIndicator class instances}
#'   }
#'   \describe{
#'     \item{\code{isOrdered}}{ A read/write active binding.
#'        If members are ordered set to true, if unordered set to false.
#'
#'                              \code{myDDI4_DataStoreLibrary$isOrdered <- mylogical} sets the isOrdered property of 'myDDI4_DataStoreLibrary' to the vector of datatype:logical
#'
#'                              \code{myDDI4_DataStoreLibrary$isOrdered} returns a vector of datatype: logical}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isStructuredBy}}{ A read/write active binding.
#'          Description of a complex structure for the Collection. A collection may be structured in more than one way for different uses.
#'
#'                              \code{myDDI4_DataStoreLibrary$isStructuredBy <- c(myDataStoreRelationStructureURN)} sets the association 'isStructuredBy' from 'myDDI4_DataStoreLibrary' to a character vector of DdiUrns for R6 DataStoreRelationStructure class instances
#'
#'                              \code{myDDI4_DataStoreLibrary$isStructuredBy} returns a character vector of of DdiUrns for R6  DataStoreRelationStructure class instances}
#'   }
#'   \describe{
#'     \item{\code{definingConcept}}{ A read/write active binding.
#'          The conceptual basis for the collection of members.
#'
#'                              \code{myDDI4_DataStoreLibrary$definingConcept <- c(myConceptURN)} sets the association 'definingConcept' from 'myDDI4_DataStoreLibrary' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_DataStoreLibrary$definingConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#'   \describe{
#'     \item{\code{isInStudySeries}}{ A read/write active binding.
#'          Whereas a DataStore is associated with a Study, a DataSoreLibrary is associated with a StudySeries. Each StudySeries has at most one DataStoreLibrary
#'
#'                              \code{myDDI4_DataStoreLibrary$isInStudySeries <- c(myStudySeriesURN)} sets the association 'isInStudySeries' from 'myDDI4_DataStoreLibrary' to a character vector of DdiUrns for R6 StudySeries class instances
#'
#'                              \code{myDDI4_DataStoreLibrary$isInStudySeries} returns a character vector of of DdiUrns for R6  StudySeries class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_DataStoreLibrary$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_DataStoreLibrary <- R6Class(classname="DDI4_DataStoreLibrary",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA,    isStructuredBy=NA,definingConcept=NA,isInStudySeries=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_DataStoreLibrary may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(type)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=type , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_DataStoreIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(isOrdered)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=isOrdered , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(isStructuredBy)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_DataStoreRelationStructure", proposedUrnVector=isStructuredBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
      }
      if(!missing(definingConcept)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=definingConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
      }
      if(!missing(isInStudySeries)){
        private$.isInStudySeries <- returnUrnVectorIfValid(validClass="DDI4_StudySeries", proposedUrnVector=isInStudySeries , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isInStudySeries, referencingAssociationName="isInStudySeries",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 5 properties: 
    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_DataStoreIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },

    isOrdered = function(val){
      if (!missing(val)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isOrdered
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    type = function(val){
      if (!missing(val)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.type
      }
    },


# There are 3 class association(s):
    definingConcept = function(val){
      if (!missing(val)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
       # invisible( private$.definingConcept)
      } else {
          private$.definingConcept
      }
    },

    isInStudySeries = function(val){
      if (!missing(val)){
        private$.isInStudySeries <- returnUrnVectorIfValid(validClass="DDI4_StudySeries", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isInStudySeries, referencingAssociationName="isInStudySeries",  referencingUrn=thisUrn)  
       # invisible( private$.isInStudySeries)
      } else {
          private$.isInStudySeries
      }
    },

    isStructuredBy = function(val){
      if (!missing(val)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_DataStoreRelationStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
       # invisible( private$.isStructuredBy)
      } else {
          private$.isStructuredBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .type = NULL,
    .name = NULL,
    .purpose = NULL,
    .contains = NULL,
    .isOrdered = NULL,
    .isStructuredBy = NULL,
    .definingConcept = NULL,
    .isInStudySeries = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_DataStoreIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isInStudySeries = c(
          min="0", 
          max="1", 
          datatype = "DDI4_StudySeries", 
          structure = "DdiUrn vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isStructuredBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_DataStoreRelationStructure", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_DataStoreLibrary


#' @title DDI4_DataStoreRelation  Defines the complex relationship between 2 or more DataStores in a DataLibrary
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/DataStoreRelation}{DataStoreRelation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Defines the complex relationship between 2 or more DataStores in a DataLibrary
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_DataStoreRelation$DdiUrn} returns the DDI URN of myDDI4_DataStoreRelation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an enumeration list
#'
#'                              \code{myDDI4_DataStoreRelation$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_DataStoreRelation' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_DataStoreRelation$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_DataStoreRelation$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_DataStoreRelation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_DataStoreRelation$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_DataStoreRelation$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_DataStoreRelation' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_DataStoreRelation$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          First DataStore in the relation
#'
#'                              \code{myDDI4_DataStoreRelation$source <- c(myDataStoreURN)} sets the association 'source' from 'myDDI4_DataStoreRelation' to a character vector of DdiUrns for R6 DataStore class instances
#'
#'                              \code{myDDI4_DataStoreRelation$source} returns a character vector of of DdiUrns for R6  DataStore class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          Second DataStore in a relationship. Note that this can be realized as a collection to support tuples.
#'
#'                              \code{myDDI4_DataStoreRelation$target <- c(myDataStoreURN)} sets the association 'target' from 'myDDI4_DataStoreRelation' to a character vector of DdiUrns for R6 DataStore class instances
#'
#'                              \code{myDDI4_DataStoreRelation$target} returns a character vector of of DdiUrns for R6  DataStore class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_DataStoreRelation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_DataStoreRelation <- R6Class(classname="DDI4_DataStoreRelation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_DataStoreRelation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_DataStore", proposedUrnVector=source , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_DataStore", proposedUrnVector=target , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_DataStore", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_DataStore", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_DataStore", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_DataStore", 
          structure = "DdiUrn vector"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_DataStoreRelation


#' @title DDI4_DataStoreRelationStructure  A structure for describing a complex relation of DataStores within a DataStoreLibrary
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/DataStoreRelationStructure}{DataStoreRelationStructure} 

#'
#'   Definition
#'
#'   ============
#'
#'   A structure for describing a complex relation of DataStores within a DataStoreLibrary
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   If the succession of DataStores are created by a StudySeries that is a time series, from one DataStore to the next part/whole relations can obtain as supplements are added and subtracted.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_DataStoreRelationStructure$DdiUrn} returns the DDI URN of myDDI4_DataStoreRelationStructure.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an enumeration list.  Use if all relations within this relation structure are of the same specification.
#'
#'                              \code{myDDI4_DataStoreRelationStructure$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_DataStoreRelationStructure' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_DataStoreRelationStructure$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_DataStoreRelationStructure$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_DataStoreRelationStructure' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_DataStoreRelationStructure$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_DataStoreRelationStructure$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_DataStoreRelationStructure' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_DataStoreRelationStructure$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMemberRelation}}{ A read/write active binding.
#'        Member relations that comprise the relation structure
#'
#'                              \code{myDDI4_DataStoreRelationStructure$hasMemberRelation <- list(myDataStoreRelation)} sets the 'hasMemberRelation' property of 'myDDI4_DataStoreRelationStructure' to a list of R6 DataStoreRelation class instances
#'
#'                              \code{myDDI4_DataStoreRelationStructure$hasMemberRelation} returns a list of of R6 : DataStoreRelation class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_DataStoreRelationStructure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_DataStoreRelationStructure <- R6Class(classname="DDI4_DataStoreRelationStructure",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_DataStoreRelationStructure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasMemberRelation)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_DataStoreRelation", proposedObjectList=hasMemberRelation , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    hasMemberRelation = function(val){
      if (!missing(val)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_DataStoreRelation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasMemberRelation
      }
    },

    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .hasMemberRelation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasMemberRelation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_DataStoreRelation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasRelationSpecification = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_DataStoreRelationStructure


#' @title DDI4_Date  Provides the structure of a single Date expressed in an ISO date structure along with equivalent expression in any number of non-ISO formats
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Date}{Date} 

#'
#'   Definition
#'
#'   ============
#'
#'   Provides the structure of a single Date expressed in an ISO date structure along with equivalent expression in any number of non-ISO formats. While it supports the use of the ISO time interval structure this should only be used when the exact date is unclear (i.e. occurring at some point in time between the two specified dates) or in specified applications. Ranges with specified start and end dates should use the DateRange as a datatype. Commonly uses property names include: eventDate, issueDate, and releaseDate.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Date allows one of a set of date-time (YYYY-MM-DDThh:mm:ss), date (YYYY-MM-DD), year-month (YYYY-MM), year (YYYY), time (hh:mm:ss) and duration (PnYnMnDnHnMnS), or time interval (YYYY-MM-DDThh:mm:ss/YYYY-MM-DDThh:mm:ss, YYYY-MM-DDThh:mm:ss/PnYnMnDnHnMnS, PnYnMnDnHnMnS/ YYYY-MM-DDThh:mm:ss) which is formatted according to ISO 8601 and backed supported by regular expressions in the BaseDateType. Time Zone designation and negative/positive prefixes are allowed as are dates before and after 0000 through 9999.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:DateType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Date$DdiUrn} returns the DDI URN of myDDI4_Date.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{isoDate}}{ A read/write active binding.
#'        Strongly recommend that ALL dates be expressed in an ISO format at a minimum. A single point in time expressed in an ISO standard structure. Note that while it supports an ISO date range structure this should be used in Date only when the single date is unclear i.e. occurring at some time between two dates. 
#'
#'                              \code{myDDI4_Date$isoDate <- list(myIsoDateType)} sets the 'isoDate' property of 'myDDI4_Date' to a list of R6 IsoDateType class instances
#'
#'                              \code{myDDI4_Date$isoDate} returns a list of of R6 : IsoDateType class instances}
#'   }
#'   \describe{
#'     \item{\code{nonIsoDate}}{ A read/write active binding.
#'        A simple date expressed in a non-ISO date format, including a specification of the date format and calendar used.
#'
#'                              \code{myDDI4_Date$nonIsoDate <- list(myNonIsoDateType)} sets the 'nonIsoDate' property of 'myDDI4_Date' to a list of R6 NonIsoDateType class instances
#'
#'                              \code{myDDI4_Date$nonIsoDate} returns a list of of R6 : NonIsoDateType class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Date$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, isoDate=NA, nonIsoDate=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Date <- R6Class(classname="DDI4_Date",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, isoDate=NA, nonIsoDate=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Date may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(isoDate)){
        private$.isoDate <- returnRegularExpressionValues(RegularExpressionObject=DDI4_IsoDateType$new(registerThisObject=FALSE), proposedCharacterVector=isoDate , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(nonIsoDate)){
        private$.nonIsoDate <- returnR6ObjectIfValid(validClass="DDI4_NonIsoDateType", proposedObjectList=nonIsoDate , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    isoDate = function(val){
      if (!missing(val)){
        private$.isoDate <- returnRegularExpressionValues(RegularExpressionObject=DDI4_IsoDateType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isoDate
      }
    },

    nonIsoDate = function(val){
      if (!missing(val)){
        private$.nonIsoDate <- returnR6ObjectIfValid(validClass="DDI4_NonIsoDateType", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.nonIsoDate
      }
    }


  ),

  private = list(
    .isoDate = NULL,
    .nonIsoDate = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        isoDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        nonIsoDate = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_NonIsoDateType", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Date


#' @title DDI4_DateRange  Expresses a date/time range using a start date and end date (both with the structure of Date and supporting the use of ISO and non-ISO date structures)
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/DateRange}{DateRange} 

#'
#'   Definition
#'
#'   ============
#'
#'   Expresses a date/time range using a start date and end date (both with the structure of Date and supporting the use of ISO and non-ISO date structures). Use in all locations where a range of dates is required, i.e. validFor, embargoPeriod, collectionPeriod, etc.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:DateType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_DateRange$DdiUrn} returns the DDI URN of myDDI4_DateRange.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{startDate}}{ A read/write active binding.
#'        The date (time) designating the beginning of the period or range.
#'
#'                              \code{myDDI4_DateRange$startDate <- list(myDate)} sets the 'startDate' property of 'myDDI4_DateRange' to a list of R6 Date class instances
#'
#'                              \code{myDDI4_DateRange$startDate} returns a list of of R6 : Date class instances}
#'   }
#'   \describe{
#'     \item{\code{endDate}}{ A read/write active binding.
#'        The date (time) designating the end of the period or range.
#'
#'                              \code{myDDI4_DateRange$endDate <- list(myDate)} sets the 'endDate' property of 'myDDI4_DateRange' to a list of R6 Date class instances
#'
#'                              \code{myDDI4_DateRange$endDate} returns a list of of R6 : Date class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_DateRange$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, startDate=NA, endDate=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_DateRange <- R6Class(classname="DDI4_DateRange",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, startDate=NA, endDate=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_DateRange may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(startDate)){
        private$.startDate <- returnR6ObjectIfValid(validClass="DDI4_Date", proposedObjectList=startDate , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(endDate)){
        private$.endDate <- returnR6ObjectIfValid(validClass="DDI4_Date", proposedObjectList=endDate , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    endDate = function(val){
      if (!missing(val)){
        private$.endDate <- returnR6ObjectIfValid(validClass="DDI4_Date", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.endDate
      }
    },

    startDate = function(val){
      if (!missing(val)){
        private$.startDate <- returnR6ObjectIfValid(validClass="DDI4_Date", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.startDate
      }
    }


  ),

  private = list(
    .startDate = NULL,
    .endDate = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        endDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Date", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        startDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Date", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_DateRange


#' @title DDI4_Datum  A Datum is a designation (a representation of a concept by a sign) with a notion of equality defined
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Datum}{Datum} 

#'
#'   Definition
#'
#'   ============
#'
#'   A Datum is a designation (a representation of a concept by a sign) with a notion of equality defined. The sign itself is a perceivable object.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A systolic blood pressure of 122 is measured. The Signifier for that measurement in this paragraph is the character string "122". The Datum in this case is a kind of Designation (Sign), which is the association of the underlying measured concept (a blood pressure at that level) with that Signifier.  The Datum has an association with an InstanceVariable which allows the attachment of a unit of measurement (mm Hg), a datatype, a Population, and the Act which produced the measurement.  These InstanceVariable attributes are critical for interpreting the Signifier.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   From GSIM 1.1 "A Datum is the actual instance of data that was collected or derived. It is the value which populates one or more Data Points. A Datum is the value found in a cell of a table." (https://statswiki.unece.org/display/GSIMclick/Datum )  A Datum could be copied to one or more datasets.
#'
#'   DDI4 takes a little more formal (semiotic) description of a Datum using the Signification Pattern. See the attached example.
#'
#'   
#'
#'   
#'
#'   NOTE: This is NOT datum from DDI3.2 (which is quite specific).
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Datum
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Designation}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Datum$DdiUrn} returns the DDI URN of myDDI4_Datum.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{representation}}{ A read/write active binding.
#'        Specification of the type of Signification
#'
#'                              \code{myDDI4_Datum$representation <- list(myValueString)} sets the 'representation' property of 'myDDI4_Datum' to a list of R6 ValueString class instances
#'
#'                              \code{myDDI4_Datum$representation} returns a list of of R6 : ValueString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isBoundedBy}}{ A read/write active binding.
#'          A Datum is bounded by an InstanceVariable. The Datum is drawn from a set of values, either substantive or sentinel described by the value domain of the InstanceVariable 
#'
#'                              \code{myDDI4_Datum$isBoundedBy <- c(myInstanceVariableURN)} sets the association 'isBoundedBy' from 'myDDI4_Datum' to a character vector of DdiUrns for R6 InstanceVariable class instances
#'
#'                              \code{myDDI4_Datum$isBoundedBy} returns a character vector of of DdiUrns for R6  InstanceVariable class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Datum$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, representation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Datum <- R6Class(classname="DDI4_Datum",
  inherit = DDI4_Designation,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, representation=NA,    isBoundedBy=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Datum may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(representation)){
        private$.representation <- returnR6ObjectIfValid(validClass="DDI4_ValueString", proposedObjectList=representation , minimumCardinality=1, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(isBoundedBy)){
        private$.isBoundedBy <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=isBoundedBy , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isBoundedBy, referencingAssociationName="isBoundedBy",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    representation = function(val){
      if (!missing(val)){
        private$.representation <- returnR6ObjectIfValid(validClass="DDI4_ValueString", proposedObjectList=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.representation
      }
    },


# There are 1 class association(s):
    isBoundedBy = function(val){
      if (!missing(val)){
        private$.isBoundedBy <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isBoundedBy, referencingAssociationName="isBoundedBy",  referencingUrn=thisUrn)  
       # invisible( private$.isBoundedBy)
      } else {
          private$.isBoundedBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .representation = NULL,
    .isBoundedBy = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        denotes = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isBoundedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InstanceVariable", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        representation = c(
          min="1", 
          max="1", 
          datatype = "DDI4_ValueString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Datum


#' @title DDI4_Design  The design pattern class may be used to specify or, again, defines how a process will be performed in general
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Design}{Design} 

#'
#'   Definition
#'
#'   ============
#'
#'   The design pattern class may be used to specify or, again, defines how a process will be performed in general. The design informs a specific or implemented process as to its general parameters.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Process Design
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Design$DdiUrn} returns the DDI URN of myDDI4_Design.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{overview}}{ A read/write active binding.
#'        Provides a high level overview or summary of the class. Can be used to inform end-users or as part of an executive summary. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_Design$overview <- list(myInternationalStructuredString)} sets the 'overview' property of 'myDDI4_Design' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Design$overview} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isDiscussedIn}}{ A read/write active binding.
#'          External materials that describe or discuss the design 
#'
#'                              \code{myDDI4_Design$isDiscussedIn <- c(myExternalMaterialURN)} sets the association 'isDiscussedIn' from 'myDDI4_Design' to a character vector of DdiUrns for R6 ExternalMaterial class instances
#'
#'                              \code{myDDI4_Design$isDiscussedIn} returns a character vector of of DdiUrns for R6  ExternalMaterial class instances}
#'   }
#'   \describe{
#'     \item{\code{expressesAlgorithm}}{ A read/write active binding.
#'          An algorithm that defines the process used by the design in a generic way.
#'
#'                              \code{myDDI4_Design$expressesAlgorithm <- c(myAlgorithmURN)} sets the association 'expressesAlgorithm' from 'myDDI4_Design' to a character vector of DdiUrns for R6 Algorithm class instances
#'
#'                              \code{myDDI4_Design$expressesAlgorithm} returns a character vector of of DdiUrns for R6  Algorithm class instances}
#'   }
#'   \describe{
#'     \item{\code{assumesPrecondition}}{ A read/write active binding.
#'          A precondition that must exist prior to the applied use of this design.
#'
#'                              \code{myDDI4_Design$assumesPrecondition <- c(myPreconditionURN)} sets the association 'assumesPrecondition' from 'myDDI4_Design' to a character vector of DdiUrns for R6 Precondition class instances
#'
#'                              \code{myDDI4_Design$assumesPrecondition} returns a character vector of of DdiUrns for R6  Precondition class instances}
#'   }
#'   \describe{
#'     \item{\code{specifiesGoal}}{ A read/write active binding.
#'          The generic goal of a design.
#'
#'                              \code{myDDI4_Design$specifiesGoal <- c(myGoalURN)} sets the association 'specifiesGoal' from 'myDDI4_Design' to a character vector of DdiUrns for R6 Goal class instances
#'
#'                              \code{myDDI4_Design$specifiesGoal} returns a character vector of of DdiUrns for R6  Goal class instances}
#'   }
#'   \describe{
#'     \item{\code{implentatedBy}}{ A read/write active binding.
#'          The design of the expressed algorithm is implemented by a Process
#'
#'                              \code{myDDI4_Design$implentatedBy <- c(myProcessURN)} sets the association 'implentatedBy' from 'myDDI4_Design' to a character vector of DdiUrns for R6 Process class instances
#'
#'                              \code{myDDI4_Design$implentatedBy} returns a character vector of of DdiUrns for R6  Process class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Design$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, overview=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Design <- R6Class(classname="DDI4_Design",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, overview=NA,    isDiscussedIn=NA,expressesAlgorithm=NA,assumesPrecondition=NA,specifiesGoal=NA,implentatedBy=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Design may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(overview)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=overview , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(isDiscussedIn)){
        private$.isDiscussedIn <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=isDiscussedIn , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isDiscussedIn, referencingAssociationName="isDiscussedIn",  referencingUrn=thisUrn)  
      }
      if(!missing(expressesAlgorithm)){
        private$.expressesAlgorithm <- returnUrnVectorIfValid(validClass="DDI4_Algorithm", proposedUrnVector=expressesAlgorithm , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.expressesAlgorithm, referencingAssociationName="expressesAlgorithm",  referencingUrn=thisUrn)  
      }
      if(!missing(assumesPrecondition)){
        private$.assumesPrecondition <- returnUrnVectorIfValid(validClass="DDI4_Precondition", proposedUrnVector=assumesPrecondition , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.assumesPrecondition, referencingAssociationName="assumesPrecondition",  referencingUrn=thisUrn)  
      }
      if(!missing(specifiesGoal)){
        private$.specifiesGoal <- returnUrnVectorIfValid(validClass="DDI4_Goal", proposedUrnVector=specifiesGoal , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.specifiesGoal, referencingAssociationName="specifiesGoal",  referencingUrn=thisUrn)  
      }
      if(!missing(implentatedBy)){
        private$.implentatedBy <- returnUrnVectorIfValid(validClass="DDI4_Process", proposedUrnVector=implentatedBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.implentatedBy, referencingAssociationName="implentatedBy",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    overview = function(val){
      if (!missing(val)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overview
      }
    },


# There are 5 class association(s):
    assumesPrecondition = function(val){
      if (!missing(val)){
        private$.assumesPrecondition <- returnUrnVectorIfValid(validClass="DDI4_Precondition", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.assumesPrecondition, referencingAssociationName="assumesPrecondition",  referencingUrn=thisUrn)  
       # invisible( private$.assumesPrecondition)
      } else {
          private$.assumesPrecondition
      }
    },

    expressesAlgorithm = function(val){
      if (!missing(val)){
        private$.expressesAlgorithm <- returnUrnVectorIfValid(validClass="DDI4_Algorithm", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.expressesAlgorithm, referencingAssociationName="expressesAlgorithm",  referencingUrn=thisUrn)  
       # invisible( private$.expressesAlgorithm)
      } else {
          private$.expressesAlgorithm
      }
    },

    implentatedBy = function(val){
      if (!missing(val)){
        private$.implentatedBy <- returnUrnVectorIfValid(validClass="DDI4_Process", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.implentatedBy, referencingAssociationName="implentatedBy",  referencingUrn=thisUrn)  
       # invisible( private$.implentatedBy)
      } else {
          private$.implentatedBy
      }
    },

    isDiscussedIn = function(val){
      if (!missing(val)){
        private$.isDiscussedIn <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isDiscussedIn, referencingAssociationName="isDiscussedIn",  referencingUrn=thisUrn)  
       # invisible( private$.isDiscussedIn)
      } else {
          private$.isDiscussedIn
      }
    },

    specifiesGoal = function(val){
      if (!missing(val)){
        private$.specifiesGoal <- returnUrnVectorIfValid(validClass="DDI4_Goal", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.specifiesGoal, referencingAssociationName="specifiesGoal",  referencingUrn=thisUrn)  
       # invisible( private$.specifiesGoal)
      } else {
          private$.specifiesGoal
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .overview = NULL,
    .isDiscussedIn = NULL,
    .expressesAlgorithm = NULL,
    .assumesPrecondition = NULL,
    .specifiesGoal = NULL,
    .implentatedBy = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        assumesPrecondition = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Precondition", 
          structure = "DdiUrn vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        expressesAlgorithm = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Algorithm", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        implentatedBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Process", 
          structure = "DdiUrn vector"
        ),
        isDiscussedIn = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        specifiesGoal = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Goal", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_Design


#' @title DDI4_DesignOverview  High level, descriptive, human informative, design statement The design may be used to specify how a process will be performed in general
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/DesignOverview}{DesignOverview} 

#'
#'   Definition
#'
#'   ============
#'
#'   High level, descriptive, human informative, design statement The design may be used to specify how a process will be performed in general. This would most commonly be used in a Codebook along with an AlgorithmOverview and a MethodologyOverview. The design informs a specific or implemented process as to its general parameters. Supports specification of any realization of Goal. 
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Allows for the use of any realized Process. The methodology, design, and algorithm of a specific realized process should be used if available. The use of a generic Process such as a WorkflowProcess containing an Act would be appropriate here. Restriction would be done by inclusion of the appropriate realized process class(es) in a Functional View.
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_DesignOverview$DdiUrn} returns the DDI URN of myDDI4_DesignOverview.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{subectOfDesign}}{ A read/write active binding.
#'        A term describing the subject of the design. Supports the use of an external controlled vocabulary
#'
#'                              \code{myDDI4_DesignOverview$subectOfDesign <- list(myExternalControlledVocabularyEntry)} sets the 'subectOfDesign' property of 'myDDI4_DesignOverview' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_DesignOverview$subectOfDesign} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{overview}}{ A read/write active binding.
#'        Short natural language account of the information obtained from the combination of properties and relationships associated with an object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_DesignOverview$overview <- list(myInternationalStructuredString)} sets the 'overview' property of 'myDDI4_DesignOverview' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_DesignOverview$overview} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isDescribedIn}}{ A read/write active binding.
#'          An external document or other source that provides descriptive information on the object.
#'
#'                              \code{myDDI4_DesignOverview$isDescribedIn <- c(myExternalMaterialURN)} sets the association 'isDescribedIn' from 'myDDI4_DesignOverview' to a character vector of DdiUrns for R6 ExternalMaterial class instances
#'
#'                              \code{myDDI4_DesignOverview$isDescribedIn} returns a character vector of of DdiUrns for R6  ExternalMaterial class instances}
#'   }
#'   \describe{
#'     \item{\code{assumesPrecondition}}{ A read/write active binding.
#'          The description of a Precondition that must exist prior to the use of this design
#'
#'                              \code{myDDI4_DesignOverview$assumesPrecondition <- c(myPreconditionURN)} sets the association 'assumesPrecondition' from 'myDDI4_DesignOverview' to a character vector of DdiUrns for R6 Precondition class instances
#'
#'                              \code{myDDI4_DesignOverview$assumesPrecondition} returns a character vector of of DdiUrns for R6  Precondition class instances}
#'   }
#'   \describe{
#'     \item{\code{specifiesGoal}}{ A read/write active binding.
#'          A description of the goal of the design. The goal may be used to evaluate the design, or implementations of the design, in terms of its ability to meet the goal.
#'
#'                              \code{myDDI4_DesignOverview$specifiesGoal <- c(myGoalURN)} sets the association 'specifiesGoal' from 'myDDI4_DesignOverview' to a character vector of DdiUrns for R6 Goal class instances
#'
#'                              \code{myDDI4_DesignOverview$specifiesGoal} returns a character vector of of DdiUrns for R6  Goal class instances}
#'   }
#'   \describe{
#'     \item{\code{expressesAlgorithm}}{ A read/write active binding.
#'          The algorithm that is expressed by the design. Constraint of the pattern relationship expressesAlgorithm by limiting the target to a DescribedAlgorithm.
#'
#'                              \code{myDDI4_DesignOverview$expressesAlgorithm <- c(myAlgorithmOverviewURN)} sets the association 'expressesAlgorithm' from 'myDDI4_DesignOverview' to a character vector of DdiUrns for R6 AlgorithmOverview class instances
#'
#'                              \code{myDDI4_DesignOverview$expressesAlgorithm} returns a character vector of of DdiUrns for R6  AlgorithmOverview class instances}
#'   }
#'   \describe{
#'     \item{\code{implementedBy}}{ A read/write active binding.
#'          The process used to implement the Design. WorkflowProcess is a generic realization of the ProcessPattern.
#'
#'                              \code{myDDI4_DesignOverview$implementedBy <- c(myWorkflowProcessURN)} sets the association 'implementedBy' from 'myDDI4_DesignOverview' to a character vector of DdiUrns for R6 WorkflowProcess class instances
#'
#'                              \code{myDDI4_DesignOverview$implementedBy} returns a character vector of of DdiUrns for R6  WorkflowProcess class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_DesignOverview$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, subectOfDesign=NA, overview=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_DesignOverview <- R6Class(classname="DDI4_DesignOverview",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, subectOfDesign=NA, overview=NA,    isDescribedIn=NA,assumesPrecondition=NA,specifiesGoal=NA,expressesAlgorithm=NA,implementedBy=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_DesignOverview may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(subectOfDesign)){
        private$.subectOfDesign <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=subectOfDesign , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(overview)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=overview , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(isDescribedIn)){
        private$.isDescribedIn <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=isDescribedIn , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isDescribedIn, referencingAssociationName="isDescribedIn",  referencingUrn=thisUrn)  
      }
      if(!missing(assumesPrecondition)){
        private$.assumesPrecondition <- returnUrnVectorIfValid(validClass="DDI4_Precondition", proposedUrnVector=assumesPrecondition , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.assumesPrecondition, referencingAssociationName="assumesPrecondition",  referencingUrn=thisUrn)  
      }
      if(!missing(specifiesGoal)){
        private$.specifiesGoal <- returnUrnVectorIfValid(validClass="DDI4_Goal", proposedUrnVector=specifiesGoal , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.specifiesGoal, referencingAssociationName="specifiesGoal",  referencingUrn=thisUrn)  
      }
      if(!missing(expressesAlgorithm)){
        private$.expressesAlgorithm <- returnUrnVectorIfValid(validClass="DDI4_AlgorithmOverview", proposedUrnVector=expressesAlgorithm , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.expressesAlgorithm, referencingAssociationName="expressesAlgorithm",  referencingUrn=thisUrn)  
      }
      if(!missing(implementedBy)){
        private$.implementedBy <- returnUrnVectorIfValid(validClass="DDI4_WorkflowProcess", proposedUrnVector=implementedBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.implementedBy, referencingAssociationName="implementedBy",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 2 properties: 
    overview = function(val){
      if (!missing(val)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overview
      }
    },

    subectOfDesign = function(val){
      if (!missing(val)){
        private$.subectOfDesign <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.subectOfDesign
      }
    },


# There are 5 class association(s):
    assumesPrecondition = function(val){
      if (!missing(val)){
        private$.assumesPrecondition <- returnUrnVectorIfValid(validClass="DDI4_Precondition", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.assumesPrecondition, referencingAssociationName="assumesPrecondition",  referencingUrn=thisUrn)  
       # invisible( private$.assumesPrecondition)
      } else {
          private$.assumesPrecondition
      }
    },

    expressesAlgorithm = function(val){
      if (!missing(val)){
        private$.expressesAlgorithm <- returnUrnVectorIfValid(validClass="DDI4_AlgorithmOverview", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.expressesAlgorithm, referencingAssociationName="expressesAlgorithm",  referencingUrn=thisUrn)  
       # invisible( private$.expressesAlgorithm)
      } else {
          private$.expressesAlgorithm
      }
    },

    implementedBy = function(val){
      if (!missing(val)){
        private$.implementedBy <- returnUrnVectorIfValid(validClass="DDI4_WorkflowProcess", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.implementedBy, referencingAssociationName="implementedBy",  referencingUrn=thisUrn)  
       # invisible( private$.implementedBy)
      } else {
          private$.implementedBy
      }
    },

    isDescribedIn = function(val){
      if (!missing(val)){
        private$.isDescribedIn <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isDescribedIn, referencingAssociationName="isDescribedIn",  referencingUrn=thisUrn)  
       # invisible( private$.isDescribedIn)
      } else {
          private$.isDescribedIn
      }
    },

    specifiesGoal = function(val){
      if (!missing(val)){
        private$.specifiesGoal <- returnUrnVectorIfValid(validClass="DDI4_Goal", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.specifiesGoal, referencingAssociationName="specifiesGoal",  referencingUrn=thisUrn)  
       # invisible( private$.specifiesGoal)
      } else {
          private$.specifiesGoal
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .subectOfDesign = NULL,
    .overview = NULL,
    .isDescribedIn = NULL,
    .assumesPrecondition = NULL,
    .specifiesGoal = NULL,
    .expressesAlgorithm = NULL,
    .implementedBy = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        assumesPrecondition = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Precondition", 
          structure = "DdiUrn vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        expressesAlgorithm = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AlgorithmOverview", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        implementedBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_WorkflowProcess", 
          structure = "DdiUrn vector"
        ),
        isDescribedIn = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        specifiesGoal = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Goal", 
          structure = "DdiUrn vector"
        ),
        subectOfDesign = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_DesignOverview


#' @title DDI4_Designation  A sign denoting a concept
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Designation}{Designation} 

#'
#'   Definition
#'
#'   ============
#'
#'   A sign denoting a concept.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A linking of the term “Unemployment” to an particular underlying concept.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   The representation of a concept by a sign (e.g., string, pictogram, bitmap) which denotes it. An example is the code M designating the marital status Married, which is a concept. In this context, M means Married.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Designation
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Designation$DdiUrn} returns the DDI URN of myDDI4_Designation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{representation}}{ A read/write active binding.
#'        A perceivable object used to denote a signified, i.e. a concept in this case.
#'
#'                              \code{myDDI4_Designation$representation <- list(mySignifier)} sets the 'representation' property of 'myDDI4_Designation' to a list of R6 Signifier class instances
#'
#'                              \code{myDDI4_Designation$representation} returns a list of of R6 : Signifier class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{denotes}}{ A read/write active binding.
#'          The concept denoted by the designation. 
#'
#'                              \code{myDDI4_Designation$denotes <- c(myConceptURN)} sets the association 'denotes' from 'myDDI4_Designation' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_Designation$denotes} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Designation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, representation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Designation <- R6Class(classname="DDI4_Designation",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, representation=NA,    denotes=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Designation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(representation)){
        private$.representation <- returnR6ObjectIfValid(validClass="DDI4_Signifier", proposedObjectList=representation , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(denotes)){
        private$.denotes <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=denotes , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.denotes, referencingAssociationName="denotes",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    representation = function(val){
      if (!missing(val)){
        private$.representation <- returnR6ObjectIfValid(validClass="DDI4_Signifier", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.representation
      }
    },


# There are 1 class association(s):
    denotes = function(val){
      if (!missing(val)){
        private$.denotes <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.denotes, referencingAssociationName="denotes",  referencingUrn=thisUrn)  
       # invisible( private$.denotes)
      } else {
          private$.denotes
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .representation = NULL,
    .denotes = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        denotes = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        representation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Signifier", 
          isAbstractDatatype = TRUE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_Designation


#' @title DDI4_DocumentInformation  Provides a consistent path to identifying information regarding the DDI Document and is automatically available for all Functional Views (/DocumentInformation/Annotation etc
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/DocumentInformation}{DocumentInformation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Provides a consistent path to identifying information regarding the DDI Document and is automatically available for all Functional Views (/DocumentInformation/Annotation etc.). It covers annotation information, coverage (general, plus specific spatial, temporal, and topical coverage), access information from the producer (persistent access control), access information from the local distributor (Local Access Control), information on related series (i.e. a study done as a series of data capture events over time, qualitative material that is part of a larger set, one of a series of funded data capture activities, etc.), funding information, and other document level information used to identify and discover information about a DDI document regardless of its Functional View type. Use the Annotation at this level to provide the full annotation of the DDI Document. Note that related materials can be entered within the Annotation fields.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Annotation associated with a specific class is the annotation of the metadata class itself, such as the Study, InstanceVariable, Concept, etc. The annotation associated with Document Information is to refer to the Document itself. Note that not all DDI instances are intended to be persistent documents. For those that are DocumentInformation provides a consistent set of discovery level information on the content of the whole rather than specific parts of the instance. 
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_DocumentInformation$DdiUrn} returns the DDI URN of myDDI4_DocumentInformation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{contentCoverage}}{ A read/write active binding.
#'        Allows for the addition of a Purpose, Table of Contents, List of Variables, or other forms of information more specific than a general abstract. Provides both the content (using description) and a type identification using an External Controlled Vocabulary Entry.
#'
#'                              \code{myDDI4_DocumentInformation$contentCoverage <- list(myTypedDescriptiveText)} sets the 'contentCoverage' property of 'myDDI4_DocumentInformation' to a list of R6 TypedDescriptiveText class instances
#'
#'                              \code{myDDI4_DocumentInformation$contentCoverage} returns a list of of R6 : TypedDescriptiveText class instances}
#'   }
#'   \describe{
#'     \item{\code{isPublished}}{ A read/write active binding.
#'        Default value is False. Indicates that the metadata instance will not be changed without versioning, and is a stable target for referencing.
#'
#'                              \code{myDDI4_DocumentInformation$isPublished <- mylogical} sets the isPublished property of 'myDDI4_DocumentInformation' to the vector of datatype:logical
#'
#'                              \code{myDDI4_DocumentInformation$isPublished} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{ofType}}{ A read/write active binding.
#'        Automatically brings in the version number for of DDI 4
#'
#'                              \code{myDDI4_DocumentInformation$ofType <- mycharacter} sets the ofType property of 'myDDI4_DocumentInformation' to the vector of datatype:character
#'
#'                              \code{myDDI4_DocumentInformation$ofType} returns a vector of datatype: character}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasDocumentCoverage}}{ A read/write active binding.
#'          Describes the coverage of the document as a whole.
#'
#'                              \code{myDDI4_DocumentInformation$hasDocumentCoverage <- c(myCoverageURN)} sets the association 'hasDocumentCoverage' from 'myDDI4_DocumentInformation' to a character vector of DdiUrns for R6 Coverage class instances
#'
#'                              \code{myDDI4_DocumentInformation$hasDocumentCoverage} returns a character vector of of DdiUrns for R6  Coverage class instances}
#'   }
#'   \describe{
#'     \item{\code{hasPersistentAccessControl}}{ A read/write active binding.
#'          Access control that will persist over time, generally imposed by the producer (i.e. embargo limits, confidentiality constraints). Provides information on access restrictions as well as the process to gain access.
#'
#'                              \code{myDDI4_DocumentInformation$hasPersistentAccessControl <- c(myAccessURN)} sets the association 'hasPersistentAccessControl' from 'myDDI4_DocumentInformation' to a character vector of DdiUrns for R6 Access class instances
#'
#'                              \code{myDDI4_DocumentInformation$hasPersistentAccessControl} returns a character vector of of DdiUrns for R6  Access class instances}
#'   }
#'   \describe{
#'     \item{\code{hasLocalAccessControl}}{ A read/write active binding.
#'          Access control imposed by the local distributor. For example, distribution only to registered users of a system or members of an association. Provides information on access restrictions as well as the process to gain access.
#'
#'                              \code{myDDI4_DocumentInformation$hasLocalAccessControl <- c(myAccessURN)} sets the association 'hasLocalAccessControl' from 'myDDI4_DocumentInformation' to a character vector of DdiUrns for R6 Access class instances
#'
#'                              \code{myDDI4_DocumentInformation$hasLocalAccessControl} returns a character vector of of DdiUrns for R6  Access class instances}
#'   }
#'   \describe{
#'     \item{\code{hasFundingSource}}{ A read/write active binding.
#'          Describes funding source by description, grant number, and link to funding agent
#'
#'                              \code{myDDI4_DocumentInformation$hasFundingSource <- c(myFundingInformationURN)} sets the association 'hasFundingSource' from 'myDDI4_DocumentInformation' to a character vector of DdiUrns for R6 FundingInformation class instances
#'
#'                              \code{myDDI4_DocumentInformation$hasFundingSource} returns a character vector of of DdiUrns for R6  FundingInformation class instances}
#'   }
#'   \describe{
#'     \item{\code{hasExternalMaterial}}{ A read/write active binding.
#'          Allows for listing information on any material external to the DDI metadata instance that is related to it.
#'
#'                              \code{myDDI4_DocumentInformation$hasExternalMaterial <- c(myExternalMaterialURN)} sets the association 'hasExternalMaterial' from 'myDDI4_DocumentInformation' to a character vector of DdiUrns for R6 ExternalMaterial class instances
#'
#'                              \code{myDDI4_DocumentInformation$hasExternalMaterial} returns a character vector of of DdiUrns for R6  ExternalMaterial class instances}
#'   }
#'   \describe{
#'     \item{\code{hasPrimaryContent}}{ A read/write active binding.
#'          Links to DDI objects contained in an XML instance or RDF graph which could be considered the primary objects or entry points. In a Codebook View, for example, the top-level Codebook object, for example. In some views, such as the Agents View, there my be more than one primary object (Individuals, Organizations, and Machines in this case).
#'
#'                              \code{myDDI4_DocumentInformation$hasPrimaryContent <- c(myIdentifiableURN)} sets the association 'hasPrimaryContent' from 'myDDI4_DocumentInformation' to a character vector of DdiUrns for R6 Identifiable class instances
#'
#'                              \code{myDDI4_DocumentInformation$hasPrimaryContent} returns a character vector of of DdiUrns for R6  Identifiable class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_DocumentInformation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, contentCoverage=NA, isPublished=NA, ofType=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_DocumentInformation <- R6Class(classname="DDI4_DocumentInformation",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, contentCoverage=NA, isPublished=NA, ofType=NA,    hasDocumentCoverage=NA,hasPersistentAccessControl=NA,hasLocalAccessControl=NA,hasFundingSource=NA,hasExternalMaterial=NA,hasPrimaryContent=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_DocumentInformation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(contentCoverage)){
        private$.contentCoverage <- returnR6ObjectIfValid(validClass="DDI4_TypedDescriptiveText", proposedObjectList=contentCoverage , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(isPublished)){
        private$.isPublished <- returnVectorIfValid(validVectorClass="logical", proposedVector=isPublished , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(ofType)){
        private$.ofType <- returnVectorIfValid(validVectorClass="character", proposedVector=ofType , minimumCardinality=1, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(hasDocumentCoverage)){
        private$.hasDocumentCoverage <- returnUrnVectorIfValid(validClass="DDI4_Coverage", proposedUrnVector=hasDocumentCoverage , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasDocumentCoverage, referencingAssociationName="hasDocumentCoverage",  referencingUrn=thisUrn)  
      }
      if(!missing(hasPersistentAccessControl)){
        private$.hasPersistentAccessControl <- returnUrnVectorIfValid(validClass="DDI4_Access", proposedUrnVector=hasPersistentAccessControl , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasPersistentAccessControl, referencingAssociationName="hasPersistentAccessControl",  referencingUrn=thisUrn)  
      }
      if(!missing(hasLocalAccessControl)){
        private$.hasLocalAccessControl <- returnUrnVectorIfValid(validClass="DDI4_Access", proposedUrnVector=hasLocalAccessControl , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasLocalAccessControl, referencingAssociationName="hasLocalAccessControl",  referencingUrn=thisUrn)  
      }
      if(!missing(hasFundingSource)){
        private$.hasFundingSource <- returnUrnVectorIfValid(validClass="DDI4_FundingInformation", proposedUrnVector=hasFundingSource , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasFundingSource, referencingAssociationName="hasFundingSource",  referencingUrn=thisUrn)  
      }
      if(!missing(hasExternalMaterial)){
        private$.hasExternalMaterial <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=hasExternalMaterial , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasExternalMaterial, referencingAssociationName="hasExternalMaterial",  referencingUrn=thisUrn)  
      }
      if(!missing(hasPrimaryContent)){
        private$.hasPrimaryContent <- returnUrnVectorIfValid(validClass="DDI4_Identifiable", proposedUrnVector=hasPrimaryContent , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasPrimaryContent, referencingAssociationName="hasPrimaryContent",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    contentCoverage = function(val){
      if (!missing(val)){
        private$.contentCoverage <- returnR6ObjectIfValid(validClass="DDI4_TypedDescriptiveText", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contentCoverage
      }
    },

    isPublished = function(val){
      if (!missing(val)){
        private$.isPublished <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.isPublished
      }
    },

    ofType = function(val){
      if (!missing(val)){
        private$.ofType <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.ofType
      }
    },


# There are 6 class association(s):
    hasDocumentCoverage = function(val){
      if (!missing(val)){
        private$.hasDocumentCoverage <- returnUrnVectorIfValid(validClass="DDI4_Coverage", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasDocumentCoverage, referencingAssociationName="hasDocumentCoverage",  referencingUrn=thisUrn)  
       # invisible( private$.hasDocumentCoverage)
      } else {
          private$.hasDocumentCoverage
      }
    },

    hasExternalMaterial = function(val){
      if (!missing(val)){
        private$.hasExternalMaterial <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasExternalMaterial, referencingAssociationName="hasExternalMaterial",  referencingUrn=thisUrn)  
       # invisible( private$.hasExternalMaterial)
      } else {
          private$.hasExternalMaterial
      }
    },

    hasFundingSource = function(val){
      if (!missing(val)){
        private$.hasFundingSource <- returnUrnVectorIfValid(validClass="DDI4_FundingInformation", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasFundingSource, referencingAssociationName="hasFundingSource",  referencingUrn=thisUrn)  
       # invisible( private$.hasFundingSource)
      } else {
          private$.hasFundingSource
      }
    },

    hasLocalAccessControl = function(val){
      if (!missing(val)){
        private$.hasLocalAccessControl <- returnUrnVectorIfValid(validClass="DDI4_Access", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasLocalAccessControl, referencingAssociationName="hasLocalAccessControl",  referencingUrn=thisUrn)  
       # invisible( private$.hasLocalAccessControl)
      } else {
          private$.hasLocalAccessControl
      }
    },

    hasPersistentAccessControl = function(val){
      if (!missing(val)){
        private$.hasPersistentAccessControl <- returnUrnVectorIfValid(validClass="DDI4_Access", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasPersistentAccessControl, referencingAssociationName="hasPersistentAccessControl",  referencingUrn=thisUrn)  
       # invisible( private$.hasPersistentAccessControl)
      } else {
          private$.hasPersistentAccessControl
      }
    },

    hasPrimaryContent = function(val){
      if (!missing(val)){
        private$.hasPrimaryContent <- returnUrnVectorIfValid(validClass="DDI4_Identifiable", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasPrimaryContent, referencingAssociationName="hasPrimaryContent",  referencingUrn=thisUrn)  
       # invisible( private$.hasPrimaryContent)
      } else {
          private$.hasPrimaryContent
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .contentCoverage = NULL,
    .isPublished = NULL,
    .ofType = NULL,
    .hasDocumentCoverage = NULL,
    .hasPersistentAccessControl = NULL,
    .hasLocalAccessControl = NULL,
    .hasFundingSource = NULL,
    .hasExternalMaterial = NULL,
    .hasPrimaryContent = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contentCoverage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_TypedDescriptiveText", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasDocumentCoverage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Coverage", 
          structure = "DdiUrn vector"
        ),
        hasExternalMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        hasFundingSource = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_FundingInformation", 
          structure = "DdiUrn vector"
        ),
        hasLocalAccessControl = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Access", 
          structure = "DdiUrn vector"
        ),
        hasPersistentAccessControl = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Access", 
          structure = "DdiUrn vector"
        ),
        hasPrimaryContent = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Identifiable", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPublished = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        ofType = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_DocumentInformation


#' @title DDI4_DoubleNumberRangeValue  Describes a bounding value for a number range expressed as an xs:double
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/DoubleNumberRangeValue}{DoubleNumberRangeValue} 

#'
#'   Definition
#'
#'   ============
#'
#'   Describes a bounding value for a number range expressed as an xs:double.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:DoubleNumberRangeValueType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_DoubleNumberRangeValue$DdiUrn} returns the DDI URN of myDDI4_DoubleNumberRangeValue.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{isInclusive}}{ A read/write active binding.
#'        Indicates that the value is included in the range. Set to false if the range includes numbers up to but no including the designated value.
#'
#'                              \code{myDDI4_DoubleNumberRangeValue$isInclusive <- mylogical} sets the isInclusive property of 'myDDI4_DoubleNumberRangeValue' to the vector of datatype:logical
#'
#'                              \code{myDDI4_DoubleNumberRangeValue$isInclusive} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{doubleValue}}{ A read/write active binding.
#'        The bounding value expressed as an xs:double
#'
#'                              \code{myDDI4_DoubleNumberRangeValue$doubleValue <- mynumeric} sets the doubleValue property of 'myDDI4_DoubleNumberRangeValue' to the vector of datatype:numeric
#'
#'                              \code{myDDI4_DoubleNumberRangeValue$doubleValue} returns a vector of datatype: numeric}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_DoubleNumberRangeValue$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, isInclusive=NA, doubleValue=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_DoubleNumberRangeValue <- R6Class(classname="DDI4_DoubleNumberRangeValue",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, isInclusive=NA, doubleValue=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_DoubleNumberRangeValue may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(isInclusive)){
        private$.isInclusive <- returnVectorIfValid(validVectorClass="logical", proposedVector=isInclusive , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(doubleValue)){
        private$.doubleValue <- returnVectorIfValid(validVectorClass="numeric", proposedVector=doubleValue , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    doubleValue = function(val){
      if (!missing(val)){
        private$.doubleValue <- returnVectorIfValid(validVectorClass="numeric", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.doubleValue
      }
    },

    isInclusive = function(val){
      if (!missing(val)){
        private$.isInclusive <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isInclusive
      }
    }


  ),

  private = list(
    .isInclusive = NULL,
    .doubleValue = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        doubleValue = c(
          min="0", 
          max="1", 
          datatype = "numeric", 
          structure = "vector"
        ),
        isInclusive = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_DoubleNumberRangeValue


#' @title DDI4_DynamicText  Structure supporting the use of dynamic text, where portions of the textual content change depending on external information (pre-loaded data, response to an earlier query, environmental situations, etc
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/DynamicText}{DynamicText} 

#'
#'   Definition
#'
#'   ============
#'
#'   Structure supporting the use of dynamic text, where portions of the textual content change depending on external information (pre-loaded data, response to an earlier query, environmental situations, etc.). 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Purpose: Provide customized number of hours within a question text
#'
#'   Input is the OutputParameter from Question "What time did you take this medication?"
#'
#'   Literal Text: "During the past " [0]
#'
#'   Conditional Text / Command: PastTime=Input; Round PastTime to nearest hour; Round CurrentTime to nearest hour; CurrentTime - PastTime = HOURS; content=HOURS [1]
#'
#'   Literal Text: " hours which of the following symptoms have you experienced?" [2]
#'
#'   
#'
#'   Result in question text for a PastTime= 06:10; CurrentTime=14:54
#'
#'   "During the past 9 hours which of the following symptoms have you experienced?"
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   d:DynamicTextType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_DynamicText$DdiUrn} returns the DDI URN of myDDI4_DynamicText.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{textContent}}{ A read/write active binding.
#'        This is the head of a substitution group and is never used directly as an element name. Instead it is replaced with either LiteralText or ConditionalText.
#'
#'                              \code{myDDI4_DynamicText$textContent <- list(myDynamicTextContent)} sets the 'textContent' property of 'myDDI4_DynamicText' to a list of R6 DynamicTextContent class instances
#'
#'                              \code{myDDI4_DynamicText$textContent} returns a list of of R6 : DynamicTextContent class instances}
#'   }
#'   \describe{
#'     \item{\code{isStructureRequired}}{ A read/write active binding.
#'        If textual structure (e.g. size, color, font, etc.) is required to understand the meaning of the content change value to "true".
#'
#'                              \code{myDDI4_DynamicText$isStructureRequired <- mylogical} sets the isStructureRequired property of 'myDDI4_DynamicText' to the vector of datatype:logical
#'
#'                              \code{myDDI4_DynamicText$isStructureRequired} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{audienceLanguage}}{ A read/write active binding.
#'        Specifies the language of the intended audience. This is particularly important for clarifying the primary language of a mixed language textual string, for example when language testing and using a foreign word withing the question text. Supports the inclusion of multiple languages.
#'
#'                              \code{myDDI4_DynamicText$audienceLanguage <- mycharacter} sets the audienceLanguage property of 'myDDI4_DynamicText' to the vector of datatype:character
#'
#'                              \code{myDDI4_DynamicText$audienceLanguage} returns a vector of datatype: character}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_DynamicText$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, textContent=NA, isStructureRequired=NA, audienceLanguage=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_DynamicText <- R6Class(classname="DDI4_DynamicText",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, textContent=NA, isStructureRequired=NA, audienceLanguage=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_DynamicText may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(textContent)){
        private$.textContent <- returnR6ObjectIfValid(validClass="DDI4_DynamicTextContent", proposedObjectList=textContent , minimumCardinality=1, maximumCardinality=Inf)  
      }
      if(!missing(isStructureRequired)){
        private$.isStructureRequired <- returnVectorIfValid(validVectorClass="logical", proposedVector=isStructureRequired , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(audienceLanguage)){
        private$.audienceLanguage <- returnVectorIfValid(validVectorClass="character", proposedVector=audienceLanguage , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    audienceLanguage = function(val){
      if (!missing(val)){
        private$.audienceLanguage <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.audienceLanguage
      }
    },

    isStructureRequired = function(val){
      if (!missing(val)){
        private$.isStructureRequired <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isStructureRequired
      }
    },

    textContent = function(val){
      if (!missing(val)){
        private$.textContent <- returnR6ObjectIfValid(validClass="DDI4_DynamicTextContent", proposedObjectList=val , minimumCardinality=1, maximumCardinality=Inf)  
      } else {
        private$.textContent
      }
    }


  ),

  private = list(
    .textContent = NULL,
    .isStructureRequired = NULL,
    .audienceLanguage = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        audienceLanguage = c(
          min="0", 
          max="Inf", 
          datatype = "character", 
          structure = "vector"
        ),
        isStructureRequired = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        textContent = c(
          min="1", 
          max="Inf", 
          datatype = "DDI4_DynamicTextContent", 
          isAbstractDatatype = TRUE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_DynamicText


#' @title DDI4_DynamicTextContent  Abstract type existing as the head of a substitution group
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/DynamicTextContent}{DynamicTextContent} 

#'
#'   Definition
#'
#'   ============
#'
#'   Abstract type existing as the head of a substitution group. May be replaced by any valid member of the substitution group TextContent. Provides the common property of purpose to all members using TextContent as an extension base. Provides a Purpose for the content and ordering information within the Dynamic Text. May be a LiteralText or ConditionalText.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   d:TextContentType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_DynamicTextContent$DdiUrn} returns the DDI URN of myDDI4_DynamicTextContent.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured tex
#'
#'                              \code{myDDI4_DynamicTextContent$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_DynamicTextContent' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_DynamicTextContent$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{orderPosition}}{ A read/write active binding.
#'        Provides the relative order of TextContent objects in a dynamic text with more than one TextContent object. Uses integer value.
#'
#'                              \code{myDDI4_DynamicTextContent$orderPosition <- myinteger} sets the orderPosition property of 'myDDI4_DynamicTextContent' to the vector of datatype:integer
#'
#'                              \code{myDDI4_DynamicTextContent$orderPosition} returns a vector of datatype: integer}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_DynamicTextContent$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, purpose=NA, orderPosition=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_DynamicTextContent <- R6Class(classname="DDI4_DynamicTextContent",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, purpose=NA, orderPosition=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_DynamicTextContent may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(orderPosition)){
        private$.orderPosition <- returnVectorIfValid(validVectorClass="integer", proposedVector=orderPosition , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    orderPosition = function(val){
      if (!missing(val)){
        private$.orderPosition <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.orderPosition
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    }


  ),

  private = list(
    .purpose = NULL,
    .orderPosition = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        orderPosition = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_DynamicTextContent


#' @title DDI4_ElectronicMessageSystem  Any non-email means of relaying a message electronically
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ElectronicMessageSystem}{ElectronicMessageSystem} 

#'
#'   Definition
#'
#'   ============
#'
#'   Any non-email means of relaying a message electronically. This would include text messaging, Skype, Twitter, ICQ, or other emerging means of electronic message conveyance. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Skype account, etc.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   a:InstantMessagingType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ElectronicMessageSystem$DdiUrn} returns the DDI URN of myDDI4_ElectronicMessageSystem.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{contactAddress}}{ A read/write active binding.
#'         Account identification for contacting
#'
#'                              \code{myDDI4_ElectronicMessageSystem$contactAddress <- mycharacter} sets the contactAddress property of 'myDDI4_ElectronicMessageSystem' to the vector of datatype:character
#'
#'                              \code{myDDI4_ElectronicMessageSystem$contactAddress} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{typeOfService}}{ A read/write active binding.
#'        Indicates the type of service used. Supports the use of a controlled vocabulary.
#'
#'                              \code{myDDI4_ElectronicMessageSystem$typeOfService <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfService' property of 'myDDI4_ElectronicMessageSystem' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ElectronicMessageSystem$typeOfService} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{effectiveDates}}{ A read/write active binding.
#'        Time period during which the account is valid.
#'
#'                              \code{myDDI4_ElectronicMessageSystem$effectiveDates <- list(myDateRange)} sets the 'effectiveDates' property of 'myDDI4_ElectronicMessageSystem' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_ElectronicMessageSystem$effectiveDates} returns a list of of R6 : DateRange class instances}
#'   }
#'   \describe{
#'     \item{\code{privacy}}{ A read/write active binding.
#'        Specify the level privacy for the address as public, restricted, or private. Supports the use of an external controlled vocabulary.
#'
#'                              \code{myDDI4_ElectronicMessageSystem$privacy <- list(myExternalControlledVocabularyEntry)} sets the 'privacy' property of 'myDDI4_ElectronicMessageSystem' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ElectronicMessageSystem$privacy} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{isPreferred}}{ A read/write active binding.
#'        Set to "true" if this is the preferred address.
#'
#'                              \code{myDDI4_ElectronicMessageSystem$isPreferred <- mylogical} sets the isPreferred property of 'myDDI4_ElectronicMessageSystem' to the vector of datatype:logical
#'
#'                              \code{myDDI4_ElectronicMessageSystem$isPreferred} returns a vector of datatype: logical}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ElectronicMessageSystem$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, contactAddress=NA, typeOfService=NA, effectiveDates=NA, privacy=NA, isPreferred=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ElectronicMessageSystem <- R6Class(classname="DDI4_ElectronicMessageSystem",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, contactAddress=NA, typeOfService=NA, effectiveDates=NA, privacy=NA, isPreferred=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ElectronicMessageSystem may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(contactAddress)){
        private$.contactAddress <- returnVectorIfValid(validVectorClass="character", proposedVector=contactAddress , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(typeOfService)){
        private$.typeOfService <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfService , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(effectiveDates)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=effectiveDates , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(privacy)){
        private$.privacy <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=privacy , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isPreferred)){
        private$.isPreferred <- returnVectorIfValid(validVectorClass="logical", proposedVector=isPreferred , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 5 properties: 
    contactAddress = function(val){
      if (!missing(val)){
        private$.contactAddress <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.contactAddress
      }
    },

    effectiveDates = function(val){
      if (!missing(val)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.effectiveDates
      }
    },

    isPreferred = function(val){
      if (!missing(val)){
        private$.isPreferred <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isPreferred
      }
    },

    privacy = function(val){
      if (!missing(val)){
        private$.privacy <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.privacy
      }
    },

    typeOfService = function(val){
      if (!missing(val)){
        private$.typeOfService <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.typeOfService
      }
    }


  ),

  private = list(
    .contactAddress = NULL,
    .typeOfService = NULL,
    .effectiveDates = NULL,
    .privacy = NULL,
    .isPreferred = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        contactAddress = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        effectiveDates = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        isPreferred = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        privacy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        typeOfService = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ElectronicMessageSystem


#' @title DDI4_ElseIfAction  An additional condition and resulting step to take if the condition in the parent IfThenElse is false
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ElseIfAction}{ElseIfAction} 

#'
#'   Definition
#'
#'   ============
#'
#'   An additional condition and resulting step to take if the condition in the parent IfThenElse is false. It cannot exist without a parent IfThenElse.
#'
#'   
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ElseIfAction$DdiUrn} returns the DDI URN of myDDI4_ElseIfAction.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{condition}}{ A read/write active binding.
#'        The condition to verify if the parent IfThenElse condition is false.
#'
#'                              \code{myDDI4_ElseIfAction$condition <- list(myCommandCode)} sets the 'condition' property of 'myDDI4_ElseIfAction' to a list of R6 CommandCode class instances
#'
#'                              \code{myDDI4_ElseIfAction$condition} returns a list of of R6 : CommandCode class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{executes}}{ A read/write active binding.
#'          The WorkflowStepSequence to execute if the condition is true
#'
#'                              \code{myDDI4_ElseIfAction$executes <- c(myWorkflowStepSequenceURN)} sets the association 'executes' from 'myDDI4_ElseIfAction' to a character vector of DdiUrns for R6 WorkflowStepSequence class instances
#'
#'                              \code{myDDI4_ElseIfAction$executes} returns a character vector of of DdiUrns for R6  WorkflowStepSequence class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ElseIfAction$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, condition=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ElseIfAction <- R6Class(classname="DDI4_ElseIfAction",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, condition=NA,    executes=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ElseIfAction may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(condition)){
        private$.condition <- returnR6ObjectIfValid(validClass="DDI4_CommandCode", proposedObjectList=condition , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(executes)){
        private$.executes <- returnUrnVectorIfValid(validClass="DDI4_WorkflowStepSequence", proposedUrnVector=executes , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.executes, referencingAssociationName="executes",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    condition = function(val){
      if (!missing(val)){
        private$.condition <- returnR6ObjectIfValid(validClass="DDI4_CommandCode", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.condition
      }
    },


# There are 1 class association(s):
    executes = function(val){
      if (!missing(val)){
        private$.executes <- returnUrnVectorIfValid(validClass="DDI4_WorkflowStepSequence", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.executes, referencingAssociationName="executes",  referencingUrn=thisUrn)  
       # invisible( private$.executes)
      } else {
          private$.executes
      }
    }

  ),

  private = list(
    .condition = NULL,
    .executes = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        condition = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CommandCode", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        executes = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowStepSequence", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ElseIfAction


#' @title DDI4_Email  An e-mail address which conforms to the internet format (RFC 822) including its type and time period for which it is valid
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Email}{Email} 

#'
#'   Definition
#'
#'   ============
#'
#'   An e-mail address which conforms to the internet format (RFC 822) including its type and time period for which it is valid.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   info@ddialliance.org; ex.ample@somewhere.org
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   e-mail address; internet email address
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:EmailType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Email$DdiUrn} returns the DDI URN of myDDI4_Email.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{internetEmail}}{ A read/write active binding.
#'        The email address expressed as a string (should follow the Internet format specification - RFC 5322) e.g. user@server.ext, more complex and flexible examples are also supported by the format.
#'
#'                              \code{myDDI4_Email$internetEmail <- mycharacter} sets the internetEmail property of 'myDDI4_Email' to the vector of datatype:character
#'
#'                              \code{myDDI4_Email$internetEmail} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{typeOfEmail}}{ A read/write active binding.
#'        Code indicating the type of e-mail address. Supports the use of an external controlled vocabulary. (e.g. home, office)
#'
#'                              \code{myDDI4_Email$typeOfEmail <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfEmail' property of 'myDDI4_Email' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_Email$typeOfEmail} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{effectiveDates}}{ A read/write active binding.
#'        Time period for which the e-mail address is valid.
#'
#'                              \code{myDDI4_Email$effectiveDates <- list(myDateRange)} sets the 'effectiveDates' property of 'myDDI4_Email' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_Email$effectiveDates} returns a list of of R6 : DateRange class instances}
#'   }
#'   \describe{
#'     \item{\code{privacy}}{ A read/write active binding.
#'        Indicates the level of privacy
#'
#'                              \code{myDDI4_Email$privacy <- list(myExternalControlledVocabularyEntry)} sets the 'privacy' property of 'myDDI4_Email' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_Email$privacy} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{isPreferred}}{ A read/write active binding.
#'        Set to true if this is the preferred email
#'
#'                              \code{myDDI4_Email$isPreferred <- mylogical} sets the isPreferred property of 'myDDI4_Email' to the vector of datatype:logical
#'
#'                              \code{myDDI4_Email$isPreferred} returns a vector of datatype: logical}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Email$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, internetEmail=NA, typeOfEmail=NA, effectiveDates=NA, privacy=NA, isPreferred=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Email <- R6Class(classname="DDI4_Email",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, internetEmail=NA, typeOfEmail=NA, effectiveDates=NA, privacy=NA, isPreferred=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Email may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(internetEmail)){
        private$.internetEmail <- returnVectorIfValid(validVectorClass="character", proposedVector=internetEmail , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(typeOfEmail)){
        private$.typeOfEmail <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfEmail , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(effectiveDates)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=effectiveDates , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(privacy)){
        private$.privacy <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=privacy , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isPreferred)){
        private$.isPreferred <- returnVectorIfValid(validVectorClass="logical", proposedVector=isPreferred , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 5 properties: 
    effectiveDates = function(val){
      if (!missing(val)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.effectiveDates
      }
    },

    internetEmail = function(val){
      if (!missing(val)){
        private$.internetEmail <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.internetEmail
      }
    },

    isPreferred = function(val){
      if (!missing(val)){
        private$.isPreferred <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isPreferred
      }
    },

    privacy = function(val){
      if (!missing(val)){
        private$.privacy <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.privacy
      }
    },

    typeOfEmail = function(val){
      if (!missing(val)){
        private$.typeOfEmail <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.typeOfEmail
      }
    }


  ),

  private = list(
    .internetEmail = NULL,
    .typeOfEmail = NULL,
    .effectiveDates = NULL,
    .privacy = NULL,
    .isPreferred = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        effectiveDates = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        internetEmail = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPreferred = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        privacy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        typeOfEmail = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Email


#' @title DDI4_Embargo  Provides information about data that are not currently available because of policies established by the principal investigators and/or data producers
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Embargo}{Embargo} 

#'
#'   Definition
#'
#'   ============
#'
#'   Provides information about data that are not currently available because of policies established by the principal investigators and/or data producers. Embargo provides a name and label of the embargo, the dates covered by the embargo, the rationale or reason for the embargo, a reference to the agency establishing the embargo, and a reference to the agency enforcing the embargo.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:EmbargoType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Embargo$DdiUrn} returns the DDI URN of myDDI4_Embargo.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A structured display label providing a fully human readable display for the identification of the object. Supports the use of multiple languages and structured text. 
#'
#'                              \code{myDDI4_Embargo$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_Embargo' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_Embargo$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#'   \describe{
#'     \item{\code{embargoDates}}{ A read/write active binding.
#'        Provides the date range of the embargo
#'
#'                              \code{myDDI4_Embargo$embargoDates <- list(myDateRange)} sets the 'embargoDates' property of 'myDDI4_Embargo' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_Embargo$embargoDates} returns a list of of R6 : DateRange class instances}
#'   }
#'   \describe{
#'     \item{\code{rationale}}{ A read/write active binding.
#'        Explanation of the reasons some decision was made or some object exists. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_Embargo$rationale <- list(myInternationalStructuredString)} sets the 'rationale' property of 'myDDI4_Embargo' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Embargo$rationale} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{responsibleAgent}}{ A read/write active binding.
#'          Agent (Organiztion, Individual, etc.) that is responsible for the embargo
#'
#'                              \code{myDDI4_Embargo$responsibleAgent <- c(myAgentURN)} sets the association 'responsibleAgent' from 'myDDI4_Embargo' to a character vector of DdiUrns for R6 Agent class instances
#'
#'                              \code{myDDI4_Embargo$responsibleAgent} returns a character vector of of DdiUrns for R6  Agent class instances}
#'   }
#'   \describe{
#'     \item{\code{enforcementAgent}}{ A read/write active binding.
#'          Agent responsible for enforcing the embargo
#'
#'                              \code{myDDI4_Embargo$enforcementAgent <- c(myAgentURN)} sets the association 'enforcementAgent' from 'myDDI4_Embargo' to a character vector of DdiUrns for R6 Agent class instances
#'
#'                              \code{myDDI4_Embargo$enforcementAgent} returns a character vector of of DdiUrns for R6  Agent class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Embargo$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, displayLabel=NA, embargoDates=NA, rationale=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Embargo <- R6Class(classname="DDI4_Embargo",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, displayLabel=NA, embargoDates=NA, rationale=NA,    responsibleAgent=NA,enforcementAgent=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Embargo may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(embargoDates)){
        private$.embargoDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=embargoDates , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(rationale)){
        private$.rationale <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=rationale , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(responsibleAgent)){
        private$.responsibleAgent <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=responsibleAgent , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.responsibleAgent, referencingAssociationName="responsibleAgent",  referencingUrn=thisUrn)  
      }
      if(!missing(enforcementAgent)){
        private$.enforcementAgent <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=enforcementAgent , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.enforcementAgent, referencingAssociationName="enforcementAgent",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },

    embargoDates = function(val){
      if (!missing(val)){
        private$.embargoDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.embargoDates
      }
    },

    rationale = function(val){
      if (!missing(val)){
        private$.rationale <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.rationale
      }
    },


# There are 2 class association(s):
    enforcementAgent = function(val){
      if (!missing(val)){
        private$.enforcementAgent <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.enforcementAgent, referencingAssociationName="enforcementAgent",  referencingUrn=thisUrn)  
       # invisible( private$.enforcementAgent)
      } else {
          private$.enforcementAgent
      }
    },

    responsibleAgent = function(val){
      if (!missing(val)){
        private$.responsibleAgent <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.responsibleAgent, referencingAssociationName="responsibleAgent",  referencingUrn=thisUrn)  
       # invisible( private$.responsibleAgent)
      } else {
          private$.responsibleAgent
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .displayLabel = NULL,
    .embargoDates = NULL,
    .rationale = NULL,
    .responsibleAgent = NULL,
    .enforcementAgent = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        embargoDates = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        enforcementAgent = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Agent", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        rationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        responsibleAgent = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Agent", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Embargo


#' @title DDI4_EnumerationDomain  An abstract base to allow all codifications (CodeList, StatisticalClassification, etc
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/EnumerationDomain}{EnumerationDomain} 

#'
#'   Definition
#'
#'   ============
#'
#'   An abstract base to allow all codifications (CodeList, StatisticalClassification, etc.) to be used as enumerated value representations.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_EnumerationDomain$DdiUrn} returns the DDI URN of myDDI4_EnumerationDomain.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{type}}{ A read/write active binding.
#'        Whether the collection is a bag or a set: a bag is a collection with duplicates allowed, a set is a collection without duplicates
#'
#'                              \code{myDDI4_EnumerationDomain$type <- list(myCollectionType)} sets the 'type' property of 'myDDI4_EnumerationDomain' to a list of R6 CollectionType class instances
#'
#'                              \code{myDDI4_EnumerationDomain$type} returns a list of of R6 : CollectionType class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided 
#'
#'                              \code{myDDI4_EnumerationDomain$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_EnumerationDomain' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_EnumerationDomain$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_EnumerationDomain$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_EnumerationDomain' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_EnumerationDomain$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{isOrdered}}{ A read/write active binding.
#'        Allows for the identification of the member and optionally provides an index for the member within an ordered array
#'
#'                              \code{myDDI4_EnumerationDomain$isOrdered <- mylogical} sets the isOrdered property of 'myDDI4_EnumerationDomain' to the vector of datatype:logical
#'
#'                              \code{myDDI4_EnumerationDomain$isOrdered} returns a vector of datatype: logical}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{references}}{ A read/write active binding.
#'          CategorySet associated with the enumeration
#'
#'                              \code{myDDI4_EnumerationDomain$references <- c(myCategorySetURN)} sets the association 'references' from 'myDDI4_EnumerationDomain' to a character vector of DdiUrns for R6 CategorySet class instances
#'
#'                              \code{myDDI4_EnumerationDomain$references} returns a character vector of of DdiUrns for R6  CategorySet class instances}
#'   }
#'   \describe{
#'     \item{\code{usesLevelStructure}}{ A read/write active binding.
#'          Has meaningful level to which members belong.
#'
#'                              \code{myDDI4_EnumerationDomain$usesLevelStructure <- c(myLevelStructureURN)} sets the association 'usesLevelStructure' from 'myDDI4_EnumerationDomain' to a character vector of DdiUrns for R6 LevelStructure class instances
#'
#'                              \code{myDDI4_EnumerationDomain$usesLevelStructure} returns a character vector of of DdiUrns for R6  LevelStructure class instances}
#'   }
#'   \describe{
#'     \item{\code{definingConcept}}{ A read/write active binding.
#'          The conceptual basis for the collection of members.
#'
#'                              \code{myDDI4_EnumerationDomain$definingConcept <- c(myConceptURN)} sets the association 'definingConcept' from 'myDDI4_EnumerationDomain' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_EnumerationDomain$definingConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_EnumerationDomain$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, type=NA, name=NA, purpose=NA, isOrdered=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_EnumerationDomain <- R6Class(classname="DDI4_EnumerationDomain",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, type=NA, name=NA, purpose=NA, isOrdered=NA,    references=NA,usesLevelStructure=NA,definingConcept=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_EnumerationDomain may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(type)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=type , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isOrdered)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=isOrdered , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(references)){
        private$.references <- returnUrnVectorIfValid(validClass="DDI4_CategorySet", proposedUrnVector=references , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.references, referencingAssociationName="references",  referencingUrn=thisUrn)  
      }
      if(!missing(usesLevelStructure)){
        private$.usesLevelStructure <- returnUrnVectorIfValid(validClass="DDI4_LevelStructure", proposedUrnVector=usesLevelStructure , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesLevelStructure, referencingAssociationName="usesLevelStructure",  referencingUrn=thisUrn)  
      }
      if(!missing(definingConcept)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=definingConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 4 properties: 
    isOrdered = function(val){
      if (!missing(val)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isOrdered
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    type = function(val){
      if (!missing(val)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.type
      }
    },


# There are 3 class association(s):
    definingConcept = function(val){
      if (!missing(val)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
       # invisible( private$.definingConcept)
      } else {
          private$.definingConcept
      }
    },

    references = function(val){
      if (!missing(val)){
        private$.references <- returnUrnVectorIfValid(validClass="DDI4_CategorySet", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.references, referencingAssociationName="references",  referencingUrn=thisUrn)  
       # invisible( private$.references)
      } else {
          private$.references
      }
    },

    usesLevelStructure = function(val){
      if (!missing(val)){
        private$.usesLevelStructure <- returnUrnVectorIfValid(validClass="DDI4_LevelStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesLevelStructure, referencingAssociationName="usesLevelStructure",  referencingUrn=thisUrn)  
       # invisible( private$.usesLevelStructure)
      } else {
          private$.usesLevelStructure
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .type = NULL,
    .name = NULL,
    .purpose = NULL,
    .isOrdered = NULL,
    .references = NULL,
    .usesLevelStructure = NULL,
    .definingConcept = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        references = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CategorySet", 
          structure = "DdiUrn vector"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        usesLevelStructure = c(
          min="0", 
          max="1", 
          datatype = "DDI4_LevelStructure", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_EnumerationDomain


#' @title DDI4_ExPostEvaluation  Evaluation for the purpose of reviewing the study, data collection, data processing, or management processes
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ExPostEvaluation}{ExPostEvaluation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Evaluation for the purpose of reviewing the study, data collection, data processing, or management processes. Results may feed into a revision process for future data collection or management. Identifies the type of evaluation undertaken, who did the evaluation, the evaluation process, outcomes and completion date.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:ExPostEvaluationType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ExPostEvaluation$DdiUrn} returns the DDI URN of myDDI4_ExPostEvaluation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{typeOfEvaluation}}{ A read/write active binding.
#'        Brief identification of the type of evaluation. Supports the use of an external controlled vocabulary.
#'
#'                              \code{myDDI4_ExPostEvaluation$typeOfEvaluation <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfEvaluation' property of 'myDDI4_ExPostEvaluation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ExPostEvaluation$typeOfEvaluation} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{evaluationProcess}}{ A read/write active binding.
#'        Describes the evaluation process. Supports multi-lingual content. Allows the optional use of structured content.
#'
#'                              \code{myDDI4_ExPostEvaluation$evaluationProcess <- list(myInternationalStructuredString)} sets the 'evaluationProcess' property of 'myDDI4_ExPostEvaluation' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_ExPostEvaluation$evaluationProcess} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{outcomes}}{ A read/write active binding.
#'        Describes the outcomes of the evaluation process. Supports multi-lingual content. Allows the optional use of structured content.
#'
#'                              \code{myDDI4_ExPostEvaluation$outcomes <- list(myInternationalStructuredString)} sets the 'outcomes' property of 'myDDI4_ExPostEvaluation' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_ExPostEvaluation$outcomes} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{completionDate}}{ A read/write active binding.
#'        Identifies the date the evaluation was completed.
#'
#'                              \code{myDDI4_ExPostEvaluation$completionDate <- list(myDate)} sets the 'completionDate' property of 'myDDI4_ExPostEvaluation' to a list of R6 Date class instances
#'
#'                              \code{myDDI4_ExPostEvaluation$completionDate} returns a list of of R6 : Date class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{evaluator}}{ A read/write active binding.
#'          Evaluator used in this process.
#'
#'                              \code{myDDI4_ExPostEvaluation$evaluator <- c(myAgentURN)} sets the association 'evaluator' from 'myDDI4_ExPostEvaluation' to a character vector of DdiUrns for R6 Agent class instances
#'
#'                              \code{myDDI4_ExPostEvaluation$evaluator} returns a character vector of of DdiUrns for R6  Agent class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ExPostEvaluation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, typeOfEvaluation=NA, evaluationProcess=NA, outcomes=NA, completionDate=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ExPostEvaluation <- R6Class(classname="DDI4_ExPostEvaluation",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, typeOfEvaluation=NA, evaluationProcess=NA, outcomes=NA, completionDate=NA,    evaluator=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ExPostEvaluation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(typeOfEvaluation)){
        private$.typeOfEvaluation <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfEvaluation , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(evaluationProcess)){
        private$.evaluationProcess <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=evaluationProcess , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(outcomes)){
        private$.outcomes <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=outcomes , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(completionDate)){
        private$.completionDate <- returnR6ObjectIfValid(validClass="DDI4_Date", proposedObjectList=completionDate , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(evaluator)){
        private$.evaluator <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=evaluator , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.evaluator, referencingAssociationName="evaluator",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 4 properties: 
    completionDate = function(val){
      if (!missing(val)){
        private$.completionDate <- returnR6ObjectIfValid(validClass="DDI4_Date", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.completionDate
      }
    },

    evaluationProcess = function(val){
      if (!missing(val)){
        private$.evaluationProcess <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.evaluationProcess
      }
    },

    outcomes = function(val){
      if (!missing(val)){
        private$.outcomes <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.outcomes
      }
    },

    typeOfEvaluation = function(val){
      if (!missing(val)){
        private$.typeOfEvaluation <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.typeOfEvaluation
      }
    },


# There are 1 class association(s):
    evaluator = function(val){
      if (!missing(val)){
        private$.evaluator <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.evaluator, referencingAssociationName="evaluator",  referencingUrn=thisUrn)  
       # invisible( private$.evaluator)
      } else {
          private$.evaluator
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .typeOfEvaluation = NULL,
    .evaluationProcess = NULL,
    .outcomes = NULL,
    .completionDate = NULL,
    .evaluator = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        completionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Date", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        evaluationProcess = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        evaluator = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Agent", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        outcomes = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        typeOfEvaluation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ExPostEvaluation


#' @title DDI4_ExternalAid  Any external material used in an instrument that aids or facilitates data capture, or that is presented to a respondent and about which measurements are made
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ExternalAid}{ExternalAid} 

#'
#'   Definition
#'
#'   ============
#'
#'   Any external material used in an instrument that aids or facilitates data capture, or that is presented to a respondent and about which measurements are made. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Image, link, external aid, stimulus, physical object.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:OtherMaterialType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ExternalMaterial}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ExternalAid$DdiUrn} returns the DDI URN of myDDI4_ExternalAid.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{stimulusType}}{ A read/write active binding.
#'        
#'
#'                              \code{myDDI4_ExternalAid$stimulusType <- list(myExternalControlledVocabularyEntry)} sets the 'stimulusType' property of 'myDDI4_ExternalAid' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ExternalAid$stimulusType} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ExternalAid$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, stimulusType=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ExternalAid <- R6Class(classname="DDI4_ExternalAid",
  inherit = DDI4_ExternalMaterial,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, stimulusType=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ExternalAid may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(stimulusType)){
        private$.stimulusType <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=stimulusType , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 1 properties: 
    stimulusType = function(val){
      if (!missing(val)){
        private$.stimulusType <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.stimulusType
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .stimulusType = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        citationOfExternalMaterial = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        descriptiveText = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        mimeType = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relationshipDescription = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        stimulusType = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        typeOfMaterial = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        uri = c(
          min="0", 
          max="Inf", 
          datatype = "character", 
          structure = "vector"
        ),
        usesSegment = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Segment", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ExternalAid


#' @title DDI4_ExternalControlledVocabularyEntry  Allows for unstructured content which may be an entry from an externally maintained controlled vocabulary
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ExternalControlledVocabularyEntry}{ExternalControlledVocabularyEntry} 

#'
#'   Definition
#'
#'   ============
#'
#'   Allows for unstructured content which may be an entry from an externally maintained controlled vocabulary.If the content is from a controlled vocabulary provide the code value of the entry, as well as a reference to the controlled vocabulary from which the value is taken. Provide as many of the identifying attributes as needed to adequately identify the controlled vocabulary. Note that DDI has published a number of controlled vocabularies applicable to several locations using the ExternalControlledVocabularyEntry structure. If the code portion of the controlled vocabulary entry is language specific (i.e. a list of keywords or subject headings) use language to specify that language. In most cases the code portion of an entry is not language specific although the description and usage may be managed in one or more languages. Use of shared controlled vocabularies helps support interoperability and machine actionability.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:CodeValueType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ExternalControlledVocabularyEntry$DdiUrn} returns the DDI URN of myDDI4_ExternalControlledVocabularyEntry.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{controlledVocabularyID}}{ A read/write active binding.
#'        The ID of the code list (controlled vocabulary) that the content was taken from.
#'
#'                              \code{myDDI4_ExternalControlledVocabularyEntry$controlledVocabularyID <- mycharacter} sets the controlledVocabularyID property of 'myDDI4_ExternalControlledVocabularyEntry' to the vector of datatype:character
#'
#'                              \code{myDDI4_ExternalControlledVocabularyEntry$controlledVocabularyID} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{controlledVocabularyName}}{ A read/write active binding.
#'        The name of the code list.
#'
#'                              \code{myDDI4_ExternalControlledVocabularyEntry$controlledVocabularyName <- mycharacter} sets the controlledVocabularyName property of 'myDDI4_ExternalControlledVocabularyEntry' to the vector of datatype:character
#'
#'                              \code{myDDI4_ExternalControlledVocabularyEntry$controlledVocabularyName} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{controlledVocabularyAgencyName}}{ A read/write active binding.
#'        The name of the agency maintaining the code list.
#'
#'                              \code{myDDI4_ExternalControlledVocabularyEntry$controlledVocabularyAgencyName <- mycharacter} sets the controlledVocabularyAgencyName property of 'myDDI4_ExternalControlledVocabularyEntry' to the vector of datatype:character
#'
#'                              \code{myDDI4_ExternalControlledVocabularyEntry$controlledVocabularyAgencyName} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{controlledVocabularyVersionID}}{ A read/write active binding.
#'        The version number of the code list (default is 1.0).
#'
#'                              \code{myDDI4_ExternalControlledVocabularyEntry$controlledVocabularyVersionID <- mycharacter} sets the controlledVocabularyVersionID property of 'myDDI4_ExternalControlledVocabularyEntry' to the vector of datatype:character
#'
#'                              \code{myDDI4_ExternalControlledVocabularyEntry$controlledVocabularyVersionID} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{otherValue}}{ A read/write active binding.
#'        If the value of the string is "Other" or the equivalent from the codelist, this attribute can provide a more specific value not found in the codelist.
#'
#'                              \code{myDDI4_ExternalControlledVocabularyEntry$otherValue <- mycharacter} sets the otherValue property of 'myDDI4_ExternalControlledVocabularyEntry' to the vector of datatype:character
#'
#'                              \code{myDDI4_ExternalControlledVocabularyEntry$otherValue} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{uri}}{ A read/write active binding.
#'        The URN or URL of the controlled vocabulary
#'
#'                              \code{myDDI4_ExternalControlledVocabularyEntry$uri <- mycharacter} sets the uri property of 'myDDI4_ExternalControlledVocabularyEntry' to the vector of datatype:character
#'
#'                              \code{myDDI4_ExternalControlledVocabularyEntry$uri} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{content}}{ A read/write active binding.
#'        The value of the entry of the controlled vocabulary. If no controlled vocabulary is used the term is entered here and none of the properties defining the controlled vocabulary location are used.
#'
#'                              \code{myDDI4_ExternalControlledVocabularyEntry$content <- mycharacter} sets the content property of 'myDDI4_ExternalControlledVocabularyEntry' to the vector of datatype:character
#'
#'                              \code{myDDI4_ExternalControlledVocabularyEntry$content} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{language}}{ A read/write active binding.
#'        Language of the content value if applicable
#'
#'                              \code{myDDI4_ExternalControlledVocabularyEntry$language <- mycharacter} sets the language property of 'myDDI4_ExternalControlledVocabularyEntry' to the vector of datatype:character
#'
#'                              \code{myDDI4_ExternalControlledVocabularyEntry$language} returns a vector of datatype: character}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ExternalControlledVocabularyEntry$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, controlledVocabularyID=NA, controlledVocabularyName=NA, controlledVocabularyAgencyName=NA, controlledVocabularyVersionID=NA, otherValue=NA, uri=NA, content=NA, language=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ExternalControlledVocabularyEntry <- R6Class(classname="DDI4_ExternalControlledVocabularyEntry",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, controlledVocabularyID=NA, controlledVocabularyName=NA, controlledVocabularyAgencyName=NA, controlledVocabularyVersionID=NA, otherValue=NA, uri=NA, content=NA, language=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ExternalControlledVocabularyEntry may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(controlledVocabularyID)){
        private$.controlledVocabularyID <- returnVectorIfValid(validVectorClass="character", proposedVector=controlledVocabularyID , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(controlledVocabularyName)){
        private$.controlledVocabularyName <- returnVectorIfValid(validVectorClass="character", proposedVector=controlledVocabularyName , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(controlledVocabularyAgencyName)){
        private$.controlledVocabularyAgencyName <- returnVectorIfValid(validVectorClass="character", proposedVector=controlledVocabularyAgencyName , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(controlledVocabularyVersionID)){
        private$.controlledVocabularyVersionID <- returnVectorIfValid(validVectorClass="character", proposedVector=controlledVocabularyVersionID , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(otherValue)){
        private$.otherValue <- returnVectorIfValid(validVectorClass="character", proposedVector=otherValue , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(uri)){
        private$.uri <- returnVectorIfValid(validVectorClass="character", proposedVector=uri , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(content)){
        private$.content <- returnVectorIfValid(validVectorClass="character", proposedVector=content , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(language)){
        private$.language <- returnVectorIfValid(validVectorClass="character", proposedVector=language , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 8 properties: 
    content = function(val){
      if (!missing(val)){
        private$.content <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.content
      }
    },

    controlledVocabularyAgencyName = function(val){
      if (!missing(val)){
        private$.controlledVocabularyAgencyName <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.controlledVocabularyAgencyName
      }
    },

    controlledVocabularyID = function(val){
      if (!missing(val)){
        private$.controlledVocabularyID <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.controlledVocabularyID
      }
    },

    controlledVocabularyName = function(val){
      if (!missing(val)){
        private$.controlledVocabularyName <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.controlledVocabularyName
      }
    },

    controlledVocabularyVersionID = function(val){
      if (!missing(val)){
        private$.controlledVocabularyVersionID <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.controlledVocabularyVersionID
      }
    },

    language = function(val){
      if (!missing(val)){
        private$.language <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.language
      }
    },

    otherValue = function(val){
      if (!missing(val)){
        private$.otherValue <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.otherValue
      }
    },

    uri = function(val){
      if (!missing(val)){
        private$.uri <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.uri
      }
    }


  ),

  private = list(
    .controlledVocabularyID = NULL,
    .controlledVocabularyName = NULL,
    .controlledVocabularyAgencyName = NULL,
    .controlledVocabularyVersionID = NULL,
    .otherValue = NULL,
    .uri = NULL,
    .content = NULL,
    .language = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        content = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        controlledVocabularyAgencyName = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        controlledVocabularyID = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        controlledVocabularyName = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        controlledVocabularyVersionID = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        language = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        otherValue = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        uri = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ExternalControlledVocabularyEntry


#' @title DDI4_ExternalMaterial  ExternalMaterial describes the location, structure, and relationship to the DDI metadata instance for any material held external to that instance
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ExternalMaterial}{ExternalMaterial} 

#'
#'   Definition
#'
#'   ============
#'
#'   ExternalMaterial describes the location, structure, and relationship to the DDI metadata instance for any material held external to that instance. This includes citations to such material, an external reference to a URL (or other URI), and a statement about the relationship between the cited ExternalMaterial the contents of the DDI instance. It should be used as follows:as a target object from a relationship which clarifies its role within a class; or as the target of a relatedResource within an annotation.
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   ExternalMaterial is used to identify material and optionally specific sections of the material that have a relationship to a class. There is a generic relatedMaterial on AnnotatedIdentifiable. This should be used to attach any additional material that the user identifies as important to the class. The properties typeOfMaterial, descriptiveText, and relationshipDescription should be used to clarify the purpose and coverage of the related material.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Within the DDI model, ExternalMaterial is used as an extension base for specific external materials found such as an External Aid. It is used as a base for specifically related material (e.g. ExternalAid) by creating a relationship whose name clarifies the purpose of the related material.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:OtherMaterialType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ExternalMaterial$DdiUrn} returns the DDI URN of myDDI4_ExternalMaterial.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{typeOfMaterial}}{ A read/write active binding.
#'        Designation of the type of material being described. Supports the use of a controlled vocabulary.
#'
#'                              \code{myDDI4_ExternalMaterial$typeOfMaterial <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfMaterial' property of 'myDDI4_ExternalMaterial' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ExternalMaterial$typeOfMaterial} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{descriptiveText}}{ A read/write active binding.
#'        A description of the referenced material. This field can map to a Dublin Core abstract. Note that Dublin Core does not support structure within the abstract element. Supports multiple languages and optional structured content.
#'
#'                              \code{myDDI4_ExternalMaterial$descriptiveText <- list(myInternationalStructuredString)} sets the 'descriptiveText' property of 'myDDI4_ExternalMaterial' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_ExternalMaterial$descriptiveText} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{uri}}{ A read/write active binding.
#'        URI for the location of the material
#'
#'                              \code{myDDI4_ExternalMaterial$uri <- mycharacter} sets the uri property of 'myDDI4_ExternalMaterial' to the vector of datatype:character
#'
#'                              \code{myDDI4_ExternalMaterial$uri} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{relationshipDescription}}{ A read/write active binding.
#'        Describes the reason the external material is being related to the DDI metadata instance.
#'
#'                              \code{myDDI4_ExternalMaterial$relationshipDescription <- list(myInternationalStructuredString)} sets the 'relationshipDescription' property of 'myDDI4_ExternalMaterial' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_ExternalMaterial$relationshipDescription} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{mimeType}}{ A read/write active binding.
#'        Provides a standard Internet MIME type for use by processing applications.
#'
#'                              \code{myDDI4_ExternalMaterial$mimeType <- list(myExternalControlledVocabularyEntry)} sets the 'mimeType' property of 'myDDI4_ExternalMaterial' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ExternalMaterial$mimeType} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{usesSegment}}{ A read/write active binding.
#'        Can describe a segment within a larger object such as a text or video segment.
#'
#'                              \code{myDDI4_ExternalMaterial$usesSegment <- list(mySegment)} sets the 'usesSegment' property of 'myDDI4_ExternalMaterial' to a list of R6 Segment class instances
#'
#'                              \code{myDDI4_ExternalMaterial$usesSegment} returns a list of of R6 : Segment class instances}
#'   }
#'   \describe{
#'     \item{\code{citationOfExternalMaterial}}{ A read/write active binding.
#'        Bibliographic citation for the external resource.
#'
#'                              \code{myDDI4_ExternalMaterial$citationOfExternalMaterial <- list(myAnnotation)} sets the 'citationOfExternalMaterial' property of 'myDDI4_ExternalMaterial' to a list of R6 Annotation class instances
#'
#'                              \code{myDDI4_ExternalMaterial$citationOfExternalMaterial} returns a list of of R6 : Annotation class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ExternalMaterial$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, typeOfMaterial=NA, descriptiveText=NA, uri=NA, relationshipDescription=NA, mimeType=NA, usesSegment=NA, citationOfExternalMaterial=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ExternalMaterial <- R6Class(classname="DDI4_ExternalMaterial",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, typeOfMaterial=NA, descriptiveText=NA, uri=NA, relationshipDescription=NA, mimeType=NA, usesSegment=NA, citationOfExternalMaterial=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ExternalMaterial may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(typeOfMaterial)){
        private$.typeOfMaterial <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfMaterial , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(descriptiveText)){
        private$.descriptiveText <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=descriptiveText , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(uri)){
        private$.uri <- returnVectorIfValid(validVectorClass="character", proposedVector=uri , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(relationshipDescription)){
        private$.relationshipDescription <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=relationshipDescription , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(mimeType)){
        private$.mimeType <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=mimeType , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(usesSegment)){
        private$.usesSegment <- returnR6ObjectIfValid(validClass="DDI4_Segment", proposedObjectList=usesSegment , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(citationOfExternalMaterial)){
        private$.citationOfExternalMaterial <- returnR6ObjectIfValid(validClass="DDI4_Annotation", proposedObjectList=citationOfExternalMaterial , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 7 properties: 
    citationOfExternalMaterial = function(val){
      if (!missing(val)){
        private$.citationOfExternalMaterial <- returnR6ObjectIfValid(validClass="DDI4_Annotation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.citationOfExternalMaterial
      }
    },

    descriptiveText = function(val){
      if (!missing(val)){
        private$.descriptiveText <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.descriptiveText
      }
    },

    mimeType = function(val){
      if (!missing(val)){
        private$.mimeType <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.mimeType
      }
    },

    relationshipDescription = function(val){
      if (!missing(val)){
        private$.relationshipDescription <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.relationshipDescription
      }
    },

    typeOfMaterial = function(val){
      if (!missing(val)){
        private$.typeOfMaterial <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.typeOfMaterial
      }
    },

    uri = function(val){
      if (!missing(val)){
        private$.uri <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.uri
      }
    },

    usesSegment = function(val){
      if (!missing(val)){
        private$.usesSegment <- returnR6ObjectIfValid(validClass="DDI4_Segment", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.usesSegment
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .typeOfMaterial = NULL,
    .descriptiveText = NULL,
    .uri = NULL,
    .relationshipDescription = NULL,
    .mimeType = NULL,
    .usesSegment = NULL,
    .citationOfExternalMaterial = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        citationOfExternalMaterial = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        descriptiveText = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        mimeType = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relationshipDescription = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        typeOfMaterial = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        uri = c(
          min="0", 
          max="Inf", 
          datatype = "character", 
          structure = "vector"
        ),
        usesSegment = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Segment", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ExternalMaterial


#' @title DDI4_FixedText  The static portion of the text expressed as a StructuredString with the ability to preserve whitespace if critical to the understanding of the content
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/FixedText}{FixedText} 

#'
#'   Definition
#'
#'   ============
#'
#'   The static portion of the text expressed as a StructuredString with the ability to preserve whitespace if critical to the understanding of the content.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   d:TextType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_LanguageSpecificStructuredStringType}}}
#'    \item{\code{\link{DDI4_LanguageSpecificStringType}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_FixedText$DdiUrn} returns the DDI URN of myDDI4_FixedText.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{whiteSpace}}{ A read/write active binding.
#'        The default setting states that leading and trailing white space will be removed and multiple adjacent white spaces will be treated as a single white space. If the existance of any of these white spaces is critical to the understanding of the content, change the value of this attribute to "preserve".
#'
#'                              \code{myDDI4_FixedText$whiteSpace <- list(myWhiteSpaceRule)} sets the 'whiteSpace' property of 'myDDI4_FixedText' to a list of R6 WhiteSpaceRule class instances
#'
#'                              \code{myDDI4_FixedText$whiteSpace} returns a list of of R6 : WhiteSpaceRule class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_FixedText$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, whiteSpace=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_FixedText <- R6Class(classname="DDI4_FixedText",
  inherit = DDI4_LanguageSpecificStructuredStringType,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, whiteSpace=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_FixedText may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(whiteSpace)){
        private$.whiteSpace <- returnEnumeratedValues(enumerationObject=DDI4_WhiteSpaceRule$new(registerThisObject=FALSE), proposedCharacterVector=whiteSpace , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 1 properties: 
    whiteSpace = function(val){
      if (!missing(val)){
        private$.whiteSpace <- returnEnumeratedValues(enumerationObject=DDI4_WhiteSpaceRule$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.whiteSpace
      }
    }


  ),

  private = list(
    .whiteSpace = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        content = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPlainText = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isTranslatable = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isTranslated = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        language = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        otherDefined = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        scope = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        structureUsed = c(
          min="0", 
          max="1", 
          datatype = "DDI4_StringStructureType", 
          structure = "Enumeration vector"
        ),
        translationDate = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        translationSourceLanguage = c(
          min="0", 
          max="Inf", 
          datatype = "character", 
          structure = "vector"
        ),
        whiteSpace = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WhiteSpaceRule", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_FixedText


#' @title DDI4_Form  A link to a form used by the metadata containing the form number, a statement regarding the contents of the form, a statement as to the mandatory nature of the form and a privacy level designation
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Form}{Form} 

#'
#'   Definition
#'
#'   ============
#'
#'   A link to a form used by the metadata containing the form number, a statement regarding the contents of the form, a statement as to the mandatory nature of the form and a privacy level designation.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   a:FormType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Form$DdiUrn} returns the DDI URN of myDDI4_Form.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{formNumber}}{ A read/write active binding.
#'        The number or other means of identifying the form.
#'
#'                              \code{myDDI4_Form$formNumber <- mycharacter} sets the formNumber property of 'myDDI4_Form' to the vector of datatype:character
#'
#'                              \code{myDDI4_Form$formNumber} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{uri}}{ A read/write active binding.
#'        The URN or URL of the form.
#'
#'                              \code{myDDI4_Form$uri <- mycharacter} sets the uri property of 'myDDI4_Form' to the vector of datatype:character
#'
#'                              \code{myDDI4_Form$uri} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{statement}}{ A read/write active binding.
#'        A statement regarding the use, coverage, and purpose of the form.
#'
#'                              \code{myDDI4_Form$statement <- list(myInternationalString)} sets the 'statement' property of 'myDDI4_Form' to a list of R6 InternationalString class instances
#'
#'                              \code{myDDI4_Form$statement} returns a list of of R6 : InternationalString class instances}
#'   }
#'   \describe{
#'     \item{\code{isRequired}}{ A read/write active binding.
#'        Set to "true" if the form is required. Set to "false" if the form is optional.
#'
#'                              \code{myDDI4_Form$isRequired <- mylogical} sets the isRequired property of 'myDDI4_Form' to the vector of datatype:logical
#'
#'                              \code{myDDI4_Form$isRequired} returns a vector of datatype: logical}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Form$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, formNumber=NA, uri=NA, statement=NA, isRequired=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Form <- R6Class(classname="DDI4_Form",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, formNumber=NA, uri=NA, statement=NA, isRequired=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Form may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(formNumber)){
        private$.formNumber <- returnVectorIfValid(validVectorClass="character", proposedVector=formNumber , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(uri)){
        private$.uri <- returnVectorIfValid(validVectorClass="character", proposedVector=uri , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(statement)){
        private$.statement <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=statement , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isRequired)){
        private$.isRequired <- returnVectorIfValid(validVectorClass="logical", proposedVector=isRequired , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    formNumber = function(val){
      if (!missing(val)){
        private$.formNumber <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.formNumber
      }
    },

    isRequired = function(val){
      if (!missing(val)){
        private$.isRequired <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isRequired
      }
    },

    statement = function(val){
      if (!missing(val)){
        private$.statement <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.statement
      }
    },

    uri = function(val){
      if (!missing(val)){
        private$.uri <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.uri
      }
    }


  ),

  private = list(
    .formNumber = NULL,
    .uri = NULL,
    .statement = NULL,
    .isRequired = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        formNumber = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isRequired = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        statement = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        uri = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Form


#' @title DDI4_FundingInformation  Provides information about the individual, agency and/or grant(s) which funded the described entity
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/FundingInformation}{FundingInformation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Provides information about the individual, agency and/or grant(s) which funded the described entity. Lists a reference to the agency or individual as described by a DDI Agent, the role of the funder, the grant number(s) and a description of the funding activity.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A "millionaire grant" (funding description) from John Beresford Tipton, Jr. (individual). Exploration of the effect of sudden wealth and basis for a television episode (funder role).
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:FundingInformationType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_FundingInformation$DdiUrn} returns the DDI URN of myDDI4_FundingInformation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{funderRole}}{ A read/write active binding.
#'        Role of the funding organization or individual. Supports the use of a controlled vocabulary.
#'
#'                              \code{myDDI4_FundingInformation$funderRole <- list(myExternalControlledVocabularyEntry)} sets the 'funderRole' property of 'myDDI4_FundingInformation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_FundingInformation$funderRole} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{grantNumber}}{ A read/write active binding.
#'        The identification code of the grant or other monetary award which provided funding for the described object.
#'
#'                              \code{myDDI4_FundingInformation$grantNumber <- mycharacter} sets the grantNumber property of 'myDDI4_FundingInformation' to the vector of datatype:character
#'
#'                              \code{myDDI4_FundingInformation$grantNumber} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured tex
#'
#'                              \code{myDDI4_FundingInformation$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_FundingInformation' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_FundingInformation$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasFunder}}{ A read/write active binding.
#'          The source of the funding expressed as an Organization or Individual.
#'
#'                              \code{myDDI4_FundingInformation$hasFunder <- c(myAgentURN)} sets the association 'hasFunder' from 'myDDI4_FundingInformation' to a character vector of DdiUrns for R6 Agent class instances
#'
#'                              \code{myDDI4_FundingInformation$hasFunder} returns a character vector of of DdiUrns for R6  Agent class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_FundingInformation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, funderRole=NA, grantNumber=NA, purpose=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_FundingInformation <- R6Class(classname="DDI4_FundingInformation",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, funderRole=NA, grantNumber=NA, purpose=NA,    hasFunder=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_FundingInformation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(funderRole)){
        private$.funderRole <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=funderRole , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(grantNumber)){
        private$.grantNumber <- returnVectorIfValid(validVectorClass="character", proposedVector=grantNumber , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(hasFunder)){
        private$.hasFunder <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=hasFunder , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasFunder, referencingAssociationName="hasFunder",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    funderRole = function(val){
      if (!missing(val)){
        private$.funderRole <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.funderRole
      }
    },

    grantNumber = function(val){
      if (!missing(val)){
        private$.grantNumber <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.grantNumber
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },


# There are 1 class association(s):
    hasFunder = function(val){
      if (!missing(val)){
        private$.hasFunder <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasFunder, referencingAssociationName="hasFunder",  referencingUrn=thisUrn)  
       # invisible( private$.hasFunder)
      } else {
          private$.hasFunder
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .funderRole = NULL,
    .grantNumber = NULL,
    .purpose = NULL,
    .hasFunder = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        funderRole = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        grantNumber = c(
          min="0", 
          max="Inf", 
          datatype = "character", 
          structure = "vector"
        ),
        hasFunder = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Agent", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_FundingInformation


#' @title DDI4_GeographicExtent  Defines the extent of a geographic unit for a specified period of time using Bounding Box, Inclusive and Exclusive Polygons, and an area coverage that notes a type of coverage (land, water, etc
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/GeographicExtent}{GeographicExtent} 

#'
#'   Definition
#'
#'   ============
#'
#'   Defines the extent of a geographic unit for a specified period of time using Bounding Box, Inclusive and Exclusive Polygons, and an area coverage that notes a type of coverage (land, water, etc.) and measurement for the coverage. The same geographic extent may be used by multiple versions of a single geographic version or by different geographic units occupying the same spatial area.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Bounding box for Burkino Faso: (N) 15.082773; (S) 9.395691; (E) 2.397927; (W) -5.520837.  Minnesota Land area: 206207.099 sq K, Water area: 18974.589 sq K 
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Clarifies the source of a change in terms of footprint of an area as opposed to a name or coding change. 
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:GeographicBoundaryType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_GeographicExtent$DdiUrn} returns the DDI URN of myDDI4_GeographicExtent.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{boundingPolygon}}{ A read/write active binding.
#'        A description of the boundaries of the polygon either in-line or by a reference to an external file containing the boundaries. Repeatable to describe non-contiguous areas such as islands or Native American Reservations in some parts of the United States.
#'
#'                              \code{myDDI4_GeographicExtent$boundingPolygon <- list(myPolygon)} sets the 'boundingPolygon' property of 'myDDI4_GeographicExtent' to a list of R6 Polygon class instances
#'
#'                              \code{myDDI4_GeographicExtent$boundingPolygon} returns a list of of R6 : Polygon class instances}
#'   }
#'   \describe{
#'     \item{\code{excludingPolygon}}{ A read/write active binding.
#'        A description of a the boundaries of a polygon internal to the bounding polygon which should be excluded. For example, for the bounding polygon describing the State of Brandenburg in Germany, the Excluding Polygon would describe the boundary of Berlin, creating hole within Brandenburg which is occupied by Berlin.
#'
#'                              \code{myDDI4_GeographicExtent$excludingPolygon <- list(myPolygon)} sets the 'excludingPolygon' property of 'myDDI4_GeographicExtent' to a list of R6 Polygon class instances
#'
#'                              \code{myDDI4_GeographicExtent$excludingPolygon} returns a list of of R6 : Polygon class instances}
#'   }
#'   \describe{
#'     \item{\code{geographicTime}}{ A read/write active binding.
#'        A time for which the polygon is an accurate description of the area. This may be a range (without an end date if currently still valid) or a single date when the shape was know to be valid if a range is not available.
#'
#'                              \code{myDDI4_GeographicExtent$geographicTime <- list(myDateRange)} sets the 'geographicTime' property of 'myDDI4_GeographicExtent' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_GeographicExtent$geographicTime} returns a list of of R6 : DateRange class instances}
#'   }
#'   \describe{
#'     \item{\code{hasAreaCoverage}}{ A read/write active binding.
#'        Means of describing the area covered by the geographic extent either in its entirety (total area) or specific subsets (land, water, urban, rural, etc.) providing a definition and measurement.
#'
#'                              \code{myDDI4_GeographicExtent$hasAreaCoverage <- list(myAreaCoverage)} sets the 'hasAreaCoverage' property of 'myDDI4_GeographicExtent' to a list of R6 AreaCoverage class instances
#'
#'                              \code{myDDI4_GeographicExtent$hasAreaCoverage} returns a list of of R6 : AreaCoverage class instances}
#'   }
#'   \describe{
#'     \item{\code{hasCentroid}}{ A read/write active binding.
#'        Identifies the centroid of a polygon as a specific point
#'
#'                              \code{myDDI4_GeographicExtent$hasCentroid <- list(mySpatialPoint)} sets the 'hasCentroid' property of 'myDDI4_GeographicExtent' to a list of R6 SpatialPoint class instances
#'
#'                              \code{myDDI4_GeographicExtent$hasCentroid} returns a list of of R6 : SpatialPoint class instances}
#'   }
#'   \describe{
#'     \item{\code{locationPoint}}{ A read/write active binding.
#'        The geographic extent is a single point.
#'
#'                              \code{myDDI4_GeographicExtent$locationPoint <- list(mySpatialPoint)} sets the 'locationPoint' property of 'myDDI4_GeographicExtent' to a list of R6 SpatialPoint class instances
#'
#'                              \code{myDDI4_GeographicExtent$locationPoint} returns a list of of R6 : SpatialPoint class instances}
#'   }
#'   \describe{
#'     \item{\code{isSpatialLine}}{ A read/write active binding.
#'        The geographic extent is a single line
#'
#'                              \code{myDDI4_GeographicExtent$isSpatialLine <- list(mySpatialLine)} sets the 'isSpatialLine' property of 'myDDI4_GeographicExtent' to a list of R6 SpatialLine class instances
#'
#'                              \code{myDDI4_GeographicExtent$isSpatialLine} returns a list of of R6 : SpatialLine class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasBoundingBox}}{ A read/write active binding.
#'          A Bounding Box (North, South Latitude and East, West Longitude) for the geographic extent.
#'
#'                              \code{myDDI4_GeographicExtent$hasBoundingBox <- c(myBoundingBoxURN)} sets the association 'hasBoundingBox' from 'myDDI4_GeographicExtent' to a character vector of DdiUrns for R6 BoundingBox class instances
#'
#'                              \code{myDDI4_GeographicExtent$hasBoundingBox} returns a character vector of of DdiUrns for R6  BoundingBox class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_GeographicExtent$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, boundingPolygon=NA, excludingPolygon=NA, geographicTime=NA, hasAreaCoverage=NA, hasCentroid=NA, locationPoint=NA, isSpatialLine=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_GeographicExtent <- R6Class(classname="DDI4_GeographicExtent",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, boundingPolygon=NA, excludingPolygon=NA, geographicTime=NA, hasAreaCoverage=NA, hasCentroid=NA, locationPoint=NA, isSpatialLine=NA,    hasBoundingBox=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_GeographicExtent may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(boundingPolygon)){
        private$.boundingPolygon <- returnR6ObjectIfValid(validClass="DDI4_Polygon", proposedObjectList=boundingPolygon , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(excludingPolygon)){
        private$.excludingPolygon <- returnR6ObjectIfValid(validClass="DDI4_Polygon", proposedObjectList=excludingPolygon , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(geographicTime)){
        private$.geographicTime <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=geographicTime , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasAreaCoverage)){
        private$.hasAreaCoverage <- returnR6ObjectIfValid(validClass="DDI4_AreaCoverage", proposedObjectList=hasAreaCoverage , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(hasCentroid)){
        private$.hasCentroid <- returnR6ObjectIfValid(validClass="DDI4_SpatialPoint", proposedObjectList=hasCentroid , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(locationPoint)){
        private$.locationPoint <- returnR6ObjectIfValid(validClass="DDI4_SpatialPoint", proposedObjectList=locationPoint , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isSpatialLine)){
        private$.isSpatialLine <- returnR6ObjectIfValid(validClass="DDI4_SpatialLine", proposedObjectList=isSpatialLine , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(hasBoundingBox)){
        private$.hasBoundingBox <- returnUrnVectorIfValid(validClass="DDI4_BoundingBox", proposedUrnVector=hasBoundingBox , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasBoundingBox, referencingAssociationName="hasBoundingBox",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 7 properties: 
    boundingPolygon = function(val){
      if (!missing(val)){
        private$.boundingPolygon <- returnR6ObjectIfValid(validClass="DDI4_Polygon", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.boundingPolygon
      }
    },

    excludingPolygon = function(val){
      if (!missing(val)){
        private$.excludingPolygon <- returnR6ObjectIfValid(validClass="DDI4_Polygon", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.excludingPolygon
      }
    },

    geographicTime = function(val){
      if (!missing(val)){
        private$.geographicTime <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.geographicTime
      }
    },

    hasAreaCoverage = function(val){
      if (!missing(val)){
        private$.hasAreaCoverage <- returnR6ObjectIfValid(validClass="DDI4_AreaCoverage", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasAreaCoverage
      }
    },

    hasCentroid = function(val){
      if (!missing(val)){
        private$.hasCentroid <- returnR6ObjectIfValid(validClass="DDI4_SpatialPoint", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasCentroid
      }
    },

    isSpatialLine = function(val){
      if (!missing(val)){
        private$.isSpatialLine <- returnR6ObjectIfValid(validClass="DDI4_SpatialLine", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isSpatialLine
      }
    },

    locationPoint = function(val){
      if (!missing(val)){
        private$.locationPoint <- returnR6ObjectIfValid(validClass="DDI4_SpatialPoint", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.locationPoint
      }
    },


# There are 1 class association(s):
    hasBoundingBox = function(val){
      if (!missing(val)){
        private$.hasBoundingBox <- returnUrnVectorIfValid(validClass="DDI4_BoundingBox", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasBoundingBox, referencingAssociationName="hasBoundingBox",  referencingUrn=thisUrn)  
       # invisible( private$.hasBoundingBox)
      } else {
          private$.hasBoundingBox
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .boundingPolygon = NULL,
    .excludingPolygon = NULL,
    .geographicTime = NULL,
    .hasAreaCoverage = NULL,
    .hasCentroid = NULL,
    .locationPoint = NULL,
    .isSpatialLine = NULL,
    .hasBoundingBox = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        boundingPolygon = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Polygon", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        excludingPolygon = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Polygon", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        geographicTime = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAreaCoverage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AreaCoverage", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasBoundingBox = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BoundingBox", 
          structure = "DdiUrn vector"
        ),
        hasCentroid = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SpatialPoint", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isSpatialLine = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SpatialLine", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        locationPoint = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SpatialPoint", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_GeographicExtent


#' @title DDI4_GeographicUnit  A specific geographic unit of a defined Unit Type
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/GeographicUnit}{GeographicUnit} 

#'
#'   Definition
#'
#'   ============
#'
#'   A specific geographic unit of a defined Unit Type. A geographic unit may change its name, composition, or geographic extent over time. This may be tracked by versioning the content of the geographic unit. Normally a new version of a geographic unit would have a geographic (spatial) overlap with its previous version (a city annexing new area). A geographic unit ends when it is no longer a unit of the same unit type.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   The State of Minnesota from 1858 to date where the Unit Type is a State as defined by the United States Census Bureau. It has a geographic extent and supersedes a portion of the Territory of Minnesota and a portion of the Territory of Wisconsin. Minnesota territory had both a broader geographic extent, different Unit Type, and earlier time period (1849-1858) as did the Territory of Wisconsin. 
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:GeographicLocationType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Unit}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_GeographicUnit$DdiUrn} returns the DDI URN of myDDI4_GeographicUnit.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{geographicTime}}{ A read/write active binding.
#'        The period for which the Geographic Unit is valid. Note that the geographic extent, name, or composition change within the valid period of this unit.
#'
#'                              \code{myDDI4_GeographicUnit$geographicTime <- list(myDateRange)} sets the 'geographicTime' property of 'myDDI4_GeographicUnit' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_GeographicUnit$geographicTime} returns a list of of R6 : DateRange class instances}
#'   }
#'   \describe{
#'     \item{\code{supercedes}}{ A read/write active binding.
#'        The Geographic Unit supercedes (follows immediately in time) the described geographic unit. Repeat for multiple geographic units.
#'
#'                              \code{myDDI4_GeographicUnit$supercedes <- list(mySpatialRelationship)} sets the 'supercedes' property of 'myDDI4_GeographicUnit' to a list of R6 SpatialRelationship class instances
#'
#'                              \code{myDDI4_GeographicUnit$supercedes} returns a list of of R6 : SpatialRelationship class instances}
#'   }
#'   \describe{
#'     \item{\code{precedes}}{ A read/write active binding.
#'        The Geographic Unit is superceded by (precedes immediately in time) the described geographic unit. Repeat for multiple geographic units.
#'
#'                              \code{myDDI4_GeographicUnit$precedes <- list(mySpatialRelationship)} sets the 'precedes' property of 'myDDI4_GeographicUnit' to a list of R6 SpatialRelationship class instances
#'
#'                              \code{myDDI4_GeographicUnit$precedes} returns a list of of R6 : SpatialRelationship class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasGeographicExtent}}{ A read/write active binding.
#'          Describes the geographic extent or coverage of the geographic unit during the specified period of time. 
#'
#'                              \code{myDDI4_GeographicUnit$hasGeographicExtent <- c(myGeographicExtentURN)} sets the association 'hasGeographicExtent' from 'myDDI4_GeographicUnit' to a character vector of DdiUrns for R6 GeographicExtent class instances
#'
#'                              \code{myDDI4_GeographicUnit$hasGeographicExtent} returns a character vector of of DdiUrns for R6  GeographicExtent class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_GeographicUnit$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, geographicTime=NA, supercedes=NA, precedes=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_GeographicUnit <- R6Class(classname="DDI4_GeographicUnit",
  inherit = DDI4_Unit,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, geographicTime=NA, supercedes=NA, precedes=NA,    hasGeographicExtent=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_GeographicUnit may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(geographicTime)){
        private$.geographicTime <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=geographicTime , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(supercedes)){
        private$.supercedes <- returnR6ObjectIfValid(validClass="DDI4_SpatialRelationship", proposedObjectList=supercedes , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(precedes)){
        private$.precedes <- returnR6ObjectIfValid(validClass="DDI4_SpatialRelationship", proposedObjectList=precedes , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(hasGeographicExtent)){
        private$.hasGeographicExtent <- returnUrnVectorIfValid(validClass="DDI4_GeographicExtent", proposedUrnVector=hasGeographicExtent , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasGeographicExtent, referencingAssociationName="hasGeographicExtent",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    geographicTime = function(val){
      if (!missing(val)){
        private$.geographicTime <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.geographicTime
      }
    },

    precedes = function(val){
      if (!missing(val)){
        private$.precedes <- returnR6ObjectIfValid(validClass="DDI4_SpatialRelationship", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.precedes
      }
    },

    supercedes = function(val){
      if (!missing(val)){
        private$.supercedes <- returnR6ObjectIfValid(validClass="DDI4_SpatialRelationship", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.supercedes
      }
    },


# There are 1 class association(s):
    hasGeographicExtent = function(val){
      if (!missing(val)){
        private$.hasGeographicExtent <- returnUrnVectorIfValid(validClass="DDI4_GeographicExtent", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasGeographicExtent, referencingAssociationName="hasGeographicExtent",  referencingUrn=thisUrn)  
       # invisible( private$.hasGeographicExtent)
      } else {
          private$.hasGeographicExtent
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .geographicTime = NULL,
    .supercedes = NULL,
    .precedes = NULL,
    .hasGeographicExtent = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definition = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        geographicTime = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasGeographicExtent = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_GeographicExtent", 
          structure = "DdiUrn vector"
        ),
        hasUnitType = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_UnitType", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        precedes = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_SpatialRelationship", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        supercedes = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_SpatialRelationship", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_GeographicUnit


#' @title DDI4_GeographicUnitClassification  Describes the classification of specific geographic units into a classification system
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/GeographicUnitClassification}{GeographicUnitClassification} 

#'
#'   Definition
#'
#'   ============
#'
#'   Describes the classification of specific geographic units into a classification system. As a subtype of Code List it can be used directly for the description of a value domain.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_EnumerationDomain}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_GeographicUnitClassification$DdiUrn} returns the DDI URN of myDDI4_GeographicUnitClassification.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        Define the code and Unit Type that is a member of the Geographic Unit Classification. They may be unordered or ordered and assigned to a specific level.
#'
#'                              \code{myDDI4_GeographicUnitClassification$contains <- list(myGeographicUnitIndicator)} sets the 'contains' property of 'myDDI4_GeographicUnitClassification' to a list of R6 GeographicUnitIndicator class instances
#'
#'                              \code{myDDI4_GeographicUnitClassification$contains} returns a list of of R6 : GeographicUnitIndicator class instances}
#'   }
#'   \describe{
#'     \item{\code{releaseDate}}{ A read/write active binding.
#'        Date the Classification was released
#'
#'                              \code{myDDI4_GeographicUnitClassification$releaseDate <- list(myDate)} sets the 'releaseDate' property of 'myDDI4_GeographicUnitClassification' to a list of R6 Date class instances
#'
#'                              \code{myDDI4_GeographicUnitClassification$releaseDate} returns a list of of R6 : Date class instances}
#'   }
#'   \describe{
#'     \item{\code{validDates}}{ A read/write active binding.
#'        The date the classification enters production use and the date on which the Classification was superseded by a successor version or otherwise ceased to be valid. (Source: GSIM Statistical Classification)
#'
#'                              \code{myDDI4_GeographicUnitClassification$validDates <- list(myDateRange)} sets the 'validDates' property of 'myDDI4_GeographicUnitClassification' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_GeographicUnitClassification$validDates} returns a list of of R6 : DateRange class instances}
#'   }
#'   \describe{
#'     \item{\code{isCurrent}}{ A read/write active binding.
#'        Indicates if the Classification is currently valid.
#'
#'                              \code{myDDI4_GeographicUnitClassification$isCurrent <- mylogical} sets the isCurrent property of 'myDDI4_GeographicUnitClassification' to the vector of datatype:logical
#'
#'                              \code{myDDI4_GeographicUnitClassification$isCurrent} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{isFloating}}{ A read/write active binding.
#'        Indicates if the Classification is a floating classification. In a floating classification, a validity period should be defined for all Classification Items which will allow the display of the item structure and content at different points of time. (Source: GSIM StatisticalClassification/Floating)
#'
#'                              \code{myDDI4_GeographicUnitClassification$isFloating <- mylogical} sets the isFloating property of 'myDDI4_GeographicUnitClassification' to the vector of datatype:logical
#'
#'                              \code{myDDI4_GeographicUnitClassification$isFloating} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A structured display label providing a fully human readable display for the identification of the object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_GeographicUnitClassification$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_GeographicUnitClassification' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_GeographicUnitClassification$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#'   \describe{
#'     \item{\code{usage}}{ A read/write active binding.
#'        Explanation of the ways in which some decision or object is employed. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_GeographicUnitClassification$usage <- list(myInternationalStructuredString)} sets the 'usage' property of 'myDDI4_GeographicUnitClassification' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_GeographicUnitClassification$usage} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{references}}{ A read/write active binding.
#'          May reference an existing concept system generally of Geographic Unit
#'
#'                              \code{myDDI4_GeographicUnitClassification$references <- c(myConceptSystemURN)} sets the association 'references' from 'myDDI4_GeographicUnitClassification' to a character vector of DdiUrns for R6 ConceptSystem class instances
#'
#'                              \code{myDDI4_GeographicUnitClassification$references} returns a character vector of of DdiUrns for R6  ConceptSystem class instances}
#'   }
#'   \describe{
#'     \item{\code{isStructuredBy}}{ A read/write active binding.
#'          A relation structure used to describe complex relationships between the members
#'
#'                              \code{myDDI4_GeographicUnitClassification$isStructuredBy <- c(myGeographicUnitRelationStructureURN)} sets the association 'isStructuredBy' from 'myDDI4_GeographicUnitClassification' to a character vector of DdiUrns for R6 GeographicUnitRelationStructure class instances
#'
#'                              \code{myDDI4_GeographicUnitClassification$isStructuredBy} returns a character vector of of DdiUrns for R6  GeographicUnitRelationStructure class instances}
#'   }
#'   \describe{
#'     \item{\code{isMaintainedBy}}{ A read/write active binding.
#'          Organization, agency, or group within an agency responsible for the maintenance and upkeep of the classification.
#'
#'                              \code{myDDI4_GeographicUnitClassification$isMaintainedBy <- c(myOrganizationURN)} sets the association 'isMaintainedBy' from 'myDDI4_GeographicUnitClassification' to a character vector of DdiUrns for R6 Organization class instances
#'
#'                              \code{myDDI4_GeographicUnitClassification$isMaintainedBy} returns a character vector of of DdiUrns for R6  Organization class instances}
#'   }
#'   \describe{
#'     \item{\code{variantOf}}{ A read/write active binding.
#'          Geographic Unit Classification on which the current variant is based, and any subsequent versions of that Classification to which it is also applicable.
#'
#'                              \code{myDDI4_GeographicUnitClassification$variantOf <- c(myGeographicUnitClassificationURN)} sets the association 'variantOf' from 'myDDI4_GeographicUnitClassification' to a character vector of DdiUrns for R6 GeographicUnitClassification class instances
#'
#'                              \code{myDDI4_GeographicUnitClassification$variantOf} returns a character vector of of DdiUrns for R6  GeographicUnitClassification class instances}
#'   }
#'   \describe{
#'     \item{\code{replaces}}{ A read/write active binding.
#'          Geographic Unit Classification superseded by the actual Classification (for those Classifications that are versions or updates).
#'
#'                              \code{myDDI4_GeographicUnitClassification$replaces <- c(myGeographicUnitClassificationURN)} sets the association 'replaces' from 'myDDI4_GeographicUnitClassification' to a character vector of DdiUrns for R6 GeographicUnitClassification class instances
#'
#'                              \code{myDDI4_GeographicUnitClassification$replaces} returns a character vector of of DdiUrns for R6  GeographicUnitClassification class instances}
#'   }
#'   \describe{
#'     \item{\code{isReplacedBy}}{ A read/write active binding.
#'          Geographic Unit Type Structure that supersedes the actual Statistical Classification (for those Statistical Classifications that are versions or updates).
#'
#'                              \code{myDDI4_GeographicUnitClassification$isReplacedBy <- c(myGeographicUnitClassificationURN)} sets the association 'isReplacedBy' from 'myDDI4_GeographicUnitClassification' to a character vector of DdiUrns for R6 GeographicUnitClassification class instances
#'
#'                              \code{myDDI4_GeographicUnitClassification$isReplacedBy} returns a character vector of of DdiUrns for R6  GeographicUnitClassification class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_GeographicUnitClassification$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, contains=NA, releaseDate=NA, validDates=NA, isCurrent=NA, isFloating=NA, displayLabel=NA, usage=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_GeographicUnitClassification <- R6Class(classname="DDI4_GeographicUnitClassification",
  inherit = DDI4_EnumerationDomain,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, contains=NA, releaseDate=NA, validDates=NA, isCurrent=NA, isFloating=NA, displayLabel=NA, usage=NA,    references=NA,isStructuredBy=NA,isMaintainedBy=NA,variantOf=NA,replaces=NA,isReplacedBy=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_GeographicUnitClassification may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_GeographicUnitIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(releaseDate)){
        private$.releaseDate <- returnR6ObjectIfValid(validClass="DDI4_Date", proposedObjectList=releaseDate , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(validDates)){
        private$.validDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=validDates , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isCurrent)){
        private$.isCurrent <- returnVectorIfValid(validVectorClass="logical", proposedVector=isCurrent , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isFloating)){
        private$.isFloating <- returnVectorIfValid(validVectorClass="logical", proposedVector=isFloating , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(usage)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=usage , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(references)){
        private$.references <- returnUrnVectorIfValid(validClass="DDI4_ConceptSystem", proposedUrnVector=references , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.references, referencingAssociationName="references",  referencingUrn=thisUrn)  
      }
      if(!missing(isStructuredBy)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnitRelationStructure", proposedUrnVector=isStructuredBy , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
      }
      if(!missing(isMaintainedBy)){
        private$.isMaintainedBy <- returnUrnVectorIfValid(validClass="DDI4_Organization", proposedUrnVector=isMaintainedBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isMaintainedBy, referencingAssociationName="isMaintainedBy",  referencingUrn=thisUrn)  
      }
      if(!missing(variantOf)){
        private$.variantOf <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnitClassification", proposedUrnVector=variantOf , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.variantOf, referencingAssociationName="variantOf",  referencingUrn=thisUrn)  
      }
      if(!missing(replaces)){
        private$.replaces <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnitClassification", proposedUrnVector=replaces , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.replaces, referencingAssociationName="replaces",  referencingUrn=thisUrn)  
      }
      if(!missing(isReplacedBy)){
        private$.isReplacedBy <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnitClassification", proposedUrnVector=isReplacedBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isReplacedBy, referencingAssociationName="isReplacedBy",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 7 properties: 
    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_GeographicUnitIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },

    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },

    isCurrent = function(val){
      if (!missing(val)){
        private$.isCurrent <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isCurrent
      }
    },

    isFloating = function(val){
      if (!missing(val)){
        private$.isFloating <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isFloating
      }
    },

    releaseDate = function(val){
      if (!missing(val)){
        private$.releaseDate <- returnR6ObjectIfValid(validClass="DDI4_Date", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.releaseDate
      }
    },

    usage = function(val){
      if (!missing(val)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.usage
      }
    },

    validDates = function(val){
      if (!missing(val)){
        private$.validDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.validDates
      }
    },


# There are 6 class association(s):
    isMaintainedBy = function(val){
      if (!missing(val)){
        private$.isMaintainedBy <- returnUrnVectorIfValid(validClass="DDI4_Organization", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isMaintainedBy, referencingAssociationName="isMaintainedBy",  referencingUrn=thisUrn)  
       # invisible( private$.isMaintainedBy)
      } else {
          private$.isMaintainedBy
      }
    },

    isReplacedBy = function(val){
      if (!missing(val)){
        private$.isReplacedBy <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnitClassification", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isReplacedBy, referencingAssociationName="isReplacedBy",  referencingUrn=thisUrn)  
       # invisible( private$.isReplacedBy)
      } else {
          private$.isReplacedBy
      }
    },

    isStructuredBy = function(val){
      if (!missing(val)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnitRelationStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
       # invisible( private$.isStructuredBy)
      } else {
          private$.isStructuredBy
      }
    },

    references = function(val){
      if (!missing(val)){
        private$.references <- returnUrnVectorIfValid(validClass="DDI4_ConceptSystem", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.references, referencingAssociationName="references",  referencingUrn=thisUrn)  
       # invisible( private$.references)
      } else {
          private$.references
      }
    },

    replaces = function(val){
      if (!missing(val)){
        private$.replaces <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnitClassification", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.replaces, referencingAssociationName="replaces",  referencingUrn=thisUrn)  
       # invisible( private$.replaces)
      } else {
          private$.replaces
      }
    },

    variantOf = function(val){
      if (!missing(val)){
        private$.variantOf <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnitClassification", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.variantOf, referencingAssociationName="variantOf",  referencingUrn=thisUrn)  
       # invisible( private$.variantOf)
      } else {
          private$.variantOf
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .contains = NULL,
    .releaseDate = NULL,
    .validDates = NULL,
    .isCurrent = NULL,
    .isFloating = NULL,
    .displayLabel = NULL,
    .usage = NULL,
    .references = NULL,
    .isStructuredBy = NULL,
    .isMaintainedBy = NULL,
    .variantOf = NULL,
    .replaces = NULL,
    .isReplacedBy = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_GeographicUnitIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isCurrent = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isFloating = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isMaintainedBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Organization", 
          structure = "DdiUrn vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isReplacedBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_GeographicUnitClassification", 
          structure = "DdiUrn vector"
        ),
        isStructuredBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_GeographicUnitRelationStructure", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        references = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ConceptSystem", 
          structure = "DdiUrn vector"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        releaseDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Date", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        replaces = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_GeographicUnitClassification", 
          structure = "DdiUrn vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        usage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usesLevelStructure = c(
          min="0", 
          max="1", 
          datatype = "DDI4_LevelStructure", 
          structure = "DdiUrn vector"
        ),
        validDates = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        variantOf = c(
          min="0", 
          max="1", 
          datatype = "DDI4_GeographicUnitClassification", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_GeographicUnitClassification


#' @title DDI4_GeographicUnitIndicator  Provides ability to declare an optional sequence or index order to a Geographic Unit
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/GeographicUnitIndicator}{GeographicUnitIndicator} 

#'
#'   Definition
#'
#'   ============
#'
#'   Provides ability to declare an optional sequence or index order to a Geographic Unit
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_GeographicUnitIndicator$DdiUrn} returns the DDI URN of myDDI4_GeographicUnitIndicator.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Index number expressed as an integer. The position of the member in an ordered array. Optional for unordered Collections.
#'
#'                              \code{myDDI4_GeographicUnitIndicator$index <- myinteger} sets the index property of 'myDDI4_GeographicUnitIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_GeographicUnitIndicator$index} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{isInLevel}}{ A read/write active binding.
#'        Indicates the level within which the CodeItem resides
#'
#'                              \code{myDDI4_GeographicUnitIndicator$isInLevel <- myinteger} sets the isInLevel property of 'myDDI4_GeographicUnitIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_GeographicUnitIndicator$isInLevel} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{member}}{ A read/write active binding.
#'          Member of the collection.
#'
#'                              \code{myDDI4_GeographicUnitIndicator$member <- c(myGeographicUnitURN)} sets the association 'member' from 'myDDI4_GeographicUnitIndicator' to a character vector of DdiUrns for R6 GeographicUnit class instances
#'
#'                              \code{myDDI4_GeographicUnitIndicator$member} returns a character vector of of DdiUrns for R6  GeographicUnit class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_GeographicUnitIndicator$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, isInLevel=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_GeographicUnitIndicator <- R6Class(classname="DDI4_GeographicUnitIndicator",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA, isInLevel=NA,    member=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_GeographicUnitIndicator may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isInLevel)){
        private$.isInLevel <- returnVectorIfValid(validVectorClass="integer", proposedVector=isInLevel , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(member)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnit", proposedUrnVector=member , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 2 properties: 
    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.index
      }
    },

    isInLevel = function(val){
      if (!missing(val)){
        private$.isInLevel <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isInLevel
      }
    },


# There are 1 class association(s):
    member = function(val){
      if (!missing(val)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnit", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
       # invisible( private$.member)
      } else {
          private$.member
      }
    }

  ),

  private = list(
    .index = NULL,
    .isInLevel = NULL,
    .member = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        index = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        isInLevel = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        member = c(
          min="1", 
          max="1", 
          datatype = "DDI4_GeographicUnit", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_GeographicUnitIndicator


#' @title DDI4_GeographicUnitRelation  Describes structured relationship between Geographic Units
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/GeographicUnitRelation}{GeographicUnitRelation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Describes structured relationship between Geographic Units.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   A member might have a ParentChild relation to several other members which is Anti-Reflexive, Anti-Symmetric, and Anti-Transitive. For a process this might have a semantic of "immediately follows". 
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_GeographicUnitRelation$DdiUrn} returns the DDI URN of myDDI4_GeographicUnitRelation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an enumeration list
#'
#'                              \code{myDDI4_GeographicUnitRelation$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_GeographicUnitRelation' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_GeographicUnitRelation$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{hasSpatialRelationSpecification}}{ A read/write active binding.
#'        Provides information on the spatial relationship between the parent and child.
#'
#'                              \code{myDDI4_GeographicUnitRelation$hasSpatialRelationSpecification <- list(mySpatialRelationSpecification)} sets the 'hasSpatialRelationSpecification' property of 'myDDI4_GeographicUnitRelation' to a list of R6 SpatialRelationSpecification class instances
#'
#'                              \code{myDDI4_GeographicUnitRelation$hasSpatialRelationSpecification} returns a list of of R6 : SpatialRelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provide a semantic that provides a context for the relationship using and External Controlled Vocabulary
#'
#'                              \code{myDDI4_GeographicUnitRelation$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_GeographicUnitRelation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_GeographicUnitRelation$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of its totality using an enumeration list.
#'
#'                              \code{myDDI4_GeographicUnitRelation$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_GeographicUnitRelation' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_GeographicUnitRelation$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          The first member of the relationship.
#'
#'                              \code{myDDI4_GeographicUnitRelation$source <- c(myGeographicUnitURN)} sets the association 'source' from 'myDDI4_GeographicUnitRelation' to a character vector of DdiUrns for R6 GeographicUnit class instances
#'
#'                              \code{myDDI4_GeographicUnitRelation$source} returns a character vector of of DdiUrns for R6  GeographicUnit class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          Second member in the relationship. Notes this could be realized as a collection supporting tuple relationship.
#'
#'                              \code{myDDI4_GeographicUnitRelation$target <- c(myGeographicUnitURN)} sets the association 'target' from 'myDDI4_GeographicUnitRelation' to a character vector of DdiUrns for R6 GeographicUnit class instances
#'
#'                              \code{myDDI4_GeographicUnitRelation$target} returns a character vector of of DdiUrns for R6  GeographicUnit class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_GeographicUnitRelation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, hasSpatialRelationSpecification=NA, semantic=NA, totality=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_GeographicUnitRelation <- R6Class(classname="DDI4_GeographicUnitRelation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, hasSpatialRelationSpecification=NA, semantic=NA, totality=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_GeographicUnitRelation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(hasSpatialRelationSpecification)){
        private$.hasSpatialRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_SpatialRelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasSpatialRelationSpecification , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnit", proposedUrnVector=source , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnit", proposedUrnVector=target , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 4 properties: 
    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    hasSpatialRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasSpatialRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_SpatialRelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasSpatialRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnit", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnit", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .hasSpatialRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        hasSpatialRelationSpecification = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SpatialRelationSpecification", 
          structure = "Enumeration vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="0", 
          max="1", 
          datatype = "DDI4_GeographicUnit", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_GeographicUnit", 
          structure = "DdiUrn vector"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_GeographicUnitRelation


#' @title DDI4_GeographicUnitRelationStructure  Defines the relationships between Geographic Unit Types in a collection
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/GeographicUnitRelationStructure}{GeographicUnitRelationStructure} 

#'
#'   Definition
#'
#'   ============
#'
#'   Defines the relationships between Geographic Unit Types in a collection. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_GeographicUnitRelationStructure$DdiUrn} returns the DDI URN of myDDI4_GeographicUnitRelationStructure.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an ennumeration list
#'
#'                              \code{myDDI4_GeographicUnitRelationStructure$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_GeographicUnitRelationStructure' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_GeographicUnitRelationStructure$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship context for the relationship using an External Controlled Vocabulary. Examples might include "parent-child", or "immediate supervisee"
#'
#'                              \code{myDDI4_GeographicUnitRelationStructure$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_GeographicUnitRelationStructure' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_GeographicUnitRelationStructure$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Controlled Vocabulary to specify whether the relation is total, partial or unknown.
#'
#'                              \code{myDDI4_GeographicUnitRelationStructure$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_GeographicUnitRelationStructure' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_GeographicUnitRelationStructure$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMemberRelation}}{ A read/write active binding.
#'        MemberRelation used to define the relationship of members within the collection
#'
#'                              \code{myDDI4_GeographicUnitRelationStructure$hasMemberRelation <- list(myGeographicUnitRelation)} sets the 'hasMemberRelation' property of 'myDDI4_GeographicUnitRelationStructure' to a list of R6 GeographicUnitRelation class instances
#'
#'                              \code{myDDI4_GeographicUnitRelationStructure$hasMemberRelation} returns a list of of R6 : GeographicUnitRelation class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_GeographicUnitRelationStructure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_GeographicUnitRelationStructure <- R6Class(classname="DDI4_GeographicUnitRelationStructure",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_GeographicUnitRelationStructure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasMemberRelation)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_GeographicUnitRelation", proposedObjectList=hasMemberRelation , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    hasMemberRelation = function(val){
      if (!missing(val)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_GeographicUnitRelation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasMemberRelation
      }
    },

    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .hasMemberRelation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasMemberRelation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_GeographicUnitRelation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_GeographicUnitRelationStructure


#' @title DDI4_GeographicUnitTypeClassification  A structured collection of Unit Types defining a geographic structure
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/GeographicUnitTypeClassification}{GeographicUnitTypeClassification} 

#'
#'   Definition
#'
#'   ============
#'
#'   A structured collection of Unit Types defining a geographic structure. As a subtype of CodeList it may be used directly to describe a value domain.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Country--State--County
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_EnumerationDomain}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_GeographicUnitTypeClassification$DdiUrn} returns the DDI URN of myDDI4_GeographicUnitTypeClassification.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        Define the code and Unit Type that is a member of the Geographic Unit Type Classification. They may be unordered or ordered and assigned to a specific level.
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$contains <- list(myGeographicUnitTypeIndicator)} sets the 'contains' property of 'myDDI4_GeographicUnitTypeClassification' to a list of R6 GeographicUnitTypeIndicator class instances
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$contains} returns a list of of R6 : GeographicUnitTypeIndicator class instances}
#'   }
#'   \describe{
#'     \item{\code{releaseDate}}{ A read/write active binding.
#'        Date the Classification was released
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$releaseDate <- list(myDate)} sets the 'releaseDate' property of 'myDDI4_GeographicUnitTypeClassification' to a list of R6 Date class instances
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$releaseDate} returns a list of of R6 : Date class instances}
#'   }
#'   \describe{
#'     \item{\code{validDates}}{ A read/write active binding.
#'        The date the statistical classification enters production use and the date on which the Classification was superseded by a successor version or otherwise ceased to be valid. (Source: GSIM Statistical Classification)
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$validDates <- list(myDateRange)} sets the 'validDates' property of 'myDDI4_GeographicUnitTypeClassification' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$validDates} returns a list of of R6 : DateRange class instances}
#'   }
#'   \describe{
#'     \item{\code{isCurrent}}{ A read/write active binding.
#'        Indicates if the Statistical Classification is currently valid.
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$isCurrent <- mylogical} sets the isCurrent property of 'myDDI4_GeographicUnitTypeClassification' to the vector of datatype:logical
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$isCurrent} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{isFloating}}{ A read/write active binding.
#'        Indicates if the Classification is a floating classification. In a floating statistical classification, a validity period should be defined for all Classification Items which will allow the display of the item structure and content at different points of time. (Source: GSIM StatisticalClassification/Floating)
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$isFloating <- mylogical} sets the isFloating property of 'myDDI4_GeographicUnitTypeClassification' to the vector of datatype:logical
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$isFloating} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A structured display label providing a fully human readable display for the identification of the object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_GeographicUnitTypeClassification' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#'   \describe{
#'     \item{\code{usage}}{ A read/write active binding.
#'        Explanation of the ways in which some decision or object is employed. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$usage <- list(myInternationalStructuredString)} sets the 'usage' property of 'myDDI4_GeographicUnitTypeClassification' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$usage} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{references}}{ A read/write active binding.
#'          May reference an existing concept system generally of Geographic Unit Types
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$references <- c(myConceptSystemURN)} sets the association 'references' from 'myDDI4_GeographicUnitTypeClassification' to a character vector of DdiUrns for R6 ConceptSystem class instances
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$references} returns a character vector of of DdiUrns for R6  ConceptSystem class instances}
#'   }
#'   \describe{
#'     \item{\code{isStructuredBy}}{ A read/write active binding.
#'          A relation structure used to describe complex relationships between the members
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$isStructuredBy <- c(myGeographicUnitTypeRelationStructureURN)} sets the association 'isStructuredBy' from 'myDDI4_GeographicUnitTypeClassification' to a character vector of DdiUrns for R6 GeographicUnitTypeRelationStructure class instances
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$isStructuredBy} returns a character vector of of DdiUrns for R6  GeographicUnitTypeRelationStructure class instances}
#'   }
#'   \describe{
#'     \item{\code{isMaintainedBy}}{ A read/write active binding.
#'          Organization, agency, or group within an agency responsible for the maintenance and upkeep of the classification.
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$isMaintainedBy <- c(myOrganizationURN)} sets the association 'isMaintainedBy' from 'myDDI4_GeographicUnitTypeClassification' to a character vector of DdiUrns for R6 Organization class instances
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$isMaintainedBy} returns a character vector of of DdiUrns for R6  Organization class instances}
#'   }
#'   \describe{
#'     \item{\code{variantOf}}{ A read/write active binding.
#'          Geographic Unit Type Structure on which the current variant is based, and any subsequent versions of that Classification to which it is also applicable. 
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$variantOf <- c(myGeographicUnitTypeClassificationURN)} sets the association 'variantOf' from 'myDDI4_GeographicUnitTypeClassification' to a character vector of DdiUrns for R6 GeographicUnitTypeClassification class instances
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$variantOf} returns a character vector of of DdiUrns for R6  GeographicUnitTypeClassification class instances}
#'   }
#'   \describe{
#'     \item{\code{replaces}}{ A read/write active binding.
#'          Geographic Unit Type Structure superseded by the actual Classification (for those Classifications that are versions or updates).
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$replaces <- c(myGeographicUnitTypeClassificationURN)} sets the association 'replaces' from 'myDDI4_GeographicUnitTypeClassification' to a character vector of DdiUrns for R6 GeographicUnitTypeClassification class instances
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$replaces} returns a character vector of of DdiUrns for R6  GeographicUnitTypeClassification class instances}
#'   }
#'   \describe{
#'     \item{\code{isReplacedBy}}{ A read/write active binding.
#'          Geographic Unit Type Structure that supersedes the actual Classification (for those Classifications that are versions or updates),
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$isReplacedBy <- c(myGeographicUnitTypeClassificationURN)} sets the association 'isReplacedBy' from 'myDDI4_GeographicUnitTypeClassification' to a character vector of DdiUrns for R6 GeographicUnitTypeClassification class instances
#'
#'                              \code{myDDI4_GeographicUnitTypeClassification$isReplacedBy} returns a character vector of of DdiUrns for R6  GeographicUnitTypeClassification class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_GeographicUnitTypeClassification$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, contains=NA, releaseDate=NA, validDates=NA, isCurrent=NA, isFloating=NA, displayLabel=NA, usage=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_GeographicUnitTypeClassification <- R6Class(classname="DDI4_GeographicUnitTypeClassification",
  inherit = DDI4_EnumerationDomain,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, contains=NA, releaseDate=NA, validDates=NA, isCurrent=NA, isFloating=NA, displayLabel=NA, usage=NA,    references=NA,isStructuredBy=NA,isMaintainedBy=NA,variantOf=NA,replaces=NA,isReplacedBy=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_GeographicUnitTypeClassification may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_GeographicUnitTypeIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(releaseDate)){
        private$.releaseDate <- returnR6ObjectIfValid(validClass="DDI4_Date", proposedObjectList=releaseDate , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(validDates)){
        private$.validDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=validDates , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isCurrent)){
        private$.isCurrent <- returnVectorIfValid(validVectorClass="logical", proposedVector=isCurrent , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isFloating)){
        private$.isFloating <- returnVectorIfValid(validVectorClass="logical", proposedVector=isFloating , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(usage)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=usage , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(references)){
        private$.references <- returnUrnVectorIfValid(validClass="DDI4_ConceptSystem", proposedUrnVector=references , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.references, referencingAssociationName="references",  referencingUrn=thisUrn)  
      }
      if(!missing(isStructuredBy)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnitTypeRelationStructure", proposedUrnVector=isStructuredBy , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
      }
      if(!missing(isMaintainedBy)){
        private$.isMaintainedBy <- returnUrnVectorIfValid(validClass="DDI4_Organization", proposedUrnVector=isMaintainedBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isMaintainedBy, referencingAssociationName="isMaintainedBy",  referencingUrn=thisUrn)  
      }
      if(!missing(variantOf)){
        private$.variantOf <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnitTypeClassification", proposedUrnVector=variantOf , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.variantOf, referencingAssociationName="variantOf",  referencingUrn=thisUrn)  
      }
      if(!missing(replaces)){
        private$.replaces <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnitTypeClassification", proposedUrnVector=replaces , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.replaces, referencingAssociationName="replaces",  referencingUrn=thisUrn)  
      }
      if(!missing(isReplacedBy)){
        private$.isReplacedBy <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnitTypeClassification", proposedUrnVector=isReplacedBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isReplacedBy, referencingAssociationName="isReplacedBy",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 7 properties: 
    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_GeographicUnitTypeIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },

    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },

    isCurrent = function(val){
      if (!missing(val)){
        private$.isCurrent <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isCurrent
      }
    },

    isFloating = function(val){
      if (!missing(val)){
        private$.isFloating <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isFloating
      }
    },

    releaseDate = function(val){
      if (!missing(val)){
        private$.releaseDate <- returnR6ObjectIfValid(validClass="DDI4_Date", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.releaseDate
      }
    },

    usage = function(val){
      if (!missing(val)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.usage
      }
    },

    validDates = function(val){
      if (!missing(val)){
        private$.validDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.validDates
      }
    },


# There are 6 class association(s):
    isMaintainedBy = function(val){
      if (!missing(val)){
        private$.isMaintainedBy <- returnUrnVectorIfValid(validClass="DDI4_Organization", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isMaintainedBy, referencingAssociationName="isMaintainedBy",  referencingUrn=thisUrn)  
       # invisible( private$.isMaintainedBy)
      } else {
          private$.isMaintainedBy
      }
    },

    isReplacedBy = function(val){
      if (!missing(val)){
        private$.isReplacedBy <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnitTypeClassification", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isReplacedBy, referencingAssociationName="isReplacedBy",  referencingUrn=thisUrn)  
       # invisible( private$.isReplacedBy)
      } else {
          private$.isReplacedBy
      }
    },

    isStructuredBy = function(val){
      if (!missing(val)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnitTypeRelationStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
       # invisible( private$.isStructuredBy)
      } else {
          private$.isStructuredBy
      }
    },

    references = function(val){
      if (!missing(val)){
        private$.references <- returnUrnVectorIfValid(validClass="DDI4_ConceptSystem", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.references, referencingAssociationName="references",  referencingUrn=thisUrn)  
       # invisible( private$.references)
      } else {
          private$.references
      }
    },

    replaces = function(val){
      if (!missing(val)){
        private$.replaces <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnitTypeClassification", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.replaces, referencingAssociationName="replaces",  referencingUrn=thisUrn)  
       # invisible( private$.replaces)
      } else {
          private$.replaces
      }
    },

    variantOf = function(val){
      if (!missing(val)){
        private$.variantOf <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnitTypeClassification", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.variantOf, referencingAssociationName="variantOf",  referencingUrn=thisUrn)  
       # invisible( private$.variantOf)
      } else {
          private$.variantOf
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .contains = NULL,
    .releaseDate = NULL,
    .validDates = NULL,
    .isCurrent = NULL,
    .isFloating = NULL,
    .displayLabel = NULL,
    .usage = NULL,
    .references = NULL,
    .isStructuredBy = NULL,
    .isMaintainedBy = NULL,
    .variantOf = NULL,
    .replaces = NULL,
    .isReplacedBy = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_GeographicUnitTypeIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isCurrent = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isFloating = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isMaintainedBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Organization", 
          structure = "DdiUrn vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isReplacedBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_GeographicUnitTypeClassification", 
          structure = "DdiUrn vector"
        ),
        isStructuredBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_GeographicUnitTypeRelationStructure", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        references = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ConceptSystem", 
          structure = "DdiUrn vector"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        releaseDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Date", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        replaces = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_GeographicUnitTypeClassification", 
          structure = "DdiUrn vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        usage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usesLevelStructure = c(
          min="0", 
          max="1", 
          datatype = "DDI4_LevelStructure", 
          structure = "DdiUrn vector"
        ),
        validDates = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        variantOf = c(
          min="0", 
          max="1", 
          datatype = "DDI4_GeographicUnitTypeClassification", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_GeographicUnitTypeClassification


#' @title DDI4_GeographicUnitTypeIndicator  Provides ability to declare an optional sequence or index order to a Unit Type describing a Geographic Unit Type
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/GeographicUnitTypeIndicator}{GeographicUnitTypeIndicator} 

#'
#'   Definition
#'
#'   ============
#'
#'   Provides ability to declare an optional sequence or index order to a Unit Type describing a Geographic Unit Type. Extended to support membership in a specific level.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_GeographicUnitTypeIndicator$DdiUrn} returns the DDI URN of myDDI4_GeographicUnitTypeIndicator.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Index number expressed as an integer. The position of the member in an ordered array. Optional for unordered Collections.
#'
#'                              \code{myDDI4_GeographicUnitTypeIndicator$index <- myinteger} sets the index property of 'myDDI4_GeographicUnitTypeIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_GeographicUnitTypeIndicator$index} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{isInLevel}}{ A read/write active binding.
#'        Indicates the level within which the CodeItem resides
#'
#'                              \code{myDDI4_GeographicUnitTypeIndicator$isInLevel <- myinteger} sets the isInLevel property of 'myDDI4_GeographicUnitTypeIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_GeographicUnitTypeIndicator$isInLevel} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{member}}{ A read/write active binding.
#'          Member of the collection.
#'
#'                              \code{myDDI4_GeographicUnitTypeIndicator$member <- c(myUnitTypeURN)} sets the association 'member' from 'myDDI4_GeographicUnitTypeIndicator' to a character vector of DdiUrns for R6 UnitType class instances
#'
#'                              \code{myDDI4_GeographicUnitTypeIndicator$member} returns a character vector of of DdiUrns for R6  UnitType class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_GeographicUnitTypeIndicator$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, isInLevel=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_GeographicUnitTypeIndicator <- R6Class(classname="DDI4_GeographicUnitTypeIndicator",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA, isInLevel=NA,    member=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_GeographicUnitTypeIndicator may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isInLevel)){
        private$.isInLevel <- returnVectorIfValid(validVectorClass="integer", proposedVector=isInLevel , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(member)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=member , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 2 properties: 
    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.index
      }
    },

    isInLevel = function(val){
      if (!missing(val)){
        private$.isInLevel <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isInLevel
      }
    },


# There are 1 class association(s):
    member = function(val){
      if (!missing(val)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
       # invisible( private$.member)
      } else {
          private$.member
      }
    }

  ),

  private = list(
    .index = NULL,
    .isInLevel = NULL,
    .member = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        index = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        isInLevel = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        member = c(
          min="1", 
          max="1", 
          datatype = "DDI4_UnitType", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_GeographicUnitTypeIndicator


#' @title DDI4_GeographicUnitTypeRelation  Describes structured relationship between Unit Types used to define Geographic Unit Types
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/GeographicUnitTypeRelation}{GeographicUnitTypeRelation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Describes structured relationship between Unit Types used to define Geographic Unit Types. Extended to include the ability to define a Spatial Relationship using a controlled vocabulary.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   A member might have a ParentChild relation to several other members which is Anti-Reflexive, Anti-Symmetric, and Anti-Transitive. For a process this might have a semantic of "immediately follows". 
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_GeographicUnitTypeRelation$DdiUrn} returns the DDI URN of myDDI4_GeographicUnitTypeRelation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an enumeration list
#'
#'                              \code{myDDI4_GeographicUnitTypeRelation$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_GeographicUnitTypeRelation' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_GeographicUnitTypeRelation$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{hasSpatialRelationSpecification}}{ A read/write active binding.
#'        Provides information on the spatial relationship between the parent and child. 
#'
#'                              \code{myDDI4_GeographicUnitTypeRelation$hasSpatialRelationSpecification <- list(mySpatialRelationSpecification)} sets the 'hasSpatialRelationSpecification' property of 'myDDI4_GeographicUnitTypeRelation' to a list of R6 SpatialRelationSpecification class instances
#'
#'                              \code{myDDI4_GeographicUnitTypeRelation$hasSpatialRelationSpecification} returns a list of of R6 : SpatialRelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{isExhaustiveCoverage}}{ A read/write active binding.
#'        An aggregation of the units of the child Unit Type will result in exhaustive coverage of the parent Unit Type
#'
#'                              \code{myDDI4_GeographicUnitTypeRelation$isExhaustiveCoverage <- mylogical} sets the isExhaustiveCoverage property of 'myDDI4_GeographicUnitTypeRelation' to the vector of datatype:logical
#'
#'                              \code{myDDI4_GeographicUnitTypeRelation$isExhaustiveCoverage} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provide a semantic that provides a context for the relationship using and External Controlled Vocabulary
#'
#'                              \code{myDDI4_GeographicUnitTypeRelation$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_GeographicUnitTypeRelation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_GeographicUnitTypeRelation$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of its totality using an enumeration list.
#'
#'                              \code{myDDI4_GeographicUnitTypeRelation$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_GeographicUnitTypeRelation' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_GeographicUnitTypeRelation$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          The first member of the relationship. 
#'
#'                              \code{myDDI4_GeographicUnitTypeRelation$source <- c(myUnitTypeURN)} sets the association 'source' from 'myDDI4_GeographicUnitTypeRelation' to a character vector of DdiUrns for R6 UnitType class instances
#'
#'                              \code{myDDI4_GeographicUnitTypeRelation$source} returns a character vector of of DdiUrns for R6  UnitType class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          Second member in the relationship. Notes this could be realized as a collection supporting tuple relationship.
#'
#'                              \code{myDDI4_GeographicUnitTypeRelation$target <- c(myUnitTypeURN)} sets the association 'target' from 'myDDI4_GeographicUnitTypeRelation' to a character vector of DdiUrns for R6 UnitType class instances
#'
#'                              \code{myDDI4_GeographicUnitTypeRelation$target} returns a character vector of of DdiUrns for R6  UnitType class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_GeographicUnitTypeRelation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, hasSpatialRelationSpecification=NA, isExhaustiveCoverage=NA, semantic=NA, totality=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_GeographicUnitTypeRelation <- R6Class(classname="DDI4_GeographicUnitTypeRelation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, hasSpatialRelationSpecification=NA, isExhaustiveCoverage=NA, semantic=NA, totality=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_GeographicUnitTypeRelation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(hasSpatialRelationSpecification)){
        private$.hasSpatialRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_SpatialRelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasSpatialRelationSpecification , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isExhaustiveCoverage)){
        private$.isExhaustiveCoverage <- returnVectorIfValid(validVectorClass="logical", proposedVector=isExhaustiveCoverage , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=source , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=target , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 5 properties: 
    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    hasSpatialRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasSpatialRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_SpatialRelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasSpatialRelationSpecification
      }
    },

    isExhaustiveCoverage = function(val){
      if (!missing(val)){
        private$.isExhaustiveCoverage <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isExhaustiveCoverage
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .hasSpatialRelationSpecification = NULL,
    .isExhaustiveCoverage = NULL,
    .semantic = NULL,
    .totality = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        hasSpatialRelationSpecification = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SpatialRelationSpecification", 
          structure = "Enumeration vector"
        ),
        isExhaustiveCoverage = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="0", 
          max="1", 
          datatype = "DDI4_UnitType", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_UnitType", 
          structure = "DdiUrn vector"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_GeographicUnitTypeRelation


#' @title DDI4_GeographicUnitTypeRelationStructure  Defines the relationships between Geographic Unit Types in a collection
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/GeographicUnitTypeRelationStructure}{GeographicUnitTypeRelationStructure} 

#'
#'   Definition
#'
#'   ============
#'
#'   Defines the relationships between Geographic Unit Types in a collection.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_GeographicUnitTypeRelationStructure$DdiUrn} returns the DDI URN of myDDI4_GeographicUnitTypeRelationStructure.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an ennumeration list
#'
#'                              \code{myDDI4_GeographicUnitTypeRelationStructure$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_GeographicUnitTypeRelationStructure' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_GeographicUnitTypeRelationStructure$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship context for the relationship using an External Controlled Vocabulary. Examples might include "parent-child", or "immediate supervisee"
#'
#'                              \code{myDDI4_GeographicUnitTypeRelationStructure$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_GeographicUnitTypeRelationStructure' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_GeographicUnitTypeRelationStructure$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Controlled Vocabulary to specify whether the relation is total, partial or unknown.
#'
#'                              \code{myDDI4_GeographicUnitTypeRelationStructure$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_GeographicUnitTypeRelationStructure' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_GeographicUnitTypeRelationStructure$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMemberRelation}}{ A read/write active binding.
#'        MemberRelation used to define the relationship of members within the collection
#'
#'                              \code{myDDI4_GeographicUnitTypeRelationStructure$hasMemberRelation <- list(myGeographicUnitTypeRelation)} sets the 'hasMemberRelation' property of 'myDDI4_GeographicUnitTypeRelationStructure' to a list of R6 GeographicUnitTypeRelation class instances
#'
#'                              \code{myDDI4_GeographicUnitTypeRelationStructure$hasMemberRelation} returns a list of of R6 : GeographicUnitTypeRelation class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_GeographicUnitTypeRelationStructure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_GeographicUnitTypeRelationStructure <- R6Class(classname="DDI4_GeographicUnitTypeRelationStructure",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_GeographicUnitTypeRelationStructure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasMemberRelation)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_GeographicUnitTypeRelation", proposedObjectList=hasMemberRelation , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    hasMemberRelation = function(val){
      if (!missing(val)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_GeographicUnitTypeRelation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasMemberRelation
      }
    },

    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .hasMemberRelation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasMemberRelation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_GeographicUnitTypeRelation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_GeographicUnitTypeRelationStructure


#' @title DDI4_Goal  Goals are the "things" a method aims to achieve
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Goal}{Goal} 

#'
#'   Definition
#'
#'   ============
#'
#'   Goals are the "things" a method aims to achieve. A goal may be a business function (GSIM) corresponding to a function in a catalog of functions such as GSBPM or GLBMN. However, goals may be specified more broadly. For example, conducting a clinical trial might be the goal of a method. Machine learning might be the goal of a method. 
#'
#'   
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   To further distinguish a "goal" from an "output", consider the construction of a statistic. A goal may be the construction of the statistic. The output is the statistic. The output is data. The statistic by itself -- the output -- is arguably meaningless even if we capture the process of its construction. 
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Business Function
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_BusinessFunction}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isDiscussedIn}}{ A read/write active binding.
#'          Identifies material discussing the goal. The material may be in DDI or other format.
#'
#'                              \code{myDDI4_Goal$isDiscussedIn <- c(myExternalMaterialURN)} sets the association 'isDiscussedIn' from 'myDDI4_Goal' to a character vector of DdiUrns for R6 ExternalMaterial class instances
#'
#'                              \code{myDDI4_Goal$isDiscussedIn} returns a character vector of of DdiUrns for R6  ExternalMaterial class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Goal$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Goal <- R6Class(classname="DDI4_Goal",
  inherit = DDI4_BusinessFunction,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, isDiscussedIn=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Goal may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)



       if(!missing(isDiscussedIn)){
        private$.isDiscussedIn <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=isDiscussedIn , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isDiscussedIn, referencingAssociationName="isDiscussedIn",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 class association(s):
    isDiscussedIn = function(val){
      if (!missing(val)){
        private$.isDiscussedIn <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isDiscussedIn, referencingAssociationName="isDiscussedIn",  referencingUrn=thisUrn)  
       # invisible( private$.isDiscussedIn)
      } else {
          private$.isDiscussedIn
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .isDiscussedIn = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isDiscussedIn = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Goal


#' @title DDI4_Guide  Provides a guide for the usage of a result within a specified application
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Guide}{Guide} 

#'
#'   Definition
#'
#'   ============
#'
#'   Provides a guide for the usage of a result within a specified application
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   The applied use of a weight determined by a weighting process in analyzing a data set
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Guide$DdiUrn} returns the DDI URN of myDDI4_Guide.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{overview}}{ A read/write active binding.
#'        Provides a high level overview or summary of the class. Can be used to inform end-users or as part of an executive summary. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_Guide$overview <- list(myInternationalStructuredString)} sets the 'overview' property of 'myDDI4_Guide' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Guide$overview} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isDiscussedIn}}{ A read/write active binding.
#'          Identifies material discussing the precondition. The material may be in DDI or other format.
#'
#'                              \code{myDDI4_Guide$isDiscussedIn <- c(myExternalMaterialURN)} sets the association 'isDiscussedIn' from 'myDDI4_Guide' to a character vector of DdiUrns for R6 ExternalMaterial class instances
#'
#'                              \code{myDDI4_Guide$isDiscussedIn} returns a character vector of of DdiUrns for R6  ExternalMaterial class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Guide$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, overview=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Guide <- R6Class(classname="DDI4_Guide",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, overview=NA,    isDiscussedIn=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Guide may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(overview)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=overview , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(isDiscussedIn)){
        private$.isDiscussedIn <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=isDiscussedIn , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isDiscussedIn, referencingAssociationName="isDiscussedIn",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    overview = function(val){
      if (!missing(val)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overview
      }
    },


# There are 1 class association(s):
    isDiscussedIn = function(val){
      if (!missing(val)){
        private$.isDiscussedIn <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isDiscussedIn, referencingAssociationName="isDiscussedIn",  referencingUrn=thisUrn)  
       # invisible( private$.isDiscussedIn)
      } else {
          private$.isDiscussedIn
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .overview = NULL,
    .isDiscussedIn = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isDiscussedIn = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Guide


#' @title DDI4_Identifiable  Used to identify objects for purposes of internal and/or external referencing
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Identifiable}{Identifiable} 

#'
#'   Definition
#'
#'   ============
#'
#'   Used to identify objects for purposes of internal and/or external referencing. Elements of this type are versioned and provide administrative metadata properties. Use for First Order Classes whose content does not need to be discoverable in its own right but needs to be related to multiple classes. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Identifiable$DdiUrn} returns the DDI URN of myDDI4_Identifiable.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{agency}}{ A read/write active binding.
#'        This is the registered agency code with optional sub-agencies separated by dots. For example, diw.soep, ucl.qss, abs.essg.
#'
#'                              \code{myDDI4_Identifiable$agency <- mycharacter} sets the agency property of 'myDDI4_Identifiable' to the vector of datatype:character
#'
#'                              \code{myDDI4_Identifiable$agency} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{id}}{ A read/write active binding.
#'        The ID of the object. This must conform to the allowed structure of the DDI Identifier and must be unique within the Agency.
#'
#'                              \code{myDDI4_Identifiable$id <- mycharacter} sets the id property of 'myDDI4_Identifiable' to the vector of datatype:character
#'
#'                              \code{myDDI4_Identifiable$id} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{version}}{ A read/write active binding.
#'        The version number of the object. The version number is incremented whenever the non-administrative metadata contained by the object changes.
#'
#'                              \code{myDDI4_Identifiable$version <- mycharacter} sets the version property of 'myDDI4_Identifiable' to the vector of datatype:character
#'
#'                              \code{myDDI4_Identifiable$version} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{versionResponsibility}}{ A read/write active binding.
#'        Contributor who has the ownership and responsibility for the current version. 
#'
#'                              \code{myDDI4_Identifiable$versionResponsibility <- mycharacter} sets the versionResponsibility property of 'myDDI4_Identifiable' to the vector of datatype:character
#'
#'                              \code{myDDI4_Identifiable$versionResponsibility} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{versionRationale}}{ A read/write active binding.
#'        The reason for making this version of the object.
#'
#'                              \code{myDDI4_Identifiable$versionRationale <- list(myRationaleDefinition)} sets the 'versionRationale' property of 'myDDI4_Identifiable' to a list of R6 RationaleDefinition class instances
#'
#'                              \code{myDDI4_Identifiable$versionRationale} returns a list of of R6 : RationaleDefinition class instances}
#'   }
#'   \describe{
#'     \item{\code{versionDate}}{ A read/write active binding.
#'        The date and time the object was changed. Supports standard ISO date and datetime formats.
#'
#'                              \code{myDDI4_Identifiable$versionDate <- list(myIsoDateType)} sets the 'versionDate' property of 'myDDI4_Identifiable' to a list of R6 IsoDateType class instances
#'
#'                              \code{myDDI4_Identifiable$versionDate} returns a list of of R6 : IsoDateType class instances}
#'   }
#'   \describe{
#'     \item{\code{isUniversallyUnique}}{ A read/write active binding.
#'        Default value is false. Usually the combination of agency and id (ignoring different versions) is unique. If isUniversallyUnique is set to true, it indicates that the id itself is universally unique (unique across systems and/or agencies) and therefore the agency part is not required to ensure uniqueness. 
#'
#'                              \code{myDDI4_Identifiable$isUniversallyUnique <- mylogical} sets the isUniversallyUnique property of 'myDDI4_Identifiable' to the vector of datatype:logical
#'
#'                              \code{myDDI4_Identifiable$isUniversallyUnique} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{isPersistent}}{ A read/write active binding.
#'        Default value is false. Usually the content of the current version is allowed to change, for example as the contributor is working on the object contents. However, when isPersistent is true, it indicates the there will be no more changes to the current version.
#'
#'                              \code{myDDI4_Identifiable$isPersistent <- mylogical} sets the isPersistent property of 'myDDI4_Identifiable' to the vector of datatype:logical
#'
#'                              \code{myDDI4_Identifiable$isPersistent} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{localId}}{ A read/write active binding.
#'        This is an identifier in a given local context that uniquely references an object, as opposed to the full ddi identifier which has an agency plus the id. For example, localId could be a variable name in a dataset. 
#'
#'                              \code{myDDI4_Identifiable$localId <- list(myLocalIdFormat)} sets the 'localId' property of 'myDDI4_Identifiable' to a list of R6 LocalIdFormat class instances
#'
#'                              \code{myDDI4_Identifiable$localId} returns a list of of R6 : LocalIdFormat class instances}
#'   }
#'   \describe{
#'     \item{\code{basedOnObject}}{ A read/write active binding.
#'        The object/version that this object version is based on.
#'
#'                              \code{myDDI4_Identifiable$basedOnObject <- list(myBasedOnObjectInformation)} sets the 'basedOnObject' property of 'myDDI4_Identifiable' to a list of R6 BasedOnObjectInformation class instances
#'
#'                              \code{myDDI4_Identifiable$basedOnObject} returns a list of of R6 : BasedOnObjectInformation class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Identifiable$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{DdiUrn()}}{returns the DDI URN for the object. The DDI URN is a globally unique identifier for the object. }
#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, agency=NA, id=NA, version=NA, versionResponsibility=NA, versionRationale=NA, versionDate=NA, isUniversallyUnique=NA, isPersistent=NA, localId=NA, basedOnObject=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Identifiable <- R6Class(classname="DDI4_Identifiable",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, agency=NA, id=NA, version=NA, versionResponsibility=NA, versionRationale=NA, versionDate=NA, isUniversallyUnique=NA, isPersistent=NA, localId=NA, basedOnObject=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Identifiable may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(agency)){
        private$.agency <- returnVectorIfValid(validVectorClass="character", proposedVector=agency , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.agency <- "example.org"
###          warning('Agency was missing. It has been set to "example.org" as a temporary value')
      }
      if(!missing(id)){
        private$.id <- returnVectorIfValid(validVectorClass="character", proposedVector=id , minimumCardinality=1, maximumCardinality=1)  
      } else {
         private$.id <- uuid::UUIDgenerate(TRUE)
###           warning(paste("Id was missing. It has been set to the unique random UUID: ",private$id, "You can change this with the set_id method."))
      }
      if(!missing(version)){
        private$.version <- returnVectorIfValid(validVectorClass="character", proposedVector=version , minimumCardinality=1, maximumCardinality=1)  
      } else {
         private$.version <- "1"
###           warning('Version was missing. It has been set to "1" as a temporary value')
      }
      if(!missing(versionResponsibility)){
        private$.versionResponsibility <- returnVectorIfValid(validVectorClass="character", proposedVector=versionResponsibility , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(versionRationale)){
        private$.versionRationale <- returnR6ObjectIfValid(validClass="DDI4_RationaleDefinition", proposedObjectList=versionRationale , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(versionDate)){
        private$.versionDate <- returnRegularExpressionValues(RegularExpressionObject=DDI4_IsoDateType$new(registerThisObject=FALSE), proposedCharacterVector=versionDate , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isUniversallyUnique)){
        private$.isUniversallyUnique <- returnVectorIfValid(validVectorClass="logical", proposedVector=isUniversallyUnique , minimumCardinality=1, maximumCardinality=1)  
      } else {
           private$.isUniversallyUnique <- FALSE
      }
      if(!missing(isPersistent)){
        private$.isPersistent <- returnVectorIfValid(validVectorClass="logical", proposedVector=isPersistent , minimumCardinality=1, maximumCardinality=1)  
      } else {
           private$.isPersistent <- FALSE
      }
      if(!missing(localId)){
        private$.localId <- returnR6ObjectIfValid(validClass="DDI4_LocalIdFormat", proposedObjectList=localId , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(basedOnObject)){
        private$.basedOnObject <- returnR6ObjectIfValid(validClass="DDI4_BasedOnObjectInformation", proposedObjectList=basedOnObject , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")

      if(registerThisObject) registerObject(DdiUrn=private$.DdiUrn, self)  



     }
  ),

  active = list(

# There are 10 properties: 
    agency = function(val){
      if (!missing(val)){
        if (private$.isPersistent) stop(" isPersistent is true for this object. The agency, id or version cannot be changed for a persistent object.") 
# this object exists deRegister it before changing its DdiUrn 
        deRegisterObject(private$.DdiUrn)  
        private$.agency <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
        private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        registerObject(DdiUrn=private$.DdiUrn, self)
      } else {
        private$.agency
      }
    },

    basedOnObject = function(val){
      if (!missing(val)){
        private$.basedOnObject <- returnR6ObjectIfValid(validClass="DDI4_BasedOnObjectInformation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.basedOnObject
      }
    },

    id = function(val){
      if (!missing(val)){
        if (private$.isPersistent) stop(" isPersistent is true for this object. The agency, id or version cannot be changed for a persistent object.") 
# this object exists deRegister it before changing its DdiUrn 
        deRegisterObject(private$.DdiUrn)  
        private$.id <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
        private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        registerObject(DdiUrn=private$.DdiUrn, self)
      } else {
        private$.id
      }
    },

    isPersistent = function(val){
      if (!missing(val)){
        private$.isPersistent <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.isPersistent
      }
    },

    isUniversallyUnique = function(val){
      if (!missing(val)){
        private$.isUniversallyUnique <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.isUniversallyUnique
      }
    },

    localId = function(val){
      if (!missing(val)){
        private$.localId <- returnR6ObjectIfValid(validClass="DDI4_LocalIdFormat", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.localId
      }
    },

    version = function(val){
      if (!missing(val)){
        if (private$.isPersistent) stop(" isPersistent is true for this object. The agency, id or version cannot be changed for a persistent object.") 
# this object exists deRegister it before changing its DdiUrn 
        deRegisterObject(private$.DdiUrn)  
        private$.version <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
        private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        registerObject(DdiUrn=private$.DdiUrn, self)
      } else {
        private$.version
      }
    },

    versionDate = function(val){
      if (!missing(val)){
        private$.versionDate <- returnRegularExpressionValues(RegularExpressionObject=DDI4_IsoDateType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.versionDate
      }
    },

    versionRationale = function(val){
      if (!missing(val)){
        private$.versionRationale <- returnR6ObjectIfValid(validClass="DDI4_RationaleDefinition", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.versionRationale
      }
    },

    versionResponsibility = function(val){
      if (!missing(val)){
        private$.versionResponsibility <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.versionResponsibility
      }
    }


  ),

  private = list(
    .agency = NULL,
    .id = NULL,
    .version = NULL,
    .versionResponsibility = NULL,
    .versionRationale = NULL,
    .versionDate = NULL,
    .isUniversallyUnique = NULL,
    .isPersistent = NULL,
    .localId = NULL,
    .basedOnObject = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_Identifiable


#' @title DDI4_IdentifierRole  An IdentifierRole identifies one or more InstanceVariables as being identifiers within a ViewPoint
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/IdentifierRole}{IdentifierRole} 

#'
#'   Definition
#'
#'   ============
#'
#'   An IdentifierRole identifies one or more InstanceVariables as being identifiers within a ViewPoint. An IdentifierRole is a SimpleCollection of InstanceVariables acting in the IdentifierRole.
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A data record with four variables: "PersonId", "Systolic", "Diastolic", "Seated" might have a Viewpoint with 
#'
#'   PersonId defined as having the IdentifierRole
#'
#'   Systolic defined as having the MeasureRole
#'
#'   Diastolic defined as having the MeasureRole
#'
#'   Seated defined as having the AttributeRole
#'
#'   
#'
#'   PersonId, Systolic, Diastolic, Seated
#'
#'   123,122,20,yes
#'
#'   145,130,90,no
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   See the Viewpoint documentation for an in depth discussion of the uses of ViewpointRoles: http://lion.ddialliance.org/ddiobjects/viewpoint
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ViewpointRole}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_IdentifierRole$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_IdentifierRole <- R6Class(classname="DDI4_IdentifierRole",
  inherit = DDI4_ViewpointRole,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_IdentifierRole may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

    }
  ),

  active = list(
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InstanceVariableIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_IdentifierRole


#' @title DDI4_IfThenElse  IfThenElse describes an if-then-else decision type of control construct
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/IfThenElse}{IfThenElse} 

#'
#'   Definition
#'
#'   ============
#'
#'   IfThenElse describes an if-then-else decision type of control construct. If the stated condition is met, then the associated Workflow Sequence in containsSubSeqence is triggered, otherwise the Workflow Sequence that is triggered is the one associated via elseContains.
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   An IfThenElse object describes the conditional logic in the flow of a questionnaire or other data collection instrument, where if a stated condition is met, one path is followed through the flow, and if the stated condition is met, another path is taken.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Contains a condition and two associations:
#'
#'   - one to the Workflow Sequence that is triggered when the condition is true (containsSubSequence), and 
#'
#'   - another to the Workflow Sequence that is triggered when the condition is false (elseContains). 
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   d:IfThenElseType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ConditionalControlStep}}}
#'    \item{\code{\link{DDI4_WorkflowControlStep}}}
#'    \item{\code{\link{DDI4_WorkflowStep}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_IfThenElse$DdiUrn} returns the DDI URN of myDDI4_IfThenElse.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{elseIf}}{ A read/write active binding.
#'        The condition to check if the IfThenElse condition is false. 
#'
#'                              \code{myDDI4_IfThenElse$elseIf <- list(myElseIfAction)} sets the 'elseIf' property of 'myDDI4_IfThenElse' to a list of R6 ElseIfAction class instances
#'
#'                              \code{myDDI4_IfThenElse$elseIf} returns a list of of R6 : ElseIfAction class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{elseExecutes}}{ A read/write active binding.
#'          Optional sequence of Process Steps that are triggered for execution when the condition is false.
#'
#'                              \code{myDDI4_IfThenElse$elseExecutes <- c(myWorkflowStepSequenceURN)} sets the association 'elseExecutes' from 'myDDI4_IfThenElse' to a character vector of DdiUrns for R6 WorkflowStepSequence class instances
#'
#'                              \code{myDDI4_IfThenElse$elseExecutes} returns a character vector of of DdiUrns for R6  WorkflowStepSequence class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_IfThenElse$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, elseIf=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_IfThenElse <- R6Class(classname="DDI4_IfThenElse",
  inherit = DDI4_ConditionalControlStep,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, elseIf=NA,    elseExecutes=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_IfThenElse may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(elseIf)){
        private$.elseIf <- returnR6ObjectIfValid(validClass="DDI4_ElseIfAction", proposedObjectList=elseIf , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(elseExecutes)){
        private$.elseExecutes <- returnUrnVectorIfValid(validClass="DDI4_WorkflowStepSequence", proposedUrnVector=elseExecutes , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.elseExecutes, referencingAssociationName="elseExecutes",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    elseIf = function(val){
      if (!missing(val)){
        private$.elseIf <- returnR6ObjectIfValid(validClass="DDI4_ElseIfAction", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.elseIf
      }
    },


# There are 1 class association(s):
    elseExecutes = function(val){
      if (!missing(val)){
        private$.elseExecutes <- returnUrnVectorIfValid(validClass="DDI4_WorkflowStepSequence", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.elseExecutes, referencingAssociationName="elseExecutes",  referencingUrn=thisUrn)  
       # invisible( private$.elseExecutes)
      } else {
          private$.elseExecutes
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .elseIf = NULL,
    .elseExecutes = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        condition = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CommandCode", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        elseExecutes = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowStepSequence", 
          structure = "DdiUrn vector"
        ),
        elseIf = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ElseIfAction", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        executes = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowStepSequence", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInformationFlow = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPerformedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowService", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_IfThenElse


#' @title DDI4_Image  A reference to an image, with a description of its properties and type
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Image}{Image} 

#'
#'   Definition
#'
#'   ============
#'
#'   A reference to an image, with a description of its properties and type.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:ImageType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Image$DdiUrn} returns the DDI URN of myDDI4_Image.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{uri}}{ A read/write active binding.
#'        A reference to the location of the image using a URI.
#'
#'                              \code{myDDI4_Image$uri <- mycharacter} sets the uri property of 'myDDI4_Image' to the vector of datatype:character
#'
#'                              \code{myDDI4_Image$uri} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{typeOfImage}}{ A read/write active binding.
#'        Brief description of the image type. Supports the use of an external controlled vocabulary.
#'
#'                              \code{myDDI4_Image$typeOfImage <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfImage' property of 'myDDI4_Image' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_Image$typeOfImage} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{dpi}}{ A read/write active binding.
#'        Provides the resolution of the image in dots per inch to assist in selecting the appropriate image for various uses.
#'
#'                              \code{myDDI4_Image$dpi <- myinteger} sets the dpi property of 'myDDI4_Image' to the vector of datatype:integer
#'
#'                              \code{myDDI4_Image$dpi} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{languageOfImage}}{ A read/write active binding.
#'        Language of image. Supports the indication of multiple languages. Supports use of codes defined by the RFC 1766.
#'
#'                              \code{myDDI4_Image$languageOfImage <- mycharacter} sets the languageOfImage property of 'myDDI4_Image' to the vector of datatype:character
#'
#'                              \code{myDDI4_Image$languageOfImage} returns a vector of datatype: character}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Image$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, uri=NA, typeOfImage=NA, dpi=NA, languageOfImage=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Image <- R6Class(classname="DDI4_Image",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, uri=NA, typeOfImage=NA, dpi=NA, languageOfImage=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Image may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(uri)){
        private$.uri <- returnVectorIfValid(validVectorClass="character", proposedVector=uri , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(typeOfImage)){
        private$.typeOfImage <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfImage , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(dpi)){
        private$.dpi <- returnVectorIfValid(validVectorClass="integer", proposedVector=dpi , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(languageOfImage)){
        private$.languageOfImage <- returnVectorIfValid(validVectorClass="character", proposedVector=languageOfImage , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    dpi = function(val){
      if (!missing(val)){
        private$.dpi <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.dpi
      }
    },

    languageOfImage = function(val){
      if (!missing(val)){
        private$.languageOfImage <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.languageOfImage
      }
    },

    typeOfImage = function(val){
      if (!missing(val)){
        private$.typeOfImage <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.typeOfImage
      }
    },

    uri = function(val){
      if (!missing(val)){
        private$.uri <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.uri
      }
    }


  ),

  private = list(
    .uri = NULL,
    .typeOfImage = NULL,
    .dpi = NULL,
    .languageOfImage = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        dpi = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        languageOfImage = c(
          min="0", 
          max="Inf", 
          datatype = "character", 
          structure = "vector"
        ),
        typeOfImage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        uri = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Image


#' @title DDI4_ImageArea  Defines the shape and area of an image used as part of a location representation
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ImageArea}{ImageArea} 

#'
#'   Definition
#'
#'   ============
#'
#'   Defines the shape and area of an image used as part of a location representation. The shape is defined as a Rectangle, Circle, or Polygon and content provides the information required to define it in the form of a comma-delimited list of x,y coordinates, listed as a set of adjacent points for rectangles and polygons, and as a center-point and a radius for circles (x,y,r).
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:ImageAreaType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ImageArea$DdiUrn} returns the DDI URN of myDDI4_ImageArea.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{content}}{ A read/write active binding.
#'        A comma-delimited list of x,y coordinates, listed as a set of adjacent points for rectangles and polygons, and as a center-point and a radius for circles (x,y,r).
#'
#'                              \code{myDDI4_ImageArea$content <- mycharacter} sets the content property of 'myDDI4_ImageArea' to the vector of datatype:character
#'
#'                              \code{myDDI4_ImageArea$content} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{shape}}{ A read/write active binding.
#'        A fixed set of valid responses includes Rectangle, Circle, and Polygon.
#'
#'                              \code{myDDI4_ImageArea$shape <- list(myShapeCoded)} sets the 'shape' property of 'myDDI4_ImageArea' to a list of R6 ShapeCoded class instances
#'
#'                              \code{myDDI4_ImageArea$shape} returns a list of of R6 : ShapeCoded class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ImageArea$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, content=NA, shape=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ImageArea <- R6Class(classname="DDI4_ImageArea",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, content=NA, shape=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ImageArea may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(content)){
        private$.content <- returnVectorIfValid(validVectorClass="character", proposedVector=content , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(shape)){
        private$.shape <- returnEnumeratedValues(enumerationObject=DDI4_ShapeCoded$new(registerThisObject=FALSE), proposedCharacterVector=shape , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    content = function(val){
      if (!missing(val)){
        private$.content <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.content
      }
    },

    shape = function(val){
      if (!missing(val)){
        private$.shape <- returnEnumeratedValues(enumerationObject=DDI4_ShapeCoded$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.shape
      }
    }


  ),

  private = list(
    .content = NULL,
    .shape = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        content = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        shape = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ShapeCoded", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ImageArea


#' @title DDI4_ImplementedInstrument  A specific data capture tool
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ImplementedInstrument}{ImplementedInstrument} 

#'
#'   Definition
#'
#'   ============
#'
#'   A specific data capture tool.  ImplementedInstruments are mode and/or unit specific.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Two versions of a conceptual instrument, computer assisted personal interviewing (CAPI) and mail, would be separate ImplementedInstruments. These might both reference the same ConceptualInstrument. 
#'
#'   A piece of equipment used to measure blood pressure.
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   aka PhysicalInstrument
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ImplementedInstrument$DdiUrn} returns the DDI URN of myDDI4_ImplementedInstrument.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A structured display label providing a fully human readable display for the identification of the object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_ImplementedInstrument$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_ImplementedInstrument' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_ImplementedInstrument$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#'   \describe{
#'     \item{\code{typeOfInstrument}}{ A read/write active binding.
#'        Specification of the type of instrument according to the classification system of the documentor.
#'
#'                              \code{myDDI4_ImplementedInstrument$typeOfInstrument <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfInstrument' property of 'myDDI4_ImplementedInstrument' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ImplementedInstrument$typeOfInstrument} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{uri}}{ A read/write active binding.
#'        A reference to an external representation of the the data collection instrument, such as an image of a questionnaire or programming script.
#'
#'                              \code{myDDI4_ImplementedInstrument$uri <- mycharacter} sets the uri property of 'myDDI4_ImplementedInstrument' to the vector of datatype:character
#'
#'                              \code{myDDI4_ImplementedInstrument$uri} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage. 
#'
#'                              \code{myDDI4_ImplementedInstrument$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_ImplementedInstrument' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_ImplementedInstrument$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{usage}}{ A read/write active binding.
#'        Explanation of the ways in which some decision or object is employed. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_ImplementedInstrument$usage <- list(myInternationalStructuredString)} sets the 'usage' property of 'myDDI4_ImplementedInstrument' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_ImplementedInstrument$usage} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{usesConceptualInstrument}}{ A read/write active binding.
#'          The ConceptualInstrument which informs the design of the ImplementedInstrument.
#'
#'                              \code{myDDI4_ImplementedInstrument$usesConceptualInstrument <- c(myConceptualInstrumentURN)} sets the association 'usesConceptualInstrument' from 'myDDI4_ImplementedInstrument' to a character vector of DdiUrns for R6 ConceptualInstrument class instances
#'
#'                              \code{myDDI4_ImplementedInstrument$usesConceptualInstrument} returns a character vector of of DdiUrns for R6  ConceptualInstrument class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ImplementedInstrument$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, displayLabel=NA, typeOfInstrument=NA, uri=NA, name=NA, usage=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ImplementedInstrument <- R6Class(classname="DDI4_ImplementedInstrument",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, displayLabel=NA, typeOfInstrument=NA, uri=NA, name=NA, usage=NA,    usesConceptualInstrument=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ImplementedInstrument may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(typeOfInstrument)){
        private$.typeOfInstrument <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfInstrument , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(uri)){
        private$.uri <- returnVectorIfValid(validVectorClass="character", proposedVector=uri , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(usage)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=usage , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(usesConceptualInstrument)){
        private$.usesConceptualInstrument <- returnUrnVectorIfValid(validClass="DDI4_ConceptualInstrument", proposedUrnVector=usesConceptualInstrument , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesConceptualInstrument, referencingAssociationName="usesConceptualInstrument",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 5 properties: 
    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    typeOfInstrument = function(val){
      if (!missing(val)){
        private$.typeOfInstrument <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.typeOfInstrument
      }
    },

    uri = function(val){
      if (!missing(val)){
        private$.uri <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.uri
      }
    },

    usage = function(val){
      if (!missing(val)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.usage
      }
    },


# There are 1 class association(s):
    usesConceptualInstrument = function(val){
      if (!missing(val)){
        private$.usesConceptualInstrument <- returnUrnVectorIfValid(validClass="DDI4_ConceptualInstrument", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesConceptualInstrument, referencingAssociationName="usesConceptualInstrument",  referencingUrn=thisUrn)  
       # invisible( private$.usesConceptualInstrument)
      } else {
          private$.usesConceptualInstrument
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .displayLabel = NULL,
    .typeOfInstrument = NULL,
    .uri = NULL,
    .name = NULL,
    .usage = NULL,
    .usesConceptualInstrument = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        typeOfInstrument = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        uri = c(
          min="0", 
          max="Inf", 
          datatype = "character", 
          structure = "vector"
        ),
        usage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usesConceptualInstrument = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ConceptualInstrument", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ImplementedInstrument


#' @title DDI4_IndexEntryRelation  Relationship between Index Entries defined by RelationSpecification
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/IndexEntryRelation}{IndexEntryRelation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Relationship between Index Entries defined by RelationSpecification.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_IndexEntryRelation$DdiUrn} returns the DDI URN of myDDI4_IndexEntryRelation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an ennumeration list
#'
#'                              \code{myDDI4_IndexEntryRelation$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_IndexEntryRelation' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_IndexEntryRelation$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_IndexEntryRelation$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_IndexEntryRelation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_IndexEntryRelation$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_IndexEntryRelation$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_IndexEntryRelation' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_IndexEntryRelation$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          Restricts soruce to  ClassificationEntry.
#'
#'                              \code{myDDI4_IndexEntryRelation$source <- c(myClassificationIndexEntryURN)} sets the association 'source' from 'myDDI4_IndexEntryRelation' to a character vector of DdiUrns for R6 ClassificationIndexEntry class instances
#'
#'                              \code{myDDI4_IndexEntryRelation$source} returns a character vector of of DdiUrns for R6  ClassificationIndexEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          Restricts target to  ClassificationEntry.
#'
#'                              \code{myDDI4_IndexEntryRelation$target <- c(myClassificationIndexEntryURN)} sets the association 'target' from 'myDDI4_IndexEntryRelation' to a character vector of DdiUrns for R6 ClassificationIndexEntry class instances
#'
#'                              \code{myDDI4_IndexEntryRelation$target} returns a character vector of of DdiUrns for R6  ClassificationIndexEntry class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_IndexEntryRelation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_IndexEntryRelation <- R6Class(classname="DDI4_IndexEntryRelation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_IndexEntryRelation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_ClassificationIndexEntry", proposedUrnVector=source , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_ClassificationIndexEntry", proposedUrnVector=target , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_ClassificationIndexEntry", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_ClassificationIndexEntry", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="1", 
          max="1", 
          datatype = "DDI4_ClassificationIndexEntry", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="1", 
          max="1", 
          datatype = "DDI4_ClassificationIndexEntry", 
          structure = "DdiUrn vector"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_IndexEntryRelation


#' @title DDI4_IndexEntryRelationStructure  Structures relationship of Classification Index Entries in a Classification Index
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/IndexEntryRelationStructure}{IndexEntryRelationStructure} 

#'
#'   Definition
#'
#'   ============
#'
#'   Structures relationship of Classification Index Entries in a Classification Index.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_IndexEntryRelationStructure$DdiUrn} returns the DDI URN of myDDI4_IndexEntryRelationStructure.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an enumeration list.  Use if all relations within this relation structure are of the same specification.
#'
#'                              \code{myDDI4_IndexEntryRelationStructure$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_IndexEntryRelationStructure' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_IndexEntryRelationStructure$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_IndexEntryRelationStructure$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_IndexEntryRelationStructure' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_IndexEntryRelationStructure$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_IndexEntryRelationStructure$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_IndexEntryRelationStructure' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_IndexEntryRelationStructure$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMemberRelation}}{ A read/write active binding.
#'        Index Entry Relations  that comprise the relationship structure
#'
#'                              \code{myDDI4_IndexEntryRelationStructure$hasMemberRelation <- list(myIndexEntryRelation)} sets the 'hasMemberRelation' property of 'myDDI4_IndexEntryRelationStructure' to a list of R6 IndexEntryRelation class instances
#'
#'                              \code{myDDI4_IndexEntryRelationStructure$hasMemberRelation} returns a list of of R6 : IndexEntryRelation class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_IndexEntryRelationStructure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_IndexEntryRelationStructure <- R6Class(classname="DDI4_IndexEntryRelationStructure",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_IndexEntryRelationStructure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasMemberRelation)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_IndexEntryRelation", proposedObjectList=hasMemberRelation , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    hasMemberRelation = function(val){
      if (!missing(val)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_IndexEntryRelation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasMemberRelation
      }
    },

    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .hasMemberRelation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasMemberRelation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_IndexEntryRelation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasRelationSpecification = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_IndexEntryRelationStructure


#' @title DDI4_Individual  A person who may have a relationship to another Agent and who may be specified as being associated with an act
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Individual}{Individual} 

#'
#'   Definition
#'
#'   ============
#'
#'   A person who may have a relationship to another Agent and who may be specified as being associated with an act.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Analyst performing edits on data, interviewer conducting an interview, a creator.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   A set of information describing an individual and means of unique identification and/or contact. Information may be repeated and provided effective date ranges to retain a history of the individual within a metadata record. Actions and relationships are specified by the use of the Individual as the target of a relationship (Creator) or within a collection of Agents in an AgentListing (employee of an Organization).
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Agent}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Individual$DdiUrn} returns the DDI URN of myDDI4_Individual.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasIndividualName}}{ A read/write active binding.
#'        The name of an individual broken out into its component parts of prefix, first/given name, middle name, last/family/surname, and suffix.
#'
#'                              \code{myDDI4_Individual$hasIndividualName <- list(myIndividualName)} sets the 'hasIndividualName' property of 'myDDI4_Individual' to a list of R6 IndividualName class instances
#'
#'                              \code{myDDI4_Individual$hasIndividualName} returns a list of of R6 : IndividualName class instances}
#'   }
#'   \describe{
#'     \item{\code{ddiId}}{ A read/write active binding.
#'        The agency identifier of the individual according to the DDI Alliance agent registry.
#'
#'                              \code{myDDI4_Individual$ddiId <- mycharacter} sets the ddiId property of 'myDDI4_Individual' to the vector of datatype:character
#'
#'                              \code{myDDI4_Individual$ddiId} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{hasContactInformation}}{ A read/write active binding.
#'        Contact information for the individual including location specification, address, URL, phone numbers, and other means of communication access. Sets of information can be repeated and date-stamped.
#'
#'                              \code{myDDI4_Individual$hasContactInformation <- list(myContactInformation)} sets the 'hasContactInformation' property of 'myDDI4_Individual' to a list of R6 ContactInformation class instances
#'
#'                              \code{myDDI4_Individual$hasContactInformation} returns a list of of R6 : ContactInformation class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Individual$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasIndividualName=NA, ddiId=NA, hasContactInformation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Individual <- R6Class(classname="DDI4_Individual",
  inherit = DDI4_Agent,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasIndividualName=NA, ddiId=NA, hasContactInformation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Individual may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(hasIndividualName)){
        private$.hasIndividualName <- returnR6ObjectIfValid(validClass="DDI4_IndividualName", proposedObjectList=hasIndividualName , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(ddiId)){
        private$.ddiId <- returnVectorIfValid(validVectorClass="character", proposedVector=ddiId , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(hasContactInformation)){
        private$.hasContactInformation <- returnR6ObjectIfValid(validClass="DDI4_ContactInformation", proposedObjectList=hasContactInformation , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    ddiId = function(val){
      if (!missing(val)){
        private$.ddiId <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.ddiId
      }
    },

    hasContactInformation = function(val){
      if (!missing(val)){
        private$.hasContactInformation <- returnR6ObjectIfValid(validClass="DDI4_ContactInformation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasContactInformation
      }
    },

    hasIndividualName = function(val){
      if (!missing(val)){
        private$.hasIndividualName <- returnR6ObjectIfValid(validClass="DDI4_IndividualName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasIndividualName
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasIndividualName = NULL,
    .ddiId = NULL,
    .hasContactInformation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        ddiId = c(
          min="0", 
          max="Inf", 
          datatype = "character", 
          structure = "vector"
        ),
        hasAgentId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AgentId", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasContactInformation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ContactInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasIndividualName = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_IndividualName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        image = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_PrivateImage", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Individual


#' @title DDI4_IndividualName  The name of an individual broken out into its component parts of prefix, first/given name, middle name, last/family/surname, and suffix
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/IndividualName}{IndividualName} 

#'
#'   Definition
#'
#'   ============
#'
#'   The name of an individual broken out into its component parts of prefix, first/given name, middle name, last/family/surname, and suffix. The preferred compilation of the name parts may also be provided. The legal or formal name of the individual should have the isFormal attribute set to true. The preferred name should be noted with the isPreferred attribute. The attribute sex provides information to assist in the appropriate use of pronouns.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   a:IndividualNameType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_IndividualName$DdiUrn} returns the DDI URN of myDDI4_IndividualName.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{prefix}}{ A read/write active binding.
#'        Title that precedes the name of the individual, such as Ms., or Dr.
#'
#'                              \code{myDDI4_IndividualName$prefix <- mycharacter} sets the prefix property of 'myDDI4_IndividualName' to the vector of datatype:character
#'
#'                              \code{myDDI4_IndividualName$prefix} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{firstGiven}}{ A read/write active binding.
#'        First (given) name of the individual
#'
#'                              \code{myDDI4_IndividualName$firstGiven <- mycharacter} sets the firstGiven property of 'myDDI4_IndividualName' to the vector of datatype:character
#'
#'                              \code{myDDI4_IndividualName$firstGiven} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{middle}}{ A read/write active binding.
#'        Middle name or initial of the individual
#'
#'                              \code{myDDI4_IndividualName$middle <- mycharacter} sets the middle property of 'myDDI4_IndividualName' to the vector of datatype:character
#'
#'                              \code{myDDI4_IndividualName$middle} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{lastFamily}}{ A read/write active binding.
#'        Last (family) name /surname of the individual
#'
#'                              \code{myDDI4_IndividualName$lastFamily <- mycharacter} sets the lastFamily property of 'myDDI4_IndividualName' to the vector of datatype:character
#'
#'                              \code{myDDI4_IndividualName$lastFamily} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{suffix}}{ A read/write active binding.
#'        Title that follows the name of the individual, such as Esq.
#'
#'                              \code{myDDI4_IndividualName$suffix <- mycharacter} sets the suffix property of 'myDDI4_IndividualName' to the vector of datatype:character
#'
#'                              \code{myDDI4_IndividualName$suffix} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{fullName}}{ A read/write active binding.
#'        This provides a means of providing a full name as a single object for display or print such as identification badges etc. For example a person with the name of William Grace for official use may prefer a display name of Bill Grace on a name tag or other informal publication.
#'
#'                              \code{myDDI4_IndividualName$fullName <- list(myInternationalString)} sets the 'fullName' property of 'myDDI4_IndividualName' to a list of R6 InternationalString class instances
#'
#'                              \code{myDDI4_IndividualName$fullName} returns a list of of R6 : InternationalString class instances}
#'   }
#'   \describe{
#'     \item{\code{effectiveDates}}{ A read/write active binding.
#'        Clarifies when the name information is accurate.
#'
#'                              \code{myDDI4_IndividualName$effectiveDates <- list(myDateRange)} sets the 'effectiveDates' property of 'myDDI4_IndividualName' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_IndividualName$effectiveDates} returns a list of of R6 : DateRange class instances}
#'   }
#'   \describe{
#'     \item{\code{abbreviation}}{ A read/write active binding.
#'        An abbreviation or acronym for the name. This may be expressed in multiple languages. It is assumed that if only a single language is provided that it may be used in any of the other languages within which the name itself is expressed.
#'
#'                              \code{myDDI4_IndividualName$abbreviation <- list(myInternationalString)} sets the 'abbreviation' property of 'myDDI4_IndividualName' to a list of R6 InternationalString class instances
#'
#'                              \code{myDDI4_IndividualName$abbreviation} returns a list of of R6 : InternationalString class instances}
#'   }
#'   \describe{
#'     \item{\code{typeOfIndividualName}}{ A read/write active binding.
#'        The type of individual name provided. the use of a controlled vocabulary is strongly recommended. At minimum his should include, e.g. PreviousFormalName, Nickname (or CommonName), Other.
#'
#'                              \code{myDDI4_IndividualName$typeOfIndividualName <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfIndividualName' property of 'myDDI4_IndividualName' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_IndividualName$typeOfIndividualName} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{sex}}{ A read/write active binding.
#'        Sex allows for the specification of male, female or neutral. The purpose of providing this information is to assist others in the appropriate use of pronouns when addressing the individual. Note that many countries/languages may offer a neutral pronoun form.
#'
#'                              \code{myDDI4_IndividualName$sex <- list(mySexSpecificationType)} sets the 'sex' property of 'myDDI4_IndividualName' to a list of R6 SexSpecificationType class instances
#'
#'                              \code{myDDI4_IndividualName$sex} returns a list of of R6 : SexSpecificationType class instances}
#'   }
#'   \describe{
#'     \item{\code{isPreferred}}{ A read/write active binding.
#'        If more than one name for the object is provided, use the isPreferred attribute to indicate which is the preferred name content. All other names should be set to isPreferred="false".
#'
#'                              \code{myDDI4_IndividualName$isPreferred <- mylogical} sets the isPreferred property of 'myDDI4_IndividualName' to the vector of datatype:logical
#'
#'                              \code{myDDI4_IndividualName$isPreferred} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{context}}{ A read/write active binding.
#'        A name may be specific to a particular context, i.e. common usage, business, social, etc.. Identify the context related to the specified name. Supports the use of an external controlled vocabulary.
#'
#'                              \code{myDDI4_IndividualName$context <- list(myExternalControlledVocabularyEntry)} sets the 'context' property of 'myDDI4_IndividualName' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_IndividualName$context} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{isFormal}}{ A read/write active binding.
#'        The legal or formal name of the individual should have the isFormal attribute set to true. To avoid confusion only one individual name should have the isFormal attribute set to true. Use the TypeOfIndividualName to further differentiate the type and applied usage when multiple names are provided.
#'
#'                              \code{myDDI4_IndividualName$isFormal <- mylogical} sets the isFormal property of 'myDDI4_IndividualName' to the vector of datatype:logical
#'
#'                              \code{myDDI4_IndividualName$isFormal} returns a vector of datatype: logical}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_IndividualName$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, prefix=NA, firstGiven=NA, middle=NA, lastFamily=NA, suffix=NA, fullName=NA, effectiveDates=NA, abbreviation=NA, typeOfIndividualName=NA, sex=NA, isPreferred=NA, context=NA, isFormal=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_IndividualName <- R6Class(classname="DDI4_IndividualName",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, prefix=NA, firstGiven=NA, middle=NA, lastFamily=NA, suffix=NA, fullName=NA, effectiveDates=NA, abbreviation=NA, typeOfIndividualName=NA, sex=NA, isPreferred=NA, context=NA, isFormal=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_IndividualName may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(prefix)){
        private$.prefix <- returnVectorIfValid(validVectorClass="character", proposedVector=prefix , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(firstGiven)){
        private$.firstGiven <- returnVectorIfValid(validVectorClass="character", proposedVector=firstGiven , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(middle)){
        private$.middle <- returnVectorIfValid(validVectorClass="character", proposedVector=middle , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(lastFamily)){
        private$.lastFamily <- returnVectorIfValid(validVectorClass="character", proposedVector=lastFamily , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(suffix)){
        private$.suffix <- returnVectorIfValid(validVectorClass="character", proposedVector=suffix , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(fullName)){
        private$.fullName <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=fullName , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(effectiveDates)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=effectiveDates , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(abbreviation)){
        private$.abbreviation <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=abbreviation , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(typeOfIndividualName)){
        private$.typeOfIndividualName <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfIndividualName , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(sex)){
        private$.sex <- returnEnumeratedValues(enumerationObject=DDI4_SexSpecificationType$new(registerThisObject=FALSE), proposedCharacterVector=sex , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isPreferred)){
        private$.isPreferred <- returnVectorIfValid(validVectorClass="logical", proposedVector=isPreferred , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(context)){
        private$.context <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=context , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isFormal)){
        private$.isFormal <- returnVectorIfValid(validVectorClass="logical", proposedVector=isFormal , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 13 properties: 
    abbreviation = function(val){
      if (!missing(val)){
        private$.abbreviation <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.abbreviation
      }
    },

    context = function(val){
      if (!missing(val)){
        private$.context <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.context
      }
    },

    effectiveDates = function(val){
      if (!missing(val)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.effectiveDates
      }
    },

    firstGiven = function(val){
      if (!missing(val)){
        private$.firstGiven <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.firstGiven
      }
    },

    fullName = function(val){
      if (!missing(val)){
        private$.fullName <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.fullName
      }
    },

    isFormal = function(val){
      if (!missing(val)){
        private$.isFormal <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isFormal
      }
    },

    isPreferred = function(val){
      if (!missing(val)){
        private$.isPreferred <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isPreferred
      }
    },

    lastFamily = function(val){
      if (!missing(val)){
        private$.lastFamily <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.lastFamily
      }
    },

    middle = function(val){
      if (!missing(val)){
        private$.middle <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.middle
      }
    },

    prefix = function(val){
      if (!missing(val)){
        private$.prefix <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.prefix
      }
    },

    sex = function(val){
      if (!missing(val)){
        private$.sex <- returnEnumeratedValues(enumerationObject=DDI4_SexSpecificationType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.sex
      }
    },

    suffix = function(val){
      if (!missing(val)){
        private$.suffix <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.suffix
      }
    },

    typeOfIndividualName = function(val){
      if (!missing(val)){
        private$.typeOfIndividualName <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.typeOfIndividualName
      }
    }


  ),

  private = list(
    .prefix = NULL,
    .firstGiven = NULL,
    .middle = NULL,
    .lastFamily = NULL,
    .suffix = NULL,
    .fullName = NULL,
    .effectiveDates = NULL,
    .abbreviation = NULL,
    .typeOfIndividualName = NULL,
    .sex = NULL,
    .isPreferred = NULL,
    .context = NULL,
    .isFormal = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        abbreviation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        context = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        effectiveDates = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        firstGiven = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        fullName = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        isFormal = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPreferred = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        lastFamily = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        middle = c(
          min="0", 
          max="Inf", 
          datatype = "character", 
          structure = "vector"
        ),
        prefix = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        sex = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SexSpecificationType", 
          structure = "Enumeration vector"
        ),
        suffix = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        typeOfIndividualName = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_IndividualName


#' @title DDI4_InstanceMeasurement  An instance measurement instantiates a represented measurement, so that it can be used as an Act in the Process Steps that define a data capture process
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/InstanceMeasurement}{InstanceMeasurement} 

#'
#'   Definition
#'
#'   ============
#'
#'   An instance measurement instantiates a represented measurement, so that it can be used as an Act in the Process Steps that define a data capture process.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_InstrumentComponent}}}
#'    \item{\code{\link{DDI4_Act}}}
#'    \item{\code{\link{DDI4_WorkflowStep}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{instantiates}}{ A read/write active binding.
#'          The measurement being instantiated in the data collection process
#'
#'                              \code{myDDI4_InstanceMeasurement$instantiates <- c(myRepresentedMeasurementURN)} sets the association 'instantiates' from 'myDDI4_InstanceMeasurement' to a character vector of DdiUrns for R6 RepresentedMeasurement class instances
#'
#'                              \code{myDDI4_InstanceMeasurement$instantiates} returns a character vector of of DdiUrns for R6  RepresentedMeasurement class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_InstanceMeasurement$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_InstanceMeasurement <- R6Class(classname="DDI4_InstanceMeasurement",
  inherit = DDI4_InstrumentComponent,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, instantiates=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_InstanceMeasurement may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)



       if(!missing(instantiates)){
        private$.instantiates <- returnUrnVectorIfValid(validClass="DDI4_RepresentedMeasurement", proposedUrnVector=instantiates , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.instantiates, referencingAssociationName="instantiates",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 class association(s):
    instantiates = function(val){
      if (!missing(val)){
        private$.instantiates <- returnUrnVectorIfValid(validClass="DDI4_RepresentedMeasurement", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.instantiates, referencingAssociationName="instantiates",  referencingUrn=thisUrn)  
       # invisible( private$.instantiates)
      } else {
          private$.instantiates
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .instantiates = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasExternalAid = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalAid", 
          structure = "DdiUrn vector"
        ),
        hasInformationFlow = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInstruction = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Instruction", 
          structure = "DdiUrn vector"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        instantiates = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RepresentedMeasurement", 
          structure = "DdiUrn vector"
        ),
        isPerformedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowService", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_InstanceMeasurement


#' @title DDI4_InstanceQuestion  An instance question is an instantiation of a represented question,to be used as an Act in the process steps that define a survey questionnaire
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/InstanceQuestion}{InstanceQuestion} 

#'
#'   Definition
#'
#'   ============
#'
#'   An instance question is an instantiation of a represented question,to be used as an Act in the process steps that define a survey questionnaire.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   “How old are you?”
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_InstrumentComponent}}}
#'    \item{\code{\link{DDI4_Act}}}
#'    \item{\code{\link{DDI4_WorkflowStep}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_InstanceQuestion$DdiUrn} returns the DDI URN of myDDI4_InstanceQuestion.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        The name of a question as used in an Instrument. Redefined to provide a more useful description. A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage.
#'
#'                              \code{myDDI4_InstanceQuestion$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_InstanceQuestion' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_InstanceQuestion$name} returns a list of of R6 : ObjectName class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{instantiates}}{ A read/write active binding.
#'          The question being used.
#'
#'                              \code{myDDI4_InstanceQuestion$instantiates <- c(myRepresentedQuestionURN)} sets the association 'instantiates' from 'myDDI4_InstanceQuestion' to a character vector of DdiUrns for R6 RepresentedQuestion class instances
#'
#'                              \code{myDDI4_InstanceQuestion$instantiates} returns a character vector of of DdiUrns for R6  RepresentedQuestion class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_InstanceQuestion$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, name=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_InstanceQuestion <- R6Class(classname="DDI4_InstanceQuestion",
  inherit = DDI4_InstrumentComponent,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, name=NA,    instantiates=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_InstanceQuestion may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(instantiates)){
        private$.instantiates <- returnUrnVectorIfValid(validClass="DDI4_RepresentedQuestion", proposedUrnVector=instantiates , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.instantiates, referencingAssociationName="instantiates",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },


# There are 1 class association(s):
    instantiates = function(val){
      if (!missing(val)){
        private$.instantiates <- returnUrnVectorIfValid(validClass="DDI4_RepresentedQuestion", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.instantiates, referencingAssociationName="instantiates",  referencingUrn=thisUrn)  
       # invisible( private$.instantiates)
      } else {
          private$.instantiates
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .name = NULL,
    .instantiates = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasExternalAid = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalAid", 
          structure = "DdiUrn vector"
        ),
        hasInformationFlow = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInstruction = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Instruction", 
          structure = "DdiUrn vector"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        instantiates = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RepresentedQuestion", 
          structure = "DdiUrn vector"
        ),
        isPerformedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowService", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_InstanceQuestion


#' @title DDI4_InstanceVariable  The use of a Represented Variable within a Data Set
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/InstanceVariable}{InstanceVariable} 

#'
#'   Definition
#'
#'   ============
#'
#'   The use of a Represented Variable within a Data Set.  The InstanceVariable describes actual instances of data that have been collected. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   1) Gender: Dan Gillman has gender <m, male>, Arofan Gregory has gender<m, male>, etc. 
#'
#'   2) Number of employees: Microsoft has 90,000 employees; IBM has 433,000 employees, etc. 
#'
#'   3) Endowment: Johns Hopkins has endowment of <3, $1,000,000 and above>, 
#'
#'   Yale has endowment of <3, $1,000,000 and above>, etc.
#'
#'   
#'
#'   Two InstanceVariables of a person's height reference the same RepresentedVariable. This indicates that they are intended to: be measured with the same unit of measurement, have the same intended data type, have the same SubstantativeValueDomain, use a SentinelValueDomain drawn from the same set of SentinelValueDomains, have the same sentinel (missing value) concepts, and draw their Population from the same Universe. In other words, the two InstanceVariables should be comparable.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   The InstanceVariable class inherits all of the properties and relationships of the RepresentedVariable (RV) class and, in turn, the ConceptualVariable (CV) class. This means that an InstanceVariable can be completely populated without the need to create an associated RepresentedVariable or ConceptualVariable. If, however, a user wishes to indicate that a particular InstanceVariable is patterned after a particular RepresentedVariable or a particular ConceptualVariable that may be indicated by including a relationship to the RV and or CV. Including these references is an important method of indicating that multiple InstanceVariables have the same representation, measure the same concept, and are drawn from the same Universe. If two InstanceVariables of a person's height reference the same RepresentedVariable. This indicates that they are intended to: be measured with the same unit of measurement, have the same intended data type, have the same SubstantativeValueDomain, use a SentinelValueDomain drawn from the same set of SentinelValueDomains, have the same sentinel (missing value) concepts, and draw their Population from the same Universe. In other words, the two InstanceVariables should be comparable. 
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Instance Variable
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_RepresentedVariable}}}
#'    \item{\code{\link{DDI4_ConceptualVariable}}}
#'    \item{\code{\link{DDI4_Concept}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_InstanceVariable$DdiUrn} returns the DDI URN of myDDI4_InstanceVariable.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{variableFunction}}{ A read/write active binding.
#'        Immutable characteristics of the variable such as, geographic designator, weight, temporal designation, etc.
#'
#'                              \code{myDDI4_InstanceVariable$variableFunction <- list(myExternalControlledVocabularyEntry)} sets the 'variableFunction' property of 'myDDI4_InstanceVariable' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_InstanceVariable$variableFunction} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{physicalDataType}}{ A read/write active binding.
#'        The data type of this variable. Supports the optional use of an external controlled vocabulary.
#'
#'                              \code{myDDI4_InstanceVariable$physicalDataType <- list(myExternalControlledVocabularyEntry)} sets the 'physicalDataType' property of 'myDDI4_InstanceVariable' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_InstanceVariable$physicalDataType} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{platformType}}{ A read/write active binding.
#'        This should match a platformType of the RepresentedVariable on which the InstanceVariable is based (if one exists). It is drawn from the same external controlled vocabulary used by the RepreentedVariable's platformType.  A sample list for the external controlled vocabulary is:  A sample list would be:  • BlankString - A Blank string indicates missing. Comparison is based on lexical order.  • EmptyString - An empty string indicates missing. Use in comparisons returns missing.   • Rstyle - Codes drawn from  NA and the IEEE 754 values of  NaN  -Inf   +Inf.   Comparisons return NA.   • SASNumeric - codes drawn from . ._ .A .B .C .D .E .F .G .H .I .J .K .L .M .N .O .P .Q .R .S .T .U .V .W .X .Y .Z    Sentinel code treated as less than any substantive value   • SPSSstyle - System missing (a dot) a set of individual values drawn from the same datatype as the SubstantiveValueDomain, and a range of values  drawn from the same datatype as the SubstantiveValueDomain.  Comparisons return system missing.  Some functions substitute with valid values (e.g. SUM replaces missing values with 0)   • StataNumeric - codes drawn from . ._ .A .B .C .D .E .F .G .H .I .J .K .L .M .N .O .P .Q .R .S .T .U .V .W .X .Y .Z  Sentinel code treated as greater than any substantive value  • Unrestricted - No restrictions on codes for sentinel values. Use in comparisons is indeterminate.  
#'
#'                              \code{myDDI4_InstanceVariable$platformType <- list(myExternalControlledVocabularyEntry)} sets the 'platformType' property of 'myDDI4_InstanceVariable' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_InstanceVariable$platformType} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{measures}}{ A read/write active binding.
#'          Set of specific units (people, entities, objects, events), usually in a given time and geography, being measured. Can be a specialization of the Universe measured by a related RepresentedVariable.
#'
#'                              \code{myDDI4_InstanceVariable$measures <- c(myPopulationURN)} sets the association 'measures' from 'myDDI4_InstanceVariable' to a character vector of DdiUrns for R6 Population class instances
#'
#'                              \code{myDDI4_InstanceVariable$measures} returns a character vector of of DdiUrns for R6  Population class instances}
#'   }
#'   \describe{
#'     \item{\code{basedOnRepresentedVariable}}{ A read/write active binding.
#'          The RepresentedVariable that can be shared by multiple InstanceVariables. Indicates comparability in substantive representation, Universe, and indirectly ConceptualDomain. If a relationship to a RepresentedVariable is defined there should be no relationship to a ConceptualVariable.
#'
#'                              \code{myDDI4_InstanceVariable$basedOnRepresentedVariable <- c(myRepresentedVariableURN)} sets the association 'basedOnRepresentedVariable' from 'myDDI4_InstanceVariable' to a character vector of DdiUrns for R6 RepresentedVariable class instances
#'
#'                              \code{myDDI4_InstanceVariable$basedOnRepresentedVariable} returns a character vector of of DdiUrns for R6  RepresentedVariable class instances}
#'   }
#'   \describe{
#'     \item{\code{basedOnConceptualVariable}}{ A read/write active binding.
#'          The ConceptualVariable that can be shared by multiple InstanceVariables. Indicates comparability ConceptualDomain and UnitType. If a relationship to a RepresentedVariable is defined there should be no relationship to a ConceptualVariable.
#'
#'                              \code{myDDI4_InstanceVariable$basedOnConceptualVariable <- c(myConceptualVariableURN)} sets the association 'basedOnConceptualVariable' from 'myDDI4_InstanceVariable' to a character vector of DdiUrns for R6 ConceptualVariable class instances
#'
#'                              \code{myDDI4_InstanceVariable$basedOnConceptualVariable} returns a character vector of of DdiUrns for R6  ConceptualVariable class instances}
#'   }
#'   \describe{
#'     \item{\code{takesPlatformSpecificSentinelValues}}{ A read/write active binding.
#'          The association to the possible sentinel (missing) values.These will be specific to one software platform.
#'
#'                              \code{myDDI4_InstanceVariable$takesPlatformSpecificSentinelValues <- c(mySentinelValueDomainURN)} sets the association 'takesPlatformSpecificSentinelValues' from 'myDDI4_InstanceVariable' to a character vector of DdiUrns for R6 SentinelValueDomain class instances
#'
#'                              \code{myDDI4_InstanceVariable$takesPlatformSpecificSentinelValues} returns a character vector of of DdiUrns for R6  SentinelValueDomain class instances}
#'   }
#'   \describe{
#'     \item{\code{generatedBy}}{ A read/write active binding.
#'          If generated directly through fielding an instrument use the instrument component containing the capture for the values that populate this InstanceVariable. A Capture is reusable and therefore it is important to know the specific use of the Capture within an Instrument. This may be any class using InstrumentComponent as an extension base, currently InstanceQuestion and InstanceMeasurement. If the content of the InstanceVariable was generated by a computation or other form of transformation with data processing, use the ComputationAction or similarly appropriate subclass of Act. If a sequence of Acts resulted in the InstanceVariable content indicate the final Act.
#'
#'                              \code{myDDI4_InstanceVariable$generatedBy <- c(myActURN)} sets the association 'generatedBy' from 'myDDI4_InstanceVariable' to a character vector of DdiUrns for R6 Act class instances
#'
#'                              \code{myDDI4_InstanceVariable$generatedBy} returns a character vector of of DdiUrns for R6  Act class instances}
#'   }
#'   \describe{
#'     \item{\code{sourceCapture}}{ A read/write active binding.
#'          The source of capture for the values that populate this InstanceVariable. This may be any class using Capture as an extension base, currently RepresentedQuestion and RepresentedMeasurement. This is the direct source of capture for the content of the data. If a transformation is required the source is not multiple captures but the transformation.
#'
#'                              \code{myDDI4_InstanceVariable$sourceCapture <- c(myCaptureURN)} sets the association 'sourceCapture' from 'myDDI4_InstanceVariable' to a character vector of DdiUrns for R6 Capture class instances
#'
#'                              \code{myDDI4_InstanceVariable$sourceCapture} returns a character vector of of DdiUrns for R6  Capture class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_InstanceVariable$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, variableFunction=NA, physicalDataType=NA, platformType=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_InstanceVariable <- R6Class(classname="DDI4_InstanceVariable",
  inherit = DDI4_RepresentedVariable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, variableFunction=NA, physicalDataType=NA, platformType=NA,    measures=NA,basedOnRepresentedVariable=NA,basedOnConceptualVariable=NA,takesPlatformSpecificSentinelValues=NA,generatedBy=NA,sourceCapture=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_InstanceVariable may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(variableFunction)){
        private$.variableFunction <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=variableFunction , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(physicalDataType)){
        private$.physicalDataType <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=physicalDataType , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(platformType)){
        private$.platformType <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=platformType , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(measures)){
        private$.measures <- returnUrnVectorIfValid(validClass="DDI4_Population", proposedUrnVector=measures , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.measures, referencingAssociationName="measures",  referencingUrn=thisUrn)  
      }
      if(!missing(basedOnRepresentedVariable)){
        private$.basedOnRepresentedVariable <- returnUrnVectorIfValid(validClass="DDI4_RepresentedVariable", proposedUrnVector=basedOnRepresentedVariable , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.basedOnRepresentedVariable, referencingAssociationName="basedOnRepresentedVariable",  referencingUrn=thisUrn)  
      }
      if(!missing(basedOnConceptualVariable)){
        private$.basedOnConceptualVariable <- returnUrnVectorIfValid(validClass="DDI4_ConceptualVariable", proposedUrnVector=basedOnConceptualVariable , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.basedOnConceptualVariable, referencingAssociationName="basedOnConceptualVariable",  referencingUrn=thisUrn)  
      }
      if(!missing(takesPlatformSpecificSentinelValues)){
        private$.takesPlatformSpecificSentinelValues <- returnUrnVectorIfValid(validClass="DDI4_SentinelValueDomain", proposedUrnVector=takesPlatformSpecificSentinelValues , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.takesPlatformSpecificSentinelValues, referencingAssociationName="takesPlatformSpecificSentinelValues",  referencingUrn=thisUrn)  
      }
      if(!missing(generatedBy)){
        private$.generatedBy <- returnUrnVectorIfValid(validClass="DDI4_Act", proposedUrnVector=generatedBy , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.generatedBy, referencingAssociationName="generatedBy",  referencingUrn=thisUrn)  
      }
      if(!missing(sourceCapture)){
        private$.sourceCapture <- returnUrnVectorIfValid(validClass="DDI4_Capture", proposedUrnVector=sourceCapture , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.sourceCapture, referencingAssociationName="sourceCapture",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    physicalDataType = function(val){
      if (!missing(val)){
        private$.physicalDataType <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.physicalDataType
      }
    },

    platformType = function(val){
      if (!missing(val)){
        private$.platformType <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.platformType
      }
    },

    variableFunction = function(val){
      if (!missing(val)){
        private$.variableFunction <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.variableFunction
      }
    },


# There are 6 class association(s):
    basedOnConceptualVariable = function(val){
      if (!missing(val)){
        private$.basedOnConceptualVariable <- returnUrnVectorIfValid(validClass="DDI4_ConceptualVariable", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.basedOnConceptualVariable, referencingAssociationName="basedOnConceptualVariable",  referencingUrn=thisUrn)  
       # invisible( private$.basedOnConceptualVariable)
      } else {
          private$.basedOnConceptualVariable
      }
    },

    basedOnRepresentedVariable = function(val){
      if (!missing(val)){
        private$.basedOnRepresentedVariable <- returnUrnVectorIfValid(validClass="DDI4_RepresentedVariable", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.basedOnRepresentedVariable, referencingAssociationName="basedOnRepresentedVariable",  referencingUrn=thisUrn)  
       # invisible( private$.basedOnRepresentedVariable)
      } else {
          private$.basedOnRepresentedVariable
      }
    },

    generatedBy = function(val){
      if (!missing(val)){
        private$.generatedBy <- returnUrnVectorIfValid(validClass="DDI4_Act", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.generatedBy, referencingAssociationName="generatedBy",  referencingUrn=thisUrn)  
       # invisible( private$.generatedBy)
      } else {
          private$.generatedBy
      }
    },

    measures = function(val){
      if (!missing(val)){
        private$.measures <- returnUrnVectorIfValid(validClass="DDI4_Population", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.measures, referencingAssociationName="measures",  referencingUrn=thisUrn)  
       # invisible( private$.measures)
      } else {
          private$.measures
      }
    },

    sourceCapture = function(val){
      if (!missing(val)){
        private$.sourceCapture <- returnUrnVectorIfValid(validClass="DDI4_Capture", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.sourceCapture, referencingAssociationName="sourceCapture",  referencingUrn=thisUrn)  
       # invisible( private$.sourceCapture)
      } else {
          private$.sourceCapture
      }
    },

    takesPlatformSpecificSentinelValues = function(val){
      if (!missing(val)){
        private$.takesPlatformSpecificSentinelValues <- returnUrnVectorIfValid(validClass="DDI4_SentinelValueDomain", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.takesPlatformSpecificSentinelValues, referencingAssociationName="takesPlatformSpecificSentinelValues",  referencingUrn=thisUrn)  
       # invisible( private$.takesPlatformSpecificSentinelValues)
      } else {
          private$.takesPlatformSpecificSentinelValues
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .variableFunction = NULL,
    .physicalDataType = NULL,
    .platformType = NULL,
    .measures = NULL,
    .basedOnRepresentedVariable = NULL,
    .basedOnConceptualVariable = NULL,
    .takesPlatformSpecificSentinelValues = NULL,
    .generatedBy = NULL,
    .sourceCapture = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnConceptualVariable = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ConceptualVariable", 
          structure = "DdiUrn vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        basedOnRepresentedVariable = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RepresentedVariable", 
          structure = "DdiUrn vector"
        ),
        definition = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        descriptiveText = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        drawsFrom = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Universe", 
          structure = "DdiUrn vector"
        ),
        generatedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Act", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasIntendedDataType = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        measures = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Population", 
          structure = "DdiUrn vector"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        physicalDataType = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        platformType = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        sourceCapture = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Capture", 
          structure = "DdiUrn vector"
        ),
        takesPlatformSpecificSentinelValues = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SentinelValueDomain", 
          structure = "DdiUrn vector"
        ),
        takesSentinelConceptsFrom = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SentinelConceptualDomain", 
          structure = "DdiUrn vector"
        ),
        takesSubstantiveConceptsFrom = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SubstantiveConceptualDomain", 
          structure = "DdiUrn vector"
        ),
        takesSubstantiveValuesFrom = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SubstantiveValueDomain", 
          structure = "DdiUrn vector"
        ),
        unitOfMeasurement = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        usesConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        usesUnitType = c(
          min="0", 
          max="1", 
          datatype = "DDI4_UnitType", 
          structure = "DdiUrn vector"
        ),
        variableFunction = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_InstanceVariable


#' @title DDI4_InstanceVariableIndicator  Allows for the identification of the InstanceVariable specifically as a member and optionally provides an index for the member within an ordered array
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/InstanceVariableIndicator}{InstanceVariableIndicator} 

#'
#'   Definition
#'
#'   ============
#'
#'   Allows for the identification of the InstanceVariable specifically as a member and optionally provides an index for the member within an ordered array. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Note if multiple types of Variables may be included in a collection use VariableIndicator.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_InstanceVariableIndicator$DdiUrn} returns the DDI URN of myDDI4_InstanceVariableIndicator.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Provides an index for the member within an ordered array
#'
#'                              \code{myDDI4_InstanceVariableIndicator$index <- myinteger} sets the index property of 'myDDI4_InstanceVariableIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_InstanceVariableIndicator$index} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{member}}{ A read/write active binding.
#'          Member of the collection
#'
#'                              \code{myDDI4_InstanceVariableIndicator$member <- c(myInstanceVariableURN)} sets the association 'member' from 'myDDI4_InstanceVariableIndicator' to a character vector of DdiUrns for R6 InstanceVariable class instances
#'
#'                              \code{myDDI4_InstanceVariableIndicator$member} returns a character vector of of DdiUrns for R6  InstanceVariable class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_InstanceVariableIndicator$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_InstanceVariableIndicator <- R6Class(classname="DDI4_InstanceVariableIndicator",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA,    member=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_InstanceVariableIndicator may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(member)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=member , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.index
      }
    },


# There are 1 class association(s):
    member = function(val){
      if (!missing(val)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
       # invisible( private$.member)
      } else {
          private$.member
      }
    }

  ),

  private = list(
    .index = NULL,
    .member = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        index = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        member = c(
          min="1", 
          max="1", 
          datatype = "DDI4_InstanceVariable", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_InstanceVariableIndicator


#' @title DDI4_InstanceVariableRelation  Defines the relationship between 2 or more InstanceVariables
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/InstanceVariableRelation}{InstanceVariableRelation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Defines the relationship between 2 or more InstanceVariables.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Use when relationships are limited to InstanceVariables only. Use VariableRelation when mixing relationships between various levels in the Variable Cascade. Use ConceptRelation when any subtype of Concept may be part of the relationship.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_InstanceVariableRelation$DdiUrn} returns the DDI URN of myDDI4_InstanceVariableRelation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_InstanceVariableRelation$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_InstanceVariableRelation' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_InstanceVariableRelation$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an ennumeration list
#'
#'                              \code{myDDI4_InstanceVariableRelation$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_InstanceVariableRelation' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_InstanceVariableRelation$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_InstanceVariableRelation$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_InstanceVariableRelation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_InstanceVariableRelation$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          First member in a relationship. 
#'
#'                              \code{myDDI4_InstanceVariableRelation$source <- c(myInstanceVariableURN)} sets the association 'source' from 'myDDI4_InstanceVariableRelation' to a character vector of DdiUrns for R6 InstanceVariable class instances
#'
#'                              \code{myDDI4_InstanceVariableRelation$source} returns a character vector of of DdiUrns for R6  InstanceVariable class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          Second member in a relationship. Note that this can be realized as a collection to support tuple
#'
#'                              \code{myDDI4_InstanceVariableRelation$target <- c(myInstanceVariableURN)} sets the association 'target' from 'myDDI4_InstanceVariableRelation' to a character vector of DdiUrns for R6 InstanceVariable class instances
#'
#'                              \code{myDDI4_InstanceVariableRelation$target} returns a character vector of of DdiUrns for R6  InstanceVariable class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_InstanceVariableRelation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, totality=NA, hasRelationSpecification=NA, semantic=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_InstanceVariableRelation <- R6Class(classname="DDI4_InstanceVariableRelation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, totality=NA, hasRelationSpecification=NA, semantic=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_InstanceVariableRelation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=source , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=target , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .totality = NULL,
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="1", 
          max="1", 
          datatype = "DDI4_InstanceVariable", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InstanceVariable", 
          structure = "DdiUrn vector"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_InstanceVariableRelation


#' @title DDI4_InstanceVariableRelationStructure  A realization of RelationStructure that is used to describe the order of InstanceVariables in a record for those cases where the variables have a more complicated logical order than a simple sequence
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/InstanceVariableRelationStructure}{InstanceVariableRelationStructure} 

#'
#'   Definition
#'
#'   ============
#'
#'   A realization of RelationStructure that is used to describe the order of InstanceVariables in a record for those cases where the variables have a more complicated logical order than a simple sequence.
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   For a simple sequence the order can be defined by the index values of the LogicalRecord's InstanceVariableIndicators that a UnitDataRecord inherits. Alternatively, the InstanceVariableRelationStructure can be used by a UnitDataRecord to describe a StructuredCollection as needed. 
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   An information model, a struct in C, a nested object in JSON
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_InstanceVariableRelationStructure$DdiUrn} returns the DDI URN of myDDI4_InstanceVariableRelationStructure.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{criteria}}{ A read/write active binding.
#'        Intensional definition of the order criteria (e.g. alphabetical, numerical, increasing, decreasing, etc.)
#'
#'                              \code{myDDI4_InstanceVariableRelationStructure$criteria <- list(myInternationalStructuredString)} sets the 'criteria' property of 'myDDI4_InstanceVariableRelationStructure' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_InstanceVariableRelationStructure$criteria} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A display label for the OrderRelation. May be expressed in multiple languages. Repeat for labels with different content, for example, labels with differing length limitations.
#'
#'                              \code{myDDI4_InstanceVariableRelationStructure$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_InstanceVariableRelationStructure' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_InstanceVariableRelationStructure$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#'   \describe{
#'     \item{\code{usage}}{ A read/write active binding.
#'        A display label for the OrderRelation. May be expressed in multiple languages. Repeat for labels with different content, for example, labels with differing length limitations.
#'
#'                              \code{myDDI4_InstanceVariableRelationStructure$usage <- list(myInternationalStructuredString)} sets the 'usage' property of 'myDDI4_InstanceVariableRelationStructure' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_InstanceVariableRelationStructure$usage} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Controlled vocabulary for the order relation semantics. It should contain, at least, the following: Self_Or_Descendant_Of, Part_Of, Less_Than_Or_Equal_To, Subtype_Of, Subclass_Of.
#'
#'                              \code{myDDI4_InstanceVariableRelationStructure$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_InstanceVariableRelationStructure' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_InstanceVariableRelationStructure$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Controlled Vocabulary to specify whether the relation is total, partial or unknown.
#'
#'                              \code{myDDI4_InstanceVariableRelationStructure$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_InstanceVariableRelationStructure' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_InstanceVariableRelationStructure$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an enumeration list.  Use if all relations within this relation structure are of the same specification.
#'
#'                              \code{myDDI4_InstanceVariableRelationStructure$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_InstanceVariableRelationStructure' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_InstanceVariableRelationStructure$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMemberRelation}}{ A read/write active binding.
#'        Member relations of InstanceVariables that comprise the relation structure
#'
#'                              \code{myDDI4_InstanceVariableRelationStructure$hasMemberRelation <- list(myInstanceVariableRelation)} sets the 'hasMemberRelation' property of 'myDDI4_InstanceVariableRelationStructure' to a list of R6 InstanceVariableRelation class instances
#'
#'                              \code{myDDI4_InstanceVariableRelationStructure$hasMemberRelation} returns a list of of R6 : InstanceVariableRelation class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_InstanceVariableRelationStructure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, criteria=NA, displayLabel=NA, usage=NA, semantic=NA, totality=NA, hasRelationSpecification=NA, hasMemberRelation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_InstanceVariableRelationStructure <- R6Class(classname="DDI4_InstanceVariableRelationStructure",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, criteria=NA, displayLabel=NA, usage=NA, semantic=NA, totality=NA, hasRelationSpecification=NA, hasMemberRelation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_InstanceVariableRelationStructure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(criteria)){
        private$.criteria <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=criteria , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(usage)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=usage , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(hasMemberRelation)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_InstanceVariableRelation", proposedObjectList=hasMemberRelation , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 7 properties: 
    criteria = function(val){
      if (!missing(val)){
        private$.criteria <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.criteria
      }
    },

    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },

    hasMemberRelation = function(val){
      if (!missing(val)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_InstanceVariableRelation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasMemberRelation
      }
    },

    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },

    usage = function(val){
      if (!missing(val)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.usage
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .criteria = NULL,
    .displayLabel = NULL,
    .usage = NULL,
    .semantic = NULL,
    .totality = NULL,
    .hasRelationSpecification = NULL,
    .hasMemberRelation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        criteria = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasMemberRelation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InstanceVariableRelation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        totality = c(
          min="1", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        ),
        usage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_InstanceVariableRelationStructure


#' @title DDI4_InstanceVariableValueMap  A key value relationship for two or more logical records where the key is one or more equivalent instance variables and the value is a defined relationship or a relationship to a set value
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/InstanceVariableValueMap}{InstanceVariableValueMap} 

#'
#'   Definition
#'
#'   ============
#'
#'   A key value relationship for two or more logical records where the key is one or more equivalent instance variables and the value is a defined relationship or a relationship to a set value. Correspondence type refers to the variables themselves rather than the value of the variables concerned. In this context Correspondence type will normally be set to ExactMatch.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Logical Records: Household and Person, Key: Household ID (HHID in Household Record, HHIDP in Person Record), ValueRelaitonship: Equal, Set Value: n.a.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_InstanceVariableValueMap$DdiUrn} returns the DDI URN of myDDI4_InstanceVariableValueMap.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{valueRelationship}}{ A read/write active binding.
#'        Relationship between the source and target InstanceVariables or to the setValue if provided
#'
#'                              \code{myDDI4_InstanceVariableValueMap$valueRelationship <- list(myValueRelationshipType)} sets the 'valueRelationship' property of 'myDDI4_InstanceVariableValueMap' to a list of R6 ValueRelationshipType class instances
#'
#'                              \code{myDDI4_InstanceVariableValueMap$valueRelationship} returns a list of of R6 : ValueRelationshipType class instances}
#'   }
#'   \describe{
#'     \item{\code{setValue}}{ A read/write active binding.
#'        A set value for the key source Instance Variables
#'
#'                              \code{myDDI4_InstanceVariableValueMap$setValue <- list(myValueString)} sets the 'setValue' property of 'myDDI4_InstanceVariableValueMap' to a list of R6 ValueString class instances
#'
#'                              \code{myDDI4_InstanceVariableValueMap$setValue} returns a list of of R6 : ValueString class instances}
#'   }
#'   \describe{
#'     \item{\code{hasCorrespondenceType}}{ A read/write active binding.
#'        Describes the relationship between the source and target members using both controlled vocabularies and descriptive text. In this context the correspondence refers to the two instance variables, not their value. The relationship would normally be exactMatch.
#'
#'                              \code{myDDI4_InstanceVariableValueMap$hasCorrespondenceType <- list(myCorrespondenceType)} sets the 'hasCorrespondenceType' property of 'myDDI4_InstanceVariableValueMap' to a list of R6 CorrespondenceType class instances
#'
#'                              \code{myDDI4_InstanceVariableValueMap$hasCorrespondenceType} returns a list of of R6 : CorrespondenceType class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          The source Intance Variable for the relationship.  
#'
#'                              \code{myDDI4_InstanceVariableValueMap$source <- c(myInstanceVariableURN)} sets the association 'source' from 'myDDI4_InstanceVariableValueMap' to a character vector of DdiUrns for R6 InstanceVariable class instances
#'
#'                              \code{myDDI4_InstanceVariableValueMap$source} returns a character vector of of DdiUrns for R6  InstanceVariable class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          Target Instance Variables if a directional relation is used
#'
#'                              \code{myDDI4_InstanceVariableValueMap$target <- c(myInstanceVariableURN)} sets the association 'target' from 'myDDI4_InstanceVariableValueMap' to a character vector of DdiUrns for R6 InstanceVariable class instances
#'
#'                              \code{myDDI4_InstanceVariableValueMap$target} returns a character vector of of DdiUrns for R6  InstanceVariable class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_InstanceVariableValueMap$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, valueRelationship=NA, setValue=NA, hasCorrespondenceType=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_InstanceVariableValueMap <- R6Class(classname="DDI4_InstanceVariableValueMap",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, valueRelationship=NA, setValue=NA, hasCorrespondenceType=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_InstanceVariableValueMap may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(valueRelationship)){
        private$.valueRelationship <- returnEnumeratedValues(enumerationObject=DDI4_ValueRelationshipType$new(registerThisObject=FALSE), proposedCharacterVector=valueRelationship , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(setValue)){
        private$.setValue <- returnR6ObjectIfValid(validClass="DDI4_ValueString", proposedObjectList=setValue , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasCorrespondenceType)){
        private$.hasCorrespondenceType <- returnR6ObjectIfValid(validClass="DDI4_CorrespondenceType", proposedObjectList=hasCorrespondenceType , minimumCardinality=1, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=source , minimumCardinality=1, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=target , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    hasCorrespondenceType = function(val){
      if (!missing(val)){
        private$.hasCorrespondenceType <- returnR6ObjectIfValid(validClass="DDI4_CorrespondenceType", proposedObjectList=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasCorrespondenceType
      }
    },

    setValue = function(val){
      if (!missing(val)){
        private$.setValue <- returnR6ObjectIfValid(validClass="DDI4_ValueString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.setValue
      }
    },

    valueRelationship = function(val){
      if (!missing(val)){
        private$.valueRelationship <- returnEnumeratedValues(enumerationObject=DDI4_ValueRelationshipType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.valueRelationship
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .valueRelationship = NULL,
    .setValue = NULL,
    .hasCorrespondenceType = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasCorrespondenceType = c(
          min="1", 
          max="1", 
          datatype = "DDI4_CorrespondenceType", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        setValue = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ValueString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="1", 
          max="Inf", 
          datatype = "DDI4_InstanceVariable", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InstanceVariable", 
          structure = "DdiUrn vector"
        ),
        valueRelationship = c(
          min="1", 
          max="1", 
          datatype = "DDI4_ValueRelationshipType", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_InstanceVariableValueMap


#' @title DDI4_Instruction  Provides the content and description of data capture instructions
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Instruction}{Instruction} 

#'
#'   Definition
#'
#'   ============
#'
#'   Provides the content and description of data capture instructions. Contains the "how to" information for administering an instrument.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Completion instructions in self-administered mail questionnaire, information for administering a blood pressure measurement, interviewer instructions for a CATI questionnaire, guidance for communicating between an interviewer and a respondent (note MIDUS cognitive assessment example).
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   d:InstructionType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Instruction$DdiUrn} returns the DDI URN of myDDI4_Instruction.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{instructionText}}{ A read/write active binding.
#'        The content of the Instruction text provided using DynamicText. Note that when using Dynamic Text, the full InstructionText must be repeated for multi-language versions of the content. The InstructionText may also be repeated to provide a dynamic and plain text version of the instruction. This allows for accurate rendering of the instruction in a non-dynamic environment like print.
#'
#'                              \code{myDDI4_Instruction$instructionText <- list(myDynamicText)} sets the 'instructionText' property of 'myDDI4_Instruction' to a list of R6 DynamicText class instances
#'
#'                              \code{myDDI4_Instruction$instructionText} returns a list of of R6 : DynamicText class instances}
#'   }
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A structured display label providing a fully human readable display for the identification of the object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_Instruction$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_Instruction' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_Instruction$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage. 
#'
#'                              \code{myDDI4_Instruction$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_Instruction' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_Instruction$name} returns a list of of R6 : ObjectName class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{associatedMaterial}}{ A read/write active binding.
#'          An image or other external material associated with the Instruction, located at the provided URN or URL.
#'
#'                              \code{myDDI4_Instruction$associatedMaterial <- c(myExternalMaterialURN)} sets the association 'associatedMaterial' from 'myDDI4_Instruction' to a character vector of DdiUrns for R6 ExternalMaterial class instances
#'
#'                              \code{myDDI4_Instruction$associatedMaterial} returns a character vector of of DdiUrns for R6  ExternalMaterial class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Instruction$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, instructionText=NA, displayLabel=NA, name=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Instruction <- R6Class(classname="DDI4_Instruction",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, instructionText=NA, displayLabel=NA, name=NA,    associatedMaterial=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Instruction may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(instructionText)){
        private$.instructionText <- returnR6ObjectIfValid(validClass="DDI4_DynamicText", proposedObjectList=instructionText , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(associatedMaterial)){
        private$.associatedMaterial <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=associatedMaterial , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.associatedMaterial, referencingAssociationName="associatedMaterial",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },

    instructionText = function(val){
      if (!missing(val)){
        private$.instructionText <- returnR6ObjectIfValid(validClass="DDI4_DynamicText", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.instructionText
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },


# There are 1 class association(s):
    associatedMaterial = function(val){
      if (!missing(val)){
        private$.associatedMaterial <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.associatedMaterial, referencingAssociationName="associatedMaterial",  referencingUrn=thisUrn)  
       # invisible( private$.associatedMaterial)
      } else {
          private$.associatedMaterial
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .instructionText = NULL,
    .displayLabel = NULL,
    .name = NULL,
    .associatedMaterial = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        associatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        instructionText = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_DynamicText", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Instruction


#' @title DDI4_InstrumentCode  An InstrumentComponent that specifies the performance of a specific computation within the context of an instrument flow
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/InstrumentCode}{InstrumentCode} 

#'
#'   Definition
#'
#'   ============
#'
#'   An InstrumentComponent that specifies the performance of a specific computation within the context of an instrument flow.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   quality control, edit check, checksums, compute filler text, compute values for use in administering the instrument
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   d:ComputationItemType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_InstrumentComponent}}}
#'    \item{\code{\link{DDI4_Act}}}
#'    \item{\code{\link{DDI4_WorkflowStep}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_InstrumentCode$DdiUrn} returns the DDI URN of myDDI4_InstrumentCode.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{purposeOfCode}}{ A read/write active binding.
#'        The purpose of the code (e.g., quality control, edit check, checksums, compute filler text, compute values for use in administering the instrument)
#'
#'                              \code{myDDI4_InstrumentCode$purposeOfCode <- list(myExternalControlledVocabularyEntry)} sets the 'purposeOfCode' property of 'myDDI4_InstrumentCode' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_InstrumentCode$purposeOfCode} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{usesCommandCode}}{ A read/write active binding.
#'        Describes the code used to execute the command using the options of inline textual description, inline code, and/or an external file.
#'
#'                              \code{myDDI4_InstrumentCode$usesCommandCode <- list(myCommandCode)} sets the 'usesCommandCode' property of 'myDDI4_InstrumentCode' to a list of R6 CommandCode class instances
#'
#'                              \code{myDDI4_InstrumentCode$usesCommandCode} returns a list of of R6 : CommandCode class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_InstrumentCode$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, purposeOfCode=NA, usesCommandCode=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_InstrumentCode <- R6Class(classname="DDI4_InstrumentCode",
  inherit = DDI4_InstrumentComponent,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, purposeOfCode=NA, usesCommandCode=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_InstrumentCode may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(purposeOfCode)){
        private$.purposeOfCode <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=purposeOfCode , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(usesCommandCode)){
        private$.usesCommandCode <- returnR6ObjectIfValid(validClass="DDI4_CommandCode", proposedObjectList=usesCommandCode , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    purposeOfCode = function(val){
      if (!missing(val)){
        private$.purposeOfCode <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purposeOfCode
      }
    },

    usesCommandCode = function(val){
      if (!missing(val)){
        private$.usesCommandCode <- returnR6ObjectIfValid(validClass="DDI4_CommandCode", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.usesCommandCode
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .purposeOfCode = NULL,
    .usesCommandCode = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasExternalAid = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalAid", 
          structure = "DdiUrn vector"
        ),
        hasInformationFlow = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInstruction = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Instruction", 
          structure = "DdiUrn vector"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPerformedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowService", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purposeOfCode = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usesCommandCode = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CommandCode", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_InstrumentCode


#' @title DDI4_InstrumentComponent  InstrumentComponent is an abstract object which extends an Act (a type of Process Step)
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/InstrumentComponent}{InstrumentComponent} 

#'
#'   Definition
#'
#'   ============
#'
#'   InstrumentComponent is an abstract object which extends an Act (a type of Process Step). The purpose of InstrumentComponent is to provide a common parent for Capture (e.g., Question, Measure), Statement, and Instructions.
#'
#'   
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   InstrumentComponent acts as a substitution head (abstract) for semantically meaningful for objects that extend it and exist in an instrument.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Act}}}
#'    \item{\code{\link{DDI4_WorkflowStep}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasInstruction}}{ A read/write active binding.
#'          An instrument compoment can have zero to many instructions
#'
#'                              \code{myDDI4_InstrumentComponent$hasInstruction <- c(myInstructionURN)} sets the association 'hasInstruction' from 'myDDI4_InstrumentComponent' to a character vector of DdiUrns for R6 Instruction class instances
#'
#'                              \code{myDDI4_InstrumentComponent$hasInstruction} returns a character vector of of DdiUrns for R6  Instruction class instances}
#'   }
#'   \describe{
#'     \item{\code{hasExternalAid}}{ A read/write active binding.
#'          Any external object used to clarify, inform, or support the role of the instrument component
#'
#'                              \code{myDDI4_InstrumentComponent$hasExternalAid <- c(myExternalAidURN)} sets the association 'hasExternalAid' from 'myDDI4_InstrumentComponent' to a character vector of DdiUrns for R6 ExternalAid class instances
#'
#'                              \code{myDDI4_InstrumentComponent$hasExternalAid} returns a character vector of of DdiUrns for R6  ExternalAid class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_InstrumentComponent$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_InstrumentComponent <- R6Class(classname="DDI4_InstrumentComponent",
  inherit = DDI4_Act,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasInstruction=NA,hasExternalAid=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_InstrumentComponent may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)



       if(!missing(hasInstruction)){
        private$.hasInstruction <- returnUrnVectorIfValid(validClass="DDI4_Instruction", proposedUrnVector=hasInstruction , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasInstruction, referencingAssociationName="hasInstruction",  referencingUrn=thisUrn)  
      }
      if(!missing(hasExternalAid)){
        private$.hasExternalAid <- returnUrnVectorIfValid(validClass="DDI4_ExternalAid", proposedUrnVector=hasExternalAid , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasExternalAid, referencingAssociationName="hasExternalAid",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 2 class association(s):
    hasExternalAid = function(val){
      if (!missing(val)){
        private$.hasExternalAid <- returnUrnVectorIfValid(validClass="DDI4_ExternalAid", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasExternalAid, referencingAssociationName="hasExternalAid",  referencingUrn=thisUrn)  
       # invisible( private$.hasExternalAid)
      } else {
          private$.hasExternalAid
      }
    },

    hasInstruction = function(val){
      if (!missing(val)){
        private$.hasInstruction <- returnUrnVectorIfValid(validClass="DDI4_Instruction", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasInstruction, referencingAssociationName="hasInstruction",  referencingUrn=thisUrn)  
       # invisible( private$.hasInstruction)
      } else {
          private$.hasInstruction
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasInstruction = NULL,
    .hasExternalAid = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasExternalAid = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalAid", 
          structure = "DdiUrn vector"
        ),
        hasInformationFlow = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInstruction = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Instruction", 
          structure = "DdiUrn vector"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPerformedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowService", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_InstrumentComponent


#' @title DDI4_InternationalIdentifier  An identifier whose scope of uniqueness is broader than the local archive
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/InternationalIdentifier}{InternationalIdentifier} 

#'
#'   Definition
#'
#'   ============
#'
#'   An identifier whose scope of uniqueness is broader than the local archive. Common forms of an international identifier are ISBN, ISSN, DOI or similar designator. Provides both the value of the identifier and the agency who manages it.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   For use in Annotation or other citation format. 
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:InternationalIdentifierType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_InternationalIdentifier$DdiUrn} returns the DDI URN of myDDI4_InternationalIdentifier.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{identifierContent}}{ A read/write active binding.
#'        An identifier as it should be listed for identification purposes.
#'
#'                              \code{myDDI4_InternationalIdentifier$identifierContent <- mycharacter} sets the identifierContent property of 'myDDI4_InternationalIdentifier' to the vector of datatype:character
#'
#'                              \code{myDDI4_InternationalIdentifier$identifierContent} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{managingAgency}}{ A read/write active binding.
#'        The identification of the Agency which assigns and manages the identifier, i.e., ISBN, ISSN, DOI, etc.
#'
#'                              \code{myDDI4_InternationalIdentifier$managingAgency <- list(myExternalControlledVocabularyEntry)} sets the 'managingAgency' property of 'myDDI4_InternationalIdentifier' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_InternationalIdentifier$managingAgency} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{isURI}}{ A read/write active binding.
#'        Set to "true" if Identifier is a URI
#'
#'                              \code{myDDI4_InternationalIdentifier$isURI <- mylogical} sets the isURI property of 'myDDI4_InternationalIdentifier' to the vector of datatype:logical
#'
#'                              \code{myDDI4_InternationalIdentifier$isURI} returns a vector of datatype: logical}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_InternationalIdentifier$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, identifierContent=NA, managingAgency=NA, isURI=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_InternationalIdentifier <- R6Class(classname="DDI4_InternationalIdentifier",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, identifierContent=NA, managingAgency=NA, isURI=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_InternationalIdentifier may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(identifierContent)){
        private$.identifierContent <- returnVectorIfValid(validVectorClass="character", proposedVector=identifierContent , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(managingAgency)){
        private$.managingAgency <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=managingAgency , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isURI)){
        private$.isURI <- returnVectorIfValid(validVectorClass="logical", proposedVector=isURI , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    identifierContent = function(val){
      if (!missing(val)){
        private$.identifierContent <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.identifierContent
      }
    },

    isURI = function(val){
      if (!missing(val)){
        private$.isURI <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isURI
      }
    },

    managingAgency = function(val){
      if (!missing(val)){
        private$.managingAgency <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.managingAgency
      }
    }


  ),

  private = list(
    .identifierContent = NULL,
    .managingAgency = NULL,
    .isURI = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        identifierContent = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isURI = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        managingAgency = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_InternationalIdentifier


#' @title DDI4_InternationalString  Packaging structure for multiple language versions of the same string content
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/InternationalString}{InternationalString} 

#'
#'   Definition
#'
#'   ============
#'
#'   Packaging structure for multiple language versions of the same string content. Where an element of this type is repeatable, the expectation is that each repetition contains different content, each of which can be expressed in multiple languages. The language designation goes on the individual String.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:InternationalStringType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_InternationalString$DdiUrn} returns the DDI URN of myDDI4_InternationalString.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{languageSpecificString}}{ A read/write active binding.
#'        A non-formatted string of text with an attribute that designates the language of the text. Repeat this object to express the same content in another language.
#'
#'                              \code{myDDI4_InternationalString$languageSpecificString <- list(myLanguageSpecificStringType)} sets the 'languageSpecificString' property of 'myDDI4_InternationalString' to a list of R6 LanguageSpecificStringType class instances
#'
#'                              \code{myDDI4_InternationalString$languageSpecificString} returns a list of of R6 : LanguageSpecificStringType class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_InternationalString$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, languageSpecificString=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_InternationalString <- R6Class(classname="DDI4_InternationalString",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, languageSpecificString=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_InternationalString may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(languageSpecificString)){
        private$.languageSpecificString <- returnR6ObjectIfValid(validClass="DDI4_LanguageSpecificStringType", proposedObjectList=languageSpecificString , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 1 properties: 
    languageSpecificString = function(val){
      if (!missing(val)){
        private$.languageSpecificString <- returnR6ObjectIfValid(validClass="DDI4_LanguageSpecificStringType", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.languageSpecificString
      }
    }


  ),

  private = list(
    .languageSpecificString = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        languageSpecificString = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LanguageSpecificStringType", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_InternationalString


#' @title DDI4_InternationalStructuredString  Packaging structure for multiple language versions of the same string content, for objects that allow for internal formatting using XHTML tags
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/InternationalStructuredString}{InternationalStructuredString} 

#'
#'   Definition
#'
#'   ============
#'
#'   Packaging structure for multiple language versions of the same string content, for objects that allow for internal formatting using XHTML tags. Where an element of this type is repeatable, the expectation is that each repetition contains different content, each of which can be expressed in multiple languages.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:StructuredStringType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_InternationalStructuredString$DdiUrn} returns the DDI URN of myDDI4_InternationalStructuredString.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{languageSpecificStructuredString}}{ A read/write active binding.
#'        Supports the optional use of XHTML formatting tags within the string structure. In addition to the language designation and information regarding translation, the attribute isPlain can be set to true to indicate that the content should be treated as plain unstructured text, including any XHTML formatting tags. Repeat the content element to provide multiple language versions of the same content.
#'
#'                              \code{myDDI4_InternationalStructuredString$languageSpecificStructuredString <- list(myLanguageSpecificStructuredStringType)} sets the 'languageSpecificStructuredString' property of 'myDDI4_InternationalStructuredString' to a list of R6 LanguageSpecificStructuredStringType class instances
#'
#'                              \code{myDDI4_InternationalStructuredString$languageSpecificStructuredString} returns a list of of R6 : LanguageSpecificStructuredStringType class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_InternationalStructuredString$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, languageSpecificStructuredString=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_InternationalStructuredString <- R6Class(classname="DDI4_InternationalStructuredString",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, languageSpecificStructuredString=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_InternationalStructuredString may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(languageSpecificStructuredString)){
        private$.languageSpecificStructuredString <- returnR6ObjectIfValid(validClass="DDI4_LanguageSpecificStructuredStringType", proposedObjectList=languageSpecificStructuredString , minimumCardinality=1, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 1 properties: 
    languageSpecificStructuredString = function(val){
      if (!missing(val)){
        private$.languageSpecificStructuredString <- returnR6ObjectIfValid(validClass="DDI4_LanguageSpecificStructuredStringType", proposedObjectList=val , minimumCardinality=1, maximumCardinality=Inf)  
      } else {
        private$.languageSpecificStructuredString
      }
    }


  ),

  private = list(
    .languageSpecificStructuredString = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        languageSpecificStructuredString = c(
          min="1", 
          max="Inf", 
          datatype = "DDI4_LanguageSpecificStructuredStringType", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_InternationalStructuredString


#' @title DDI4_LabelForDisplay  A structured display label
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/LabelForDisplay}{LabelForDisplay} 

#'
#'   Definition
#'
#'   ============
#'
#'   A structured display label. Label provides display content of a fully human readable display for the identification of the object. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:LabelType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_InternationalStructuredString}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_LabelForDisplay$DdiUrn} returns the DDI URN of myDDI4_LabelForDisplay.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{locationVariant}}{ A read/write active binding.
#'        Indicate the locality specification for content that is specific to a geographic area. May be a country code, sub-country code, or area name.
#'
#'                              \code{myDDI4_LabelForDisplay$locationVariant <- list(myExternalControlledVocabularyEntry)} sets the 'locationVariant' property of 'myDDI4_LabelForDisplay' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_LabelForDisplay$locationVariant} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{validDates}}{ A read/write active binding.
#'        Allows for the specification of a starting date and ending date for the period that this label is valid. 
#'
#'                              \code{myDDI4_LabelForDisplay$validDates <- list(myDateRange)} sets the 'validDates' property of 'myDDI4_LabelForDisplay' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_LabelForDisplay$validDates} returns a list of of R6 : DateRange class instances}
#'   }
#'   \describe{
#'     \item{\code{maxLength}}{ A read/write active binding.
#'        A positive integer indicating the maximum number of characters in the label.
#'
#'                              \code{myDDI4_LabelForDisplay$maxLength <- myinteger} sets the maxLength property of 'myDDI4_LabelForDisplay' to the vector of datatype:integer
#'
#'                              \code{myDDI4_LabelForDisplay$maxLength} returns a vector of datatype: integer}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_LabelForDisplay$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, locationVariant=NA, validDates=NA, maxLength=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_LabelForDisplay <- R6Class(classname="DDI4_LabelForDisplay",
  inherit = DDI4_InternationalStructuredString,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, locationVariant=NA, validDates=NA, maxLength=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_LabelForDisplay may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(locationVariant)){
        private$.locationVariant <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=locationVariant , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(validDates)){
        private$.validDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=validDates , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(maxLength)){
        private$.maxLength <- returnVectorIfValid(validVectorClass="integer", proposedVector=maxLength , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    locationVariant = function(val){
      if (!missing(val)){
        private$.locationVariant <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.locationVariant
      }
    },

    maxLength = function(val){
      if (!missing(val)){
        private$.maxLength <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.maxLength
      }
    },

    validDates = function(val){
      if (!missing(val)){
        private$.validDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.validDates
      }
    }


  ),

  private = list(
    .locationVariant = NULL,
    .validDates = NULL,
    .maxLength = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        languageSpecificStructuredString = c(
          min="1", 
          max="Inf", 
          datatype = "DDI4_LanguageSpecificStructuredStringType", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        locationVariant = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        maxLength = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        validDates = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_LabelForDisplay


#' @title DDI4_LanguageSpecificStringType  Allows for non-formatted language specific strings that may be translations from other languages, or that may be translatable into other languages
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/LanguageSpecificStringType}{LanguageSpecificStringType} 

#'
#'   Definition
#'
#'   ============
#'
#'   Allows for non-formatted language specific strings that may be translations from other languages, or that may be translatable into other languages. Only one string per language/scope type is allowed. LanguageSpecificString contains the following attributes, xmlang to designate the language, isTranslated with a default value of false to designate if an object is a translation of another language, isTranslatable with a default value of true to designate if the content can be translated, translationSourceLanguage to indicate the source languages used in creating this translation,  translationDate, and scope which can be used to define intended audience or use such as internal, external, etc.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:StringType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_LanguageSpecificStringType$DdiUrn} returns the DDI URN of myDDI4_LanguageSpecificStringType.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{content}}{ A read/write active binding.
#'        Value of this string
#'
#'                              \code{myDDI4_LanguageSpecificStringType$content <- mycharacter} sets the content property of 'myDDI4_LanguageSpecificStringType' to the vector of datatype:character
#'
#'                              \code{myDDI4_LanguageSpecificStringType$content} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{language}}{ A read/write active binding.
#'        Indicates the language of content. Note that language allows for a simple 2 or 3 character language code or a language code extended by a country code , for example en-au for English as used in Australia.
#'
#'                              \code{myDDI4_LanguageSpecificStringType$language <- mycharacter} sets the language property of 'myDDI4_LanguageSpecificStringType' to the vector of datatype:character
#'
#'                              \code{myDDI4_LanguageSpecificStringType$language} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{isTranslated}}{ A read/write active binding.
#'        Indicates whether content is a translation (true) or an original (false).
#'
#'                              \code{myDDI4_LanguageSpecificStringType$isTranslated <- mylogical} sets the isTranslated property of 'myDDI4_LanguageSpecificStringType' to the vector of datatype:logical
#'
#'                              \code{myDDI4_LanguageSpecificStringType$isTranslated} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{isTranslatable}}{ A read/write active binding.
#'        Indicates whether content is translatable (true) or not (false). An example of something that is not translatable would be a MNEMONIC of an object or a number.
#'
#'                              \code{myDDI4_LanguageSpecificStringType$isTranslatable <- mylogical} sets the isTranslatable property of 'myDDI4_LanguageSpecificStringType' to the vector of datatype:logical
#'
#'                              \code{myDDI4_LanguageSpecificStringType$isTranslatable} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{translationSourceLanguage}}{ A read/write active binding.
#'        List the language code of the source. Repeat of multiple language sources are used. 
#'
#'                              \code{myDDI4_LanguageSpecificStringType$translationSourceLanguage <- mycharacter} sets the translationSourceLanguage property of 'myDDI4_LanguageSpecificStringType' to the vector of datatype:character
#'
#'                              \code{myDDI4_LanguageSpecificStringType$translationSourceLanguage} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{translationDate}}{ A read/write active binding.
#'        The date the content was translated. Provision of translation date allows user to verify if translation was available during data collection or other time linked activity.
#'
#'                              \code{myDDI4_LanguageSpecificStringType$translationDate <- mycharacter} sets the translationDate property of 'myDDI4_LanguageSpecificStringType' to the vector of datatype:character
#'
#'                              \code{myDDI4_LanguageSpecificStringType$translationDate} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{scope}}{ A read/write active binding.
#'        Supports specification of scope for the contained content. Use with the language specification to filter application of content.
#'
#'                              \code{myDDI4_LanguageSpecificStringType$scope <- mycharacter} sets the scope property of 'myDDI4_LanguageSpecificStringType' to the vector of datatype:character
#'
#'                              \code{myDDI4_LanguageSpecificStringType$scope} returns a vector of datatype: character}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_LanguageSpecificStringType$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, content=NA, language=NA, isTranslated=NA, isTranslatable=NA, translationSourceLanguage=NA, translationDate=NA, scope=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_LanguageSpecificStringType <- R6Class(classname="DDI4_LanguageSpecificStringType",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, content=NA, language=NA, isTranslated=NA, isTranslatable=NA, translationSourceLanguage=NA, translationDate=NA, scope=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_LanguageSpecificStringType may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(content)){
        private$.content <- returnVectorIfValid(validVectorClass="character", proposedVector=content , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(language)){
        private$.language <- returnVectorIfValid(validVectorClass="character", proposedVector=language , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isTranslated)){
        private$.isTranslated <- returnVectorIfValid(validVectorClass="logical", proposedVector=isTranslated , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isTranslatable)){
        private$.isTranslatable <- returnVectorIfValid(validVectorClass="logical", proposedVector=isTranslatable , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(translationSourceLanguage)){
        private$.translationSourceLanguage <- returnVectorIfValid(validVectorClass="character", proposedVector=translationSourceLanguage , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(translationDate)){
        private$.translationDate <- returnVectorIfValid(validVectorClass="character", proposedVector=translationDate , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(scope)){
        private$.scope <- returnVectorIfValid(validVectorClass="character", proposedVector=scope , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 7 properties: 
    content = function(val){
      if (!missing(val)){
        private$.content <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.content
      }
    },

    isTranslatable = function(val){
      if (!missing(val)){
        private$.isTranslatable <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isTranslatable
      }
    },

    isTranslated = function(val){
      if (!missing(val)){
        private$.isTranslated <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isTranslated
      }
    },

    language = function(val){
      if (!missing(val)){
        private$.language <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.language
      }
    },

    scope = function(val){
      if (!missing(val)){
        private$.scope <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.scope
      }
    },

    translationDate = function(val){
      if (!missing(val)){
        private$.translationDate <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.translationDate
      }
    },

    translationSourceLanguage = function(val){
      if (!missing(val)){
        private$.translationSourceLanguage <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.translationSourceLanguage
      }
    }


  ),

  private = list(
    .content = NULL,
    .language = NULL,
    .isTranslated = NULL,
    .isTranslatable = NULL,
    .translationSourceLanguage = NULL,
    .translationDate = NULL,
    .scope = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        content = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isTranslatable = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isTranslated = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        language = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        scope = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        translationDate = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        translationSourceLanguage = c(
          min="0", 
          max="Inf", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_LanguageSpecificStringType


#' @title DDI4_LanguageSpecificStructuredStringType  Supports the optional use of formatting tags expressed with escape characters within the string structure
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/LanguageSpecificStructuredStringType}{LanguageSpecificStructuredStringType} 

#'
#'   Definition
#'
#'   ============
#'
#'   Supports the optional use of formatting tags expressed with escape characters within the string structure. Language of the string is defined by the attribute language. Scope of the string is defined by scope. Language and Scope are used to filter selection and use of content based on unique combinations of these two properties. The content can be identified as translated (isTranslated), subject to translation (isTranslatable), the result of translation from one or more languages (translationSourceLanguages), carry an indication whether or not it should be treated as plain text (isPlain), identification of the structure used (structureUsed) which is an internal enumeration, and the ability to identify a structure not available in the enumeration by declaring structureUsed="Other" and specifying the other structure (otherDefined).
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   The content:
#'
#'   <p>a bit of text</p> <p>with some specialties <span class="highlight">highlighted</span> </p> 
#'
#'   
#'
#'   Would be entered as:
#'
#'   <p>with some specialties <span class="highlight">highlighted</span> </p> 
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:ContentType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_LanguageSpecificStringType}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_LanguageSpecificStructuredStringType$DdiUrn} returns the DDI URN of myDDI4_LanguageSpecificStructuredStringType.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{isPlainText}}{ A read/write active binding.
#'        Indicates that the content is to be treated as plain text (no formatting). You may use DDIProfile to fix the value of this attribute to 'true' in cases where you wish to indicate that your system treats all content should be treated as plain text.
#'
#'                              \code{myDDI4_LanguageSpecificStructuredStringType$isPlainText <- mylogical} sets the isPlainText property of 'myDDI4_LanguageSpecificStructuredStringType' to the vector of datatype:logical
#'
#'                              \code{myDDI4_LanguageSpecificStructuredStringType$isPlainText} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{structureUsed}}{ A read/write active binding.
#'        Uses a enumerated value to specify the structuring elements used. 
#'
#'                              \code{myDDI4_LanguageSpecificStructuredStringType$structureUsed <- list(myStringStructureType)} sets the 'structureUsed' property of 'myDDI4_LanguageSpecificStructuredStringType' to a list of R6 StringStructureType class instances
#'
#'                              \code{myDDI4_LanguageSpecificStructuredStringType$structureUsed} returns a list of of R6 : StringStructureType class instances}
#'   }
#'   \describe{
#'     \item{\code{otherDefined}}{ A read/write active binding.
#'        If the value of structureUsed is Other enter the structue type.
#'
#'                              \code{myDDI4_LanguageSpecificStructuredStringType$otherDefined <- mycharacter} sets the otherDefined property of 'myDDI4_LanguageSpecificStructuredStringType' to the vector of datatype:character
#'
#'                              \code{myDDI4_LanguageSpecificStructuredStringType$otherDefined} returns a vector of datatype: character}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_LanguageSpecificStructuredStringType$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, isPlainText=NA, structureUsed=NA, otherDefined=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_LanguageSpecificStructuredStringType <- R6Class(classname="DDI4_LanguageSpecificStructuredStringType",
  inherit = DDI4_LanguageSpecificStringType,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, isPlainText=NA, structureUsed=NA, otherDefined=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_LanguageSpecificStructuredStringType may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(isPlainText)){
        private$.isPlainText <- returnVectorIfValid(validVectorClass="logical", proposedVector=isPlainText , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(structureUsed)){
        private$.structureUsed <- returnEnumeratedValues(enumerationObject=DDI4_StringStructureType$new(registerThisObject=FALSE), proposedCharacterVector=structureUsed , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(otherDefined)){
        private$.otherDefined <- returnVectorIfValid(validVectorClass="character", proposedVector=otherDefined , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    isPlainText = function(val){
      if (!missing(val)){
        private$.isPlainText <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isPlainText
      }
    },

    otherDefined = function(val){
      if (!missing(val)){
        private$.otherDefined <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.otherDefined
      }
    },

    structureUsed = function(val){
      if (!missing(val)){
        private$.structureUsed <- returnEnumeratedValues(enumerationObject=DDI4_StringStructureType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.structureUsed
      }
    }


  ),

  private = list(
    .isPlainText = NULL,
    .structureUsed = NULL,
    .otherDefined = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        content = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPlainText = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isTranslatable = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isTranslated = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        language = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        otherDefined = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        scope = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        structureUsed = c(
          min="0", 
          max="1", 
          datatype = "DDI4_StringStructureType", 
          structure = "Enumeration vector"
        ),
        translationDate = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        translationSourceLanguage = c(
          min="0", 
          max="Inf", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_LanguageSpecificStructuredStringType


#' @title DDI4_Level  Provides Level information for the members of the LevelStructure
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Level}{Level} 

#'
#'   Definition
#'
#'   ============
#'
#'   Provides Level information for the members of the LevelStructure. levelNumber provides the level number which may or may not be associated with a category which defines level.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   ISCO-08: index='1' label of associated category 'Major',  index='2' label of associated category 'Sub-Major',  index='3' label of associated category 'Minor', 
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Level$DdiUrn} returns the DDI URN of myDDI4_Level.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{levelNumber}}{ A read/write active binding.
#'        Provides an association between a level number and optional concept which defines it within an ordered array. Use is required.
#'
#'                              \code{myDDI4_Level$levelNumber <- myinteger} sets the levelNumber property of 'myDDI4_Level' to the vector of datatype:integer
#'
#'                              \code{myDDI4_Level$levelNumber} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A display label for the object. May be expressed in multiple languages. Repeat for labels with different content, for example, labels with differing length limitations.
#'
#'                              \code{myDDI4_Level$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_Level' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_Level$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{definedBy}}{ A read/write active binding.
#'          A concept or concept sub-type which describes the level.
#'
#'                              \code{myDDI4_Level$definedBy <- c(myConceptURN)} sets the association 'definedBy' from 'myDDI4_Level' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_Level$definedBy} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Level$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, levelNumber=NA, displayLabel=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Level <- R6Class(classname="DDI4_Level",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, levelNumber=NA, displayLabel=NA,    definedBy=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Level may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(levelNumber)){
        private$.levelNumber <- returnVectorIfValid(validVectorClass="integer", proposedVector=levelNumber , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(definedBy)){
        private$.definedBy <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=definedBy , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definedBy, referencingAssociationName="definedBy",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 2 properties: 
    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },

    levelNumber = function(val){
      if (!missing(val)){
        private$.levelNumber <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.levelNumber
      }
    },


# There are 1 class association(s):
    definedBy = function(val){
      if (!missing(val)){
        private$.definedBy <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definedBy, referencingAssociationName="definedBy",  referencingUrn=thisUrn)  
       # invisible( private$.definedBy)
      } else {
          private$.definedBy
      }
    }

  ),

  private = list(
    .levelNumber = NULL,
    .displayLabel = NULL,
    .definedBy = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        definedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        levelNumber = c(
          min="1", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Level


#' @title DDI4_LevelStructure  The LevelStructure describes the nesting structure of a hierarchical collection
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/LevelStructure}{LevelStructure} 

#'
#'   Definition
#'
#'   ============
#'
#'   The LevelStructure describes the nesting structure of a hierarchical collection. The levels within the structure begin at the root level '1' and continue as an ordered array through each level of nesting.
#'
#'   
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   ISCO-08 (Major, Sub-Major, and Minor) or NAICS (2 digit sector codes, 3 digit subsector code list, 4 digit industry group code list, and 5 digit industry code list)
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Levels are used to organize a hierarchy. Usually, a hierarchy often contains one root member at the top, though it could contain several. These are the first Level. All members directly related to those  in the first Level compose the second Level. The third and subsequent Levels are defined similarly. 
#'
#'   
#'
#'   A Level often is associated with a Concept, which defines it. These correspond to kinds of aggregates. For example, in the US Standard Occupational Classification (2010), the Level below the top is called Major Occupation Groups, and the next Level is called Minor Occupational Groups. These ideas convey the structure. In particular, Health Care Practitioners (a Major Group) can be broken into Chiropractors, Dentists, Physicians, Vets, Therapists, etc. (Minor Groups) The Categories in the Nodes at the lower Level aggregate to the Category in Node above them.
#'
#'   
#'
#'   "Classification schemes are frequently organized in nested levels of increasing detail. ISCO-08, for example, has four levels: at the top level are ten major groups, each of which contain sub-major groups, which in turn are subdivided in minor groups, which contain unit groups. Even when a classification is not structured in levels ("flat classification"), the usual convention, which is adopted here, is to consider that it contains one unique level." (http://rdf-vocabulary.ddialliance.org/xkos.html#'
#'   ) Individual classification items organized in a hierarchy may be associated with a specific level.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   l:LevelType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Level
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_LevelStructure$DdiUrn} returns the DDI URN of myDDI4_LevelStructure.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage.
#'
#'                              \code{myDDI4_LevelStructure$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_LevelStructure' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_LevelStructure$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{usage}}{ A read/write active binding.
#'        Explanation of the ways in which some decision or object is employed. Supports the use of multiple languages and structured text. Use in locations where you need to relay information on the usage of the instantiated class.
#'
#'                              \code{myDDI4_LevelStructure$usage <- list(myInternationalStructuredString)} sets the 'usage' property of 'myDDI4_LevelStructure' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_LevelStructure$usage} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{containsLevel}}{ A read/write active binding.
#'        The association of a level number in an ordered array with the description of that level
#'
#'                              \code{myDDI4_LevelStructure$containsLevel <- list(myLevel)} sets the 'containsLevel' property of 'myDDI4_LevelStructure' to a list of R6 Level class instances
#'
#'                              \code{myDDI4_LevelStructure$containsLevel} returns a list of of R6 : Level class instances}
#'   }
#'   \describe{
#'     \item{\code{validDateRange}}{ A read/write active binding.
#'        The dates for which the level structure is valid.  A structured DateRange with start and end Date (both with the structure of Date and supporting the use of ISO and non-ISO date structures); Use to relate a period with a start and end date.
#'
#'                              \code{myDDI4_LevelStructure$validDateRange <- list(myDateRange)} sets the 'validDateRange' property of 'myDDI4_LevelStructure' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_LevelStructure$validDateRange} returns a list of of R6 : DateRange class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_LevelStructure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, name=NA, usage=NA, containsLevel=NA, validDateRange=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_LevelStructure <- R6Class(classname="DDI4_LevelStructure",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, name=NA, usage=NA, containsLevel=NA, validDateRange=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_LevelStructure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(usage)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=usage , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(containsLevel)){
        private$.containsLevel <- returnR6ObjectIfValid(validClass="DDI4_Level", proposedObjectList=containsLevel , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(validDateRange)){
        private$.validDateRange <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=validDateRange , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    containsLevel = function(val){
      if (!missing(val)){
        private$.containsLevel <- returnR6ObjectIfValid(validClass="DDI4_Level", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.containsLevel
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    usage = function(val){
      if (!missing(val)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.usage
      }
    },

    validDateRange = function(val){
      if (!missing(val)){
        private$.validDateRange <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.validDateRange
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .name = NULL,
    .usage = NULL,
    .containsLevel = NULL,
    .validDateRange = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        containsLevel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Level", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        validDateRange = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_LevelStructure


#' @title DDI4_LineParameter  Specification of the line and offset for the beginning and end of the segment
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/LineParameter}{LineParameter} 

#'
#'   Definition
#'
#'   ============
#'
#'   Specification of the line and offset for the beginning and end of the segment.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:LineParameterType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_LineParameter$DdiUrn} returns the DDI URN of myDDI4_LineParameter.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{startLine}}{ A read/write active binding.
#'        Number of lines from beginning of the document.
#'
#'                              \code{myDDI4_LineParameter$startLine <- myinteger} sets the startLine property of 'myDDI4_LineParameter' to the vector of datatype:integer
#'
#'                              \code{myDDI4_LineParameter$startLine} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{startOffset}}{ A read/write active binding.
#'        Number of characters from start of the line specified in StartLine.
#'
#'                              \code{myDDI4_LineParameter$startOffset <- myinteger} sets the startOffset property of 'myDDI4_LineParameter' to the vector of datatype:integer
#'
#'                              \code{myDDI4_LineParameter$startOffset} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{endLine}}{ A read/write active binding.
#'        Number of lines from beginning of the document.
#'
#'                              \code{myDDI4_LineParameter$endLine <- myinteger} sets the endLine property of 'myDDI4_LineParameter' to the vector of datatype:integer
#'
#'                              \code{myDDI4_LineParameter$endLine} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{endOffset}}{ A read/write active binding.
#'        Number of characters from the start of the line specified in EndLine.
#'
#'                              \code{myDDI4_LineParameter$endOffset <- myinteger} sets the endOffset property of 'myDDI4_LineParameter' to the vector of datatype:integer
#'
#'                              \code{myDDI4_LineParameter$endOffset} returns a vector of datatype: integer}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_LineParameter$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, startLine=NA, startOffset=NA, endLine=NA, endOffset=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_LineParameter <- R6Class(classname="DDI4_LineParameter",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, startLine=NA, startOffset=NA, endLine=NA, endOffset=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_LineParameter may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(startLine)){
        private$.startLine <- returnVectorIfValid(validVectorClass="integer", proposedVector=startLine , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(startOffset)){
        private$.startOffset <- returnVectorIfValid(validVectorClass="integer", proposedVector=startOffset , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(endLine)){
        private$.endLine <- returnVectorIfValid(validVectorClass="integer", proposedVector=endLine , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(endOffset)){
        private$.endOffset <- returnVectorIfValid(validVectorClass="integer", proposedVector=endOffset , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    endLine = function(val){
      if (!missing(val)){
        private$.endLine <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.endLine
      }
    },

    endOffset = function(val){
      if (!missing(val)){
        private$.endOffset <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.endOffset
      }
    },

    startLine = function(val){
      if (!missing(val)){
        private$.startLine <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.startLine
      }
    },

    startOffset = function(val){
      if (!missing(val)){
        private$.startOffset <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.startOffset
      }
    }


  ),

  private = list(
    .startLine = NULL,
    .startOffset = NULL,
    .endLine = NULL,
    .endOffset = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        endLine = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        endOffset = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        startLine = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        startOffset = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_LineParameter


#' @title DDI4_LiteralText  Literal (static) text to be used in an instrument
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/LiteralText}{LiteralText} 

#'
#'   Definition
#'
#'   ============
#'
#'   Literal (static) text to be used in an instrument.  The value is a LanguageSpecificStruccturedString with the addition of an attribute allowing for the specification that white space is to be preserved.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   The text string "     Comment vous appelez-vous?" with the leading spaces included, described as being in French.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   d:LiteralTextType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_DynamicTextContent}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_LiteralText$DdiUrn} returns the DDI URN of myDDI4_LiteralText.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{text}}{ A read/write active binding.
#'        The value of the static text string. Supports the optional use of XHTML formatting tags within the string structure. If the content of a literal text contains more than one language, i.e. "What is your understanding of the German word 'Gesundheit'?", the foreign language element should be placed in a separate LiteralText component with the appropriate xmlang value and, in this case, isTranslatable set to "false". If the existence of white space is critical to the understanding of the content (such as inclusion of a leading or trailing white space), set the attribute of Text xml:space to "preserve".
#'
#'                              \code{myDDI4_LiteralText$text <- list(myLanguageSpecificStructuredStringType)} sets the 'text' property of 'myDDI4_LiteralText' to a list of R6 LanguageSpecificStructuredStringType class instances
#'
#'                              \code{myDDI4_LiteralText$text} returns a list of of R6 : LanguageSpecificStructuredStringType class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_LiteralText$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, text=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_LiteralText <- R6Class(classname="DDI4_LiteralText",
  inherit = DDI4_DynamicTextContent,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, text=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_LiteralText may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(text)){
        private$.text <- returnR6ObjectIfValid(validClass="DDI4_LanguageSpecificStructuredStringType", proposedObjectList=text , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 1 properties: 
    text = function(val){
      if (!missing(val)){
        private$.text <- returnR6ObjectIfValid(validClass="DDI4_LanguageSpecificStructuredStringType", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.text
      }
    }


  ),

  private = list(
    .text = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        orderPosition = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        text = c(
          min="0", 
          max="1", 
          datatype = "DDI4_LanguageSpecificStructuredStringType", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_LiteralText


#' @title DDI4_LocalIdFormat  This is an identifier in a given local context that uniquely references an object, as opposed to the full ddi identifier which has an agency plus the id
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/LocalIdFormat}{LocalIdFormat} 

#'
#'   Definition
#'
#'   ============
#'
#'   This is an identifier in a given local context that uniquely references an object, as opposed to the full ddi identifier which has an agency plus the id.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   The name of a variable within a dataset is a localId. Software systems might use their own identifier systems for performance reasons (e.g. incrementing integers).
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_LocalIdFormat$DdiUrn} returns the DDI URN of myDDI4_LocalIdFormat.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{localIdValue}}{ A read/write active binding.
#'        Value of the local ID.
#'
#'                              \code{myDDI4_LocalIdFormat$localIdValue <- mycharacter} sets the localIdValue property of 'myDDI4_LocalIdFormat' to the vector of datatype:character
#'
#'                              \code{myDDI4_LocalIdFormat$localIdValue} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{localIdType}}{ A read/write active binding.
#'        Type of identifier, specifying the context of the identifier.
#'
#'                              \code{myDDI4_LocalIdFormat$localIdType <- mycharacter} sets the localIdType property of 'myDDI4_LocalIdFormat' to the vector of datatype:character
#'
#'                              \code{myDDI4_LocalIdFormat$localIdType} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{localIdVersion}}{ A read/write active binding.
#'        Version of the Local ID.
#'
#'                              \code{myDDI4_LocalIdFormat$localIdVersion <- mycharacter} sets the localIdVersion property of 'myDDI4_LocalIdFormat' to the vector of datatype:character
#'
#'                              \code{myDDI4_LocalIdFormat$localIdVersion} returns a vector of datatype: character}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_LocalIdFormat$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, localIdValue=NA, localIdType=NA, localIdVersion=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_LocalIdFormat <- R6Class(classname="DDI4_LocalIdFormat",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, localIdValue=NA, localIdType=NA, localIdVersion=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_LocalIdFormat may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(localIdValue)){
        private$.localIdValue <- returnVectorIfValid(validVectorClass="character", proposedVector=localIdValue , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(localIdType)){
        private$.localIdType <- returnVectorIfValid(validVectorClass="character", proposedVector=localIdType , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(localIdVersion)){
        private$.localIdVersion <- returnVectorIfValid(validVectorClass="character", proposedVector=localIdVersion , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    localIdType = function(val){
      if (!missing(val)){
        private$.localIdType <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.localIdType
      }
    },

    localIdValue = function(val){
      if (!missing(val)){
        private$.localIdValue <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.localIdValue
      }
    },

    localIdVersion = function(val){
      if (!missing(val)){
        private$.localIdVersion <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.localIdVersion
      }
    }


  ),

  private = list(
    .localIdValue = NULL,
    .localIdType = NULL,
    .localIdVersion = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        localIdType = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        localIdValue = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        localIdVersion = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_LocalIdFormat


#' @title DDI4_LocationName  Name of the location using the DDI Name structure and the ability to add an effective date
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/LocationName}{LocationName} 

#'
#'   Definition
#'
#'   ============
#'
#'   Name of the location using the DDI Name structure and the ability to add an effective date.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   a:LocationNameType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ObjectName}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_LocationName$DdiUrn} returns the DDI URN of myDDI4_LocationName.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{effectiveDates}}{ A read/write active binding.
#'        The time period for which this name is accurate and in use.
#'
#'                              \code{myDDI4_LocationName$effectiveDates <- list(myDateRange)} sets the 'effectiveDates' property of 'myDDI4_LocationName' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_LocationName$effectiveDates} returns a list of of R6 : DateRange class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_LocationName$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, effectiveDates=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_LocationName <- R6Class(classname="DDI4_LocationName",
  inherit = DDI4_ObjectName,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, effectiveDates=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_LocationName may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(effectiveDates)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=effectiveDates , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 1 properties: 
    effectiveDates = function(val){
      if (!missing(val)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.effectiveDates
      }
    }


  ),

  private = list(
    .effectiveDates = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        content = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        context = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        effectiveDates = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_LocationName


#' @title DDI4_LogicalRecord  The LogicalRecord is a record definition
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/LogicalRecord}{LogicalRecord} 

#'
#'   Definition
#'
#'   ============
#'
#'   The LogicalRecord is a record definition. It is abstract.
#'
#'   
#'
#'   The actual realization of record definitions are provided by the UnitDataRecord and the DataCube. UnitDataRecords support two types of record definitions. The first is based on the SimpleCollection it inherits from LogicalRecord. The second type is a StructuredCollection that specializes the LogicalRecord. As a SimpleCollection a UnitDataRecord can, for example, provide table definitions. As a StructuredCollection, UnitDataRecord can define structures comparable to a "struct" in C or a JSON nested object. A UnitDataRecord, then, inherits SimpleCollection from LogicalRecord but adds its own StructuredCollection into the mix. The DataCube brings into play its own StructuredCollection which is distinct from the UnitDataRecord StructuredCollection.The DataCube StructuredCollection is not currently in scope.
#'
#'   
#'
#'   The various record definitions -- simple and structured -- when they are actually instantiated with data is described by the PhysicalLayout of a LogicalRecord in the FormatDescription package. The PhysicalLayout places data into DataPoints formed at the direction of a LogicalRecord and its instance variables. At this point a LogicalRecord turn into a "dataset" that hosts unit records, A record definition is not a record. So we conflate metadata and data when we refer to a LogicalRecord as an empty table.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   SQL Data Definition Language (DDL) traffics in record definitions. SQL queries on "system tables" discover record definitions.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   In GSIM a DataPoint is a member of a "DataSet" and a UnitDataRecord. Since a DataPoint contains "Datum", this can lead to the conflation of a DataSet and a record definition. In the presence of a DataPoint it is difficult to be clear that a record definition does not have any rows. 
#'
#'   
#'
#'   In DDI4 we defer the introduction of DataPoints until PhysicalLayouts are described. In this approach the instance variables that make up a LogicalRecord work downstream where they help to give each DataPoint in a PhysicalRecord its format.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   Record Type, Data Structure
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   l:LogicalRecordType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Data Structure
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_LogicalRecord$DdiUrn} returns the DDI URN of myDDI4_LogicalRecord.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{type}}{ A read/write active binding.
#'        Whether the collection is a bag or a set: a bag is a collection with duplicates allowed, a set is a collection without duplicates.
#'
#'                              \code{myDDI4_LogicalRecord$type <- list(myCollectionType)} sets the 'type' property of 'myDDI4_LogicalRecord' to a list of R6 CollectionType class instances
#'
#'                              \code{myDDI4_LogicalRecord$type} returns a list of of R6 : CollectionType class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage.
#'
#'                              \code{myDDI4_LogicalRecord$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_LogicalRecord' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_LogicalRecord$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_LogicalRecord$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_LogicalRecord' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_LogicalRecord$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        Specifies contents of Unit Record as Instance Variable values.
#'
#'                              \code{myDDI4_LogicalRecord$contains <- list(myInstanceVariableIndicator)} sets the 'contains' property of 'myDDI4_LogicalRecord' to a list of R6 InstanceVariableIndicator class instances
#'
#'                              \code{myDDI4_LogicalRecord$contains} returns a list of of R6 : InstanceVariableIndicator class instances}
#'   }
#'   \describe{
#'     \item{\code{isOrdered}}{ A read/write active binding.
#'        If members are ordered set to true, if unordered set to false.
#'
#'                              \code{myDDI4_LogicalRecord$isOrdered <- mylogical} sets the isOrdered property of 'myDDI4_LogicalRecord' to the vector of datatype:logical
#'
#'                              \code{myDDI4_LogicalRecord$isOrdered} returns a vector of datatype: logical}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{definingConcept}}{ A read/write active binding.
#'          The conceptual basis for the collection of members.
#'
#'                              \code{myDDI4_LogicalRecord$definingConcept <- c(myConceptURN)} sets the association 'definingConcept' from 'myDDI4_LogicalRecord' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_LogicalRecord$definingConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_LogicalRecord$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_LogicalRecord <- R6Class(classname="DDI4_LogicalRecord",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA,    definingConcept=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_LogicalRecord may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(type)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=type , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_InstanceVariableIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(isOrdered)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=isOrdered , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(definingConcept)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=definingConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 5 properties: 
    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_InstanceVariableIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },

    isOrdered = function(val){
      if (!missing(val)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isOrdered
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    type = function(val){
      if (!missing(val)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.type
      }
    },


# There are 1 class association(s):
    definingConcept = function(val){
      if (!missing(val)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
       # invisible( private$.definingConcept)
      } else {
          private$.definingConcept
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .type = NULL,
    .name = NULL,
    .purpose = NULL,
    .contains = NULL,
    .isOrdered = NULL,
    .definingConcept = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InstanceVariableIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_LogicalRecord


#' @title DDI4_LogicalRecordIndicator  Member Indicator for use with member type LogicalRecord
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/LogicalRecordIndicator}{LogicalRecordIndicator} 

#'
#'   Definition
#'
#'   ============
#'
#'   Member Indicator for use with member type LogicalRecord
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_LogicalRecordIndicator$DdiUrn} returns the DDI URN of myDDI4_LogicalRecordIndicator.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Index value of member in an ordered array
#'
#'                              \code{myDDI4_LogicalRecordIndicator$index <- myinteger} sets the index property of 'myDDI4_LogicalRecordIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_LogicalRecordIndicator$index} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{member}}{ A read/write active binding.
#'          Restricts member target class to LogicalRecord
#'
#'                              \code{myDDI4_LogicalRecordIndicator$member <- c(myLogicalRecordURN)} sets the association 'member' from 'myDDI4_LogicalRecordIndicator' to a character vector of DdiUrns for R6 LogicalRecord class instances
#'
#'                              \code{myDDI4_LogicalRecordIndicator$member} returns a character vector of of DdiUrns for R6  LogicalRecord class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_LogicalRecordIndicator$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_LogicalRecordIndicator <- R6Class(classname="DDI4_LogicalRecordIndicator",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA,    member=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_LogicalRecordIndicator may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(member)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_LogicalRecord", proposedUrnVector=member , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.index
      }
    },


# There are 1 class association(s):
    member = function(val){
      if (!missing(val)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_LogicalRecord", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
       # invisible( private$.member)
      } else {
          private$.member
      }
    }

  ),

  private = list(
    .index = NULL,
    .member = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        index = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        member = c(
          min="1", 
          max="1", 
          datatype = "DDI4_LogicalRecord", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_LogicalRecordIndicator


#' @title DDI4_LogicalRecordRelation  Defines complex relationship between LogicalRecords
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/LogicalRecordRelation}{LogicalRecordRelation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Defines complex relationship between LogicalRecords. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_LogicalRecordRelation$DdiUrn} returns the DDI URN of myDDI4_LogicalRecordRelation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an enumeration list
#'
#'                              \code{myDDI4_LogicalRecordRelation$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_LogicalRecordRelation' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_LogicalRecordRelation$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_LogicalRecordRelation$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_LogicalRecordRelation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_LogicalRecordRelation$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_LogicalRecordRelation$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_LogicalRecordRelation' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_LogicalRecordRelation$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          First member in the relationship. 
#'
#'                              \code{myDDI4_LogicalRecordRelation$source <- c(myLogicalRecordURN)} sets the association 'source' from 'myDDI4_LogicalRecordRelation' to a character vector of DdiUrns for R6 LogicalRecord class instances
#'
#'                              \code{myDDI4_LogicalRecordRelation$source} returns a character vector of of DdiUrns for R6  LogicalRecord class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          Second member in a relationship. Note that this can be realized as a collection to support tuples.
#'
#'                              \code{myDDI4_LogicalRecordRelation$target <- c(myLogicalRecordURN)} sets the association 'target' from 'myDDI4_LogicalRecordRelation' to a character vector of DdiUrns for R6 LogicalRecord class instances
#'
#'                              \code{myDDI4_LogicalRecordRelation$target} returns a character vector of of DdiUrns for R6  LogicalRecord class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_LogicalRecordRelation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_LogicalRecordRelation <- R6Class(classname="DDI4_LogicalRecordRelation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_LogicalRecordRelation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_LogicalRecord", proposedUrnVector=source , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_LogicalRecord", proposedUrnVector=target , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_LogicalRecord", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_LogicalRecord", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="0", 
          max="1", 
          datatype = "DDI4_LogicalRecord", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LogicalRecord", 
          structure = "DdiUrn vector"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_LogicalRecordRelation


#' @title DDI4_LogicalRecordRelationStructure  Allows for the complex structuring of relationships between LogicalRecords in a DataStore
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/LogicalRecordRelationStructure}{LogicalRecordRelationStructure} 

#'
#'   Definition
#'
#'   ============
#'
#'   Allows for the complex structuring of relationships between LogicalRecords in a DataStore
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A DataStore with a Household, Family, and Person LogicalRecord type. Allows for describing parent/child, whole/part, or other relationships as appropriate
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_LogicalRecordRelationStructure$DdiUrn} returns the DDI URN of myDDI4_LogicalRecordRelationStructure.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an enumeration list.  Use if all relations within this relation structure are of the same specification.
#'
#'                              \code{myDDI4_LogicalRecordRelationStructure$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_LogicalRecordRelationStructure' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_LogicalRecordRelationStructure$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_LogicalRecordRelationStructure$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_LogicalRecordRelationStructure' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_LogicalRecordRelationStructure$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_LogicalRecordRelationStructure$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_LogicalRecordRelationStructure' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_LogicalRecordRelationStructure$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMemberRelation}}{ A read/write active binding.
#'        Logical Record relations that comprise the relation structure
#'
#'                              \code{myDDI4_LogicalRecordRelationStructure$hasMemberRelation <- list(myLogicalRecordRelation)} sets the 'hasMemberRelation' property of 'myDDI4_LogicalRecordRelationStructure' to a list of R6 LogicalRecordRelation class instances
#'
#'                              \code{myDDI4_LogicalRecordRelationStructure$hasMemberRelation} returns a list of of R6 : LogicalRecordRelation class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_LogicalRecordRelationStructure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_LogicalRecordRelationStructure <- R6Class(classname="DDI4_LogicalRecordRelationStructure",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_LogicalRecordRelationStructure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasMemberRelation)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_LogicalRecordRelation", proposedObjectList=hasMemberRelation , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    hasMemberRelation = function(val){
      if (!missing(val)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_LogicalRecordRelation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasMemberRelation
      }
    },

    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .hasMemberRelation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasMemberRelation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LogicalRecordRelation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasRelationSpecification = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_LogicalRecordRelationStructure


#' @title DDI4_Loop  Iterative control structure to be repeated a specified number of times based on one or more conditions
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Loop}{Loop} 

#'
#'   Definition
#'
#'   ============
#'
#'   Iterative control structure to be repeated a specified number of times based on one or more conditions. Inside the loop, one or more Workflow Steps are evaluated and processed in the order they appear.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A loop is set to repeat a fixed number of times (initialValue). A stepValue is initially specified. At the end of the loop the current value is decremented by the stepValue. The current value might be the initialValue or it might be the current value after the initialValue has been decremented one or more times.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   The Workflow Sequence contained in the Loop is executed until the condition is met, and then control is handed back to the containing control construct.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   ForLoop
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   d:LoopType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ConditionalControlStep}}}
#'    \item{\code{\link{DDI4_WorkflowControlStep}}}
#'    \item{\code{\link{DDI4_WorkflowStep}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Loop$DdiUrn} returns the DDI URN of myDDI4_Loop.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{initialValue}}{ A read/write active binding.
#'        The command used to set the initial value for the process. Could be a simple value.
#'
#'                              \code{myDDI4_Loop$initialValue <- list(myCommandCode)} sets the 'initialValue' property of 'myDDI4_Loop' to a list of R6 CommandCode class instances
#'
#'                              \code{myDDI4_Loop$initialValue} returns a list of of R6 : CommandCode class instances}
#'   }
#'   \describe{
#'     \item{\code{stepValue}}{ A read/write active binding.
#'        The command used to set the incremental or step value for the process. Could be a simple value.
#'
#'                              \code{myDDI4_Loop$stepValue <- list(myCommandCode)} sets the 'stepValue' property of 'myDDI4_Loop' to a list of R6 CommandCode class instances
#'
#'                              \code{myDDI4_Loop$stepValue} returns a list of of R6 : CommandCode class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Loop$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, initialValue=NA, stepValue=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Loop <- R6Class(classname="DDI4_Loop",
  inherit = DDI4_ConditionalControlStep,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, initialValue=NA, stepValue=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Loop may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(initialValue)){
        private$.initialValue <- returnR6ObjectIfValid(validClass="DDI4_CommandCode", proposedObjectList=initialValue , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(stepValue)){
        private$.stepValue <- returnR6ObjectIfValid(validClass="DDI4_CommandCode", proposedObjectList=stepValue , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    initialValue = function(val){
      if (!missing(val)){
        private$.initialValue <- returnR6ObjectIfValid(validClass="DDI4_CommandCode", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.initialValue
      }
    },

    stepValue = function(val){
      if (!missing(val)){
        private$.stepValue <- returnR6ObjectIfValid(validClass="DDI4_CommandCode", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.stepValue
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .initialValue = NULL,
    .stepValue = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        condition = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CommandCode", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        executes = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowStepSequence", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInformationFlow = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        initialValue = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CommandCode", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        isPerformedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowService", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        stepValue = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CommandCode", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Loop


#' @title DDI4_Machine  Mechanism or computer program used to implement a process
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Machine}{Machine} 

#'
#'   Definition
#'
#'   ============
#'
#'   Mechanism or computer program used to implement a process.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   SAS program, photocopier
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   May be used as the target to describe how an action was performed. Relevent to data capture and data processing or wherever a role is performed by a Machine.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Agent}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Machine$DdiUrn} returns the DDI URN of myDDI4_Machine.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{typeOfMachine}}{ A read/write active binding.
#'        The kind of machine used - software, web service, physical machine, from a controlled vocabulary
#'
#'                              \code{myDDI4_Machine$typeOfMachine <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfMachine' property of 'myDDI4_Machine' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_Machine$typeOfMachine} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        The name of the machine. A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage.
#'
#'                              \code{myDDI4_Machine$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_Machine' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_Machine$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{hasAccessLocation}}{ A read/write active binding.
#'        The locations where the machine can be access
#'
#'                              \code{myDDI4_Machine$hasAccessLocation <- list(myAccessLocation)} sets the 'hasAccessLocation' property of 'myDDI4_Machine' to a list of R6 AccessLocation class instances
#'
#'                              \code{myDDI4_Machine$hasAccessLocation} returns a list of of R6 : AccessLocation class instances}
#'   }
#'   \describe{
#'     \item{\code{function.}}{ A read/write active binding.
#'        The function of the machine
#'
#'                              \code{myDDI4_Machine$function. <- list(myExternalControlledVocabularyEntry)} sets the 'function.' property of 'myDDI4_Machine' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_Machine$function.} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{machineInterface}}{ A read/write active binding.
#'        Specified the machine interface. Supports the use of a controlled vocabulary.
#'
#'                              \code{myDDI4_Machine$machineInterface <- list(myExternalControlledVocabularyEntry)} sets the 'machineInterface' property of 'myDDI4_Machine' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_Machine$machineInterface} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{ownerOperatorContact}}{ A read/write active binding.
#'        Contact information for the owner/operator including location specification, address, URL, phone numbers, and other means of communication access. Sets of information can be repeated and date-stamped.
#'
#'                              \code{myDDI4_Machine$ownerOperatorContact <- list(myContactInformation)} sets the 'ownerOperatorContact' property of 'myDDI4_Machine' to a list of R6 ContactInformation class instances
#'
#'                              \code{myDDI4_Machine$ownerOperatorContact} returns a list of of R6 : ContactInformation class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Machine$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, typeOfMachine=NA, name=NA, hasAccessLocation=NA, function.=NA, machineInterface=NA, ownerOperatorContact=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Machine <- R6Class(classname="DDI4_Machine",
  inherit = DDI4_Agent,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, typeOfMachine=NA, name=NA, hasAccessLocation=NA, function.=NA, machineInterface=NA, ownerOperatorContact=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Machine may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(typeOfMachine)){
        private$.typeOfMachine <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfMachine , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(hasAccessLocation)){
        private$.hasAccessLocation <- returnR6ObjectIfValid(validClass="DDI4_AccessLocation", proposedObjectList=hasAccessLocation , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(function.)){
        private$.function. <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=function. , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(machineInterface)){
        private$.machineInterface <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=machineInterface , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(ownerOperatorContact)){
        private$.ownerOperatorContact <- returnR6ObjectIfValid(validClass="DDI4_ContactInformation", proposedObjectList=ownerOperatorContact , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 6 properties: 
    function. = function(val){
      if (!missing(val)){
        private$.function. <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.function.
      }
    },

    hasAccessLocation = function(val){
      if (!missing(val)){
        private$.hasAccessLocation <- returnR6ObjectIfValid(validClass="DDI4_AccessLocation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasAccessLocation
      }
    },

    machineInterface = function(val){
      if (!missing(val)){
        private$.machineInterface <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.machineInterface
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    ownerOperatorContact = function(val){
      if (!missing(val)){
        private$.ownerOperatorContact <- returnR6ObjectIfValid(validClass="DDI4_ContactInformation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.ownerOperatorContact
      }
    },

    typeOfMachine = function(val){
      if (!missing(val)){
        private$.typeOfMachine <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.typeOfMachine
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .typeOfMachine = NULL,
    .name = NULL,
    .hasAccessLocation = NULL,
    .function. = NULL,
    .machineInterface = NULL,
    .ownerOperatorContact = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        function. = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAccessLocation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_AccessLocation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAgentId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AgentId", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        image = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_PrivateImage", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        machineInterface = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        ownerOperatorContact = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ContactInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        typeOfMachine = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Machine


#' @title DDI4_Map  Describes the correspondence between concepts in a correspondence table related to one or more Statistical Classifications
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Map}{Map} 

#'
#'   Definition
#'
#'   ============
#'
#'   Describes the correspondence between concepts in a correspondence table related to one or more Statistical Classifications.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   A Map is the pairing of similar Concepts. Each Concept in the Map belongs to a different Collection. The collection of maps for all the Concepts in corresponding Collections is a Correspondence Table.
#'
#'   
#'
#'   A simple example might map the following 2 martial status category sets:
#'
#'   MS1 -
#'
#'   single
#'
#'   married
#'
#'   widowed
#'
#'   divorced
#'
#'   
#'
#'   MS2 -
#'
#'   single
#'
#'   married
#'
#'   
#'
#'   So, a correspondence table between these 2 category sets might look like this:
#'
#'   MS1                                               MS2
#'
#'   single                                             single
#'
#'   widowed                                          "
#'
#'   divorced                                          "
#'
#'   married                                            married
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Map
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Map$DdiUrn} returns the DDI URN of myDDI4_Map.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{validDates}}{ A read/write active binding.
#'        Date from which the Map became valid. The date must be defined if the Map belongs to a floating CorrespondenceTable. Date at which the Map became invalid. The date must be defined if the Map belongs to a floating Correspondence Table and is no longer valid.
#'
#'                              \code{myDDI4_Map$validDates <- list(myDateRange)} sets the 'validDates' property of 'myDDI4_Map' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_Map$validDates} returns a list of of R6 : DateRange class instances}
#'   }
#'   \describe{
#'     \item{\code{hasCorrespondenceType}}{ A read/write active binding.
#'        Type of correspondence in terms of commonalities and differences between two members.
#'
#'                              \code{myDDI4_Map$hasCorrespondenceType <- list(myCorrespondenceType)} sets the 'hasCorrespondenceType' property of 'myDDI4_Map' to a list of R6 CorrespondenceType class instances
#'
#'                              \code{myDDI4_Map$hasCorrespondenceType} returns a list of of R6 : CorrespondenceType class instances}
#'   }
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A display label for the OrderedMemberCorrespondence. May be expressed in multiple languages. Repeat for labels with different content, for example, labels with differing length limitations.
#'
#'                              \code{myDDI4_Map$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_Map' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_Map$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#'   \describe{
#'     \item{\code{usage}}{ A read/write active binding.
#'        Explanation of the ways in which some decision or object is employed. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_Map$usage <- list(myInternationalStructuredString)} sets the 'usage' property of 'myDDI4_Map' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Map$usage} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          Concept or any subtype of concepts
#'
#'                              \code{myDDI4_Map$source <- c(myConceptURN)} sets the association 'source' from 'myDDI4_Map' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_Map$source} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          Concept or any subtype of concept
#'
#'                              \code{myDDI4_Map$target <- c(myConceptURN)} sets the association 'target' from 'myDDI4_Map' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_Map$target} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Map$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, validDates=NA, hasCorrespondenceType=NA, displayLabel=NA, usage=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Map <- R6Class(classname="DDI4_Map",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, validDates=NA, hasCorrespondenceType=NA, displayLabel=NA, usage=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Map may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(validDates)){
        private$.validDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=validDates , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasCorrespondenceType)){
        private$.hasCorrespondenceType <- returnR6ObjectIfValid(validClass="DDI4_CorrespondenceType", proposedObjectList=hasCorrespondenceType , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(usage)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=usage , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=source , minimumCardinality=1, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=target , minimumCardinality=1, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 4 properties: 
    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.displayLabel
      }
    },

    hasCorrespondenceType = function(val){
      if (!missing(val)){
        private$.hasCorrespondenceType <- returnR6ObjectIfValid(validClass="DDI4_CorrespondenceType", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasCorrespondenceType
      }
    },

    usage = function(val){
      if (!missing(val)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.usage
      }
    },

    validDates = function(val){
      if (!missing(val)){
        private$.validDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.validDates
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .validDates = NULL,
    .hasCorrespondenceType = NULL,
    .displayLabel = NULL,
    .usage = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        displayLabel = c(
          min="0", 
          max="1", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasCorrespondenceType = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CorrespondenceType", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="1", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="1", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        usage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        validDates = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Map


#' @title DDI4_MeasureRole  A MeasureRole identifies one or more InstanceVariables as being measures within a ViewPoint
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/MeasureRole}{MeasureRole} 

#'
#'   Definition
#'
#'   ============
#'
#'   A MeasureRole identifies one or more InstanceVariables as being measures within a ViewPoint. A MeasureRole is a SimpleCollection of InstanceVariables acting in the MeasureRole.
#'
#'   
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A data record with four variables: "PersonId", "Systolic", "Diastolic", "Seated" might have a Viewpoint with 
#'
#'   PersonId defined as having the IdentifierRole
#'
#'   Systolic defined as having the MeasureRole
#'
#'   Diastolic defined as having the MeasureRole
#'
#'   Seated defined as having the AttributeRole
#'
#'   
#'
#'   PersonId, Systolic, Diastolic, Seated
#'
#'   123,122,20,yes
#'
#'   145,130,90,no
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   See the Viewpoint documentation for an in depth discussion of the uses of ViewpointRoles: http://lion.ddialliance.org/ddiobjects/viewpoint
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ViewpointRole}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_MeasureRole$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_MeasureRole <- R6Class(classname="DDI4_MeasureRole",
  inherit = DDI4_ViewpointRole,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_MeasureRole may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

    }
  ),

  active = list(
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InstanceVariableIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_MeasureRole


#' @title DDI4_MemberIndicator  Provides ability to declare an optional sequence or index order to a CollectionMember
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/MemberIndicator}{MemberIndicator} 

#'
#'   Definition
#'
#'   ============
#'
#'   Provides ability to declare an optional sequence or index order to a CollectionMember. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_MemberIndicator$DdiUrn} returns the DDI URN of myDDI4_MemberIndicator.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Index number expressed as an integer. The position of the member in an ordered array. Optional for unordered Collections.
#'
#'                              \code{myDDI4_MemberIndicator$index <- myinteger} sets the index property of 'myDDI4_MemberIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_MemberIndicator$index} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{member}}{ A read/write active binding.
#'          Member of the collection.
#'
#'                              \code{myDDI4_MemberIndicator$member <- c(myCollectionMemberURN)} sets the association 'member' from 'myDDI4_MemberIndicator' to a character vector of DdiUrns for R6 CollectionMember class instances
#'
#'                              \code{myDDI4_MemberIndicator$member} returns a character vector of of DdiUrns for R6  CollectionMember class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_MemberIndicator$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_MemberIndicator <- R6Class(classname="DDI4_MemberIndicator",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA,    member=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_MemberIndicator may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(member)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_CollectionMember", proposedUrnVector=member , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.index
      }
    },


# There are 1 class association(s):
    member = function(val){
      if (!missing(val)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_CollectionMember", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
       # invisible( private$.member)
      } else {
          private$.member
      }
    }

  ),

  private = list(
    .index = NULL,
    .member = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        index = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        member = c(
          min="1", 
          max="1", 
          datatype = "DDI4_CollectionMember", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_MemberIndicator


#' @title DDI4_MemberRelation  Defines one kind of relationship between one member (source) and possibly several members (target)
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/MemberRelation}{MemberRelation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Defines one kind of relationship between one member (source) and possibly several members (target).  The type of the relationship, a semantic, and totality definition may also be specified. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A member might have a ParentChild relation to several other members which is Anti-Reflexive, Anti-Symmetric, and Anti-Transitive. For a process this might have a semantic of "immediately follows". 
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   A MemberRelation functions like an adjacency list (https://en.wikipedia.org/wiki/Adjacency_list) in graph theory, describing the relationships of one Member to multiple other members. All of these relationships must have the same RelationSpecification, Totality, and semantic. 
#'
#'   These might not be the only relations from the source Member if there are other relations with different properties.
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_MemberRelation$DdiUrn} returns the DDI URN of myDDI4_MemberRelation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an enumeration list
#'
#'                              \code{myDDI4_MemberRelation$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_MemberRelation' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_MemberRelation$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provide a semantic that provides a context for the relationship using and External Controlled Vocabulary
#'
#'                              \code{myDDI4_MemberRelation$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_MemberRelation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_MemberRelation$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of its totality using an enumeration list.
#'
#'                              \code{myDDI4_MemberRelation$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_MemberRelation' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_MemberRelation$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          The first member of the relationship. 
#'
#'                              \code{myDDI4_MemberRelation$source <- c(myCollectionMemberURN)} sets the association 'source' from 'myDDI4_MemberRelation' to a character vector of DdiUrns for R6 CollectionMember class instances
#'
#'                              \code{myDDI4_MemberRelation$source} returns a character vector of of DdiUrns for R6  CollectionMember class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          Second member in the relationship. Notes this could be realized as a collection supporting tuple relationship.
#'
#'                              \code{myDDI4_MemberRelation$target <- c(myCollectionMemberURN)} sets the association 'target' from 'myDDI4_MemberRelation' to a character vector of DdiUrns for R6 CollectionMember class instances
#'
#'                              \code{myDDI4_MemberRelation$target} returns a character vector of of DdiUrns for R6  CollectionMember class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_MemberRelation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_MemberRelation <- R6Class(classname="DDI4_MemberRelation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_MemberRelation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_CollectionMember", proposedUrnVector=source , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_CollectionMember", proposedUrnVector=target , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_CollectionMember", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_CollectionMember", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionMember", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_CollectionMember", 
          structure = "DdiUrn vector"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_MemberRelation


#' @title DDI4_MetadataDrivenAction  MetadataDrivenActions are Acts in which their logic is defined by validation and transformation rules expressed in some standard rule language
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/MetadataDrivenAction}{MetadataDrivenAction} 

#'
#'   Definition
#'
#'   ============
#'
#'   MetadataDrivenActions are Acts in which their logic is defined by validation and transformation rules expressed in some standard rule language.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Some data management systems present users with a menu of actions they can use in a sequence to perform a specific task. The system engages the user in a dialog driven by an action template. The dialog completes the template which the system then uses to perform the action. The user doesn't write any code.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Whereas ComputationActions are used in statistical packages like SPSS, Stata, SAS and R to perform data management and data transformations, MetadataDrivenActions are used by data pipelines, e.g. ETL (Extract Transform Load) platforms, along with ComputationActions.
#'
#'   
#'
#'   In a data pipeline the user is presented with a menu of MetadataDrivenActions that are captured here in an external controlled vocabulary. Users enter into a dialog with the platform through which they customize the MetadataDrivenAction. The user writes no code. The dialog is saved.
#'
#'   
#'
#'   In the course of this dialog the user might specify a data source, specify which variables to keep or drop, rename variables, specify join specifics, create a value map between two variables and so forth.
#'
#'   
#'
#'   MetadataDrivenActions represent this dialog in some standard rule language, e.g. Structured Data Transformation Language (SDTL) , Validation and Transformation Language (VTL), etc.. In fact,  MetaDataDrivenAction can use VTL with certain "extensions". Together they form quasi-VTL. quasi-VTL in turn conforms to a known schema. Using this schema users can render the quasi-VTL as XHTML and record it in the MetadataDrivenAction quasiVTL property. quasiVTL supports structured text.
#'
#'   
#'
#'   Note that we would like NOT to create specific subtypes of MetadataDrivenAction -- one for each transformation. In fact there are a dozen or more such actions that will be captured in the MetadataDrivenAction types external controlled vocabulary. As an alternative, users can optionally write structured text based on a known schema. In this approach we are able to "evolve" SDTL as needed without changing our model. Instead, as our language "evolves", we one have to update its schema.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Act}}}
#'    \item{\code{\link{DDI4_WorkflowStep}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_MetadataDrivenAction$DdiUrn} returns the DDI URN of myDDI4_MetadataDrivenAction.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{activityDescription}}{ A read/write active binding.
#'        Describes the transformation activity
#'
#'                              \code{myDDI4_MetadataDrivenAction$activityDescription <- list(myInternationalStructuredString)} sets the 'activityDescription' property of 'myDDI4_MetadataDrivenAction' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_MetadataDrivenAction$activityDescription} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{typeOfMetadataDrivenAction}}{ A read/write active binding.
#'        Allows for the specification of a transformation type supporting the use of an external controlled vocabulary
#'
#'                              \code{myDDI4_MetadataDrivenAction$typeOfMetadataDrivenAction <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfMetadataDrivenAction' property of 'myDDI4_MetadataDrivenAction' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_MetadataDrivenAction$typeOfMetadataDrivenAction} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{quasiVTL}}{ A read/write active binding.
#'        A definition of the action in XHTML that conforms to the quasi-VTL schema. Each action has its own definition that a user optionally completes.
#'
#'                              \code{myDDI4_MetadataDrivenAction$quasiVTL <- list(myInternationalStructuredString)} sets the 'quasiVTL' property of 'myDDI4_MetadataDrivenAction' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_MetadataDrivenAction$quasiVTL} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_MetadataDrivenAction$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, activityDescription=NA, typeOfMetadataDrivenAction=NA, quasiVTL=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_MetadataDrivenAction <- R6Class(classname="DDI4_MetadataDrivenAction",
  inherit = DDI4_Act,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, activityDescription=NA, typeOfMetadataDrivenAction=NA, quasiVTL=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_MetadataDrivenAction may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(activityDescription)){
        private$.activityDescription <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=activityDescription , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(typeOfMetadataDrivenAction)){
        private$.typeOfMetadataDrivenAction <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfMetadataDrivenAction , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(quasiVTL)){
        private$.quasiVTL <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=quasiVTL , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    activityDescription = function(val){
      if (!missing(val)){
        private$.activityDescription <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.activityDescription
      }
    },

    quasiVTL = function(val){
      if (!missing(val)){
        private$.quasiVTL <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.quasiVTL
      }
    },

    typeOfMetadataDrivenAction = function(val){
      if (!missing(val)){
        private$.typeOfMetadataDrivenAction <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.typeOfMetadataDrivenAction
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .activityDescription = NULL,
    .typeOfMetadataDrivenAction = NULL,
    .quasiVTL = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        activityDescription = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInformationFlow = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPerformedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowService", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        quasiVTL = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        typeOfMetadataDrivenAction = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_MetadataDrivenAction


#' @title DDI4_Methodology  Methodology brings together the design, algorithm, and process as a logically structured approach to an activity such as sampling, weighting, harmonization, imputation, collection management, etc
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Methodology}{Methodology} 

#'
#'   Definition
#'
#'   ============
#'
#'   Methodology brings together the design, algorithm, and process as a logically structured approach to an activity such as sampling, weighting, harmonization, imputation, collection management, etc. A methodology in normally informed by earlier research and clarifies how earlier research methods were incorporated into the current work.
#'
#'   
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Methodology$DdiUrn} returns the DDI URN of myDDI4_Methodology.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human  understandable name (word, phrase, or  mnemonic) that reflects the ISO/IEC  11179-5 naming principles. If more than  one name is provided provide a context to  differentiate usage. 
#'
#'                              \code{myDDI4_Methodology$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_Methodology' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_Methodology$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{usage}}{ A read/write active binding.
#'        Explanation of the ways in which some  decision or object is employed. Supports  the use of multiple languages and  structured text.  
#'
#'                              \code{myDDI4_Methodology$usage <- list(myInternationalStructuredString)} sets the 'usage' property of 'myDDI4_Methodology' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Methodology$usage} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{rationale}}{ A read/write active binding.
#'        Explanation of the reasons some decision  was made or some object exists. Supports  the use of multiple languages and  structured text.  
#'
#'                              \code{myDDI4_Methodology$rationale <- list(myInternationalStructuredString)} sets the 'rationale' property of 'myDDI4_Methodology' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Methodology$rationale} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{overview}}{ A read/write active binding.
#'        Short natural language account of the  information obtained from the  combination of properties and  relationships associated with an object.  Supports the use of multiple languages  and structured text. 
#'
#'                              \code{myDDI4_Methodology$overview <- list(myInternationalStructuredString)} sets the 'overview' property of 'myDDI4_Methodology' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Methodology$overview} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasDesign}}{ A read/write active binding.
#'          Design used to form the Methodology
#'
#'                              \code{myDDI4_Methodology$hasDesign <- c(myDesignURN)} sets the association 'hasDesign' from 'myDDI4_Methodology' to a character vector of DdiUrns for R6 Design class instances
#'
#'                              \code{myDDI4_Methodology$hasDesign} returns a character vector of of DdiUrns for R6  Design class instances}
#'   }
#'   \describe{
#'     \item{\code{hasProcess}}{ A read/write active binding.
#'          Process used to implement the Methodology
#'
#'                              \code{myDDI4_Methodology$hasProcess <- c(myProcessURN)} sets the association 'hasProcess' from 'myDDI4_Methodology' to a character vector of DdiUrns for R6 Process class instances
#'
#'                              \code{myDDI4_Methodology$hasProcess} returns a character vector of of DdiUrns for R6  Process class instances}
#'   }
#'   \describe{
#'     \item{\code{isExpressedBy}}{ A read/write active binding.
#'          An algorithm that defines the Methodology in a generic way.
#'
#'                              \code{myDDI4_Methodology$isExpressedBy <- c(myAlgorithmURN)} sets the association 'isExpressedBy' from 'myDDI4_Methodology' to a character vector of DdiUrns for R6 Algorithm class instances
#'
#'                              \code{myDDI4_Methodology$isExpressedBy} returns a character vector of of DdiUrns for R6  Algorithm class instances}
#'   }
#'   \describe{
#'     \item{\code{componentMethodology}}{ A read/write active binding.
#'          A methodology which is a component part of this Methodology
#'
#'                              \code{myDDI4_Methodology$componentMethodology <- c(myMethodologyURN)} sets the association 'componentMethodology' from 'myDDI4_Methodology' to a character vector of DdiUrns for R6 Methodology class instances
#'
#'                              \code{myDDI4_Methodology$componentMethodology} returns a character vector of of DdiUrns for R6  Methodology class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Methodology$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, name=NA, usage=NA, rationale=NA, overview=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Methodology <- R6Class(classname="DDI4_Methodology",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, name=NA, usage=NA, rationale=NA, overview=NA,    hasDesign=NA,hasProcess=NA,isExpressedBy=NA,componentMethodology=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Methodology may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(usage)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=usage , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(rationale)){
        private$.rationale <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=rationale , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(overview)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=overview , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(hasDesign)){
        private$.hasDesign <- returnUrnVectorIfValid(validClass="DDI4_Design", proposedUrnVector=hasDesign , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasDesign, referencingAssociationName="hasDesign",  referencingUrn=thisUrn)  
      }
      if(!missing(hasProcess)){
        private$.hasProcess <- returnUrnVectorIfValid(validClass="DDI4_Process", proposedUrnVector=hasProcess , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasProcess, referencingAssociationName="hasProcess",  referencingUrn=thisUrn)  
      }
      if(!missing(isExpressedBy)){
        private$.isExpressedBy <- returnUrnVectorIfValid(validClass="DDI4_Algorithm", proposedUrnVector=isExpressedBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isExpressedBy, referencingAssociationName="isExpressedBy",  referencingUrn=thisUrn)  
      }
      if(!missing(componentMethodology)){
        private$.componentMethodology <- returnUrnVectorIfValid(validClass="DDI4_Methodology", proposedUrnVector=componentMethodology , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.componentMethodology, referencingAssociationName="componentMethodology",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 4 properties: 
    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    overview = function(val){
      if (!missing(val)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overview
      }
    },

    rationale = function(val){
      if (!missing(val)){
        private$.rationale <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.rationale
      }
    },

    usage = function(val){
      if (!missing(val)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.usage
      }
    },


# There are 4 class association(s):
    componentMethodology = function(val){
      if (!missing(val)){
        private$.componentMethodology <- returnUrnVectorIfValid(validClass="DDI4_Methodology", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.componentMethodology, referencingAssociationName="componentMethodology",  referencingUrn=thisUrn)  
       # invisible( private$.componentMethodology)
      } else {
          private$.componentMethodology
      }
    },

    hasDesign = function(val){
      if (!missing(val)){
        private$.hasDesign <- returnUrnVectorIfValid(validClass="DDI4_Design", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasDesign, referencingAssociationName="hasDesign",  referencingUrn=thisUrn)  
       # invisible( private$.hasDesign)
      } else {
          private$.hasDesign
      }
    },

    hasProcess = function(val){
      if (!missing(val)){
        private$.hasProcess <- returnUrnVectorIfValid(validClass="DDI4_Process", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasProcess, referencingAssociationName="hasProcess",  referencingUrn=thisUrn)  
       # invisible( private$.hasProcess)
      } else {
          private$.hasProcess
      }
    },

    isExpressedBy = function(val){
      if (!missing(val)){
        private$.isExpressedBy <- returnUrnVectorIfValid(validClass="DDI4_Algorithm", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isExpressedBy, referencingAssociationName="isExpressedBy",  referencingUrn=thisUrn)  
       # invisible( private$.isExpressedBy)
      } else {
          private$.isExpressedBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .name = NULL,
    .usage = NULL,
    .rationale = NULL,
    .overview = NULL,
    .hasDesign = NULL,
    .hasProcess = NULL,
    .isExpressedBy = NULL,
    .componentMethodology = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        componentMethodology = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Methodology", 
          structure = "DdiUrn vector"
        ),
        hasDesign = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Design", 
          structure = "DdiUrn vector"
        ),
        hasProcess = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Process", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isExpressedBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Algorithm", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        rationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_Methodology


#' @title DDI4_MethodologyOverview  High level, descriptive, human informative methodology statement used to describe the overall methodology, identify related design, algorithm, and process information
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/MethodologyOverview}{MethodologyOverview} 

#'
#'   Definition
#'
#'   ============
#'
#'   High level, descriptive, human informative methodology statement used to describe the overall methodology, identify related design, algorithm, and process information. A methodology is normally informed by earlier research and clarifies how earlier research methods were incorporated into the current work. 
#'
#'   
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   The target of the relationship from Study: "hasMethodology"
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   This would most commonly be used in a Codebook along with an AlgorithmOverview and a DesignOverview. Note that Process may be described in more detail than a high level overview.
#'
#'   Note that the algorithm may be implemented by multiple processes which are not limited to any specific type of Process. This can be constrained by the inclusion of only specific realizations of Process within a Functional View. Note that this MethodologyOverview can be used as a collective description of specific methodologies used by a Study or other broad set of metadata.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_MethodologyOverview$DdiUrn} returns the DDI URN of myDDI4_MethodologyOverview.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{subjectOfMethodology}}{ A read/write active binding.
#'        A term describing the subject of the Methodology. Supports the use of a controlled vocabulary
#'
#'                              \code{myDDI4_MethodologyOverview$subjectOfMethodology <- list(myExternalControlledVocabularyEntry)} sets the 'subjectOfMethodology' property of 'myDDI4_MethodologyOverview' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_MethodologyOverview$subjectOfMethodology} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided, provide a context to differentiate usage.
#'
#'                              \code{myDDI4_MethodologyOverview$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_MethodologyOverview' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_MethodologyOverview$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{usage}}{ A read/write active binding.
#'        Explanation of the ways in which some decision or object is employed. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_MethodologyOverview$usage <- list(myInternationalStructuredString)} sets the 'usage' property of 'myDDI4_MethodologyOverview' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_MethodologyOverview$usage} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{rationale}}{ A read/write active binding.
#'        Explanation of the reasons some decision was made or some object exists. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_MethodologyOverview$rationale <- list(myInternationalStructuredString)} sets the 'rationale' property of 'myDDI4_MethodologyOverview' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_MethodologyOverview$rationale} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{overview}}{ A read/write active binding.
#'        Short natural language account of the information obtained from the combination of properties and relationships associated with an object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_MethodologyOverview$overview <- list(myInternationalStructuredString)} sets the 'overview' property of 'myDDI4_MethodologyOverview' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_MethodologyOverview$overview} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isDescribedIn}}{ A read/write active binding.
#'          An external document or other source that provides descriptive information on the object.
#'
#'                              \code{myDDI4_MethodologyOverview$isDescribedIn <- c(myExternalMaterialURN)} sets the association 'isDescribedIn' from 'myDDI4_MethodologyOverview' to a character vector of DdiUrns for R6 ExternalMaterial class instances
#'
#'                              \code{myDDI4_MethodologyOverview$isDescribedIn} returns a character vector of of DdiUrns for R6  ExternalMaterial class instances}
#'   }
#'   \describe{
#'     \item{\code{hasDesign}}{ A read/write active binding.
#'          The design used by the methodology. A restriction of the Methdology class to constrain the target to DescribedDesign.
#'
#'                              \code{myDDI4_MethodologyOverview$hasDesign <- c(myDesignOverviewURN)} sets the association 'hasDesign' from 'myDDI4_MethodologyOverview' to a character vector of DdiUrns for R6 DesignOverview class instances
#'
#'                              \code{myDDI4_MethodologyOverview$hasDesign} returns a character vector of of DdiUrns for R6  DesignOverview class instances}
#'   }
#'   \describe{
#'     \item{\code{isExpressedBy}}{ A read/write active binding.
#'          The algorithm which expresses the methodology. A restriction of the Methdology class to constrain the target to DescribedAlgorithm.
#'
#'                              \code{myDDI4_MethodologyOverview$isExpressedBy <- c(myAlgorithmOverviewURN)} sets the association 'isExpressedBy' from 'myDDI4_MethodologyOverview' to a character vector of DdiUrns for R6 AlgorithmOverview class instances
#'
#'                              \code{myDDI4_MethodologyOverview$isExpressedBy} returns a character vector of of DdiUrns for R6  AlgorithmOverview class instances}
#'   }
#'   \describe{
#'     \item{\code{componentMethodology}}{ A read/write active binding.
#'          A methodology which is a component part of this methodology. The target may be any realized Methodology. This allows the DescribedMethodology to incorporate more explicit or detailed methodologies for different forms of use without replicating the descriptive content.
#'
#'                              \code{myDDI4_MethodologyOverview$componentMethodology <- c(myMethodologyOverviewURN)} sets the association 'componentMethodology' from 'myDDI4_MethodologyOverview' to a character vector of DdiUrns for R6 MethodologyOverview class instances
#'
#'                              \code{myDDI4_MethodologyOverview$componentMethodology} returns a character vector of of DdiUrns for R6  MethodologyOverview class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_MethodologyOverview$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, subjectOfMethodology=NA, name=NA, usage=NA, rationale=NA, overview=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_MethodologyOverview <- R6Class(classname="DDI4_MethodologyOverview",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, subjectOfMethodology=NA, name=NA, usage=NA, rationale=NA, overview=NA,    isDescribedIn=NA,hasDesign=NA,isExpressedBy=NA,componentMethodology=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_MethodologyOverview may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(subjectOfMethodology)){
        private$.subjectOfMethodology <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=subjectOfMethodology , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(usage)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=usage , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(rationale)){
        private$.rationale <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=rationale , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(overview)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=overview , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(isDescribedIn)){
        private$.isDescribedIn <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=isDescribedIn , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isDescribedIn, referencingAssociationName="isDescribedIn",  referencingUrn=thisUrn)  
      }
      if(!missing(hasDesign)){
        private$.hasDesign <- returnUrnVectorIfValid(validClass="DDI4_DesignOverview", proposedUrnVector=hasDesign , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasDesign, referencingAssociationName="hasDesign",  referencingUrn=thisUrn)  
      }
      if(!missing(isExpressedBy)){
        private$.isExpressedBy <- returnUrnVectorIfValid(validClass="DDI4_AlgorithmOverview", proposedUrnVector=isExpressedBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isExpressedBy, referencingAssociationName="isExpressedBy",  referencingUrn=thisUrn)  
      }
      if(!missing(componentMethodology)){
        private$.componentMethodology <- returnUrnVectorIfValid(validClass="DDI4_MethodologyOverview", proposedUrnVector=componentMethodology , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.componentMethodology, referencingAssociationName="componentMethodology",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 5 properties: 
    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    overview = function(val){
      if (!missing(val)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overview
      }
    },

    rationale = function(val){
      if (!missing(val)){
        private$.rationale <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.rationale
      }
    },

    subjectOfMethodology = function(val){
      if (!missing(val)){
        private$.subjectOfMethodology <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.subjectOfMethodology
      }
    },

    usage = function(val){
      if (!missing(val)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.usage
      }
    },


# There are 4 class association(s):
    componentMethodology = function(val){
      if (!missing(val)){
        private$.componentMethodology <- returnUrnVectorIfValid(validClass="DDI4_MethodologyOverview", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.componentMethodology, referencingAssociationName="componentMethodology",  referencingUrn=thisUrn)  
       # invisible( private$.componentMethodology)
      } else {
          private$.componentMethodology
      }
    },

    hasDesign = function(val){
      if (!missing(val)){
        private$.hasDesign <- returnUrnVectorIfValid(validClass="DDI4_DesignOverview", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasDesign, referencingAssociationName="hasDesign",  referencingUrn=thisUrn)  
       # invisible( private$.hasDesign)
      } else {
          private$.hasDesign
      }
    },

    isDescribedIn = function(val){
      if (!missing(val)){
        private$.isDescribedIn <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isDescribedIn, referencingAssociationName="isDescribedIn",  referencingUrn=thisUrn)  
       # invisible( private$.isDescribedIn)
      } else {
          private$.isDescribedIn
      }
    },

    isExpressedBy = function(val){
      if (!missing(val)){
        private$.isExpressedBy <- returnUrnVectorIfValid(validClass="DDI4_AlgorithmOverview", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isExpressedBy, referencingAssociationName="isExpressedBy",  referencingUrn=thisUrn)  
       # invisible( private$.isExpressedBy)
      } else {
          private$.isExpressedBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .subjectOfMethodology = NULL,
    .name = NULL,
    .usage = NULL,
    .rationale = NULL,
    .overview = NULL,
    .isDescribedIn = NULL,
    .hasDesign = NULL,
    .isExpressedBy = NULL,
    .componentMethodology = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        componentMethodology = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_MethodologyOverview", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasDesign = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_DesignOverview", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isDescribedIn = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        isExpressedBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AlgorithmOverview", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        rationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        subjectOfMethodology = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_MethodologyOverview


#' @title DDI4_NonIsoDateType  Used to preserve an historical date, formatted in a non-ISO fashion
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/NonIsoDateType}{NonIsoDateType} 

#'
#'   Definition
#'
#'   ============
#'
#'   Used to preserve an historical date, formatted in a non-ISO fashion.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:HistoricalDateType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_NonIsoDateType$DdiUrn} returns the DDI URN of myDDI4_NonIsoDateType.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{dateContent}}{ A read/write active binding.
#'        This is the date expressed in a non-ISO compliant structure. Primarily used to retain legacy content or to express non-Gregorian calender dates.
#'
#'                              \code{myDDI4_NonIsoDateType$dateContent <- mycharacter} sets the dateContent property of 'myDDI4_NonIsoDateType' to the vector of datatype:character
#'
#'                              \code{myDDI4_NonIsoDateType$dateContent} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{nonIsoDateFormat}}{ A read/write active binding.
#'        Indicate the structure of the date provided in NonISODate. For example if the NonISODate contained 4/1/2000 the Historical Date Format would be mm/dd/yyyy. The use of a controlled vocabulary is strongly recommended to support interoperability.
#'
#'                              \code{myDDI4_NonIsoDateType$nonIsoDateFormat <- list(myExternalControlledVocabularyEntry)} sets the 'nonIsoDateFormat' property of 'myDDI4_NonIsoDateType' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_NonIsoDateType$nonIsoDateFormat} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{calendar}}{ A read/write active binding.
#'        Specifies the type of calendar used (e.g., Gregorian, Julian, Jewish).
#'
#'                              \code{myDDI4_NonIsoDateType$calendar <- list(myExternalControlledVocabularyEntry)} sets the 'calendar' property of 'myDDI4_NonIsoDateType' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_NonIsoDateType$calendar} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_NonIsoDateType$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, dateContent=NA, nonIsoDateFormat=NA, calendar=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_NonIsoDateType <- R6Class(classname="DDI4_NonIsoDateType",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, dateContent=NA, nonIsoDateFormat=NA, calendar=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_NonIsoDateType may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(dateContent)){
        private$.dateContent <- returnVectorIfValid(validVectorClass="character", proposedVector=dateContent , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(nonIsoDateFormat)){
        private$.nonIsoDateFormat <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=nonIsoDateFormat , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(calendar)){
        private$.calendar <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=calendar , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    calendar = function(val){
      if (!missing(val)){
        private$.calendar <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.calendar
      }
    },

    dateContent = function(val){
      if (!missing(val)){
        private$.dateContent <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.dateContent
      }
    },

    nonIsoDateFormat = function(val){
      if (!missing(val)){
        private$.nonIsoDateFormat <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.nonIsoDateFormat
      }
    }


  ),

  private = list(
    .dateContent = NULL,
    .nonIsoDateFormat = NULL,
    .calendar = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        calendar = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        dateContent = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        nonIsoDateFormat = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_NonIsoDateType


#' @title DDI4_NumberRange  Structures a numeric range
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/NumberRange}{NumberRange} 

#'
#'   Definition
#'
#'   ============
#'
#'   Structures a numeric range. Low and High values are designated. The structure identifies Low values that should be treated as bottom coded (Stated value and bellow, High values that should be treated as top coded (stated value and higher), and provides a regular expression to further define the valid content of the range.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:NumberRangeType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_NumberRange$DdiUrn} returns the DDI URN of myDDI4_NumberRange.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A display label for the number range. May be expressed in multiple languages. Repeat for labels with different content, for example, labels with differing length limitations.
#'
#'                              \code{myDDI4_NumberRange$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_NumberRange' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_NumberRange$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#'   \describe{
#'     \item{\code{highCode}}{ A read/write active binding.
#'        High or top value on the range expressed as an xs:decimal within an isInclusive Boolean flag.
#'
#'                              \code{myDDI4_NumberRange$highCode <- list(myNumberRangeValue)} sets the 'highCode' property of 'myDDI4_NumberRange' to a list of R6 NumberRangeValue class instances
#'
#'                              \code{myDDI4_NumberRange$highCode} returns a list of of R6 : NumberRangeValue class instances}
#'   }
#'   \describe{
#'     \item{\code{highCodeDouble}}{ A read/write active binding.
#'        High or top value on the range expressed as an xs:double within an isInclusive Boolean flag.
#'
#'                              \code{myDDI4_NumberRange$highCodeDouble <- list(myDoubleNumberRangeValue)} sets the 'highCodeDouble' property of 'myDDI4_NumberRange' to a list of R6 DoubleNumberRangeValue class instances
#'
#'                              \code{myDDI4_NumberRange$highCodeDouble} returns a list of of R6 : DoubleNumberRangeValue class instances}
#'   }
#'   \describe{
#'     \item{\code{lowCode}}{ A read/write active binding.
#'        High or top value on the range expressed as an xs:decimal within an isInclusive Boolean flag.
#'
#'                              \code{myDDI4_NumberRange$lowCode <- list(myNumberRangeValue)} sets the 'lowCode' property of 'myDDI4_NumberRange' to a list of R6 NumberRangeValue class instances
#'
#'                              \code{myDDI4_NumberRange$lowCode} returns a list of of R6 : NumberRangeValue class instances}
#'   }
#'   \describe{
#'     \item{\code{lowCodeDouble}}{ A read/write active binding.
#'        Low or bottom value on the range expressed as an xs:double within an isInclusive Boolean flag.
#'
#'                              \code{myDDI4_NumberRange$lowCodeDouble <- list(myDoubleNumberRangeValue)} sets the 'lowCodeDouble' property of 'myDDI4_NumberRange' to a list of R6 DoubleNumberRangeValue class instances
#'
#'                              \code{myDDI4_NumberRange$lowCodeDouble} returns a list of of R6 : DoubleNumberRangeValue class instances}
#'   }
#'   \describe{
#'     \item{\code{regExp}}{ A read/write active binding.
#'        Regular expression defining the allowed syntax of the number.
#'
#'                              \code{myDDI4_NumberRange$regExp <- mycharacter} sets the regExp property of 'myDDI4_NumberRange' to the vector of datatype:character
#'
#'                              \code{myDDI4_NumberRange$regExp} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{isTopCoded}}{ A read/write active binding.
#'        Default value is False. The High or top code represents the value expressed and anything higher.
#'
#'                              \code{myDDI4_NumberRange$isTopCoded <- mylogical} sets the isTopCoded property of 'myDDI4_NumberRange' to the vector of datatype:logical
#'
#'                              \code{myDDI4_NumberRange$isTopCoded} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{isBottomCoded}}{ A read/write active binding.
#'        Default value is False. The Low or bottom code represents the value expressed and anything lower.
#'
#'                              \code{myDDI4_NumberRange$isBottomCoded <- mylogical} sets the isBottomCoded property of 'myDDI4_NumberRange' to the vector of datatype:logical
#'
#'                              \code{myDDI4_NumberRange$isBottomCoded} returns a vector of datatype: logical}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_NumberRange$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, displayLabel=NA, highCode=NA, highCodeDouble=NA, lowCode=NA, lowCodeDouble=NA, regExp=NA, isTopCoded=NA, isBottomCoded=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_NumberRange <- R6Class(classname="DDI4_NumberRange",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, displayLabel=NA, highCode=NA, highCodeDouble=NA, lowCode=NA, lowCodeDouble=NA, regExp=NA, isTopCoded=NA, isBottomCoded=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_NumberRange may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(highCode)){
        private$.highCode <- returnR6ObjectIfValid(validClass="DDI4_NumberRangeValue", proposedObjectList=highCode , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(highCodeDouble)){
        private$.highCodeDouble <- returnR6ObjectIfValid(validClass="DDI4_DoubleNumberRangeValue", proposedObjectList=highCodeDouble , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(lowCode)){
        private$.lowCode <- returnR6ObjectIfValid(validClass="DDI4_NumberRangeValue", proposedObjectList=lowCode , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(lowCodeDouble)){
        private$.lowCodeDouble <- returnR6ObjectIfValid(validClass="DDI4_DoubleNumberRangeValue", proposedObjectList=lowCodeDouble , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(regExp)){
        private$.regExp <- returnVectorIfValid(validVectorClass="character", proposedVector=regExp , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isTopCoded)){
        private$.isTopCoded <- returnVectorIfValid(validVectorClass="logical", proposedVector=isTopCoded , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(isBottomCoded)){
        private$.isBottomCoded <- returnVectorIfValid(validVectorClass="logical", proposedVector=isBottomCoded , minimumCardinality=1, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 8 properties: 
    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },

    highCode = function(val){
      if (!missing(val)){
        private$.highCode <- returnR6ObjectIfValid(validClass="DDI4_NumberRangeValue", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.highCode
      }
    },

    highCodeDouble = function(val){
      if (!missing(val)){
        private$.highCodeDouble <- returnR6ObjectIfValid(validClass="DDI4_DoubleNumberRangeValue", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.highCodeDouble
      }
    },

    isBottomCoded = function(val){
      if (!missing(val)){
        private$.isBottomCoded <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.isBottomCoded
      }
    },

    isTopCoded = function(val){
      if (!missing(val)){
        private$.isTopCoded <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.isTopCoded
      }
    },

    lowCode = function(val){
      if (!missing(val)){
        private$.lowCode <- returnR6ObjectIfValid(validClass="DDI4_NumberRangeValue", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.lowCode
      }
    },

    lowCodeDouble = function(val){
      if (!missing(val)){
        private$.lowCodeDouble <- returnR6ObjectIfValid(validClass="DDI4_DoubleNumberRangeValue", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.lowCodeDouble
      }
    },

    regExp = function(val){
      if (!missing(val)){
        private$.regExp <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.regExp
      }
    }


  ),

  private = list(
    .displayLabel = NULL,
    .highCode = NULL,
    .highCodeDouble = NULL,
    .lowCode = NULL,
    .lowCodeDouble = NULL,
    .regExp = NULL,
    .isTopCoded = NULL,
    .isBottomCoded = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        highCode = c(
          min="0", 
          max="1", 
          datatype = "DDI4_NumberRangeValue", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        highCodeDouble = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DoubleNumberRangeValue", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        isBottomCoded = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isTopCoded = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        lowCode = c(
          min="0", 
          max="1", 
          datatype = "DDI4_NumberRangeValue", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        lowCodeDouble = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DoubleNumberRangeValue", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        regExp = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_NumberRange


#' @title DDI4_NumberRangeValue  Describes a bounding value for a number range expressed as an xs:demical
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/NumberRangeValue}{NumberRangeValue} 

#'
#'   Definition
#'
#'   ============
#'
#'   Describes a bounding value for a number range expressed as an xs:demical.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:NumberRangeValueType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_NumberRangeValue$DdiUrn} returns the DDI URN of myDDI4_NumberRangeValue.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{isInclusive}}{ A read/write active binding.
#'        Indicates that the value is included in the range. Set to false if the range includes numbers up to but no including the designated value.
#'
#'                              \code{myDDI4_NumberRangeValue$isInclusive <- mylogical} sets the isInclusive property of 'myDDI4_NumberRangeValue' to the vector of datatype:logical
#'
#'                              \code{myDDI4_NumberRangeValue$isInclusive} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{decimalValue}}{ A read/write active binding.
#'        Bounding value expressed as an xs:double
#'
#'                              \code{myDDI4_NumberRangeValue$decimalValue <- mynumeric} sets the decimalValue property of 'myDDI4_NumberRangeValue' to the vector of datatype:numeric
#'
#'                              \code{myDDI4_NumberRangeValue$decimalValue} returns a vector of datatype: numeric}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_NumberRangeValue$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, isInclusive=NA, decimalValue=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_NumberRangeValue <- R6Class(classname="DDI4_NumberRangeValue",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, isInclusive=NA, decimalValue=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_NumberRangeValue may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(isInclusive)){
        private$.isInclusive <- returnVectorIfValid(validVectorClass="logical", proposedVector=isInclusive , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(decimalValue)){
        private$.decimalValue <- returnVectorIfValid(validVectorClass="numeric", proposedVector=decimalValue , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    decimalValue = function(val){
      if (!missing(val)){
        private$.decimalValue <- returnVectorIfValid(validVectorClass="numeric", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.decimalValue
      }
    },

    isInclusive = function(val){
      if (!missing(val)){
        private$.isInclusive <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isInclusive
      }
    }


  ),

  private = list(
    .isInclusive = NULL,
    .decimalValue = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        decimalValue = c(
          min="0", 
          max="1", 
          datatype = "numeric", 
          structure = "vector"
        ),
        isInclusive = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_NumberRangeValue


#' @title DDI4_NumericResponseDomain  A response domain capturing a numeric response (the intent is to analyze the response as a number) for a question
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/NumericResponseDomain}{NumericResponseDomain} 

#'
#'   Definition
#'
#'   ============
#'
#'   A response domain capturing a numeric response (the intent is to analyze the response as a number) for a question.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Systolic blood pressure level, as an integer from 0 to 200
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   d:NumericDomainType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ResponseDomain}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_NumericResponseDomain$DdiUrn} returns the DDI URN of myDDI4_NumericResponseDomain.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{numericTypeCode}}{ A read/write active binding.
#'        Identification of the numeric type such as integer, decimal, etc. supports the use of an external controlled vocabulary.
#'
#'                              \code{myDDI4_NumericResponseDomain$numericTypeCode <- list(myExternalControlledVocabularyEntry)} sets the 'numericTypeCode' property of 'myDDI4_NumericResponseDomain' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_NumericResponseDomain$numericTypeCode} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{unit}}{ A read/write active binding.
#'        An optional unit of measurement.  Examples include units from the International System of Units (SI) such as meter, seconds, mole, kilograms. Other examples include miles, tons, pints.
#'
#'                              \code{myDDI4_NumericResponseDomain$unit <- list(myExternalControlledVocabularyEntry)} sets the 'unit' property of 'myDDI4_NumericResponseDomain' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_NumericResponseDomain$unit} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{usesNumberRange}}{ A read/write active binding.
#'        Defines the valid number range or number values for the representation.
#'
#'                              \code{myDDI4_NumericResponseDomain$usesNumberRange <- list(myNumberRange)} sets the 'usesNumberRange' property of 'myDDI4_NumericResponseDomain' to a list of R6 NumberRange class instances
#'
#'                              \code{myDDI4_NumericResponseDomain$usesNumberRange} returns a list of of R6 : NumberRange class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_NumericResponseDomain$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, numericTypeCode=NA, unit=NA, usesNumberRange=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_NumericResponseDomain <- R6Class(classname="DDI4_NumericResponseDomain",
  inherit = DDI4_ResponseDomain,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, numericTypeCode=NA, unit=NA, usesNumberRange=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_NumericResponseDomain may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(numericTypeCode)){
        private$.numericTypeCode <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=numericTypeCode , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(unit)){
        private$.unit <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=unit , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(usesNumberRange)){
        private$.usesNumberRange <- returnR6ObjectIfValid(validClass="DDI4_NumberRange", proposedObjectList=usesNumberRange , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    numericTypeCode = function(val){
      if (!missing(val)){
        private$.numericTypeCode <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.numericTypeCode
      }
    },

    unit = function(val){
      if (!missing(val)){
        private$.unit <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.unit
      }
    },

    usesNumberRange = function(val){
      if (!missing(val)){
        private$.usesNumberRange <- returnR6ObjectIfValid(validClass="DDI4_NumberRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.usesNumberRange
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .numericTypeCode = NULL,
    .unit = NULL,
    .usesNumberRange = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="1", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        intendedRepresentation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RepresentedVariable", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        numericTypeCode = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        unit = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usesNumberRange = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_NumberRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_NumericResponseDomain


#' @title DDI4_ObjectName  A standard means of expressing a Name for a class object
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ObjectName}{ObjectName} 

#'
#'   Definition
#'
#'   ============
#'
#'   A standard means of expressing a Name for a class object.  A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   USE in Model: In general the property name should be "name" as it is the name of the class object which contains it. Use a specific name (i.e. xxxName) only when naming something other than the class object which contains it.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:NameType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ObjectName$DdiUrn} returns the DDI URN of myDDI4_ObjectName.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{content}}{ A read/write active binding.
#'        The expressed name of the object.
#'
#'                              \code{myDDI4_ObjectName$content <- mycharacter} sets the content property of 'myDDI4_ObjectName' to the vector of datatype:character
#'
#'                              \code{myDDI4_ObjectName$content} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{context}}{ A read/write active binding.
#'        A name may be specific to a particular context, i.e., a type of software, or a section of a registry. Identify the context related to the specified name. 
#'
#'                              \code{myDDI4_ObjectName$context <- list(myExternalControlledVocabularyEntry)} sets the 'context' property of 'myDDI4_ObjectName' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ObjectName$context} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ObjectName$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, content=NA, context=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ObjectName <- R6Class(classname="DDI4_ObjectName",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, content=NA, context=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ObjectName may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(content)){
        private$.content <- returnVectorIfValid(validVectorClass="character", proposedVector=content , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(context)){
        private$.context <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=context , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    content = function(val){
      if (!missing(val)){
        private$.content <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.content
      }
    },

    context = function(val){
      if (!missing(val)){
        private$.context <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.context
      }
    }


  ),

  private = list(
    .content = NULL,
    .context = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        content = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        context = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ObjectName


#' @title DDI4_Organization  A framework of authority designated to act toward some purpose
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Organization}{Organization} 

#'
#'   Definition
#'
#'   ============
#'
#'   A framework of authority designated to act toward some purpose.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   U.S. Census Bureau, University of Michigan/ISR, Norwegian Social Data Archive 
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   related to org:Organization which is described as "Represents a collection of people organized together into a community or other..."
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Agent}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Organization$DdiUrn} returns the DDI URN of myDDI4_Organization.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasOrganizationName}}{ A read/write active binding.
#'        Names by which the organization is known.
#'
#'                              \code{myDDI4_Organization$hasOrganizationName <- list(myOrganizationName)} sets the 'hasOrganizationName' property of 'myDDI4_Organization' to a list of R6 OrganizationName class instances
#'
#'                              \code{myDDI4_Organization$hasOrganizationName} returns a list of of R6 : OrganizationName class instances}
#'   }
#'   \describe{
#'     \item{\code{ddiId}}{ A read/write active binding.
#'        The agency identifier of the organization as registered at the DDI Alliance register.
#'
#'                              \code{myDDI4_Organization$ddiId <- mycharacter} sets the ddiId property of 'myDDI4_Organization' to the vector of datatype:character
#'
#'                              \code{myDDI4_Organization$ddiId} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{hasContactInformation}}{ A read/write active binding.
#'        Contact information for the organization including location specification, address, URL, phone numbers, and other means of communication access. Sets of information can be repeated and date-stamped.
#'
#'                              \code{myDDI4_Organization$hasContactInformation <- list(myContactInformation)} sets the 'hasContactInformation' property of 'myDDI4_Organization' to a list of R6 ContactInformation class instances
#'
#'                              \code{myDDI4_Organization$hasContactInformation} returns a list of of R6 : ContactInformation class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Organization$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasOrganizationName=NA, ddiId=NA, hasContactInformation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Organization <- R6Class(classname="DDI4_Organization",
  inherit = DDI4_Agent,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasOrganizationName=NA, ddiId=NA, hasContactInformation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Organization may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(hasOrganizationName)){
        private$.hasOrganizationName <- returnR6ObjectIfValid(validClass="DDI4_OrganizationName", proposedObjectList=hasOrganizationName , minimumCardinality=1, maximumCardinality=Inf)  
      }
      if(!missing(ddiId)){
        private$.ddiId <- returnVectorIfValid(validVectorClass="character", proposedVector=ddiId , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(hasContactInformation)){
        private$.hasContactInformation <- returnR6ObjectIfValid(validClass="DDI4_ContactInformation", proposedObjectList=hasContactInformation , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    ddiId = function(val){
      if (!missing(val)){
        private$.ddiId <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.ddiId
      }
    },

    hasContactInformation = function(val){
      if (!missing(val)){
        private$.hasContactInformation <- returnR6ObjectIfValid(validClass="DDI4_ContactInformation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasContactInformation
      }
    },

    hasOrganizationName = function(val){
      if (!missing(val)){
        private$.hasOrganizationName <- returnR6ObjectIfValid(validClass="DDI4_OrganizationName", proposedObjectList=val , minimumCardinality=1, maximumCardinality=Inf)  
      } else {
        private$.hasOrganizationName
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasOrganizationName = NULL,
    .ddiId = NULL,
    .hasContactInformation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        ddiId = c(
          min="0", 
          max="Inf", 
          datatype = "character", 
          structure = "vector"
        ),
        hasAgentId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AgentId", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasContactInformation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ContactInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOrganizationName = c(
          min="1", 
          max="Inf", 
          datatype = "DDI4_OrganizationName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        image = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_PrivateImage", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Organization


#' @title DDI4_OrganizationName  Names by which the organization is known
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/OrganizationName}{OrganizationName} 

#'
#'   Definition
#'
#'   ============
#'
#'   Names by which the organization is known. Use the attribute isFormal="true" to designate the legal or formal name of the Organization. The preferred name should be noted with the isPreferred attribute. Names may be typed with TypeOfOrganizationName to indicate their appropriate usage.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   a:OrganizationNameType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ObjectName}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_OrganizationName$DdiUrn} returns the DDI URN of myDDI4_OrganizationName.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{abbreviation}}{ A read/write active binding.
#'        An abbreviation or acronym for the name. This may be expressed in multiple languages. It is assumed that if only a single language is provided that it may be used in any of the other languages within which the name itself is expressed.
#'
#'                              \code{myDDI4_OrganizationName$abbreviation <- list(myInternationalString)} sets the 'abbreviation' property of 'myDDI4_OrganizationName' to a list of R6 InternationalString class instances
#'
#'                              \code{myDDI4_OrganizationName$abbreviation} returns a list of of R6 : InternationalString class instances}
#'   }
#'   \describe{
#'     \item{\code{typeOfOrganizationName}}{ A read/write active binding.
#'        The type of organization name provided. the use of a controlled vocabulary is strongly recommended. At minimum this should include, e.g. PreviousFormalName, Nickname (or CommonName), Other.
#'
#'                              \code{myDDI4_OrganizationName$typeOfOrganizationName <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfOrganizationName' property of 'myDDI4_OrganizationName' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_OrganizationName$typeOfOrganizationName} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{effectiveDates}}{ A read/write active binding.
#'        The time period for which this name is accurate and in use.
#'
#'                              \code{myDDI4_OrganizationName$effectiveDates <- list(myDateRange)} sets the 'effectiveDates' property of 'myDDI4_OrganizationName' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_OrganizationName$effectiveDates} returns a list of of R6 : DateRange class instances}
#'   }
#'   \describe{
#'     \item{\code{isFormal}}{ A read/write active binding.
#'        The legal or formal name of the organization should have the isFormal attribute set to true. To avoid confusion only one organization name should have the isFormal attribute set to true. Use the TypeOfOrganizationName to further differentiate the type and applied usage when multiple names are provided.
#'
#'                              \code{myDDI4_OrganizationName$isFormal <- mylogical} sets the isFormal property of 'myDDI4_OrganizationName' to the vector of datatype:logical
#'
#'                              \code{myDDI4_OrganizationName$isFormal} returns a vector of datatype: logical}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_OrganizationName$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, abbreviation=NA, typeOfOrganizationName=NA, effectiveDates=NA, isFormal=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_OrganizationName <- R6Class(classname="DDI4_OrganizationName",
  inherit = DDI4_ObjectName,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, abbreviation=NA, typeOfOrganizationName=NA, effectiveDates=NA, isFormal=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_OrganizationName may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(abbreviation)){
        private$.abbreviation <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=abbreviation , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(typeOfOrganizationName)){
        private$.typeOfOrganizationName <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfOrganizationName , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(effectiveDates)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=effectiveDates , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isFormal)){
        private$.isFormal <- returnVectorIfValid(validVectorClass="logical", proposedVector=isFormal , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    abbreviation = function(val){
      if (!missing(val)){
        private$.abbreviation <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.abbreviation
      }
    },

    effectiveDates = function(val){
      if (!missing(val)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.effectiveDates
      }
    },

    isFormal = function(val){
      if (!missing(val)){
        private$.isFormal <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isFormal
      }
    },

    typeOfOrganizationName = function(val){
      if (!missing(val)){
        private$.typeOfOrganizationName <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.typeOfOrganizationName
      }
    }


  ),

  private = list(
    .abbreviation = NULL,
    .typeOfOrganizationName = NULL,
    .effectiveDates = NULL,
    .isFormal = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        abbreviation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        content = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        context = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        effectiveDates = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        isFormal = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        typeOfOrganizationName = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_OrganizationName


#' @title DDI4_PairedExternalControlledVocabularyEntry  A tightly bound pair of items from an external controlled vocabulary
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/PairedExternalControlledVocabularyEntry}{PairedExternalControlledVocabularyEntry} 

#'
#'   Definition
#'
#'   ============
#'
#'   A tightly bound pair of items from an external controlled vocabulary. The extent property describes the extent to which the parent term applies for the specific case. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   When used to assign a role to an actor within a specific activity this term would express the degree of contribution. Contributor with Role=Editor and extent=Lead.
#'
#'   
#'
#'   Alternatively. the term might be a controlled vocabulary from a list of controlled vocabularies, e.g. the Generic Longitudinal Business Process Model (GLBPM) in a list that could include other business process model frameworks. In this context the extent becomes the name of a business process model task, e.g. "integrate data" from the GLBPM.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_PairedExternalControlledVocabularyEntry$DdiUrn} returns the DDI URN of myDDI4_PairedExternalControlledVocabularyEntry.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{term}}{ A read/write active binding.
#'        The term attributed to the parent class, for example the role of a Contributor.
#'
#'                              \code{myDDI4_PairedExternalControlledVocabularyEntry$term <- list(myExternalControlledVocabularyEntry)} sets the 'term' property of 'myDDI4_PairedExternalControlledVocabularyEntry' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_PairedExternalControlledVocabularyEntry$term} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{extent}}{ A read/write active binding.
#'        Describes the extent to which the parent term applies for the specific case using an external controlled vocabulary. When associated with a role from the CASRAI Contributor Roles Taxonomy an appropriate vocabulary should be specified as either ‘lead’, ‘equal’, or ‘supporting’.
#'
#'                              \code{myDDI4_PairedExternalControlledVocabularyEntry$extent <- list(myExternalControlledVocabularyEntry)} sets the 'extent' property of 'myDDI4_PairedExternalControlledVocabularyEntry' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_PairedExternalControlledVocabularyEntry$extent} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_PairedExternalControlledVocabularyEntry$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, term=NA, extent=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_PairedExternalControlledVocabularyEntry <- R6Class(classname="DDI4_PairedExternalControlledVocabularyEntry",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, term=NA, extent=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_PairedExternalControlledVocabularyEntry may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(term)){
        private$.term <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=term , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(extent)){
        private$.extent <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=extent , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    extent = function(val){
      if (!missing(val)){
        private$.extent <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.extent
      }
    },

    term = function(val){
      if (!missing(val)){
        private$.term <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.term
      }
    }


  ),

  private = list(
    .term = NULL,
    .extent = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        extent = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        term = c(
          min="1", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_PairedExternalControlledVocabularyEntry


#' @title DDI4_Parameter  An Input or Output to a Process Step defined by as a unique identifier
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Parameter}{Parameter} 

#'
#'   Definition
#'
#'   ============
#'
#'   An Input or Output to a Process Step defined by as a unique identifier. It may be a single value or an array. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A question might have dynamic text or, again, "fills". Depending on the gender of the subject, a question might say "he" or "she" and/or "him" or "her". A gender variable is passed into the question, and question code resolves the dynamic text.
#'
#'   A computation action may be expecting a numeric array of values [result of looping an "Age in years" question through every member of a household]. isArray = "true"; valueRepresentation would link to a SubstantiveValueDomain defining a numeric value with a possible valid range; and because the parameter occurs in the context of a computation action it is necessary to specify an alias (e.g. AGE). The alias links the parameter to the code in the computation.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   When used as an hasInputParameter it defines a value being used by Process Step as a direct input or as an alias value within an equation. When used as an hasOutputParameter it defines itself as the value of the variable, capture, or explicit value of the parent class or as a specific value in a computation output by assigning the related Alias and/or defining the item within an array. Providing a defaultValue ensures that when bound to another Parameter that some value will be passed or received. A Value Domain expressed as a SubstantiveValueDomain or a SentinalValueDomain may be designated to define the datatype and range( of the expected value.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Parameter$DdiUrn} returns the DDI URN of myDDI4_Parameter.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{alias}}{ A read/write active binding.
#'        If the content of the parameter is being used in a generic set of code as an alias for the value of an object in a formula (for example source code for a statistical program) enter that name here. This provides a link from the identified parameter to the alias in the code.
#'
#'                              \code{myDDI4_Parameter$alias <- mycharacter} sets the alias property of 'myDDI4_Parameter' to the vector of datatype:character
#'
#'                              \code{myDDI4_Parameter$alias} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{defaultValue}}{ A read/write active binding.
#'        Provides a default value for the parameter if there is no value provided by the object it is bound to or the process that was intended to produce a value.
#'
#'                              \code{myDDI4_Parameter$defaultValue <- list(myValueString)} sets the 'defaultValue' property of 'myDDI4_Parameter' to a list of R6 ValueString class instances
#'
#'                              \code{myDDI4_Parameter$defaultValue} returns a list of of R6 : ValueString class instances}
#'   }
#'   \describe{
#'     \item{\code{isArray}}{ A read/write active binding.
#'        If set to "true" indicates that the content of the parameter is a delimited array rather than a single object and should be processed as such.limitArrayIndex
#'
#'                              \code{myDDI4_Parameter$isArray <- mylogical} sets the isArray property of 'myDDI4_Parameter' to the vector of datatype:logical
#'
#'                              \code{myDDI4_Parameter$isArray} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{limitArrayIndex}}{ A read/write active binding.
#'        When the Parameter represents an array of items, this attribute specifies the index identification of the items within the zero-based array which should be treated as that associated with the parameter. If not specified, the full array is treated as the parameter.
#'
#'                              \code{myDDI4_Parameter$limitArrayIndex <- mycharacter} sets the limitArrayIndex property of 'myDDI4_Parameter' to the vector of datatype:character
#'
#'                              \code{myDDI4_Parameter$limitArrayIndex} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{agency}}{ A read/write active binding.
#'        This is the registered agency code with optional sub-agencies separated by dots. For example, diw.soep, ucl.qss, abs.essg.
#'
#'                              \code{myDDI4_Parameter$agency <- mycharacter} sets the agency property of 'myDDI4_Parameter' to the vector of datatype:character
#'
#'                              \code{myDDI4_Parameter$agency} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{id}}{ A read/write active binding.
#'        The ID of the object. This must conform to the allowed structure of the DDI Identifier and must be unique within the Agency.
#'
#'                              \code{myDDI4_Parameter$id <- mycharacter} sets the id property of 'myDDI4_Parameter' to the vector of datatype:character
#'
#'                              \code{myDDI4_Parameter$id} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{version}}{ A read/write active binding.
#'        The version number of the object. The version number is incremented whenever the non-administrative metadata contained by the object changes.
#'
#'                              \code{myDDI4_Parameter$version <- mycharacter} sets the version property of 'myDDI4_Parameter' to the vector of datatype:character
#'
#'                              \code{myDDI4_Parameter$version} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage. 
#'
#'                              \code{myDDI4_Parameter$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_Parameter' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_Parameter$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of the Agent Listing. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_Parameter$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_Parameter' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Parameter$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{valueRepresentation}}{ A read/write active binding.
#'          If the content of the parameter contains representational content, such as codes, provide the representation here. ValueDomain is the abstract with a number of subtype and may be replaced with any valid substitution for ValueDomain. Inclusion of the valueRepresentation is recommended if you will be sharing data with others as it provides information on the structure of what they can expect to receive when the parameter is processed.
#'
#'                              \code{myDDI4_Parameter$valueRepresentation <- c(myValueDomainURN)} sets the association 'valueRepresentation' from 'myDDI4_Parameter' to a character vector of DdiUrns for R6 ValueDomain class instances
#'
#'                              \code{myDDI4_Parameter$valueRepresentation} returns a character vector of of DdiUrns for R6  ValueDomain class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Parameter$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, alias=NA, defaultValue=NA, isArray=NA, limitArrayIndex=NA, agency=NA, id=NA, version=NA, name=NA, purpose=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Parameter <- R6Class(classname="DDI4_Parameter",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, alias=NA, defaultValue=NA, isArray=NA, limitArrayIndex=NA, agency=NA, id=NA, version=NA, name=NA, purpose=NA,    valueRepresentation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Parameter may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(alias)){
        private$.alias <- returnVectorIfValid(validVectorClass="character", proposedVector=alias , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(defaultValue)){
        private$.defaultValue <- returnR6ObjectIfValid(validClass="DDI4_ValueString", proposedObjectList=defaultValue , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isArray)){
        private$.isArray <- returnVectorIfValid(validVectorClass="logical", proposedVector=isArray , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(limitArrayIndex)){
        private$.limitArrayIndex <- returnVectorIfValid(validVectorClass="character", proposedVector=limitArrayIndex , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(agency)){
        private$.agency <- returnVectorIfValid(validVectorClass="character", proposedVector=agency , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.agency <- "example.org"
###          warning('Agency was missing. It has been set to "example.org" as a temporary value')
      }
      if(!missing(id)){
        private$.id <- returnVectorIfValid(validVectorClass="character", proposedVector=id , minimumCardinality=1, maximumCardinality=1)  
      } else {
         private$.id <- uuid::UUIDgenerate(TRUE)
###           warning(paste("Id was missing. It has been set to the unique random UUID: ",private$id, "You can change this with the set_id method."))
      }
      if(!missing(version)){
        private$.version <- returnVectorIfValid(validVectorClass="character", proposedVector=version , minimumCardinality=1, maximumCardinality=1)  
      } else {
         private$.version <- "1"
###           warning('Version was missing. It has been set to "1" as a temporary value')
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(valueRepresentation)){
        private$.valueRepresentation <- returnUrnVectorIfValid(validClass="DDI4_ValueDomain", proposedUrnVector=valueRepresentation , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.valueRepresentation, referencingAssociationName="valueRepresentation",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 9 properties: 
    agency = function(val){
      if (!missing(val)){
        if (private$.isPersistent) stop(" isPersistent is true for this object. The agency, id or version cannot be changed for a persistent object.") 
# this object exists deRegister it before changing its DdiUrn 
        deRegisterObject(private$.DdiUrn)  
        private$.agency <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
        private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        registerObject(DdiUrn=private$.DdiUrn, self)
      } else {
        private$.agency
      }
    },

    alias = function(val){
      if (!missing(val)){
        private$.alias <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.alias
      }
    },

    defaultValue = function(val){
      if (!missing(val)){
        private$.defaultValue <- returnR6ObjectIfValid(validClass="DDI4_ValueString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.defaultValue
      }
    },

    id = function(val){
      if (!missing(val)){
        if (private$.isPersistent) stop(" isPersistent is true for this object. The agency, id or version cannot be changed for a persistent object.") 
# this object exists deRegister it before changing its DdiUrn 
        deRegisterObject(private$.DdiUrn)  
        private$.id <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
        private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        registerObject(DdiUrn=private$.DdiUrn, self)
      } else {
        private$.id
      }
    },

    isArray = function(val){
      if (!missing(val)){
        private$.isArray <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isArray
      }
    },

    limitArrayIndex = function(val){
      if (!missing(val)){
        private$.limitArrayIndex <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.limitArrayIndex
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    version = function(val){
      if (!missing(val)){
        if (private$.isPersistent) stop(" isPersistent is true for this object. The agency, id or version cannot be changed for a persistent object.") 
# this object exists deRegister it before changing its DdiUrn 
        deRegisterObject(private$.DdiUrn)  
        private$.version <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
        private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        registerObject(DdiUrn=private$.DdiUrn, self)
      } else {
        private$.version
      }
    },


# There are 1 class association(s):
    valueRepresentation = function(val){
      if (!missing(val)){
        private$.valueRepresentation <- returnUrnVectorIfValid(validClass="DDI4_ValueDomain", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.valueRepresentation, referencingAssociationName="valueRepresentation",  referencingUrn=thisUrn)  
       # invisible( private$.valueRepresentation)
      } else {
          private$.valueRepresentation
      }
    }

  ),

  private = list(
    .alias = NULL,
    .defaultValue = NULL,
    .isArray = NULL,
    .limitArrayIndex = NULL,
    .agency = NULL,
    .id = NULL,
    .version = NULL,
    .name = NULL,
    .purpose = NULL,
    .valueRepresentation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        alias = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        defaultValue = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ValueString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isArray = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        limitArrayIndex = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        valueRepresentation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ValueDomain", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Parameter


#' @title DDI4_PhysicalDataSet  The information needed for understanding the physical structure of data coming from a file or other source
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/PhysicalDataSet}{PhysicalDataSet} 

#'
#'   Definition
#'
#'   ============
#'
#'   The information needed for understanding the physical structure of data coming from a file or other source. A StructureDescription also points to the Data Store it physically represents.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   The PhysicalDataProduct is the entry point for information about a file or other source. It includes information about the name of a file, the structure of segments in a file and the layout of segments.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Multiple styles of structural description are supported: including describing files as unit-record (UnitSegmentLayout) files; describing cubes; and describing event-history (spell) data.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   p:PhysicalDataProductType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_PhysicalDataSet$DdiUrn} returns the DDI URN of myDDI4_PhysicalDataSet.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{overview}}{ A read/write active binding.
#'        Short natural language account of the information obtained from the combination of properties and relationships associated with an object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_PhysicalDataSet$overview <- list(myInternationalStructuredString)} sets the 'overview' property of 'myDDI4_PhysicalDataSet' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_PhysicalDataSet$overview} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{physicalFileName}}{ A read/write active binding.
#'        Use when multiple physical segments are stored in a single file
#'
#'                              \code{myDDI4_PhysicalDataSet$physicalFileName <- mycharacter} sets the physicalFileName property of 'myDDI4_PhysicalDataSet' to the vector of datatype:character
#'
#'                              \code{myDDI4_PhysicalDataSet$physicalFileName} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{numberOfSegments}}{ A read/write active binding.
#'        The number of segments in a Physical Data Product
#'
#'                              \code{myDDI4_PhysicalDataSet$numberOfSegments <- myinteger} sets the numberOfSegments property of 'myDDI4_PhysicalDataSet' to the vector of datatype:integer
#'
#'                              \code{myDDI4_PhysicalDataSet$numberOfSegments} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        A physical description of the data store encompasses a description of its gross record structure. This structure consists of one or more PhysicalRecordSegments and their relationships. Allows for the identification of the member and optionally provides an index for the member within an ordered array 
#'
#'                              \code{myDDI4_PhysicalDataSet$contains <- list(myPhysicalRecordSegmentIndicator)} sets the 'contains' property of 'myDDI4_PhysicalDataSet' to a list of R6 PhysicalRecordSegmentIndicator class instances
#'
#'                              \code{myDDI4_PhysicalDataSet$contains} returns a list of of R6 : PhysicalRecordSegmentIndicator class instances}
#'   }
#'   \describe{
#'     \item{\code{isOrdered}}{ A read/write active binding.
#'        If members are ordered set to true, if unordered set to false.
#'
#'                              \code{myDDI4_PhysicalDataSet$isOrdered <- mylogical} sets the isOrdered property of 'myDDI4_PhysicalDataSet' to the vector of datatype:logical
#'
#'                              \code{myDDI4_PhysicalDataSet$isOrdered} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{type}}{ A read/write active binding.
#'        Whether the dataset is a bag or a set: a bag is a collection with duplicates allowed, a set is a collection without duplicates. 
#'
#'                              \code{myDDI4_PhysicalDataSet$type <- list(myCollectionType)} sets the 'type' property of 'myDDI4_PhysicalDataSet' to a list of R6 CollectionType class instances
#'
#'                              \code{myDDI4_PhysicalDataSet$type} returns a list of of R6 : CollectionType class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        An explanation of the purpose of the physical dataset
#'
#'                              \code{myDDI4_PhysicalDataSet$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_PhysicalDataSet' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_PhysicalDataSet$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{formatsDataStore}}{ A read/write active binding.
#'          Data Store physically represented by the Structure Description. 
#'
#'                              \code{myDDI4_PhysicalDataSet$formatsDataStore <- c(myDataStoreURN)} sets the association 'formatsDataStore' from 'myDDI4_PhysicalDataSet' to a character vector of DdiUrns for R6 DataStore class instances
#'
#'                              \code{myDDI4_PhysicalDataSet$formatsDataStore} returns a character vector of of DdiUrns for R6  DataStore class instances}
#'   }
#'   \describe{
#'     \item{\code{isStructuredBy}}{ A read/write active binding.
#'          Description of a complex structure for the Collection. A collection may be structured in more than one way for different uses.
#'
#'                              \code{myDDI4_PhysicalDataSet$isStructuredBy <- c(myPhysicalOrderRelationStructureURN)} sets the association 'isStructuredBy' from 'myDDI4_PhysicalDataSet' to a character vector of DdiUrns for R6 PhysicalOrderRelationStructure class instances
#'
#'                              \code{myDDI4_PhysicalDataSet$isStructuredBy} returns a character vector of of DdiUrns for R6  PhysicalOrderRelationStructure class instances}
#'   }
#'   \describe{
#'     \item{\code{hasVariableStatistics}}{ A read/write active binding.
#'          Variable Statistics related to the Physical Data Set. May be summary or category level statistics.
#'
#'                              \code{myDDI4_PhysicalDataSet$hasVariableStatistics <- c(myVariableStatisticsURN)} sets the association 'hasVariableStatistics' from 'myDDI4_PhysicalDataSet' to a character vector of DdiUrns for R6 VariableStatistics class instances
#'
#'                              \code{myDDI4_PhysicalDataSet$hasVariableStatistics} returns a character vector of of DdiUrns for R6  VariableStatistics class instances}
#'   }
#'   \describe{
#'     \item{\code{definingConcept}}{ A read/write active binding.
#'          The conceptual basis for the collection of members.
#'
#'                              \code{myDDI4_PhysicalDataSet$definingConcept <- c(myConceptURN)} sets the association 'definingConcept' from 'myDDI4_PhysicalDataSet' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_PhysicalDataSet$definingConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_PhysicalDataSet$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, overview=NA, physicalFileName=NA, numberOfSegments=NA, contains=NA, isOrdered=NA, type=NA, purpose=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_PhysicalDataSet <- R6Class(classname="DDI4_PhysicalDataSet",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, overview=NA, physicalFileName=NA, numberOfSegments=NA, contains=NA, isOrdered=NA, type=NA, purpose=NA,    formatsDataStore=NA,isStructuredBy=NA,hasVariableStatistics=NA,definingConcept=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_PhysicalDataSet may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(overview)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=overview , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(physicalFileName)){
        private$.physicalFileName <- returnVectorIfValid(validVectorClass="character", proposedVector=physicalFileName , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(numberOfSegments)){
        private$.numberOfSegments <- returnVectorIfValid(validVectorClass="integer", proposedVector=numberOfSegments , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_PhysicalRecordSegmentIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(isOrdered)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=isOrdered , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(type)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=type , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(formatsDataStore)){
        private$.formatsDataStore <- returnUrnVectorIfValid(validClass="DDI4_DataStore", proposedUrnVector=formatsDataStore , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.formatsDataStore, referencingAssociationName="formatsDataStore",  referencingUrn=thisUrn)  
      }
      if(!missing(isStructuredBy)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_PhysicalOrderRelationStructure", proposedUrnVector=isStructuredBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
      }
      if(!missing(hasVariableStatistics)){
        private$.hasVariableStatistics <- returnUrnVectorIfValid(validClass="DDI4_VariableStatistics", proposedUrnVector=hasVariableStatistics , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasVariableStatistics, referencingAssociationName="hasVariableStatistics",  referencingUrn=thisUrn)  
      }
      if(!missing(definingConcept)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=definingConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 7 properties: 
    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_PhysicalRecordSegmentIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },

    isOrdered = function(val){
      if (!missing(val)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isOrdered
      }
    },

    numberOfSegments = function(val){
      if (!missing(val)){
        private$.numberOfSegments <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.numberOfSegments
      }
    },

    overview = function(val){
      if (!missing(val)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overview
      }
    },

    physicalFileName = function(val){
      if (!missing(val)){
        private$.physicalFileName <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.physicalFileName
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    type = function(val){
      if (!missing(val)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.type
      }
    },


# There are 4 class association(s):
    definingConcept = function(val){
      if (!missing(val)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
       # invisible( private$.definingConcept)
      } else {
          private$.definingConcept
      }
    },

    formatsDataStore = function(val){
      if (!missing(val)){
        private$.formatsDataStore <- returnUrnVectorIfValid(validClass="DDI4_DataStore", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.formatsDataStore, referencingAssociationName="formatsDataStore",  referencingUrn=thisUrn)  
       # invisible( private$.formatsDataStore)
      } else {
          private$.formatsDataStore
      }
    },

    hasVariableStatistics = function(val){
      if (!missing(val)){
        private$.hasVariableStatistics <- returnUrnVectorIfValid(validClass="DDI4_VariableStatistics", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasVariableStatistics, referencingAssociationName="hasVariableStatistics",  referencingUrn=thisUrn)  
       # invisible( private$.hasVariableStatistics)
      } else {
          private$.hasVariableStatistics
      }
    },

    isStructuredBy = function(val){
      if (!missing(val)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_PhysicalOrderRelationStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
       # invisible( private$.isStructuredBy)
      } else {
          private$.isStructuredBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .overview = NULL,
    .physicalFileName = NULL,
    .numberOfSegments = NULL,
    .contains = NULL,
    .isOrdered = NULL,
    .type = NULL,
    .purpose = NULL,
    .formatsDataStore = NULL,
    .isStructuredBy = NULL,
    .hasVariableStatistics = NULL,
    .definingConcept = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_PhysicalRecordSegmentIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        formatsDataStore = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_DataStore", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasVariableStatistics = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_VariableStatistics", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isStructuredBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_PhysicalOrderRelationStructure", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        numberOfSegments = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        physicalFileName = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_PhysicalDataSet


#' @title DDI4_PhysicalLayoutRelationStructure  A realization of RelationStructure that is used to describe the sequence of Value Mappings in a Physical Layout
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/PhysicalLayoutRelationStructure}{PhysicalLayoutRelationStructure} 

#'
#'   Definition
#'
#'   ============
#'
#'   A realization of RelationStructure that is used to describe the sequence of Value Mappings in a Physical Layout. This can be more complex than a simple sequence. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   The W3C Tabular Data on the Web specification allows for a list datatype. In the example below there are three top level InstanceVariables
#'
#'   PersonID – the person identifier
#'
#'   AgeYr – age in year
#'
#'   BpSys_Dia – blood pressure (a list containing Systolic and Diastolic values)
#'
#'   
#'
#'   There are two variables at a secondary level of the hierarchy
#'
#'   Systolic – the systolic pressure
#'
#'   Diastolic – the diastolic pressure
#'
#'   
#'
#'   The delimited file below uses the comma to separate "columns" and forward slash to separate elements of a blood pressure list.
#'
#'   
#'
#'   PersonID, AgeYr, BpSys_Dia
#'
#'   1,22,119/67
#'
#'   2,68,122/70
#'
#'   
#'
#'   The PhysicalRelationStructure in this case would describe a BpSys_Dia  list variable as containing an ordered sequence of the Systolic and Diastolic InstanceVariables.
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_PhysicalLayoutRelationStructure$DdiUrn} returns the DDI URN of myDDI4_PhysicalLayoutRelationStructure.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{criteria}}{ A read/write active binding.
#'        Intensional definition of the order criteria (e.g. alphabetical, numerical, increasing, decreasing, etc.)
#'
#'                              \code{myDDI4_PhysicalLayoutRelationStructure$criteria <- list(myInternationalStructuredString)} sets the 'criteria' property of 'myDDI4_PhysicalLayoutRelationStructure' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_PhysicalLayoutRelationStructure$criteria} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an ennumeration list.  Use if all relations within this relation structure are of the same specification. 
#'
#'                              \code{myDDI4_PhysicalLayoutRelationStructure$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_PhysicalLayoutRelationStructure' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_PhysicalLayoutRelationStructure$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_PhysicalLayoutRelationStructure$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_PhysicalLayoutRelationStructure' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_PhysicalLayoutRelationStructure$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_PhysicalLayoutRelationStructure$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_PhysicalLayoutRelationStructure' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_PhysicalLayoutRelationStructure$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMemberRelation}}{ A read/write active binding.
#'        Specifics of relations between InstanceVariables in the PhysicalLayout
#'
#'                              \code{myDDI4_PhysicalLayoutRelationStructure$hasMemberRelation <- list(myValueMappingRelation)} sets the 'hasMemberRelation' property of 'myDDI4_PhysicalLayoutRelationStructure' to a list of R6 ValueMappingRelation class instances
#'
#'                              \code{myDDI4_PhysicalLayoutRelationStructure$hasMemberRelation} returns a list of of R6 : ValueMappingRelation class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_PhysicalLayoutRelationStructure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, criteria=NA, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_PhysicalLayoutRelationStructure <- R6Class(classname="DDI4_PhysicalLayoutRelationStructure",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, criteria=NA, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_PhysicalLayoutRelationStructure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(criteria)){
        private$.criteria <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=criteria , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasMemberRelation)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_ValueMappingRelation", proposedObjectList=hasMemberRelation , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 5 properties: 
    criteria = function(val){
      if (!missing(val)){
        private$.criteria <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.criteria
      }
    },

    hasMemberRelation = function(val){
      if (!missing(val)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_ValueMappingRelation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasMemberRelation
      }
    },

    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .criteria = NULL,
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .hasMemberRelation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        criteria = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasMemberRelation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ValueMappingRelation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasRelationSpecification = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_PhysicalLayoutRelationStructure


#' @title DDI4_PhysicalOrderRelationStructure  PhysicalStructureOrder orders thePhysicalRecordSegments  which map to a LogicalRecord
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/PhysicalOrderRelationStructure}{PhysicalOrderRelationStructure} 

#'
#'   Definition
#'
#'   ============
#'
#'   PhysicalStructureOrder orders thePhysicalRecordSegments  which map to a LogicalRecord.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   The same LogicalRecordLayout may be the sourceMember in several adjacency lists. This can happen when PhysicalRecordSegments are also population specific. In this instance each adjacency list associated with a LogicalRecordLayout is associated with a different population.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_PhysicalOrderRelationStructure$DdiUrn} returns the DDI URN of myDDI4_PhysicalOrderRelationStructure.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an enumeration list.  Use if all relations within this relation structure are of the same specification.
#'
#'                              \code{myDDI4_PhysicalOrderRelationStructure$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_PhysicalOrderRelationStructure' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_PhysicalOrderRelationStructure$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_PhysicalOrderRelationStructure$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_PhysicalOrderRelationStructure' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_PhysicalOrderRelationStructure$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_PhysicalOrderRelationStructure$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_PhysicalOrderRelationStructure' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_PhysicalOrderRelationStructure$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMemberRelation}}{ A read/write active binding.
#'        Member relations that comprise the relation structure
#'
#'                              \code{myDDI4_PhysicalOrderRelationStructure$hasMemberRelation <- list(myPhysicalRecordSegmentRelation)} sets the 'hasMemberRelation' property of 'myDDI4_PhysicalOrderRelationStructure' to a list of R6 PhysicalRecordSegmentRelation class instances
#'
#'                              \code{myDDI4_PhysicalOrderRelationStructure$hasMemberRelation} returns a list of of R6 : PhysicalRecordSegmentRelation class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_PhysicalOrderRelationStructure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_PhysicalOrderRelationStructure <- R6Class(classname="DDI4_PhysicalOrderRelationStructure",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_PhysicalOrderRelationStructure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasMemberRelation)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_PhysicalRecordSegmentRelation", proposedObjectList=hasMemberRelation , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    hasMemberRelation = function(val){
      if (!missing(val)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_PhysicalRecordSegmentRelation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasMemberRelation
      }
    },

    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .hasMemberRelation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasMemberRelation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_PhysicalRecordSegmentRelation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasRelationSpecification = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_PhysicalOrderRelationStructure


#' @title DDI4_PhysicalRecordSegment  A description of each physical storage segment required to completely cover the logical record
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/PhysicalRecordSegment}{PhysicalRecordSegment} 

#'
#'   Definition
#'
#'   ============
#'
#'   A description of each physical storage segment required to completely cover the logical record. A logical record may be stored in one or more segments housed hierarchically in a single file or in separate data files. All logical records have at least one segment.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   The file below has four InstanceVariables: PersonId, SegmentId, AgeYr, HeightCm. The data for each person (identified by PersonId) is recorded in two segments (identified by SegmentId), "a" and "b". AgeYr is on physical segment a, and HeightCm is on segment b. These are the same data as described in the UnitSegmentLayout documentation.
#'
#'   1 a  22
#'
#'   1 b 183
#'
#'   2 a 45
#'
#'   2 b 175
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   p:PhysicalRecordSegmentType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_PhysicalRecordSegment$DdiUrn} returns the DDI URN of myDDI4_PhysicalRecordSegment.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{physicalFileName}}{ A read/write active binding.
#'        Use when each physical segment is stored in its own file.
#'
#'                              \code{myDDI4_PhysicalRecordSegment$physicalFileName <- mycharacter} sets the physicalFileName property of 'myDDI4_PhysicalRecordSegment' to the vector of datatype:character
#'
#'                              \code{myDDI4_PhysicalRecordSegment$physicalFileName} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        A PhysicalRecordSegment has a sequence of DataPoints. Think of these as the cells in the row of a (spreadsheet) table.
#'
#'                              \code{myDDI4_PhysicalRecordSegment$contains <- list(myDataPointIndicator)} sets the 'contains' property of 'myDDI4_PhysicalRecordSegment' to a list of R6 DataPointIndicator class instances
#'
#'                              \code{myDDI4_PhysicalRecordSegment$contains} returns a list of of R6 : DataPointIndicator class instances}
#'   }
#'   \describe{
#'     \item{\code{isOrdered}}{ A read/write active binding.
#'        The DataPointIndicators for this PhysicalRecordSegment have index values that indicate the order of the DataPoints.
#'
#'                              \code{myDDI4_PhysicalRecordSegment$isOrdered <- mylogical} sets the isOrdered property of 'myDDI4_PhysicalRecordSegment' to the vector of datatype:logical
#'
#'                              \code{myDDI4_PhysicalRecordSegment$isOrdered} returns a vector of datatype: logical}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasPhysicalLayout}}{ A read/write active binding.
#'          A PhysicalRecordSegment is associated with a one or more reusable PhysicalLayouts 
#'
#'                              \code{myDDI4_PhysicalRecordSegment$hasPhysicalLayout <- c(myPhysicalSegmentLayoutURN)} sets the association 'hasPhysicalLayout' from 'myDDI4_PhysicalRecordSegment' to a character vector of DdiUrns for R6 PhysicalSegmentLayout class instances
#'
#'                              \code{myDDI4_PhysicalRecordSegment$hasPhysicalLayout} returns a character vector of of DdiUrns for R6  PhysicalSegmentLayout class instances}
#'   }
#'   \describe{
#'     \item{\code{representsPopulation}}{ A read/write active binding.
#'          A record segment may represent a specific population or sub-population within a larger set of segments. Allows for the identification of this filter for membership in the segment.
#'
#'                              \code{myDDI4_PhysicalRecordSegment$representsPopulation <- c(myPopulationURN)} sets the association 'representsPopulation' from 'myDDI4_PhysicalRecordSegment' to a character vector of DdiUrns for R6 Population class instances
#'
#'                              \code{myDDI4_PhysicalRecordSegment$representsPopulation} returns a character vector of of DdiUrns for R6  Population class instances}
#'   }
#'   \describe{
#'     \item{\code{mapTo}}{ A read/write active binding.
#'          Every DataRecord has at o..n PhysicalRecordSegments, It is possible to describe a PhysicalDataProduct and its RecordRecordSegment(s) without reference to a DataRecord
#'
#'                              \code{myDDI4_PhysicalRecordSegment$mapTo <- c(myLogicalRecordURN)} sets the association 'mapTo' from 'myDDI4_PhysicalRecordSegment' to a character vector of DdiUrns for R6 LogicalRecord class instances
#'
#'                              \code{myDDI4_PhysicalRecordSegment$mapTo} returns a character vector of of DdiUrns for R6  LogicalRecord class instances}
#'   }
#'   \describe{
#'     \item{\code{isStructuredBy}}{ A read/write active binding.
#'          There may be cases where there is a more complex structure to a sequence of DataPoints. A cell containing a list, for example, might be considered to have nested DataPoints that are the elements of the list.
#'
#'                              \code{myDDI4_PhysicalRecordSegment$isStructuredBy <- c(myDataPointRelationStructureURN)} sets the association 'isStructuredBy' from 'myDDI4_PhysicalRecordSegment' to a character vector of DdiUrns for R6 DataPointRelationStructure class instances
#'
#'                              \code{myDDI4_PhysicalRecordSegment$isStructuredBy} returns a character vector of of DdiUrns for R6  DataPointRelationStructure class instances}
#'   }
#'   \describe{
#'     \item{\code{definingConcept}}{ A read/write active binding.
#'          The conceptual basis for the collection of members.
#'
#'                              \code{myDDI4_PhysicalRecordSegment$definingConcept <- c(myConceptURN)} sets the association 'definingConcept' from 'myDDI4_PhysicalRecordSegment' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_PhysicalRecordSegment$definingConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_PhysicalRecordSegment$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, physicalFileName=NA, contains=NA, isOrdered=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_PhysicalRecordSegment <- R6Class(classname="DDI4_PhysicalRecordSegment",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, physicalFileName=NA, contains=NA, isOrdered=NA,    hasPhysicalLayout=NA,representsPopulation=NA,mapTo=NA,isStructuredBy=NA,definingConcept=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_PhysicalRecordSegment may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(physicalFileName)){
        private$.physicalFileName <- returnVectorIfValid(validVectorClass="character", proposedVector=physicalFileName , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_DataPointIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(isOrdered)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=isOrdered , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(hasPhysicalLayout)){
        private$.hasPhysicalLayout <- returnUrnVectorIfValid(validClass="DDI4_PhysicalSegmentLayout", proposedUrnVector=hasPhysicalLayout , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasPhysicalLayout, referencingAssociationName="hasPhysicalLayout",  referencingUrn=thisUrn)  
      }
      if(!missing(representsPopulation)){
        private$.representsPopulation <- returnUrnVectorIfValid(validClass="DDI4_Population", proposedUrnVector=representsPopulation , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.representsPopulation, referencingAssociationName="representsPopulation",  referencingUrn=thisUrn)  
      }
      if(!missing(mapTo)){
        private$.mapTo <- returnUrnVectorIfValid(validClass="DDI4_LogicalRecord", proposedUrnVector=mapTo , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.mapTo, referencingAssociationName="mapTo",  referencingUrn=thisUrn)  
      }
      if(!missing(isStructuredBy)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_DataPointRelationStructure", proposedUrnVector=isStructuredBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
      }
      if(!missing(definingConcept)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=definingConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_DataPointIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },

    isOrdered = function(val){
      if (!missing(val)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isOrdered
      }
    },

    physicalFileName = function(val){
      if (!missing(val)){
        private$.physicalFileName <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.physicalFileName
      }
    },


# There are 5 class association(s):
    definingConcept = function(val){
      if (!missing(val)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
       # invisible( private$.definingConcept)
      } else {
          private$.definingConcept
      }
    },

    hasPhysicalLayout = function(val){
      if (!missing(val)){
        private$.hasPhysicalLayout <- returnUrnVectorIfValid(validClass="DDI4_PhysicalSegmentLayout", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasPhysicalLayout, referencingAssociationName="hasPhysicalLayout",  referencingUrn=thisUrn)  
       # invisible( private$.hasPhysicalLayout)
      } else {
          private$.hasPhysicalLayout
      }
    },

    isStructuredBy = function(val){
      if (!missing(val)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_DataPointRelationStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
       # invisible( private$.isStructuredBy)
      } else {
          private$.isStructuredBy
      }
    },

    mapTo = function(val){
      if (!missing(val)){
        private$.mapTo <- returnUrnVectorIfValid(validClass="DDI4_LogicalRecord", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.mapTo, referencingAssociationName="mapTo",  referencingUrn=thisUrn)  
       # invisible( private$.mapTo)
      } else {
          private$.mapTo
      }
    },

    representsPopulation = function(val){
      if (!missing(val)){
        private$.representsPopulation <- returnUrnVectorIfValid(validClass="DDI4_Population", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.representsPopulation, referencingAssociationName="representsPopulation",  referencingUrn=thisUrn)  
       # invisible( private$.representsPopulation)
      } else {
          private$.representsPopulation
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .physicalFileName = NULL,
    .contains = NULL,
    .isOrdered = NULL,
    .hasPhysicalLayout = NULL,
    .representsPopulation = NULL,
    .mapTo = NULL,
    .isStructuredBy = NULL,
    .definingConcept = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_DataPointIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasPhysicalLayout = c(
          min="0", 
          max="1", 
          datatype = "DDI4_PhysicalSegmentLayout", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isStructuredBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_DataPointRelationStructure", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        mapTo = c(
          min="0", 
          max="1", 
          datatype = "DDI4_LogicalRecord", 
          structure = "DdiUrn vector"
        ),
        physicalFileName = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        representsPopulation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Population", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_PhysicalRecordSegment


#' @title DDI4_PhysicalRecordSegmentIndicator  Member Indicator for use with member type PhysicalRecordSegment
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/PhysicalRecordSegmentIndicator}{PhysicalRecordSegmentIndicator} 

#'
#'   Definition
#'
#'   ============
#'
#'   Member Indicator for use with member type PhysicalRecordSegment
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_PhysicalRecordSegmentIndicator$DdiUrn} returns the DDI URN of myDDI4_PhysicalRecordSegmentIndicator.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Index value of member in an ordered array
#'
#'                              \code{myDDI4_PhysicalRecordSegmentIndicator$index <- myinteger} sets the index property of 'myDDI4_PhysicalRecordSegmentIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_PhysicalRecordSegmentIndicator$index} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{member}}{ A read/write active binding.
#'          Restricts member target class to PhysicalRecordSegment
#'
#'                              \code{myDDI4_PhysicalRecordSegmentIndicator$member <- c(myPhysicalRecordSegmentURN)} sets the association 'member' from 'myDDI4_PhysicalRecordSegmentIndicator' to a character vector of DdiUrns for R6 PhysicalRecordSegment class instances
#'
#'                              \code{myDDI4_PhysicalRecordSegmentIndicator$member} returns a character vector of of DdiUrns for R6  PhysicalRecordSegment class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_PhysicalRecordSegmentIndicator$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_PhysicalRecordSegmentIndicator <- R6Class(classname="DDI4_PhysicalRecordSegmentIndicator",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA,    member=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_PhysicalRecordSegmentIndicator may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(member)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_PhysicalRecordSegment", proposedUrnVector=member , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.index
      }
    },


# There are 1 class association(s):
    member = function(val){
      if (!missing(val)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_PhysicalRecordSegment", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
       # invisible( private$.member)
      } else {
          private$.member
      }
    }

  ),

  private = list(
    .index = NULL,
    .member = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        index = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        member = c(
          min="1", 
          max="1", 
          datatype = "DDI4_PhysicalRecordSegment", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_PhysicalRecordSegmentIndicator


#' @title DDI4_PhysicalRecordSegmentRelation  Defines structured relationship between PhysicalRecordSegments
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/PhysicalRecordSegmentRelation}{PhysicalRecordSegmentRelation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Defines structured relationship between PhysicalRecordSegments.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_PhysicalRecordSegmentRelation$DdiUrn} returns the DDI URN of myDDI4_PhysicalRecordSegmentRelation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an ennumeration list
#'
#'                              \code{myDDI4_PhysicalRecordSegmentRelation$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_PhysicalRecordSegmentRelation' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_PhysicalRecordSegmentRelation$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_PhysicalRecordSegmentRelation$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_PhysicalRecordSegmentRelation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_PhysicalRecordSegmentRelation$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_PhysicalRecordSegmentRelation$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_PhysicalRecordSegmentRelation' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_PhysicalRecordSegmentRelation$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          first member in the relationship. Constrained to a single source
#'
#'                              \code{myDDI4_PhysicalRecordSegmentRelation$source <- c(myPhysicalRecordSegmentURN)} sets the association 'source' from 'myDDI4_PhysicalRecordSegmentRelation' to a character vector of DdiUrns for R6 PhysicalRecordSegment class instances
#'
#'                              \code{myDDI4_PhysicalRecordSegmentRelation$source} returns a character vector of of DdiUrns for R6  PhysicalRecordSegment class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          second member in the relationship
#'
#'                              \code{myDDI4_PhysicalRecordSegmentRelation$target <- c(myPhysicalRecordSegmentURN)} sets the association 'target' from 'myDDI4_PhysicalRecordSegmentRelation' to a character vector of DdiUrns for R6 PhysicalRecordSegment class instances
#'
#'                              \code{myDDI4_PhysicalRecordSegmentRelation$target} returns a character vector of of DdiUrns for R6  PhysicalRecordSegment class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_PhysicalRecordSegmentRelation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_PhysicalRecordSegmentRelation <- R6Class(classname="DDI4_PhysicalRecordSegmentRelation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_PhysicalRecordSegmentRelation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_PhysicalRecordSegment", proposedUrnVector=source , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_PhysicalRecordSegment", proposedUrnVector=target , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_PhysicalRecordSegment", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_PhysicalRecordSegment", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="0", 
          max="1", 
          datatype = "DDI4_PhysicalRecordSegment", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_PhysicalRecordSegment", 
          structure = "DdiUrn vector"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_PhysicalRecordSegmentRelation


#' @title DDI4_PhysicalSegmentLayout  The PhysicalSegmentLayout is an abstract class used as an extension point in the description of the different layout styles of data structure description
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/PhysicalSegmentLayout}{PhysicalSegmentLayout} 

#'
#'   Definition
#'
#'   ============
#'
#'   The PhysicalSegmentLayout is an abstract class used as an extension point in the description of the different layout styles of data structure description.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Examples include UnitSegmentLayouts, event data layouts, and cube layouts (e.g. summary data).
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   A PhysicalLayout is a physical description (e.g. UnitSegmentLayout) of the associated Logical Record Layout consisting of a Collection of Value Mappings describing the physical interrelationship of each related ValueMapping and associated Instance Variable.
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_PhysicalSegmentLayout$DdiUrn} returns the DDI URN of myDDI4_PhysicalSegmentLayout.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{isDelimited}}{ A read/write active binding.
#'        Indicates that the data are in a delimited format. If not set to true, is FixedWitdh must be set to true. 
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$isDelimited <- mylogical} sets the isDelimited property of 'myDDI4_PhysicalSegmentLayout' to the vector of datatype:logical
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$isDelimited} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{delimiter}}{ A read/write active binding.
#'        The Delimiting character in the data. Must be used if isDelimited is true. "The separator between cells, set by the delimiter property of a dialect description. The default is ,." see https://www.w3.org/TR/tabular-data-model/#encoding  From https://www.w3.org/TR/tabular-metadata/  5.9 Dialect  delimiter: “An atomic property that sets the delimiter flag to the single provided value, which MUST be a string. The default is ",".” From https://www.w3.org/TR/tabular-data-model/ 8 “The separator between cells, set by the delimiter property of a dialect description. The default is ,.” From http://specs.frictionlessdata.io/csv-dialect/ delimiter:  “specifies a one-character string to use as the field separator. Default = ,” 
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$delimiter <- mycharacter} sets the delimiter property of 'myDDI4_PhysicalSegmentLayout' to the vector of datatype:character
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$delimiter} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{isFixedWidth}}{ A read/write active binding.
#'        Set to true if the file is fixed-width. If true, isDelimited must be set to false.
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$isFixedWidth <- mylogical} sets the isFixedWidth property of 'myDDI4_PhysicalSegmentLayout' to the vector of datatype:logical
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$isFixedWidth} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{escapeCharacter}}{ A read/write active binding.
#'        "The string that is used to escape the quote character within escaped cells, or null"  see https://www.w3.org/TR/tabular-data-model/#encoding From https://www.w3.org/TR/tabular-metadata/  5.9 Dialect   “ doupleQuote: A boolean atomic property that, if true, sets the escape character flag to ". If false, to \\\\. The default is true.” From http://specs.frictionlessdata.io/csv-dialect/ doubleQuote: “controls the handling of quotes inside fields. If true, two consecutive quotes should be interpreted as one. Default = true” 
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$escapeCharacter <- mycharacter} sets the escapeCharacter property of 'myDDI4_PhysicalSegmentLayout' to the vector of datatype:character
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$escapeCharacter} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{lineTerminator}}{ A read/write active binding.
#'        "The strings that can be used at the end of a row, set by the lineTerminators property of a dialect description. The default is [CRLF, LF]."  see https://www.w3.org/TR/tabular-data-model/#encoding From https://www.w3.org/TR/tabular-metadata/  5.9 Dialect  lineTerminators: “An atomic property that sets the line terminators flag to either an array containing the single provided string value, or the provided array. The default is ["\\\\r\\\\n", "\\\\n"].” From http://specs.frictionlessdata.io/csv-dialect/ lineTerminator: “specifies the character sequence which should terminate rows. Default = \\\\r\\\\n” 
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$lineTerminator <- mycharacter} sets the lineTerminator property of 'myDDI4_PhysicalSegmentLayout' to the vector of datatype:character
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$lineTerminator} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{quoteCharacter}}{ A read/write active binding.
#'        The string that is used around escaped cells, or null, set by the quoteChar property of a dialect description. The default is ". see https://www.w3.org/TR/tabular-data-model/#encoding From https://www.w3.org/TR/tabular-metadata/  5.9 Dialect  quoteChar: “An atomic property that sets the quote character flag to the single provided value, which MUST be a string or null. If the value is null, the escape character flag is also set to null. The default is '"'.” From http://specs.frictionlessdata.io/csv-dialect/ quoteChar: “specifies a one-character string to use as the quoting character. Default = "” 
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$quoteCharacter <- mycharacter} sets the quoteCharacter property of 'myDDI4_PhysicalSegmentLayout' to the vector of datatype:character
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$quoteCharacter} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{commentPrefix}}{ A read/write active binding.
#'        A string used to indicate that an input line is a comment, a string which precedes a comment in the data file.  From https://www.w3.org/TR/tabular-metadata/  5.9 Dialect  commentPrefix: “An atomic property that sets the comment prefix flag to the single provided value, which MUST be a string. The default is "#".”
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$commentPrefix <- mycharacter} sets the commentPrefix property of 'myDDI4_PhysicalSegmentLayout' to the vector of datatype:character
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$commentPrefix} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{encoding}}{ A read/write active binding.
#'        The character encoding of the represented data. From https://www.w3.org/TR/tabular-metadata/ 5.9 Dialect: encoding: “An atomic property that sets the encoding flag to the single provided string value, which MUST be a defined in [encoding]. The default is "utf-8".”     From https://www.w3.org/TR/tabular-metadata/ 7.2 Encoding:  “CSV files should be encoded using UTF-8, and should be in Unicode Normal Form C as defined in [UAX15]. If a CSV file is not encoded using UTF-8, the encoding should be specified through the charset parameter in the Content-Type header:” 
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$encoding <- list(myExternalControlledVocabularyEntry)} sets the 'encoding' property of 'myDDI4_PhysicalSegmentLayout' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$encoding} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{hasHeader}}{ A read/write active binding.
#'        True if the file contains a header containing column names. From https://www.w3.org/TR/tabular-metadata/  5.9 Dialect  header: “A boolean atomic property that, if true, sets the header row count flag to 1, and if false to 0, unless headerRowCount is provided, in which case the value provided for the header property is ignored. The default is true.” From http://specs.frictionlessdata.io/csv-dialect/ header: “indicates whether the file includes a header row. If true the first row in the file is a header row, not data. Default = true”  
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$hasHeader <- mylogical} sets the hasHeader property of 'myDDI4_PhysicalSegmentLayout' to the vector of datatype:logical
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$hasHeader} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{headerRowCount}}{ A read/write active binding.
#'        The number of lines in the header From https://www.w3.org/TR/tabular-metadata/  5.9 Dialect  headerRowCount: “A numeric atomic property that sets the header row count flag to the single provided value, which MUST be a non-negative integer. The default is 1.”  
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$headerRowCount <- myinteger} sets the headerRowCount property of 'myDDI4_PhysicalSegmentLayout' to the vector of datatype:integer
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$headerRowCount} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{skipBlankRows}}{ A read/write active binding.
#'        If True, blank rows are ignored. From https://www.w3.org/TR/tabular-metadata/  5.9 Dialect  skipBlankRows: “A boolean atomic property that sets the skip blank rows flag to the single provided boolean value. The default is false.”
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$skipBlankRows <- mylogical} sets the skipBlankRows property of 'myDDI4_PhysicalSegmentLayout' to the vector of datatype:logical
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$skipBlankRows} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{skipDataColumns}}{ A read/write active binding.
#'        The number of columns to skip at the beginning of the row. From https://www.w3.org/TR/tabular-metadata/  5.9 Dialect  skipColumns: “A numeric atomic property that sets the skip columns flag to the single provided numeric value, which MUST be a non-negative integer. The default is 0.” See also https://www.w3.org/TR/tabular-metadata/ 8 skip columns: “The number of columns to skip at the beginning of each row, set by the skipColumns property of a dialect description. The default is 0. A value other than 0 will mean that the source numbers of columns will be different from their numbers.” 
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$skipDataColumns <- myinteger} sets the skipDataColumns property of 'myDDI4_PhysicalSegmentLayout' to the vector of datatype:integer
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$skipDataColumns} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{skipInitialSpace}}{ A read/write active binding.
#'        If true skip spaces at the beginning of a line or following a delimiter. From https://www.w3.org/TR/tabular-metadata/  5.9 Dialect  skipInitialSpace: “A boolean atomic property that, if true, sets the trim flag to "start" and if false, to false. If the trim property is provided, the skipInitialSpace property is ignored. The default is false.” From http://specs.frictionlessdata.io/csv-dialect/ skipInitialSpace: “specifies how to interpret whitespace which immediately follows a delimiter; if false, it means that whitespace immediately after a delimiter should be treated as part of the following field. Default = true” 
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$skipInitialSpace <- mylogical} sets the skipInitialSpace property of 'myDDI4_PhysicalSegmentLayout' to the vector of datatype:logical
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$skipInitialSpace} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{skipRows}}{ A read/write active binding.
#'        Number of input rows to skip preceding the header or data From https://www.w3.org/TR/tabular-metadata/  5.9 Dialect  skipRows: “A numeric atomic property that sets the skip rows flag to the single provided numeric value, which MUST be a non-negative integer. The default is 0.” From https://www.w3.org/TR/tabular-metadata/ 8  skip rows: “The number of rows to skip at the beginning of the file, before a header row or tabular data, set by the skipRows property of a dialect description. The default is 0. A value greater than 0 will mean that the source numbers of rows will be different from their numbers.” 
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$skipRows <- myinteger} sets the skipRows property of 'myDDI4_PhysicalSegmentLayout' to the vector of datatype:integer
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$skipRows} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{trim}}{ A read/write active binding.
#'        Specifies which spaces to remove from a data value (start, end, both, neither) From https://www.w3.org/TR/tabular-metadata/  5.9 Dialect  trim: “An atomic property that, if the boolean true, sets the trim flag to true and if the boolean false to false. If the value provided is a string, sets the trim flag to the provided value, which MUST be one of "true", "false", "start", or "end". The default is true.” 
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$trim <- list(myTrimValues)} sets the 'trim' property of 'myDDI4_PhysicalSegmentLayout' to a list of R6 TrimValues class instances
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$trim} returns a list of of R6 : TrimValues class instances}
#'   }
#'   \describe{
#'     \item{\code{nullSequence}}{ A read/write active binding.
#'        A string indicating a null value. From https://www.w3.org/TR/tabular-metadata/ 4.3 “null — the string or strings which cause the value of cells having string value matching any of these values to be null.” From Inherited 5.7 null: “An atomic property giving the string or strings used for null values within the data. If the string value of the cell is equal to any one of these values, the cell value is null. See Parsing Cells in [tabular-data-model] for more details. If not specified, the default for the null property is the empty string "". The value of this property becomes the null annotation for the described column.” From http://specs.frictionlessdata.io/csv-dialect/ nullSequence: “specifies the null sequence (for example \\\\N). Not set by default” 
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$nullSequence <- mycharacter} sets the nullSequence property of 'myDDI4_PhysicalSegmentLayout' to the vector of datatype:character
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$nullSequence} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{headerIsCaseSensitive}}{ A read/write active binding.
#'        If True, the case of the labels in the header is significant. From http://specs.frictionlessdata.io/csv-dialect/ caseSensitiveHeader: “indicates that case in the header is meaningful. For example, columns CAT and Cat should not be equated. Default = false”
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$headerIsCaseSensitive <- mylogical} sets the headerIsCaseSensitive property of 'myDDI4_PhysicalSegmentLayout' to the vector of datatype:logical
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$headerIsCaseSensitive} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{arrayBase}}{ A read/write active binding.
#'        Number cells, rows, columns starting from this value. This is a DDI notion allowing numbering of columns, rows, etc. from either 0 or 1. W3C appears to just use 1. From https://www.w3.org/TR/tabular-data-model/ 4.3 “number — the position of the column amongst the columns for the associated table, starting from 1.” 
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$arrayBase <- myinteger} sets the arrayBase property of 'myDDI4_PhysicalSegmentLayout' to the vector of datatype:integer
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$arrayBase} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{treatConsecutiveDelimitersAsOne}}{ A read/write active binding.
#'        If True, consecutive delimiters are treated the same way as a single delimiter, if false consiecutive delimiters indicate a missing value. From PHDD consecutiveDelimitersAsOne: “Indicates how consecutive delimiters should be handed by the software. Equivalent element in DDI 3.2: p:Delimiter/@treatConsecutiveDelimiterAsOne DDI 3.2 Documentation: http://www.ddialliance.org/Specification/DDI-Lifecycle/3.2/XMLSchema/FieldLevelDocumentation/schemas/reusable_xsd/complexTypes/DelimiterType.html#a6” 
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$treatConsecutiveDelimitersAsOne <- mylogical} sets the treatConsecutiveDelimitersAsOne property of 'myDDI4_PhysicalSegmentLayout' to the vector of datatype:logical
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$treatConsecutiveDelimitersAsOne} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{overview}}{ A read/write active binding.
#'        Short natural language account of the physical layout obtained from the combination of associated  properties and relationships. Supports the use of multiple languages and structured text.  
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$overview <- list(myInternationalStructuredString)} sets the 'overview' property of 'myDDI4_PhysicalSegmentLayout' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$overview} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{tableDirection}}{ A read/write active binding.
#'        Indicates the direction in which columns are arranged in each row. From https://www.w3.org/TR/tabular-metadata/  5.3.2 tableDirection: “An atomic property that MUST have a single string value that is one of "rtl", "ltr", or "auto". Indicates whether the tables in the group should be displayed with the first column on the right, on the left, or based on the first character in the table that has a specific direction. The value of this property becomes the value of the table direction annotation for all the tables in the table group. See Bidirectional Tables in [tabular-data-model] for details. The default value for this property is "auto".”  
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$tableDirection <- list(myTableDirectionValues)} sets the 'tableDirection' property of 'myDDI4_PhysicalSegmentLayout' to a list of R6 TableDirectionValues class instances
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$tableDirection} returns a list of of R6 : TableDirectionValues class instances}
#'   }
#'   \describe{
#'     \item{\code{textDirection}}{ A read/write active binding.
#'        Indicates the reading order of text within cells. From https://www.w3.org/TR/tabular-metadata/ Inherited 5.7 textDirection: “An atomic property that MUST have a single string value that is one of "ltr", "rtl", "auto" or "inherit" (the default). Indicates whether the text within cells should be displayed as left-to-right text (ltr), as right-to-left text (rtl), according to the content of the cell (auto) or in the direction inherited from the table direction annotation of the table. The value of this property determines the text direction annotation for the column, and the text direction annotation for the cells within that column: if the value is inherit then the value of the text direction annotation is the value of the table direction annotation on the table, otherwise it is the value of this property. See Bidirectional Tables in [tabular-data-model] for details.”
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$textDirection <- list(myTextDirectionValues)} sets the 'textDirection' property of 'myDDI4_PhysicalSegmentLayout' to a list of R6 TextDirectionValues class instances
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$textDirection} returns a list of of R6 : TextDirectionValues class instances}
#'   }
#'   \describe{
#'     \item{\code{type}}{ A read/write active binding.
#'          Whether the collection is a bag or a set: a bag is a collection with duplicates allowed, a set is a collection without duplicates.
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$type <- list(myCollectionType)} sets the 'type' property of 'myDDI4_PhysicalSegmentLayout' to a list of R6 CollectionType class instances
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$type} returns a list of of R6 : CollectionType class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage.
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_PhysicalSegmentLayout' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'          Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_PhysicalSegmentLayout' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        The PhysicalLayout contains a description of the physical representation of each InstanceVariable in a ValueMapping. This relationship fills the role of "Contains" from the Collection pattern. Allows for the identification of the member and optionally provides an index for the member within an ordered array
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$contains <- list(myValueMappingIndicator)} sets the 'contains' property of 'myDDI4_PhysicalSegmentLayout' to a list of R6 ValueMappingIndicator class instances
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$contains} returns a list of of R6 : ValueMappingIndicator class instances}
#'   }
#'   \describe{
#'     \item{\code{isOrdered}}{ A read/write active binding.
#'        If members are ordered set to true, if unordered set to false.
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$isOrdered <- mylogical} sets the isOrdered property of 'myDDI4_PhysicalSegmentLayout' to the vector of datatype:logical
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$isOrdered} returns a vector of datatype: logical}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{formatsLogicalRecord}}{ A read/write active binding.
#'          Logical Record physically represented by the Physical Layout.
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$formatsLogicalRecord <- c(myLogicalRecordURN)} sets the association 'formatsLogicalRecord' from 'myDDI4_PhysicalSegmentLayout' to a character vector of DdiUrns for R6 LogicalRecord class instances
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$formatsLogicalRecord} returns a character vector of of DdiUrns for R6  LogicalRecord class instances}
#'   }
#'   \describe{
#'     \item{\code{isStructuredBy}}{ A read/write active binding.
#'          Description of a complex structure for the Collection. A collection may be structured in more than one way for different uses.
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$isStructuredBy <- c(myPhysicalLayoutRelationStructureURN)} sets the association 'isStructuredBy' from 'myDDI4_PhysicalSegmentLayout' to a character vector of DdiUrns for R6 PhysicalLayoutRelationStructure class instances
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$isStructuredBy} returns a character vector of of DdiUrns for R6  PhysicalLayoutRelationStructure class instances}
#'   }
#'   \describe{
#'     \item{\code{definingConcept}}{ A read/write active binding.
#'          The conceptual basis for the collection of members.
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$definingConcept <- c(myConceptURN)} sets the association 'definingConcept' from 'myDDI4_PhysicalSegmentLayout' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_PhysicalSegmentLayout$definingConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_PhysicalSegmentLayout$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, isDelimited=NA, delimiter=NA, isFixedWidth=NA, escapeCharacter=NA, lineTerminator=NA, quoteCharacter=NA, commentPrefix=NA, encoding=NA, hasHeader=NA, headerRowCount=NA, skipBlankRows=NA, skipDataColumns=NA, skipInitialSpace=NA, skipRows=NA, trim=NA, nullSequence=NA, headerIsCaseSensitive=NA, arrayBase=NA, treatConsecutiveDelimitersAsOne=NA, overview=NA, tableDirection=NA, textDirection=NA, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_PhysicalSegmentLayout <- R6Class(classname="DDI4_PhysicalSegmentLayout",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, isDelimited=NA, delimiter=NA, isFixedWidth=NA, escapeCharacter=NA, lineTerminator=NA, quoteCharacter=NA, commentPrefix=NA, encoding=NA, hasHeader=NA, headerRowCount=NA, skipBlankRows=NA, skipDataColumns=NA, skipInitialSpace=NA, skipRows=NA, trim=NA, nullSequence=NA, headerIsCaseSensitive=NA, arrayBase=NA, treatConsecutiveDelimitersAsOne=NA, overview=NA, tableDirection=NA, textDirection=NA, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA,    formatsLogicalRecord=NA,isStructuredBy=NA,definingConcept=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_PhysicalSegmentLayout may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(isDelimited)){
        private$.isDelimited <- returnVectorIfValid(validVectorClass="logical", proposedVector=isDelimited , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(delimiter)){
        private$.delimiter <- returnVectorIfValid(validVectorClass="character", proposedVector=delimiter , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isFixedWidth)){
        private$.isFixedWidth <- returnVectorIfValid(validVectorClass="logical", proposedVector=isFixedWidth , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(escapeCharacter)){
        private$.escapeCharacter <- returnVectorIfValid(validVectorClass="character", proposedVector=escapeCharacter , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(lineTerminator)){
        private$.lineTerminator <- returnVectorIfValid(validVectorClass="character", proposedVector=lineTerminator , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(quoteCharacter)){
        private$.quoteCharacter <- returnVectorIfValid(validVectorClass="character", proposedVector=quoteCharacter , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(commentPrefix)){
        private$.commentPrefix <- returnVectorIfValid(validVectorClass="character", proposedVector=commentPrefix , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(encoding)){
        private$.encoding <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=encoding , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasHeader)){
        private$.hasHeader <- returnVectorIfValid(validVectorClass="logical", proposedVector=hasHeader , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(headerRowCount)){
        private$.headerRowCount <- returnVectorIfValid(validVectorClass="integer", proposedVector=headerRowCount , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(skipBlankRows)){
        private$.skipBlankRows <- returnVectorIfValid(validVectorClass="logical", proposedVector=skipBlankRows , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(skipDataColumns)){
        private$.skipDataColumns <- returnVectorIfValid(validVectorClass="integer", proposedVector=skipDataColumns , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(skipInitialSpace)){
        private$.skipInitialSpace <- returnVectorIfValid(validVectorClass="logical", proposedVector=skipInitialSpace , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(skipRows)){
        private$.skipRows <- returnVectorIfValid(validVectorClass="integer", proposedVector=skipRows , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(trim)){
        private$.trim <- returnEnumeratedValues(enumerationObject=DDI4_TrimValues$new(registerThisObject=FALSE), proposedCharacterVector=trim , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(nullSequence)){
        private$.nullSequence <- returnVectorIfValid(validVectorClass="character", proposedVector=nullSequence , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(headerIsCaseSensitive)){
        private$.headerIsCaseSensitive <- returnVectorIfValid(validVectorClass="logical", proposedVector=headerIsCaseSensitive , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(arrayBase)){
        private$.arrayBase <- returnVectorIfValid(validVectorClass="integer", proposedVector=arrayBase , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(treatConsecutiveDelimitersAsOne)){
        private$.treatConsecutiveDelimitersAsOne <- returnVectorIfValid(validVectorClass="logical", proposedVector=treatConsecutiveDelimitersAsOne , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(overview)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=overview , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(tableDirection)){
        private$.tableDirection <- returnEnumeratedValues(enumerationObject=DDI4_TableDirectionValues$new(registerThisObject=FALSE), proposedCharacterVector=tableDirection , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(textDirection)){
        private$.textDirection <- returnEnumeratedValues(enumerationObject=DDI4_TextDirectionValues$new(registerThisObject=FALSE), proposedCharacterVector=textDirection , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(type)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=type , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_ValueMappingIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(isOrdered)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=isOrdered , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(formatsLogicalRecord)){
        private$.formatsLogicalRecord <- returnUrnVectorIfValid(validClass="DDI4_LogicalRecord", proposedUrnVector=formatsLogicalRecord , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.formatsLogicalRecord, referencingAssociationName="formatsLogicalRecord",  referencingUrn=thisUrn)  
      }
      if(!missing(isStructuredBy)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_PhysicalLayoutRelationStructure", proposedUrnVector=isStructuredBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
      }
      if(!missing(definingConcept)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=definingConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 27 properties: 
    arrayBase = function(val){
      if (!missing(val)){
        private$.arrayBase <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.arrayBase
      }
    },

    commentPrefix = function(val){
      if (!missing(val)){
        private$.commentPrefix <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.commentPrefix
      }
    },

    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_ValueMappingIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },

    delimiter = function(val){
      if (!missing(val)){
        private$.delimiter <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.delimiter
      }
    },

    encoding = function(val){
      if (!missing(val)){
        private$.encoding <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.encoding
      }
    },

    escapeCharacter = function(val){
      if (!missing(val)){
        private$.escapeCharacter <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.escapeCharacter
      }
    },

    hasHeader = function(val){
      if (!missing(val)){
        private$.hasHeader <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasHeader
      }
    },

    headerIsCaseSensitive = function(val){
      if (!missing(val)){
        private$.headerIsCaseSensitive <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.headerIsCaseSensitive
      }
    },

    headerRowCount = function(val){
      if (!missing(val)){
        private$.headerRowCount <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.headerRowCount
      }
    },

    isDelimited = function(val){
      if (!missing(val)){
        private$.isDelimited <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.isDelimited
      }
    },

    isFixedWidth = function(val){
      if (!missing(val)){
        private$.isFixedWidth <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.isFixedWidth
      }
    },

    isOrdered = function(val){
      if (!missing(val)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isOrdered
      }
    },

    lineTerminator = function(val){
      if (!missing(val)){
        private$.lineTerminator <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.lineTerminator
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    nullSequence = function(val){
      if (!missing(val)){
        private$.nullSequence <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.nullSequence
      }
    },

    overview = function(val){
      if (!missing(val)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overview
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    quoteCharacter = function(val){
      if (!missing(val)){
        private$.quoteCharacter <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.quoteCharacter
      }
    },

    skipBlankRows = function(val){
      if (!missing(val)){
        private$.skipBlankRows <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.skipBlankRows
      }
    },

    skipDataColumns = function(val){
      if (!missing(val)){
        private$.skipDataColumns <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.skipDataColumns
      }
    },

    skipInitialSpace = function(val){
      if (!missing(val)){
        private$.skipInitialSpace <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.skipInitialSpace
      }
    },

    skipRows = function(val){
      if (!missing(val)){
        private$.skipRows <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.skipRows
      }
    },

    tableDirection = function(val){
      if (!missing(val)){
        private$.tableDirection <- returnEnumeratedValues(enumerationObject=DDI4_TableDirectionValues$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.tableDirection
      }
    },

    textDirection = function(val){
      if (!missing(val)){
        private$.textDirection <- returnEnumeratedValues(enumerationObject=DDI4_TextDirectionValues$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.textDirection
      }
    },

    treatConsecutiveDelimitersAsOne = function(val){
      if (!missing(val)){
        private$.treatConsecutiveDelimitersAsOne <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.treatConsecutiveDelimitersAsOne
      }
    },

    trim = function(val){
      if (!missing(val)){
        private$.trim <- returnEnumeratedValues(enumerationObject=DDI4_TrimValues$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.trim
      }
    },

    type = function(val){
      if (!missing(val)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.type
      }
    },


# There are 3 class association(s):
    definingConcept = function(val){
      if (!missing(val)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
       # invisible( private$.definingConcept)
      } else {
          private$.definingConcept
      }
    },

    formatsLogicalRecord = function(val){
      if (!missing(val)){
        private$.formatsLogicalRecord <- returnUrnVectorIfValid(validClass="DDI4_LogicalRecord", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.formatsLogicalRecord, referencingAssociationName="formatsLogicalRecord",  referencingUrn=thisUrn)  
       # invisible( private$.formatsLogicalRecord)
      } else {
          private$.formatsLogicalRecord
      }
    },

    isStructuredBy = function(val){
      if (!missing(val)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_PhysicalLayoutRelationStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
       # invisible( private$.isStructuredBy)
      } else {
          private$.isStructuredBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .isDelimited = NULL,
    .delimiter = NULL,
    .isFixedWidth = NULL,
    .escapeCharacter = NULL,
    .lineTerminator = NULL,
    .quoteCharacter = NULL,
    .commentPrefix = NULL,
    .encoding = NULL,
    .hasHeader = NULL,
    .headerRowCount = NULL,
    .skipBlankRows = NULL,
    .skipDataColumns = NULL,
    .skipInitialSpace = NULL,
    .skipRows = NULL,
    .trim = NULL,
    .nullSequence = NULL,
    .headerIsCaseSensitive = NULL,
    .arrayBase = NULL,
    .treatConsecutiveDelimitersAsOne = NULL,
    .overview = NULL,
    .tableDirection = NULL,
    .textDirection = NULL,
    .type = NULL,
    .name = NULL,
    .purpose = NULL,
    .contains = NULL,
    .isOrdered = NULL,
    .formatsLogicalRecord = NULL,
    .isStructuredBy = NULL,
    .definingConcept = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        arrayBase = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        commentPrefix = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ValueMappingIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        delimiter = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        encoding = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        escapeCharacter = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        formatsLogicalRecord = c(
          min="0", 
          max="1", 
          datatype = "DDI4_LogicalRecord", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasHeader = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        headerIsCaseSensitive = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        headerRowCount = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isDelimited = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isFixedWidth = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isStructuredBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_PhysicalLayoutRelationStructure", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        lineTerminator = c(
          min="0", 
          max="Inf", 
          datatype = "character", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        nullSequence = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        quoteCharacter = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        skipBlankRows = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        skipDataColumns = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        skipInitialSpace = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        skipRows = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        tableDirection = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TableDirectionValues", 
          structure = "Enumeration vector"
        ),
        textDirection = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TextDirectionValues", 
          structure = "Enumeration vector"
        ),
        treatConsecutiveDelimitersAsOne = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        trim = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TrimValues", 
          structure = "Enumeration vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_PhysicalSegmentLayout


#' @title DDI4_PhysicalSegmentLocation  Among other things defines the location of a DataPoint in a PhysicalSegment (e
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/PhysicalSegmentLocation}{PhysicalSegmentLocation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Among other things defines the location of a DataPoint in a PhysicalSegment (e.g. the location of the representation of a variable in a text file). This is abstract since there are many different ways to describe the location of a segment - character counts, start and end times, etc.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A segment of text in a plain text file beginning at character 3 and ending at character 123.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   While this has no properties or relationships of its own, it is useful as a target of relationships where its extensions may serve.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_PhysicalSegmentLocation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_PhysicalSegmentLocation <- R6Class(classname="DDI4_PhysicalSegmentLocation",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_PhysicalSegmentLocation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

    }
  ),

  active = list(
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_PhysicalSegmentLocation


#' @title DDI4_Polygon  A closed plane figure bounded by three or more line segments, representing a geographic area
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Polygon}{Polygon} 

#'
#'   Definition
#'
#'   ============
#'
#'   A closed plane figure bounded by three or more line segments, representing a geographic area. Contains either the URI of the file containing the polygon, a specific link code for the shape within the file, and a file format, or a minimum of 4 points to describe the polygon in-line. Note that the first and last point must be identical in order to close the polygon. A triangle has 4 points.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:PolygonType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Polygon$DdiUrn} returns the DDI URN of myDDI4_Polygon.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{uri}}{ A read/write active binding.
#'        A URI for the boundary file containing the polygon
#'
#'                              \code{myDDI4_Polygon$uri <- mycharacter} sets the uri property of 'myDDI4_Polygon' to the vector of datatype:character
#'
#'                              \code{myDDI4_Polygon$uri} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{polygonLinkCode}}{ A read/write active binding.
#'        The PolygonLinkCode is the identifier of the specific polygon within the file. For example in an NHGIS file the LinkCodeForPolygon for Tract 101.01 in Hennepin County in Minnesota is 2700530010101.
#'
#'                              \code{myDDI4_Polygon$polygonLinkCode <- mycharacter} sets the polygonLinkCode property of 'myDDI4_Polygon' to the vector of datatype:character
#'
#'                              \code{myDDI4_Polygon$polygonLinkCode} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{shapeFileFormat}}{ A read/write active binding.
#'        The format of the shape file existing at the location indicated by the sibling ExternalURI element.
#'
#'                              \code{myDDI4_Polygon$shapeFileFormat <- list(myExternalControlledVocabularyEntry)} sets the 'shapeFileFormat' property of 'myDDI4_Polygon' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_Polygon$shapeFileFormat} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{point}}{ A read/write active binding.
#'        A geographic point defined by a latitude and longitude. A minimum of 4 points is required as the first and last point should be identical in order to close the polygon. Note that a triangle has three sides and requires 3 unique points plus a fourth point replicating the first point in order to close the polygon.
#'
#'                              \code{myDDI4_Polygon$point <- list(mySpatialPoint)} sets the 'point' property of 'myDDI4_Polygon' to a list of R6 SpatialPoint class instances
#'
#'                              \code{myDDI4_Polygon$point} returns a list of of R6 : SpatialPoint class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Polygon$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, uri=NA, polygonLinkCode=NA, shapeFileFormat=NA, point=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Polygon <- R6Class(classname="DDI4_Polygon",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, uri=NA, polygonLinkCode=NA, shapeFileFormat=NA, point=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Polygon may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(uri)){
        private$.uri <- returnVectorIfValid(validVectorClass="character", proposedVector=uri , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(polygonLinkCode)){
        private$.polygonLinkCode <- returnVectorIfValid(validVectorClass="character", proposedVector=polygonLinkCode , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(shapeFileFormat)){
        private$.shapeFileFormat <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=shapeFileFormat , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(point)){
        private$.point <- returnR6ObjectIfValid(validClass="DDI4_SpatialPoint", proposedObjectList=point , minimumCardinality=4, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    point = function(val){
      if (!missing(val)){
        private$.point <- returnR6ObjectIfValid(validClass="DDI4_SpatialPoint", proposedObjectList=val , minimumCardinality=4, maximumCardinality=Inf)  
      } else {
        private$.point
      }
    },

    polygonLinkCode = function(val){
      if (!missing(val)){
        private$.polygonLinkCode <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.polygonLinkCode
      }
    },

    shapeFileFormat = function(val){
      if (!missing(val)){
        private$.shapeFileFormat <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.shapeFileFormat
      }
    },

    uri = function(val){
      if (!missing(val)){
        private$.uri <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.uri
      }
    }


  ),

  private = list(
    .uri = NULL,
    .polygonLinkCode = NULL,
    .shapeFileFormat = NULL,
    .point = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        point = c(
          min="4", 
          max="Inf", 
          datatype = "DDI4_SpatialPoint", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        polygonLinkCode = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        shapeFileFormat = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        uri = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Polygon


#' @title DDI4_Population  Set of specific units (people, entities, objects, events) with specification of time and geography
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Population}{Population} 

#'
#'   Definition
#'
#'   ============
#'
#'   Set of specific units (people, entities, objects, events) with specification of time and geography.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   1. Canadian adult persons residing in Canada on 13 November 1956
#'
#'   2. US computer companies at the end of 2012  
#'
#'   3. Universities in Denmark 1 January 2011.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Population is the most specific in the conceptually narrowing hierarchy of UnitType, Universe and Population. Several Populations having differing time and or geography may specialize the same Universe.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Population
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Universe}}}
#'    \item{\code{\link{DDI4_Concept}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Population$DdiUrn} returns the DDI URN of myDDI4_Population.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{timePeriodOfPopulation}}{ A read/write active binding.
#'        The time period associated with the Population
#'
#'                              \code{myDDI4_Population$timePeriodOfPopulation <- list(myDateRange)} sets the 'timePeriodOfPopulation' property of 'myDDI4_Population' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_Population$timePeriodOfPopulation} returns a list of of R6 : DateRange class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{composedOf}}{ A read/write active binding.
#'          Units in the Population 
#'
#'                              \code{myDDI4_Population$composedOf <- c(myUnitURN)} sets the association 'composedOf' from 'myDDI4_Population' to a character vector of DdiUrns for R6 Unit class instances
#'
#'                              \code{myDDI4_Population$composedOf} returns a character vector of of DdiUrns for R6  Unit class instances}
#'   }
#'   \describe{
#'     \item{\code{narrowsUniverse}}{ A read/write active binding.
#'          Reference to a Universe that the Population narrows
#'
#'                              \code{myDDI4_Population$narrowsUniverse <- c(myUniverseURN)} sets the association 'narrowsUniverse' from 'myDDI4_Population' to a character vector of DdiUrns for R6 Universe class instances
#'
#'                              \code{myDDI4_Population$narrowsUniverse} returns a character vector of of DdiUrns for R6  Universe class instances}
#'   }
#'   \describe{
#'     \item{\code{geographyOfPopulation}}{ A read/write active binding.
#'          The geographic location related to this population. May be repeated for inclusion of multiple locations.
#'
#'                              \code{myDDI4_Population$geographyOfPopulation <- c(myGeographicUnitURN)} sets the association 'geographyOfPopulation' from 'myDDI4_Population' to a character vector of DdiUrns for R6 GeographicUnit class instances
#'
#'                              \code{myDDI4_Population$geographyOfPopulation} returns a character vector of of DdiUrns for R6  GeographicUnit class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Population$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, timePeriodOfPopulation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Population <- R6Class(classname="DDI4_Population",
  inherit = DDI4_Universe,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, timePeriodOfPopulation=NA,    composedOf=NA,narrowsUniverse=NA,geographyOfPopulation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Population may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(timePeriodOfPopulation)){
        private$.timePeriodOfPopulation <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=timePeriodOfPopulation , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(composedOf)){
        private$.composedOf <- returnUrnVectorIfValid(validClass="DDI4_Unit", proposedUrnVector=composedOf , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.composedOf, referencingAssociationName="composedOf",  referencingUrn=thisUrn)  
      }
      if(!missing(narrowsUniverse)){
        private$.narrowsUniverse <- returnUrnVectorIfValid(validClass="DDI4_Universe", proposedUrnVector=narrowsUniverse , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.narrowsUniverse, referencingAssociationName="narrowsUniverse",  referencingUrn=thisUrn)  
      }
      if(!missing(geographyOfPopulation)){
        private$.geographyOfPopulation <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnit", proposedUrnVector=geographyOfPopulation , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.geographyOfPopulation, referencingAssociationName="geographyOfPopulation",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    timePeriodOfPopulation = function(val){
      if (!missing(val)){
        private$.timePeriodOfPopulation <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.timePeriodOfPopulation
      }
    },


# There are 3 class association(s):
    composedOf = function(val){
      if (!missing(val)){
        private$.composedOf <- returnUrnVectorIfValid(validClass="DDI4_Unit", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.composedOf, referencingAssociationName="composedOf",  referencingUrn=thisUrn)  
       # invisible( private$.composedOf)
      } else {
          private$.composedOf
      }
    },

    geographyOfPopulation = function(val){
      if (!missing(val)){
        private$.geographyOfPopulation <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnit", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.geographyOfPopulation, referencingAssociationName="geographyOfPopulation",  referencingUrn=thisUrn)  
       # invisible( private$.geographyOfPopulation)
      } else {
          private$.geographyOfPopulation
      }
    },

    narrowsUniverse = function(val){
      if (!missing(val)){
        private$.narrowsUniverse <- returnUrnVectorIfValid(validClass="DDI4_Universe", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.narrowsUniverse, referencingAssociationName="narrowsUniverse",  referencingUrn=thisUrn)  
       # invisible( private$.narrowsUniverse)
      } else {
          private$.narrowsUniverse
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .timePeriodOfPopulation = NULL,
    .composedOf = NULL,
    .narrowsUniverse = NULL,
    .geographyOfPopulation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        composedOf = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Unit", 
          structure = "DdiUrn vector"
        ),
        definition = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        descriptiveText = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        geographyOfPopulation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_GeographicUnit", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isInclusive = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        narrowsUnitType = c(
          min="0", 
          max="1", 
          datatype = "DDI4_UnitType", 
          structure = "DdiUrn vector"
        ),
        narrowsUniverse = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Universe", 
          structure = "DdiUrn vector"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        timePeriodOfPopulation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usesConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Population


#' @title DDI4_Precondition  A precondition is a state
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Precondition}{Precondition} 

#'
#'   Definition
#'
#'   ============
#'
#'   A precondition is a state. The state includes one or more goals that were previously achieved. This state is the necessary condition before a process can begin.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Using goals and preconditions processes can be chained to form work flows. Note that these work flows do not include data flows. Instead data flows consisting ouf outputs, inputs and bindings are a separate swim lane. 
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   A precondition related to a design defines the state that must exist in order for a design being applied. For example in applying a Sampling Design there may be a precondition for the existance of a sampling frame meeting certain specifications.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_BusinessFunction}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isDiscussedIn}}{ A read/write active binding.
#'          Identifies material discussing the precondition. The material may be in DDI or other format.
#'
#'                              \code{myDDI4_Precondition$isDiscussedIn <- c(myExternalMaterialURN)} sets the association 'isDiscussedIn' from 'myDDI4_Precondition' to a character vector of DdiUrns for R6 ExternalMaterial class instances
#'
#'                              \code{myDDI4_Precondition$isDiscussedIn} returns a character vector of of DdiUrns for R6  ExternalMaterial class instances}
#'   }
#'   \describe{
#'     \item{\code{basedOnPriorResult}}{ A read/write active binding.
#'          The Precondition may be based on the Results of earlier Processes, indicated with this relationship.
#'
#'                              \code{myDDI4_Precondition$basedOnPriorResult <- c(myResultURN)} sets the association 'basedOnPriorResult' from 'myDDI4_Precondition' to a character vector of DdiUrns for R6 Result class instances
#'
#'                              \code{myDDI4_Precondition$basedOnPriorResult} returns a character vector of of DdiUrns for R6  Result class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Precondition$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Precondition <- R6Class(classname="DDI4_Precondition",
  inherit = DDI4_BusinessFunction,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, isDiscussedIn=NA,basedOnPriorResult=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Precondition may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)



       if(!missing(isDiscussedIn)){
        private$.isDiscussedIn <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=isDiscussedIn , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isDiscussedIn, referencingAssociationName="isDiscussedIn",  referencingUrn=thisUrn)  
      }
      if(!missing(basedOnPriorResult)){
        private$.basedOnPriorResult <- returnUrnVectorIfValid(validClass="DDI4_Result", proposedUrnVector=basedOnPriorResult , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.basedOnPriorResult, referencingAssociationName="basedOnPriorResult",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 2 class association(s):
    basedOnPriorResult = function(val){
      if (!missing(val)){
        private$.basedOnPriorResult <- returnUrnVectorIfValid(validClass="DDI4_Result", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.basedOnPriorResult, referencingAssociationName="basedOnPriorResult",  referencingUrn=thisUrn)  
       # invisible( private$.basedOnPriorResult)
      } else {
          private$.basedOnPriorResult
      }
    },

    isDiscussedIn = function(val){
      if (!missing(val)){
        private$.isDiscussedIn <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isDiscussedIn, referencingAssociationName="isDiscussedIn",  referencingUrn=thisUrn)  
       # invisible( private$.isDiscussedIn)
      } else {
          private$.isDiscussedIn
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .isDiscussedIn = NULL,
    .basedOnPriorResult = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        basedOnPriorResult = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Result", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isDiscussedIn = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Precondition


#' @title DDI4_PrivateImage  References an image using the standard Image description
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/PrivateImage}{PrivateImage} 

#'
#'   Definition
#'
#'   ============
#'
#'   References an image using the standard Image description. In addition to the standard attributes provides an effective date (period), the type of image, and a privacy ranking.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   a:PrivateImageType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Image}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_PrivateImage$DdiUrn} returns the DDI URN of myDDI4_PrivateImage.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{effectiveDates}}{ A read/write active binding.
#'        The period for which this image is effective/valid.
#'
#'                              \code{myDDI4_PrivateImage$effectiveDates <- list(myDateRange)} sets the 'effectiveDates' property of 'myDDI4_PrivateImage' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_PrivateImage$effectiveDates} returns a list of of R6 : DateRange class instances}
#'   }
#'   \describe{
#'     \item{\code{privacy}}{ A read/write active binding.
#'        Specify the level privacy for the image as public, restricted, or private. Supports the use of an external controlled vocabulary.
#'
#'                              \code{myDDI4_PrivateImage$privacy <- list(myExternalControlledVocabularyEntry)} sets the 'privacy' property of 'myDDI4_PrivateImage' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_PrivateImage$privacy} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_PrivateImage$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, effectiveDates=NA, privacy=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_PrivateImage <- R6Class(classname="DDI4_PrivateImage",
  inherit = DDI4_Image,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, effectiveDates=NA, privacy=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_PrivateImage may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(effectiveDates)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=effectiveDates , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(privacy)){
        private$.privacy <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=privacy , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    effectiveDates = function(val){
      if (!missing(val)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.effectiveDates
      }
    },

    privacy = function(val){
      if (!missing(val)){
        private$.privacy <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.privacy
      }
    }


  ),

  private = list(
    .effectiveDates = NULL,
    .privacy = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        dpi = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        effectiveDates = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        languageOfImage = c(
          min="0", 
          max="Inf", 
          datatype = "character", 
          structure = "vector"
        ),
        privacy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        typeOfImage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        uri = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_PrivateImage


#' @title DDI4_Process  Process is an implementation of an algorithm
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Process}{Process} 

#'
#'   Definition
#'
#'   ============
#'
#'   Process is an implementation of an algorithm. It is the set of process steps taken as a whole. It is decomposable into a single ProcessStep or multiple ProcessSteps organized by a Process Control Step.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   The Generic Longitudinal Business Process Model (GLBPM) identifies a series of processes like data integration, anonymization, analysis variable construction and so forth in the data lifecycle.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Process Design
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Process$DdiUrn} returns the DDI URN of myDDI4_Process.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{overview}}{ A read/write active binding.
#'        Provides a high level overview or summary of the class. Can be used to inform end-users or as part of an executive summary. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_Process$overview <- list(myInternationalStructuredString)} sets the 'overview' property of 'myDDI4_Process' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Process$overview} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage.
#'
#'                              \code{myDDI4_Process$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_Process' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_Process$name} returns a list of of R6 : ObjectName class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{implementsAlgorithm}}{ A read/write active binding.
#'          An implementation and/or execution of an algorithm which is associated with the design of the process.
#'
#'                              \code{myDDI4_Process$implementsAlgorithm <- c(myAlgorithmURN)} sets the association 'implementsAlgorithm' from 'myDDI4_Process' to a character vector of DdiUrns for R6 Algorithm class instances
#'
#'                              \code{myDDI4_Process$implementsAlgorithm} returns a character vector of of DdiUrns for R6  Algorithm class instances}
#'   }
#'   \describe{
#'     \item{\code{isSpecifiedBy}}{ A read/write active binding.
#'          The Process Step which describes the organization of the process flow. This could be realizes as a sequence, a temporal relationship, or conditional control step. When appropriate to a realization it could be realized as a single ProcessStep (such as an Act). It should provide a clear entry and exit point(s) for the process.
#'
#'                              \code{myDDI4_Process$isSpecifiedBy <- c(myProcessStepURN)} sets the association 'isSpecifiedBy' from 'myDDI4_Process' to a character vector of DdiUrns for R6 ProcessStep class instances
#'
#'                              \code{myDDI4_Process$isSpecifiedBy} returns a character vector of of DdiUrns for R6  ProcessStep class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Process$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, overview=NA, name=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Process <- R6Class(classname="DDI4_Process",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, overview=NA, name=NA,    implementsAlgorithm=NA,isSpecifiedBy=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Process may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(overview)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=overview , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(implementsAlgorithm)){
        private$.implementsAlgorithm <- returnUrnVectorIfValid(validClass="DDI4_Algorithm", proposedUrnVector=implementsAlgorithm , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.implementsAlgorithm, referencingAssociationName="implementsAlgorithm",  referencingUrn=thisUrn)  
      }
      if(!missing(isSpecifiedBy)){
        private$.isSpecifiedBy <- returnUrnVectorIfValid(validClass="DDI4_ProcessStep", proposedUrnVector=isSpecifiedBy , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isSpecifiedBy, referencingAssociationName="isSpecifiedBy",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 2 properties: 
    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    overview = function(val){
      if (!missing(val)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overview
      }
    },


# There are 2 class association(s):
    implementsAlgorithm = function(val){
      if (!missing(val)){
        private$.implementsAlgorithm <- returnUrnVectorIfValid(validClass="DDI4_Algorithm", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.implementsAlgorithm, referencingAssociationName="implementsAlgorithm",  referencingUrn=thisUrn)  
       # invisible( private$.implementsAlgorithm)
      } else {
          private$.implementsAlgorithm
      }
    },

    isSpecifiedBy = function(val){
      if (!missing(val)){
        private$.isSpecifiedBy <- returnUrnVectorIfValid(validClass="DDI4_ProcessStep", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isSpecifiedBy, referencingAssociationName="isSpecifiedBy",  referencingUrn=thisUrn)  
       # invisible( private$.isSpecifiedBy)
      } else {
          private$.isSpecifiedBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .overview = NULL,
    .name = NULL,
    .implementsAlgorithm = NULL,
    .isSpecifiedBy = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        implementsAlgorithm = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Algorithm", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isSpecifiedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ProcessStep", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_Process


#' @title DDI4_ProcessControlStep  A Process Step that controls the ordering of execution for one or more Process Steps
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ProcessControlStep}{ProcessControlStep} 

#'
#'   Definition
#'
#'   ============
#'
#'   A Process Step that controls the ordering of execution for one or more Process Steps. May be realized to provide needed controls such as sequencing, temporal relationships, or conditions as the means of ordering or triggering a Process Step. The abstract class does not dictate the means of designating the trigger for initiating a Process Step or ordering of those Process Steps. This provides the flexibility to define these in a way most appropriate to the realization and use.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ProcessStep}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{executes}}{ A read/write active binding.
#'          The ProcessSteps whose execution order is defined by the control structure.
#'
#'                              \code{myDDI4_ProcessControlStep$executes <- c(myProcessStepURN)} sets the association 'executes' from 'myDDI4_ProcessControlStep' to a character vector of DdiUrns for R6 ProcessStep class instances
#'
#'                              \code{myDDI4_ProcessControlStep$executes} returns a character vector of of DdiUrns for R6  ProcessStep class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ProcessControlStep$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ProcessControlStep <- R6Class(classname="DDI4_ProcessControlStep",
  inherit = DDI4_ProcessStep,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, executes=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ProcessControlStep may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)



       if(!missing(executes)){
        private$.executes <- returnUrnVectorIfValid(validClass="DDI4_ProcessStep", proposedUrnVector=executes , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.executes, referencingAssociationName="executes",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 class association(s):
    executes = function(val){
      if (!missing(val)){
        private$.executes <- returnUrnVectorIfValid(validClass="DDI4_ProcessStep", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.executes, referencingAssociationName="executes",  referencingUrn=thisUrn)  
       # invisible( private$.executes)
      } else {
          private$.executes
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .executes = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        executes = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ProcessStep", 
          structure = "DdiUrn vector"
        ),
        hasInformationFlow = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPerformedBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Service", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_ProcessControlStep


#' @title DDI4_ProcessOverview  Process is an implementation of an algorithm
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ProcessOverview}{ProcessOverview} 

#'
#'   Definition
#'
#'   ============
#'
#'   Process is an implementation of an algorithm. It is the series of steps taken as a whole. It is decomposable into ProcessSteps, but this decomposition is not necessary. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   In descriptive document regarding ingest of a study into a repository: "Validation processes were run comparing all data against the provided documentation to check for out-of-range and invalid content."
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   In a descriptive document, Process Overview provides a means of presenting the reader with a clear view of an overall process without going into the details of the process steps. Process Overview should be used when a description of the process in general is all that is desired.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ProcessOverview$DdiUrn} returns the DDI URN of myDDI4_ProcessOverview.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{overview}}{ A read/write active binding.
#'        Provides a high level overview or summary of the class. Can be used to inform end-users or as part of an executive summary. Supports the use of multiple languages and structured text. Edit
#'
#'                              \code{myDDI4_ProcessOverview$overview <- list(myInternationalStructuredString)} sets the 'overview' property of 'myDDI4_ProcessOverview' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_ProcessOverview$overview} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage.
#'
#'                              \code{myDDI4_ProcessOverview$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_ProcessOverview' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_ProcessOverview$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{subjectOfProcess}}{ A read/write active binding.
#'        A term describing the subject of the Process. Supports the use of a controlled vocabulary
#'
#'                              \code{myDDI4_ProcessOverview$subjectOfProcess <- list(myExternalControlledVocabularyEntry)} sets the 'subjectOfProcess' property of 'myDDI4_ProcessOverview' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ProcessOverview$subjectOfProcess} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isSpecifiedBy}}{ A read/write active binding.
#'          The Process Control Step which describes the organization of the process flow. This could be realizes as a sequence, a temporal relationship, or conditional control step. It should provide a clear entry and exit point(s) for the process.
#'
#'                              \code{myDDI4_ProcessOverview$isSpecifiedBy <- c(myWorkflowControlStepURN)} sets the association 'isSpecifiedBy' from 'myDDI4_ProcessOverview' to a character vector of DdiUrns for R6 WorkflowControlStep class instances
#'
#'                              \code{myDDI4_ProcessOverview$isSpecifiedBy} returns a character vector of of DdiUrns for R6  WorkflowControlStep class instances}
#'   }
#'   \describe{
#'     \item{\code{implementsAlgorithm}}{ A read/write active binding.
#'          An implementation and/or execution of an algorithm which is associated with the design of the process.
#'
#'                              \code{myDDI4_ProcessOverview$implementsAlgorithm <- c(myAlgorithmOverviewURN)} sets the association 'implementsAlgorithm' from 'myDDI4_ProcessOverview' to a character vector of DdiUrns for R6 AlgorithmOverview class instances
#'
#'                              \code{myDDI4_ProcessOverview$implementsAlgorithm} returns a character vector of of DdiUrns for R6  AlgorithmOverview class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ProcessOverview$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, overview=NA, name=NA, subjectOfProcess=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ProcessOverview <- R6Class(classname="DDI4_ProcessOverview",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, overview=NA, name=NA, subjectOfProcess=NA,    isSpecifiedBy=NA,implementsAlgorithm=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ProcessOverview may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(overview)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=overview , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(subjectOfProcess)){
        private$.subjectOfProcess <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=subjectOfProcess , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(isSpecifiedBy)){
        private$.isSpecifiedBy <- returnUrnVectorIfValid(validClass="DDI4_WorkflowControlStep", proposedUrnVector=isSpecifiedBy , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isSpecifiedBy, referencingAssociationName="isSpecifiedBy",  referencingUrn=thisUrn)  
      }
      if(!missing(implementsAlgorithm)){
        private$.implementsAlgorithm <- returnUrnVectorIfValid(validClass="DDI4_AlgorithmOverview", proposedUrnVector=implementsAlgorithm , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.implementsAlgorithm, referencingAssociationName="implementsAlgorithm",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    overview = function(val){
      if (!missing(val)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overview
      }
    },

    subjectOfProcess = function(val){
      if (!missing(val)){
        private$.subjectOfProcess <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.subjectOfProcess
      }
    },


# There are 2 class association(s):
    implementsAlgorithm = function(val){
      if (!missing(val)){
        private$.implementsAlgorithm <- returnUrnVectorIfValid(validClass="DDI4_AlgorithmOverview", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.implementsAlgorithm, referencingAssociationName="implementsAlgorithm",  referencingUrn=thisUrn)  
       # invisible( private$.implementsAlgorithm)
      } else {
          private$.implementsAlgorithm
      }
    },

    isSpecifiedBy = function(val){
      if (!missing(val)){
        private$.isSpecifiedBy <- returnUrnVectorIfValid(validClass="DDI4_WorkflowControlStep", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isSpecifiedBy, referencingAssociationName="isSpecifiedBy",  referencingUrn=thisUrn)  
       # invisible( private$.isSpecifiedBy)
      } else {
          private$.isSpecifiedBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .overview = NULL,
    .name = NULL,
    .subjectOfProcess = NULL,
    .isSpecifiedBy = NULL,
    .implementsAlgorithm = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        implementsAlgorithm = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AlgorithmOverview", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isSpecifiedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowControlStep", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        subjectOfProcess = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ProcessOverview


#' @title DDI4_ProcessStep  One of the constituents of a Process
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ProcessStep}{ProcessStep} 

#'
#'   Definition
#'
#'   ============
#'
#'   One of the constituents of a Process. It can be a composition or atomic and might be performed by a Service.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Each step and substep in the Generic Longitudinal Business Process Model is a ProcessStep. 
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ProcessStep$DdiUrn} returns the DDI URN of myDDI4_ProcessStep.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasInformationFlow}}{ A read/write active binding.
#'        Bindings used to pass data into and out of a Process Step.
#'
#'                              \code{myDDI4_ProcessStep$hasInformationFlow <- list(myBinding)} sets the 'hasInformationFlow' property of 'myDDI4_ProcessStep' to a list of R6 Binding class instances
#'
#'                              \code{myDDI4_ProcessStep$hasInformationFlow} returns a list of of R6 : Binding class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isPerformedBy}}{ A read/write active binding.
#'          Service performing the Process Step.
#'
#'                              \code{myDDI4_ProcessStep$isPerformedBy <- c(myServiceURN)} sets the association 'isPerformedBy' from 'myDDI4_ProcessStep' to a character vector of DdiUrns for R6 Service class instances
#'
#'                              \code{myDDI4_ProcessStep$isPerformedBy} returns a character vector of of DdiUrns for R6  Service class instances}
#'   }
#'   \describe{
#'     \item{\code{hasInputParameter}}{ A read/write active binding.
#'          An identification for a Parameter related to the ProcessStep and referred to by a Binding as Input to the ProcessStep. 
#'
#'                              \code{myDDI4_ProcessStep$hasInputParameter <- c(myParameterURN)} sets the association 'hasInputParameter' from 'myDDI4_ProcessStep' to a character vector of DdiUrns for R6 Parameter class instances
#'
#'                              \code{myDDI4_ProcessStep$hasInputParameter} returns a character vector of of DdiUrns for R6  Parameter class instances}
#'   }
#'   \describe{
#'     \item{\code{hasOutputParameter}}{ A read/write active binding.
#'          An identification for a Parameter related to the ProcessStep and referred to by a Binding as Output from the ProcessStep. 
#'
#'                              \code{myDDI4_ProcessStep$hasOutputParameter <- c(myParameterURN)} sets the association 'hasOutputParameter' from 'myDDI4_ProcessStep' to a character vector of DdiUrns for R6 Parameter class instances
#'
#'                              \code{myDDI4_ProcessStep$hasOutputParameter} returns a character vector of of DdiUrns for R6  Parameter class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ProcessStep$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasInformationFlow=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ProcessStep <- R6Class(classname="DDI4_ProcessStep",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasInformationFlow=NA,    isPerformedBy=NA,hasInputParameter=NA,hasOutputParameter=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ProcessStep may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(hasInformationFlow)){
        private$.hasInformationFlow <- returnR6ObjectIfValid(validClass="DDI4_Binding", proposedObjectList=hasInformationFlow , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(isPerformedBy)){
        private$.isPerformedBy <- returnUrnVectorIfValid(validClass="DDI4_Service", proposedUrnVector=isPerformedBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isPerformedBy, referencingAssociationName="isPerformedBy",  referencingUrn=thisUrn)  
      }
      if(!missing(hasInputParameter)){
        private$.hasInputParameter <- returnUrnVectorIfValid(validClass="DDI4_Parameter", proposedUrnVector=hasInputParameter , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasInputParameter, referencingAssociationName="hasInputParameter",  referencingUrn=thisUrn)  
      }
      if(!missing(hasOutputParameter)){
        private$.hasOutputParameter <- returnUrnVectorIfValid(validClass="DDI4_Parameter", proposedUrnVector=hasOutputParameter , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasOutputParameter, referencingAssociationName="hasOutputParameter",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    hasInformationFlow = function(val){
      if (!missing(val)){
        private$.hasInformationFlow <- returnR6ObjectIfValid(validClass="DDI4_Binding", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasInformationFlow
      }
    },


# There are 3 class association(s):
    hasInputParameter = function(val){
      if (!missing(val)){
        private$.hasInputParameter <- returnUrnVectorIfValid(validClass="DDI4_Parameter", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasInputParameter, referencingAssociationName="hasInputParameter",  referencingUrn=thisUrn)  
       # invisible( private$.hasInputParameter)
      } else {
          private$.hasInputParameter
      }
    },

    hasOutputParameter = function(val){
      if (!missing(val)){
        private$.hasOutputParameter <- returnUrnVectorIfValid(validClass="DDI4_Parameter", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasOutputParameter, referencingAssociationName="hasOutputParameter",  referencingUrn=thisUrn)  
       # invisible( private$.hasOutputParameter)
      } else {
          private$.hasOutputParameter
      }
    },

    isPerformedBy = function(val){
      if (!missing(val)){
        private$.isPerformedBy <- returnUrnVectorIfValid(validClass="DDI4_Service", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isPerformedBy, referencingAssociationName="isPerformedBy",  referencingUrn=thisUrn)  
       # invisible( private$.isPerformedBy)
      } else {
          private$.isPerformedBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasInformationFlow = NULL,
    .isPerformedBy = NULL,
    .hasInputParameter = NULL,
    .hasOutputParameter = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInformationFlow = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPerformedBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Service", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_ProcessStep


#' @title DDI4_QualityStatement  A statement of quality which may be related to an external standard or contain a simple overview, rationale, and usage statement
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/QualityStatement}{QualityStatement} 

#'
#'   Definition
#'
#'   ============
#'
#'   A statement of quality which may be related to an external standard or contain a simple overview, rationale, and usage statement. When relating to an external standard information on compliance may be added via the ComplianceStatement referenced by the Standard class.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:QualityStatementType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_QualityStatement$DdiUrn} returns the DDI URN of myDDI4_QualityStatement.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage.
#'
#'                              \code{myDDI4_QualityStatement$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_QualityStatement' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_QualityStatement$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A structured display label providing a fully human readable display for the identification of the object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_QualityStatement$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_QualityStatement' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_QualityStatement$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#'   \describe{
#'     \item{\code{overview}}{ A read/write active binding.
#'        Short natural language account of the information obtained from the combination of properties and relationships associated with an object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_QualityStatement$overview <- list(myInternationalStructuredString)} sets the 'overview' property of 'myDDI4_QualityStatement' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_QualityStatement$overview} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{rationale}}{ A read/write active binding.
#'        Explanation of the reasons some decision was made or some object exists. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_QualityStatement$rationale <- list(myInternationalStructuredString)} sets the 'rationale' property of 'myDDI4_QualityStatement' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_QualityStatement$rationale} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{usage}}{ A read/write active binding.
#'        Explanation of the ways in which some decision or object is employed. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_QualityStatement$usage <- list(myInternationalStructuredString)} sets the 'usage' property of 'myDDI4_QualityStatement' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_QualityStatement$usage} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{usesStandard}}{ A read/write active binding.
#'          Identifies the external standard used and describes the level of compliance with the standard in terms specific aspects of the standard's content.
#'
#'                              \code{myDDI4_QualityStatement$usesStandard <- c(myStandardURN)} sets the association 'usesStandard' from 'myDDI4_QualityStatement' to a character vector of DdiUrns for R6 Standard class instances
#'
#'                              \code{myDDI4_QualityStatement$usesStandard} returns a character vector of of DdiUrns for R6  Standard class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_QualityStatement$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, name=NA, displayLabel=NA, overview=NA, rationale=NA, usage=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_QualityStatement <- R6Class(classname="DDI4_QualityStatement",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, name=NA, displayLabel=NA, overview=NA, rationale=NA, usage=NA,    usesStandard=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_QualityStatement may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(overview)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=overview , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(rationale)){
        private$.rationale <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=rationale , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(usage)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=usage , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(usesStandard)){
        private$.usesStandard <- returnUrnVectorIfValid(validClass="DDI4_Standard", proposedUrnVector=usesStandard , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesStandard, referencingAssociationName="usesStandard",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 5 properties: 
    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    overview = function(val){
      if (!missing(val)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overview
      }
    },

    rationale = function(val){
      if (!missing(val)){
        private$.rationale <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.rationale
      }
    },

    usage = function(val){
      if (!missing(val)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.usage
      }
    },


# There are 1 class association(s):
    usesStandard = function(val){
      if (!missing(val)){
        private$.usesStandard <- returnUrnVectorIfValid(validClass="DDI4_Standard", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesStandard, referencingAssociationName="usesStandard",  referencingUrn=thisUrn)  
       # invisible( private$.usesStandard)
      } else {
          private$.usesStandard
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .name = NULL,
    .displayLabel = NULL,
    .overview = NULL,
    .rationale = NULL,
    .usage = NULL,
    .usesStandard = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        rationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usesStandard = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Standard", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_QualityStatement


#' @title DDI4_RangeValue  Describes a bounding value of a string
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/RangeValue}{RangeValue} 

#'
#'   Definition
#'
#'   ============
#'
#'   Describes a bounding value of a string.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:RangeValueType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ValueString}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_RangeValue$DdiUrn} returns the DDI URN of myDDI4_RangeValue.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{included}}{ A read/write active binding.
#'        Set to "true" if the value is included in the range.
#'
#'                              \code{myDDI4_RangeValue$included <- mylogical} sets the included property of 'myDDI4_RangeValue' to the vector of datatype:logical
#'
#'                              \code{myDDI4_RangeValue$included} returns a vector of datatype: logical}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_RangeValue$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, included=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_RangeValue <- R6Class(classname="DDI4_RangeValue",
  inherit = DDI4_ValueString,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, included=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_RangeValue may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(included)){
        private$.included <- returnVectorIfValid(validVectorClass="logical", proposedVector=included , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 1 properties: 
    included = function(val){
      if (!missing(val)){
        private$.included <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.included
      }
    }


  ),

  private = list(
    .included = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        content = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        included = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        whiteSpace = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WhiteSpaceRule", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_RangeValue


#' @title DDI4_RankingResponseDomain  A response domain capturing a ranking response which supports a "ranking" or "Ordering" of provided categories
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/RankingResponseDomain}{RankingResponseDomain} 

#'
#'   Definition
#'
#'   ============
#'
#'   A response domain capturing a ranking response which supports a "ranking" or "Ordering" of provided categories. 
#'
#'   Note: This item still must be modeled and is incomplete at this time.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ResponseDomain}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_RankingResponseDomain$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_RankingResponseDomain <- R6Class(classname="DDI4_RankingResponseDomain",
  inherit = DDI4_ResponseDomain,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_RankingResponseDomain may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

    }
  ),

  active = list(
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="1", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        intendedRepresentation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RepresentedVariable", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_RankingResponseDomain


#' @title DDI4_RationaleDefinition  Textual description of the rationale/purpose for the version change and a coded value to provide an internal processing flag within and organization or system
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/RationaleDefinition}{RationaleDefinition} 

#'
#'   Definition
#'
#'   ============
#'
#'   Textual description of the rationale/purpose for the version change and a coded value to provide an internal processing flag within and organization or system. Note that versioning can only take place on objects owned by the specified DDI Agency. If you are creating a local instance of an object from another agency for current or future modification use BasedOnObject. If the changes being made result in what you determine to be new object rather than a version of a previous object, i.e. the change is too extensive to consider it a version of the existing object, create a new object and use BasedOnObject to provide a link to the object or objects that were a basis for the new object.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:VersionRationaleType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_RationaleDefinition$DdiUrn} returns the DDI URN of myDDI4_RationaleDefinition.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{rationaleDescription}}{ A read/write active binding.
#'        Textual description of the rationale/purpose for the version change to inform users as to the extent and implication of the version change. May be expressed in multiple languages.
#'
#'                              \code{myDDI4_RationaleDefinition$rationaleDescription <- list(myInternationalString)} sets the 'rationaleDescription' property of 'myDDI4_RationaleDefinition' to a list of R6 InternationalString class instances
#'
#'                              \code{myDDI4_RationaleDefinition$rationaleDescription} returns a list of of R6 : InternationalString class instances}
#'   }
#'   \describe{
#'     \item{\code{rationaleCode}}{ A read/write active binding.
#'        RationaleCode is primarily for internal processing flags within an organization or system. Supports the use of an external controlled vocabulary.
#'
#'                              \code{myDDI4_RationaleDefinition$rationaleCode <- list(myExternalControlledVocabularyEntry)} sets the 'rationaleCode' property of 'myDDI4_RationaleDefinition' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_RationaleDefinition$rationaleCode} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_RationaleDefinition$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, rationaleDescription=NA, rationaleCode=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_RationaleDefinition <- R6Class(classname="DDI4_RationaleDefinition",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, rationaleDescription=NA, rationaleCode=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_RationaleDefinition may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(rationaleDescription)){
        private$.rationaleDescription <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=rationaleDescription , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(rationaleCode)){
        private$.rationaleCode <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=rationaleCode , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    rationaleCode = function(val){
      if (!missing(val)){
        private$.rationaleCode <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.rationaleCode
      }
    },

    rationaleDescription = function(val){
      if (!missing(val)){
        private$.rationaleDescription <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.rationaleDescription
      }
    }


  ),

  private = list(
    .rationaleDescription = NULL,
    .rationaleCode = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        rationaleCode = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        rationaleDescription = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_RationaleDefinition


#' @title DDI4_RecordRelation  The RecordRelation object is used to indicate relationships among record types within and between LogicalRecords
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/RecordRelation}{RecordRelation} 

#'
#'   Definition
#'
#'   ============
#'
#'   The RecordRelation object is used to indicate relationships among record types within and between LogicalRecords. For InstanceVariables existing in a LogicalRecord with multiple record layouts, pairs of InstanceVariables may function as paired keys to permit the expression of hierarchical links between records of different types. These links between keys in different record types could also be used to link records in a relational structure.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   One LogicalRecord containing a PersonIdentifier and a PersonName and another LogicalRecord containing a MeasurementID, a PersonID, a SystolicPressure, and a DiastolicPressure could be linked by a RecordRelation. The RecordRelation could employ an InstanceVariableValueMap to describe the linkage between  PersonIdentifier and PersonID.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   A household-level LogicalRecord might contain an InstanceVariable called HouseholdID and a person-level LogicalRecord might contain an InstanceVariable called HID. Describing a link between HouseholdID and HID would allow the linking of a person-level LogicalRecord to their corresponding household-level LogicalRecord.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_RecordRelation$DdiUrn} returns the DDI URN of myDDI4_RecordRelation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A display label for the CollectionCorrespondence. May be expressed in multiple languages. Repeat for labels with different content, for example, labels with differing length limitations.
#'
#'                              \code{myDDI4_RecordRelation$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_RecordRelation' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_RecordRelation$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_RecordRelation$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_RecordRelation' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_RecordRelation$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{usage}}{ A read/write active binding.
#'        Explanation of the ways in which some decision or object is employed. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_RecordRelation$usage <- list(myInternationalStructuredString)} sets the 'usage' property of 'myDDI4_RecordRelation' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_RecordRelation$usage} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{correspondence}}{ A read/write active binding.
#'        Correspondences between Instance Variables of different Logical Record Layout. 
#'
#'                              \code{myDDI4_RecordRelation$correspondence <- list(myInstanceVariableValueMap)} sets the 'correspondence' property of 'myDDI4_RecordRelation' to a list of R6 InstanceVariableValueMap class instances
#'
#'                              \code{myDDI4_RecordRelation$correspondence} returns a list of of R6 : InstanceVariableValueMap class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{maps}}{ A read/write active binding.
#'          Map related record types, which are Collections of Instance Variables. Realization of structures in Symmetric Relation.

#'
#'                              \code{myDDI4_RecordRelation$maps <- c(myUnitDataRecordURN)} sets the association 'maps' from 'myDDI4_RecordRelation' to a character vector of DdiUrns for R6 UnitDataRecord class instances
#'
#'                              \code{myDDI4_RecordRelation$maps} returns a character vector of of DdiUrns for R6  UnitDataRecord class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_RecordRelation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, displayLabel=NA, purpose=NA, usage=NA, correspondence=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_RecordRelation <- R6Class(classname="DDI4_RecordRelation",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, displayLabel=NA, purpose=NA, usage=NA, correspondence=NA,    maps=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_RecordRelation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(usage)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=usage , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(correspondence)){
        private$.correspondence <- returnR6ObjectIfValid(validClass="DDI4_InstanceVariableValueMap", proposedObjectList=correspondence , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(maps)){
        private$.maps <- returnUrnVectorIfValid(validClass="DDI4_UnitDataRecord", proposedUrnVector=maps , minimumCardinality=2, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.maps, referencingAssociationName="maps",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 4 properties: 
    correspondence = function(val){
      if (!missing(val)){
        private$.correspondence <- returnR6ObjectIfValid(validClass="DDI4_InstanceVariableValueMap", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.correspondence
      }
    },

    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    usage = function(val){
      if (!missing(val)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.usage
      }
    },


# There are 1 class association(s):
    maps = function(val){
      if (!missing(val)){
        private$.maps <- returnUrnVectorIfValid(validClass="DDI4_UnitDataRecord", proposedUrnVector=val , minimumCardinality=2, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.maps, referencingAssociationName="maps",  referencingUrn=thisUrn)  
       # invisible( private$.maps)
      } else {
          private$.maps
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .displayLabel = NULL,
    .purpose = NULL,
    .usage = NULL,
    .correspondence = NULL,
    .maps = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        correspondence = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InstanceVariableValueMap", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        maps = c(
          min="2", 
          max="Inf", 
          datatype = "DDI4_UnitDataRecord", 
          structure = "DdiUrn vector"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_RecordRelation


#' @title DDI4_ReferenceDate  The date covered by the annotated object
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ReferenceDate}{ReferenceDate} 

#'
#'   Definition
#'
#'   ============
#'
#'   The date covered by the annotated object. In addition to specifying a type of date (e.g. collection period, census year, etc.) the date or time span may be associated with a particular subject or keyword. This allows for the expression of a referent date associated with specific subjects or keywords. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A set of date items on income and labor force status may have a referent date for the year prior to the collection date. To express a duration the preference is for Start and End dates expressing two time points separated by a "/". 
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Note that if needed you may use Start and Duration or Duration and End. These options may not be recognizable by all systems.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:ReferenceDateType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotationDate}}}
#'    \item{\code{\link{DDI4_Date}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ReferenceDate$DdiUrn} returns the DDI URN of myDDI4_ReferenceDate.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{subject}}{ A read/write active binding.
#'        If the date is for a subset of data only such as a referent date for residence 5 years ago, use Subject to specify the coverage of the data this date applies to. May be repeated to reflect multiple subjects.
#'
#'                              \code{myDDI4_ReferenceDate$subject <- list(myExternalControlledVocabularyEntry)} sets the 'subject' property of 'myDDI4_ReferenceDate' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ReferenceDate$subject} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{keyword}}{ A read/write active binding.
#'        If the date is for a subset of data only such as a referent date for residence 5 years ago, use keyword to specify the coverage of the data this date applies to. May be repeated to reflect multiple keywords.
#'
#'                              \code{myDDI4_ReferenceDate$keyword <- list(myExternalControlledVocabularyEntry)} sets the 'keyword' property of 'myDDI4_ReferenceDate' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ReferenceDate$keyword} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ReferenceDate$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, subject=NA, keyword=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ReferenceDate <- R6Class(classname="DDI4_ReferenceDate",
  inherit = DDI4_AnnotationDate,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, subject=NA, keyword=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ReferenceDate may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(subject)){
        private$.subject <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=subject , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(keyword)){
        private$.keyword <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=keyword , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    keyword = function(val){
      if (!missing(val)){
        private$.keyword <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.keyword
      }
    },

    subject = function(val){
      if (!missing(val)){
        private$.subject <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.subject
      }
    }


  ),

  private = list(
    .subject = NULL,
    .keyword = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        isoDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        keyword = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        nonIsoDate = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_NonIsoDateType", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        subject = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        typeOfDate = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ReferenceDate


#' @title DDI4_RelationStructure  The set of MemberRelations used to structure a Collection
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/RelationStructure}{RelationStructure} 

#'
#'   Definition
#'
#'   ============
#'
#'   The set of MemberRelations used to structure a Collection
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   The optional RelationStructure is used to describe relations among Members of the Collection that are more complex than a simple list. There can be multiple types of relation structures for a Collection and these in turn may each have a different semantic. These differences can be described by the properties of the Relation Structure, or in the case of hybrid structures described by the individual MemberRelations.
#'
#'   Each MemberRelations functions like an adjacency list (https://en.wikipedia.org/wiki/Adjacency_list) in graph theory, describing the relationships of one Member to multiple other members.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_RelationStructure$DdiUrn} returns the DDI URN of myDDI4_RelationStructure.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Controlled Vocabulary to specify whether the relation is total, partial or unknown.
#'
#'                              \code{myDDI4_RelationStructure$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_RelationStructure' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_RelationStructure$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an ennumeration list
#'
#'                              \code{myDDI4_RelationStructure$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_RelationStructure' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_RelationStructure$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship context for the relationship using an External Controlled Vocabulary. Examples might include "parent-child", or "immediate supervisee"
#'
#'                              \code{myDDI4_RelationStructure$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_RelationStructure' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_RelationStructure$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMemberRelation}}{ A read/write active binding.
#'        MemberRelation used to define the relationship of members within the collection
#'
#'                              \code{myDDI4_RelationStructure$hasMemberRelation <- list(myMemberRelation)} sets the 'hasMemberRelation' property of 'myDDI4_RelationStructure' to a list of R6 MemberRelation class instances
#'
#'                              \code{myDDI4_RelationStructure$hasMemberRelation} returns a list of of R6 : MemberRelation class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_RelationStructure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, totality=NA, hasRelationSpecification=NA, semantic=NA, hasMemberRelation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_RelationStructure <- R6Class(classname="DDI4_RelationStructure",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, totality=NA, hasRelationSpecification=NA, semantic=NA, hasMemberRelation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_RelationStructure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasMemberRelation)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_MemberRelation", proposedObjectList=hasMemberRelation , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    hasMemberRelation = function(val){
      if (!missing(val)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_MemberRelation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasMemberRelation
      }
    },

    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .totality = NULL,
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .hasMemberRelation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasMemberRelation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_MemberRelation", 
          isAbstractDatatype = TRUE,
          structure = "StructuredDatatype list"
        ),
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        totality = c(
          min="1", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_RelationStructure


#' @title DDI4_RepeatUntil  Iterative control structure to be repeated until a specified condition is met
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/RepeatUntil}{RepeatUntil} 

#'
#'   Definition
#'
#'   ============
#'
#'   Iterative control structure to be repeated until a specified condition is met. After each iteration the condition is tested. If the condition is not met, the associated Workflow Sequence in contains (inherited from Conditional Control Construct) is triggered. When the condition is met, control passes back to the containing Workflow Step.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A RepeatUntil loop is similar to a RepeatWhile loop, except that it tests the Boolean expression after the loop rather than before. Thus, a RepeatUntil loop always executes the loop once, whereas if the Boolean expression is initially False, a RepeatWhile loop does not execute the loop at all. For example, in a household there is at last one person and a loop might contain a block of person questions. After the first person the last question in the block might be "Anyone else in the household"? The RepeatUntil would continue iterating until there was no-one else in the household.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   d:RepeatUntilType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ConditionalControlStep}}}
#'    \item{\code{\link{DDI4_WorkflowControlStep}}}
#'    \item{\code{\link{DDI4_WorkflowStep}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_RepeatUntil$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_RepeatUntil <- R6Class(classname="DDI4_RepeatUntil",
  inherit = DDI4_ConditionalControlStep,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_RepeatUntil may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

    }
  ),

  active = list(
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        condition = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CommandCode", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        executes = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowStepSequence", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInformationFlow = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPerformedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowService", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_RepeatUntil


#' @title DDI4_RepeatWhile  Iterative control structure to be repeated while a specified condition is met
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/RepeatWhile}{RepeatWhile} 

#'
#'   Definition
#'
#'   ============
#'
#'   Iterative control structure to be repeated while a specified condition is met. Before each iteration the condition is tested. If the condition is met, the associated Workflow Sequence in contains (inherited from Conditional Control Construct) is triggered. When the condition is not met, control passes back to the containing Workflow Step.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A RepeatWhile loop is similar to a RepeatUntil loop, except that it tests the Boolean expression before the loop rather than after. Thus, a RepeatUntil loop always executes the loop once, whereas if the Boolean expression is initially False, a RepeatWhile loop does not execute the loop at all. For example, RepeatWhile may be used to pre-test a certain condition like if "the number of new people in a household is greater than 0" in which case a block of questions might be asked for each new person.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   d:RepeatWhileType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ConditionalControlStep}}}
#'    \item{\code{\link{DDI4_WorkflowControlStep}}}
#'    \item{\code{\link{DDI4_WorkflowStep}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_RepeatWhile$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_RepeatWhile <- R6Class(classname="DDI4_RepeatWhile",
  inherit = DDI4_ConditionalControlStep,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_RepeatWhile may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

    }
  ),

  active = list(
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        condition = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CommandCode", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        executes = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowStepSequence", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInformationFlow = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPerformedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowService", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_RepeatWhile


#' @title DDI4_RepresentedMeasurement  The description of a reusable non-question measurement that can be used as a template that describes the components of a measurement
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/RepresentedMeasurement}{RepresentedMeasurement} 

#'
#'   Definition
#'
#'   ============
#'
#'   The description of a reusable non-question measurement that can be used as a template that describes the components of a measurement. A type code can be used to describe the type of measure that was used.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A measure providing a geographic point; A blood pressure protocol; A protocol for measuring an ocean current
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Non-question measurements often involve the use of specific machines or protocols to ensure consistency and comparability of the measure. The RepresentedMeasurement allows the reusable format and protocol to be housed in a repository for use by multiple studies or repeated studies.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Capture}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_RepresentedMeasurement$DdiUrn} returns the DDI URN of myDDI4_RepresentedMeasurement.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{measurementType}}{ A read/write active binding.
#'        The type of measurement performed
#'
#'                              \code{myDDI4_RepresentedMeasurement$measurementType <- list(myExternalControlledVocabularyEntry)} sets the 'measurementType' property of 'myDDI4_RepresentedMeasurement' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_RepresentedMeasurement$measurementType} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasRepresentedVariable}}{ A read/write active binding.
#'          An optional link to a represented variable which can be used by each instance variable created by a use of this measure.
#'
#'                              \code{myDDI4_RepresentedMeasurement$hasRepresentedVariable <- c(myRepresentedVariableURN)} sets the association 'hasRepresentedVariable' from 'myDDI4_RepresentedMeasurement' to a character vector of DdiUrns for R6 RepresentedVariable class instances
#'
#'                              \code{myDDI4_RepresentedMeasurement$hasRepresentedVariable} returns a character vector of of DdiUrns for R6  RepresentedVariable class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_RepresentedMeasurement$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, measurementType=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_RepresentedMeasurement <- R6Class(classname="DDI4_RepresentedMeasurement",
  inherit = DDI4_Capture,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, measurementType=NA,    hasRepresentedVariable=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_RepresentedMeasurement may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(measurementType)){
        private$.measurementType <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=measurementType , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(hasRepresentedVariable)){
        private$.hasRepresentedVariable <- returnUrnVectorIfValid(validClass="DDI4_RepresentedVariable", proposedUrnVector=hasRepresentedVariable , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasRepresentedVariable, referencingAssociationName="hasRepresentedVariable",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    measurementType = function(val){
      if (!missing(val)){
        private$.measurementType <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.measurementType
      }
    },


# There are 1 class association(s):
    hasRepresentedVariable = function(val){
      if (!missing(val)){
        private$.hasRepresentedVariable <- returnUrnVectorIfValid(validClass="DDI4_RepresentedVariable", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasRepresentedVariable, referencingAssociationName="hasRepresentedVariable",  referencingUrn=thisUrn)  
       # invisible( private$.hasRepresentedVariable)
      } else {
          private$.hasRepresentedVariable
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .measurementType = NULL,
    .hasRepresentedVariable = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        analysisUnit = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        captureSource = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasExternalAid = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalAid", 
          structure = "DdiUrn vector"
        ),
        hasInstruction = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Instruction", 
          structure = "DdiUrn vector"
        ),
        hasRepresentedVariable = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RepresentedVariable", 
          structure = "DdiUrn vector"
        ),
        hasResponseDomain = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ResponseDomain", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        measurementType = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_RepresentedMeasurement


#' @title DDI4_RepresentedQuestion  The description of a reusable question, that can be used as a template that describes the components of a question
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/RepresentedQuestion}{RepresentedQuestion} 

#'
#'   Definition
#'
#'   ============
#'
#'   The description of a reusable question, that can be used as a template that describes the components of a question
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A question in a question bank such as the content and format for a question on Race for all survey/censuses fielded by U.S. agencies as prescribed by the U.S. Office of Management and Budget (OMB)
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   A question that is repeated across waves of a panel study can be reused and also allow reference to a RepresentedVariable. A question that has been tested for consistency of response and is used by multiple studies.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   d:QuestionItemType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Capture}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_RepresentedQuestion$DdiUrn} returns the DDI URN of myDDI4_RepresentedQuestion.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{questionText}}{ A read/write active binding.
#'        The text of the question which may be literal or dynamic (altered to personalize the question text) in terms of content.
#'
#'                              \code{myDDI4_RepresentedQuestion$questionText <- list(myDynamicText)} sets the 'questionText' property of 'myDDI4_RepresentedQuestion' to a list of R6 DynamicText class instances
#'
#'                              \code{myDDI4_RepresentedQuestion$questionText} returns a list of of R6 : DynamicText class instances}
#'   }
#'   \describe{
#'     \item{\code{questionIntent}}{ A read/write active binding.
#'        The purpose or intent of the question.
#'
#'                              \code{myDDI4_RepresentedQuestion$questionIntent <- list(myInternationalStructuredString)} sets the 'questionIntent' property of 'myDDI4_RepresentedQuestion' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_RepresentedQuestion$questionIntent} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{estimatedResponseTimeInSeconds}}{ A read/write active binding.
#'        An estimation of the number of seconds required to respond to the question. Used for estimating overall questionnaire completion time.
#'
#'                              \code{myDDI4_RepresentedQuestion$estimatedResponseTimeInSeconds <- mynumeric} sets the estimatedResponseTimeInSeconds property of 'myDDI4_RepresentedQuestion' to the vector of datatype:numeric
#'
#'                              \code{myDDI4_RepresentedQuestion$estimatedResponseTimeInSeconds} returns a vector of datatype: numeric}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasRepresentedVariable}}{ A read/write active binding.
#'          An optional link to a represented variable which can be used by each instance variable created by a use of this question.
#'
#'                              \code{myDDI4_RepresentedQuestion$hasRepresentedVariable <- c(myRepresentedVariableURN)} sets the association 'hasRepresentedVariable' from 'myDDI4_RepresentedQuestion' to a character vector of DdiUrns for R6 RepresentedVariable class instances
#'
#'                              \code{myDDI4_RepresentedQuestion$hasRepresentedVariable} returns a character vector of of DdiUrns for R6  RepresentedVariable class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_RepresentedQuestion$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, questionText=NA, questionIntent=NA, estimatedResponseTimeInSeconds=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_RepresentedQuestion <- R6Class(classname="DDI4_RepresentedQuestion",
  inherit = DDI4_Capture,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, questionText=NA, questionIntent=NA, estimatedResponseTimeInSeconds=NA,    hasRepresentedVariable=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_RepresentedQuestion may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(questionText)){
        private$.questionText <- returnR6ObjectIfValid(validClass="DDI4_DynamicText", proposedObjectList=questionText , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(questionIntent)){
        private$.questionIntent <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=questionIntent , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(estimatedResponseTimeInSeconds)){
        private$.estimatedResponseTimeInSeconds <- returnVectorIfValid(validVectorClass="numeric", proposedVector=estimatedResponseTimeInSeconds , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(hasRepresentedVariable)){
        private$.hasRepresentedVariable <- returnUrnVectorIfValid(validClass="DDI4_RepresentedVariable", proposedUrnVector=hasRepresentedVariable , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasRepresentedVariable, referencingAssociationName="hasRepresentedVariable",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    estimatedResponseTimeInSeconds = function(val){
      if (!missing(val)){
        private$.estimatedResponseTimeInSeconds <- returnVectorIfValid(validVectorClass="numeric", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.estimatedResponseTimeInSeconds
      }
    },

    questionIntent = function(val){
      if (!missing(val)){
        private$.questionIntent <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.questionIntent
      }
    },

    questionText = function(val){
      if (!missing(val)){
        private$.questionText <- returnR6ObjectIfValid(validClass="DDI4_DynamicText", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.questionText
      }
    },


# There are 1 class association(s):
    hasRepresentedVariable = function(val){
      if (!missing(val)){
        private$.hasRepresentedVariable <- returnUrnVectorIfValid(validClass="DDI4_RepresentedVariable", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasRepresentedVariable, referencingAssociationName="hasRepresentedVariable",  referencingUrn=thisUrn)  
       # invisible( private$.hasRepresentedVariable)
      } else {
          private$.hasRepresentedVariable
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .questionText = NULL,
    .questionIntent = NULL,
    .estimatedResponseTimeInSeconds = NULL,
    .hasRepresentedVariable = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        analysisUnit = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        captureSource = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        estimatedResponseTimeInSeconds = c(
          min="0", 
          max="1", 
          datatype = "numeric", 
          structure = "vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasExternalAid = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalAid", 
          structure = "DdiUrn vector"
        ),
        hasInstruction = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Instruction", 
          structure = "DdiUrn vector"
        ),
        hasRepresentedVariable = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_RepresentedVariable", 
          structure = "DdiUrn vector"
        ),
        hasResponseDomain = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ResponseDomain", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        questionIntent = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        questionText = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_DynamicText", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_RepresentedQuestion


#' @title DDI4_RepresentedVariable  A combination of a characteristic of a universe to be measured and how that measure will be represented
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/RepresentedVariable}{RepresentedVariable} 

#'
#'   Definition
#'
#'   ============
#'
#'   A combination of a characteristic of a universe to be measured and how that measure will be represented.
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   The pair (Number of Employees, Integer), where "Number of Employees" is the characteristic of the population (Variable) and "Integer" is how that measure will be represented (Value Domain).
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Extends from ConceptualVariable and can contain all descriptive fields without creating a ConceptualVariable. By referencing a ConceptualVariable it can indicate a common relationship with RepresentedVariables expressing the same characteristic of a universe measured in another way, such as Age of Persons in hours rather than years. RepresentedVariable constrains the coverage of the UnitType to a specific Universe. In the above case the Universe with the measurement of Age in hours may be constrained to Persons under 5 days (120 hours old). RepresentedVariable can define sentinel values for multiple storage systems which have the same conceptual domain but specialized value domains.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   l:RepresentedVariableType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ConceptualVariable}}}
#'    \item{\code{\link{DDI4_Concept}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_RepresentedVariable$DdiUrn} returns the DDI URN of myDDI4_RepresentedVariable.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{unitOfMeasurement}}{ A read/write active binding.
#'        The unit in which the data values are measured (kg, pound, euro).
#'
#'                              \code{myDDI4_RepresentedVariable$unitOfMeasurement <- mycharacter} sets the unitOfMeasurement property of 'myDDI4_RepresentedVariable' to the vector of datatype:character
#'
#'                              \code{myDDI4_RepresentedVariable$unitOfMeasurement} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{hasIntendedDataType}}{ A read/write active binding.
#'        The data type intended to be used by this variable. Supports the optional use of an external controlled vocabulary.
#'
#'                              \code{myDDI4_RepresentedVariable$hasIntendedDataType <- list(myExternalControlledVocabularyEntry)} sets the 'hasIntendedDataType' property of 'myDDI4_RepresentedVariable' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_RepresentedVariable$hasIntendedDataType} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{takesSubstantiveValuesFrom}}{ A read/write active binding.
#'          The substantive representation (SubstantiveValueDomain) of the variable. This is equivalent to the relationship "Measures" in GSIM although GSIM makes no distinction between substantive and sentinel values
#'
#'                              \code{myDDI4_RepresentedVariable$takesSubstantiveValuesFrom <- c(mySubstantiveValueDomainURN)} sets the association 'takesSubstantiveValuesFrom' from 'myDDI4_RepresentedVariable' to a character vector of DdiUrns for R6 SubstantiveValueDomain class instances
#'
#'                              \code{myDDI4_RepresentedVariable$takesSubstantiveValuesFrom} returns a character vector of of DdiUrns for R6  SubstantiveValueDomain class instances}
#'   }
#'   \describe{
#'     \item{\code{drawsFrom}}{ A read/write active binding.
#'          The defined class of people, entities, events, or objects to be measured.
#'
#'                              \code{myDDI4_RepresentedVariable$drawsFrom <- c(myUniverseURN)} sets the association 'drawsFrom' from 'myDDI4_RepresentedVariable' to a character vector of DdiUrns for R6 Universe class instances
#'
#'                              \code{myDDI4_RepresentedVariable$drawsFrom} returns a character vector of of DdiUrns for R6  Universe class instances}
#'   }
#'   \describe{
#'     \item{\code{basedOnConceptualVariable}}{ A read/write active binding.
#'          The ConceptualVariable that can be shared by a set of multiple RepresentedVariables. Indicates comparability in  ConceptualDomain and UnitType. 
#'
#'                              \code{myDDI4_RepresentedVariable$basedOnConceptualVariable <- c(myConceptualVariableURN)} sets the association 'basedOnConceptualVariable' from 'myDDI4_RepresentedVariable' to a character vector of DdiUrns for R6 ConceptualVariable class instances
#'
#'                              \code{myDDI4_RepresentedVariable$basedOnConceptualVariable} returns a character vector of of DdiUrns for R6  ConceptualVariable class instances}
#'   }
#'   \describe{
#'     \item{\code{takesPlatformSpecificSentinelValues}}{ A read/write active binding.
#'          A RepresentedVariable may have more than one sets of SentinelValueDomains, one for each type of software platform on which related InstanceVariables might be instantiated. All of the SentinelValueDomains must have SentinelConceptualDomains that correspond exactly. This allows codes for missing values to be explicitly matched across platforms.
#'
#'                              \code{myDDI4_RepresentedVariable$takesPlatformSpecificSentinelValues <- c(mySentinelValueDomainURN)} sets the association 'takesPlatformSpecificSentinelValues' from 'myDDI4_RepresentedVariable' to a character vector of DdiUrns for R6 SentinelValueDomain class instances
#'
#'                              \code{myDDI4_RepresentedVariable$takesPlatformSpecificSentinelValues} returns a character vector of of DdiUrns for R6  SentinelValueDomain class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_RepresentedVariable$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, unitOfMeasurement=NA, hasIntendedDataType=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_RepresentedVariable <- R6Class(classname="DDI4_RepresentedVariable",
  inherit = DDI4_ConceptualVariable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, unitOfMeasurement=NA, hasIntendedDataType=NA,    takesSubstantiveValuesFrom=NA,drawsFrom=NA,basedOnConceptualVariable=NA,takesPlatformSpecificSentinelValues=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_RepresentedVariable may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(unitOfMeasurement)){
        private$.unitOfMeasurement <- returnVectorIfValid(validVectorClass="character", proposedVector=unitOfMeasurement , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasIntendedDataType)){
        private$.hasIntendedDataType <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=hasIntendedDataType , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(takesSubstantiveValuesFrom)){
        private$.takesSubstantiveValuesFrom <- returnUrnVectorIfValid(validClass="DDI4_SubstantiveValueDomain", proposedUrnVector=takesSubstantiveValuesFrom , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.takesSubstantiveValuesFrom, referencingAssociationName="takesSubstantiveValuesFrom",  referencingUrn=thisUrn)  
      }
      if(!missing(drawsFrom)){
        private$.drawsFrom <- returnUrnVectorIfValid(validClass="DDI4_Universe", proposedUrnVector=drawsFrom , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.drawsFrom, referencingAssociationName="drawsFrom",  referencingUrn=thisUrn)  
      }
      if(!missing(basedOnConceptualVariable)){
        private$.basedOnConceptualVariable <- returnUrnVectorIfValid(validClass="DDI4_ConceptualVariable", proposedUrnVector=basedOnConceptualVariable , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.basedOnConceptualVariable, referencingAssociationName="basedOnConceptualVariable",  referencingUrn=thisUrn)  
      }
      if(!missing(takesPlatformSpecificSentinelValues)){
        private$.takesPlatformSpecificSentinelValues <- returnUrnVectorIfValid(validClass="DDI4_SentinelValueDomain", proposedUrnVector=takesPlatformSpecificSentinelValues , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.takesPlatformSpecificSentinelValues, referencingAssociationName="takesPlatformSpecificSentinelValues",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 2 properties: 
    hasIntendedDataType = function(val){
      if (!missing(val)){
        private$.hasIntendedDataType <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasIntendedDataType
      }
    },

    unitOfMeasurement = function(val){
      if (!missing(val)){
        private$.unitOfMeasurement <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.unitOfMeasurement
      }
    },


# There are 4 class association(s):
    basedOnConceptualVariable = function(val){
      if (!missing(val)){
        private$.basedOnConceptualVariable <- returnUrnVectorIfValid(validClass="DDI4_ConceptualVariable", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.basedOnConceptualVariable, referencingAssociationName="basedOnConceptualVariable",  referencingUrn=thisUrn)  
       # invisible( private$.basedOnConceptualVariable)
      } else {
          private$.basedOnConceptualVariable
      }
    },

    drawsFrom = function(val){
      if (!missing(val)){
        private$.drawsFrom <- returnUrnVectorIfValid(validClass="DDI4_Universe", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.drawsFrom, referencingAssociationName="drawsFrom",  referencingUrn=thisUrn)  
       # invisible( private$.drawsFrom)
      } else {
          private$.drawsFrom
      }
    },

    takesPlatformSpecificSentinelValues = function(val){
      if (!missing(val)){
        private$.takesPlatformSpecificSentinelValues <- returnUrnVectorIfValid(validClass="DDI4_SentinelValueDomain", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.takesPlatformSpecificSentinelValues, referencingAssociationName="takesPlatformSpecificSentinelValues",  referencingUrn=thisUrn)  
       # invisible( private$.takesPlatformSpecificSentinelValues)
      } else {
          private$.takesPlatformSpecificSentinelValues
      }
    },

    takesSubstantiveValuesFrom = function(val){
      if (!missing(val)){
        private$.takesSubstantiveValuesFrom <- returnUrnVectorIfValid(validClass="DDI4_SubstantiveValueDomain", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.takesSubstantiveValuesFrom, referencingAssociationName="takesSubstantiveValuesFrom",  referencingUrn=thisUrn)  
       # invisible( private$.takesSubstantiveValuesFrom)
      } else {
          private$.takesSubstantiveValuesFrom
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .unitOfMeasurement = NULL,
    .hasIntendedDataType = NULL,
    .takesSubstantiveValuesFrom = NULL,
    .drawsFrom = NULL,
    .basedOnConceptualVariable = NULL,
    .takesPlatformSpecificSentinelValues = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnConceptualVariable = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ConceptualVariable", 
          structure = "DdiUrn vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definition = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        descriptiveText = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        drawsFrom = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Universe", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasIntendedDataType = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        takesPlatformSpecificSentinelValues = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_SentinelValueDomain", 
          structure = "DdiUrn vector"
        ),
        takesSentinelConceptsFrom = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SentinelConceptualDomain", 
          structure = "DdiUrn vector"
        ),
        takesSubstantiveConceptsFrom = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SubstantiveConceptualDomain", 
          structure = "DdiUrn vector"
        ),
        takesSubstantiveValuesFrom = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SubstantiveValueDomain", 
          structure = "DdiUrn vector"
        ),
        unitOfMeasurement = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        usesConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        usesUnitType = c(
          min="0", 
          max="1", 
          datatype = "DDI4_UnitType", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_RepresentedVariable


#' @title DDI4_ResourceIdentifier  Provides a means of identifying a related resource and provides the typeOfRelationship
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ResourceIdentifier}{ResourceIdentifier} 

#'
#'   Definition
#'
#'   ============
#'
#'   Provides a means of identifying a related resource and provides the typeOfRelationship.  
#'
#'   
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Standard usage may include: describesDate, isDescribedBy,  isFormatOf, isPartOf, isReferencedBy, isReplacedBy, isRequiredBy, isVersionOf, references, replaces, requires, etc.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Makes use of a controlled vocabulary for typing the relationship.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_InternationalIdentifier}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ResourceIdentifier$DdiUrn} returns the DDI URN of myDDI4_ResourceIdentifier.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{typeOfRelatedResource}}{ A read/write active binding.
#'        The type of relationship between the annotated object and the related resource. Standard usage may include: describesDate, isDescribedBy,  isFormatOf, isPartOf, isReferencedBy, isReplacedBy, isRequiredBy, isVersionOf, references, replaces, requires, etc.
#'
#'                              \code{myDDI4_ResourceIdentifier$typeOfRelatedResource <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfRelatedResource' property of 'myDDI4_ResourceIdentifier' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ResourceIdentifier$typeOfRelatedResource} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ResourceIdentifier$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, typeOfRelatedResource=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ResourceIdentifier <- R6Class(classname="DDI4_ResourceIdentifier",
  inherit = DDI4_InternationalIdentifier,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, typeOfRelatedResource=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ResourceIdentifier may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(typeOfRelatedResource)){
        private$.typeOfRelatedResource <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfRelatedResource , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 1 properties: 
    typeOfRelatedResource = function(val){
      if (!missing(val)){
        private$.typeOfRelatedResource <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.typeOfRelatedResource
      }
    }


  ),

  private = list(
    .typeOfRelatedResource = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        identifierContent = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isURI = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        managingAgency = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        typeOfRelatedResource = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ResourceIdentifier


#' @title DDI4_ResponseDomain  The possible list of values that are allowed by a Capture
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ResponseDomain}{ResponseDomain} 

#'
#'   Definition
#'
#'   ============
#'
#'   The possible list of values that are allowed by a Capture.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Yes/No, Male/Female, Age in years, Open-ended text, Temperature, BP reading
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Identifies both the sentinel and substantive value domains used for capturing the response to a question
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   GSIM & DDI: ResponseDomain
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ResponseDomain$DdiUrn} returns the DDI URN of myDDI4_ResponseDomain.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A display label for the domain. May be expressed in multiple languages.
#'
#'                              \code{myDDI4_ResponseDomain$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_ResponseDomain' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_ResponseDomain$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{intendedRepresentation}}{ A read/write active binding.
#'          Associates the ResponseDomain with the RepresentedVariable that the question will populate into an InstanceVariable when fielded.
#'
#'                              \code{myDDI4_ResponseDomain$intendedRepresentation <- c(myRepresentedVariableURN)} sets the association 'intendedRepresentation' from 'myDDI4_ResponseDomain' to a character vector of DdiUrns for R6 RepresentedVariable class instances
#'
#'                              \code{myDDI4_ResponseDomain$intendedRepresentation} returns a character vector of of DdiUrns for R6  RepresentedVariable class instances}
#'   }
#'   \describe{
#'     \item{\code{hasOutputParameter}}{ A read/write active binding.
#'          A parameter which can be used to bind the output of this response domain to another parameter to direct the flow of the captured datum.
#'
#'                              \code{myDDI4_ResponseDomain$hasOutputParameter <- c(myParameterURN)} sets the association 'hasOutputParameter' from 'myDDI4_ResponseDomain' to a character vector of DdiUrns for R6 Parameter class instances
#'
#'                              \code{myDDI4_ResponseDomain$hasOutputParameter} returns a character vector of of DdiUrns for R6  Parameter class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ResponseDomain$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, displayLabel=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ResponseDomain <- R6Class(classname="DDI4_ResponseDomain",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, displayLabel=NA,    intendedRepresentation=NA,hasOutputParameter=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ResponseDomain may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(intendedRepresentation)){
        private$.intendedRepresentation <- returnUrnVectorIfValid(validClass="DDI4_RepresentedVariable", proposedUrnVector=intendedRepresentation , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.intendedRepresentation, referencingAssociationName="intendedRepresentation",  referencingUrn=thisUrn)  
      }
      if(!missing(hasOutputParameter)){
        private$.hasOutputParameter <- returnUrnVectorIfValid(validClass="DDI4_Parameter", proposedUrnVector=hasOutputParameter , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasOutputParameter, referencingAssociationName="hasOutputParameter",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.displayLabel
      }
    },


# There are 2 class association(s):
    hasOutputParameter = function(val){
      if (!missing(val)){
        private$.hasOutputParameter <- returnUrnVectorIfValid(validClass="DDI4_Parameter", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasOutputParameter, referencingAssociationName="hasOutputParameter",  referencingUrn=thisUrn)  
       # invisible( private$.hasOutputParameter)
      } else {
          private$.hasOutputParameter
      }
    },

    intendedRepresentation = function(val){
      if (!missing(val)){
        private$.intendedRepresentation <- returnUrnVectorIfValid(validClass="DDI4_RepresentedVariable", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.intendedRepresentation, referencingAssociationName="intendedRepresentation",  referencingUrn=thisUrn)  
       # invisible( private$.intendedRepresentation)
      } else {
          private$.intendedRepresentation
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .displayLabel = NULL,
    .intendedRepresentation = NULL,
    .hasOutputParameter = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="1", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        intendedRepresentation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RepresentedVariable", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ResponseDomain


#' @title DDI4_Result  Describes the results of a process for the purpose of linking these results to guidance for future usage in specified situations
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Result}{Result} 

#'
#'   Definition
#'
#'   ============
#'
#'   Describes the results of a process for the purpose of linking these results to guidance for future usage in specified situations. Result is abstract and serves as a substitution base for the specified result of a specific instantiation of a methodology process.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   The use of a weight resulting from a sampling process by an analyst within the context of a specific set of variables. The class containing the weight would use the extension base Result, adding any additional required properties and relationships to describe it.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Result$DdiUrn} returns the DDI URN of myDDI4_Result.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{overview}}{ A read/write active binding.
#'        Provides a high level overview or summary of the class. Can be used to inform end-users or as part of an executive summary. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_Result$overview <- list(myInternationalStructuredString)} sets the 'overview' property of 'myDDI4_Result' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Result$overview} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{hasBinding}}{ A read/write active binding.
#'        Can bind data attached to parameters into, out of, and between processes.
#'
#'                              \code{myDDI4_Result$hasBinding <- list(myBinding)} sets the 'hasBinding' property of 'myDDI4_Result' to a list of R6 Binding class instances
#'
#'                              \code{myDDI4_Result$hasBinding} returns a list of of R6 : Binding class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasAppliedUse}}{ A read/write active binding.
#'          The result may have one or more specific usages which provides guidance for the use of the result with a specific unit type.
#'
#'                              \code{myDDI4_Result$hasAppliedUse <- c(myAppliedUseURN)} sets the association 'hasAppliedUse' from 'myDDI4_Result' to a character vector of DdiUrns for R6 AppliedUse class instances
#'
#'                              \code{myDDI4_Result$hasAppliedUse} returns a character vector of of DdiUrns for R6  AppliedUse class instances}
#'   }
#'   \describe{
#'     \item{\code{evaluateAgainstGoal}}{ A read/write active binding.
#'          Comparison of Result of the Process against the Goal to inform further activities.
#'
#'                              \code{myDDI4_Result$evaluateAgainstGoal <- c(myGoalURN)} sets the association 'evaluateAgainstGoal' from 'myDDI4_Result' to a character vector of DdiUrns for R6 Goal class instances
#'
#'                              \code{myDDI4_Result$evaluateAgainstGoal} returns a character vector of of DdiUrns for R6  Goal class instances}
#'   }
#'   \describe{
#'     \item{\code{hasInputParameter}}{ A read/write active binding.
#'          Used by Binding to link the output of another class to the input of this class 
#'
#'                              \code{myDDI4_Result$hasInputParameter <- c(myParameterURN)} sets the association 'hasInputParameter' from 'myDDI4_Result' to a character vector of DdiUrns for R6 Parameter class instances
#'
#'                              \code{myDDI4_Result$hasInputParameter} returns a character vector of of DdiUrns for R6  Parameter class instances}
#'   }
#'   \describe{
#'     \item{\code{hasOutputParameter}}{ A read/write active binding.
#'          Used by Binding to link the output of this class to the input of another class 
#'
#'                              \code{myDDI4_Result$hasOutputParameter <- c(myParameterURN)} sets the association 'hasOutputParameter' from 'myDDI4_Result' to a character vector of DdiUrns for R6 Parameter class instances
#'
#'                              \code{myDDI4_Result$hasOutputParameter} returns a character vector of of DdiUrns for R6  Parameter class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Result$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, overview=NA, hasBinding=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Result <- R6Class(classname="DDI4_Result",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, overview=NA, hasBinding=NA,    hasAppliedUse=NA,evaluateAgainstGoal=NA,hasInputParameter=NA,hasOutputParameter=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Result may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(overview)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=overview , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasBinding)){
        private$.hasBinding <- returnR6ObjectIfValid(validClass="DDI4_Binding", proposedObjectList=hasBinding , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(hasAppliedUse)){
        private$.hasAppliedUse <- returnUrnVectorIfValid(validClass="DDI4_AppliedUse", proposedUrnVector=hasAppliedUse , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasAppliedUse, referencingAssociationName="hasAppliedUse",  referencingUrn=thisUrn)  
      }
      if(!missing(evaluateAgainstGoal)){
        private$.evaluateAgainstGoal <- returnUrnVectorIfValid(validClass="DDI4_Goal", proposedUrnVector=evaluateAgainstGoal , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.evaluateAgainstGoal, referencingAssociationName="evaluateAgainstGoal",  referencingUrn=thisUrn)  
      }
      if(!missing(hasInputParameter)){
        private$.hasInputParameter <- returnUrnVectorIfValid(validClass="DDI4_Parameter", proposedUrnVector=hasInputParameter , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasInputParameter, referencingAssociationName="hasInputParameter",  referencingUrn=thisUrn)  
      }
      if(!missing(hasOutputParameter)){
        private$.hasOutputParameter <- returnUrnVectorIfValid(validClass="DDI4_Parameter", proposedUrnVector=hasOutputParameter , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasOutputParameter, referencingAssociationName="hasOutputParameter",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 2 properties: 
    hasBinding = function(val){
      if (!missing(val)){
        private$.hasBinding <- returnR6ObjectIfValid(validClass="DDI4_Binding", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasBinding
      }
    },

    overview = function(val){
      if (!missing(val)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overview
      }
    },


# There are 4 class association(s):
    evaluateAgainstGoal = function(val){
      if (!missing(val)){
        private$.evaluateAgainstGoal <- returnUrnVectorIfValid(validClass="DDI4_Goal", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.evaluateAgainstGoal, referencingAssociationName="evaluateAgainstGoal",  referencingUrn=thisUrn)  
       # invisible( private$.evaluateAgainstGoal)
      } else {
          private$.evaluateAgainstGoal
      }
    },

    hasAppliedUse = function(val){
      if (!missing(val)){
        private$.hasAppliedUse <- returnUrnVectorIfValid(validClass="DDI4_AppliedUse", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasAppliedUse, referencingAssociationName="hasAppliedUse",  referencingUrn=thisUrn)  
       # invisible( private$.hasAppliedUse)
      } else {
          private$.hasAppliedUse
      }
    },

    hasInputParameter = function(val){
      if (!missing(val)){
        private$.hasInputParameter <- returnUrnVectorIfValid(validClass="DDI4_Parameter", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasInputParameter, referencingAssociationName="hasInputParameter",  referencingUrn=thisUrn)  
       # invisible( private$.hasInputParameter)
      } else {
          private$.hasInputParameter
      }
    },

    hasOutputParameter = function(val){
      if (!missing(val)){
        private$.hasOutputParameter <- returnUrnVectorIfValid(validClass="DDI4_Parameter", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasOutputParameter, referencingAssociationName="hasOutputParameter",  referencingUrn=thisUrn)  
       # invisible( private$.hasOutputParameter)
      } else {
          private$.hasOutputParameter
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .overview = NULL,
    .hasBinding = NULL,
    .hasAppliedUse = NULL,
    .evaluateAgainstGoal = NULL,
    .hasInputParameter = NULL,
    .hasOutputParameter = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        evaluateAgainstGoal = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Goal", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAppliedUse = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AppliedUse", 
          structure = "DdiUrn vector"
        ),
        hasBinding = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_Result


#' @title DDI4_SampleFrame  A set of information used to identify a sample population within a larger population
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/SampleFrame}{SampleFrame} 

#'
#'   Definition
#'
#'   ============
#'
#'   A set of information used to identify a sample population within a larger population. The sample frame will contain identifying information for individual units within the frame as well as characteristics which will support analysis and the division of the frame population into sub-groups.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A listing of all business enterprises by their primary office address with information on their industry classification, work staff size, and production costs. A telephone directory. An address list of housing units.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_SampleFrame$DdiUrn} returns the DDI URN of myDDI4_SampleFrame.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage.
#'
#'                              \code{myDDI4_SampleFrame$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_SampleFrame' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_SampleFrame$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A structured display label providing a fully human readable display for the identification of the object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_SampleFrame$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_SampleFrame' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_SampleFrame$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#'   \describe{
#'     \item{\code{usage}}{ A read/write active binding.
#'        Explanation of the ways in which the sample frame is to be employed. Supports use of multiple languages and structured text.
#'
#'                              \code{myDDI4_SampleFrame$usage <- list(myInternationalStructuredString)} sets the 'usage' property of 'myDDI4_SampleFrame' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_SampleFrame$usage} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        An explanation of the intent of the sample frame. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_SampleFrame$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_SampleFrame' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_SampleFrame$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{limitations}}{ A read/write active binding.
#'        An explanation of any limitations of the sample frame. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_SampleFrame$limitations <- list(myInternationalStructuredString)} sets the 'limitations' property of 'myDDI4_SampleFrame' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_SampleFrame$limitations} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{updateProcedures}}{ A read/write active binding.
#'        A description of how the sample frame is updated and maintained. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_SampleFrame$updateProcedures <- list(myInternationalStructuredString)} sets the 'updateProcedures' property of 'myDDI4_SampleFrame' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_SampleFrame$updateProcedures} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{referencePeriod}}{ A read/write active binding.
#'        The reference period for the sample frame. 
#'
#'                              \code{myDDI4_SampleFrame$referencePeriod <- list(myDateRange)} sets the 'referencePeriod' property of 'myDDI4_SampleFrame' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_SampleFrame$referencePeriod} returns a list of of R6 : DateRange class instances}
#'   }
#'   \describe{
#'     \item{\code{validPeriod}}{ A read/write active binding.
#'        The period for which the current version is valid
#'
#'                              \code{myDDI4_SampleFrame$validPeriod <- list(myDateRange)} sets the 'validPeriod' property of 'myDDI4_SampleFrame' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_SampleFrame$validPeriod} returns a list of of R6 : DateRange class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasCustodian}}{ A read/write active binding.
#'          The agent who is primarily responsible for the maintenance of the sample frame
#'
#'                              \code{myDDI4_SampleFrame$hasCustodian <- c(myAgentURN)} sets the association 'hasCustodian' from 'myDDI4_SampleFrame' to a character vector of DdiUrns for R6 Agent class instances
#'
#'                              \code{myDDI4_SampleFrame$hasCustodian} returns a character vector of of DdiUrns for R6  Agent class instances}
#'   }
#'   \describe{
#'     \item{\code{hasAccess}}{ A read/write active binding.
#'          Accessibility of the sample frame including means of accessing
#'
#'                              \code{myDDI4_SampleFrame$hasAccess <- c(myAccessURN)} sets the association 'hasAccess' from 'myDDI4_SampleFrame' to a character vector of DdiUrns for R6 Access class instances
#'
#'                              \code{myDDI4_SampleFrame$hasAccess} returns a character vector of of DdiUrns for R6  Access class instances}
#'   }
#'   \describe{
#'     \item{\code{hasPopulation}}{ A read/write active binding.
#'          The population included in the sample frame
#'
#'                              \code{myDDI4_SampleFrame$hasPopulation <- c(myPopulationURN)} sets the association 'hasPopulation' from 'myDDI4_SampleFrame' to a character vector of DdiUrns for R6 Population class instances
#'
#'                              \code{myDDI4_SampleFrame$hasPopulation} returns a character vector of of DdiUrns for R6  Population class instances}
#'   }
#'   \describe{
#'     \item{\code{hasPrimaryUnitType}}{ A read/write active binding.
#'          The primary unit type of the population
#'
#'                              \code{myDDI4_SampleFrame$hasPrimaryUnitType <- c(myUnitTypeURN)} sets the association 'hasPrimaryUnitType' from 'myDDI4_SampleFrame' to a character vector of DdiUrns for R6 UnitType class instances
#'
#'                              \code{myDDI4_SampleFrame$hasPrimaryUnitType} returns a character vector of of DdiUrns for R6  UnitType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasSecondaryUnitType}}{ A read/write active binding.
#'          A secondary unit type identifiable with the population
#'
#'                              \code{myDDI4_SampleFrame$hasSecondaryUnitType <- c(myUnitTypeURN)} sets the association 'hasSecondaryUnitType' from 'myDDI4_SampleFrame' to a character vector of DdiUrns for R6 UnitType class instances
#'
#'                              \code{myDDI4_SampleFrame$hasSecondaryUnitType} returns a character vector of of DdiUrns for R6  UnitType class instances}
#'   }
#'   \describe{
#'     \item{\code{sourceFrame}}{ A read/write active binding.
#'          A sample frame which served as a source for the sample frame being described
#'
#'                              \code{myDDI4_SampleFrame$sourceFrame <- c(mySampleFrameURN)} sets the association 'sourceFrame' from 'myDDI4_SampleFrame' to a character vector of DdiUrns for R6 SampleFrame class instances
#'
#'                              \code{myDDI4_SampleFrame$sourceFrame} returns a character vector of of DdiUrns for R6  SampleFrame class instances}
#'   }
#'   \describe{
#'     \item{\code{hasInputParameter}}{ A read/write active binding.
#'          SampleFrame can serve as an input to a sampling process.
#'
#'                              \code{myDDI4_SampleFrame$hasInputParameter <- c(myParameterURN)} sets the association 'hasInputParameter' from 'myDDI4_SampleFrame' to a character vector of DdiUrns for R6 Parameter class instances
#'
#'                              \code{myDDI4_SampleFrame$hasInputParameter} returns a character vector of of DdiUrns for R6  Parameter class instances}
#'   }
#'   \describe{
#'     \item{\code{hasOutputParameter}}{ A read/write active binding.
#'          SampleFrame can be an output related to a sampling process.
#'
#'                              \code{myDDI4_SampleFrame$hasOutputParameter <- c(myParameterURN)} sets the association 'hasOutputParameter' from 'myDDI4_SampleFrame' to a character vector of DdiUrns for R6 Parameter class instances
#'
#'                              \code{myDDI4_SampleFrame$hasOutputParameter} returns a character vector of of DdiUrns for R6  Parameter class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_SampleFrame$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, name=NA, displayLabel=NA, usage=NA, purpose=NA, limitations=NA, updateProcedures=NA, referencePeriod=NA, validPeriod=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_SampleFrame <- R6Class(classname="DDI4_SampleFrame",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, name=NA, displayLabel=NA, usage=NA, purpose=NA, limitations=NA, updateProcedures=NA, referencePeriod=NA, validPeriod=NA,    hasCustodian=NA,hasAccess=NA,hasPopulation=NA,hasPrimaryUnitType=NA,hasSecondaryUnitType=NA,sourceFrame=NA,hasInputParameter=NA,hasOutputParameter=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_SampleFrame may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(usage)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=usage , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(limitations)){
        private$.limitations <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=limitations , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(updateProcedures)){
        private$.updateProcedures <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=updateProcedures , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(referencePeriod)){
        private$.referencePeriod <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=referencePeriod , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(validPeriod)){
        private$.validPeriod <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=validPeriod , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(hasCustodian)){
        private$.hasCustodian <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=hasCustodian , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasCustodian, referencingAssociationName="hasCustodian",  referencingUrn=thisUrn)  
      }
      if(!missing(hasAccess)){
        private$.hasAccess <- returnUrnVectorIfValid(validClass="DDI4_Access", proposedUrnVector=hasAccess , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasAccess, referencingAssociationName="hasAccess",  referencingUrn=thisUrn)  
      }
      if(!missing(hasPopulation)){
        private$.hasPopulation <- returnUrnVectorIfValid(validClass="DDI4_Population", proposedUrnVector=hasPopulation , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasPopulation, referencingAssociationName="hasPopulation",  referencingUrn=thisUrn)  
      }
      if(!missing(hasPrimaryUnitType)){
        private$.hasPrimaryUnitType <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=hasPrimaryUnitType , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasPrimaryUnitType, referencingAssociationName="hasPrimaryUnitType",  referencingUrn=thisUrn)  
      }
      if(!missing(hasSecondaryUnitType)){
        private$.hasSecondaryUnitType <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=hasSecondaryUnitType , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasSecondaryUnitType, referencingAssociationName="hasSecondaryUnitType",  referencingUrn=thisUrn)  
      }
      if(!missing(sourceFrame)){
        private$.sourceFrame <- returnUrnVectorIfValid(validClass="DDI4_SampleFrame", proposedUrnVector=sourceFrame , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.sourceFrame, referencingAssociationName="sourceFrame",  referencingUrn=thisUrn)  
      }
      if(!missing(hasInputParameter)){
        private$.hasInputParameter <- returnUrnVectorIfValid(validClass="DDI4_Parameter", proposedUrnVector=hasInputParameter , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasInputParameter, referencingAssociationName="hasInputParameter",  referencingUrn=thisUrn)  
      }
      if(!missing(hasOutputParameter)){
        private$.hasOutputParameter <- returnUrnVectorIfValid(validClass="DDI4_Parameter", proposedUrnVector=hasOutputParameter , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasOutputParameter, referencingAssociationName="hasOutputParameter",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 8 properties: 
    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },

    limitations = function(val){
      if (!missing(val)){
        private$.limitations <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.limitations
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    referencePeriod = function(val){
      if (!missing(val)){
        private$.referencePeriod <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.referencePeriod
      }
    },

    updateProcedures = function(val){
      if (!missing(val)){
        private$.updateProcedures <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.updateProcedures
      }
    },

    usage = function(val){
      if (!missing(val)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.usage
      }
    },

    validPeriod = function(val){
      if (!missing(val)){
        private$.validPeriod <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.validPeriod
      }
    },


# There are 8 class association(s):
    hasAccess = function(val){
      if (!missing(val)){
        private$.hasAccess <- returnUrnVectorIfValid(validClass="DDI4_Access", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasAccess, referencingAssociationName="hasAccess",  referencingUrn=thisUrn)  
       # invisible( private$.hasAccess)
      } else {
          private$.hasAccess
      }
    },

    hasCustodian = function(val){
      if (!missing(val)){
        private$.hasCustodian <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasCustodian, referencingAssociationName="hasCustodian",  referencingUrn=thisUrn)  
       # invisible( private$.hasCustodian)
      } else {
          private$.hasCustodian
      }
    },

    hasInputParameter = function(val){
      if (!missing(val)){
        private$.hasInputParameter <- returnUrnVectorIfValid(validClass="DDI4_Parameter", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasInputParameter, referencingAssociationName="hasInputParameter",  referencingUrn=thisUrn)  
       # invisible( private$.hasInputParameter)
      } else {
          private$.hasInputParameter
      }
    },

    hasOutputParameter = function(val){
      if (!missing(val)){
        private$.hasOutputParameter <- returnUrnVectorIfValid(validClass="DDI4_Parameter", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasOutputParameter, referencingAssociationName="hasOutputParameter",  referencingUrn=thisUrn)  
       # invisible( private$.hasOutputParameter)
      } else {
          private$.hasOutputParameter
      }
    },

    hasPopulation = function(val){
      if (!missing(val)){
        private$.hasPopulation <- returnUrnVectorIfValid(validClass="DDI4_Population", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasPopulation, referencingAssociationName="hasPopulation",  referencingUrn=thisUrn)  
       # invisible( private$.hasPopulation)
      } else {
          private$.hasPopulation
      }
    },

    hasPrimaryUnitType = function(val){
      if (!missing(val)){
        private$.hasPrimaryUnitType <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasPrimaryUnitType, referencingAssociationName="hasPrimaryUnitType",  referencingUrn=thisUrn)  
       # invisible( private$.hasPrimaryUnitType)
      } else {
          private$.hasPrimaryUnitType
      }
    },

    hasSecondaryUnitType = function(val){
      if (!missing(val)){
        private$.hasSecondaryUnitType <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasSecondaryUnitType, referencingAssociationName="hasSecondaryUnitType",  referencingUrn=thisUrn)  
       # invisible( private$.hasSecondaryUnitType)
      } else {
          private$.hasSecondaryUnitType
      }
    },

    sourceFrame = function(val){
      if (!missing(val)){
        private$.sourceFrame <- returnUrnVectorIfValid(validClass="DDI4_SampleFrame", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.sourceFrame, referencingAssociationName="sourceFrame",  referencingUrn=thisUrn)  
       # invisible( private$.sourceFrame)
      } else {
          private$.sourceFrame
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .name = NULL,
    .displayLabel = NULL,
    .usage = NULL,
    .purpose = NULL,
    .limitations = NULL,
    .updateProcedures = NULL,
    .referencePeriod = NULL,
    .validPeriod = NULL,
    .hasCustodian = NULL,
    .hasAccess = NULL,
    .hasPopulation = NULL,
    .hasPrimaryUnitType = NULL,
    .hasSecondaryUnitType = NULL,
    .sourceFrame = NULL,
    .hasInputParameter = NULL,
    .hasOutputParameter = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAccess = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Access", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasCustodian = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Agent", 
          structure = "DdiUrn vector"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasPopulation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Population", 
          structure = "DdiUrn vector"
        ),
        hasPrimaryUnitType = c(
          min="0", 
          max="1", 
          datatype = "DDI4_UnitType", 
          structure = "DdiUrn vector"
        ),
        hasSecondaryUnitType = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_UnitType", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        limitations = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        referencePeriod = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        sourceFrame = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_SampleFrame", 
          structure = "DdiUrn vector"
        ),
        updateProcedures = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        validPeriod = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_SampleFrame


#' @title DDI4_SamplePopulationResult  The Sample Population is the result of a sampling process
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/SamplePopulationResult}{SamplePopulationResult} 

#'
#'   Definition
#'
#'   ============
#'
#'   The Sample Population is the result of a sampling process. It is the subset of the population selected from the Sample Frame or Frames that will be used as respondents to represent the target population. Sample Populations may occur at a number of stages in the sampling process and serve as input to a subsequent sampling stage. The Sample Population should be validated against the intended sample population as described in the goal of the design and/or process.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A list of phone numbers representing a random sample from a telephone book. The Counties selected from within a State from which Households will be selected using another sampling method.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Result}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_SamplePopulationResult$DdiUrn} returns the DDI URN of myDDI4_SamplePopulationResult.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{sizeOfSample}}{ A read/write active binding.
#'        The overall size of the sample
#'
#'                              \code{myDDI4_SamplePopulationResult$sizeOfSample <- myinteger} sets the sizeOfSample property of 'myDDI4_SamplePopulationResult' to the vector of datatype:integer
#'
#'                              \code{myDDI4_SamplePopulationResult$sizeOfSample} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{strataName}}{ A read/write active binding.
#'        The name or number of the specific strata if the sample frame has been stratified prior to sampling within the specified stage
#'
#'                              \code{myDDI4_SamplePopulationResult$strataName <- mycharacter} sets the strataName property of 'myDDI4_SamplePopulationResult' to the vector of datatype:character
#'
#'                              \code{myDDI4_SamplePopulationResult$strataName} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{sampleDate}}{ A read/write active binding.
#'        The date the sample was selected from the sample frame
#'
#'                              \code{myDDI4_SamplePopulationResult$sampleDate <- list(myDate)} sets the 'sampleDate' property of 'myDDI4_SamplePopulationResult' to a list of R6 Date class instances
#'
#'                              \code{myDDI4_SamplePopulationResult$sampleDate} returns a list of of R6 : Date class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{sampleUnitType}}{ A read/write active binding.
#'          The Unit Type of the units within the sample.
#'
#'                              \code{myDDI4_SamplePopulationResult$sampleUnitType <- c(myUnitTypeURN)} sets the association 'sampleUnitType' from 'myDDI4_SamplePopulationResult' to a character vector of DdiUrns for R6 UnitType class instances
#'
#'                              \code{myDDI4_SamplePopulationResult$sampleUnitType} returns a character vector of of DdiUrns for R6  UnitType class instances}
#'   }
#'   \describe{
#'     \item{\code{populationSampled}}{ A read/write active binding.
#'          The population represented by the sample
#'
#'                              \code{myDDI4_SamplePopulationResult$populationSampled <- c(myPopulationURN)} sets the association 'populationSampled' from 'myDDI4_SamplePopulationResult' to a character vector of DdiUrns for R6 Population class instances
#'
#'                              \code{myDDI4_SamplePopulationResult$populationSampled} returns a character vector of of DdiUrns for R6  Population class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_SamplePopulationResult$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, sizeOfSample=NA, strataName=NA, sampleDate=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_SamplePopulationResult <- R6Class(classname="DDI4_SamplePopulationResult",
  inherit = DDI4_Result,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, sizeOfSample=NA, strataName=NA, sampleDate=NA,    sampleUnitType=NA,populationSampled=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_SamplePopulationResult may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(sizeOfSample)){
        private$.sizeOfSample <- returnVectorIfValid(validVectorClass="integer", proposedVector=sizeOfSample , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(strataName)){
        private$.strataName <- returnVectorIfValid(validVectorClass="character", proposedVector=strataName , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(sampleDate)){
        private$.sampleDate <- returnR6ObjectIfValid(validClass="DDI4_Date", proposedObjectList=sampleDate , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(sampleUnitType)){
        private$.sampleUnitType <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=sampleUnitType , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.sampleUnitType, referencingAssociationName="sampleUnitType",  referencingUrn=thisUrn)  
      }
      if(!missing(populationSampled)){
        private$.populationSampled <- returnUrnVectorIfValid(validClass="DDI4_Population", proposedUrnVector=populationSampled , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.populationSampled, referencingAssociationName="populationSampled",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    sampleDate = function(val){
      if (!missing(val)){
        private$.sampleDate <- returnR6ObjectIfValid(validClass="DDI4_Date", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.sampleDate
      }
    },

    sizeOfSample = function(val){
      if (!missing(val)){
        private$.sizeOfSample <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.sizeOfSample
      }
    },

    strataName = function(val){
      if (!missing(val)){
        private$.strataName <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.strataName
      }
    },


# There are 2 class association(s):
    populationSampled = function(val){
      if (!missing(val)){
        private$.populationSampled <- returnUrnVectorIfValid(validClass="DDI4_Population", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.populationSampled, referencingAssociationName="populationSampled",  referencingUrn=thisUrn)  
       # invisible( private$.populationSampled)
      } else {
          private$.populationSampled
      }
    },

    sampleUnitType = function(val){
      if (!missing(val)){
        private$.sampleUnitType <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.sampleUnitType, referencingAssociationName="sampleUnitType",  referencingUrn=thisUrn)  
       # invisible( private$.sampleUnitType)
      } else {
          private$.sampleUnitType
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .sizeOfSample = NULL,
    .strataName = NULL,
    .sampleDate = NULL,
    .sampleUnitType = NULL,
    .populationSampled = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        evaluateAgainstGoal = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Goal", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAppliedUse = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AppliedUse", 
          structure = "DdiUrn vector"
        ),
        hasBinding = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        populationSampled = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Population", 
          structure = "DdiUrn vector"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        sampleDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Date", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        sampleUnitType = c(
          min="0", 
          max="1", 
          datatype = "DDI4_UnitType", 
          structure = "DdiUrn vector"
        ),
        sizeOfSample = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        strataName = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_SamplePopulationResult


#' @title DDI4_SamplingAlgorithm  Generic description of a sampling algorithm expressed by a Sampling Design and implemented by a Sampling Process
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/SamplingAlgorithm}{SamplingAlgorithm} 

#'
#'   Definition
#'
#'   ============
#'
#'   Generic description of a sampling algorithm expressed by a Sampling Design and implemented by a Sampling Process. May include common algebraic formula or statistical package instructions (use of specific selection models).
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Random Sample; Stratified Random Sample
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AlgorithmOverview}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_SamplingAlgorithm$DdiUrn} returns the DDI URN of myDDI4_SamplingAlgorithm.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{codifiedExpressionOfAlgorithm}}{ A read/write active binding.
#'        The algorithm expressed as algebraic or computer code.
#'
#'                              \code{myDDI4_SamplingAlgorithm$codifiedExpressionOfAlgorithm <- list(myCommandCode)} sets the 'codifiedExpressionOfAlgorithm' property of 'myDDI4_SamplingAlgorithm' to a list of R6 CommandCode class instances
#'
#'                              \code{myDDI4_SamplingAlgorithm$codifiedExpressionOfAlgorithm} returns a list of of R6 : CommandCode class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_SamplingAlgorithm$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, codifiedExpressionOfAlgorithm=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_SamplingAlgorithm <- R6Class(classname="DDI4_SamplingAlgorithm",
  inherit = DDI4_AlgorithmOverview,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, codifiedExpressionOfAlgorithm=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_SamplingAlgorithm may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(codifiedExpressionOfAlgorithm)){
        private$.codifiedExpressionOfAlgorithm <- returnR6ObjectIfValid(validClass="DDI4_CommandCode", proposedObjectList=codifiedExpressionOfAlgorithm , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 1 properties: 
    codifiedExpressionOfAlgorithm = function(val){
      if (!missing(val)){
        private$.codifiedExpressionOfAlgorithm <- returnR6ObjectIfValid(validClass="DDI4_CommandCode", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.codifiedExpressionOfAlgorithm
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .codifiedExpressionOfAlgorithm = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        codifiedExpressionOfAlgorithm = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CommandCode", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isDescribedIn = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        subjectOfAlgorithm = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_SamplingAlgorithm


#' @title DDI4_SamplingDesign  Sampling plan defines the how the sample is to be obtained through the description of methodology or a model based approach, defining the intended target population, stratification or split procedures and recommended sample frames
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/SamplingDesign}{SamplingDesign} 

#'
#'   Definition
#'
#'   ============
#'
#'   Sampling plan defines the how the sample is to be obtained through the description of methodology or a model based approach, defining the intended target population, stratification or split procedures and recommended sample frames. The content of a sampling plan is intended to be reusable. The application of the sample plan is captured in Sampling Process.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_DesignOverview}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{expressesAlgorithm}}{ A read/write active binding.
#'          Is an expression of this Sampling Algorithm
#'
#'                              \code{myDDI4_SamplingDesign$expressesAlgorithm <- c(mySamplingAlgorithmURN)} sets the association 'expressesAlgorithm' from 'myDDI4_SamplingDesign' to a character vector of DdiUrns for R6 SamplingAlgorithm class instances
#'
#'                              \code{myDDI4_SamplingDesign$expressesAlgorithm} returns a character vector of of DdiUrns for R6  SamplingAlgorithm class instances}
#'   }
#'   \describe{
#'     \item{\code{specifiesGoal}}{ A read/write active binding.
#'          Specification of the goal for the Sampling Design. Constrained to SamplingGoal.
#'
#'                              \code{myDDI4_SamplingDesign$specifiesGoal <- c(mySamplingGoalURN)} sets the association 'specifiesGoal' from 'myDDI4_SamplingDesign' to a character vector of DdiUrns for R6 SamplingGoal class instances
#'
#'                              \code{myDDI4_SamplingDesign$specifiesGoal} returns a character vector of of DdiUrns for R6  SamplingGoal class instances}
#'   }
#'   \describe{
#'     \item{\code{implementedBy}}{ A read/write active binding.
#'          The design of the expressed algorithm is implemented by the Sampling Process. Constrains impliementedBy of the Design Pattern by limiting the target to a Sampling Process.
#'
#'                              \code{myDDI4_SamplingDesign$implementedBy <- c(mySamplingProcessURN)} sets the association 'implementedBy' from 'myDDI4_SamplingDesign' to a character vector of DdiUrns for R6 SamplingProcess class instances
#'
#'                              \code{myDDI4_SamplingDesign$implementedBy} returns a character vector of of DdiUrns for R6  SamplingProcess class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_SamplingDesign$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_SamplingDesign <- R6Class(classname="DDI4_SamplingDesign",
  inherit = DDI4_DesignOverview,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, expressesAlgorithm=NA,specifiesGoal=NA,implementedBy=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_SamplingDesign may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)



       if(!missing(expressesAlgorithm)){
        private$.expressesAlgorithm <- returnUrnVectorIfValid(validClass="DDI4_SamplingAlgorithm", proposedUrnVector=expressesAlgorithm , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.expressesAlgorithm, referencingAssociationName="expressesAlgorithm",  referencingUrn=thisUrn)  
      }
      if(!missing(specifiesGoal)){
        private$.specifiesGoal <- returnUrnVectorIfValid(validClass="DDI4_SamplingGoal", proposedUrnVector=specifiesGoal , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.specifiesGoal, referencingAssociationName="specifiesGoal",  referencingUrn=thisUrn)  
      }
      if(!missing(implementedBy)){
        private$.implementedBy <- returnUrnVectorIfValid(validClass="DDI4_SamplingProcess", proposedUrnVector=implementedBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.implementedBy, referencingAssociationName="implementedBy",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 class association(s):
    expressesAlgorithm = function(val){
      if (!missing(val)){
        private$.expressesAlgorithm <- returnUrnVectorIfValid(validClass="DDI4_SamplingAlgorithm", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.expressesAlgorithm, referencingAssociationName="expressesAlgorithm",  referencingUrn=thisUrn)  
       # invisible( private$.expressesAlgorithm)
      } else {
          private$.expressesAlgorithm
      }
    },

    implementedBy = function(val){
      if (!missing(val)){
        private$.implementedBy <- returnUrnVectorIfValid(validClass="DDI4_SamplingProcess", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.implementedBy, referencingAssociationName="implementedBy",  referencingUrn=thisUrn)  
       # invisible( private$.implementedBy)
      } else {
          private$.implementedBy
      }
    },

    specifiesGoal = function(val){
      if (!missing(val)){
        private$.specifiesGoal <- returnUrnVectorIfValid(validClass="DDI4_SamplingGoal", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.specifiesGoal, referencingAssociationName="specifiesGoal",  referencingUrn=thisUrn)  
       # invisible( private$.specifiesGoal)
      } else {
          private$.specifiesGoal
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .expressesAlgorithm = NULL,
    .specifiesGoal = NULL,
    .implementedBy = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        assumesPrecondition = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Precondition", 
          structure = "DdiUrn vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        expressesAlgorithm = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_SamplingAlgorithm", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        implementedBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_SamplingProcess", 
          structure = "DdiUrn vector"
        ),
        isDescribedIn = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        specifiesGoal = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_SamplingGoal", 
          structure = "DdiUrn vector"
        ),
        subectOfDesign = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_SamplingDesign


#' @title DDI4_SamplingGoal  Provides general description and specific targets for sample sizes including sub-populations
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/SamplingGoal}{SamplingGoal} 

#'
#'   Definition
#'
#'   ============
#'
#'   Provides general description and specific targets for sample sizes including sub-populations.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Goal}}}
#'    \item{\code{\link{DDI4_BusinessFunction}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_SamplingGoal$DdiUrn} returns the DDI URN of myDDI4_SamplingGoal.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{overallTargetSampleSize}}{ A read/write active binding.
#'        The goal for the overall size of the sample expressed as an integer.
#'
#'                              \code{myDDI4_SamplingGoal$overallTargetSampleSize <- myinteger} sets the overallTargetSampleSize property of 'myDDI4_SamplingGoal' to the vector of datatype:integer
#'
#'                              \code{myDDI4_SamplingGoal$overallTargetSampleSize} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{overallTargetSamplePercent}}{ A read/write active binding.
#'        The goal for the overall size of the sample expressed as a decimal (i.e. 70% entered as .7). 
#'
#'                              \code{myDDI4_SamplingGoal$overallTargetSamplePercent <- mynumeric} sets the overallTargetSamplePercent property of 'myDDI4_SamplingGoal' to the vector of datatype:numeric
#'
#'                              \code{myDDI4_SamplingGoal$overallTargetSamplePercent} returns a vector of datatype: numeric}
#'   }
#'   \describe{
#'     \item{\code{targetSampleSize}}{ A read/write active binding.
#'        Repeat for primary and secondary target samples. Supports identification of size, percent, unit type, and universe
#'
#'                              \code{myDDI4_SamplingGoal$targetSampleSize <- list(myTargetSample)} sets the 'targetSampleSize' property of 'myDDI4_SamplingGoal' to a list of R6 TargetSample class instances
#'
#'                              \code{myDDI4_SamplingGoal$targetSampleSize} returns a list of of R6 : TargetSample class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_SamplingGoal$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, overallTargetSampleSize=NA, overallTargetSamplePercent=NA, targetSampleSize=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_SamplingGoal <- R6Class(classname="DDI4_SamplingGoal",
  inherit = DDI4_Goal,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, overallTargetSampleSize=NA, overallTargetSamplePercent=NA, targetSampleSize=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_SamplingGoal may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(overallTargetSampleSize)){
        private$.overallTargetSampleSize <- returnVectorIfValid(validVectorClass="integer", proposedVector=overallTargetSampleSize , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(overallTargetSamplePercent)){
        private$.overallTargetSamplePercent <- returnVectorIfValid(validVectorClass="numeric", proposedVector=overallTargetSamplePercent , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(targetSampleSize)){
        private$.targetSampleSize <- returnR6ObjectIfValid(validClass="DDI4_TargetSample", proposedObjectList=targetSampleSize , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    overallTargetSamplePercent = function(val){
      if (!missing(val)){
        private$.overallTargetSamplePercent <- returnVectorIfValid(validVectorClass="numeric", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overallTargetSamplePercent
      }
    },

    overallTargetSampleSize = function(val){
      if (!missing(val)){
        private$.overallTargetSampleSize <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overallTargetSampleSize
      }
    },

    targetSampleSize = function(val){
      if (!missing(val)){
        private$.targetSampleSize <- returnR6ObjectIfValid(validClass="DDI4_TargetSample", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.targetSampleSize
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .overallTargetSampleSize = NULL,
    .overallTargetSamplePercent = NULL,
    .targetSampleSize = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isDiscussedIn = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overallTargetSamplePercent = c(
          min="0", 
          max="1", 
          datatype = "numeric", 
          structure = "vector"
        ),
        overallTargetSampleSize = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        targetSampleSize = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_TargetSample", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_SamplingGoal


#' @title DDI4_SamplingProcedure  The Sampling Procedure describes the population being sampled, the sampling frame and the sampling plan including steps and sub steps by referencing the universe, sample frames and sampling plans described in schemes
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/SamplingProcedure}{SamplingProcedure} 

#'
#'   Definition
#'
#'   ============
#'
#'   The Sampling Procedure describes the population being sampled, the sampling frame and the sampling plan including steps and sub steps by referencing the universe, sample frames and sampling plans described in schemes. Target sample sizes for each stage can be noted as well as the process for determining sample size, the date of the sample and the person or organization responsible for drawing the sample.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Census of Population and Housing, 1980 Summary Tape File 4, chapter on Technical Information. Overview is the Introduction generally describing the sample; there is a sample design section which provides a more detailed overview of the design. Additional external documentation provides detailed processes for sample adjustments needed for small area estimations, sample frames etc.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   This is a summary of the methodology of sampling providing both an overview of purpose and use of a particular sampling approach and optionally providing specific details regarding the algorithm, design, and process of selecting a sample.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_MethodologyOverview}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasDesign}}{ A read/write active binding.
#'          Sampling Design used to perform the Sampling Procedure
#'
#'                              \code{myDDI4_SamplingProcedure$hasDesign <- c(mySamplingDesignURN)} sets the association 'hasDesign' from 'myDDI4_SamplingProcedure' to a character vector of DdiUrns for R6 SamplingDesign class instances
#'
#'                              \code{myDDI4_SamplingProcedure$hasDesign} returns a character vector of of DdiUrns for R6  SamplingDesign class instances}
#'   }
#'   \describe{
#'     \item{\code{hasProcess}}{ A read/write active binding.
#'          Sampling Process used to perform the Sampling Procedure
#'
#'                              \code{myDDI4_SamplingProcedure$hasProcess <- c(mySamplingProcessURN)} sets the association 'hasProcess' from 'myDDI4_SamplingProcedure' to a character vector of DdiUrns for R6 SamplingProcess class instances
#'
#'                              \code{myDDI4_SamplingProcedure$hasProcess} returns a character vector of of DdiUrns for R6  SamplingProcess class instances}
#'   }
#'   \describe{
#'     \item{\code{isExpressedBy}}{ A read/write active binding.
#'          Sampling Algorithm that defines the methodology in a generic way
#'
#'                              \code{myDDI4_SamplingProcedure$isExpressedBy <- c(mySamplingAlgorithmURN)} sets the association 'isExpressedBy' from 'myDDI4_SamplingProcedure' to a character vector of DdiUrns for R6 SamplingAlgorithm class instances
#'
#'                              \code{myDDI4_SamplingProcedure$isExpressedBy} returns a character vector of of DdiUrns for R6  SamplingAlgorithm class instances}
#'   }
#'   \describe{
#'     \item{\code{componentMethodology}}{ A read/write active binding.
#'          A Sampling Procedure that is a component part of this Sampling Procedure
#'
#'                              \code{myDDI4_SamplingProcedure$componentMethodology <- c(mySamplingProcedureURN)} sets the association 'componentMethodology' from 'myDDI4_SamplingProcedure' to a character vector of DdiUrns for R6 SamplingProcedure class instances
#'
#'                              \code{myDDI4_SamplingProcedure$componentMethodology} returns a character vector of of DdiUrns for R6  SamplingProcedure class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_SamplingProcedure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_SamplingProcedure <- R6Class(classname="DDI4_SamplingProcedure",
  inherit = DDI4_MethodologyOverview,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasDesign=NA,hasProcess=NA,isExpressedBy=NA,componentMethodology=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_SamplingProcedure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)



       if(!missing(hasDesign)){
        private$.hasDesign <- returnUrnVectorIfValid(validClass="DDI4_SamplingDesign", proposedUrnVector=hasDesign , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasDesign, referencingAssociationName="hasDesign",  referencingUrn=thisUrn)  
      }
      if(!missing(hasProcess)){
        private$.hasProcess <- returnUrnVectorIfValid(validClass="DDI4_SamplingProcess", proposedUrnVector=hasProcess , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasProcess, referencingAssociationName="hasProcess",  referencingUrn=thisUrn)  
      }
      if(!missing(isExpressedBy)){
        private$.isExpressedBy <- returnUrnVectorIfValid(validClass="DDI4_SamplingAlgorithm", proposedUrnVector=isExpressedBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isExpressedBy, referencingAssociationName="isExpressedBy",  referencingUrn=thisUrn)  
      }
      if(!missing(componentMethodology)){
        private$.componentMethodology <- returnUrnVectorIfValid(validClass="DDI4_SamplingProcedure", proposedUrnVector=componentMethodology , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.componentMethodology, referencingAssociationName="componentMethodology",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 4 class association(s):
    componentMethodology = function(val){
      if (!missing(val)){
        private$.componentMethodology <- returnUrnVectorIfValid(validClass="DDI4_SamplingProcedure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.componentMethodology, referencingAssociationName="componentMethodology",  referencingUrn=thisUrn)  
       # invisible( private$.componentMethodology)
      } else {
          private$.componentMethodology
      }
    },

    hasDesign = function(val){
      if (!missing(val)){
        private$.hasDesign <- returnUrnVectorIfValid(validClass="DDI4_SamplingDesign", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasDesign, referencingAssociationName="hasDesign",  referencingUrn=thisUrn)  
       # invisible( private$.hasDesign)
      } else {
          private$.hasDesign
      }
    },

    hasProcess = function(val){
      if (!missing(val)){
        private$.hasProcess <- returnUrnVectorIfValid(validClass="DDI4_SamplingProcess", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasProcess, referencingAssociationName="hasProcess",  referencingUrn=thisUrn)  
       # invisible( private$.hasProcess)
      } else {
          private$.hasProcess
      }
    },

    isExpressedBy = function(val){
      if (!missing(val)){
        private$.isExpressedBy <- returnUrnVectorIfValid(validClass="DDI4_SamplingAlgorithm", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isExpressedBy, referencingAssociationName="isExpressedBy",  referencingUrn=thisUrn)  
       # invisible( private$.isExpressedBy)
      } else {
          private$.isExpressedBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasDesign = NULL,
    .hasProcess = NULL,
    .isExpressedBy = NULL,
    .componentMethodology = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        componentMethodology = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_SamplingProcedure", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasDesign = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_SamplingDesign", 
          structure = "DdiUrn vector"
        ),
        hasProcess = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_SamplingProcess", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isDescribedIn = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        isExpressedBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_SamplingAlgorithm", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        rationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        subjectOfMethodology = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_SamplingProcedure


#' @title DDI4_SamplingProcess  Provides the details of the process used to apply the sampling design and obtain the sample
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/SamplingProcess}{SamplingProcess} 

#'
#'   Definition
#'
#'   ============
#'
#'   Provides the details of the process used to apply the sampling design and obtain the sample. Sampling Process contains a Workflow Step which provides the constituents of a Workflow. It can be a composition (a set of Control Constructs) or atomic (an Act) and may be performed by a service. This allows the use of existing Workflow models to express various sampling process actions. Sampling Frames and Sample Population can be used as designated Inputs or Outputs using Parameters.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A Split which takes a sampling stage and divides the sample frame into different subsets. A different sampling technique is applied to each subset. Once a split occurs each subset can have stages underneath, and the number of states under each split subset may differ.
#'
#'   A Stage is the application of a single sampling algorithm applied to a sampling frame. For instance, the US Current Population Survey samples geographic areas first before identifying household to contact within each of those areas.
#'
#'   A Stratification of a stage into multiple subsets. Each stratified group will be sampled using the same sampling approach. For example stratifying a state by ZIP Code areas in each of 5 mean income quintiles and then doing a random sample of the households in a set of Zip Codes. Allows for oversampling of identified subpopulations.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_SamplingProcess$DdiUrn} returns the DDI URN of myDDI4_SamplingProcess.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{overview}}{ A read/write active binding.
#'        Provides a high level overview or summary of the class. Can be used to inform end-users or as part of an executive summary. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_SamplingProcess$overview <- list(myInternationalStructuredString)} sets the 'overview' property of 'myDDI4_SamplingProcess' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_SamplingProcess$overview} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isDiscussedIn}}{ A read/write active binding.
#'          Description and link to external material discussing this Sampling Process
#'
#'                              \code{myDDI4_SamplingProcess$isDiscussedIn <- c(myExternalMaterialURN)} sets the association 'isDiscussedIn' from 'myDDI4_SamplingProcess' to a character vector of DdiUrns for R6 ExternalMaterial class instances
#'
#'                              \code{myDDI4_SamplingProcess$isDiscussedIn} returns a character vector of of DdiUrns for R6  ExternalMaterial class instances}
#'   }
#'   \describe{
#'     \item{\code{isSpecifiedBy}}{ A read/write active binding.
#'          The Process Sequence which initiate the process flow and describes the sequence of the process flow
#'
#'                              \code{myDDI4_SamplingProcess$isSpecifiedBy <- c(myWorkflowStepSequenceURN)} sets the association 'isSpecifiedBy' from 'myDDI4_SamplingProcess' to a character vector of DdiUrns for R6 WorkflowStepSequence class instances
#'
#'                              \code{myDDI4_SamplingProcess$isSpecifiedBy} returns a character vector of of DdiUrns for R6  WorkflowStepSequence class instances}
#'   }
#'   \describe{
#'     \item{\code{hasResults}}{ A read/write active binding.
#'          Results of the Sampling Process. Restriction to SamplePopulation, a  type of Result 
#'
#'                              \code{myDDI4_SamplingProcess$hasResults <- c(mySamplePopulationResultURN)} sets the association 'hasResults' from 'myDDI4_SamplingProcess' to a character vector of DdiUrns for R6 SamplePopulationResult class instances
#'
#'                              \code{myDDI4_SamplingProcess$hasResults} returns a character vector of of DdiUrns for R6  SamplePopulationResult class instances}
#'   }
#'   \describe{
#'     \item{\code{implementsAlgorithm}}{ A read/write active binding.
#'          The sampling algorithm implanted by this sampling process. Constraint of the relationship implementsAlgorithm in Process pattern class by limiting the target to a SamplingAlgorithm
#'
#'                              \code{myDDI4_SamplingProcess$implementsAlgorithm <- c(mySamplingAlgorithmURN)} sets the association 'implementsAlgorithm' from 'myDDI4_SamplingProcess' to a character vector of DdiUrns for R6 SamplingAlgorithm class instances
#'
#'                              \code{myDDI4_SamplingProcess$implementsAlgorithm} returns a character vector of of DdiUrns for R6  SamplingAlgorithm class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_SamplingProcess$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, overview=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_SamplingProcess <- R6Class(classname="DDI4_SamplingProcess",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, overview=NA,    isDiscussedIn=NA,isSpecifiedBy=NA,hasResults=NA,implementsAlgorithm=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_SamplingProcess may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(overview)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=overview , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(isDiscussedIn)){
        private$.isDiscussedIn <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=isDiscussedIn , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isDiscussedIn, referencingAssociationName="isDiscussedIn",  referencingUrn=thisUrn)  
      }
      if(!missing(isSpecifiedBy)){
        private$.isSpecifiedBy <- returnUrnVectorIfValid(validClass="DDI4_WorkflowStepSequence", proposedUrnVector=isSpecifiedBy , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isSpecifiedBy, referencingAssociationName="isSpecifiedBy",  referencingUrn=thisUrn)  
      }
      if(!missing(hasResults)){
        private$.hasResults <- returnUrnVectorIfValid(validClass="DDI4_SamplePopulationResult", proposedUrnVector=hasResults , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasResults, referencingAssociationName="hasResults",  referencingUrn=thisUrn)  
      }
      if(!missing(implementsAlgorithm)){
        private$.implementsAlgorithm <- returnUrnVectorIfValid(validClass="DDI4_SamplingAlgorithm", proposedUrnVector=implementsAlgorithm , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.implementsAlgorithm, referencingAssociationName="implementsAlgorithm",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    overview = function(val){
      if (!missing(val)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overview
      }
    },


# There are 4 class association(s):
    hasResults = function(val){
      if (!missing(val)){
        private$.hasResults <- returnUrnVectorIfValid(validClass="DDI4_SamplePopulationResult", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasResults, referencingAssociationName="hasResults",  referencingUrn=thisUrn)  
       # invisible( private$.hasResults)
      } else {
          private$.hasResults
      }
    },

    implementsAlgorithm = function(val){
      if (!missing(val)){
        private$.implementsAlgorithm <- returnUrnVectorIfValid(validClass="DDI4_SamplingAlgorithm", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.implementsAlgorithm, referencingAssociationName="implementsAlgorithm",  referencingUrn=thisUrn)  
       # invisible( private$.implementsAlgorithm)
      } else {
          private$.implementsAlgorithm
      }
    },

    isDiscussedIn = function(val){
      if (!missing(val)){
        private$.isDiscussedIn <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isDiscussedIn, referencingAssociationName="isDiscussedIn",  referencingUrn=thisUrn)  
       # invisible( private$.isDiscussedIn)
      } else {
          private$.isDiscussedIn
      }
    },

    isSpecifiedBy = function(val){
      if (!missing(val)){
        private$.isSpecifiedBy <- returnUrnVectorIfValid(validClass="DDI4_WorkflowStepSequence", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isSpecifiedBy, referencingAssociationName="isSpecifiedBy",  referencingUrn=thisUrn)  
       # invisible( private$.isSpecifiedBy)
      } else {
          private$.isSpecifiedBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .overview = NULL,
    .isDiscussedIn = NULL,
    .isSpecifiedBy = NULL,
    .hasResults = NULL,
    .implementsAlgorithm = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasResults = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_SamplePopulationResult", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        implementsAlgorithm = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_SamplingAlgorithm", 
          structure = "DdiUrn vector"
        ),
        isDiscussedIn = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isSpecifiedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowStepSequence", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_SamplingProcess


#' @title DDI4_ScaleResponseDomain  A response domain capturing a scaled response, such as a Likert scale
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ScaleResponseDomain}{ScaleResponseDomain} 

#'
#'   Definition
#'
#'   ============
#'
#'   A response domain capturing a scaled response, such as a Likert scale. 
#'
#'   Note: This item still must be modeled and is incomplete at this time.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ResponseDomain}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ScaleResponseDomain$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ScaleResponseDomain <- R6Class(classname="DDI4_ScaleResponseDomain",
  inherit = DDI4_ResponseDomain,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ScaleResponseDomain may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

    }
  ),

  active = list(
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="1", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        intendedRepresentation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RepresentedVariable", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ScaleResponseDomain


#' @title DDI4_Segment  A structure used to express explicit segments or regions within different types of external materials (Textual, Audio, Video, XML, and Image)
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Segment}{Segment} 

#'
#'   Definition
#'
#'   ============
#'
#'   A structure used to express explicit segments or regions within different types of external materials (Textual, Audio, Video, XML, and Image). Provides the appropriate start, stop, or region definitions for each type.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:SegmentType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Segment$DdiUrn} returns the DDI URN of myDDI4_Segment.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{usesAudioSegment}}{ A read/write active binding.
#'        Describes the type and length of the audio segment.
#'
#'                              \code{myDDI4_Segment$usesAudioSegment <- list(myAudioSegment)} sets the 'usesAudioSegment' property of 'myDDI4_Segment' to a list of R6 AudioSegment class instances
#'
#'                              \code{myDDI4_Segment$usesAudioSegment} returns a list of of R6 : AudioSegment class instances}
#'   }
#'   \describe{
#'     \item{\code{usesVideoSegment}}{ A read/write active binding.
#'        Describes the type and length of the video segment.
#'
#'                              \code{myDDI4_Segment$usesVideoSegment <- list(myVideoSegment)} sets the 'usesVideoSegment' property of 'myDDI4_Segment' to a list of R6 VideoSegment class instances
#'
#'                              \code{myDDI4_Segment$usesVideoSegment} returns a list of of R6 : VideoSegment class instances}
#'   }
#'   \describe{
#'     \item{\code{xml}}{ A read/write active binding.
#'        An X-Pointer expression identifying a node in the XML document.
#'
#'                              \code{myDDI4_Segment$xml <- mycharacter} sets the xml property of 'myDDI4_Segment' to the vector of datatype:character
#'
#'                              \code{myDDI4_Segment$xml} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{usesTextualSegment}}{ A read/write active binding.
#'        Defines the segment of textual content used by the parent object. Can identify a set of lines and or characters used to define the segment
#'
#'                              \code{myDDI4_Segment$usesTextualSegment <- list(myTextualSegment)} sets the 'usesTextualSegment' property of 'myDDI4_Segment' to a list of R6 TextualSegment class instances
#'
#'                              \code{myDDI4_Segment$usesTextualSegment} returns a list of of R6 : TextualSegment class instances}
#'   }
#'   \describe{
#'     \item{\code{usesImageArea}}{ A read/write active binding.
#'        Defines the shape and area of an image used as part of a location representation. The shape is defined as a Rectangle, Circle, or Polygon and Coordinates provides the information required to define it.
#'
#'                              \code{myDDI4_Segment$usesImageArea <- list(myImageArea)} sets the 'usesImageArea' property of 'myDDI4_Segment' to a list of R6 ImageArea class instances
#'
#'                              \code{myDDI4_Segment$usesImageArea} returns a list of of R6 : ImageArea class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Segment$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, usesAudioSegment=NA, usesVideoSegment=NA, xml=NA, usesTextualSegment=NA, usesImageArea=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Segment <- R6Class(classname="DDI4_Segment",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, usesAudioSegment=NA, usesVideoSegment=NA, xml=NA, usesTextualSegment=NA, usesImageArea=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Segment may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(usesAudioSegment)){
        private$.usesAudioSegment <- returnR6ObjectIfValid(validClass="DDI4_AudioSegment", proposedObjectList=usesAudioSegment , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(usesVideoSegment)){
        private$.usesVideoSegment <- returnR6ObjectIfValid(validClass="DDI4_VideoSegment", proposedObjectList=usesVideoSegment , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(xml)){
        private$.xml <- returnVectorIfValid(validVectorClass="character", proposedVector=xml , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(usesTextualSegment)){
        private$.usesTextualSegment <- returnR6ObjectIfValid(validClass="DDI4_TextualSegment", proposedObjectList=usesTextualSegment , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(usesImageArea)){
        private$.usesImageArea <- returnR6ObjectIfValid(validClass="DDI4_ImageArea", proposedObjectList=usesImageArea , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 5 properties: 
    usesAudioSegment = function(val){
      if (!missing(val)){
        private$.usesAudioSegment <- returnR6ObjectIfValid(validClass="DDI4_AudioSegment", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.usesAudioSegment
      }
    },

    usesImageArea = function(val){
      if (!missing(val)){
        private$.usesImageArea <- returnR6ObjectIfValid(validClass="DDI4_ImageArea", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.usesImageArea
      }
    },

    usesTextualSegment = function(val){
      if (!missing(val)){
        private$.usesTextualSegment <- returnR6ObjectIfValid(validClass="DDI4_TextualSegment", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.usesTextualSegment
      }
    },

    usesVideoSegment = function(val){
      if (!missing(val)){
        private$.usesVideoSegment <- returnR6ObjectIfValid(validClass="DDI4_VideoSegment", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.usesVideoSegment
      }
    },

    xml = function(val){
      if (!missing(val)){
        private$.xml <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.xml
      }
    }


  ),

  private = list(
    .usesAudioSegment = NULL,
    .usesVideoSegment = NULL,
    .xml = NULL,
    .usesTextualSegment = NULL,
    .usesImageArea = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        usesAudioSegment = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AudioSegment", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usesImageArea = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ImageArea", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usesTextualSegment = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_TextualSegment", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usesVideoSegment = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_VideoSegment", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        xml = c(
          min="0", 
          max="Inf", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Segment


#' @title DDI4_SegmentByText  Defines the location of a segment of text through character, and optionally line, counts
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/SegmentByText}{SegmentByText} 

#'
#'   Definition
#'
#'   ============
#'
#'   Defines the location of a segment of text through character, and optionally line, counts. An adequate description will always include a startCharacterPosition and then may include an endCharacterPosition or a characterLength.
#'
#'   If StartLine is specified, the character counts begin within that line.
#'
#'   An endCharacterPosition of 0 indicates that whole lines are specified
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   The segment beginning at line 3, character 4 and ending at line 27 character 13.
#'
#'   Alternatively the segment beginning at character 257 and ending at character 1350 of the whole body of text.
#'
#'   StartLine of 10, endLine of 12, startCharacterPosition of 1, endCharacterPosition of 0 means all of lines 10,11, and 12. 
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_PhysicalSegmentLocation}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_SegmentByText$DdiUrn} returns the DDI URN of myDDI4_SegmentByText.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{startLine}}{ A read/write active binding.
#'        The line number, where 1 is the first line, on which to begin counting characters. If missing this defaults to 1 (the first line).
#'
#'                              \code{myDDI4_SegmentByText$startLine <- myinteger} sets the startLine property of 'myDDI4_SegmentByText' to the vector of datatype:integer
#'
#'                              \code{myDDI4_SegmentByText$startLine} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{endLine}}{ A read/write active binding.
#'        The last line on which to count characters. If missing this defaults to startLine. 
#'
#'                              \code{myDDI4_SegmentByText$endLine <- myinteger} sets the endLine property of 'myDDI4_SegmentByText' to the vector of datatype:integer
#'
#'                              \code{myDDI4_SegmentByText$endLine} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{startCharacterPosition}}{ A read/write active binding.
#'        The character position of the first character of the segment, with the count beginning at character 1 of startLine. 
#'
#'                              \code{myDDI4_SegmentByText$startCharacterPosition <- myinteger} sets the startCharacterPosition property of 'myDDI4_SegmentByText' to the vector of datatype:integer
#'
#'                              \code{myDDI4_SegmentByText$startCharacterPosition} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{endCharacterPosition}}{ A read/write active binding.
#'        The character position of the last character of the segment.  If endLine is specified, the count begins at character 1 of endLine. If startLine and endLine are not specified, the count begins at character 1 of the first line of the resource and the count includes any line termination characters. The resulting segment may include text from multiple lines of the resource.
#'
#'                              \code{myDDI4_SegmentByText$endCharacterPosition <- myinteger} sets the endCharacterPosition property of 'myDDI4_SegmentByText' to the vector of datatype:integer
#'
#'                              \code{myDDI4_SegmentByText$endCharacterPosition} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{characterLength}}{ A read/write active binding.
#'        The number of characters in the segment. The segment may include text from multiple lines of the resource. If it does, the length includes any line termination characters.
#'
#'                              \code{myDDI4_SegmentByText$characterLength <- myinteger} sets the characterLength property of 'myDDI4_SegmentByText' to the vector of datatype:integer
#'
#'                              \code{myDDI4_SegmentByText$characterLength} returns a vector of datatype: integer}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_SegmentByText$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, startLine=NA, endLine=NA, startCharacterPosition=NA, endCharacterPosition=NA, characterLength=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_SegmentByText <- R6Class(classname="DDI4_SegmentByText",
  inherit = DDI4_PhysicalSegmentLocation,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, startLine=NA, endLine=NA, startCharacterPosition=NA, endCharacterPosition=NA, characterLength=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_SegmentByText may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(startLine)){
        private$.startLine <- returnVectorIfValid(validVectorClass="integer", proposedVector=startLine , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(endLine)){
        private$.endLine <- returnVectorIfValid(validVectorClass="integer", proposedVector=endLine , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(startCharacterPosition)){
        private$.startCharacterPosition <- returnVectorIfValid(validVectorClass="integer", proposedVector=startCharacterPosition , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(endCharacterPosition)){
        private$.endCharacterPosition <- returnVectorIfValid(validVectorClass="integer", proposedVector=endCharacterPosition , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(characterLength)){
        private$.characterLength <- returnVectorIfValid(validVectorClass="integer", proposedVector=characterLength , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 5 properties: 
    characterLength = function(val){
      if (!missing(val)){
        private$.characterLength <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.characterLength
      }
    },

    endCharacterPosition = function(val){
      if (!missing(val)){
        private$.endCharacterPosition <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.endCharacterPosition
      }
    },

    endLine = function(val){
      if (!missing(val)){
        private$.endLine <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.endLine
      }
    },

    startCharacterPosition = function(val){
      if (!missing(val)){
        private$.startCharacterPosition <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.startCharacterPosition
      }
    },

    startLine = function(val){
      if (!missing(val)){
        private$.startLine <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.startLine
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .startLine = NULL,
    .endLine = NULL,
    .startCharacterPosition = NULL,
    .endCharacterPosition = NULL,
    .characterLength = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        characterLength = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        endCharacterPosition = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        endLine = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        startCharacterPosition = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        startLine = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_SegmentByText


#' @title DDI4_SentinelConceptualDomain  Description or list of possible sentinel concepts , e
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/SentinelConceptualDomain}{SentinelConceptualDomain} 

#'
#'   Definition
#'
#'   ============
#'
#'   Description or list of possible sentinel concepts , e.g. missing values.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Refused, 
#'
#'   Don't know, 
#'
#'   Lost in processing
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   missing categories
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ConceptualDomain}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_SentinelConceptualDomain$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_SentinelConceptualDomain <- R6Class(classname="DDI4_SentinelConceptualDomain",
  inherit = DDI4_ConceptualDomain,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_SentinelConceptualDomain may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

    }
  ),

  active = list(
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        describedConceptualDomain = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ValueAndConceptDescription", 
          structure = "DdiUrn vector"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        enumeratedConceptualDomain = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ConceptSystem", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_SentinelConceptualDomain


#' @title DDI4_SentinelValueDomain  The Value Domain for a sentinel conceptual domain
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/SentinelValueDomain}{SentinelValueDomain} 

#'
#'   Definition
#'
#'   ============
#'
#'   The Value Domain for a sentinel conceptual domain. Sentinel values are defined in ISO 11404 as
#'
#'   "element of a value space that is not completely consistent with a datatype's properties and characterizing operations...". A common example would be codes for missing values.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Missing categories expressed as Codes <-9, refused>; <-8, Don't Know> for a numeric variable with values greater than zero.  
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Sentinel values are used for processing, not to describe subject matter. Typical examples include missing values or invalid entry codes. Sentinel Value Domains are typically of the enumerated type, but they can be the described type, too.
#'
#'   
#'
#'   This is not to say that sentinel values carry no information. Data on gender might be enumerated by "0, male" and "1, female". These are the substantive values (see Substantive Value Domain). However, there may be the need to include missing values along with that data, such as "m, missing" and "r, refused". These are sentinel values.
#'
#'   
#'
#'   ISO/IEC 11404 - General purpose datatypes, defines sentinel values in terms of how that standard defines datatypes. But, the fact that the sentinels don't fit in terms of the calculations and statistics one would perform on the "clean" data is a distinguishing characteristic. In the example above, one would not include missing or refused data in calculating a ratio of females to the total population.
#'
#'   
#'
#'   Sentinel values may be described rather than enumerated. For instance, there might be a range of values, each representing an out of range value, but there could be too many to enumerate. It is easier to describe the range.
#'
#'   
#'
#'   In some software missing values are represented as values not in the datatype of the valid values. R has values of NA, NaN, Inf, and -Inf. SAS and Stata have values displayed as ".", ".A" through ".Z", and "._"
#'
#'   
#'
#'   Other software might use values like 999 for missing that would otherwise be the same datatype as valid values but outside the parameters of the domain.
#'
#'   
#'
#'   In the gender example above:
#'
#'   For SPSS the sentinel values might be represented as:
#'
#'   998 = "refused"
#'
#'   999 = "not recorded"
#'
#'   
#'
#'   For SAS or Stata the sentinel values might be represented as:
#'
#'   .R = "refused"
#'
#'   .N = "not recorded"
#'
#'   
#'
#'   Sentinel values can also be used for other purposes beyond missing. For a numeric variable "A" might represent a value somewhere in a defined range to prevent disclosure of information about an individual. This might be considered a "semi-missing value".
#'
#'   In SAS or Stata for example:
#'
#'   .A = "greater than or equal to 100 and less than 1000 "
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Enumerated Value Domain
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ValueDomain}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_SentinelValueDomain$DdiUrn} returns the DDI URN of myDDI4_SentinelValueDomain.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{platformType}}{ A read/write active binding.
#'        The type of platform under which sentinel codes will be used. Statistical software platforms use different sets of codes to indicate missing values. The external controlled vocabulary should list platform tpes and a description of the allowed missing value types. A sample list would be: • BlankString - A Blank string indicates missing. Comparison is based on lexical order.  • EmptyString - An empty string indicates missing. Use in comparisons returns missing.   • Rstyle - Codes drawn from  NA and the IEEE 754 values of  NaN  -Inf   +Inf.   Comparisons return NA.   • SASNumeric - codes drawn from . ._ .A .B .C .D .E .F .G .H .I .J .K .L .M .N .O .P .Q .R .S .T .U .V .W .X .Y .Z    Sentinel code treated as less than any substantive value   • SPSSstyle - System missing (a dot) a set of individual values drawn from the same datatype as the SubstantiveValueDomain, and a range of values  drawn from the same datatype as the SubstantiveValueDomain.  Comparisons return system missing.  Some functions substitute with valid values (e.g. SUM replaces missing values with 0)   • StataNumeric - codes drawn from . ._ .A .B .C .D .E .F .G .H .I .J .K .L .M .N .O .P .Q .R .S .T .U .V .W .X .Y .Z  Sentinel code treated as greater than any substantive value  • Unrestricted - No restrictions on codes for sentinel values. Use in comparisons is indeterminate.  
#'
#'                              \code{myDDI4_SentinelValueDomain$platformType <- list(myExternalControlledVocabularyEntry)} sets the 'platformType' property of 'myDDI4_SentinelValueDomain' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_SentinelValueDomain$platformType} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{takesConceptsFrom}}{ A read/write active binding.
#'          Corresponding conceptual definition given by a SentinelConceptualDomain.
#'
#'                              \code{myDDI4_SentinelValueDomain$takesConceptsFrom <- c(mySentinelConceptualDomainURN)} sets the association 'takesConceptsFrom' from 'myDDI4_SentinelValueDomain' to a character vector of DdiUrns for R6 SentinelConceptualDomain class instances
#'
#'                              \code{myDDI4_SentinelValueDomain$takesConceptsFrom} returns a character vector of of DdiUrns for R6  SentinelConceptualDomain class instances}
#'   }
#'   \describe{
#'     \item{\code{describedValueDomain}}{ A read/write active binding.
#'          A formal description of the set of valid values - for described value domains.
#'
#'                              \code{myDDI4_SentinelValueDomain$describedValueDomain <- c(myValueAndConceptDescriptionURN)} sets the association 'describedValueDomain' from 'myDDI4_SentinelValueDomain' to a character vector of DdiUrns for R6 ValueAndConceptDescription class instances
#'
#'                              \code{myDDI4_SentinelValueDomain$describedValueDomain} returns a character vector of of DdiUrns for R6  ValueAndConceptDescription class instances}
#'   }
#'   \describe{
#'     \item{\code{enumeratedValueDomain}}{ A read/write active binding.
#'          Any subtype of an Enumeration Domain enumerating the set of valid values.
#'
#'                              \code{myDDI4_SentinelValueDomain$enumeratedValueDomain <- c(myEnumerationDomainURN)} sets the association 'enumeratedValueDomain' from 'myDDI4_SentinelValueDomain' to a character vector of DdiUrns for R6 EnumerationDomain class instances
#'
#'                              \code{myDDI4_SentinelValueDomain$enumeratedValueDomain} returns a character vector of of DdiUrns for R6  EnumerationDomain class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_SentinelValueDomain$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, platformType=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_SentinelValueDomain <- R6Class(classname="DDI4_SentinelValueDomain",
  inherit = DDI4_ValueDomain,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, platformType=NA,    takesConceptsFrom=NA,describedValueDomain=NA,enumeratedValueDomain=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_SentinelValueDomain may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(platformType)){
        private$.platformType <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=platformType , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(takesConceptsFrom)){
        private$.takesConceptsFrom <- returnUrnVectorIfValid(validClass="DDI4_SentinelConceptualDomain", proposedUrnVector=takesConceptsFrom , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.takesConceptsFrom, referencingAssociationName="takesConceptsFrom",  referencingUrn=thisUrn)  
      }
      if(!missing(describedValueDomain)){
        private$.describedValueDomain <- returnUrnVectorIfValid(validClass="DDI4_ValueAndConceptDescription", proposedUrnVector=describedValueDomain , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.describedValueDomain, referencingAssociationName="describedValueDomain",  referencingUrn=thisUrn)  
      }
      if(!missing(enumeratedValueDomain)){
        private$.enumeratedValueDomain <- returnUrnVectorIfValid(validClass="DDI4_EnumerationDomain", proposedUrnVector=enumeratedValueDomain , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.enumeratedValueDomain, referencingAssociationName="enumeratedValueDomain",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    platformType = function(val){
      if (!missing(val)){
        private$.platformType <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.platformType
      }
    },


# There are 3 class association(s):
    describedValueDomain = function(val){
      if (!missing(val)){
        private$.describedValueDomain <- returnUrnVectorIfValid(validClass="DDI4_ValueAndConceptDescription", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.describedValueDomain, referencingAssociationName="describedValueDomain",  referencingUrn=thisUrn)  
       # invisible( private$.describedValueDomain)
      } else {
          private$.describedValueDomain
      }
    },

    enumeratedValueDomain = function(val){
      if (!missing(val)){
        private$.enumeratedValueDomain <- returnUrnVectorIfValid(validClass="DDI4_EnumerationDomain", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.enumeratedValueDomain, referencingAssociationName="enumeratedValueDomain",  referencingUrn=thisUrn)  
       # invisible( private$.enumeratedValueDomain)
      } else {
          private$.enumeratedValueDomain
      }
    },

    takesConceptsFrom = function(val){
      if (!missing(val)){
        private$.takesConceptsFrom <- returnUrnVectorIfValid(validClass="DDI4_SentinelConceptualDomain", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.takesConceptsFrom, referencingAssociationName="takesConceptsFrom",  referencingUrn=thisUrn)  
       # invisible( private$.takesConceptsFrom)
      } else {
          private$.takesConceptsFrom
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .platformType = NULL,
    .takesConceptsFrom = NULL,
    .describedValueDomain = NULL,
    .enumeratedValueDomain = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        describedValueDomain = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ValueAndConceptDescription", 
          structure = "DdiUrn vector"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        enumeratedValueDomain = c(
          min="0", 
          max="1", 
          datatype = "DDI4_EnumerationDomain", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        platformType = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        recommendedDataType = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        takesConceptsFrom = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SentinelConceptualDomain", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_SentinelValueDomain


#' @title DDI4_Service  A means of performing a Process Step as part of a Business Function (an ability that an organization possesses, typically expressed in general and high level terms and requiring a combination of organization, people, processes and technology to achieve)
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Service}{Service} 

#'
#'   Definition
#'
#'   ============
#'
#'   A means of performing a Process Step as part of a Business Function (an ability that an organization possesses, typically expressed in general and high level terms and requiring a combination of organization, people, processes and technology to achieve). 
#'
#'   
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Business Service
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Service$DdiUrn} returns the DDI URN of myDDI4_Service.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{serviceLocation}}{ A read/write active binding.
#'        Specifies where the service can be accessed.
#'
#'                              \code{myDDI4_Service$serviceLocation <- list(myExternalControlledVocabularyEntry)} sets the 'serviceLocation' property of 'myDDI4_Service' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_Service$serviceLocation} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasAgent}}{ A read/write active binding.
#'          Actor that performs a role in the service,
#'
#'                              \code{myDDI4_Service$hasAgent <- c(myAgentURN)} sets the association 'hasAgent' from 'myDDI4_Service' to a character vector of DdiUrns for R6 Agent class instances
#'
#'                              \code{myDDI4_Service$hasAgent} returns a character vector of of DdiUrns for R6  Agent class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Service$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, serviceLocation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Service <- R6Class(classname="DDI4_Service",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, serviceLocation=NA,    hasAgent=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Service may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(serviceLocation)){
        private$.serviceLocation <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=serviceLocation , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(hasAgent)){
        private$.hasAgent <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=hasAgent , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasAgent, referencingAssociationName="hasAgent",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    serviceLocation = function(val){
      if (!missing(val)){
        private$.serviceLocation <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.serviceLocation
      }
    },


# There are 1 class association(s):
    hasAgent = function(val){
      if (!missing(val)){
        private$.hasAgent <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasAgent, referencingAssociationName="hasAgent",  referencingUrn=thisUrn)  
       # invisible( private$.hasAgent)
      } else {
          private$.hasAgent
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .serviceLocation = NULL,
    .hasAgent = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAgent = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Agent", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        serviceLocation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_Service


#' @title DDI4_Sign  Something that suggests the presence or existence of a fact, condition, or quality
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Sign}{Sign} 

#'
#'   Definition
#'
#'   ============
#'
#'   Something that suggests the presence or existence of a fact, condition, or quality.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   The terms “Unemployment” and “Arbeitslosigkeit”
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   It is a perceivable object used to denote a concept or an object.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_CollectionMember}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Sign$DdiUrn} returns the DDI URN of myDDI4_Sign.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{representation}}{ A read/write active binding.
#'        A perceivable object used to denote a signified.
#'
#'                              \code{myDDI4_Sign$representation <- list(mySignifier)} sets the 'representation' property of 'myDDI4_Sign' to a list of R6 Signifier class instances
#'
#'                              \code{myDDI4_Sign$representation} returns a list of of R6 : Signifier class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{denotes}}{ A read/write active binding.
#'          Concept or object denoted by the signifier associated to the sign.
#'
#'                              \code{myDDI4_Sign$denotes <- c(mySignifiedURN)} sets the association 'denotes' from 'myDDI4_Sign' to a character vector of DdiUrns for R6 Signified class instances
#'
#'                              \code{myDDI4_Sign$denotes} returns a character vector of of DdiUrns for R6  Signified class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Sign$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, representation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Sign <- R6Class(classname="DDI4_Sign",
  inherit = DDI4_CollectionMember,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, representation=NA,    denotes=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Sign may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(representation)){
        private$.representation <- returnR6ObjectIfValid(validClass="DDI4_Signifier", proposedObjectList=representation , minimumCardinality=1, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(denotes)){
        private$.denotes <- returnUrnVectorIfValid(validClass="DDI4_Signified", proposedUrnVector=denotes , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.denotes, referencingAssociationName="denotes",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    representation = function(val){
      if (!missing(val)){
        private$.representation <- returnR6ObjectIfValid(validClass="DDI4_Signifier", proposedObjectList=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.representation
      }
    },


# There are 1 class association(s):
    denotes = function(val){
      if (!missing(val)){
        private$.denotes <- returnUrnVectorIfValid(validClass="DDI4_Signified", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.denotes, referencingAssociationName="denotes",  referencingUrn=thisUrn)  
       # invisible( private$.denotes)
      } else {
          private$.denotes
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .representation = NULL,
    .denotes = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        denotes = c(
          min="1", 
          max="1", 
          datatype = "DDI4_Signified", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        representation = c(
          min="1", 
          max="1", 
          datatype = "DDI4_Signifier", 
          isAbstractDatatype = TRUE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_Sign


#' @title DDI4_Signified  Concept or object denoted by the signifier associated to a sign
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Signified}{Signified} 

#'
#'   Definition
#'
#'   ============
#'
#'   Concept or object denoted by the signifier associated to a sign.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Concept or object represented by the sign.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_CollectionMember}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Signified$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Signified <- R6Class(classname="DDI4_Signified",
  inherit = DDI4_CollectionMember,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Signified may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

    }
  ),

  active = list(
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_Signified


#' @title DDI4_Signifier  Concept whose extension includes perceivable objects
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Signifier}{Signifier} 

#'
#'   Definition
#'
#'   ============
#'
#'   Concept whose extension includes perceivable objects. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A perceivable object in the extension of a signifier is a token. For instance, the object 5 is a token of “the numeral five”, a signifier.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   A signifier has the potential to refer to an object.  In this case, the referring signifier is a label.  If that object is a concept, then the referring signifier is a designation.  A label is a representation of an object by a signifier which denotes it.  For instance, the token http://www.bls.gov is a label for the web site maintained for the US Bureau of Labor Statistics. The label is also an identifier, since it is intended to dereference the BLS web site. It is a locator, since the HTTP service is associated with it. Finally, ‘Dan Gillman’ is a name for a co-author of this paper, since the token is linguistic.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Signifier$DdiUrn} returns the DDI URN of myDDI4_Signifier.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{content}}{ A read/write active binding.
#'        Perceivable object in the extension of the signifier. The actual content of this value as a string.
#'
#'                              \code{myDDI4_Signifier$content <- mycharacter} sets the content property of 'myDDI4_Signifier' to the vector of datatype:character
#'
#'                              \code{myDDI4_Signifier$content} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{whiteSpace}}{ A read/write active binding.
#'        The usual setting "collapse" states that leading and trailing white space will be removed and multiple adjacent white spaces will be treated as a single white space. When setting to "replace" all occurrences of #x9 (tab), #xA (line feed) and #xD (carriage return) are replaced with #x20 (space) but leading and trailing spaces will be retained. If the existence of any of these white spaces is critical to the understanding of the content, change the value of this attribute to "preserve".
#'
#'                              \code{myDDI4_Signifier$whiteSpace <- list(myWhiteSpaceRule)} sets the 'whiteSpace' property of 'myDDI4_Signifier' to a list of R6 WhiteSpaceRule class instances
#'
#'                              \code{myDDI4_Signifier$whiteSpace} returns a list of of R6 : WhiteSpaceRule class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Signifier$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, content=NA, whiteSpace=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Signifier <- R6Class(classname="DDI4_Signifier",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, content=NA, whiteSpace=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Signifier may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(content)){
        private$.content <- returnVectorIfValid(validVectorClass="character", proposedVector=content , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(whiteSpace)){
        private$.whiteSpace <- returnEnumeratedValues(enumerationObject=DDI4_WhiteSpaceRule$new(registerThisObject=FALSE), proposedCharacterVector=whiteSpace , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    content = function(val){
      if (!missing(val)){
        private$.content <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.content
      }
    },

    whiteSpace = function(val){
      if (!missing(val)){
        private$.whiteSpace <- returnEnumeratedValues(enumerationObject=DDI4_WhiteSpaceRule$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.whiteSpace
      }
    }


  ),

  private = list(
    .content = NULL,
    .whiteSpace = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        content = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        whiteSpace = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WhiteSpaceRule", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_Signifier


#' @title DDI4_SimpleCollection  Simple Collection container (set or bag) that may be unordered or ordered using Member Indicator
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/SimpleCollection}{SimpleCollection} 

#'
#'   Definition
#'
#'   ============
#'
#'   Simple Collection container (set or bag) that may be unordered or ordered using Member Indicator. This collection type is limited to expressing only unordered sets or bags, and simple sequences. Use when there is a need to limit the organization of the collection to one of these forms of organization 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   The sequencing of questions within a questionnaire where ordering and routing is done through conditional constructs such as IfThenElse, etc.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_CollectionMember}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_SimpleCollection$DdiUrn} returns the DDI URN of myDDI4_SimpleCollection.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{type}}{ A read/write active binding.
#'        Whether the collection is a bag or a set: a bag is a collection with duplicates allowed, a set is a collection without duplicates.
#'
#'                              \code{myDDI4_SimpleCollection$type <- list(myCollectionType)} sets the 'type' property of 'myDDI4_SimpleCollection' to a list of R6 CollectionType class instances
#'
#'                              \code{myDDI4_SimpleCollection$type} returns a list of of R6 : CollectionType class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage.
#'
#'                              \code{myDDI4_SimpleCollection$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_SimpleCollection' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_SimpleCollection$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_SimpleCollection$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_SimpleCollection' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_SimpleCollection$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        Pointers to members of the collection that are unordered or in an indexed ordered array.
#'
#'                              \code{myDDI4_SimpleCollection$contains <- list(myMemberIndicator)} sets the 'contains' property of 'myDDI4_SimpleCollection' to a list of R6 MemberIndicator class instances
#'
#'                              \code{myDDI4_SimpleCollection$contains} returns a list of of R6 : MemberIndicator class instances}
#'   }
#'   \describe{
#'     \item{\code{isOrdered}}{ A read/write active binding.
#'        Set to true if the members are in an ordered set. If unordered set to false.
#'
#'                              \code{myDDI4_SimpleCollection$isOrdered <- mylogical} sets the isOrdered property of 'myDDI4_SimpleCollection' to the vector of datatype:logical
#'
#'                              \code{myDDI4_SimpleCollection$isOrdered} returns a vector of datatype: logical}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{definingConcept}}{ A read/write active binding.
#'          The conceptual basis for the collection of members.
#'
#'                              \code{myDDI4_SimpleCollection$definingConcept <- c(myConceptURN)} sets the association 'definingConcept' from 'myDDI4_SimpleCollection' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_SimpleCollection$definingConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_SimpleCollection$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_SimpleCollection <- R6Class(classname="DDI4_SimpleCollection",
  inherit = DDI4_CollectionMember,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA,    definingConcept=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_SimpleCollection may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(type)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=type , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_MemberIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(isOrdered)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=isOrdered , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(definingConcept)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=definingConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 5 properties: 
    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_MemberIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },

    isOrdered = function(val){
      if (!missing(val)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isOrdered
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    type = function(val){
      if (!missing(val)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.type
      }
    },


# There are 1 class association(s):
    definingConcept = function(val){
      if (!missing(val)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
       # invisible( private$.definingConcept)
      } else {
          private$.definingConcept
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .type = NULL,
    .name = NULL,
    .purpose = NULL,
    .contains = NULL,
    .isOrdered = NULL,
    .definingConcept = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_MemberIndicator", 
          isAbstractDatatype = TRUE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_SimpleCollection


#' @title DDI4_SpatialCoordinate  Lists the value and format type for the coordinate value
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/SpatialCoordinate}{SpatialCoordinate} 

#'
#'   Definition
#'
#'   ============
#'
#'   Lists the value and format type for the coordinate value. Note that this is a single value (X coordinate or Y coordinate) rather than a coordinate pair.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:SpatialCoordinateType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_SpatialCoordinate$DdiUrn} returns the DDI URN of myDDI4_SpatialCoordinate.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{content}}{ A read/write active binding.
#'        The value of the coordinate expressed as a string.
#'
#'                              \code{myDDI4_SpatialCoordinate$content <- mycharacter} sets the content property of 'myDDI4_SpatialCoordinate' to the vector of datatype:character
#'
#'                              \code{myDDI4_SpatialCoordinate$content} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{coordinateType}}{ A read/write active binding.
#'        Identifies the type of point coordinate system using a controlled vocabulary. Point formats include decimal degree, degrees minutes seconds, decimal minutes, meters, and feet.
#'
#'                              \code{myDDI4_SpatialCoordinate$coordinateType <- list(myPointFormat)} sets the 'coordinateType' property of 'myDDI4_SpatialCoordinate' to a list of R6 PointFormat class instances
#'
#'                              \code{myDDI4_SpatialCoordinate$coordinateType} returns a list of of R6 : PointFormat class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_SpatialCoordinate$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, content=NA, coordinateType=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_SpatialCoordinate <- R6Class(classname="DDI4_SpatialCoordinate",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, content=NA, coordinateType=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_SpatialCoordinate may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(content)){
        private$.content <- returnVectorIfValid(validVectorClass="character", proposedVector=content , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(coordinateType)){
        private$.coordinateType <- returnEnumeratedValues(enumerationObject=DDI4_PointFormat$new(registerThisObject=FALSE), proposedCharacterVector=coordinateType , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    content = function(val){
      if (!missing(val)){
        private$.content <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.content
      }
    },

    coordinateType = function(val){
      if (!missing(val)){
        private$.coordinateType <- returnEnumeratedValues(enumerationObject=DDI4_PointFormat$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.coordinateType
      }
    }


  ),

  private = list(
    .content = NULL,
    .coordinateType = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        content = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        coordinateType = c(
          min="0", 
          max="1", 
          datatype = "DDI4_PointFormat", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_SpatialCoordinate


#' @title DDI4_SpatialCoverage  A description of spatial coverage (geographic coverage) of the annotated object
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/SpatialCoverage}{SpatialCoverage} 

#'
#'   Definition
#'
#'   ============
#'
#'   A description of spatial coverage (geographic coverage) of the annotated object. Spatial coverage is described using a number of objects that support searching by a wide range of systems (geospatial coordinates, geographic classification systems, and general systems using dcterms:spatial).
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A country, a neighborhood, the inside of a polygon on the surface of the earth, along a street, at a particular intersection, or perhaps even in a certain orbit around the planet Mars. For a data set within a study this may be used to define the geographic restriction of the data set within the geographic coverage of the study (eg. The study may cover all of Sweden but the spatial coverage of the data set is Stockholm). 
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Different systems support different approaches to descriptions of geographic or spatial coverage. Dublin Core has a descriptive text field that is the equivalent of SpatialCoverage/description. Geographers need to understand the spatial objects available (spatialObject, highest and lowest level geography, geographic units and unit types) so they know the building blocks they have to create a map. Spatial search engines do a first pass search by a simple bounding box (the north and south latitudes and east and west longitudes that define the spatial coverage area). Spatial Coverage provides basic support for all of these uses.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   r:SpatialCoverage, place
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_SpatialCoverage$DdiUrn} returns the DDI URN of myDDI4_SpatialCoverage.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{description}}{ A read/write active binding.
#'        A textual description of the spatial coverage to support general searches.
#'
#'                              \code{myDDI4_SpatialCoverage$description <- list(myInternationalStructuredString)} sets the 'description' property of 'myDDI4_SpatialCoverage' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_SpatialCoverage$description} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{spatialAreaCode}}{ A read/write active binding.
#'        Supports the use of a standardized code such as ISO 3166-1,  the Getty Thesaurus of Geographic Names, FIPS-5, etc. 
#'
#'                              \code{myDDI4_SpatialCoverage$spatialAreaCode <- list(myExternalControlledVocabularyEntry)} sets the 'spatialAreaCode' property of 'myDDI4_SpatialCoverage' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_SpatialCoverage$spatialAreaCode} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{spatialObject}}{ A read/write active binding.
#'        Indicates the most discrete spatial object type identified for a single case. Note that data can be collected at a geographic point (address) and reported as such in a protected file, and then aggregated to a polygon for a public file.
#'
#'                              \code{myDDI4_SpatialCoverage$spatialObject <- list(mySpatialObjectType)} sets the 'spatialObject' property of 'myDDI4_SpatialCoverage' to a list of R6 SpatialObjectType class instances
#'
#'                              \code{myDDI4_SpatialCoverage$spatialObject} returns a list of of R6 : SpatialObjectType class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasBoundingBox}}{ A read/write active binding.
#'          The north and south latitudes and east and west longitudes that define the spatial coverage area.
#'
#'                              \code{myDDI4_SpatialCoverage$hasBoundingBox <- c(myBoundingBoxURN)} sets the association 'hasBoundingBox' from 'myDDI4_SpatialCoverage' to a character vector of DdiUrns for R6 BoundingBox class instances
#'
#'                              \code{myDDI4_SpatialCoverage$hasBoundingBox} returns a character vector of of DdiUrns for R6  BoundingBox class instances}
#'   }
#'   \describe{
#'     \item{\code{usesGeographicUnitTypeClassification}}{ A read/write active binding.
#'          A classification which describes the geographic unit types and relationships used within this coverage.
#'
#'                              \code{myDDI4_SpatialCoverage$usesGeographicUnitTypeClassification <- c(myGeographicUnitTypeClassificationURN)} sets the association 'usesGeographicUnitTypeClassification' from 'myDDI4_SpatialCoverage' to a character vector of DdiUrns for R6 GeographicUnitTypeClassification class instances
#'
#'                              \code{myDDI4_SpatialCoverage$usesGeographicUnitTypeClassification} returns a character vector of of DdiUrns for R6  GeographicUnitTypeClassification class instances}
#'   }
#'   \describe{
#'     \item{\code{usesGeographicUnitClassification}}{ A read/write active binding.
#'          A classification which describes the geographic units (specific locations) and relationships used within this coverage.
#'
#'                              \code{myDDI4_SpatialCoverage$usesGeographicUnitClassification <- c(myGeographicUnitClassificationURN)} sets the association 'usesGeographicUnitClassification' from 'myDDI4_SpatialCoverage' to a character vector of DdiUrns for R6 GeographicUnitClassification class instances
#'
#'                              \code{myDDI4_SpatialCoverage$usesGeographicUnitClassification} returns a character vector of of DdiUrns for R6  GeographicUnitClassification class instances}
#'   }
#'   \describe{
#'     \item{\code{includesGeographicUnitType}}{ A read/write active binding.
#'          The geographic unit types used within this coverage.
#'
#'                              \code{myDDI4_SpatialCoverage$includesGeographicUnitType <- c(myUnitTypeURN)} sets the association 'includesGeographicUnitType' from 'myDDI4_SpatialCoverage' to a character vector of DdiUrns for R6 UnitType class instances
#'
#'                              \code{myDDI4_SpatialCoverage$includesGeographicUnitType} returns a character vector of of DdiUrns for R6  UnitType class instances}
#'   }
#'   \describe{
#'     \item{\code{includesGeographicUnit}}{ A read/write active binding.
#'          The geographic units used within this coverage.
#'
#'                              \code{myDDI4_SpatialCoverage$includesGeographicUnit <- c(myGeographicUnitURN)} sets the association 'includesGeographicUnit' from 'myDDI4_SpatialCoverage' to a character vector of DdiUrns for R6 GeographicUnit class instances
#'
#'                              \code{myDDI4_SpatialCoverage$includesGeographicUnit} returns a character vector of of DdiUrns for R6  GeographicUnit class instances}
#'   }
#'   \describe{
#'     \item{\code{highestGeographicUnitTypeCovered}}{ A read/write active binding.
#'          The unit type describing the highest or broadest geographic unit type that can be assembled within this coverage. In some cases with multiple hierarchies there may be multiple geographic unit types assembled.
#'
#'                              \code{myDDI4_SpatialCoverage$highestGeographicUnitTypeCovered <- c(myUnitTypeURN)} sets the association 'highestGeographicUnitTypeCovered' from 'myDDI4_SpatialCoverage' to a character vector of DdiUrns for R6 UnitType class instances
#'
#'                              \code{myDDI4_SpatialCoverage$highestGeographicUnitTypeCovered} returns a character vector of of DdiUrns for R6  UnitType class instances}
#'   }
#'   \describe{
#'     \item{\code{lowestGeographicUnitTypeCovered}}{ A read/write active binding.
#'          The lowest geographic unit type identified within this coverage. Repeat if multiple geographic hierarchies are used and there is no single lowest level that can be used to compile geographic areas into higher levels. For example a file that has one hierarchy using State-County-CensusTract/BNA and also data identified for Place which cannot be constructed directly from Census Tracts as they may cross Place boundaries.
#'
#'                              \code{myDDI4_SpatialCoverage$lowestGeographicUnitTypeCovered <- c(myUnitTypeURN)} sets the association 'lowestGeographicUnitTypeCovered' from 'myDDI4_SpatialCoverage' to a character vector of DdiUrns for R6 UnitType class instances
#'
#'                              \code{myDDI4_SpatialCoverage$lowestGeographicUnitTypeCovered} returns a character vector of of DdiUrns for R6  UnitType class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_SpatialCoverage$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, description=NA, spatialAreaCode=NA, spatialObject=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_SpatialCoverage <- R6Class(classname="DDI4_SpatialCoverage",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, description=NA, spatialAreaCode=NA, spatialObject=NA,    hasBoundingBox=NA,usesGeographicUnitTypeClassification=NA,usesGeographicUnitClassification=NA,includesGeographicUnitType=NA,includesGeographicUnit=NA,highestGeographicUnitTypeCovered=NA,lowestGeographicUnitTypeCovered=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_SpatialCoverage may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(description)){
        private$.description <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=description , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(spatialAreaCode)){
        private$.spatialAreaCode <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=spatialAreaCode , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(spatialObject)){
        private$.spatialObject <- returnEnumeratedValues(enumerationObject=DDI4_SpatialObjectType$new(registerThisObject=FALSE), proposedCharacterVector=spatialObject , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(hasBoundingBox)){
        private$.hasBoundingBox <- returnUrnVectorIfValid(validClass="DDI4_BoundingBox", proposedUrnVector=hasBoundingBox , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasBoundingBox, referencingAssociationName="hasBoundingBox",  referencingUrn=thisUrn)  
      }
      if(!missing(usesGeographicUnitTypeClassification)){
        private$.usesGeographicUnitTypeClassification <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnitTypeClassification", proposedUrnVector=usesGeographicUnitTypeClassification , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesGeographicUnitTypeClassification, referencingAssociationName="usesGeographicUnitTypeClassification",  referencingUrn=thisUrn)  
      }
      if(!missing(usesGeographicUnitClassification)){
        private$.usesGeographicUnitClassification <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnitClassification", proposedUrnVector=usesGeographicUnitClassification , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesGeographicUnitClassification, referencingAssociationName="usesGeographicUnitClassification",  referencingUrn=thisUrn)  
      }
      if(!missing(includesGeographicUnitType)){
        private$.includesGeographicUnitType <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=includesGeographicUnitType , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.includesGeographicUnitType, referencingAssociationName="includesGeographicUnitType",  referencingUrn=thisUrn)  
      }
      if(!missing(includesGeographicUnit)){
        private$.includesGeographicUnit <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnit", proposedUrnVector=includesGeographicUnit , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.includesGeographicUnit, referencingAssociationName="includesGeographicUnit",  referencingUrn=thisUrn)  
      }
      if(!missing(highestGeographicUnitTypeCovered)){
        private$.highestGeographicUnitTypeCovered <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=highestGeographicUnitTypeCovered , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.highestGeographicUnitTypeCovered, referencingAssociationName="highestGeographicUnitTypeCovered",  referencingUrn=thisUrn)  
      }
      if(!missing(lowestGeographicUnitTypeCovered)){
        private$.lowestGeographicUnitTypeCovered <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=lowestGeographicUnitTypeCovered , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.lowestGeographicUnitTypeCovered, referencingAssociationName="lowestGeographicUnitTypeCovered",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    description = function(val){
      if (!missing(val)){
        private$.description <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.description
      }
    },

    spatialAreaCode = function(val){
      if (!missing(val)){
        private$.spatialAreaCode <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.spatialAreaCode
      }
    },

    spatialObject = function(val){
      if (!missing(val)){
        private$.spatialObject <- returnEnumeratedValues(enumerationObject=DDI4_SpatialObjectType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.spatialObject
      }
    },


# There are 7 class association(s):
    hasBoundingBox = function(val){
      if (!missing(val)){
        private$.hasBoundingBox <- returnUrnVectorIfValid(validClass="DDI4_BoundingBox", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasBoundingBox, referencingAssociationName="hasBoundingBox",  referencingUrn=thisUrn)  
       # invisible( private$.hasBoundingBox)
      } else {
          private$.hasBoundingBox
      }
    },

    highestGeographicUnitTypeCovered = function(val){
      if (!missing(val)){
        private$.highestGeographicUnitTypeCovered <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.highestGeographicUnitTypeCovered, referencingAssociationName="highestGeographicUnitTypeCovered",  referencingUrn=thisUrn)  
       # invisible( private$.highestGeographicUnitTypeCovered)
      } else {
          private$.highestGeographicUnitTypeCovered
      }
    },

    includesGeographicUnit = function(val){
      if (!missing(val)){
        private$.includesGeographicUnit <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnit", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.includesGeographicUnit, referencingAssociationName="includesGeographicUnit",  referencingUrn=thisUrn)  
       # invisible( private$.includesGeographicUnit)
      } else {
          private$.includesGeographicUnit
      }
    },

    includesGeographicUnitType = function(val){
      if (!missing(val)){
        private$.includesGeographicUnitType <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.includesGeographicUnitType, referencingAssociationName="includesGeographicUnitType",  referencingUrn=thisUrn)  
       # invisible( private$.includesGeographicUnitType)
      } else {
          private$.includesGeographicUnitType
      }
    },

    lowestGeographicUnitTypeCovered = function(val){
      if (!missing(val)){
        private$.lowestGeographicUnitTypeCovered <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.lowestGeographicUnitTypeCovered, referencingAssociationName="lowestGeographicUnitTypeCovered",  referencingUrn=thisUrn)  
       # invisible( private$.lowestGeographicUnitTypeCovered)
      } else {
          private$.lowestGeographicUnitTypeCovered
      }
    },

    usesGeographicUnitClassification = function(val){
      if (!missing(val)){
        private$.usesGeographicUnitClassification <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnitClassification", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesGeographicUnitClassification, referencingAssociationName="usesGeographicUnitClassification",  referencingUrn=thisUrn)  
       # invisible( private$.usesGeographicUnitClassification)
      } else {
          private$.usesGeographicUnitClassification
      }
    },

    usesGeographicUnitTypeClassification = function(val){
      if (!missing(val)){
        private$.usesGeographicUnitTypeClassification <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnitTypeClassification", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesGeographicUnitTypeClassification, referencingAssociationName="usesGeographicUnitTypeClassification",  referencingUrn=thisUrn)  
       # invisible( private$.usesGeographicUnitTypeClassification)
      } else {
          private$.usesGeographicUnitTypeClassification
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .description = NULL,
    .spatialAreaCode = NULL,
    .spatialObject = NULL,
    .hasBoundingBox = NULL,
    .usesGeographicUnitTypeClassification = NULL,
    .usesGeographicUnitClassification = NULL,
    .includesGeographicUnitType = NULL,
    .includesGeographicUnit = NULL,
    .highestGeographicUnitTypeCovered = NULL,
    .lowestGeographicUnitTypeCovered = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        description = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasBoundingBox = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_BoundingBox", 
          structure = "DdiUrn vector"
        ),
        highestGeographicUnitTypeCovered = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_UnitType", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        includesGeographicUnit = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_GeographicUnit", 
          structure = "DdiUrn vector"
        ),
        includesGeographicUnitType = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_UnitType", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        lowestGeographicUnitTypeCovered = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_UnitType", 
          structure = "DdiUrn vector"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        spatialAreaCode = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        spatialObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SpatialObjectType", 
          structure = "Enumeration vector"
        ),
        usesGeographicUnitClassification = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_GeographicUnitClassification", 
          structure = "DdiUrn vector"
        ),
        usesGeographicUnitTypeClassification = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_GeographicUnitTypeClassification", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_SpatialCoverage


#' @title DDI4_SpatialLine  Defines a geographic line (minimum of 2 points)
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/SpatialLine}{SpatialLine} 

#'
#'   Definition
#'
#'   ============
#'
#'   Defines a geographic line (minimum of 2 points)
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Street, River
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_SpatialLine$DdiUrn} returns the DDI URN of myDDI4_SpatialLine.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{uri}}{ A read/write active binding.
#'        A URI for the spatial file containing the line
#'
#'                              \code{myDDI4_SpatialLine$uri <- mycharacter} sets the uri property of 'myDDI4_SpatialLine' to the vector of datatype:character
#'
#'                              \code{myDDI4_SpatialLine$uri} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{lineLinkCode}}{ A read/write active binding.
#'        The lineLinkCode is the identifier of the specific line within the file. 
#'
#'                              \code{myDDI4_SpatialLine$lineLinkCode <- mycharacter} sets the lineLinkCode property of 'myDDI4_SpatialLine' to the vector of datatype:character
#'
#'                              \code{myDDI4_SpatialLine$lineLinkCode} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{shapeFileFormat}}{ A read/write active binding.
#'        The format of the shape file existing at the location indicated by the sibling ExternalURI element.
#'
#'                              \code{myDDI4_SpatialLine$shapeFileFormat <- list(myExternalControlledVocabularyEntry)} sets the 'shapeFileFormat' property of 'myDDI4_SpatialLine' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_SpatialLine$shapeFileFormat} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{point}}{ A read/write active binding.
#'        A geographic point defined by a latitude and longitude. A minimum of 2 points is required in order to define the line. 
#'
#'                              \code{myDDI4_SpatialLine$point <- list(mySpatialPoint)} sets the 'point' property of 'myDDI4_SpatialLine' to a list of R6 SpatialPoint class instances
#'
#'                              \code{myDDI4_SpatialLine$point} returns a list of of R6 : SpatialPoint class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_SpatialLine$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, uri=NA, lineLinkCode=NA, shapeFileFormat=NA, point=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_SpatialLine <- R6Class(classname="DDI4_SpatialLine",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, uri=NA, lineLinkCode=NA, shapeFileFormat=NA, point=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_SpatialLine may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(uri)){
        private$.uri <- returnVectorIfValid(validVectorClass="character", proposedVector=uri , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(lineLinkCode)){
        private$.lineLinkCode <- returnVectorIfValid(validVectorClass="character", proposedVector=lineLinkCode , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(shapeFileFormat)){
        private$.shapeFileFormat <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=shapeFileFormat , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(point)){
        private$.point <- returnR6ObjectIfValid(validClass="DDI4_SpatialPoint", proposedObjectList=point , minimumCardinality=2, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    lineLinkCode = function(val){
      if (!missing(val)){
        private$.lineLinkCode <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.lineLinkCode
      }
    },

    point = function(val){
      if (!missing(val)){
        private$.point <- returnR6ObjectIfValid(validClass="DDI4_SpatialPoint", proposedObjectList=val , minimumCardinality=2, maximumCardinality=Inf)  
      } else {
        private$.point
      }
    },

    shapeFileFormat = function(val){
      if (!missing(val)){
        private$.shapeFileFormat <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.shapeFileFormat
      }
    },

    uri = function(val){
      if (!missing(val)){
        private$.uri <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.uri
      }
    }


  ),

  private = list(
    .uri = NULL,
    .lineLinkCode = NULL,
    .shapeFileFormat = NULL,
    .point = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        lineLinkCode = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        point = c(
          min="2", 
          max="Inf", 
          datatype = "DDI4_SpatialPoint", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        shapeFileFormat = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        uri = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_SpatialLine


#' @title DDI4_SpatialPoint  A geographic point consisting of an X and Y coordinate
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/SpatialPoint}{SpatialPoint} 

#'
#'   Definition
#'
#'   ============
#'
#'   A geographic point consisting of an X and Y coordinate. Each coordinate value is expressed separately providing its value and format.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:PointType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_SpatialPoint$DdiUrn} returns the DDI URN of myDDI4_SpatialPoint.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{xCoordinate}}{ A read/write active binding.
#'        An X coordinate (latitudinal equivalent) value and format expressed using the Spatial Coordinate structure.
#'
#'                              \code{myDDI4_SpatialPoint$xCoordinate <- list(mySpatialCoordinate)} sets the 'xCoordinate' property of 'myDDI4_SpatialPoint' to a list of R6 SpatialCoordinate class instances
#'
#'                              \code{myDDI4_SpatialPoint$xCoordinate} returns a list of of R6 : SpatialCoordinate class instances}
#'   }
#'   \describe{
#'     \item{\code{yCoordinate}}{ A read/write active binding.
#'        A Y coordinate (longitudinal equivalent) value and format expressed using the Spatial Coordinate structure.
#'
#'                              \code{myDDI4_SpatialPoint$yCoordinate <- list(mySpatialCoordinate)} sets the 'yCoordinate' property of 'myDDI4_SpatialPoint' to a list of R6 SpatialCoordinate class instances
#'
#'                              \code{myDDI4_SpatialPoint$yCoordinate} returns a list of of R6 : SpatialCoordinate class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_SpatialPoint$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, xCoordinate=NA, yCoordinate=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_SpatialPoint <- R6Class(classname="DDI4_SpatialPoint",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, xCoordinate=NA, yCoordinate=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_SpatialPoint may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(xCoordinate)){
        private$.xCoordinate <- returnR6ObjectIfValid(validClass="DDI4_SpatialCoordinate", proposedObjectList=xCoordinate , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(yCoordinate)){
        private$.yCoordinate <- returnR6ObjectIfValid(validClass="DDI4_SpatialCoordinate", proposedObjectList=yCoordinate , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    xCoordinate = function(val){
      if (!missing(val)){
        private$.xCoordinate <- returnR6ObjectIfValid(validClass="DDI4_SpatialCoordinate", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.xCoordinate
      }
    },

    yCoordinate = function(val){
      if (!missing(val)){
        private$.yCoordinate <- returnR6ObjectIfValid(validClass="DDI4_SpatialCoordinate", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.yCoordinate
      }
    }


  ),

  private = list(
    .xCoordinate = NULL,
    .yCoordinate = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        xCoordinate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SpatialCoordinate", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        yCoordinate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SpatialCoordinate", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_SpatialPoint


#' @title DDI4_SpatialRelationship  Used to specify a relationship between one spatial object and another
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/SpatialRelationship}{SpatialRelationship} 

#'
#'   Definition
#'
#'   ============
#'
#'   Used to specify a relationship between one spatial object and another. Includes definition of the spatial object types being related, the spatial relation specification, and the event date.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_SpatialRelationship$DdiUrn} returns the DDI URN of myDDI4_SpatialRelationship.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasSpatialObjectPair}}{ A read/write active binding.
#'        Provides the types of the two spatial objects.
#'
#'                              \code{myDDI4_SpatialRelationship$hasSpatialObjectPair <- list(mySpatialObjectPairs)} sets the 'hasSpatialObjectPair' property of 'myDDI4_SpatialRelationship' to a list of R6 SpatialObjectPairs class instances
#'
#'                              \code{myDDI4_SpatialRelationship$hasSpatialObjectPair} returns a list of of R6 : SpatialObjectPairs class instances}
#'   }
#'   \describe{
#'     \item{\code{hasSpatialRelationSpecification}}{ A read/write active binding.
#'        Defines the relationship of the two spatial objects
#'
#'                              \code{myDDI4_SpatialRelationship$hasSpatialRelationSpecification <- list(mySpatialRelationSpecification)} sets the 'hasSpatialRelationSpecification' property of 'myDDI4_SpatialRelationship' to a list of R6 SpatialRelationSpecification class instances
#'
#'                              \code{myDDI4_SpatialRelationship$hasSpatialRelationSpecification} returns a list of of R6 : SpatialRelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{eventDate}}{ A read/write active binding.
#'        The date of the relationship change.
#'
#'                              \code{myDDI4_SpatialRelationship$eventDate <- list(myDate)} sets the 'eventDate' property of 'myDDI4_SpatialRelationship' to a list of R6 Date class instances
#'
#'                              \code{myDDI4_SpatialRelationship$eventDate} returns a list of of R6 : Date class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{relatesTo}}{ A read/write active binding.
#'          The geographic unit related to by the parent geographic unit
#'
#'                              \code{myDDI4_SpatialRelationship$relatesTo <- c(myGeographicUnitURN)} sets the association 'relatesTo' from 'myDDI4_SpatialRelationship' to a character vector of DdiUrns for R6 GeographicUnit class instances
#'
#'                              \code{myDDI4_SpatialRelationship$relatesTo} returns a character vector of of DdiUrns for R6  GeographicUnit class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_SpatialRelationship$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasSpatialObjectPair=NA, hasSpatialRelationSpecification=NA, eventDate=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_SpatialRelationship <- R6Class(classname="DDI4_SpatialRelationship",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasSpatialObjectPair=NA, hasSpatialRelationSpecification=NA, eventDate=NA,    relatesTo=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_SpatialRelationship may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(hasSpatialObjectPair)){
        private$.hasSpatialObjectPair <- returnEnumeratedValues(enumerationObject=DDI4_SpatialObjectPairs$new(registerThisObject=FALSE), proposedCharacterVector=hasSpatialObjectPair , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasSpatialRelationSpecification)){
        private$.hasSpatialRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_SpatialRelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasSpatialRelationSpecification , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(eventDate)){
        private$.eventDate <- returnR6ObjectIfValid(validClass="DDI4_Date", proposedObjectList=eventDate , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(relatesTo)){
        private$.relatesTo <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnit", proposedUrnVector=relatesTo , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.relatesTo, referencingAssociationName="relatesTo",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    eventDate = function(val){
      if (!missing(val)){
        private$.eventDate <- returnR6ObjectIfValid(validClass="DDI4_Date", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.eventDate
      }
    },

    hasSpatialObjectPair = function(val){
      if (!missing(val)){
        private$.hasSpatialObjectPair <- returnEnumeratedValues(enumerationObject=DDI4_SpatialObjectPairs$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasSpatialObjectPair
      }
    },

    hasSpatialRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasSpatialRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_SpatialRelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasSpatialRelationSpecification
      }
    },


# There are 1 class association(s):
    relatesTo = function(val){
      if (!missing(val)){
        private$.relatesTo <- returnUrnVectorIfValid(validClass="DDI4_GeographicUnit", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.relatesTo, referencingAssociationName="relatesTo",  referencingUrn=thisUrn)  
       # invisible( private$.relatesTo)
      } else {
          private$.relatesTo
      }
    }

  ),

  private = list(
    .hasSpatialObjectPair = NULL,
    .hasSpatialRelationSpecification = NULL,
    .eventDate = NULL,
    .relatesTo = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        eventDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Date", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasSpatialObjectPair = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SpatialObjectPairs", 
          structure = "Enumeration vector"
        ),
        hasSpatialRelationSpecification = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SpatialRelationSpecification", 
          structure = "Enumeration vector"
        ),
        relatesTo = c(
          min="0", 
          max="1", 
          datatype = "DDI4_GeographicUnit", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_SpatialRelationship


#' @title DDI4_Split  The components of a Split consists of a number of process steps to be executed concurrently with partial synchronization
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Split}{Split} 

#'
#'   Definition
#'
#'   ============
#'
#'   The components of a Split consists of a number of process steps to be executed concurrently with partial synchronization. Split completes as soon as all of its component process steps have been scheduled for execution.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Supports parallel processing that does not require completion to exit. 
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_TemporalRelationControlStep}}}
#'    \item{\code{\link{DDI4_WorkflowControlStep}}}
#'    \item{\code{\link{DDI4_WorkflowStep}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Split$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Split <- R6Class(classname="DDI4_Split",
  inherit = DDI4_TemporalRelationControlStep,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Split may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

    }
  ),

  active = list(
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        executes = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_WorkflowStep", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInformationFlow = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPerformedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowService", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        typeOfTemporalRelationship = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TemporalRelationSpecification", 
          structure = "Enumeration vector"
        ),
        usage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Split


#' @title DDI4_SplitJoin  SplitJoin consists of process steps that are executed concurrently (execution with barrier synchronization)
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/SplitJoin}{SplitJoin} 

#'
#'   Definition
#'
#'   ============
#'
#'   SplitJoin consists of process steps that are executed concurrently (execution with barrier synchronization). That is, SplitJoin completes when all of its components processes have completed. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A GANTT chart where a number of processes running parallel to each other are all prerequisites of a subsequent step.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Supports parallel processing that requires completion of all included process steps to exit.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_TemporalRelationControlStep}}}
#'    \item{\code{\link{DDI4_WorkflowControlStep}}}
#'    \item{\code{\link{DDI4_WorkflowStep}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_SplitJoin$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_SplitJoin <- R6Class(classname="DDI4_SplitJoin",
  inherit = DDI4_TemporalRelationControlStep,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_SplitJoin may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

    }
  ),

  active = list(
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        executes = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_WorkflowStep", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInformationFlow = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPerformedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowService", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        typeOfTemporalRelationship = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TemporalRelationSpecification", 
          structure = "Enumeration vector"
        ),
        usage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_SplitJoin


#' @title DDI4_Standard  Identifies the external standard used and describes the level of compliance with the standard in terms specific aspects of the standard's content
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Standard}{Standard} 

#'
#'   Definition
#'
#'   ============
#'
#'   Identifies the external standard used and describes the level of compliance with the standard in terms specific aspects of the standard's content.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:StandardType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{compliance}}{ A read/write active binding.
#'          Allows for a quality statement based on frameworks to be described using itemized properties. A reference to a concept, a coded value, or both can be used to specify the property from the standard framework identified in StandardUsed. Usage can provide further details or a general description of compliance with a standard.
#'
#'                              \code{myDDI4_Standard$compliance <- c(myComplianceStatementURN)} sets the association 'compliance' from 'myDDI4_Standard' to a character vector of DdiUrns for R6 ComplianceStatement class instances
#'
#'                              \code{myDDI4_Standard$compliance} returns a character vector of of DdiUrns for R6  ComplianceStatement class instances}
#'   }
#'   \describe{
#'     \item{\code{standardUsed}}{ A read/write active binding.
#'          Provide the citation and location of the published standard using the OtherMaterialType.
#'
#'                              \code{myDDI4_Standard$standardUsed <- c(myExternalMaterialURN)} sets the association 'standardUsed' from 'myDDI4_Standard' to a character vector of DdiUrns for R6 ExternalMaterial class instances
#'
#'                              \code{myDDI4_Standard$standardUsed} returns a character vector of of DdiUrns for R6  ExternalMaterial class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Standard$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Standard <- R6Class(classname="DDI4_Standard",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, compliance=NA,standardUsed=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Standard may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)



       if(!missing(compliance)){
        private$.compliance <- returnUrnVectorIfValid(validClass="DDI4_ComplianceStatement", proposedUrnVector=compliance , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.compliance, referencingAssociationName="compliance",  referencingUrn=thisUrn)  
      }
      if(!missing(standardUsed)){
        private$.standardUsed <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=standardUsed , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.standardUsed, referencingAssociationName="standardUsed",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 2 class association(s):
    compliance = function(val){
      if (!missing(val)){
        private$.compliance <- returnUrnVectorIfValid(validClass="DDI4_ComplianceStatement", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.compliance, referencingAssociationName="compliance",  referencingUrn=thisUrn)  
       # invisible( private$.compliance)
      } else {
          private$.compliance
      }
    },

    standardUsed = function(val){
      if (!missing(val)){
        private$.standardUsed <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.standardUsed, referencingAssociationName="standardUsed",  referencingUrn=thisUrn)  
       # invisible( private$.standardUsed)
      } else {
          private$.standardUsed
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .compliance = NULL,
    .standardUsed = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        compliance = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ComplianceStatement", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        standardUsed = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Standard


#' @title DDI4_StandardWeight  Provides an identified value for a standard weight expressed as an xs:float
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/StandardWeight}{StandardWeight} 

#'
#'   Definition
#'
#'   ============
#'
#'   Provides an identified value for a standard weight expressed as an xs:float. This object may be referenced by a variable or statistic and used as a weight for analysis.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A simple sample survey without an individual weight variable will have a StandardWeight to be used on all cases: 1% sample has a sample weight of 100
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   A simple random sample survey without an individual weight variable will have a StandardWeight to be used on all cases. This is common when the sample is a simple random sample of a population with no oversampling to provide larger case counts for small populations.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   d:StandardWeightType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_StandardWeight$DdiUrn} returns the DDI URN of myDDI4_StandardWeight.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{standardWeightValue}}{ A read/write active binding.
#'        Provides the standard weight used for weighted analysis of data expressed as an xs:float. This element is referenced by the variable and/or statistics calculated using the standard weight.
#'
#'                              \code{myDDI4_StandardWeight$standardWeightValue <- mynumeric} sets the standardWeightValue property of 'myDDI4_StandardWeight' to the vector of datatype:numeric
#'
#'                              \code{myDDI4_StandardWeight$standardWeightValue} returns a vector of datatype: numeric}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_StandardWeight$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, standardWeightValue=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_StandardWeight <- R6Class(classname="DDI4_StandardWeight",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, standardWeightValue=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_StandardWeight may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(standardWeightValue)){
        private$.standardWeightValue <- returnVectorIfValid(validVectorClass="numeric", proposedVector=standardWeightValue , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 1 properties: 
    standardWeightValue = function(val){
      if (!missing(val)){
        private$.standardWeightValue <- returnVectorIfValid(validVectorClass="numeric", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.standardWeightValue
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .standardWeightValue = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        standardWeightValue = c(
          min="0", 
          max="1", 
          datatype = "numeric", 
          structure = "vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_StandardWeight


#' @title DDI4_Statement  A Statement is a type of Instrument Component containing  human readable text or referred material
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Statement}{Statement} 

#'
#'   Definition
#'
#'   ============
#'
#'   A Statement is a type of Instrument Component containing  human readable text or referred material. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Introductory text; Explanations; Labels, Headers, Help screens, etc.  "Thank you for agreeing to take this survey. We will start with a brief set of demographic questions." "The following set of questions are related to your household income. Please consider all members of the household and all sources of income when answering these questions."
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   It is not directly related to another specific Instrument Component such as an InstanceQueston or InstanceMeasurement. It may be placed anywhere in a WorkflowStepSequence. 
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   DDI:StatementItem
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   d:StatementItemType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_InstrumentComponent}}}
#'    \item{\code{\link{DDI4_Act}}}
#'    \item{\code{\link{DDI4_WorkflowStep}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Statement$DdiUrn} returns the DDI URN of myDDI4_Statement.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{statementText}}{ A read/write active binding.
#'        Structured human-readable text that allows for dynamic content. For example, the insertion of a name or gender specific pronoun. Repeat ONLY for capturing the same content in multiple languages.
#'
#'                              \code{myDDI4_Statement$statementText <- list(myDynamicText)} sets the 'statementText' property of 'myDDI4_Statement' to a list of R6 DynamicText class instances
#'
#'                              \code{myDDI4_Statement$statementText} returns a list of of R6 : DynamicText class instances}
#'   }
#'   \describe{
#'     \item{\code{purposeOfStatement}}{ A read/write active binding.
#'        Describes the use of the statement within the instrument. For example, a section divider, welcome statement, or other text.
#'
#'                              \code{myDDI4_Statement$purposeOfStatement <- list(myExternalControlledVocabularyEntry)} sets the 'purposeOfStatement' property of 'myDDI4_Statement' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_Statement$purposeOfStatement} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Statement$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, statementText=NA, purposeOfStatement=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Statement <- R6Class(classname="DDI4_Statement",
  inherit = DDI4_InstrumentComponent,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, statementText=NA, purposeOfStatement=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Statement may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(statementText)){
        private$.statementText <- returnR6ObjectIfValid(validClass="DDI4_DynamicText", proposedObjectList=statementText , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purposeOfStatement)){
        private$.purposeOfStatement <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=purposeOfStatement , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    purposeOfStatement = function(val){
      if (!missing(val)){
        private$.purposeOfStatement <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purposeOfStatement
      }
    },

    statementText = function(val){
      if (!missing(val)){
        private$.statementText <- returnR6ObjectIfValid(validClass="DDI4_DynamicText", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.statementText
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .statementText = NULL,
    .purposeOfStatement = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasExternalAid = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalAid", 
          structure = "DdiUrn vector"
        ),
        hasInformationFlow = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInstruction = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Instruction", 
          structure = "DdiUrn vector"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPerformedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowService", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purposeOfStatement = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        statementText = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_DynamicText", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Statement


#' @title DDI4_Statistic  The value of the statistic expressed as an xs:decimal and/or xs:double
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Statistic}{Statistic} 

#'
#'   Definition
#'
#'   ============
#'
#'   The value of the statistic expressed as an xs:decimal and/or xs:double. Indicates whether it is weighted value and the computation base.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   pi:StatisticType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Statistic$DdiUrn} returns the DDI URN of myDDI4_Statistic.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{isWeighted}}{ A read/write active binding.
#'        Set to "true" if the statistic is weighted using the weight designated in VariableStatistics.
#'
#'                              \code{myDDI4_Statistic$isWeighted <- mylogical} sets the isWeighted property of 'myDDI4_Statistic' to the vector of datatype:logical
#'
#'                              \code{myDDI4_Statistic$isWeighted} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{computationBase}}{ A read/write active binding.
#'        Defines the cases included in determining the statistic. The options are total=all cases, valid and missing (invalid); validOnly=Only valid values, missing (invalid) are not included in the calculation; missingOnly=Only missing (invalid) cases included in the calculation.
#'
#'                              \code{myDDI4_Statistic$computationBase <- list(myComputationBaseList)} sets the 'computationBase' property of 'myDDI4_Statistic' to a list of R6 ComputationBaseList class instances
#'
#'                              \code{myDDI4_Statistic$computationBase} returns a list of of R6 : ComputationBaseList class instances}
#'   }
#'   \describe{
#'     \item{\code{typeOfNumericValue}}{ A read/write active binding.
#'        Indicate the type of numeric value as decimal, float, double
#'
#'                              \code{myDDI4_Statistic$typeOfNumericValue <- mycharacter} sets the typeOfNumericValue property of 'myDDI4_Statistic' to the vector of datatype:character
#'
#'                              \code{myDDI4_Statistic$typeOfNumericValue} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{content}}{ A read/write active binding.
#'        The value of the statistic expressed as a decimal, float, or double.
#'
#'                              \code{myDDI4_Statistic$content <- mynumeric} sets the content property of 'myDDI4_Statistic' to the vector of datatype:numeric
#'
#'                              \code{myDDI4_Statistic$content} returns a vector of datatype: numeric}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Statistic$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, isWeighted=NA, computationBase=NA, typeOfNumericValue=NA, content=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Statistic <- R6Class(classname="DDI4_Statistic",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, isWeighted=NA, computationBase=NA, typeOfNumericValue=NA, content=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Statistic may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(isWeighted)){
        private$.isWeighted <- returnVectorIfValid(validVectorClass="logical", proposedVector=isWeighted , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(computationBase)){
        private$.computationBase <- returnEnumeratedValues(enumerationObject=DDI4_ComputationBaseList$new(registerThisObject=FALSE), proposedCharacterVector=computationBase , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(typeOfNumericValue)){
        private$.typeOfNumericValue <- returnVectorIfValid(validVectorClass="character", proposedVector=typeOfNumericValue , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(content)){
        private$.content <- returnVectorIfValid(validVectorClass="numeric", proposedVector=content , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    computationBase = function(val){
      if (!missing(val)){
        private$.computationBase <- returnEnumeratedValues(enumerationObject=DDI4_ComputationBaseList$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.computationBase
      }
    },

    content = function(val){
      if (!missing(val)){
        private$.content <- returnVectorIfValid(validVectorClass="numeric", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.content
      }
    },

    isWeighted = function(val){
      if (!missing(val)){
        private$.isWeighted <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isWeighted
      }
    },

    typeOfNumericValue = function(val){
      if (!missing(val)){
        private$.typeOfNumericValue <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.typeOfNumericValue
      }
    }


  ),

  private = list(
    .isWeighted = NULL,
    .computationBase = NULL,
    .typeOfNumericValue = NULL,
    .content = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        computationBase = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ComputationBaseList", 
          structure = "Enumeration vector"
        ),
        content = c(
          min="0", 
          max="1", 
          datatype = "numeric", 
          structure = "vector"
        ),
        isWeighted = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        typeOfNumericValue = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Statistic


#' @title DDI4_StatisticalClassification  A Statistical Classification is a set of Categories represented by Classification Items which may be assigned to one or more variables registered in statistical surveys or administrative files, and used in the production and dissemination of statistics
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/StatisticalClassification}{StatisticalClassification} 

#'
#'   Definition
#'
#'   ============
#'
#'   A Statistical Classification is a set of Categories represented by Classification Items which may be assigned to one or more variables registered in statistical surveys or administrative files, and used in the production and dissemination of statistics. The Categories at each Level of the classification structure must be mutually exclusive and jointly exhaustive of all objects/units in the population of interest. (Source: GSIM StatisticalClassification)
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   A Classification Item represents a Category at a certain Level within a Statistical Classification. The Categories are defined with reference to one or more characteristics of a particular universe of units of observation. A Statistical Classification may have a flat, linear structure or may be hierarchically structured, such that all Categories at lower Levels are sub-Categories of Categories at the next Level up. (Source: GSIM StatisticalClassification) 
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Statistical Classification
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_EnumerationDomain}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_StatisticalClassification$DdiUrn} returns the DDI URN of myDDI4_StatisticalClassification.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{releaseDate}}{ A read/write active binding.
#'        Date the Statistical Classification was released
#'
#'                              \code{myDDI4_StatisticalClassification$releaseDate <- list(myDate)} sets the 'releaseDate' property of 'myDDI4_StatisticalClassification' to a list of R6 Date class instances
#'
#'                              \code{myDDI4_StatisticalClassification$releaseDate} returns a list of of R6 : Date class instances}
#'   }
#'   \describe{
#'     \item{\code{validDates}}{ A read/write active binding.
#'        The date the statistical classification enters production use and the date on which the Statistical Classification was superseded by a successor version or otherwise ceased to be valid. (Source: GSIM Statistical Classification)
#'
#'                              \code{myDDI4_StatisticalClassification$validDates <- list(myDateRange)} sets the 'validDates' property of 'myDDI4_StatisticalClassification' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_StatisticalClassification$validDates} returns a list of of R6 : DateRange class instances}
#'   }
#'   \describe{
#'     \item{\code{isCurrent}}{ A read/write active binding.
#'        Indicates if the Statistical Classification is currently valid.
#'
#'                              \code{myDDI4_StatisticalClassification$isCurrent <- mylogical} sets the isCurrent property of 'myDDI4_StatisticalClassification' to the vector of datatype:logical
#'
#'                              \code{myDDI4_StatisticalClassification$isCurrent} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{isFloating}}{ A read/write active binding.
#'        Indicates if the Statistical Classification is a floating classification. In a floating statistical classification, a validity period should be defined for all Classification Items which will allow the display of the item structure and content at different points of time. (Source: GSIM StatisticalClassification/Floating
#'
#'                              \code{myDDI4_StatisticalClassification$isFloating <- mylogical} sets the isFloating property of 'myDDI4_StatisticalClassification' to the vector of datatype:logical
#'
#'                              \code{myDDI4_StatisticalClassification$isFloating} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{changeFromBase}}{ A read/write active binding.
#'        Describes the relationship between the variant and its base Statistical Classification, including regroupings, aggregations added and extensions. (Source: GSIM StatisticalClassification/Changes from base Statistical Classification)
#'
#'                              \code{myDDI4_StatisticalClassification$changeFromBase <- list(myInternationalStructuredString)} sets the 'changeFromBase' property of 'myDDI4_StatisticalClassification' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_StatisticalClassification$changeFromBase} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{purposeOfVariant}}{ A read/write active binding.
#'        If the Statistical Classification is a variant, notes the specific purpose for which it was developed. (Source: GSIM StatisticalClassification/Purpose of variant)
#'
#'                              \code{myDDI4_StatisticalClassification$purposeOfVariant <- list(myInternationalStructuredString)} sets the 'purposeOfVariant' property of 'myDDI4_StatisticalClassification' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_StatisticalClassification$purposeOfVariant} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{copyright}}{ A read/write active binding.
#'        Copyright of the statistical classification.
#'
#'                              \code{myDDI4_StatisticalClassification$copyright <- list(myInternationalString)} sets the 'copyright' property of 'myDDI4_StatisticalClassification' to a list of R6 InternationalString class instances
#'
#'                              \code{myDDI4_StatisticalClassification$copyright} returns a list of of R6 : InternationalString class instances}
#'   }
#'   \describe{
#'     \item{\code{updateChanges}}{ A read/write active binding.
#'        Summary description of changes which have occurred since the most recent classification version or classification update came into force.
#'
#'                              \code{myDDI4_StatisticalClassification$updateChanges <- list(myInternationalStructuredString)} sets the 'updateChanges' property of 'myDDI4_StatisticalClassification' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_StatisticalClassification$updateChanges} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{availableLanguage}}{ A read/write active binding.
#'        A list of languages in which the Statistical Classification is available. Supports the indication of multiple languages within a single property. Supports use of codes defined by the RFC 1766.
#'
#'                              \code{myDDI4_StatisticalClassification$availableLanguage <- mycharacter} sets the availableLanguage property of 'myDDI4_StatisticalClassification' to the vector of datatype:character
#'
#'                              \code{myDDI4_StatisticalClassification$availableLanguage} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A structured display label providing a fully human readable display for the identification of the object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_StatisticalClassification$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_StatisticalClassification' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_StatisticalClassification$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#'   \describe{
#'     \item{\code{rationale}}{ A read/write active binding.
#'        Explanation of the reasons some decision was made or some object exists. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_StatisticalClassification$rationale <- list(myInternationalStructuredString)} sets the 'rationale' property of 'myDDI4_StatisticalClassification' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_StatisticalClassification$rationale} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{usage}}{ A read/write active binding.
#'        Explanation of the ways in which some decision or object is employed. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_StatisticalClassification$usage <- list(myInternationalStructuredString)} sets the 'usage' property of 'myDDI4_StatisticalClassification' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_StatisticalClassification$usage} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        Allows for the identification of the member and optionally provides an index for the member within an ordered array
#'
#'                              \code{myDDI4_StatisticalClassification$contains <- list(myClassificationItemIndicator)} sets the 'contains' property of 'myDDI4_StatisticalClassification' to a list of R6 ClassificationItemIndicator class instances
#'
#'                              \code{myDDI4_StatisticalClassification$contains} returns a list of of R6 : ClassificationItemIndicator class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isMaintainedBy}}{ A read/write active binding.
#'          Organization, agency, or group within an agency responsible for the maintenance and upkeep of the statistical classification.
#'
#'                              \code{myDDI4_StatisticalClassification$isMaintainedBy <- c(myOrganizationURN)} sets the association 'isMaintainedBy' from 'myDDI4_StatisticalClassification' to a character vector of DdiUrns for R6 Organization class instances
#'
#'                              \code{myDDI4_StatisticalClassification$isMaintainedBy} returns a character vector of of DdiUrns for R6  Organization class instances}
#'   }
#'   \describe{
#'     \item{\code{distributedAs}}{ A read/write active binding.
#'          Description and link to a publication, including print, PDF, HTML and other electronic formats, in which the Statistical Classification has been published. This is similar to dcat:Distribution.
#'
#'                              \code{myDDI4_StatisticalClassification$distributedAs <- c(myExternalMaterialURN)} sets the association 'distributedAs' from 'myDDI4_StatisticalClassification' to a character vector of DdiUrns for R6 ExternalMaterial class instances
#'
#'                              \code{myDDI4_StatisticalClassification$distributedAs} returns a character vector of of DdiUrns for R6  ExternalMaterial class instances}
#'   }
#'   \describe{
#'     \item{\code{variantOf}}{ A read/write active binding.
#'          Statistical Classification on which the current variant is based, and any subsequent versions of that Statistical Classification to which it is also applicable.

#'
#'                              \code{myDDI4_StatisticalClassification$variantOf <- c(myStatisticalClassificationURN)} sets the association 'variantOf' from 'myDDI4_StatisticalClassification' to a character vector of DdiUrns for R6 StatisticalClassification class instances
#'
#'                              \code{myDDI4_StatisticalClassification$variantOf} returns a character vector of of DdiUrns for R6  StatisticalClassification class instances}
#'   }
#'   \describe{
#'     \item{\code{isIndexedBy}}{ A read/write active binding.
#'          ClassificationIndex(es) related to the StatisticalClassification.
#'
#'                              \code{myDDI4_StatisticalClassification$isIndexedBy <- c(myClassificationIndexURN)} sets the association 'isIndexedBy' from 'myDDI4_StatisticalClassification' to a character vector of DdiUrns for R6 ClassificationIndex class instances
#'
#'                              \code{myDDI4_StatisticalClassification$isIndexedBy} returns a character vector of of DdiUrns for R6  ClassificationIndex class instances}
#'   }
#'   \describe{
#'     \item{\code{predecessor}}{ A read/write active binding.
#'          Statistical Classification superseded by the actual Statistical Classification (for those Statistical Classifications that are versions or updates).


#'
#'                              \code{myDDI4_StatisticalClassification$predecessor <- c(myStatisticalClassificationURN)} sets the association 'predecessor' from 'myDDI4_StatisticalClassification' to a character vector of DdiUrns for R6 StatisticalClassification class instances
#'
#'                              \code{myDDI4_StatisticalClassification$predecessor} returns a character vector of of DdiUrns for R6  StatisticalClassification class instances}
#'   }
#'   \describe{
#'     \item{\code{successor}}{ A read/write active binding.
#'          Statistical Classification that supersedes the actual Statistical Classification (for those Statistical Classifications that are versions or updates), 
#'
#'                              \code{myDDI4_StatisticalClassification$successor <- c(myStatisticalClassificationURN)} sets the association 'successor' from 'myDDI4_StatisticalClassification' to a character vector of DdiUrns for R6 StatisticalClassification class instances
#'
#'                              \code{myDDI4_StatisticalClassification$successor} returns a character vector of of DdiUrns for R6  StatisticalClassification class instances}
#'   }
#'   \describe{
#'     \item{\code{isStructuredBy}}{ A read/write active binding.
#'          A complex structure, hierarchical, for a ClassificationItem. Restricted to single structure.
#'
#'                              \code{myDDI4_StatisticalClassification$isStructuredBy <- c(myClassificationItemRelationStructureURN)} sets the association 'isStructuredBy' from 'myDDI4_StatisticalClassification' to a character vector of DdiUrns for R6 ClassificationItemRelationStructure class instances
#'
#'                              \code{myDDI4_StatisticalClassification$isStructuredBy} returns a character vector of of DdiUrns for R6  ClassificationItemRelationStructure class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_StatisticalClassification$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, releaseDate=NA, validDates=NA, isCurrent=NA, isFloating=NA, changeFromBase=NA, purposeOfVariant=NA, copyright=NA, updateChanges=NA, availableLanguage=NA, displayLabel=NA, rationale=NA, usage=NA, contains=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_StatisticalClassification <- R6Class(classname="DDI4_StatisticalClassification",
  inherit = DDI4_EnumerationDomain,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, releaseDate=NA, validDates=NA, isCurrent=NA, isFloating=NA, changeFromBase=NA, purposeOfVariant=NA, copyright=NA, updateChanges=NA, availableLanguage=NA, displayLabel=NA, rationale=NA, usage=NA, contains=NA,    isMaintainedBy=NA,distributedAs=NA,variantOf=NA,isIndexedBy=NA,predecessor=NA,successor=NA,isStructuredBy=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_StatisticalClassification may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(releaseDate)){
        private$.releaseDate <- returnR6ObjectIfValid(validClass="DDI4_Date", proposedObjectList=releaseDate , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(validDates)){
        private$.validDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=validDates , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isCurrent)){
        private$.isCurrent <- returnVectorIfValid(validVectorClass="logical", proposedVector=isCurrent , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isFloating)){
        private$.isFloating <- returnVectorIfValid(validVectorClass="logical", proposedVector=isFloating , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(changeFromBase)){
        private$.changeFromBase <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=changeFromBase , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(purposeOfVariant)){
        private$.purposeOfVariant <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purposeOfVariant , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(copyright)){
        private$.copyright <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=copyright , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(updateChanges)){
        private$.updateChanges <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=updateChanges , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(availableLanguage)){
        private$.availableLanguage <- returnVectorIfValid(validVectorClass="character", proposedVector=availableLanguage , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(rationale)){
        private$.rationale <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=rationale , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(usage)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=usage , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_ClassificationItemIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(isMaintainedBy)){
        private$.isMaintainedBy <- returnUrnVectorIfValid(validClass="DDI4_Organization", proposedUrnVector=isMaintainedBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isMaintainedBy, referencingAssociationName="isMaintainedBy",  referencingUrn=thisUrn)  
      }
      if(!missing(distributedAs)){
        private$.distributedAs <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=distributedAs , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.distributedAs, referencingAssociationName="distributedAs",  referencingUrn=thisUrn)  
      }
      if(!missing(variantOf)){
        private$.variantOf <- returnUrnVectorIfValid(validClass="DDI4_StatisticalClassification", proposedUrnVector=variantOf , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.variantOf, referencingAssociationName="variantOf",  referencingUrn=thisUrn)  
      }
      if(!missing(isIndexedBy)){
        private$.isIndexedBy <- returnUrnVectorIfValid(validClass="DDI4_ClassificationIndex", proposedUrnVector=isIndexedBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isIndexedBy, referencingAssociationName="isIndexedBy",  referencingUrn=thisUrn)  
      }
      if(!missing(predecessor)){
        private$.predecessor <- returnUrnVectorIfValid(validClass="DDI4_StatisticalClassification", proposedUrnVector=predecessor , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.predecessor, referencingAssociationName="predecessor",  referencingUrn=thisUrn)  
      }
      if(!missing(successor)){
        private$.successor <- returnUrnVectorIfValid(validClass="DDI4_StatisticalClassification", proposedUrnVector=successor , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.successor, referencingAssociationName="successor",  referencingUrn=thisUrn)  
      }
      if(!missing(isStructuredBy)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_ClassificationItemRelationStructure", proposedUrnVector=isStructuredBy , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 13 properties: 
    availableLanguage = function(val){
      if (!missing(val)){
        private$.availableLanguage <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.availableLanguage
      }
    },

    changeFromBase = function(val){
      if (!missing(val)){
        private$.changeFromBase <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.changeFromBase
      }
    },

    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_ClassificationItemIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },

    copyright = function(val){
      if (!missing(val)){
        private$.copyright <- returnR6ObjectIfValid(validClass="DDI4_InternationalString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.copyright
      }
    },

    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },

    isCurrent = function(val){
      if (!missing(val)){
        private$.isCurrent <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isCurrent
      }
    },

    isFloating = function(val){
      if (!missing(val)){
        private$.isFloating <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isFloating
      }
    },

    purposeOfVariant = function(val){
      if (!missing(val)){
        private$.purposeOfVariant <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purposeOfVariant
      }
    },

    rationale = function(val){
      if (!missing(val)){
        private$.rationale <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.rationale
      }
    },

    releaseDate = function(val){
      if (!missing(val)){
        private$.releaseDate <- returnR6ObjectIfValid(validClass="DDI4_Date", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.releaseDate
      }
    },

    updateChanges = function(val){
      if (!missing(val)){
        private$.updateChanges <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.updateChanges
      }
    },

    usage = function(val){
      if (!missing(val)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.usage
      }
    },

    validDates = function(val){
      if (!missing(val)){
        private$.validDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.validDates
      }
    },


# There are 7 class association(s):
    distributedAs = function(val){
      if (!missing(val)){
        private$.distributedAs <- returnUrnVectorIfValid(validClass="DDI4_ExternalMaterial", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.distributedAs, referencingAssociationName="distributedAs",  referencingUrn=thisUrn)  
       # invisible( private$.distributedAs)
      } else {
          private$.distributedAs
      }
    },

    isIndexedBy = function(val){
      if (!missing(val)){
        private$.isIndexedBy <- returnUrnVectorIfValid(validClass="DDI4_ClassificationIndex", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isIndexedBy, referencingAssociationName="isIndexedBy",  referencingUrn=thisUrn)  
       # invisible( private$.isIndexedBy)
      } else {
          private$.isIndexedBy
      }
    },

    isMaintainedBy = function(val){
      if (!missing(val)){
        private$.isMaintainedBy <- returnUrnVectorIfValid(validClass="DDI4_Organization", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isMaintainedBy, referencingAssociationName="isMaintainedBy",  referencingUrn=thisUrn)  
       # invisible( private$.isMaintainedBy)
      } else {
          private$.isMaintainedBy
      }
    },

    isStructuredBy = function(val){
      if (!missing(val)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_ClassificationItemRelationStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
       # invisible( private$.isStructuredBy)
      } else {
          private$.isStructuredBy
      }
    },

    predecessor = function(val){
      if (!missing(val)){
        private$.predecessor <- returnUrnVectorIfValid(validClass="DDI4_StatisticalClassification", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.predecessor, referencingAssociationName="predecessor",  referencingUrn=thisUrn)  
       # invisible( private$.predecessor)
      } else {
          private$.predecessor
      }
    },

    successor = function(val){
      if (!missing(val)){
        private$.successor <- returnUrnVectorIfValid(validClass="DDI4_StatisticalClassification", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.successor, referencingAssociationName="successor",  referencingUrn=thisUrn)  
       # invisible( private$.successor)
      } else {
          private$.successor
      }
    },

    variantOf = function(val){
      if (!missing(val)){
        private$.variantOf <- returnUrnVectorIfValid(validClass="DDI4_StatisticalClassification", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.variantOf, referencingAssociationName="variantOf",  referencingUrn=thisUrn)  
       # invisible( private$.variantOf)
      } else {
          private$.variantOf
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .releaseDate = NULL,
    .validDates = NULL,
    .isCurrent = NULL,
    .isFloating = NULL,
    .changeFromBase = NULL,
    .purposeOfVariant = NULL,
    .copyright = NULL,
    .updateChanges = NULL,
    .availableLanguage = NULL,
    .displayLabel = NULL,
    .rationale = NULL,
    .usage = NULL,
    .contains = NULL,
    .isMaintainedBy = NULL,
    .distributedAs = NULL,
    .variantOf = NULL,
    .isIndexedBy = NULL,
    .predecessor = NULL,
    .successor = NULL,
    .isStructuredBy = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        availableLanguage = c(
          min="0", 
          max="Inf", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        changeFromBase = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ClassificationItemIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        copyright = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        distributedAs = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isCurrent = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isFloating = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isIndexedBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ClassificationIndex", 
          structure = "DdiUrn vector"
        ),
        isMaintainedBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Organization", 
          structure = "DdiUrn vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isStructuredBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ClassificationItemRelationStructure", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        predecessor = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_StatisticalClassification", 
          structure = "DdiUrn vector"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purposeOfVariant = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        rationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        references = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CategorySet", 
          structure = "DdiUrn vector"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        releaseDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Date", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        successor = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_StatisticalClassification", 
          structure = "DdiUrn vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        updateChanges = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usesLevelStructure = c(
          min="0", 
          max="1", 
          datatype = "DDI4_LevelStructure", 
          structure = "DdiUrn vector"
        ),
        validDates = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        variantOf = c(
          min="0", 
          max="1", 
          datatype = "DDI4_StatisticalClassification", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_StatisticalClassification


#' @title DDI4_StatisticalClassificationIndicator  Member Indicator for use with member type StatisticalClassification
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/StatisticalClassificationIndicator}{StatisticalClassificationIndicator} 

#'
#'   Definition
#'
#'   ============
#'
#'   Member Indicator for use with member type StatisticalClassification
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_StatisticalClassificationIndicator$DdiUrn} returns the DDI URN of myDDI4_StatisticalClassificationIndicator.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Index value of member in an ordered array
#'
#'                              \code{myDDI4_StatisticalClassificationIndicator$index <- myinteger} sets the index property of 'myDDI4_StatisticalClassificationIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_StatisticalClassificationIndicator$index} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{member}}{ A read/write active binding.
#'          Restricts member target class to StatisticalClassification
#'
#'                              \code{myDDI4_StatisticalClassificationIndicator$member <- c(myStatisticalClassificationURN)} sets the association 'member' from 'myDDI4_StatisticalClassificationIndicator' to a character vector of DdiUrns for R6 StatisticalClassification class instances
#'
#'                              \code{myDDI4_StatisticalClassificationIndicator$member} returns a character vector of of DdiUrns for R6  StatisticalClassification class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_StatisticalClassificationIndicator$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_StatisticalClassificationIndicator <- R6Class(classname="DDI4_StatisticalClassificationIndicator",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA,    member=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_StatisticalClassificationIndicator may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(member)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_StatisticalClassification", proposedUrnVector=member , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.index
      }
    },


# There are 1 class association(s):
    member = function(val){
      if (!missing(val)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_StatisticalClassification", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
       # invisible( private$.member)
      } else {
          private$.member
      }
    }

  ),

  private = list(
    .index = NULL,
    .member = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        index = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        member = c(
          min="1", 
          max="1", 
          datatype = "DDI4_StatisticalClassification", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_StatisticalClassificationIndicator


#' @title DDI4_StatisticalClassificationRelation  Specifies the Statistical Classifications for the source and target of the complex relationship and defines the relationship
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/StatisticalClassificationRelation}{StatisticalClassificationRelation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Specifies the Statistical Classifications for the source and target of the complex relationship and defines the relationship.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_StatisticalClassificationRelation$DdiUrn} returns the DDI URN of myDDI4_StatisticalClassificationRelation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an enumeration list
#'
#'                              \code{myDDI4_StatisticalClassificationRelation$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_StatisticalClassificationRelation' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_StatisticalClassificationRelation$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_StatisticalClassificationRelation$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_StatisticalClassificationRelation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_StatisticalClassificationRelation$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_StatisticalClassificationRelation$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_StatisticalClassificationRelation' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_StatisticalClassificationRelation$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          First member of the relationship
#'
#'                              \code{myDDI4_StatisticalClassificationRelation$source <- c(myStatisticalClassificationURN)} sets the association 'source' from 'myDDI4_StatisticalClassificationRelation' to a character vector of DdiUrns for R6 StatisticalClassification class instances
#'
#'                              \code{myDDI4_StatisticalClassificationRelation$source} returns a character vector of of DdiUrns for R6  StatisticalClassification class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          The second member of the relationship
#'
#'                              \code{myDDI4_StatisticalClassificationRelation$target <- c(myStatisticalClassificationURN)} sets the association 'target' from 'myDDI4_StatisticalClassificationRelation' to a character vector of DdiUrns for R6 StatisticalClassification class instances
#'
#'                              \code{myDDI4_StatisticalClassificationRelation$target} returns a character vector of of DdiUrns for R6  StatisticalClassification class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_StatisticalClassificationRelation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_StatisticalClassificationRelation <- R6Class(classname="DDI4_StatisticalClassificationRelation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_StatisticalClassificationRelation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_StatisticalClassification", proposedUrnVector=source , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_StatisticalClassification", proposedUrnVector=target , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_StatisticalClassification", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_StatisticalClassification", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_StatisticalClassification", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_StatisticalClassification", 
          structure = "DdiUrn vector"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_StatisticalClassificationRelation


#' @title DDI4_StatisticalClassificationRelationStructure  A structure for describing the complex relationship between statistical classifications in a classification series
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/StatisticalClassificationRelationStructure}{StatisticalClassificationRelationStructure} 

#'
#'   Definition
#'
#'   ============
#'
#'   A structure for describing the complex relationship between statistical classifications in a classification series
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A classification series that branches into separately versioned classifications
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Can use relation specification information to more fully describe the relationship between members such as parent/child, whole/part, general/specific, equivalence, etc.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_StatisticalClassificationRelationStructure$DdiUrn} returns the DDI URN of myDDI4_StatisticalClassificationRelationStructure.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an enumeration list.  Use if all relations within this relation structure are of the same specification.
#'
#'                              \code{myDDI4_StatisticalClassificationRelationStructure$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_StatisticalClassificationRelationStructure' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_StatisticalClassificationRelationStructure$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_StatisticalClassificationRelationStructure$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_StatisticalClassificationRelationStructure' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_StatisticalClassificationRelationStructure$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_StatisticalClassificationRelationStructure$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_StatisticalClassificationRelationStructure' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_StatisticalClassificationRelationStructure$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMemberRelation}}{ A read/write active binding.
#'        Member relations that comprise the relation structure
#'
#'                              \code{myDDI4_StatisticalClassificationRelationStructure$hasMemberRelation <- list(myStatisticalClassificationRelation)} sets the 'hasMemberRelation' property of 'myDDI4_StatisticalClassificationRelationStructure' to a list of R6 StatisticalClassificationRelation class instances
#'
#'                              \code{myDDI4_StatisticalClassificationRelationStructure$hasMemberRelation} returns a list of of R6 : StatisticalClassificationRelation class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_StatisticalClassificationRelationStructure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_StatisticalClassificationRelationStructure <- R6Class(classname="DDI4_StatisticalClassificationRelationStructure",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_StatisticalClassificationRelationStructure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasMemberRelation)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_StatisticalClassificationRelation", proposedObjectList=hasMemberRelation , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    hasMemberRelation = function(val){
      if (!missing(val)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_StatisticalClassificationRelation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasMemberRelation
      }
    },

    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .hasMemberRelation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasMemberRelation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_StatisticalClassificationRelation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasRelationSpecification = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_StatisticalClassificationRelationStructure


#' @title DDI4_StructuredCollection  Structured Collection container extends a Simple Collection allowing it to optionally have an RelationshpStructure to model hierarchies and nesting or more complex network structures
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/StructuredCollection}{StructuredCollection} 

#'
#'   Definition
#'
#'   ============
#'
#'   Structured Collection container extends a Simple Collection allowing it to optionally have an RelationshpStructure to model hierarchies and nesting or more complex network structures. Like Simple Collection it is also a subtype of Member to allow for nested collections. A Collection can be described directly as having unordered members or an indexed array of members through a set of MemberIndicators. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Node Sets, Schemes, Groups, Concept Systems are all types of Collections.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Members have to belong to some Collection, except in the case of nested Collections where the top level Collection is a Member that doesn't belong to any Collection.
#'
#'   If a Collection is ordered as a simple list that ordering can be indicated by an index property of each MemberIndicator. If the Collection is unordered, the index is not necessary.
#'
#'   Collection is not extended, it is realized (see Collection Pattern documentation).
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Node Set
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_SimpleCollection}}}
#'    \item{\code{\link{DDI4_CollectionMember}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isStructuredBy}}{ A read/write active binding.
#'          A relation structure that provides complex organization of the collection
#'
#'                              \code{myDDI4_StructuredCollection$isStructuredBy <- c(myRelationStructureURN)} sets the association 'isStructuredBy' from 'myDDI4_StructuredCollection' to a character vector of DdiUrns for R6 RelationStructure class instances
#'
#'                              \code{myDDI4_StructuredCollection$isStructuredBy} returns a character vector of of DdiUrns for R6  RelationStructure class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_StructuredCollection$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_StructuredCollection <- R6Class(classname="DDI4_StructuredCollection",
  inherit = DDI4_SimpleCollection,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, isStructuredBy=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_StructuredCollection may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)



       if(!missing(isStructuredBy)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_RelationStructure", proposedUrnVector=isStructuredBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 class association(s):
    isStructuredBy = function(val){
      if (!missing(val)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_RelationStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
       # invisible( private$.isStructuredBy)
      } else {
          private$.isStructuredBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .isStructuredBy = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_MemberIndicator", 
          isAbstractDatatype = TRUE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isStructuredBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_RelationStructure", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_StructuredCollection


#' @title DDI4_Study  Study is the conceptual anchor for an individual collection activity resulting in a data set or data file
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Study}{Study} 

#'
#'   Definition
#'
#'   ============
#'
#'   Study is the conceptual anchor for an individual collection activity resulting in a data set or data file. It can be both a part of a StudySeries as a wave etc. or an unrelated activity like a single survey project.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   ICPSR study 35575 Americans and the Arts [1973 - 1992] (ICPSR 35575). https://www.icpsr.umich.edu/icpsrweb/ICPSR/studies/35575?dataFormat%5B0%5D=SAS&keyword%5B0%5D=public+opinion&geography%5B0%5D=United+States&searchSource=revise  
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   The Study class brings together many properties and relationships describing a set of data – coverage, kind of data, methodology, citation information, access information and more.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   s:StudyUnitType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Study$DdiUrn} returns the DDI URN of myDDI4_Study.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{kindOfData}}{ A read/write active binding.
#'        Describes, with a string or a term from a controlled vocabulary, the kind of data documented in the logical product(s) of a study unit. Examples include survey data, census/enumeration data, administrative data, measurement data, assessment data, demographic data, voting data, etc.
#'
#'                              \code{myDDI4_Study$kindOfData <- list(myExternalControlledVocabularyEntry)} sets the 'kindOfData' property of 'myDDI4_Study' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_Study$kindOfData} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{overview}}{ A read/write active binding.
#'        A general descriptive overview of the Study
#'
#'                              \code{myDDI4_Study$overview <- list(myInternationalStructuredString)} sets the 'overview' property of 'myDDI4_Study' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Study$overview} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{bibliographicCitation}}{ A read/write active binding.
#'        Complete bibliographic reference containing all of the standard elements of a citation that can be used to cite the work. The "format" attribute is provided to enable specification of the particular citation style used, e.g., APA, MLA, Chicago, etc.
#'
#'                              \code{myDDI4_Study$bibliographicCitation <- list(myInternationalStructuredString)} sets the 'bibliographicCitation' property of 'myDDI4_Study' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Study$bibliographicCitation} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{partOfSeries}}{ A read/write active binding.
#'          A series in which this Study takes part.
#'
#'                              \code{myDDI4_Study$partOfSeries <- c(myStudySeriesURN)} sets the association 'partOfSeries' from 'myDDI4_Study' to a character vector of DdiUrns for R6 StudySeries class instances
#'
#'                              \code{myDDI4_Study$partOfSeries} returns a character vector of of DdiUrns for R6  StudySeries class instances}
#'   }
#'   \describe{
#'     \item{\code{hasCoverage}}{ A read/write active binding.
#'          Spatial, Temporal, and Topical coverage information for the Study
#'
#'                              \code{myDDI4_Study$hasCoverage <- c(myCoverageURN)} sets the association 'hasCoverage' from 'myDDI4_Study' to a character vector of DdiUrns for R6 Coverage class instances
#'
#'                              \code{myDDI4_Study$hasCoverage} returns a character vector of of DdiUrns for R6  Coverage class instances}
#'   }
#'   \describe{
#'     \item{\code{hasAnalysisUnit}}{ A read/write active binding.
#'          Unit(s) of analysis for the study, e.g. persons, households.
#'
#'                              \code{myDDI4_Study$hasAnalysisUnit <- c(myUnitTypeURN)} sets the association 'hasAnalysisUnit' from 'myDDI4_Study' to a character vector of DdiUrns for R6 UnitType class instances
#'
#'                              \code{myDDI4_Study$hasAnalysisUnit} returns a character vector of of DdiUrns for R6  UnitType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasPopulation}}{ A read/write active binding.
#'          The set of specific Units studied. Examples: 1. Canadian adult persons residing in Canada on 13 November 1956 2. US computer companies at the end of 2012 3. Universities in Denmark 1 January 2011.
#'
#'                              \code{myDDI4_Study$hasPopulation <- c(myPopulationURN)} sets the association 'hasPopulation' from 'myDDI4_Study' to a character vector of DdiUrns for R6 Population class instances
#'
#'                              \code{myDDI4_Study$hasPopulation} returns a character vector of of DdiUrns for R6  Population class instances}
#'   }
#'   \describe{
#'     \item{\code{hasUniverse}}{ A read/write active binding.
#'          The defined classes of Units related to the data,  Examples: 1. Canadian adults (not limited to those residing in Canada) 2. Computer companies 3. Universities
#'
#'                              \code{myDDI4_Study$hasUniverse <- c(myUniverseURN)} sets the association 'hasUniverse' from 'myDDI4_Study' to a character vector of DdiUrns for R6 Universe class instances
#'
#'                              \code{myDDI4_Study$hasUniverse} returns a character vector of of DdiUrns for R6  Universe class instances}
#'   }
#'   \describe{
#'     \item{\code{hasDesign}}{ A read/write active binding.
#'          The design for the Study
#'
#'                              \code{myDDI4_Study$hasDesign <- c(myDesignOverviewURN)} sets the association 'hasDesign' from 'myDDI4_Study' to a character vector of DdiUrns for R6 DesignOverview class instances
#'
#'                              \code{myDDI4_Study$hasDesign} returns a character vector of of DdiUrns for R6  DesignOverview class instances}
#'   }
#'   \describe{
#'     \item{\code{hasProcess}}{ A read/write active binding.
#'          Overview of the Process employed in the Study
#'
#'                              \code{myDDI4_Study$hasProcess <- c(myProcessOverviewURN)} sets the association 'hasProcess' from 'myDDI4_Study' to a character vector of DdiUrns for R6 ProcessOverview class instances
#'
#'                              \code{myDDI4_Study$hasProcess} returns a character vector of of DdiUrns for R6  ProcessOverview class instances}
#'   }
#'   \describe{
#'     \item{\code{hasInstrument}}{ A read/write active binding.
#'          The implemented instruments used in capturing the data. The ImplentedInstrument designates the mode of delivery, a relation to an external representation of the instrument, and reference to the conceptual instrument that contains more detailed information on the content of the instrument.
#'
#'                              \code{myDDI4_Study$hasInstrument <- c(myImplementedInstrumentURN)} sets the association 'hasInstrument' from 'myDDI4_Study' to a character vector of DdiUrns for R6 ImplementedInstrument class instances
#'
#'                              \code{myDDI4_Study$hasInstrument} returns a character vector of of DdiUrns for R6  ImplementedInstrument class instances}
#'   }
#'   \describe{
#'     \item{\code{hasEmbargo}}{ A read/write active binding.
#'          Information about data that are not currently available because of policies established by the principal investigators and/or data producers.
#'
#'                              \code{myDDI4_Study$hasEmbargo <- c(myEmbargoURN)} sets the association 'hasEmbargo' from 'myDDI4_Study' to a character vector of DdiUrns for R6 Embargo class instances
#'
#'                              \code{myDDI4_Study$hasEmbargo} returns a character vector of of DdiUrns for R6  Embargo class instances}
#'   }
#'   \describe{
#'     \item{\code{accessInformation}}{ A read/write active binding.
#'          Information relating to the availability of data from the study.
#'
#'                              \code{myDDI4_Study$accessInformation <- c(myAccessURN)} sets the association 'accessInformation' from 'myDDI4_Study' to a character vector of DdiUrns for R6 Access class instances
#'
#'                              \code{myDDI4_Study$accessInformation} returns a character vector of of DdiUrns for R6  Access class instances}
#'   }
#'   \describe{
#'     \item{\code{hasFundingInformation}}{ A read/write active binding.
#'          Information about the individual, agency and/or grant(s) which funded the described entity. 
#'
#'                              \code{myDDI4_Study$hasFundingInformation <- c(myFundingInformationURN)} sets the association 'hasFundingInformation' from 'myDDI4_Study' to a character vector of DdiUrns for R6 FundingInformation class instances
#'
#'                              \code{myDDI4_Study$hasFundingInformation} returns a character vector of of DdiUrns for R6  FundingInformation class instances}
#'   }
#'   \describe{
#'     \item{\code{hasAuthorizationSource}}{ A read/write active binding.
#'          The authorizing agency and allows for the full text of the authorization (law, regulation, or other form of authorization).
#'
#'                              \code{myDDI4_Study$hasAuthorizationSource <- c(myAuthorizationSourceURN)} sets the association 'hasAuthorizationSource' from 'myDDI4_Study' to a character vector of DdiUrns for R6 AuthorizationSource class instances
#'
#'                              \code{myDDI4_Study$hasAuthorizationSource} returns a character vector of of DdiUrns for R6  AuthorizationSource class instances}
#'   }
#'   \describe{
#'     \item{\code{hasQualityStatement}}{ A read/write active binding.
#'          A statement of quality which may be related to an external standard or contain a simple statement of internal quality goals or expectations.
#'
#'                              \code{myDDI4_Study$hasQualityStatement <- c(myQualityStatementURN)} sets the association 'hasQualityStatement' from 'myDDI4_Study' to a character vector of DdiUrns for R6 QualityStatement class instances
#'
#'                              \code{myDDI4_Study$hasQualityStatement} returns a character vector of of DdiUrns for R6  QualityStatement class instances}
#'   }
#'   \describe{
#'     \item{\code{hasBudget}}{ A read/write active binding.
#'          A description of the budget for the Study
#'
#'                              \code{myDDI4_Study$hasBudget <- c(myBudgetURN)} sets the association 'hasBudget' from 'myDDI4_Study' to a character vector of DdiUrns for R6 Budget class instances
#'
#'                              \code{myDDI4_Study$hasBudget} returns a character vector of of DdiUrns for R6  Budget class instances}
#'   }
#'   \describe{
#'     \item{\code{hasInstanceVariable}}{ A read/write active binding.
#'          A variable describing actual instances of data as recorded in the study. An example might be self reported gender, with categories of male encoded as "0"  and female encoded as "1", with missing values recorded as  "-9" for refused and "-8" as "not asked", recorded (physically) as an integer.
#'
#'                              \code{myDDI4_Study$hasInstanceVariable <- c(myInstanceVariableURN)} sets the association 'hasInstanceVariable' from 'myDDI4_Study' to a character vector of DdiUrns for R6 InstanceVariable class instances
#'
#'                              \code{myDDI4_Study$hasInstanceVariable} returns a character vector of of DdiUrns for R6  InstanceVariable class instances}
#'   }
#'   \describe{
#'     \item{\code{hasExPostEvaluation}}{ A read/write active binding.
#'          Evaluation for the purpose of reviewing the study, data collection, data processing, or management processes.
#'
#'                              \code{myDDI4_Study$hasExPostEvaluation <- c(myExPostEvaluationURN)} sets the association 'hasExPostEvaluation' from 'myDDI4_Study' to a character vector of DdiUrns for R6 ExPostEvaluation class instances
#'
#'                              \code{myDDI4_Study$hasExPostEvaluation} returns a character vector of of DdiUrns for R6  ExPostEvaluation class instances}
#'   }
#'   \describe{
#'     \item{\code{hasSamplingProcedure}}{ A read/write active binding.
#'          The methodologies for sampling employed by the study.
#'
#'                              \code{myDDI4_Study$hasSamplingProcedure <- c(mySamplingProcedureURN)} sets the association 'hasSamplingProcedure' from 'myDDI4_Study' to a character vector of DdiUrns for R6 SamplingProcedure class instances
#'
#'                              \code{myDDI4_Study$hasSamplingProcedure} returns a character vector of of DdiUrns for R6  SamplingProcedure class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMethodology}}{ A read/write active binding.
#'          A relationship to a description of a simple overview, in which case the MethodologyOverView class should be the target. 
#'
#'                              \code{myDDI4_Study$hasMethodology <- c(myMethodologyOverviewURN)} sets the association 'hasMethodology' from 'myDDI4_Study' to a character vector of DdiUrns for R6 MethodologyOverview class instances
#'
#'                              \code{myDDI4_Study$hasMethodology} returns a character vector of of DdiUrns for R6  MethodologyOverview class instances}
#'   }
#'   \describe{
#'     \item{\code{hasAlgorithm}}{ A read/write active binding.
#'          A relationship to a description of a simple overview, in which case the AlgorithmOverView class should be the target. 
#'
#'                              \code{myDDI4_Study$hasAlgorithm <- c(myAlgorithmOverviewURN)} sets the association 'hasAlgorithm' from 'myDDI4_Study' to a character vector of DdiUrns for R6 AlgorithmOverview class instances
#'
#'                              \code{myDDI4_Study$hasAlgorithm} returns a character vector of of DdiUrns for R6  AlgorithmOverview class instances}
#'   }
#'   \describe{
#'     \item{\code{hasConcept}}{ A read/write active binding.
#'          A Study may relate to multiple concepts. These may also be denoted by keywords in an Annotation.
#'
#'                              \code{myDDI4_Study$hasConcept <- c(myConceptURN)} sets the association 'hasConcept' from 'myDDI4_Study' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_Study$hasConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#'   \describe{
#'     \item{\code{hasVariableCollection}}{ A read/write active binding.
#'          A Study may relate to a multiple collections of variables which may be defined by type, subject, use, source, etc.
#'
#'                              \code{myDDI4_Study$hasVariableCollection <- c(myVariableCollectionURN)} sets the association 'hasVariableCollection' from 'myDDI4_Study' to a character vector of DdiUrns for R6 VariableCollection class instances
#'
#'                              \code{myDDI4_Study$hasVariableCollection} returns a character vector of of DdiUrns for R6  VariableCollection class instances}
#'   }
#'   \describe{
#'     \item{\code{hasAgentListing}}{ A read/write active binding.
#'          Makes use of the Agent Listing to define relationships between agents.
#'
#'                              \code{myDDI4_Study$hasAgentListing <- c(myAgentListingURN)} sets the association 'hasAgentListing' from 'myDDI4_Study' to a character vector of DdiUrns for R6 AgentListing class instances
#'
#'                              \code{myDDI4_Study$hasAgentListing} returns a character vector of of DdiUrns for R6  AgentListing class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Study$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, kindOfData=NA, overview=NA, bibliographicCitation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Study <- R6Class(classname="DDI4_Study",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, kindOfData=NA, overview=NA, bibliographicCitation=NA,    partOfSeries=NA,hasCoverage=NA,hasAnalysisUnit=NA,hasPopulation=NA,hasUniverse=NA,hasDesign=NA,hasProcess=NA,hasInstrument=NA,hasEmbargo=NA,accessInformation=NA,hasFundingInformation=NA,hasAuthorizationSource=NA,hasQualityStatement=NA,hasBudget=NA,hasInstanceVariable=NA,hasExPostEvaluation=NA,hasSamplingProcedure=NA,hasMethodology=NA,hasAlgorithm=NA,hasConcept=NA,hasVariableCollection=NA,hasAgentListing=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Study may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(kindOfData)){
        private$.kindOfData <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=kindOfData , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(overview)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=overview , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(bibliographicCitation)){
        private$.bibliographicCitation <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=bibliographicCitation , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(partOfSeries)){
        private$.partOfSeries <- returnUrnVectorIfValid(validClass="DDI4_StudySeries", proposedUrnVector=partOfSeries , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.partOfSeries, referencingAssociationName="partOfSeries",  referencingUrn=thisUrn)  
      }
      if(!missing(hasCoverage)){
        private$.hasCoverage <- returnUrnVectorIfValid(validClass="DDI4_Coverage", proposedUrnVector=hasCoverage , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasCoverage, referencingAssociationName="hasCoverage",  referencingUrn=thisUrn)  
      }
      if(!missing(hasAnalysisUnit)){
        private$.hasAnalysisUnit <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=hasAnalysisUnit , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasAnalysisUnit, referencingAssociationName="hasAnalysisUnit",  referencingUrn=thisUrn)  
      }
      if(!missing(hasPopulation)){
        private$.hasPopulation <- returnUrnVectorIfValid(validClass="DDI4_Population", proposedUrnVector=hasPopulation , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasPopulation, referencingAssociationName="hasPopulation",  referencingUrn=thisUrn)  
      }
      if(!missing(hasUniverse)){
        private$.hasUniverse <- returnUrnVectorIfValid(validClass="DDI4_Universe", proposedUrnVector=hasUniverse , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasUniverse, referencingAssociationName="hasUniverse",  referencingUrn=thisUrn)  
      }
      if(!missing(hasDesign)){
        private$.hasDesign <- returnUrnVectorIfValid(validClass="DDI4_DesignOverview", proposedUrnVector=hasDesign , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasDesign, referencingAssociationName="hasDesign",  referencingUrn=thisUrn)  
      }
      if(!missing(hasProcess)){
        private$.hasProcess <- returnUrnVectorIfValid(validClass="DDI4_ProcessOverview", proposedUrnVector=hasProcess , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasProcess, referencingAssociationName="hasProcess",  referencingUrn=thisUrn)  
      }
      if(!missing(hasInstrument)){
        private$.hasInstrument <- returnUrnVectorIfValid(validClass="DDI4_ImplementedInstrument", proposedUrnVector=hasInstrument , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasInstrument, referencingAssociationName="hasInstrument",  referencingUrn=thisUrn)  
      }
      if(!missing(hasEmbargo)){
        private$.hasEmbargo <- returnUrnVectorIfValid(validClass="DDI4_Embargo", proposedUrnVector=hasEmbargo , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasEmbargo, referencingAssociationName="hasEmbargo",  referencingUrn=thisUrn)  
      }
      if(!missing(accessInformation)){
        private$.accessInformation <- returnUrnVectorIfValid(validClass="DDI4_Access", proposedUrnVector=accessInformation , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.accessInformation, referencingAssociationName="accessInformation",  referencingUrn=thisUrn)  
      }
      if(!missing(hasFundingInformation)){
        private$.hasFundingInformation <- returnUrnVectorIfValid(validClass="DDI4_FundingInformation", proposedUrnVector=hasFundingInformation , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasFundingInformation, referencingAssociationName="hasFundingInformation",  referencingUrn=thisUrn)  
      }
      if(!missing(hasAuthorizationSource)){
        private$.hasAuthorizationSource <- returnUrnVectorIfValid(validClass="DDI4_AuthorizationSource", proposedUrnVector=hasAuthorizationSource , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasAuthorizationSource, referencingAssociationName="hasAuthorizationSource",  referencingUrn=thisUrn)  
      }
      if(!missing(hasQualityStatement)){
        private$.hasQualityStatement <- returnUrnVectorIfValid(validClass="DDI4_QualityStatement", proposedUrnVector=hasQualityStatement , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasQualityStatement, referencingAssociationName="hasQualityStatement",  referencingUrn=thisUrn)  
      }
      if(!missing(hasBudget)){
        private$.hasBudget <- returnUrnVectorIfValid(validClass="DDI4_Budget", proposedUrnVector=hasBudget , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasBudget, referencingAssociationName="hasBudget",  referencingUrn=thisUrn)  
      }
      if(!missing(hasInstanceVariable)){
        private$.hasInstanceVariable <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=hasInstanceVariable , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasInstanceVariable, referencingAssociationName="hasInstanceVariable",  referencingUrn=thisUrn)  
      }
      if(!missing(hasExPostEvaluation)){
        private$.hasExPostEvaluation <- returnUrnVectorIfValid(validClass="DDI4_ExPostEvaluation", proposedUrnVector=hasExPostEvaluation , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasExPostEvaluation, referencingAssociationName="hasExPostEvaluation",  referencingUrn=thisUrn)  
      }
      if(!missing(hasSamplingProcedure)){
        private$.hasSamplingProcedure <- returnUrnVectorIfValid(validClass="DDI4_SamplingProcedure", proposedUrnVector=hasSamplingProcedure , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasSamplingProcedure, referencingAssociationName="hasSamplingProcedure",  referencingUrn=thisUrn)  
      }
      if(!missing(hasMethodology)){
        private$.hasMethodology <- returnUrnVectorIfValid(validClass="DDI4_MethodologyOverview", proposedUrnVector=hasMethodology , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasMethodology, referencingAssociationName="hasMethodology",  referencingUrn=thisUrn)  
      }
      if(!missing(hasAlgorithm)){
        private$.hasAlgorithm <- returnUrnVectorIfValid(validClass="DDI4_AlgorithmOverview", proposedUrnVector=hasAlgorithm , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasAlgorithm, referencingAssociationName="hasAlgorithm",  referencingUrn=thisUrn)  
      }
      if(!missing(hasConcept)){
        private$.hasConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=hasConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasConcept, referencingAssociationName="hasConcept",  referencingUrn=thisUrn)  
      }
      if(!missing(hasVariableCollection)){
        private$.hasVariableCollection <- returnUrnVectorIfValid(validClass="DDI4_VariableCollection", proposedUrnVector=hasVariableCollection , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasVariableCollection, referencingAssociationName="hasVariableCollection",  referencingUrn=thisUrn)  
      }
      if(!missing(hasAgentListing)){
        private$.hasAgentListing <- returnUrnVectorIfValid(validClass="DDI4_AgentListing", proposedUrnVector=hasAgentListing , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasAgentListing, referencingAssociationName="hasAgentListing",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    bibliographicCitation = function(val){
      if (!missing(val)){
        private$.bibliographicCitation <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.bibliographicCitation
      }
    },

    kindOfData = function(val){
      if (!missing(val)){
        private$.kindOfData <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.kindOfData
      }
    },

    overview = function(val){
      if (!missing(val)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overview
      }
    },


# There are 22 class association(s):
    accessInformation = function(val){
      if (!missing(val)){
        private$.accessInformation <- returnUrnVectorIfValid(validClass="DDI4_Access", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.accessInformation, referencingAssociationName="accessInformation",  referencingUrn=thisUrn)  
       # invisible( private$.accessInformation)
      } else {
          private$.accessInformation
      }
    },

    hasAgentListing = function(val){
      if (!missing(val)){
        private$.hasAgentListing <- returnUrnVectorIfValid(validClass="DDI4_AgentListing", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasAgentListing, referencingAssociationName="hasAgentListing",  referencingUrn=thisUrn)  
       # invisible( private$.hasAgentListing)
      } else {
          private$.hasAgentListing
      }
    },

    hasAlgorithm = function(val){
      if (!missing(val)){
        private$.hasAlgorithm <- returnUrnVectorIfValid(validClass="DDI4_AlgorithmOverview", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasAlgorithm, referencingAssociationName="hasAlgorithm",  referencingUrn=thisUrn)  
       # invisible( private$.hasAlgorithm)
      } else {
          private$.hasAlgorithm
      }
    },

    hasAnalysisUnit = function(val){
      if (!missing(val)){
        private$.hasAnalysisUnit <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasAnalysisUnit, referencingAssociationName="hasAnalysisUnit",  referencingUrn=thisUrn)  
       # invisible( private$.hasAnalysisUnit)
      } else {
          private$.hasAnalysisUnit
      }
    },

    hasAuthorizationSource = function(val){
      if (!missing(val)){
        private$.hasAuthorizationSource <- returnUrnVectorIfValid(validClass="DDI4_AuthorizationSource", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasAuthorizationSource, referencingAssociationName="hasAuthorizationSource",  referencingUrn=thisUrn)  
       # invisible( private$.hasAuthorizationSource)
      } else {
          private$.hasAuthorizationSource
      }
    },

    hasBudget = function(val){
      if (!missing(val)){
        private$.hasBudget <- returnUrnVectorIfValid(validClass="DDI4_Budget", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasBudget, referencingAssociationName="hasBudget",  referencingUrn=thisUrn)  
       # invisible( private$.hasBudget)
      } else {
          private$.hasBudget
      }
    },

    hasConcept = function(val){
      if (!missing(val)){
        private$.hasConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasConcept, referencingAssociationName="hasConcept",  referencingUrn=thisUrn)  
       # invisible( private$.hasConcept)
      } else {
          private$.hasConcept
      }
    },

    hasCoverage = function(val){
      if (!missing(val)){
        private$.hasCoverage <- returnUrnVectorIfValid(validClass="DDI4_Coverage", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasCoverage, referencingAssociationName="hasCoverage",  referencingUrn=thisUrn)  
       # invisible( private$.hasCoverage)
      } else {
          private$.hasCoverage
      }
    },

    hasDesign = function(val){
      if (!missing(val)){
        private$.hasDesign <- returnUrnVectorIfValid(validClass="DDI4_DesignOverview", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasDesign, referencingAssociationName="hasDesign",  referencingUrn=thisUrn)  
       # invisible( private$.hasDesign)
      } else {
          private$.hasDesign
      }
    },

    hasEmbargo = function(val){
      if (!missing(val)){
        private$.hasEmbargo <- returnUrnVectorIfValid(validClass="DDI4_Embargo", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasEmbargo, referencingAssociationName="hasEmbargo",  referencingUrn=thisUrn)  
       # invisible( private$.hasEmbargo)
      } else {
          private$.hasEmbargo
      }
    },

    hasExPostEvaluation = function(val){
      if (!missing(val)){
        private$.hasExPostEvaluation <- returnUrnVectorIfValid(validClass="DDI4_ExPostEvaluation", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasExPostEvaluation, referencingAssociationName="hasExPostEvaluation",  referencingUrn=thisUrn)  
       # invisible( private$.hasExPostEvaluation)
      } else {
          private$.hasExPostEvaluation
      }
    },

    hasFundingInformation = function(val){
      if (!missing(val)){
        private$.hasFundingInformation <- returnUrnVectorIfValid(validClass="DDI4_FundingInformation", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasFundingInformation, referencingAssociationName="hasFundingInformation",  referencingUrn=thisUrn)  
       # invisible( private$.hasFundingInformation)
      } else {
          private$.hasFundingInformation
      }
    },

    hasInstanceVariable = function(val){
      if (!missing(val)){
        private$.hasInstanceVariable <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasInstanceVariable, referencingAssociationName="hasInstanceVariable",  referencingUrn=thisUrn)  
       # invisible( private$.hasInstanceVariable)
      } else {
          private$.hasInstanceVariable
      }
    },

    hasInstrument = function(val){
      if (!missing(val)){
        private$.hasInstrument <- returnUrnVectorIfValid(validClass="DDI4_ImplementedInstrument", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasInstrument, referencingAssociationName="hasInstrument",  referencingUrn=thisUrn)  
       # invisible( private$.hasInstrument)
      } else {
          private$.hasInstrument
      }
    },

    hasMethodology = function(val){
      if (!missing(val)){
        private$.hasMethodology <- returnUrnVectorIfValid(validClass="DDI4_MethodologyOverview", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasMethodology, referencingAssociationName="hasMethodology",  referencingUrn=thisUrn)  
       # invisible( private$.hasMethodology)
      } else {
          private$.hasMethodology
      }
    },

    hasPopulation = function(val){
      if (!missing(val)){
        private$.hasPopulation <- returnUrnVectorIfValid(validClass="DDI4_Population", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasPopulation, referencingAssociationName="hasPopulation",  referencingUrn=thisUrn)  
       # invisible( private$.hasPopulation)
      } else {
          private$.hasPopulation
      }
    },

    hasProcess = function(val){
      if (!missing(val)){
        private$.hasProcess <- returnUrnVectorIfValid(validClass="DDI4_ProcessOverview", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasProcess, referencingAssociationName="hasProcess",  referencingUrn=thisUrn)  
       # invisible( private$.hasProcess)
      } else {
          private$.hasProcess
      }
    },

    hasQualityStatement = function(val){
      if (!missing(val)){
        private$.hasQualityStatement <- returnUrnVectorIfValid(validClass="DDI4_QualityStatement", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasQualityStatement, referencingAssociationName="hasQualityStatement",  referencingUrn=thisUrn)  
       # invisible( private$.hasQualityStatement)
      } else {
          private$.hasQualityStatement
      }
    },

    hasSamplingProcedure = function(val){
      if (!missing(val)){
        private$.hasSamplingProcedure <- returnUrnVectorIfValid(validClass="DDI4_SamplingProcedure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasSamplingProcedure, referencingAssociationName="hasSamplingProcedure",  referencingUrn=thisUrn)  
       # invisible( private$.hasSamplingProcedure)
      } else {
          private$.hasSamplingProcedure
      }
    },

    hasUniverse = function(val){
      if (!missing(val)){
        private$.hasUniverse <- returnUrnVectorIfValid(validClass="DDI4_Universe", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasUniverse, referencingAssociationName="hasUniverse",  referencingUrn=thisUrn)  
       # invisible( private$.hasUniverse)
      } else {
          private$.hasUniverse
      }
    },

    hasVariableCollection = function(val){
      if (!missing(val)){
        private$.hasVariableCollection <- returnUrnVectorIfValid(validClass="DDI4_VariableCollection", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasVariableCollection, referencingAssociationName="hasVariableCollection",  referencingUrn=thisUrn)  
       # invisible( private$.hasVariableCollection)
      } else {
          private$.hasVariableCollection
      }
    },

    partOfSeries = function(val){
      if (!missing(val)){
        private$.partOfSeries <- returnUrnVectorIfValid(validClass="DDI4_StudySeries", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.partOfSeries, referencingAssociationName="partOfSeries",  referencingUrn=thisUrn)  
       # invisible( private$.partOfSeries)
      } else {
          private$.partOfSeries
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .kindOfData = NULL,
    .overview = NULL,
    .bibliographicCitation = NULL,
    .partOfSeries = NULL,
    .hasCoverage = NULL,
    .hasAnalysisUnit = NULL,
    .hasPopulation = NULL,
    .hasUniverse = NULL,
    .hasDesign = NULL,
    .hasProcess = NULL,
    .hasInstrument = NULL,
    .hasEmbargo = NULL,
    .accessInformation = NULL,
    .hasFundingInformation = NULL,
    .hasAuthorizationSource = NULL,
    .hasQualityStatement = NULL,
    .hasBudget = NULL,
    .hasInstanceVariable = NULL,
    .hasExPostEvaluation = NULL,
    .hasSamplingProcedure = NULL,
    .hasMethodology = NULL,
    .hasAlgorithm = NULL,
    .hasConcept = NULL,
    .hasVariableCollection = NULL,
    .hasAgentListing = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        accessInformation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Access", 
          structure = "DdiUrn vector"
        ),
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        bibliographicCitation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAgentListing = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AgentListing", 
          structure = "DdiUrn vector"
        ),
        hasAlgorithm = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AlgorithmOverview", 
          structure = "DdiUrn vector"
        ),
        hasAnalysisUnit = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_UnitType", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAuthorizationSource = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AuthorizationSource", 
          structure = "DdiUrn vector"
        ),
        hasBudget = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Budget", 
          structure = "DdiUrn vector"
        ),
        hasConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasCoverage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Coverage", 
          structure = "DdiUrn vector"
        ),
        hasDesign = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DesignOverview", 
          structure = "DdiUrn vector"
        ),
        hasEmbargo = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Embargo", 
          structure = "DdiUrn vector"
        ),
        hasExPostEvaluation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExPostEvaluation", 
          structure = "DdiUrn vector"
        ),
        hasFundingInformation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_FundingInformation", 
          structure = "DdiUrn vector"
        ),
        hasInstanceVariable = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InstanceVariable", 
          structure = "DdiUrn vector"
        ),
        hasInstrument = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ImplementedInstrument", 
          structure = "DdiUrn vector"
        ),
        hasMethodology = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_MethodologyOverview", 
          structure = "DdiUrn vector"
        ),
        hasPopulation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Population", 
          structure = "DdiUrn vector"
        ),
        hasProcess = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ProcessOverview", 
          structure = "DdiUrn vector"
        ),
        hasQualityStatement = c(
          min="0", 
          max="1", 
          datatype = "DDI4_QualityStatement", 
          structure = "DdiUrn vector"
        ),
        hasSamplingProcedure = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_SamplingProcedure", 
          structure = "DdiUrn vector"
        ),
        hasUniverse = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Universe", 
          structure = "DdiUrn vector"
        ),
        hasVariableCollection = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_VariableCollection", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        kindOfData = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        partOfSeries = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_StudySeries", 
          structure = "DdiUrn vector"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Study


#' @title DDI4_StudyControl  StudyControl references a Study
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/StudyControl}{StudyControl} 

#'
#'   Definition
#'
#'   ============
#'
#'   StudyControl references a Study. StudyControl enables the representation of a research protocol at the Study level. [If the only purpose of the object is to carry a Study, it should be seriously considered for melting with another object. Modeling rules state that any object needs to have a reality of its own. Carrying another object is not a reality. If I misunderstood, please clarify the description instead]
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Act}}}
#'    \item{\code{\link{DDI4_WorkflowStep}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasStudy}}{ A read/write active binding.
#'          Study that is the target of the StudyControl
#'
#'                              \code{myDDI4_StudyControl$hasStudy <- c(myStudyURN)} sets the association 'hasStudy' from 'myDDI4_StudyControl' to a character vector of DdiUrns for R6 Study class instances
#'
#'                              \code{myDDI4_StudyControl$hasStudy} returns a character vector of of DdiUrns for R6  Study class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_StudyControl$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_StudyControl <- R6Class(classname="DDI4_StudyControl",
  inherit = DDI4_Act,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasStudy=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_StudyControl may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)



       if(!missing(hasStudy)){
        private$.hasStudy <- returnUrnVectorIfValid(validClass="DDI4_Study", proposedUrnVector=hasStudy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasStudy, referencingAssociationName="hasStudy",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 class association(s):
    hasStudy = function(val){
      if (!missing(val)){
        private$.hasStudy <- returnUrnVectorIfValid(validClass="DDI4_Study", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasStudy, referencingAssociationName="hasStudy",  referencingUrn=thisUrn)  
       # invisible( private$.hasStudy)
      } else {
          private$.hasStudy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasStudy = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInformationFlow = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasStudy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Study", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPerformedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowService", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_StudyControl


#' @title DDI4_StudyIndicator  Member Indicator for use with member type Study
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/StudyIndicator}{StudyIndicator} 

#'
#'   Definition
#'
#'   ============
#'
#'   Member Indicator for use with member type Study
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_StudyIndicator$DdiUrn} returns the DDI URN of myDDI4_StudyIndicator.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Index value of member in an ordered array
#'
#'                              \code{myDDI4_StudyIndicator$index <- myinteger} sets the index property of 'myDDI4_StudyIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_StudyIndicator$index} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{member}}{ A read/write active binding.
#'          Restricts member target class to Study
#'
#'                              \code{myDDI4_StudyIndicator$member <- c(myStudyURN)} sets the association 'member' from 'myDDI4_StudyIndicator' to a character vector of DdiUrns for R6 Study class instances
#'
#'                              \code{myDDI4_StudyIndicator$member} returns a character vector of of DdiUrns for R6  Study class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_StudyIndicator$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_StudyIndicator <- R6Class(classname="DDI4_StudyIndicator",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA,    member=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_StudyIndicator may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(member)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_Study", proposedUrnVector=member , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.index
      }
    },


# There are 1 class association(s):
    member = function(val){
      if (!missing(val)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_Study", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
       # invisible( private$.member)
      } else {
          private$.member
      }
    }

  ),

  private = list(
    .index = NULL,
    .member = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        index = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        member = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Study", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_StudyIndicator


#' @title DDI4_StudyRelation  Relation specification between source and target studies in a relation structure
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/StudyRelation}{StudyRelation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Relation specification between source and target studies in a relation structure
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_StudyRelation$DdiUrn} returns the DDI URN of myDDI4_StudyRelation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an enumeration list
#'
#'                              \code{myDDI4_StudyRelation$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_StudyRelation' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_StudyRelation$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship 
#'
#'                              \code{myDDI4_StudyRelation$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_StudyRelation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_StudyRelation$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_StudyRelation$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_StudyRelation' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_StudyRelation$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          First member in the relationship
#'
#'                              \code{myDDI4_StudyRelation$source <- c(myStudyURN)} sets the association 'source' from 'myDDI4_StudyRelation' to a character vector of DdiUrns for R6 Study class instances
#'
#'                              \code{myDDI4_StudyRelation$source} returns a character vector of of DdiUrns for R6  Study class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          Second member in a relationship. Note that this can be realized as a collection to support tuples.
#'
#'                              \code{myDDI4_StudyRelation$target <- c(myStudyURN)} sets the association 'target' from 'myDDI4_StudyRelation' to a character vector of DdiUrns for R6 Study class instances
#'
#'                              \code{myDDI4_StudyRelation$target} returns a character vector of of DdiUrns for R6  Study class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_StudyRelation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_StudyRelation <- R6Class(classname="DDI4_StudyRelation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_StudyRelation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_Study", proposedUrnVector=source , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_Study", proposedUrnVector=target , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_Study", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_Study", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Study", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Study", 
          structure = "DdiUrn vector"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_StudyRelation


#' @title DDI4_StudyRelationStructure  Allows for a complex nested structure for a Study Series
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/StudyRelationStructure}{StudyRelationStructure} 

#'
#'   Definition
#'
#'   ============
#'
#'   Allows for a complex nested structure for a Study Series
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_StudyRelationStructure$DdiUrn} returns the DDI URN of myDDI4_StudyRelationStructure.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an enumeration list.  Use if all relations within this relation structure are of the same specification.
#'
#'                              \code{myDDI4_StudyRelationStructure$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_StudyRelationStructure' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_StudyRelationStructure$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_StudyRelationStructure$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_StudyRelationStructure' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_StudyRelationStructure$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_StudyRelationStructure$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_StudyRelationStructure' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_StudyRelationStructure$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMemberRelation}}{ A read/write active binding.
#'        Member relations that comprise the relation structure
#'
#'                              \code{myDDI4_StudyRelationStructure$hasMemberRelation <- list(myStudyRelation)} sets the 'hasMemberRelation' property of 'myDDI4_StudyRelationStructure' to a list of R6 StudyRelation class instances
#'
#'                              \code{myDDI4_StudyRelationStructure$hasMemberRelation} returns a list of of R6 : StudyRelation class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_StudyRelationStructure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_StudyRelationStructure <- R6Class(classname="DDI4_StudyRelationStructure",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_StudyRelationStructure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasMemberRelation)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_StudyRelation", proposedObjectList=hasMemberRelation , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    hasMemberRelation = function(val){
      if (!missing(val)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_StudyRelation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasMemberRelation
      }
    },

    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .hasMemberRelation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasMemberRelation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_StudyRelation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasRelationSpecification = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_StudyRelationStructure


#' @title DDI4_StudySeries  A collection of studies which can be structured as a simple sequence, or with a more complex structure
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/StudySeries}{StudySeries} 

#'
#'   Definition
#'
#'   ============
#'
#'   A collection of studies which can be structured as a simple sequence, or with a more complex structure.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   An annual series of surveys.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   A set of studies may be defined in many ways. A study may be repeated over time. One or more studies may attempt to replicate an earlier study. The StudySeries allows for the description of the relationships among a set of studies.
#'
#'   
#'
#'   A simple ordered or unordered sequence of studies can be described via the "contains StudyIndicator" property. More complex relationships among studies may also be described using the optional "isStructuredBy StudyRelationsStructure".
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_StudySeries$DdiUrn} returns the DDI URN of myDDI4_StudySeries.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        The name of the series as a whole
#'
#'                              \code{myDDI4_StudySeries$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_StudySeries' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_StudySeries$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{overview}}{ A read/write active binding.
#'        A general descriptive overview of the series.
#'
#'                              \code{myDDI4_StudySeries$overview <- list(myInternationalStructuredString)} sets the 'overview' property of 'myDDI4_StudySeries' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_StudySeries$overview} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{type}}{ A read/write active binding.
#'        Whether the collection is a bag or a set: a bag is a collection with duplicates allowed, a set is a collection without duplicates.
#'
#'                              \code{myDDI4_StudySeries$type <- list(myCollectionType)} sets the 'type' property of 'myDDI4_StudySeries' to a list of R6 CollectionType class instances
#'
#'                              \code{myDDI4_StudySeries$type} returns a list of of R6 : CollectionType class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_StudySeries$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_StudySeries' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_StudySeries$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        Allows for the identification of the member and optionally provides an index for the member within an ordered array
#'
#'                              \code{myDDI4_StudySeries$contains <- list(myStudyIndicator)} sets the 'contains' property of 'myDDI4_StudySeries' to a list of R6 StudyIndicator class instances
#'
#'                              \code{myDDI4_StudySeries$contains} returns a list of of R6 : StudyIndicator class instances}
#'   }
#'   \describe{
#'     \item{\code{isOrdered}}{ A read/write active binding.
#'        If members are ordered set to true, if unordered set to false.
#'
#'                              \code{myDDI4_StudySeries$isOrdered <- mylogical} sets the isOrdered property of 'myDDI4_StudySeries' to the vector of datatype:logical
#'
#'                              \code{myDDI4_StudySeries$isOrdered} returns a vector of datatype: logical}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isStructuredBy}}{ A read/write active binding.
#'          Description of a complex structure for the Collection. A collection may be structured in more than one way for different uses
#'
#'                              \code{myDDI4_StudySeries$isStructuredBy <- c(myStudyRelationStructureURN)} sets the association 'isStructuredBy' from 'myDDI4_StudySeries' to a character vector of DdiUrns for R6 StudyRelationStructure class instances
#'
#'                              \code{myDDI4_StudySeries$isStructuredBy} returns a character vector of of DdiUrns for R6  StudyRelationStructure class instances}
#'   }
#'   \describe{
#'     \item{\code{definingConcept}}{ A read/write active binding.
#'          The conceptual basis for the collection of members.
#'
#'                              \code{myDDI4_StudySeries$definingConcept <- c(myConceptURN)} sets the association 'definingConcept' from 'myDDI4_StudySeries' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_StudySeries$definingConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_StudySeries$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, name=NA, overview=NA, type=NA, purpose=NA, contains=NA, isOrdered=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_StudySeries <- R6Class(classname="DDI4_StudySeries",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, name=NA, overview=NA, type=NA, purpose=NA, contains=NA, isOrdered=NA,    isStructuredBy=NA,definingConcept=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_StudySeries may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(overview)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=overview , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(type)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=type , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_StudyIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(isOrdered)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=isOrdered , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(isStructuredBy)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_StudyRelationStructure", proposedUrnVector=isStructuredBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
      }
      if(!missing(definingConcept)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=definingConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 6 properties: 
    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_StudyIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },

    isOrdered = function(val){
      if (!missing(val)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isOrdered
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    overview = function(val){
      if (!missing(val)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overview
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    type = function(val){
      if (!missing(val)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.type
      }
    },


# There are 2 class association(s):
    definingConcept = function(val){
      if (!missing(val)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
       # invisible( private$.definingConcept)
      } else {
          private$.definingConcept
      }
    },

    isStructuredBy = function(val){
      if (!missing(val)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_StudyRelationStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
       # invisible( private$.isStructuredBy)
      } else {
          private$.isStructuredBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .name = NULL,
    .overview = NULL,
    .type = NULL,
    .purpose = NULL,
    .contains = NULL,
    .isOrdered = NULL,
    .isStructuredBy = NULL,
    .definingConcept = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_StudyIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isStructuredBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_StudyRelationStructure", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_StudySeries


#' @title DDI4_SubstantiveConceptualDomain  Set of valid Concepts
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/SubstantiveConceptualDomain}{SubstantiveConceptualDomain} 

#'
#'   Definition
#'
#'   ============
#'
#'   Set of valid Concepts. The Concepts can be described by either enumeration or by an expression.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   An enumeration of concepts for a categorical variable like "male" and "female" for gender.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   A ConceptualVariable links a UnitType to a SubstantiveConceptualDomain. The latter can be an enumeration or description of the values (Signified) that the variable may take on. In the enumerated case these are the categories in a CategorySet that can be values, not the codes that represent the values. 
#'
#'   An example might be the conceptual domain for a variable representing self-identified gender. An enumeration might include the concept of “male” and the concept of “female”. These, in turn, would be represented in a SubstantiveValueDomain by codes in a CodeList like “m” and “f”, or “0” and “1”. 
#'
#'   A conceptual domain might be described through a ValueAndConceptDescription’s description property of “a real number greater than 0” or through a more formal logicalExpression of “(all reals x such that x >0) “. Even in the described case, what is being described are conceptual, not the symbols used to represent the values. This may be a subtle distinction, but allows specifying that the same numeric value might be represented by 32 bits or by 64 bits or by an Arabic numeral or a Roman numeral.
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Conceptual Domain
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ConceptualDomain}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_SubstantiveConceptualDomain$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_SubstantiveConceptualDomain <- R6Class(classname="DDI4_SubstantiveConceptualDomain",
  inherit = DDI4_ConceptualDomain,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_SubstantiveConceptualDomain may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

    }
  ),

  active = list(
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        describedConceptualDomain = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ValueAndConceptDescription", 
          structure = "DdiUrn vector"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        enumeratedConceptualDomain = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ConceptSystem", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_SubstantiveConceptualDomain


#' @title DDI4_SubstantiveValueDomain  The Value Domain for a substantive conceptual domain
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/SubstantiveValueDomain}{SubstantiveValueDomain} 

#'
#'   Definition
#'
#'   ============
#'
#'   The Value Domain for a substantive conceptual domain. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   All real decimal numbers relating to the subject matter of interest between 0 and 1 specified in Arabic numerals. [GSIM 1.1].
#'
#'   The codes "M" male and "F" for female .
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   In DDI4 the value domain for a variable is separated into “substantive” and “sentinel” values. Substantive values are the values of primary interest. Sentinel values are additional values that may carry supplementary information, such as reasons for missing. This duality is described in ISO 11404. Substantive values for height might be real numbers expressed in meters. The full value domain might also include codes for different kinds of missing values - one code for “refused” and another for “don’t know”. Sentinel Values may also convey some substantive information and at the same time represent missing values. An example might be where a numeric variable like number of employees be sometimes a count and sometimes a code representing a range of counts in order to avoid disclosure of information about a specific entity. 
#'
#'   
#'
#'   The SubstantiveValueDomain may use either a ValueDescription, for described values,  or a CodeList for enumerated values, or both. 
#'
#'   
#'
#'   A value domain may consist of substantive values or sentinel values. Substantive values are those associated directly with some subject matter area. They do not address concerns around processing, such as missing values. Substantive values are called "regular values" in ISO/IEC 11404 - General purpose datatypes.
#'
#'   
#'
#'   The enumerated case is one where all values are listed. An example is the categorical values for gender:
#'
#'   The conceptual domain could consist of the concept of male and the concept of female. These might be represented in codes and associated labels as
#'
#'   1 (="Male")
#'
#'   2 (="Female")
#'
#'   
#'
#'   The described case is one where some description is needed to define the set of values.
#'
#'   Take the following description for height in meters:
#'
#'   "a real number between 0 and 3, represented to two Arabic decimal places". This description might be structured in some way to be machine actionable (datatype="double", min=0, max=3, decimals=2).
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   value domain [ISO11179]
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ValueDomain}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{takesConceptsFrom}}{ A read/write active binding.
#'          Corresponding conceptual definition given by an SubstantiveConceptualDomain.
#'
#'                              \code{myDDI4_SubstantiveValueDomain$takesConceptsFrom <- c(mySubstantiveConceptualDomainURN)} sets the association 'takesConceptsFrom' from 'myDDI4_SubstantiveValueDomain' to a character vector of DdiUrns for R6 SubstantiveConceptualDomain class instances
#'
#'                              \code{myDDI4_SubstantiveValueDomain$takesConceptsFrom} returns a character vector of of DdiUrns for R6  SubstantiveConceptualDomain class instances}
#'   }
#'   \describe{
#'     \item{\code{describedValueDomain}}{ A read/write active binding.
#'          A formal description of the set of valid values - for described value domains.
#'
#'                              \code{myDDI4_SubstantiveValueDomain$describedValueDomain <- c(myValueAndConceptDescriptionURN)} sets the association 'describedValueDomain' from 'myDDI4_SubstantiveValueDomain' to a character vector of DdiUrns for R6 ValueAndConceptDescription class instances
#'
#'                              \code{myDDI4_SubstantiveValueDomain$describedValueDomain} returns a character vector of of DdiUrns for R6  ValueAndConceptDescription class instances}
#'   }
#'   \describe{
#'     \item{\code{enumeratedValueDomain}}{ A read/write active binding.
#'          Any subtype of an enumeration domain enumerating the set of valid values.
#'
#'                              \code{myDDI4_SubstantiveValueDomain$enumeratedValueDomain <- c(myEnumerationDomainURN)} sets the association 'enumeratedValueDomain' from 'myDDI4_SubstantiveValueDomain' to a character vector of DdiUrns for R6 EnumerationDomain class instances
#'
#'                              \code{myDDI4_SubstantiveValueDomain$enumeratedValueDomain} returns a character vector of of DdiUrns for R6  EnumerationDomain class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_SubstantiveValueDomain$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_SubstantiveValueDomain <- R6Class(classname="DDI4_SubstantiveValueDomain",
  inherit = DDI4_ValueDomain,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, takesConceptsFrom=NA,describedValueDomain=NA,enumeratedValueDomain=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_SubstantiveValueDomain may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)



       if(!missing(takesConceptsFrom)){
        private$.takesConceptsFrom <- returnUrnVectorIfValid(validClass="DDI4_SubstantiveConceptualDomain", proposedUrnVector=takesConceptsFrom , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.takesConceptsFrom, referencingAssociationName="takesConceptsFrom",  referencingUrn=thisUrn)  
      }
      if(!missing(describedValueDomain)){
        private$.describedValueDomain <- returnUrnVectorIfValid(validClass="DDI4_ValueAndConceptDescription", proposedUrnVector=describedValueDomain , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.describedValueDomain, referencingAssociationName="describedValueDomain",  referencingUrn=thisUrn)  
      }
      if(!missing(enumeratedValueDomain)){
        private$.enumeratedValueDomain <- returnUrnVectorIfValid(validClass="DDI4_EnumerationDomain", proposedUrnVector=enumeratedValueDomain , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.enumeratedValueDomain, referencingAssociationName="enumeratedValueDomain",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 class association(s):
    describedValueDomain = function(val){
      if (!missing(val)){
        private$.describedValueDomain <- returnUrnVectorIfValid(validClass="DDI4_ValueAndConceptDescription", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.describedValueDomain, referencingAssociationName="describedValueDomain",  referencingUrn=thisUrn)  
       # invisible( private$.describedValueDomain)
      } else {
          private$.describedValueDomain
      }
    },

    enumeratedValueDomain = function(val){
      if (!missing(val)){
        private$.enumeratedValueDomain <- returnUrnVectorIfValid(validClass="DDI4_EnumerationDomain", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.enumeratedValueDomain, referencingAssociationName="enumeratedValueDomain",  referencingUrn=thisUrn)  
       # invisible( private$.enumeratedValueDomain)
      } else {
          private$.enumeratedValueDomain
      }
    },

    takesConceptsFrom = function(val){
      if (!missing(val)){
        private$.takesConceptsFrom <- returnUrnVectorIfValid(validClass="DDI4_SubstantiveConceptualDomain", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.takesConceptsFrom, referencingAssociationName="takesConceptsFrom",  referencingUrn=thisUrn)  
       # invisible( private$.takesConceptsFrom)
      } else {
          private$.takesConceptsFrom
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .takesConceptsFrom = NULL,
    .describedValueDomain = NULL,
    .enumeratedValueDomain = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        describedValueDomain = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ValueAndConceptDescription", 
          structure = "DdiUrn vector"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        enumeratedValueDomain = c(
          min="0", 
          max="1", 
          datatype = "DDI4_EnumerationDomain", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        recommendedDataType = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        takesConceptsFrom = c(
          min="0", 
          max="1", 
          datatype = "DDI4_SubstantiveConceptualDomain", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_SubstantiveValueDomain


#' @title DDI4_SummaryStatistic  Describes summary statistics for a variable
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/SummaryStatistic}{SummaryStatistic} 

#'
#'   Definition
#'
#'   ============
#'
#'   Describes summary statistics for a variable.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Mean, standard deviation
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   pi:SummaryStatisticType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_SummaryStatistic$DdiUrn} returns the DDI URN of myDDI4_SummaryStatistic.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{typeOfSummaryStatistic}}{ A read/write active binding.
#'        Type of summary statistic, such as count, mean, mode, median, etc. Supports the use of an external controlled vocabulary. DDI strongly recommends the use of a controlled vocabulary.
#'
#'                              \code{myDDI4_SummaryStatistic$typeOfSummaryStatistic <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfSummaryStatistic' property of 'myDDI4_SummaryStatistic' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_SummaryStatistic$typeOfSummaryStatistic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{hasStatistic}}{ A read/write active binding.
#'        The value of the statistics and whether it is weighted and/or includes missing values. May expressed as xs:decimal and/or xs:double
#'
#'                              \code{myDDI4_SummaryStatistic$hasStatistic <- list(myStatistic)} sets the 'hasStatistic' property of 'myDDI4_SummaryStatistic' to a list of R6 Statistic class instances
#'
#'                              \code{myDDI4_SummaryStatistic$hasStatistic} returns a list of of R6 : Statistic class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_SummaryStatistic$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, typeOfSummaryStatistic=NA, hasStatistic=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_SummaryStatistic <- R6Class(classname="DDI4_SummaryStatistic",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, typeOfSummaryStatistic=NA, hasStatistic=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_SummaryStatistic may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(typeOfSummaryStatistic)){
        private$.typeOfSummaryStatistic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfSummaryStatistic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasStatistic)){
        private$.hasStatistic <- returnR6ObjectIfValid(validClass="DDI4_Statistic", proposedObjectList=hasStatistic , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    hasStatistic = function(val){
      if (!missing(val)){
        private$.hasStatistic <- returnR6ObjectIfValid(validClass="DDI4_Statistic", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasStatistic
      }
    },

    typeOfSummaryStatistic = function(val){
      if (!missing(val)){
        private$.typeOfSummaryStatistic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.typeOfSummaryStatistic
      }
    }


  ),

  private = list(
    .typeOfSummaryStatistic = NULL,
    .hasStatistic = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasStatistic = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Statistic", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        typeOfSummaryStatistic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_SummaryStatistic


#' @title DDI4_TargetSample  Specifies details of target sample size, percentage, type, and universe
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/TargetSample}{TargetSample} 

#'
#'   Definition
#'
#'   ============
#'
#'   Specifies details of target sample size, percentage, type, and universe
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_TargetSample$DdiUrn} returns the DDI URN of myDDI4_TargetSample.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{isPrimary}}{ A read/write active binding.
#'        Default value is True. The information is for the primary sample unit or universe. Change to False if the information relates to a secondary or sub- sample universe.
#'
#'                              \code{myDDI4_TargetSample$isPrimary <- mylogical} sets the isPrimary property of 'myDDI4_TargetSample' to the vector of datatype:logical
#'
#'                              \code{myDDI4_TargetSample$isPrimary} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{targetSize}}{ A read/write active binding.
#'        The target size of the sample expressed as an integer.
#'
#'                              \code{myDDI4_TargetSample$targetSize <- myinteger} sets the targetSize property of 'myDDI4_TargetSample' to the vector of datatype:integer
#'
#'                              \code{myDDI4_TargetSample$targetSize} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{targetPercent}}{ A read/write active binding.
#'        The target size of the sample expressed as an integer (70% expressed as .7)
#'
#'                              \code{myDDI4_TargetSample$targetPercent <- mynumeric} sets the targetPercent property of 'myDDI4_TargetSample' to the vector of datatype:numeric
#'
#'                              \code{myDDI4_TargetSample$targetPercent} returns a vector of datatype: numeric}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasUnitType}}{ A read/write active binding.
#'          Unit type of the sample
#'
#'                              \code{myDDI4_TargetSample$hasUnitType <- c(myUnitTypeURN)} sets the association 'hasUnitType' from 'myDDI4_TargetSample' to a character vector of DdiUrns for R6 UnitType class instances
#'
#'                              \code{myDDI4_TargetSample$hasUnitType} returns a character vector of of DdiUrns for R6  UnitType class instances}
#'   }
#'   \describe{
#'     \item{\code{fromUniverse}}{ A read/write active binding.
#'          The universe of the sample. Note that when the Design is implemented the Population should be a Population within this Universe.
#'
#'                              \code{myDDI4_TargetSample$fromUniverse <- c(myUniverseURN)} sets the association 'fromUniverse' from 'myDDI4_TargetSample' to a character vector of DdiUrns for R6 Universe class instances
#'
#'                              \code{myDDI4_TargetSample$fromUniverse} returns a character vector of of DdiUrns for R6  Universe class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_TargetSample$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, isPrimary=NA, targetSize=NA, targetPercent=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_TargetSample <- R6Class(classname="DDI4_TargetSample",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, isPrimary=NA, targetSize=NA, targetPercent=NA,    hasUnitType=NA,fromUniverse=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_TargetSample may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(isPrimary)){
        private$.isPrimary <- returnVectorIfValid(validVectorClass="logical", proposedVector=isPrimary , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(targetSize)){
        private$.targetSize <- returnVectorIfValid(validVectorClass="integer", proposedVector=targetSize , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(targetPercent)){
        private$.targetPercent <- returnVectorIfValid(validVectorClass="numeric", proposedVector=targetPercent , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(hasUnitType)){
        private$.hasUnitType <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=hasUnitType , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasUnitType, referencingAssociationName="hasUnitType",  referencingUrn=thisUrn)  
      }
      if(!missing(fromUniverse)){
        private$.fromUniverse <- returnUrnVectorIfValid(validClass="DDI4_Universe", proposedUrnVector=fromUniverse , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.fromUniverse, referencingAssociationName="fromUniverse",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    isPrimary = function(val){
      if (!missing(val)){
        private$.isPrimary <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.isPrimary
      }
    },

    targetPercent = function(val){
      if (!missing(val)){
        private$.targetPercent <- returnVectorIfValid(validVectorClass="numeric", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.targetPercent
      }
    },

    targetSize = function(val){
      if (!missing(val)){
        private$.targetSize <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.targetSize
      }
    },


# There are 2 class association(s):
    fromUniverse = function(val){
      if (!missing(val)){
        private$.fromUniverse <- returnUrnVectorIfValid(validClass="DDI4_Universe", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.fromUniverse, referencingAssociationName="fromUniverse",  referencingUrn=thisUrn)  
       # invisible( private$.fromUniverse)
      } else {
          private$.fromUniverse
      }
    },

    hasUnitType = function(val){
      if (!missing(val)){
        private$.hasUnitType <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasUnitType, referencingAssociationName="hasUnitType",  referencingUrn=thisUrn)  
       # invisible( private$.hasUnitType)
      } else {
          private$.hasUnitType
      }
    }

  ),

  private = list(
    .isPrimary = NULL,
    .targetSize = NULL,
    .targetPercent = NULL,
    .hasUnitType = NULL,
    .fromUniverse = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        fromUniverse = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Universe", 
          structure = "DdiUrn vector"
        ),
        hasUnitType = c(
          min="0", 
          max="1", 
          datatype = "DDI4_UnitType", 
          structure = "DdiUrn vector"
        ),
        isPrimary = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        targetPercent = c(
          min="0", 
          max="1", 
          datatype = "numeric", 
          structure = "vector"
        ),
        targetSize = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_TargetSample


#' @title DDI4_Telephone  Details of a telephone number including the number, type of number, a privacy setting and an indication of whether this is the preferred contact number
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Telephone}{Telephone} 

#'
#'   Definition
#'
#'   ============
#'
#'   Details of a telephone number including the number, type of number, a privacy setting and an indication of whether this is the preferred contact number.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   +12 345 67890123
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   a:TelephoneType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Telephone$DdiUrn} returns the DDI URN of myDDI4_Telephone.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{telephoneNumber}}{ A read/write active binding.
#'        The telephone number including country code if appropriate.
#'
#'                              \code{myDDI4_Telephone$telephoneNumber <- mycharacter} sets the telephoneNumber property of 'myDDI4_Telephone' to the vector of datatype:character
#'
#'                              \code{myDDI4_Telephone$telephoneNumber} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{typeOfTelephone}}{ A read/write active binding.
#'        Indicates type of telephone number provided (home, fax, office, cell, etc.). Supports the use of a controlled vocabulary.
#'
#'                              \code{myDDI4_Telephone$typeOfTelephone <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfTelephone' property of 'myDDI4_Telephone' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_Telephone$typeOfTelephone} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{effectiveDates}}{ A read/write active binding.
#'        Time period during which the telephone number is valid.
#'
#'                              \code{myDDI4_Telephone$effectiveDates <- list(myDateRange)} sets the 'effectiveDates' property of 'myDDI4_Telephone' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_Telephone$effectiveDates} returns a list of of R6 : DateRange class instances}
#'   }
#'   \describe{
#'     \item{\code{privacy}}{ A read/write active binding.
#'        Specify the level privacy for the telephone number as public, restricted, or private. Supports the use of an external controlled vocabulary.
#'
#'                              \code{myDDI4_Telephone$privacy <- list(myExternalControlledVocabularyEntry)} sets the 'privacy' property of 'myDDI4_Telephone' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_Telephone$privacy} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{isPreferred}}{ A read/write active binding.
#'        Set to "true" if this is the preferred telephone number for contact.
#'
#'                              \code{myDDI4_Telephone$isPreferred <- mylogical} sets the isPreferred property of 'myDDI4_Telephone' to the vector of datatype:logical
#'
#'                              \code{myDDI4_Telephone$isPreferred} returns a vector of datatype: logical}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Telephone$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, telephoneNumber=NA, typeOfTelephone=NA, effectiveDates=NA, privacy=NA, isPreferred=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Telephone <- R6Class(classname="DDI4_Telephone",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, telephoneNumber=NA, typeOfTelephone=NA, effectiveDates=NA, privacy=NA, isPreferred=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Telephone may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(telephoneNumber)){
        private$.telephoneNumber <- returnVectorIfValid(validVectorClass="character", proposedVector=telephoneNumber , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(typeOfTelephone)){
        private$.typeOfTelephone <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfTelephone , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(effectiveDates)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=effectiveDates , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(privacy)){
        private$.privacy <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=privacy , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isPreferred)){
        private$.isPreferred <- returnVectorIfValid(validVectorClass="logical", proposedVector=isPreferred , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 5 properties: 
    effectiveDates = function(val){
      if (!missing(val)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.effectiveDates
      }
    },

    isPreferred = function(val){
      if (!missing(val)){
        private$.isPreferred <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isPreferred
      }
    },

    privacy = function(val){
      if (!missing(val)){
        private$.privacy <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.privacy
      }
    },

    telephoneNumber = function(val){
      if (!missing(val)){
        private$.telephoneNumber <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.telephoneNumber
      }
    },

    typeOfTelephone = function(val){
      if (!missing(val)){
        private$.typeOfTelephone <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.typeOfTelephone
      }
    }


  ),

  private = list(
    .telephoneNumber = NULL,
    .typeOfTelephone = NULL,
    .effectiveDates = NULL,
    .privacy = NULL,
    .isPreferred = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        effectiveDates = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        isPreferred = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        privacy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        telephoneNumber = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        typeOfTelephone = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Telephone


#' @title DDI4_TemporalCoverage  Describes the temporal coverage of the annotated object by specifying dates and optionally associating them with Subject and Keyword
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/TemporalCoverage}{TemporalCoverage} 

#'
#'   Definition
#'
#'   ============
#'
#'   Describes the temporal coverage of the annotated object by specifying dates and optionally associating them with Subject and Keyword. The date itself may be expressed as either and ISO and/or NonISO date. Dates may be catagorized by use of optional typeOfDate using and external controlled vocabulary entry.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   The association of subjects or keywords with the date periods allows for describing the meaning of the temporal period. A survey, for example might be administered in one time period, but ask questions about the preceding decade. Temporal coverage might then include the data collection coverage as well as the referenced time period.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_TemporalCoverage$DdiUrn} returns the DDI URN of myDDI4_TemporalCoverage.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{coverageDate}}{ A read/write active binding.
#'        A date referencing a specific aspect of temporal coverage. The date may be typed to reflect coverage date, collection date, referent date, etc. Subject and Keywords may be associated with the date to specify a specific set of topical information (i.e. Residence associated with a date 5 years prior to the collection date).
#'
#'                              \code{myDDI4_TemporalCoverage$coverageDate <- list(myReferenceDate)} sets the 'coverageDate' property of 'myDDI4_TemporalCoverage' to a list of R6 ReferenceDate class instances
#'
#'                              \code{myDDI4_TemporalCoverage$coverageDate} returns a list of of R6 : ReferenceDate class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_TemporalCoverage$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, coverageDate=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_TemporalCoverage <- R6Class(classname="DDI4_TemporalCoverage",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, coverageDate=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_TemporalCoverage may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(coverageDate)){
        private$.coverageDate <- returnR6ObjectIfValid(validClass="DDI4_ReferenceDate", proposedObjectList=coverageDate , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 1 properties: 
    coverageDate = function(val){
      if (!missing(val)){
        private$.coverageDate <- returnR6ObjectIfValid(validClass="DDI4_ReferenceDate", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.coverageDate
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .coverageDate = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        coverageDate = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ReferenceDate", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_TemporalCoverage


#' @title DDI4_TemporalRelationControlStep  Defines complex synchronous or asynchronous temporal ordering control
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/TemporalRelationControlStep}{TemporalRelationControlStep} 

#'
#'   Definition
#'
#'   ============
#'
#'   Defines complex synchronous or asynchronous temporal ordering control. Defines temporal relationships using Allen's Intervals.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   When creating a specific subtype identify the workflow steps included in a structure that is appropriate to the subtype. For example, Split contains a relationship "executeConcurrently".
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_WorkflowControlStep}}}
#'    \item{\code{\link{DDI4_WorkflowStep}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_TemporalRelationControlStep$DdiUrn} returns the DDI URN of myDDI4_TemporalRelationControlStep.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{typeOfTemporalRelationship}}{ A read/write active binding.
#'        Specify the type of temporal relationhship between identified WorkflowSteps using Allen's Interval definitions.
#'
#'                              \code{myDDI4_TemporalRelationControlStep$typeOfTemporalRelationship <- list(myTemporalRelationSpecification)} sets the 'typeOfTemporalRelationship' property of 'myDDI4_TemporalRelationControlStep' to a list of R6 TemporalRelationSpecification class instances
#'
#'                              \code{myDDI4_TemporalRelationControlStep$typeOfTemporalRelationship} returns a list of of R6 : TemporalRelationSpecification class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_TemporalRelationControlStep$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, typeOfTemporalRelationship=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_TemporalRelationControlStep <- R6Class(classname="DDI4_TemporalRelationControlStep",
  inherit = DDI4_WorkflowControlStep,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, typeOfTemporalRelationship=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_TemporalRelationControlStep may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(typeOfTemporalRelationship)){
        private$.typeOfTemporalRelationship <- returnEnumeratedValues(enumerationObject=DDI4_TemporalRelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=typeOfTemporalRelationship , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 1 properties: 
    typeOfTemporalRelationship = function(val){
      if (!missing(val)){
        private$.typeOfTemporalRelationship <- returnEnumeratedValues(enumerationObject=DDI4_TemporalRelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.typeOfTemporalRelationship
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .typeOfTemporalRelationship = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        executes = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_WorkflowStep", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInformationFlow = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPerformedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowService", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        typeOfTemporalRelationship = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TemporalRelationSpecification", 
          structure = "Enumeration vector"
        ),
        usage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_TemporalRelationControlStep


#' @title DDI4_TextResponseDomain  A response domain capturing a textual response including the length of the text and restriction of content using a regular expression
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/TextResponseDomain}{TextResponseDomain} 

#'
#'   Definition
#'
#'   ============
#'
#'   A response domain capturing a textual response including the length of the text and restriction of content using a regular expression.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Collecting the first name on an individual in an open ended text field
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:TextDomainType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_ResponseDomain}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_TextResponseDomain$DdiUrn} returns the DDI URN of myDDI4_TextResponseDomain.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{maximumLength}}{ A read/write active binding.
#'        The maximum number of characters allowed.
#'
#'                              \code{myDDI4_TextResponseDomain$maximumLength <- myinteger} sets the maximumLength property of 'myDDI4_TextResponseDomain' to the vector of datatype:integer
#'
#'                              \code{myDDI4_TextResponseDomain$maximumLength} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{minimumLength}}{ A read/write active binding.
#'        The minimum number of characters allowed.
#'
#'                              \code{myDDI4_TextResponseDomain$minimumLength <- myinteger} sets the minimumLength property of 'myDDI4_TextResponseDomain' to the vector of datatype:integer
#'
#'                              \code{myDDI4_TextResponseDomain$minimumLength} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{regularExpression}}{ A read/write active binding.
#'        A regular expression limiting the allowed characters or character order of the content. Use typeOfContent to specify the syntax of the regularExpression found in content.
#'
#'                              \code{myDDI4_TextResponseDomain$regularExpression <- list(myTypedString)} sets the 'regularExpression' property of 'myDDI4_TextResponseDomain' to a list of R6 TypedString class instances
#'
#'                              \code{myDDI4_TextResponseDomain$regularExpression} returns a list of of R6 : TypedString class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_TextResponseDomain$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, maximumLength=NA, minimumLength=NA, regularExpression=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_TextResponseDomain <- R6Class(classname="DDI4_TextResponseDomain",
  inherit = DDI4_ResponseDomain,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, maximumLength=NA, minimumLength=NA, regularExpression=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_TextResponseDomain may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(maximumLength)){
        private$.maximumLength <- returnVectorIfValid(validVectorClass="integer", proposedVector=maximumLength , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(minimumLength)){
        private$.minimumLength <- returnVectorIfValid(validVectorClass="integer", proposedVector=minimumLength , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(regularExpression)){
        private$.regularExpression <- returnR6ObjectIfValid(validClass="DDI4_TypedString", proposedObjectList=regularExpression , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    maximumLength = function(val){
      if (!missing(val)){
        private$.maximumLength <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.maximumLength
      }
    },

    minimumLength = function(val){
      if (!missing(val)){
        private$.minimumLength <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.minimumLength
      }
    },

    regularExpression = function(val){
      if (!missing(val)){
        private$.regularExpression <- returnR6ObjectIfValid(validClass="DDI4_TypedString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.regularExpression
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .maximumLength = NULL,
    .minimumLength = NULL,
    .regularExpression = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="1", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        intendedRepresentation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RepresentedVariable", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        maximumLength = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        minimumLength = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        regularExpression = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TypedString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_TextResponseDomain


#' @title DDI4_TextualSegment  Defines the segment of textual content used by the parent object
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/TextualSegment}{TextualSegment} 

#'
#'   Definition
#'
#'   ============
#'
#'   Defines the segment of textual content used by the parent object. Can identify a set of lines and or characters used to define the segment.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:TextualType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_TextualSegment$DdiUrn} returns the DDI URN of myDDI4_TextualSegment.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{lineParamenter}}{ A read/write active binding.
#'        Specification of the line and offset for the beginning and end of the segment.
#'
#'                              \code{myDDI4_TextualSegment$lineParamenter <- list(myLineParameter)} sets the 'lineParamenter' property of 'myDDI4_TextualSegment' to a list of R6 LineParameter class instances
#'
#'                              \code{myDDI4_TextualSegment$lineParamenter} returns a list of of R6 : LineParameter class instances}
#'   }
#'   \describe{
#'     \item{\code{characterParameter}}{ A read/write active binding.
#'        Specification of the character offset for the beginning and end of the segment.
#'
#'                              \code{myDDI4_TextualSegment$characterParameter <- list(myCharacterOffset)} sets the 'characterParameter' property of 'myDDI4_TextualSegment' to a list of R6 CharacterOffset class instances
#'
#'                              \code{myDDI4_TextualSegment$characterParameter} returns a list of of R6 : CharacterOffset class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_TextualSegment$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, lineParamenter=NA, characterParameter=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_TextualSegment <- R6Class(classname="DDI4_TextualSegment",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, lineParamenter=NA, characterParameter=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_TextualSegment may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(lineParamenter)){
        private$.lineParamenter <- returnR6ObjectIfValid(validClass="DDI4_LineParameter", proposedObjectList=lineParamenter , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(characterParameter)){
        private$.characterParameter <- returnR6ObjectIfValid(validClass="DDI4_CharacterOffset", proposedObjectList=characterParameter , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    characterParameter = function(val){
      if (!missing(val)){
        private$.characterParameter <- returnR6ObjectIfValid(validClass="DDI4_CharacterOffset", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.characterParameter
      }
    },

    lineParamenter = function(val){
      if (!missing(val)){
        private$.lineParamenter <- returnR6ObjectIfValid(validClass="DDI4_LineParameter", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.lineParamenter
      }
    }


  ),

  private = list(
    .lineParamenter = NULL,
    .characterParameter = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        characterParameter = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CharacterOffset", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        lineParamenter = c(
          min="0", 
          max="1", 
          datatype = "DDI4_LineParameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_TextualSegment


#' @title DDI4_TopicalCoverage  Describes the topical coverage of the module using Subject and Keyword
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/TopicalCoverage}{TopicalCoverage} 

#'
#'   Definition
#'
#'   ============
#'
#'   Describes the topical coverage of the module using Subject and Keyword. Subjects are members of structured classification systems such as formal subject headings in libraries. Keywords may be structured (e.g. TheSoz thesauri) or unstructured and reflect the terminology found in the document and other related (broader or similar) terms.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:TopicalCoverageType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_TopicalCoverage$DdiUrn} returns the DDI URN of myDDI4_TopicalCoverage.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{subject}}{ A read/write active binding.
#'        A subject that describes the topical coverage of the content of the annotated object. Subjects are members of structured classification systems such as formal subject headings in libraries. Uses and InternationalCodeValue and may indicate the language of the code used. 
#'
#'                              \code{myDDI4_TopicalCoverage$subject <- list(myExternalControlledVocabularyEntry)} sets the 'subject' property of 'myDDI4_TopicalCoverage' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_TopicalCoverage$subject} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{keyword}}{ A read/write active binding.
#'        A keyword that describes the topical coverage of the content of the annotated object.  Keywords may be structured (e.g. TheSoz thesauri) or unstructured and reflect the terminology found in the document and other related (broader or similar) terms.  Uses and InternationalCodeValue and may indicate the language of the code used.
#'
#'                              \code{myDDI4_TopicalCoverage$keyword <- list(myExternalControlledVocabularyEntry)} sets the 'keyword' property of 'myDDI4_TopicalCoverage' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_TopicalCoverage$keyword} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_TopicalCoverage$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, subject=NA, keyword=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_TopicalCoverage <- R6Class(classname="DDI4_TopicalCoverage",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, subject=NA, keyword=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_TopicalCoverage may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(subject)){
        private$.subject <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=subject , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(keyword)){
        private$.keyword <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=keyword , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    keyword = function(val){
      if (!missing(val)){
        private$.keyword <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.keyword
      }
    },

    subject = function(val){
      if (!missing(val)){
        private$.subject <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.subject
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .subject = NULL,
    .keyword = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        keyword = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        subject = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_TopicalCoverage


#' @title DDI4_TypedDescriptiveText  This Complex Data Type bundles a descriptiveText with an External Controlled Vocabulary Entry allowing structured content and a means of typing that content
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/TypedDescriptiveText}{TypedDescriptiveText} 

#'
#'   Definition
#'
#'   ============
#'
#'   This Complex Data Type bundles a descriptiveText with an External Controlled Vocabulary Entry allowing structured content and a means of typing that content. For example specifying that the description provides a Table of Contents for a document.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_TypedDescriptiveText$DdiUrn} returns the DDI URN of myDDI4_TypedDescriptiveText.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{typeOfContent}}{ A read/write active binding.
#'        Uses a controlled vocabulary entry to classify the description provided.
#'
#'                              \code{myDDI4_TypedDescriptiveText$typeOfContent <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfContent' property of 'myDDI4_TypedDescriptiveText' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_TypedDescriptiveText$typeOfContent} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{descriptiveText}}{ A read/write active binding.
#'        A short natural language account of the characteristics of the object.
#'
#'                              \code{myDDI4_TypedDescriptiveText$descriptiveText <- list(myInternationalStructuredString)} sets the 'descriptiveText' property of 'myDDI4_TypedDescriptiveText' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_TypedDescriptiveText$descriptiveText} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_TypedDescriptiveText$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, typeOfContent=NA, descriptiveText=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_TypedDescriptiveText <- R6Class(classname="DDI4_TypedDescriptiveText",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, typeOfContent=NA, descriptiveText=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_TypedDescriptiveText may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(typeOfContent)){
        private$.typeOfContent <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfContent , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(descriptiveText)){
        private$.descriptiveText <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=descriptiveText , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    descriptiveText = function(val){
      if (!missing(val)){
        private$.descriptiveText <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.descriptiveText
      }
    },

    typeOfContent = function(val){
      if (!missing(val)){
        private$.typeOfContent <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.typeOfContent
      }
    }


  ),

  private = list(
    .typeOfContent = NULL,
    .descriptiveText = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        descriptiveText = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        typeOfContent = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_TypedDescriptiveText


#' @title DDI4_TypedString  TypedString combines a typeOfContent with a content defined as a simple string
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/TypedString}{TypedString} 

#'
#'   Definition
#'
#'   ============
#'
#'   TypedString combines a typeOfContent with a content defined as a simple string. May be used wherever a simple string needs to support a type definition to clarify its content
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   content is a regular expression and typeOfContent is used to define the syntax used.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   This is a generic type + string where property name and documentation should be used to define any specification for the content. If international structured string content is required use TypedStructuredString 
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_TypedString$DdiUrn} returns the DDI URN of myDDI4_TypedString.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{typeOfContent}}{ A read/write active binding.
#'        Optional use of a controlled vocabulary to specifically type the associated content.
#'
#'                              \code{myDDI4_TypedString$typeOfContent <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfContent' property of 'myDDI4_TypedString' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_TypedString$typeOfContent} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{content}}{ A read/write active binding.
#'        Content of the property expressed as a simple string.
#'
#'                              \code{myDDI4_TypedString$content <- mycharacter} sets the content property of 'myDDI4_TypedString' to the vector of datatype:character
#'
#'                              \code{myDDI4_TypedString$content} returns a vector of datatype: character}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_TypedString$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, typeOfContent=NA, content=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_TypedString <- R6Class(classname="DDI4_TypedString",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, typeOfContent=NA, content=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_TypedString may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(typeOfContent)){
        private$.typeOfContent <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfContent , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(content)){
        private$.content <- returnVectorIfValid(validVectorClass="character", proposedVector=content , minimumCardinality=1, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    content = function(val){
      if (!missing(val)){
        private$.content <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.content
      }
    },

    typeOfContent = function(val){
      if (!missing(val)){
        private$.typeOfContent <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.typeOfContent
      }
    }


  ),

  private = list(
    .typeOfContent = NULL,
    .content = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        content = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        typeOfContent = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_TypedString


#' @title DDI4_Unit  The object of interest in a process step related to the collection or use of observational data
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Unit}{Unit} 

#'
#'   Definition
#'
#'   ============
#'
#'   The object of interest in a process step related to the collection or use of observational data.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Here are 3 examples - 1. Individual US person (i.e., Arofan Gregory, Dan Gillman, Barack Obama, etc.) 2. Individual US computer companies (i.e., Microsoft, Apple, IBM, etc.) 3. Individual US universities (i.e., Johns Hopkins, University of Maryland, Yale, etc.) [GSIM 1.1]
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   In a traditional data table each column might represent some variable (measurement). Each row would represent the entity (Unit)  to which those variables relate. Height measurements might be made on persons (UnitType) of primary school age (Universe) at Pinckney Elementary School on September 1, 2005 (Population). The height for Mary Roe (Unit)  might be 139 cm.
#'
#'   The Unit is not invariably tied to some value. A median income might be calculated for a block in the U.S. but then used as an attribute of a person residing on that block. For the initial measurement the Unit was the block. In the reuse the unit would be that specific person to which the value was applied.
#'
#'   In a big data table each row represents a unit/variable double. Together a unit identifier and a variable identifier define the key. And for each key there is just one value – the measure of the unit  on the variable. 
#'
#'   A big data table is sometimes referred to as a column-oriented data store whereas a traditional database is sometimes referred to as a row-oriented data store. The unit plays an identifier role in both types of data stores.
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Unit
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Unit$DdiUrn} returns the DDI URN of myDDI4_Unit.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A structured display label providing a fully human readable display for the identification of the object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_Unit$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_Unit' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_Unit$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage. 
#'
#'                              \code{myDDI4_Unit$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_Unit' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_Unit$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{definition}}{ A read/write active binding.
#'        Natural language statement conveying the meaning of a concept, differentiating it from other concepts. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_Unit$definition <- list(myInternationalStructuredString)} sets the 'definition' property of 'myDDI4_Unit' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Unit$definition} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasUnitType}}{ A read/write active binding.
#'          The UnitType of the Unit
#'
#'                              \code{myDDI4_Unit$hasUnitType <- c(myUnitTypeURN)} sets the association 'hasUnitType' from 'myDDI4_Unit' to a character vector of DdiUrns for R6 UnitType class instances
#'
#'                              \code{myDDI4_Unit$hasUnitType} returns a character vector of of DdiUrns for R6  UnitType class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Unit$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, displayLabel=NA, name=NA, definition=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Unit <- R6Class(classname="DDI4_Unit",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, displayLabel=NA, name=NA, definition=NA,    hasUnitType=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Unit may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(definition)){
        private$.definition <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=definition , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(hasUnitType)){
        private$.hasUnitType <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=hasUnitType , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasUnitType, referencingAssociationName="hasUnitType",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    definition = function(val){
      if (!missing(val)){
        private$.definition <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.definition
      }
    },

    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },


# There are 1 class association(s):
    hasUnitType = function(val){
      if (!missing(val)){
        private$.hasUnitType <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasUnitType, referencingAssociationName="hasUnitType",  referencingUrn=thisUrn)  
       # invisible( private$.hasUnitType)
      } else {
          private$.hasUnitType
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .displayLabel = NULL,
    .name = NULL,
    .definition = NULL,
    .hasUnitType = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definition = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasUnitType = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_UnitType", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Unit


#' @title DDI4_UnitDataRecord  Gives a UnitDataRecord structure to a Logical Record
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/UnitDataRecord}{UnitDataRecord} 

#'
#'   Definition
#'
#'   ============
#'
#'   Gives a UnitDataRecord structure to a Logical Record. UnitDataRecord structures a LogicalRecord as either a SimpleCollection or a StructuredCollection (InstanceVariableRelationStructure) of instance variables.
#'
#'   
#'
#'   The UnitDataRecord also organizes instance variables in a UnitDataViewpoint. The UnitDataViewpoint assigns roles to Instance Variables. In any one UnitDataViewpoint UnitDataRecord instance variables play one of three roles: identifier, measure or attribute. The same UnitDataRecord may have many UnitDataViewpoints.
#'
#'   
#'
#'   The UnitDataViewPoint can work in conjunction with the InstanceVariableRelationStructure. Together they can break out instance variables into complex (hierarchical) identifier, measure and attribute groups. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   The UnitDataRecord can be used to specify the structure of a table in a relational database. It can be used to specify the structure of both the fact and dimension tables in a data warehouse. It can be used to specify the structure of a big data table. Likewise it can be used to specify the structures of "column-based" and "row-based" tables.
#'
#'   
#'
#'   The UnitDataRecord may also form an information model if it is a StructuredCollection. Blood Pressure as described by an openEHR archetype is an information model. So is a collection of FHIR (HL7) resources that form an Electronic Health Record (EHR). 
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   The <a href="http://www.openehr.org/ckm/">openEHR archetype</a> is a use case that motivates the use of the InstanceVariableRelationStructure together with a UnitDataViewPoint. In the Blood Pressure archetype as well as most other archetypes attributes are not a SimpleCollection. Instead they are part of a StructuredCollection that breaks out attributes into "Protocol", "State" and "Events". In other archetypes the measure may be structured. For example body composition data includes a "base model" (fat mass, fat percentage, fat free mass), an "atomic level" (chemical elements, hydrogen, carbon, oxygen), a "molecular level" (minerals, protein, fat, water) and so forth.
#'
#'   
#'
#'   It is, however, possible to break out attributes into not one but many SimpleCollections in the event attributes and/or data do not form StructuredCollections. That is because the UnitDataViewpoint supports not just one but many attribute roles within the same viewpoint.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   Table Definition if the UnitDataRecord is a SimpleCollection, Information Model if the UnitDataRecord is a StructuredCollection
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_LogicalRecord}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isStructuredBy}}{ A read/write active binding.
#'          Describe the order of InstanceVariables in a record for those cases where the variables have a more complicated logical order than a simple sequence.
#'
#'                              \code{myDDI4_UnitDataRecord$isStructuredBy <- c(myInstanceVariableRelationStructureURN)} sets the association 'isStructuredBy' from 'myDDI4_UnitDataRecord' to a character vector of DdiUrns for R6 InstanceVariableRelationStructure class instances
#'
#'                              \code{myDDI4_UnitDataRecord$isStructuredBy} returns a character vector of of DdiUrns for R6  InstanceVariableRelationStructure class instances}
#'   }
#'   \describe{
#'     \item{\code{isViewedFrom}}{ A read/write active binding.
#'          A UnitDataRecord may have multiple ViewPoints, each of which assigns among roles identifier, measure, and attribute to the InstanceVariables associated with each Datapoint
#'
#'                              \code{myDDI4_UnitDataRecord$isViewedFrom <- c(myUnitDataViewpointURN)} sets the association 'isViewedFrom' from 'myDDI4_UnitDataRecord' to a character vector of DdiUrns for R6 UnitDataViewpoint class instances
#'
#'                              \code{myDDI4_UnitDataRecord$isViewedFrom} returns a character vector of of DdiUrns for R6  UnitDataViewpoint class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_UnitDataRecord$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_UnitDataRecord <- R6Class(classname="DDI4_UnitDataRecord",
  inherit = DDI4_LogicalRecord,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, isStructuredBy=NA,isViewedFrom=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_UnitDataRecord may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)



       if(!missing(isStructuredBy)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariableRelationStructure", proposedUrnVector=isStructuredBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
      }
      if(!missing(isViewedFrom)){
        private$.isViewedFrom <- returnUrnVectorIfValid(validClass="DDI4_UnitDataViewpoint", proposedUrnVector=isViewedFrom , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isViewedFrom, referencingAssociationName="isViewedFrom",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 2 class association(s):
    isStructuredBy = function(val){
      if (!missing(val)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariableRelationStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
       # invisible( private$.isStructuredBy)
      } else {
          private$.isStructuredBy
      }
    },

    isViewedFrom = function(val){
      if (!missing(val)){
        private$.isViewedFrom <- returnUrnVectorIfValid(validClass="DDI4_UnitDataViewpoint", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isViewedFrom, referencingAssociationName="isViewedFrom",  referencingUrn=thisUrn)  
       # invisible( private$.isViewedFrom)
      } else {
          private$.isViewedFrom
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .isStructuredBy = NULL,
    .isViewedFrom = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InstanceVariableIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isStructuredBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InstanceVariableRelationStructure", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isViewedFrom = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_UnitDataViewpoint", 
          structure = "DdiUrn vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_UnitDataRecord


#' @title DDI4_UnitDataViewpoint  The assignment of measure, identifier and attribute roles to InstanceVariables
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/UnitDataViewpoint}{UnitDataViewpoint} 

#'
#'   Definition
#'
#'   ============
#'
#'   The assignment of measure, identifier and attribute roles to InstanceVariables
#'
#'   
#'
#'   Each of three roles within a UnitDataViewpoint is a SimpleCollection of instance variables. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Assigning 
#'
#'   PatientID the role of identifier
#'
#'   systolic the role of measure
#'
#'   diastolic the role of measure
#'
#'   device the role of attribute
#'
#'   time the role of attribute
#'
#'   position (sitting, standing) the role of attribute
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Viewpoint is a UnitDataViewpoint to underline that the composition of a UnitDataRecord viewpoint is different from the composition of DataCube viewpoint. The DataCube viewpoint is out of scope currently. 
#'
#'   It is sometimes useful to describe a set of variables and their roles within the set. In the blood pressure example above, values for a set of variables are captured in one setting - the taking of a blood pressure. Some of the variables may be the measures of interest, for example the systolic and diastolic pressure. Other variables may take on the role of identifiers, such as the hospital’s patient ID and a government ID such as the U.S. Social Security Number. A third subgroup of variables may serve as attributes. The age of the patient, whether she is sitting or standing, the time of day. The assignment of these three roles, identifier, measure, and attribute, is the function of the Viewpoint.
#'
#'   Viewpoints are not fixed attributes of variables. This is why there is a separate Viewpoint class which maps to InstanceVariables.   In the example above, the Viewpoint is within the context of the original measurement. In a reanalysis of the data the roles might change. Time of day might be the measure of interest in a study of hospital practices.
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasIdentifierRole}}{ A read/write active binding.
#'          The InstanceVaribles functioning as identifiers
#'
#'                              \code{myDDI4_UnitDataViewpoint$hasIdentifierRole <- c(myIdentifierRoleURN)} sets the association 'hasIdentifierRole' from 'myDDI4_UnitDataViewpoint' to a character vector of DdiUrns for R6 IdentifierRole class instances
#'
#'                              \code{myDDI4_UnitDataViewpoint$hasIdentifierRole} returns a character vector of of DdiUrns for R6  IdentifierRole class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMeasureRole}}{ A read/write active binding.
#'          The InstanceVaribles functioning as measures
#'
#'                              \code{myDDI4_UnitDataViewpoint$hasMeasureRole <- c(myMeasureRoleURN)} sets the association 'hasMeasureRole' from 'myDDI4_UnitDataViewpoint' to a character vector of DdiUrns for R6 MeasureRole class instances
#'
#'                              \code{myDDI4_UnitDataViewpoint$hasMeasureRole} returns a character vector of of DdiUrns for R6  MeasureRole class instances}
#'   }
#'   \describe{
#'     \item{\code{hasAttributeRole}}{ A read/write active binding.
#'          The InstanceVaribles functioning as attributes
#'
#'                              \code{myDDI4_UnitDataViewpoint$hasAttributeRole <- c(myAttributeRoleURN)} sets the association 'hasAttributeRole' from 'myDDI4_UnitDataViewpoint' to a character vector of DdiUrns for R6 AttributeRole class instances
#'
#'                              \code{myDDI4_UnitDataViewpoint$hasAttributeRole} returns a character vector of of DdiUrns for R6  AttributeRole class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_UnitDataViewpoint$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_UnitDataViewpoint <- R6Class(classname="DDI4_UnitDataViewpoint",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasIdentifierRole=NA,hasMeasureRole=NA,hasAttributeRole=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_UnitDataViewpoint may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)



       if(!missing(hasIdentifierRole)){
        private$.hasIdentifierRole <- returnUrnVectorIfValid(validClass="DDI4_IdentifierRole", proposedUrnVector=hasIdentifierRole , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasIdentifierRole, referencingAssociationName="hasIdentifierRole",  referencingUrn=thisUrn)  
      }
      if(!missing(hasMeasureRole)){
        private$.hasMeasureRole <- returnUrnVectorIfValid(validClass="DDI4_MeasureRole", proposedUrnVector=hasMeasureRole , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasMeasureRole, referencingAssociationName="hasMeasureRole",  referencingUrn=thisUrn)  
      }
      if(!missing(hasAttributeRole)){
        private$.hasAttributeRole <- returnUrnVectorIfValid(validClass="DDI4_AttributeRole", proposedUrnVector=hasAttributeRole , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasAttributeRole, referencingAssociationName="hasAttributeRole",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 class association(s):
    hasAttributeRole = function(val){
      if (!missing(val)){
        private$.hasAttributeRole <- returnUrnVectorIfValid(validClass="DDI4_AttributeRole", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasAttributeRole, referencingAssociationName="hasAttributeRole",  referencingUrn=thisUrn)  
       # invisible( private$.hasAttributeRole)
      } else {
          private$.hasAttributeRole
      }
    },

    hasIdentifierRole = function(val){
      if (!missing(val)){
        private$.hasIdentifierRole <- returnUrnVectorIfValid(validClass="DDI4_IdentifierRole", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasIdentifierRole, referencingAssociationName="hasIdentifierRole",  referencingUrn=thisUrn)  
       # invisible( private$.hasIdentifierRole)
      } else {
          private$.hasIdentifierRole
      }
    },

    hasMeasureRole = function(val){
      if (!missing(val)){
        private$.hasMeasureRole <- returnUrnVectorIfValid(validClass="DDI4_MeasureRole", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasMeasureRole, referencingAssociationName="hasMeasureRole",  referencingUrn=thisUrn)  
       # invisible( private$.hasMeasureRole)
      } else {
          private$.hasMeasureRole
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasIdentifierRole = NULL,
    .hasMeasureRole = NULL,
    .hasAttributeRole = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAttributeRole = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AttributeRole", 
          structure = "DdiUrn vector"
        ),
        hasIdentifierRole = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_IdentifierRole", 
          structure = "DdiUrn vector"
        ),
        hasMeasureRole = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_MeasureRole", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_UnitDataViewpoint


#' @title DDI4_UnitSegmentLayout  UnitSegmentLayout supports the description of unit-record ("wide") data sets, where each row in the data set provides the same group of values for variables all relating to a single unit
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/UnitSegmentLayout}{UnitSegmentLayout} 

#'
#'   Definition
#'
#'   ============
#'
#'   UnitSegmentLayout supports the description of unit-record ("wide") data sets, where each row in the data set provides the same group of values for variables all relating to a single unit. Each logical column will contain data relating to the values for a single variable. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A simple spreadsheet. Commonly the first row of the table will contain variable names or descriptions.
#'
#'   
#'
#'   The following csv file has a rectangular layout and would import into a simple table in a spreadsheet:
#'
#'   PersonId,AgeYr,HeightCm
#'
#'   1,22,183
#'
#'   2,45,175
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   This is the classic rectangular data table used by most statistical packages, with rows/cases/observations and columns/variables/measurements. Each cell (DataPoint) in the table is the intersection of a Unit (row) and an InstanceVariable. 
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_PhysicalSegmentLayout}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_UnitSegmentLayout$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_UnitSegmentLayout <- R6Class(classname="DDI4_UnitSegmentLayout",
  inherit = DDI4_PhysicalSegmentLayout,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_UnitSegmentLayout may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

    }
  ),

  active = list(
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        arrayBase = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        commentPrefix = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ValueMappingIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        delimiter = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        encoding = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        escapeCharacter = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        formatsLogicalRecord = c(
          min="0", 
          max="1", 
          datatype = "DDI4_LogicalRecord", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasHeader = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        headerIsCaseSensitive = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        headerRowCount = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isDelimited = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isFixedWidth = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isStructuredBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_PhysicalLayoutRelationStructure", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        lineTerminator = c(
          min="0", 
          max="Inf", 
          datatype = "character", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        nullSequence = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        quoteCharacter = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        skipBlankRows = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        skipDataColumns = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        skipInitialSpace = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        skipRows = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        tableDirection = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TableDirectionValues", 
          structure = "Enumeration vector"
        ),
        textDirection = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TextDirectionValues", 
          structure = "Enumeration vector"
        ),
        treatConsecutiveDelimitersAsOne = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        trim = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TrimValues", 
          structure = "Enumeration vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_UnitSegmentLayout


#' @title DDI4_UnitType  A Unit Type is a type or class of objects of interest
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/UnitType}{UnitType} 

#'
#'   Definition
#'
#'   ============
#'
#'   A Unit Type is a type or class of objects of interest.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Person, Establishment, Household, State, Country, Dog, Automobile
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   UnitType is the most general in the hierarchy of UnitType, Universe, and Population. It is a description of the basic characteristic for a general set of Units.  A Universe is a set of entities defined by a more narrow specification than that of an underlying UnitType. A Population further narrows the specification to a specific time and geography.
#'
#'   A Unit Type is used to describe a class or group of Units based on a single characteristic with no specification of time and geography.  For example, the Unit Type of “Person” groups together a set of Units based on the characteristic that they are ‘Persons’.
#'
#'   It concerns not only Unit Types used in dissemination, but anywhere in the statistical process. E.g. using administrative data might involve the use of a fiscal unit. [GSIM 1.1]
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   Object Class [ISO11179]
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Unit Type
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Concept}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_UnitType$DdiUrn} returns the DDI URN of myDDI4_UnitType.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{descriptiveText}}{ A read/write active binding.
#'        A short natural language account of the characteristics of the object.
#'
#'                              \code{myDDI4_UnitType$descriptiveText <- list(myInternationalStructuredString)} sets the 'descriptiveText' property of 'myDDI4_UnitType' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_UnitType$descriptiveText} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{usesConcept}}{ A read/write active binding.
#'          Reference to the Concept that is being used
#'
#'                              \code{myDDI4_UnitType$usesConcept <- c(myConceptURN)} sets the association 'usesConcept' from 'myDDI4_UnitType' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_UnitType$usesConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_UnitType$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, descriptiveText=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_UnitType <- R6Class(classname="DDI4_UnitType",
  inherit = DDI4_Concept,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, descriptiveText=NA,    usesConcept=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_UnitType may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(descriptiveText)){
        private$.descriptiveText <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=descriptiveText , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(usesConcept)){
        private$.usesConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=usesConcept , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesConcept, referencingAssociationName="usesConcept",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    descriptiveText = function(val){
      if (!missing(val)){
        private$.descriptiveText <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.descriptiveText
      }
    },


# There are 1 class association(s):
    usesConcept = function(val){
      if (!missing(val)){
        private$.usesConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesConcept, referencingAssociationName="usesConcept",  referencingUrn=thisUrn)  
       # invisible( private$.usesConcept)
      } else {
          private$.usesConcept
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .descriptiveText = NULL,
    .usesConcept = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definition = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        descriptiveText = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usesConcept = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_UnitType


#' @title DDI4_Universe  A defined set or class of people, entities, events, or objects, with no specification of time and geography, contextualizing a Unit Type
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/Universe}{Universe} 

#'
#'   Definition
#'
#'   ============
#'
#'   A defined set or class of people, entities, events, or objects, with no specification of time and geography, contextualizing a Unit Type
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   1. Canadian adults (not limited to those residing in Canada)
#'
#'   2. Computer companies 
#'
#'   3. Universities
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   Universe sits in a hierarchy between UnitType and Population, with UnitType being most general and Population most specific. A Universe is a set of entities defined by a more narrow specification than that of an underlying UnitType. A Population further narrows the specification to a specific time and geography.
#'
#'   
#'
#'   If the Universe consists of people, a number of factors may be used in defining membership in the Universe, such as age, sex, race, residence, income, veteran status, criminal convictions, etc. The universe may consist of elements other than persons, such as housing units, court cases, deaths, countries, etc. A universe may be described as "inclusive" or "exclusive". 
#'
#'   
#'
#'   Not to be confused with Population, which includes the specification of time and geography.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   c:UniverseType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Population
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Concept}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_Universe$DdiUrn} returns the DDI URN of myDDI4_Universe.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{isInclusive}}{ A read/write active binding.
#'        Default value is True. The description statement of a universe is generally stated in inclusive terms such as "All persons with university degree". Occasionally a universe is defined by what it excludes, i.e., "All persons except those with university degree". In this case the value would be changed to "false".
#'
#'                              \code{myDDI4_Universe$isInclusive <- mylogical} sets the isInclusive property of 'myDDI4_Universe' to the vector of datatype:logical
#'
#'                              \code{myDDI4_Universe$isInclusive} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{descriptiveText}}{ A read/write active binding.
#'        A short natural language account of the characteristics of the object.
#'
#'                              \code{myDDI4_Universe$descriptiveText <- list(myInternationalStructuredString)} sets the 'descriptiveText' property of 'myDDI4_Universe' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_Universe$descriptiveText} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{narrowsUnitType}}{ A read/write active binding.
#'          Reference to the Unit Type that the Universe definition narrows.
#'
#'                              \code{myDDI4_Universe$narrowsUnitType <- c(myUnitTypeURN)} sets the association 'narrowsUnitType' from 'myDDI4_Universe' to a character vector of DdiUrns for R6 UnitType class instances
#'
#'                              \code{myDDI4_Universe$narrowsUnitType} returns a character vector of of DdiUrns for R6  UnitType class instances}
#'   }
#'   \describe{
#'     \item{\code{usesConcept}}{ A read/write active binding.
#'          Reference to the Concept that is being used
#'
#'                              \code{myDDI4_Universe$usesConcept <- c(myConceptURN)} sets the association 'usesConcept' from 'myDDI4_Universe' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_Universe$usesConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_Universe$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, isInclusive=NA, descriptiveText=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_Universe <- R6Class(classname="DDI4_Universe",
  inherit = DDI4_Concept,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, isInclusive=NA, descriptiveText=NA,    narrowsUnitType=NA,usesConcept=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_Universe may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(isInclusive)){
        private$.isInclusive <- returnVectorIfValid(validVectorClass="logical", proposedVector=isInclusive , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(descriptiveText)){
        private$.descriptiveText <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=descriptiveText , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(narrowsUnitType)){
        private$.narrowsUnitType <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=narrowsUnitType , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.narrowsUnitType, referencingAssociationName="narrowsUnitType",  referencingUrn=thisUrn)  
      }
      if(!missing(usesConcept)){
        private$.usesConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=usesConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesConcept, referencingAssociationName="usesConcept",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 2 properties: 
    descriptiveText = function(val){
      if (!missing(val)){
        private$.descriptiveText <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.descriptiveText
      }
    },

    isInclusive = function(val){
      if (!missing(val)){
        private$.isInclusive <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isInclusive
      }
    },


# There are 2 class association(s):
    narrowsUnitType = function(val){
      if (!missing(val)){
        private$.narrowsUnitType <- returnUrnVectorIfValid(validClass="DDI4_UnitType", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.narrowsUnitType, referencingAssociationName="narrowsUnitType",  referencingUrn=thisUrn)  
       # invisible( private$.narrowsUnitType)
      } else {
          private$.narrowsUnitType
      }
    },

    usesConcept = function(val){
      if (!missing(val)){
        private$.usesConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesConcept, referencingAssociationName="usesConcept",  referencingUrn=thisUrn)  
       # invisible( private$.usesConcept)
      } else {
          private$.usesConcept
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .isInclusive = NULL,
    .descriptiveText = NULL,
    .narrowsUnitType = NULL,
    .usesConcept = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definition = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        descriptiveText = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isInclusive = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        narrowsUnitType = c(
          min="0", 
          max="1", 
          datatype = "DDI4_UnitType", 
          structure = "DdiUrn vector"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usesConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_Universe


#' @title DDI4_ValueAndConceptDescription  A formal description of a set of values
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ValueAndConceptDescription}{ValueAndConceptDescription} 

#'
#'   Definition
#'
#'   ============
#'
#'   A formal description of a set of values. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Example 1) 
#'
#'   The integers between 1 and 10 inclusive.  
#'
#'   The values of x satisfying the logicalExpression property:
#'
#'   " (1 <=x <= 10) AND mod(x,10)=0"
#'
#'   Also described with minimumValueInclusive = 1 and maximumValueInclusive = 10 (and datatype of integer)
#'
#'   
#'
#'   Example 2)
#'
#'   The upper case letters A through C and X
#'
#'   described with the regularExpression "/[A-CX]/"
#'
#'   
#'
#'   
#'
#'   Example 3)
#'
#'   A date-time described with the Unicode Locale Data Markup Language pattern yyyy.MM.dd G 'at' HH:mm:ss zzz
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   The ValueAndConceptDescription may be used to describe either a value domain or a conceptual domain. For a value domain, the ValueAndConceptDescription contains the details for a “described” domain (as opposed to an enumerated domain). There are a number of properties which can be used for the description. The description could be just text such as: “an even number greater than zero”. Or a more formal logical expression like “x>0 and mod(x,2)=0”. A regular expression might be useful for describing a textual domain. It could also employ a format pattern from the Unicode Locale Data Markup Language (LDML) (http://www.unicode.org/reports/tr35/tr35.html.
#'
#'   Some Conceptual Domains might be described with just a narrative. Others, though, might be described in much the same way as a value domain depending on the specificity of the concept.
#'
#'   
#'
#'   In ISO 11404 a value domain may be described either through enumeration or description, or both. This class provides the facility for that description. It may be just a text description, but a description through a logical expression is machine actionable and might, for example, be rendered as an integrity constraint. If both text and a logical expression are provided the logical expression is to be taken as the canonical description.
#'
#'   The logical expression should conform to the expression syntax of VTL. https://sdmx.org/?page_id=5096 
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ValueAndConceptDescription$DdiUrn} returns the DDI URN of myDDI4_ValueAndConceptDescription.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{description}}{ A read/write active binding.
#'        A formal description of the set of values.
#'
#'                              \code{myDDI4_ValueAndConceptDescription$description <- list(myInternationalStructuredString)} sets the 'description' property of 'myDDI4_ValueAndConceptDescription' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_ValueAndConceptDescription$description} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{logicalExpression}}{ A read/write active binding.
#'        A logical expression where the values of "x" making the expression true are the members of the set of valid values.  Example: (all reals x such that  x >0) describes the real numbers greater than 0
#'
#'                              \code{myDDI4_ValueAndConceptDescription$logicalExpression <- list(myExternalControlledVocabularyEntry)} sets the 'logicalExpression' property of 'myDDI4_ValueAndConceptDescription' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ValueAndConceptDescription$logicalExpression} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{regularExpression}}{ A read/write active binding.
#'        A regular expression where strings matching the expression belong to the set of valid values. Use typeOfContent to specify the syntax of the regularExpression found in content.
#'
#'                              \code{myDDI4_ValueAndConceptDescription$regularExpression <- list(myTypedString)} sets the 'regularExpression' property of 'myDDI4_ValueAndConceptDescription' to a list of R6 TypedString class instances
#'
#'                              \code{myDDI4_ValueAndConceptDescription$regularExpression} returns a list of of R6 : TypedString class instances}
#'   }
#'   \describe{
#'     \item{\code{minimumValueInclusive}}{ A read/write active binding.
#'        A string denoting the minimum possible value. From https://www.w3.org/TR/tabular-metadata/  5.11.2 minimum: “An atomic property that contains a single number or string that is the minimum valid value (inclusive); equivalent to minInclusive. The value of this property becomes the minimum annotation for the described datatype. See Value Constraints in [tabular-data-model] for details.”
#'
#'                              \code{myDDI4_ValueAndConceptDescription$minimumValueInclusive <- mycharacter} sets the minimumValueInclusive property of 'myDDI4_ValueAndConceptDescription' to the vector of datatype:character
#'
#'                              \code{myDDI4_ValueAndConceptDescription$minimumValueInclusive} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{maximumValueInclusive}}{ A read/write active binding.
#'        A string denoting the maximum possible value. From https://www.w3.org/TR/tabular-metadata/  5.11.2 maximum: “An atomic property that contains a single number or string that is the maximum valid value (inclusive); equivalent to maxInclusive. The value of this property becomes the maximum annotation for the described datatype. See Value Constraints in [tabular-data-model] for details.”
#'
#'                              \code{myDDI4_ValueAndConceptDescription$maximumValueInclusive <- mycharacter} sets the maximumValueInclusive property of 'myDDI4_ValueAndConceptDescription' to the vector of datatype:character
#'
#'                              \code{myDDI4_ValueAndConceptDescription$maximumValueInclusive} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{minimumValueExclusive}}{ A read/write active binding.
#'        A string denoting the minimum possible value (excluding this value) From https://www.w3.org/TR/tabular-metadata/  5.11.2 minExclusive: “An atomic property that contains a single number or string that is the minimum valid value (exclusive). The value of this property becomes the minimum exclusive annotation for the described datatype. See Value Constraints in [tabular-data-model] for details.”   DDI3.2 handles this with a Boolean isInclusive attribute. 
#'
#'                              \code{myDDI4_ValueAndConceptDescription$minimumValueExclusive <- mycharacter} sets the minimumValueExclusive property of 'myDDI4_ValueAndConceptDescription' to the vector of datatype:character
#'
#'                              \code{myDDI4_ValueAndConceptDescription$minimumValueExclusive} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{maximumValueExclusive}}{ A read/write active binding.
#'        A string denotFrom https://www.w3.org/TR/tabular-metadata/  5.11.2 maxExclusive: “An atomic property that contains a single number or string that is the maximum valid value (exclusive). The value of this property becomes the maximum exclusive annotation for the described datatype. See Value Constraints in [tabular-data-model] for details.”   DDI3.2 handles this with a Boolean isInclusive attribute. ing the maximumpossible value (excluding this value)
#'
#'                              \code{myDDI4_ValueAndConceptDescription$maximumValueExclusive <- mycharacter} sets the maximumValueExclusive property of 'myDDI4_ValueAndConceptDescription' to the vector of datatype:character
#'
#'                              \code{myDDI4_ValueAndConceptDescription$maximumValueExclusive} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{classificationLevel}}{ A read/write active binding.
#'        Indicates the type of relationship, nominal, ordinal, interval, ratio, or continuous. Use where appropriate for the representation type.
#'
#'                              \code{myDDI4_ValueAndConceptDescription$classificationLevel <- list(myCategoryRelationCode)} sets the 'classificationLevel' property of 'myDDI4_ValueAndConceptDescription' to a list of R6 CategoryRelationCode class instances
#'
#'                              \code{myDDI4_ValueAndConceptDescription$classificationLevel} returns a list of of R6 : CategoryRelationCode class instances}
#'   }
#'   \describe{
#'     \item{\code{formatPattern}}{ A read/write active binding.
#'        A pattern for a number as described in Unicode Locale Data Markup Language (LDML) (http://www.unicode.org/reports/tr35/tr35.html) Part 3: Numbers  (http://www.unicode.org/reports/tr35/tr35-numbers.html#Number_Format_Patterns) and Part 4. Dates (http://www.unicode.org/reports/tr35/tr35-dates.html#Date_Format_Patterns) . Examples would be    #,##0.### to describe the pattern for a decimal number, or yyyy.MM.dd G 'at' HH:mm:ss zzz for a datetime pattern.
#'
#'                              \code{myDDI4_ValueAndConceptDescription$formatPattern <- list(myExternalControlledVocabularyEntry)} sets the 'formatPattern' property of 'myDDI4_ValueAndConceptDescription' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ValueAndConceptDescription$formatPattern} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ValueAndConceptDescription$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, description=NA, logicalExpression=NA, regularExpression=NA, minimumValueInclusive=NA, maximumValueInclusive=NA, minimumValueExclusive=NA, maximumValueExclusive=NA, classificationLevel=NA, formatPattern=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ValueAndConceptDescription <- R6Class(classname="DDI4_ValueAndConceptDescription",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, description=NA, logicalExpression=NA, regularExpression=NA, minimumValueInclusive=NA, maximumValueInclusive=NA, minimumValueExclusive=NA, maximumValueExclusive=NA, classificationLevel=NA, formatPattern=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ValueAndConceptDescription may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(description)){
        private$.description <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=description , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(logicalExpression)){
        private$.logicalExpression <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=logicalExpression , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(regularExpression)){
        private$.regularExpression <- returnR6ObjectIfValid(validClass="DDI4_TypedString", proposedObjectList=regularExpression , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(minimumValueInclusive)){
        private$.minimumValueInclusive <- returnVectorIfValid(validVectorClass="character", proposedVector=minimumValueInclusive , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(maximumValueInclusive)){
        private$.maximumValueInclusive <- returnVectorIfValid(validVectorClass="character", proposedVector=maximumValueInclusive , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(minimumValueExclusive)){
        private$.minimumValueExclusive <- returnVectorIfValid(validVectorClass="character", proposedVector=minimumValueExclusive , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(maximumValueExclusive)){
        private$.maximumValueExclusive <- returnVectorIfValid(validVectorClass="character", proposedVector=maximumValueExclusive , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(classificationLevel)){
        private$.classificationLevel <- returnEnumeratedValues(enumerationObject=DDI4_CategoryRelationCode$new(registerThisObject=FALSE), proposedCharacterVector=classificationLevel , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(formatPattern)){
        private$.formatPattern <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=formatPattern , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 9 properties: 
    classificationLevel = function(val){
      if (!missing(val)){
        private$.classificationLevel <- returnEnumeratedValues(enumerationObject=DDI4_CategoryRelationCode$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.classificationLevel
      }
    },

    description = function(val){
      if (!missing(val)){
        private$.description <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.description
      }
    },

    formatPattern = function(val){
      if (!missing(val)){
        private$.formatPattern <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.formatPattern
      }
    },

    logicalExpression = function(val){
      if (!missing(val)){
        private$.logicalExpression <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.logicalExpression
      }
    },

    maximumValueExclusive = function(val){
      if (!missing(val)){
        private$.maximumValueExclusive <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.maximumValueExclusive
      }
    },

    maximumValueInclusive = function(val){
      if (!missing(val)){
        private$.maximumValueInclusive <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.maximumValueInclusive
      }
    },

    minimumValueExclusive = function(val){
      if (!missing(val)){
        private$.minimumValueExclusive <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.minimumValueExclusive
      }
    },

    minimumValueInclusive = function(val){
      if (!missing(val)){
        private$.minimumValueInclusive <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.minimumValueInclusive
      }
    },

    regularExpression = function(val){
      if (!missing(val)){
        private$.regularExpression <- returnR6ObjectIfValid(validClass="DDI4_TypedString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.regularExpression
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .description = NULL,
    .logicalExpression = NULL,
    .regularExpression = NULL,
    .minimumValueInclusive = NULL,
    .maximumValueInclusive = NULL,
    .minimumValueExclusive = NULL,
    .maximumValueExclusive = NULL,
    .classificationLevel = NULL,
    .formatPattern = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        classificationLevel = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CategoryRelationCode", 
          structure = "Enumeration vector"
        ),
        description = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        formatPattern = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        logicalExpression = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        maximumValueExclusive = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        maximumValueInclusive = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        minimumValueExclusive = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        minimumValueInclusive = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        regularExpression = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TypedString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ValueAndConceptDescription


#' @title DDI4_ValueDomain  The permitted range of values for a characteristic of a variable
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ValueDomain}{ValueDomain} 

#'
#'   Definition
#'
#'   ============
#'
#'   The permitted range of values for a characteristic of a variable. [GSIM 1.1]
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Age categories with a numeric code list; Age in years; Young, Middle-aged and Old
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   The values can be described by enumeration or by an expression. Value domains can be either substantive/sentinel, or described/enumeration
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   BUT NOT Grid/Numeric/Code/*_ResponseDomain [DDI-L/Questions] - this is "ResponseDomain"
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:RepresentationType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Value Domain
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ValueDomain$DdiUrn} returns the DDI URN of myDDI4_ValueDomain.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A display label for the object. May be expressed in multiple languages. Repeat for labels with different content, for example, labels with differing length limitations.
#'
#'                              \code{myDDI4_ValueDomain$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_ValueDomain' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_ValueDomain$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#'   \describe{
#'     \item{\code{recommendedDataType}}{ A read/write active binding.
#'        The data types that are recommended for use with this domain
#'
#'                              \code{myDDI4_ValueDomain$recommendedDataType <- list(myExternalControlledVocabularyEntry)} sets the 'recommendedDataType' property of 'myDDI4_ValueDomain' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ValueDomain$recommendedDataType} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ValueDomain$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, displayLabel=NA, recommendedDataType=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ValueDomain <- R6Class(classname="DDI4_ValueDomain",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, displayLabel=NA, recommendedDataType=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ValueDomain may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(recommendedDataType)){
        private$.recommendedDataType <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=recommendedDataType , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },

    recommendedDataType = function(val){
      if (!missing(val)){
        private$.recommendedDataType <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.recommendedDataType
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .displayLabel = NULL,
    .recommendedDataType = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        recommendedDataType = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_ValueDomain


#' @title DDI4_ValueMapping  Provides physical characteristics for an InstanceVariable as part of a PhysicalSegmentLayout
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ValueMapping}{ValueMapping} 

#'
#'   Definition
#'
#'   ============
#'
#'   Provides physical characteristics for an InstanceVariable as part of a PhysicalSegmentLayout
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A variable "age" might be represented in a file as a string with a maximum length of 5 characters and a number pattern of #'
#'   #'
#'   0.0
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   An InstanceVariable has details of value domain and datatype, but will not have the final details of how a value is physically represented in a data file. A variable for height, for example, may be represented as a real number, but may be represented as a string in multiple ways. The decimal separator might be, for example a period or a comma. The string representing the value of a payment might be preceded by a currency symbol. The same numeric value might be written as “1,234,567” or “1.234567 E6”. A missing value might be written as “.”, “NA”, “.R” or as “R”. 
#'
#'   The ValueMapping describes how the value of an InstanceVariable is physically expressed. The properties of the ValueMapping as intended to be compatible with the W3C Metadata Vocabulary for Tabular Data (https://www.w3.org/TR/tabular-metadata/ ) as well as common programming languages and statistical packages. The “format” property, for example can draw from an external controlled vocabulary such as the set of formats for Stata, SPSS, or SAS.
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ValueMapping$DdiUrn} returns the DDI URN of myDDI4_ValueMapping.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{physicalDataType}}{ A read/write active binding.
#'        The base datatype of the physical representation. An integer InstanceVariable might, for example, be stored as a floating point number. From https://www.w3.org/TR/tabular-metadata/ Inherited 5.7 datatype: “An atomic property that contains either a single string that is the main datatype of the values of the cell or a datatype description object. If the value of this property is a string, it MUST be the name of one of the built-in datatypes defined in section 5.11.1 Built-in Datatypes and this value is normalized to an object whose base property is the original string value. If it is an object then it describes a more specialized datatype. If a cell contains a sequence (i.e. the separator property is specified and not null) then this property specifies the datatype of each value within that sequence. See 5.11 Datatypes and Parsing Cells in [tabular-data-model] for more details.  The normalized value of this property becomes the datatype annotation for the described column. ” 
#'
#'                              \code{myDDI4_ValueMapping$physicalDataType <- list(myExternalControlledVocabularyEntry)} sets the 'physicalDataType' property of 'myDDI4_ValueMapping' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ValueMapping$physicalDataType} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{defaultDecimalSeparator}}{ A read/write active binding.
#'        Default value is . (period). The string separating the integer part from the fractional part of a decimal or real number. In W3C part of the datatype format From https://www.w3.org/TR/tabular-metadata/ tabular 6.4.2 decimalChar: “A string whose value is used to represent a decimal point within the number. If the supplied value is not a string, implementations MUST issue a warning and proceed as if the property had not been specified.” 
#'
#'                              \code{myDDI4_ValueMapping$defaultDecimalSeparator <- list(myChar)} sets the 'defaultDecimalSeparator' property of 'myDDI4_ValueMapping' to a list of R6 Char class instances
#'
#'                              \code{myDDI4_ValueMapping$defaultDecimalSeparator} returns a list of of R6 : Char class instances}
#'   }
#'   \describe{
#'     \item{\code{defaultDigitGroupSeparator}}{ A read/write active binding.
#'        Default value is null. A string separating groups of digits (for readability). In W3C part of the datatype format From https://www.w3.org/TR/tabular-metadata/ tabular 6.4.2 groupChar: “A string whose value is used to group digits within the number. If the supplied value is not a string, implementations MUST issue a warning and proceed as if the property had not been specified.” 
#'
#'                              \code{myDDI4_ValueMapping$defaultDigitGroupSeparator <- list(myChar)} sets the 'defaultDigitGroupSeparator' property of 'myDDI4_ValueMapping' to a list of R6 Char class instances
#'
#'                              \code{myDDI4_ValueMapping$defaultDigitGroupSeparator} returns a list of of R6 : Char class instances}
#'   }
#'   \describe{
#'     \item{\code{numberPattern}}{ A read/write active binding.
#'        A pattern description of the format of a numeric value. In W3C part of the datatype format From https://www.w3.org/TR/tabular-metadata/ tabular 6.4.2 pattern: “A number format pattern as defined in [UAX35] http://www.unicode.org/reports/tr35/tr35-31/tr35-numbers.html#Number_Format_Patterns . Implementations MUST recognise number format patterns containing the symbols 0, #, the specified decimalChar (or "." if unspecified), the specified groupChar (or "," if unspecified), E, +, % and ‰. Implementations MAY additionally recognise number format patterns containing other special pattern characters defined in [UAX35]. If the supplied value is not a string, or if it contains an invalid number format pattern or uses special pattern characters that the implementation does not recognise, implementations MUST issue a warning and proceed as if the property had not been specified.   f the datatype format annotation is a single string, this is interpreted in the same way as if it were an object with a pattern property whose value is that string. If the groupChar is specified, but no pattern is supplied, when parsing the string value of a cell against this format specification, implementations MUST recognise and parse numbers that consist of:  an optional + or - sign, …  Implementations MAY also recognise numeric values that are in any of the standard-decimal, standard-percent or standard-scientific formats listed in the Unicode Common Locale Data Repository. …” 
#'
#'                              \code{myDDI4_ValueMapping$numberPattern <- mycharacter} sets the numberPattern property of 'myDDI4_ValueMapping' to the vector of datatype:character
#'
#'                              \code{myDDI4_ValueMapping$numberPattern} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{defaultValue}}{ A read/write active binding.
#'        A default string indicating the value to substitute for an empty string. From https://www.w3.org/TR/tabular-metadata/ Inherited 5.7  default “An atomic property holding a single string that is used to create a default value for the cell in cases where the original string value is an empty string. See Parsing Cells in [tabular-data-model] for more details. If not specified, the default for the default property is the empty string, "". The value of this property becomes the default annotation for the described column.”
#'
#'                              \code{myDDI4_ValueMapping$defaultValue <- list(myValueString)} sets the 'defaultValue' property of 'myDDI4_ValueMapping' to a list of R6 ValueString class instances
#'
#'                              \code{myDDI4_ValueMapping$defaultValue} returns a list of of R6 : ValueString class instances}
#'   }
#'   \describe{
#'     \item{\code{nullSequence}}{ A read/write active binding.
#'        A string indicating a null value. From https://www.w3.org/TR/tabular-metadata/ 4.3 null — “the string or strings which cause the value of cells having string value matching any of these values to be null.” From Inherited 5.7 null: “An atomic property giving the string or strings used for null values within the data. If the string value of the cell is equal to any one of these values, the cell value is null. See Parsing Cells in [tabular-data-model] for more details. If not specified, the default for the null property is the empty string "". The value of this property becomes the null annotation for the described column.” 
#'
#'                              \code{myDDI4_ValueMapping$nullSequence <- mycharacter} sets the nullSequence property of 'myDDI4_ValueMapping' to the vector of datatype:character
#'
#'                              \code{myDDI4_ValueMapping$nullSequence} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{format}}{ A read/write active binding.
#'        This defines the format of the physical representation of the value. From https://www.w3.org/TR/tabular-metadata/  5.11.2 format: “An atomic property that contains either a single string or an object that defines the format of a value of this type, used when parsing a string value as described in Parsing Cells in [tabular-data-model]. The value of this property becomes the format annotation for the described datatype.” See https://www.w3.org/TR/tabular-metadata/ Tabular 6.4.2 ‘Formats for numeric datatypes’  this may include decimalChar, groupChar, pattern  “By default, numeric values must be in the formats defined in [xmlschema11-2]. It is not uncommon for numbers within tabular data to be formatted for human consumption, which may involve using commas for decimal points, grouping digits in the number using commas, or adding percent signs to the number.” See https://www.w3.org/TR/tabular-metadata/ Tabular 6.4. Formats for Booleans   ” Boolean values may be represented in many ways aside from the standard 1 and 0 or true and false.” See https://www.w3.org/TR/tabular-metadata/ 6.4.4. Formats for dates and times “By default, dates and times are assumed to be in the format defined in [xmlschema11-2]. However dates and times are commonly represented in tabular data in other formats.” See https://www.w3.org/TR/tabular-metadata/ 6.4.5 Formats for durations “Durations MUST be formatted and interpreted as defined in [xmlschema11-2], using the [ISO8601] format -?PnYnMnDTnHnMnS. For example, the duration P1Y1D is used for a year and a day; the duration PT2H30M for 2 hours and 30 minutes.” See  https://www.w3.org/TR/tabular-metadata/ 6.4.6 Formats for other types “If the datatype base is not numeric, boolean, a date/time type, or a duration type, the datatype format annotation provides a regular expression for the string values, with syntax and processing defined by [ECMASCRIPT]. If the supplied value is not a valid regular expression, implementations MUST issue a warning and proceed as if no format had been provided.” From DDI3.2 ManagedNumericRepresentation@format “A format for number expressed as a string.” From DDI3.2 ManagedDateTimeRepresentation_DateFieldFormat “Describes the format of the date field, in formats such as YYYY/MM or MM-DD-YY, etc. If this element is omitted, then the format is assumed to be the XML Schema format corresponding to the type attribute value.” 
#'
#'                              \code{myDDI4_ValueMapping$format <- list(myExternalControlledVocabularyEntry)} sets the 'format' property of 'myDDI4_ValueMapping' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ValueMapping$format} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{length}}{ A read/write active binding.
#'        The length of the physical representation of the value. From https://www.w3.org/TR/tabular-metadata/  5.11.2 length: “A numeric atomic property that contains a single integer that is the exact length of the value. The value of this property becomes the length annotation for the described datatype. See Length Constraints in [tabular-data-model] for details.” Corresponds to DDI2.5 var/location/width and DDI 3.2 PhysicalLocation/Width
#'
#'                              \code{myDDI4_ValueMapping$length <- myinteger} sets the length property of 'myDDI4_ValueMapping' to the vector of datatype:integer
#'
#'                              \code{myDDI4_ValueMapping$length} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{minimumLength}}{ A read/write active binding.
#'        The smallest possible value for the length of the physical representation of the value. From https://www.w3.org/TR/tabular-metadata/  5.11.2 minLength: “An atomic property that contains a single integer that is the minimum length of the value. The value of this property becomes the minimum length annotation for the described datatype. See Length Constraints in [tabular-data-model] for details.”
#'
#'                              \code{myDDI4_ValueMapping$minimumLength <- myinteger} sets the minimumLength property of 'myDDI4_ValueMapping' to the vector of datatype:integer
#'
#'                              \code{myDDI4_ValueMapping$minimumLength} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{maximumLength}}{ A read/write active binding.
#'        The largest possible value of the length of the physical representation of the value. From https://www.w3.org/TR/tabular-metadata/  5.11.2 maxLength: “A numeric atomic property that contains a single integer that is the maximum length of the value. The value of this property becomes the maximum length annotation for the described datatype. See Length Constraints in [tabular-data-model] for details.”
#'
#'                              \code{myDDI4_ValueMapping$maximumLength <- myinteger} sets the maximumLength property of 'myDDI4_ValueMapping' to the vector of datatype:integer
#'
#'                              \code{myDDI4_ValueMapping$maximumLength} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{scale}}{ A read/write active binding.
#'        The scale of the number expressed as an integer (for example a number expressed in 100's, 5 = 500 would have a scale of 100).   From DDI 3.2 ManagedNumericRepresentation@scale: 
#'
#'                              \code{myDDI4_ValueMapping$scale <- myinteger} sets the scale property of 'myDDI4_ValueMapping' to the vector of datatype:integer
#'
#'                              \code{myDDI4_ValueMapping$scale} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{decimalPositions}}{ A read/write active binding.
#'        The number of decimal positions expressed as an integer. Used when the decimal position is implied (no decimal separator is present) See DDI 3.2 ManagedNumericRepresentation@decimalPositions
#'
#'                              \code{myDDI4_ValueMapping$decimalPositions <- myinteger} sets the decimalPositions property of 'myDDI4_ValueMapping' to the vector of datatype:integer
#'
#'                              \code{myDDI4_ValueMapping$decimalPositions} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{isRequired}}{ A read/write active binding.
#'        If True a value is required for this variable. NOTE: this might be better at the InstanceVariable or higher in the variable cascade. From https://www.w3.org/TR/tabular-metadata/ Inherited 5.7 required: “A boolean atomic property taking a single value which indicates whether the cell value can be null. See Parsing Cells in [tabular-data-model] for more details. The default is false, which means cells can have null values. The value of this property becomes the required annotation for the described column.”
#'
#'                              \code{myDDI4_ValueMapping$isRequired <- mylogical} sets the isRequired property of 'myDDI4_ValueMapping' to the vector of datatype:logical
#'
#'                              \code{myDDI4_ValueMapping$isRequired} returns a vector of datatype: logical}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{formatsDataPoint}}{ A read/write active binding.
#'          Describes the physical representation of the InstanceVariable 
#'
#'                              \code{myDDI4_ValueMapping$formatsDataPoint <- c(myDataPointURN)} sets the association 'formatsDataPoint' from 'myDDI4_ValueMapping' to a character vector of DdiUrns for R6 DataPoint class instances
#'
#'                              \code{myDDI4_ValueMapping$formatsDataPoint} returns a character vector of of DdiUrns for R6  DataPoint class instances}
#'   }
#'   \describe{
#'     \item{\code{describesSegmentLocation}}{ A read/write active binding.
#'          Uses a PhysicalSegmentLocation to describe where in the physical record a segment representing the InstanceVarible is. This could be, for example, described as a start position and end position value for characters in a text record via the SegmentByText extension of PhysicalSegmentLocation.
#'
#'                              \code{myDDI4_ValueMapping$describesSegmentLocation <- c(myPhysicalSegmentLocationURN)} sets the association 'describesSegmentLocation' from 'myDDI4_ValueMapping' to a character vector of DdiUrns for R6 PhysicalSegmentLocation class instances
#'
#'                              \code{myDDI4_ValueMapping$describesSegmentLocation} returns a character vector of of DdiUrns for R6  PhysicalSegmentLocation class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ValueMapping$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, physicalDataType=NA, defaultDecimalSeparator=NA, defaultDigitGroupSeparator=NA, numberPattern=NA, defaultValue=NA, nullSequence=NA, format=NA, length=NA, minimumLength=NA, maximumLength=NA, scale=NA, decimalPositions=NA, isRequired=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ValueMapping <- R6Class(classname="DDI4_ValueMapping",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, physicalDataType=NA, defaultDecimalSeparator=NA, defaultDigitGroupSeparator=NA, numberPattern=NA, defaultValue=NA, nullSequence=NA, format=NA, length=NA, minimumLength=NA, maximumLength=NA, scale=NA, decimalPositions=NA, isRequired=NA,    formatsDataPoint=NA,describesSegmentLocation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ValueMapping may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(physicalDataType)){
        private$.physicalDataType <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=physicalDataType , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(defaultDecimalSeparator)){
        private$.defaultDecimalSeparator <- returnRegularExpressionValues(RegularExpressionObject=DDI4_Char$new(registerThisObject=FALSE), proposedCharacterVector=defaultDecimalSeparator , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(defaultDigitGroupSeparator)){
        private$.defaultDigitGroupSeparator <- returnRegularExpressionValues(RegularExpressionObject=DDI4_Char$new(registerThisObject=FALSE), proposedCharacterVector=defaultDigitGroupSeparator , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(numberPattern)){
        private$.numberPattern <- returnVectorIfValid(validVectorClass="character", proposedVector=numberPattern , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(defaultValue)){
        private$.defaultValue <- returnR6ObjectIfValid(validClass="DDI4_ValueString", proposedObjectList=defaultValue , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(nullSequence)){
        private$.nullSequence <- returnVectorIfValid(validVectorClass="character", proposedVector=nullSequence , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(format)){
        private$.format <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=format , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(length)){
        private$.length <- returnVectorIfValid(validVectorClass="integer", proposedVector=length , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(minimumLength)){
        private$.minimumLength <- returnVectorIfValid(validVectorClass="integer", proposedVector=minimumLength , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(maximumLength)){
        private$.maximumLength <- returnVectorIfValid(validVectorClass="integer", proposedVector=maximumLength , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(scale)){
        private$.scale <- returnVectorIfValid(validVectorClass="integer", proposedVector=scale , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(decimalPositions)){
        private$.decimalPositions <- returnVectorIfValid(validVectorClass="integer", proposedVector=decimalPositions , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(isRequired)){
        private$.isRequired <- returnVectorIfValid(validVectorClass="logical", proposedVector=isRequired , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(formatsDataPoint)){
        private$.formatsDataPoint <- returnUrnVectorIfValid(validClass="DDI4_DataPoint", proposedUrnVector=formatsDataPoint , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.formatsDataPoint, referencingAssociationName="formatsDataPoint",  referencingUrn=thisUrn)  
      }
      if(!missing(describesSegmentLocation)){
        private$.describesSegmentLocation <- returnUrnVectorIfValid(validClass="DDI4_PhysicalSegmentLocation", proposedUrnVector=describesSegmentLocation , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.describesSegmentLocation, referencingAssociationName="describesSegmentLocation",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 13 properties: 
    decimalPositions = function(val){
      if (!missing(val)){
        private$.decimalPositions <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.decimalPositions
      }
    },

    defaultDecimalSeparator = function(val){
      if (!missing(val)){
        private$.defaultDecimalSeparator <- returnRegularExpressionValues(RegularExpressionObject=DDI4_Char$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.defaultDecimalSeparator
      }
    },

    defaultDigitGroupSeparator = function(val){
      if (!missing(val)){
        private$.defaultDigitGroupSeparator <- returnRegularExpressionValues(RegularExpressionObject=DDI4_Char$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.defaultDigitGroupSeparator
      }
    },

    defaultValue = function(val){
      if (!missing(val)){
        private$.defaultValue <- returnR6ObjectIfValid(validClass="DDI4_ValueString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.defaultValue
      }
    },

    format = function(val){
      if (!missing(val)){
        private$.format <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.format
      }
    },

    isRequired = function(val){
      if (!missing(val)){
        private$.isRequired <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isRequired
      }
    },

    length = function(val){
      if (!missing(val)){
        private$.length <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.length
      }
    },

    maximumLength = function(val){
      if (!missing(val)){
        private$.maximumLength <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.maximumLength
      }
    },

    minimumLength = function(val){
      if (!missing(val)){
        private$.minimumLength <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.minimumLength
      }
    },

    nullSequence = function(val){
      if (!missing(val)){
        private$.nullSequence <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.nullSequence
      }
    },

    numberPattern = function(val){
      if (!missing(val)){
        private$.numberPattern <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.numberPattern
      }
    },

    physicalDataType = function(val){
      if (!missing(val)){
        private$.physicalDataType <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.physicalDataType
      }
    },

    scale = function(val){
      if (!missing(val)){
        private$.scale <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.scale
      }
    },


# There are 2 class association(s):
    describesSegmentLocation = function(val){
      if (!missing(val)){
        private$.describesSegmentLocation <- returnUrnVectorIfValid(validClass="DDI4_PhysicalSegmentLocation", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.describesSegmentLocation, referencingAssociationName="describesSegmentLocation",  referencingUrn=thisUrn)  
       # invisible( private$.describesSegmentLocation)
      } else {
          private$.describesSegmentLocation
      }
    },

    formatsDataPoint = function(val){
      if (!missing(val)){
        private$.formatsDataPoint <- returnUrnVectorIfValid(validClass="DDI4_DataPoint", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.formatsDataPoint, referencingAssociationName="formatsDataPoint",  referencingUrn=thisUrn)  
       # invisible( private$.formatsDataPoint)
      } else {
          private$.formatsDataPoint
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .physicalDataType = NULL,
    .defaultDecimalSeparator = NULL,
    .defaultDigitGroupSeparator = NULL,
    .numberPattern = NULL,
    .defaultValue = NULL,
    .nullSequence = NULL,
    .format = NULL,
    .length = NULL,
    .minimumLength = NULL,
    .maximumLength = NULL,
    .scale = NULL,
    .decimalPositions = NULL,
    .isRequired = NULL,
    .formatsDataPoint = NULL,
    .describesSegmentLocation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        decimalPositions = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        defaultDecimalSeparator = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Char", 
          structure = "Regex vector"
        ),
        defaultDigitGroupSeparator = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Char", 
          structure = "Regex vector"
        ),
        defaultValue = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ValueString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        describesSegmentLocation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_PhysicalSegmentLocation", 
          structure = "DdiUrn vector"
        ),
        format = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        formatsDataPoint = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_DataPoint", 
          structure = "DdiUrn vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isRequired = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        length = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        maximumLength = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        minimumLength = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        nullSequence = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        numberPattern = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        physicalDataType = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        scale = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ValueMapping


#' @title DDI4_ValueMappingIndicator  Member Indicator for use with member type ValueMapping
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ValueMappingIndicator}{ValueMappingIndicator} 

#'
#'   Definition
#'
#'   ============
#'
#'   Member Indicator for use with member type ValueMapping
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ValueMappingIndicator$DdiUrn} returns the DDI URN of myDDI4_ValueMappingIndicator.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Index value of member in an ordered array
#'
#'                              \code{myDDI4_ValueMappingIndicator$index <- myinteger} sets the index property of 'myDDI4_ValueMappingIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_ValueMappingIndicator$index} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{member}}{ A read/write active binding.
#'          Restricts member target class to ValueMapping
#'
#'                              \code{myDDI4_ValueMappingIndicator$member <- c(myValueMappingURN)} sets the association 'member' from 'myDDI4_ValueMappingIndicator' to a character vector of DdiUrns for R6 ValueMapping class instances
#'
#'                              \code{myDDI4_ValueMappingIndicator$member} returns a character vector of of DdiUrns for R6  ValueMapping class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ValueMappingIndicator$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ValueMappingIndicator <- R6Class(classname="DDI4_ValueMappingIndicator",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA,    member=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ValueMappingIndicator may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(member)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_ValueMapping", proposedUrnVector=member , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.index
      }
    },


# There are 1 class association(s):
    member = function(val){
      if (!missing(val)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_ValueMapping", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
       # invisible( private$.member)
      } else {
          private$.member
      }
    }

  ),

  private = list(
    .index = NULL,
    .member = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        index = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        member = c(
          min="1", 
          max="1", 
          datatype = "DDI4_ValueMapping", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ValueMappingIndicator


#' @title DDI4_ValueMappingRelation  Variable relationships in PhysicalLayout as defined using ValueMappingRelation
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ValueMappingRelation}{ValueMappingRelation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Variable relationships in PhysicalLayout as defined using ValueMappingRelation
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ValueMappingRelation$DdiUrn} returns the DDI URN of myDDI4_ValueMappingRelation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an ennumeration list
#'
#'                              \code{myDDI4_ValueMappingRelation$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_ValueMappingRelation' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_ValueMappingRelation$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_ValueMappingRelation$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_ValueMappingRelation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_ValueMappingRelation$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection.
#'
#'                              \code{myDDI4_ValueMappingRelation$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_ValueMappingRelation' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_ValueMappingRelation$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          Specialization of source to VariableMapping. The First Variableapping in the pair.
#'
#'                              \code{myDDI4_ValueMappingRelation$source <- c(myValueMappingURN)} sets the association 'source' from 'myDDI4_ValueMappingRelation' to a character vector of DdiUrns for R6 ValueMapping class instances
#'
#'                              \code{myDDI4_ValueMappingRelation$source} returns a character vector of of DdiUrns for R6  ValueMapping class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          Specialization of target to VariableMapping. Restricts cardinality. The following VariableMapping in the pair.
#'
#'                              \code{myDDI4_ValueMappingRelation$target <- c(myValueMappingURN)} sets the association 'target' from 'myDDI4_ValueMappingRelation' to a character vector of DdiUrns for R6 ValueMapping class instances
#'
#'                              \code{myDDI4_ValueMappingRelation$target} returns a character vector of of DdiUrns for R6  ValueMapping class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ValueMappingRelation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ValueMappingRelation <- R6Class(classname="DDI4_ValueMappingRelation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ValueMappingRelation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_ValueMapping", proposedUrnVector=source , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_ValueMapping", proposedUrnVector=target , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_ValueMapping", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_ValueMapping", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="1", 
          max="1", 
          datatype = "DDI4_ValueMapping", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="1", 
          max="1", 
          datatype = "DDI4_ValueMapping", 
          structure = "DdiUrn vector"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ValueMappingRelation


#' @title DDI4_ValueString  The Value expressed as an xs:string with the ability to preserve whitespace if critical to the understanding of the content
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ValueString}{ValueString} 

#'
#'   Definition
#'
#'   ============
#'
#'   The Value expressed as an xs:string with the ability to preserve whitespace if critical to the understanding of the content. 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:ValueType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ValueString$DdiUrn} returns the DDI URN of myDDI4_ValueString.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{content}}{ A read/write active binding.
#'        The actual content of this value as a string
#'
#'                              \code{myDDI4_ValueString$content <- mycharacter} sets the content property of 'myDDI4_ValueString' to the vector of datatype:character
#'
#'                              \code{myDDI4_ValueString$content} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{whiteSpace}}{ A read/write active binding.
#'        The usual setting "collapse" states that leading and trailing white space will be removed and multiple adjacent white spaces will be treated as a single white space. When setting to "replace" all occurrences of #x9 (tab), #xA (line feed) and #xD (carriage return) are replaced with #x20 (space) but leading and trailing spaces will be retained. If the existence of any of these white spaces is critical to the understanding of the content, change the value of this attribute to "preserve".
#'
#'                              \code{myDDI4_ValueString$whiteSpace <- list(myWhiteSpaceRule)} sets the 'whiteSpace' property of 'myDDI4_ValueString' to a list of R6 WhiteSpaceRule class instances
#'
#'                              \code{myDDI4_ValueString$whiteSpace} returns a list of of R6 : WhiteSpaceRule class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ValueString$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, content=NA, whiteSpace=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ValueString <- R6Class(classname="DDI4_ValueString",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, content=NA, whiteSpace=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ValueString may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(content)){
        private$.content <- returnVectorIfValid(validVectorClass="character", proposedVector=content , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(whiteSpace)){
        private$.whiteSpace <- returnEnumeratedValues(enumerationObject=DDI4_WhiteSpaceRule$new(registerThisObject=FALSE), proposedCharacterVector=whiteSpace , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    content = function(val){
      if (!missing(val)){
        private$.content <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.content
      }
    },

    whiteSpace = function(val){
      if (!missing(val)){
        private$.whiteSpace <- returnEnumeratedValues(enumerationObject=DDI4_WhiteSpaceRule$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.whiteSpace
      }
    }


# There are 0 class association(s):

  ),

  private = list(
    .content = NULL,
    .whiteSpace = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        content = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        whiteSpace = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WhiteSpaceRule", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_ValueString


#' @title DDI4_VariableCollection  A collection (group) of any type of Variable within the Variable Cascade (Conceptual, Represented, Instance) for purposes of management, conceptualization or anything other than organizing a logical record or physical layout
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/VariableCollection}{VariableCollection} 

#'
#'   Definition
#'
#'   ============
#'
#'   A collection (group) of any type of Variable within the Variable Cascade (Conceptual, Represented, Instance) for purposes of management, conceptualization or anything other than organizing a logical record or physical layout.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Variables within a specific section of a study, Variables related to a specific subject or keyword. Variables at a specified level of development or review.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   A simple ordered or unordered list of variables can be described via a set of VariableIndicator parameters. An optional VariableRelationStructure can describe a more complex structure for the collection. We might, for example, use the VariableRelationStructure to group variables by content within a Study or across a StudySeries.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   l:VariableGroupType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_VariableCollection$DdiUrn} returns the DDI URN of myDDI4_VariableCollection.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{type}}{ A read/write active binding.
#'        Whether the collection is a bag or a set: a bag is a collection with duplicates allowed, a set is a collection without duplicates.
#'
#'                              \code{myDDI4_VariableCollection$type <- list(myCollectionType)} sets the 'type' property of 'myDDI4_VariableCollection' to a list of R6 CollectionType class instances
#'
#'                              \code{myDDI4_VariableCollection$type} returns a list of of R6 : CollectionType class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage.
#'
#'                              \code{myDDI4_VariableCollection$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_VariableCollection' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_VariableCollection$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_VariableCollection$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_VariableCollection' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_VariableCollection$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A structured display label providing a fully human readable display for the identification of the object. Supports the use of multiple languages and structured text. 
#'
#'                              \code{myDDI4_VariableCollection$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_VariableCollection' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_VariableCollection$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#'   \describe{
#'     \item{\code{usage}}{ A read/write active binding.
#'        Explanation of the ways in which some decision or object is employed. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_VariableCollection$usage <- list(myInternationalStructuredString)} sets the 'usage' property of 'myDDI4_VariableCollection' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_VariableCollection$usage} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{groupingSemantic}}{ A read/write active binding.
#'        A semantic term defining the factor used for defining this group
#'
#'                              \code{myDDI4_VariableCollection$groupingSemantic <- list(myExternalControlledVocabularyEntry)} sets the 'groupingSemantic' property of 'myDDI4_VariableCollection' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_VariableCollection$groupingSemantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        Allows for the identification of the member and optionally provides an index for the member within an ordered array
#'
#'                              \code{myDDI4_VariableCollection$contains <- list(myVariableIndicator)} sets the 'contains' property of 'myDDI4_VariableCollection' to a list of R6 VariableIndicator class instances
#'
#'                              \code{myDDI4_VariableCollection$contains} returns a list of of R6 : VariableIndicator class instances}
#'   }
#'   \describe{
#'     \item{\code{isOrdered}}{ A read/write active binding.
#'        If members are ordered set to true, if unordered set to false.
#'
#'                              \code{myDDI4_VariableCollection$isOrdered <- mylogical} sets the isOrdered property of 'myDDI4_VariableCollection' to the vector of datatype:logical
#'
#'                              \code{myDDI4_VariableCollection$isOrdered} returns a vector of datatype: logical}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isStructuredBy}}{ A read/write active binding.
#'          This supplies information for linking disparate records in a dataset. RecordRelations describe corresponding InstanceVariables between record types.
#'
#'                              \code{myDDI4_VariableCollection$isStructuredBy <- c(myVariableRelationStructureURN)} sets the association 'isStructuredBy' from 'myDDI4_VariableCollection' to a character vector of DdiUrns for R6 VariableRelationStructure class instances
#'
#'                              \code{myDDI4_VariableCollection$isStructuredBy} returns a character vector of of DdiUrns for R6  VariableRelationStructure class instances}
#'   }
#'   \describe{
#'     \item{\code{definingConcept}}{ A read/write active binding.
#'          The conceptual basis for the collection of members.
#'
#'                              \code{myDDI4_VariableCollection$definingConcept <- c(myConceptURN)} sets the association 'definingConcept' from 'myDDI4_VariableCollection' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_VariableCollection$definingConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_VariableCollection$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, type=NA, name=NA, purpose=NA, displayLabel=NA, usage=NA, groupingSemantic=NA, contains=NA, isOrdered=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_VariableCollection <- R6Class(classname="DDI4_VariableCollection",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, type=NA, name=NA, purpose=NA, displayLabel=NA, usage=NA, groupingSemantic=NA, contains=NA, isOrdered=NA,    isStructuredBy=NA,definingConcept=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_VariableCollection may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(type)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=type , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(usage)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=usage , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(groupingSemantic)){
        private$.groupingSemantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=groupingSemantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_VariableIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(isOrdered)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=isOrdered , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(isStructuredBy)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_VariableRelationStructure", proposedUrnVector=isStructuredBy , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
      }
      if(!missing(definingConcept)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=definingConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 8 properties: 
    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_VariableIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },

    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },

    groupingSemantic = function(val){
      if (!missing(val)){
        private$.groupingSemantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.groupingSemantic
      }
    },

    isOrdered = function(val){
      if (!missing(val)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isOrdered
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    type = function(val){
      if (!missing(val)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.type
      }
    },

    usage = function(val){
      if (!missing(val)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.usage
      }
    },


# There are 2 class association(s):
    definingConcept = function(val){
      if (!missing(val)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
       # invisible( private$.definingConcept)
      } else {
          private$.definingConcept
      }
    },

    isStructuredBy = function(val){
      if (!missing(val)){
        private$.isStructuredBy <- returnUrnVectorIfValid(validClass="DDI4_VariableRelationStructure", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isStructuredBy, referencingAssociationName="isStructuredBy",  referencingUrn=thisUrn)  
       # invisible( private$.isStructuredBy)
      } else {
          private$.isStructuredBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .type = NULL,
    .name = NULL,
    .purpose = NULL,
    .displayLabel = NULL,
    .usage = NULL,
    .groupingSemantic = NULL,
    .contains = NULL,
    .isOrdered = NULL,
    .isStructuredBy = NULL,
    .definingConcept = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_VariableIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        groupingSemantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isStructuredBy = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_VariableRelationStructure", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        usage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_VariableCollection


#' @title DDI4_VariableIndicator  Member Indicator for use with member type ConceptualVariable, RepresentedVariable, and InstanceVariable
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/VariableIndicator}{VariableIndicator} 

#'
#'   Definition
#'
#'   ============
#'
#'   Member Indicator for use with member type ConceptualVariable, RepresentedVariable, and InstanceVariable
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_VariableIndicator$DdiUrn} returns the DDI URN of myDDI4_VariableIndicator.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Index value of member in an ordered array
#'
#'                              \code{myDDI4_VariableIndicator$index <- myinteger} sets the index property of 'myDDI4_VariableIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_VariableIndicator$index} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{member}}{ A read/write active binding.
#'          Restricts member target class to ConceptualVariable, RepresentedVariable, and InstanceVariable
#'
#'                              \code{myDDI4_VariableIndicator$member <- c(myConceptualVariableURN)} sets the association 'member' from 'myDDI4_VariableIndicator' to a character vector of DdiUrns for R6 ConceptualVariable class instances
#'
#'                              \code{myDDI4_VariableIndicator$member} returns a character vector of of DdiUrns for R6  ConceptualVariable class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_VariableIndicator$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_VariableIndicator <- R6Class(classname="DDI4_VariableIndicator",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA,    member=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_VariableIndicator may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(member)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_ConceptualVariable", proposedUrnVector=member , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.index
      }
    },


# There are 1 class association(s):
    member = function(val){
      if (!missing(val)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_ConceptualVariable", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
       # invisible( private$.member)
      } else {
          private$.member
      }
    }

  ),

  private = list(
    .index = NULL,
    .member = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        index = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        member = c(
          min="1", 
          max="1", 
          datatype = "DDI4_ConceptualVariable", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_VariableIndicator


#' @title DDI4_VariableRelation  Ordered relations between any Variables in the Variable Cascade (Conceptual, Represented, Instance)
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/VariableRelation}{VariableRelation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Ordered relations between any Variables in the Variable Cascade (Conceptual, Represented, Instance)
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_VariableRelation$DdiUrn} returns the DDI URN of myDDI4_VariableRelation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an ennumeration list
#'
#'                              \code{myDDI4_VariableRelation$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_VariableRelation' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_VariableRelation$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_VariableRelation$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_VariableRelation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_VariableRelation$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection. 
#'
#'                              \code{myDDI4_VariableRelation$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_VariableRelation' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_VariableRelation$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          First member in a relationship.

#'
#'                              \code{myDDI4_VariableRelation$source <- c(myConceptualVariableURN)} sets the association 'source' from 'myDDI4_VariableRelation' to a character vector of DdiUrns for R6 ConceptualVariable class instances
#'
#'                              \code{myDDI4_VariableRelation$source} returns a character vector of of DdiUrns for R6  ConceptualVariable class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          Second member in a relationship. Note that this can be realized as a collection to support tuples. 
#'
#'                              \code{myDDI4_VariableRelation$target <- c(myConceptualVariableURN)} sets the association 'target' from 'myDDI4_VariableRelation' to a character vector of DdiUrns for R6 ConceptualVariable class instances
#'
#'                              \code{myDDI4_VariableRelation$target} returns a character vector of of DdiUrns for R6  ConceptualVariable class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_VariableRelation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_VariableRelation <- R6Class(classname="DDI4_VariableRelation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_VariableRelation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_ConceptualVariable", proposedUrnVector=source , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_ConceptualVariable", proposedUrnVector=target , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_ConceptualVariable", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_ConceptualVariable", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ConceptualVariable", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ConceptualVariable", 
          structure = "DdiUrn vector"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_VariableRelation


#' @title DDI4_VariableRelationStructure  RelationStructure for use with any mixture of Variables in the Variable Cascade (Conceptual, Represented, Instance)
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/VariableRelationStructure}{VariableRelationStructure} 

#'
#'   Definition
#'
#'   ============
#'
#'   RelationStructure for use with any mixture of Variables in the Variable Cascade (Conceptual, Represented, Instance).
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   A VariableCollection can be described as a simple list, or as having a more complex structure. All of the "administrative" variables for a study might be described as just a simple list without a VariableRelationStructure. If, however, the administrative variables needed to be described in subgroups, a VariableRelationStructure could be employed.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_VariableRelationStructure$DdiUrn} returns the DDI URN of myDDI4_VariableRelationStructure.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an ennumeration list.  Use if all relations within this relation structure are of the same specification.
#'
#'                              \code{myDDI4_VariableRelationStructure$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_VariableRelationStructure' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_VariableRelationStructure$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_VariableRelationStructure$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_VariableRelationStructure' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_VariableRelationStructure$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection. 
#'
#'                              \code{myDDI4_VariableRelationStructure$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_VariableRelationStructure' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_VariableRelationStructure$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMemberRelation}}{ A read/write active binding.
#'        Member relations that comprise the relation structure
#'
#'                              \code{myDDI4_VariableRelationStructure$hasMemberRelation <- list(myVariableRelation)} sets the 'hasMemberRelation' property of 'myDDI4_VariableRelationStructure' to a list of R6 VariableRelation class instances
#'
#'                              \code{myDDI4_VariableRelationStructure$hasMemberRelation} returns a list of of R6 : VariableRelation class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_VariableRelationStructure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_VariableRelationStructure <- R6Class(classname="DDI4_VariableRelationStructure",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_VariableRelationStructure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasMemberRelation)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_VariableRelation", proposedObjectList=hasMemberRelation , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    hasMemberRelation = function(val){
      if (!missing(val)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_VariableRelation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasMemberRelation
      }
    },

    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .hasMemberRelation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasMemberRelation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_VariableRelation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasRelationSpecification = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_VariableRelationStructure


#' @title DDI4_VariableStatistics  Contains summary and category level statistics for the referenced variable
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/VariableStatistics}{VariableStatistics} 

#'
#'   Definition
#'
#'   ============
#'
#'   Contains summary and category level statistics for the referenced variable.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A mean of values for a variable "Height". Counts for each level of variable "Gender" (male and female)
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'    Includes information on the total number of responses, the weights in calculating the statistics, variable level summary statistics, and category statistics. The category statistics may be provided as unfiltered values or filtered through a single variable. For example the category statistics for Sex filtered by the variable Country for a multi-national data file. Note that if no weighting factor is identified, all of the statistics provided are unweighted.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   pi:VariableStatisticsType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_VariableStatistics$DdiUrn} returns the DDI URN of myDDI4_VariableStatistics.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{totalResponses}}{ A read/write active binding.
#'        The total number of responses to this variable. This element is especially useful if the number of responses does not match added case counts. It may also be used to sum the frequencies for variable categories.
#'
#'                              \code{myDDI4_VariableStatistics$totalResponses <- myinteger} sets the totalResponses property of 'myDDI4_VariableStatistics' to the vector of datatype:integer
#'
#'                              \code{myDDI4_VariableStatistics$totalResponses} returns a vector of datatype: integer}
#'   }
#'   \describe{
#'     \item{\code{hasSummaryStatistic}}{ A read/write active binding.
#'        Statistic that provides a summary value, filtered or unfiltered, weighted or unweighted, expressed as an xs:decimal or xs:double
#'
#'                              \code{myDDI4_VariableStatistics$hasSummaryStatistic <- list(mySummaryStatistic)} sets the 'hasSummaryStatistic' property of 'myDDI4_VariableStatistics' to a list of R6 SummaryStatistic class instances
#'
#'                              \code{myDDI4_VariableStatistics$hasSummaryStatistic} returns a list of of R6 : SummaryStatistic class instances}
#'   }
#'   \describe{
#'     \item{\code{hasCategoryStatistic}}{ A read/write active binding.
#'        Statistics that provides a category specific values, filtered or unfiltered, weighted or unweighted, expressed as an xs:decimal or xs:double
#'
#'                              \code{myDDI4_VariableStatistics$hasCategoryStatistic <- list(myCategoryStatistic)} sets the 'hasCategoryStatistic' property of 'myDDI4_VariableStatistics' to a list of R6 CategoryStatistic class instances
#'
#'                              \code{myDDI4_VariableStatistics$hasCategoryStatistic} returns a list of of R6 : CategoryStatistic class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{forInstanceVariable}}{ A read/write active binding.
#'          Reference to the variable to which the statistics apply.
#'
#'                              \code{myDDI4_VariableStatistics$forInstanceVariable <- c(myInstanceVariableURN)} sets the association 'forInstanceVariable' from 'myDDI4_VariableStatistics' to a character vector of DdiUrns for R6 InstanceVariable class instances
#'
#'                              \code{myDDI4_VariableStatistics$forInstanceVariable} returns a character vector of of DdiUrns for R6  InstanceVariable class instances}
#'   }
#'   \describe{
#'     \item{\code{usesStandardWeight}}{ A read/write active binding.
#'          Reference to the StandardWeight value provided in Weighting.
#'
#'                              \code{myDDI4_VariableStatistics$usesStandardWeight <- c(myStandardWeightURN)} sets the association 'usesStandardWeight' from 'myDDI4_VariableStatistics' to a character vector of DdiUrns for R6 StandardWeight class instances
#'
#'                              \code{myDDI4_VariableStatistics$usesStandardWeight} returns a character vector of of DdiUrns for R6  StandardWeight class instances}
#'   }
#'   \describe{
#'     \item{\code{applicableWeightVariable}}{ A read/write active binding.
#'          Reference to a variable to use for weight in calculating the statistic.
#'
#'                              \code{myDDI4_VariableStatistics$applicableWeightVariable <- c(myInstanceVariableURN)} sets the association 'applicableWeightVariable' from 'myDDI4_VariableStatistics' to a character vector of DdiUrns for R6 InstanceVariable class instances
#'
#'                              \code{myDDI4_VariableStatistics$applicableWeightVariable} returns a character vector of of DdiUrns for R6  InstanceVariable class instances}
#'   }
#'   \describe{
#'     \item{\code{hasFilterVariable}}{ A read/write active binding.
#'          A variable that may be used to filter the summary and/or category statistics (for example by country or by gender)
#'
#'                              \code{myDDI4_VariableStatistics$hasFilterVariable <- c(myInstanceVariableURN)} sets the association 'hasFilterVariable' from 'myDDI4_VariableStatistics' to a character vector of DdiUrns for R6 InstanceVariable class instances
#'
#'                              \code{myDDI4_VariableStatistics$hasFilterVariable} returns a character vector of of DdiUrns for R6  InstanceVariable class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_VariableStatistics$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, totalResponses=NA, hasSummaryStatistic=NA, hasCategoryStatistic=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_VariableStatistics <- R6Class(classname="DDI4_VariableStatistics",
  inherit = DDI4_Identifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, totalResponses=NA, hasSummaryStatistic=NA, hasCategoryStatistic=NA,    forInstanceVariable=NA,usesStandardWeight=NA,applicableWeightVariable=NA,hasFilterVariable=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_VariableStatistics may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(totalResponses)){
        private$.totalResponses <- returnVectorIfValid(validVectorClass="integer", proposedVector=totalResponses , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasSummaryStatistic)){
        private$.hasSummaryStatistic <- returnR6ObjectIfValid(validClass="DDI4_SummaryStatistic", proposedObjectList=hasSummaryStatistic , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(hasCategoryStatistic)){
        private$.hasCategoryStatistic <- returnR6ObjectIfValid(validClass="DDI4_CategoryStatistic", proposedObjectList=hasCategoryStatistic , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(forInstanceVariable)){
        private$.forInstanceVariable <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=forInstanceVariable , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.forInstanceVariable, referencingAssociationName="forInstanceVariable",  referencingUrn=thisUrn)  
      }
      if(!missing(usesStandardWeight)){
        private$.usesStandardWeight <- returnUrnVectorIfValid(validClass="DDI4_StandardWeight", proposedUrnVector=usesStandardWeight , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesStandardWeight, referencingAssociationName="usesStandardWeight",  referencingUrn=thisUrn)  
      }
      if(!missing(applicableWeightVariable)){
        private$.applicableWeightVariable <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=applicableWeightVariable , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.applicableWeightVariable, referencingAssociationName="applicableWeightVariable",  referencingUrn=thisUrn)  
      }
      if(!missing(hasFilterVariable)){
        private$.hasFilterVariable <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=hasFilterVariable , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasFilterVariable, referencingAssociationName="hasFilterVariable",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    hasCategoryStatistic = function(val){
      if (!missing(val)){
        private$.hasCategoryStatistic <- returnR6ObjectIfValid(validClass="DDI4_CategoryStatistic", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasCategoryStatistic
      }
    },

    hasSummaryStatistic = function(val){
      if (!missing(val)){
        private$.hasSummaryStatistic <- returnR6ObjectIfValid(validClass="DDI4_SummaryStatistic", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasSummaryStatistic
      }
    },

    totalResponses = function(val){
      if (!missing(val)){
        private$.totalResponses <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totalResponses
      }
    },


# There are 4 class association(s):
    applicableWeightVariable = function(val){
      if (!missing(val)){
        private$.applicableWeightVariable <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.applicableWeightVariable, referencingAssociationName="applicableWeightVariable",  referencingUrn=thisUrn)  
       # invisible( private$.applicableWeightVariable)
      } else {
          private$.applicableWeightVariable
      }
    },

    forInstanceVariable = function(val){
      if (!missing(val)){
        private$.forInstanceVariable <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.forInstanceVariable, referencingAssociationName="forInstanceVariable",  referencingUrn=thisUrn)  
       # invisible( private$.forInstanceVariable)
      } else {
          private$.forInstanceVariable
      }
    },

    hasFilterVariable = function(val){
      if (!missing(val)){
        private$.hasFilterVariable <- returnUrnVectorIfValid(validClass="DDI4_InstanceVariable", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasFilterVariable, referencingAssociationName="hasFilterVariable",  referencingUrn=thisUrn)  
       # invisible( private$.hasFilterVariable)
      } else {
          private$.hasFilterVariable
      }
    },

    usesStandardWeight = function(val){
      if (!missing(val)){
        private$.usesStandardWeight <- returnUrnVectorIfValid(validClass="DDI4_StandardWeight", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.usesStandardWeight, referencingAssociationName="usesStandardWeight",  referencingUrn=thisUrn)  
       # invisible( private$.usesStandardWeight)
      } else {
          private$.usesStandardWeight
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .totalResponses = NULL,
    .hasSummaryStatistic = NULL,
    .hasCategoryStatistic = NULL,
    .forInstanceVariable = NULL,
    .usesStandardWeight = NULL,
    .applicableWeightVariable = NULL,
    .hasFilterVariable = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        applicableWeightVariable = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InstanceVariable", 
          structure = "DdiUrn vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        forInstanceVariable = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InstanceVariable", 
          structure = "DdiUrn vector"
        ),
        hasCategoryStatistic = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_CategoryStatistic", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasFilterVariable = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InstanceVariable", 
          structure = "DdiUrn vector"
        ),
        hasSummaryStatistic = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_SummaryStatistic", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        totalResponses = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        usesStandardWeight = c(
          min="0", 
          max="1", 
          datatype = "DDI4_StandardWeight", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_VariableStatistics


#' @title DDI4_VideoSegment  Describes the type and length of the video segment
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/VideoSegment}{VideoSegment} 

#'
#'   Definition
#'
#'   ============
#'
#'   Describes the type and length of the video segment.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   r:VideoType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_VideoSegment$DdiUrn} returns the DDI URN of myDDI4_VideoSegment.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{typeOfVideoClip}}{ A read/write active binding.
#'        The type of video clip provided. Supports the use of a controlled vocabulary.
#'
#'                              \code{myDDI4_VideoSegment$typeOfVideoClip <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfVideoClip' property of 'myDDI4_VideoSegment' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_VideoSegment$typeOfVideoClip} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{videoClipBegin}}{ A read/write active binding.
#'        The point to begin the video clip. If no point is provided the assumption is that the start point is the beginning of the clip provided.
#'
#'                              \code{myDDI4_VideoSegment$videoClipBegin <- mycharacter} sets the videoClipBegin property of 'myDDI4_VideoSegment' to the vector of datatype:character
#'
#'                              \code{myDDI4_VideoSegment$videoClipBegin} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{videoClipEnd}}{ A read/write active binding.
#'        The point to end the video clip. If no point is provided the assumption is that the end point is the end of the clip provided.
#'
#'                              \code{myDDI4_VideoSegment$videoClipEnd <- mycharacter} sets the videoClipEnd property of 'myDDI4_VideoSegment' to the vector of datatype:character
#'
#'                              \code{myDDI4_VideoSegment$videoClipEnd} returns a vector of datatype: character}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_VideoSegment$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, typeOfVideoClip=NA, videoClipBegin=NA, videoClipEnd=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_VideoSegment <- R6Class(classname="DDI4_VideoSegment",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, typeOfVideoClip=NA, videoClipBegin=NA, videoClipEnd=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_VideoSegment may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(typeOfVideoClip)){
        private$.typeOfVideoClip <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfVideoClip , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(videoClipBegin)){
        private$.videoClipBegin <- returnVectorIfValid(validVectorClass="character", proposedVector=videoClipBegin , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(videoClipEnd)){
        private$.videoClipEnd <- returnVectorIfValid(validVectorClass="character", proposedVector=videoClipEnd , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 3 properties: 
    typeOfVideoClip = function(val){
      if (!missing(val)){
        private$.typeOfVideoClip <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.typeOfVideoClip
      }
    },

    videoClipBegin = function(val){
      if (!missing(val)){
        private$.videoClipBegin <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.videoClipBegin
      }
    },

    videoClipEnd = function(val){
      if (!missing(val)){
        private$.videoClipEnd <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.videoClipEnd
      }
    }


  ),

  private = list(
    .typeOfVideoClip = NULL,
    .videoClipBegin = NULL,
    .videoClipEnd = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        typeOfVideoClip = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        videoClipBegin = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        videoClipEnd = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_VideoSegment


#' @title DDI4_ViewpointRole  A ViewpointRole designates the function an InstanceVariable performs in the context of the Viewpoint
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/ViewpointRole}{ViewpointRole} 

#'
#'   Definition
#'
#'   ============
#'
#'   A ViewpointRole designates the function an InstanceVariable performs in the context of the Viewpoint. (IdentifierRole, AttributeRole, or MeasureRole of interest).
#'
#'   
#'
#'   Each of three roles within a Viewpoint may be a collection. This happens when a role is mapped to multiple instance variables. In this event a role forms a SimpleCollection. There are SimpleCollections of instance variables in each role.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   See the Viewpoint documentation for an in depth discussion of the uses of ViewpointRoles: http://lion.ddialliance.org/ddiobjects/viewpoint
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_ViewpointRole$DdiUrn} returns the DDI URN of myDDI4_ViewpointRole.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{type}}{ A read/write active binding.
#'        Whether the collection is a bag or a set: a bag is a collection with duplicates allowed, a set is a collection without duplicates
#'
#'                              \code{myDDI4_ViewpointRole$type <- list(myCollectionType)} sets the 'type' property of 'myDDI4_ViewpointRole' to a list of R6 CollectionType class instances
#'
#'                              \code{myDDI4_ViewpointRole$type} returns a list of of R6 : CollectionType class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage.
#'
#'                              \code{myDDI4_ViewpointRole$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_ViewpointRole' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_ViewpointRole$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_ViewpointRole$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_ViewpointRole' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_ViewpointRole$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{contains}}{ A read/write active binding.
#'        Allows for the identification of the member and optionally provides an index for the member within an ordered array
#'
#'                              \code{myDDI4_ViewpointRole$contains <- list(myInstanceVariableIndicator)} sets the 'contains' property of 'myDDI4_ViewpointRole' to a list of R6 InstanceVariableIndicator class instances
#'
#'                              \code{myDDI4_ViewpointRole$contains} returns a list of of R6 : InstanceVariableIndicator class instances}
#'   }
#'   \describe{
#'     \item{\code{isOrdered}}{ A read/write active binding.
#'        If members are ordered set to true, if unordered set to false.
#'
#'                              \code{myDDI4_ViewpointRole$isOrdered <- mylogical} sets the isOrdered property of 'myDDI4_ViewpointRole' to the vector of datatype:logical
#'
#'                              \code{myDDI4_ViewpointRole$isOrdered} returns a vector of datatype: logical}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{definingConcept}}{ A read/write active binding.
#'          The conceptual basis for the collection of members.
#'
#'                              \code{myDDI4_ViewpointRole$definingConcept <- c(myConceptURN)} sets the association 'definingConcept' from 'myDDI4_ViewpointRole' to a character vector of DdiUrns for R6 Concept class instances
#'
#'                              \code{myDDI4_ViewpointRole$definingConcept} returns a character vector of of DdiUrns for R6  Concept class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_ViewpointRole$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_ViewpointRole <- R6Class(classname="DDI4_ViewpointRole",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, type=NA, name=NA, purpose=NA, contains=NA, isOrdered=NA,    definingConcept=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_ViewpointRole may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(type)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=type , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(contains)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_InstanceVariableIndicator", proposedObjectList=contains , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(isOrdered)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=isOrdered , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(definingConcept)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=definingConcept , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 5 properties: 
    contains = function(val){
      if (!missing(val)){
        private$.contains <- returnR6ObjectIfValid(validClass="DDI4_InstanceVariableIndicator", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.contains
      }
    },

    isOrdered = function(val){
      if (!missing(val)){
        private$.isOrdered <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isOrdered
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    type = function(val){
      if (!missing(val)){
        private$.type <- returnEnumeratedValues(enumerationObject=DDI4_CollectionType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.type
      }
    },


# There are 1 class association(s):
    definingConcept = function(val){
      if (!missing(val)){
        private$.definingConcept <- returnUrnVectorIfValid(validClass="DDI4_Concept", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.definingConcept, referencingAssociationName="definingConcept",  referencingUrn=thisUrn)  
       # invisible( private$.definingConcept)
      } else {
          private$.definingConcept
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .type = NULL,
    .name = NULL,
    .purpose = NULL,
    .contains = NULL,
    .isOrdered = NULL,
    .definingConcept = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        contains = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InstanceVariableIndicator", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definingConcept = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Concept", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isOrdered = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        type = c(
          min="0", 
          max="1", 
          datatype = "DDI4_CollectionType", 
          structure = "Enumeration vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_ViewpointRole


#' @title DDI4_VocabularyEntry  One entry term and its definition in an ordered list comprising a controlled vocabulary
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/VocabularyEntry}{VocabularyEntry} 

#'
#'   Definition
#'
#'   ============
#'
#'   One entry term and its definition in an ordered list comprising a controlled vocabulary.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   aunt - The female sibling of a parent
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_VocabularyEntry$DdiUrn} returns the DDI URN of myDDI4_VocabularyEntry.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{entryTerm}}{ A read/write active binding.
#'        the term being defined
#'
#'                              \code{myDDI4_VocabularyEntry$entryTerm <- list(myInternationalStructuredString)} sets the 'entryTerm' property of 'myDDI4_VocabularyEntry' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_VocabularyEntry$entryTerm} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{definition}}{ A read/write active binding.
#'        An explanation (definition) of the value
#'
#'                              \code{myDDI4_VocabularyEntry$definition <- list(myInternationalStructuredString)} sets the 'definition' property of 'myDDI4_VocabularyEntry' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_VocabularyEntry$definition} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_VocabularyEntry$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, entryTerm=NA, definition=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_VocabularyEntry <- R6Class(classname="DDI4_VocabularyEntry",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, entryTerm=NA, definition=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_VocabularyEntry may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(entryTerm)){
        private$.entryTerm <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=entryTerm , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(definition)){
        private$.definition <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=definition , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 2 properties: 
    definition = function(val){
      if (!missing(val)){
        private$.definition <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.definition
      }
    },

    entryTerm = function(val){
      if (!missing(val)){
        private$.entryTerm <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.entryTerm
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .entryTerm = NULL,
    .definition = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        definition = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        entryTerm = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_VocabularyEntry


#' @title DDI4_VocabularyEntryIndicator  Member Indicator for use with member type VocabularyEntry
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/VocabularyEntryIndicator}{VocabularyEntryIndicator} 

#'
#'   Definition
#'
#'   ============
#'
#'   Member Indicator for use with member type VocabularyEntry
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_VocabularyEntryIndicator$DdiUrn} returns the DDI URN of myDDI4_VocabularyEntryIndicator.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Index value of member in an ordered array
#'
#'                              \code{myDDI4_VocabularyEntryIndicator$index <- myinteger} sets the index property of 'myDDI4_VocabularyEntryIndicator' to the vector of datatype:integer
#'
#'                              \code{myDDI4_VocabularyEntryIndicator$index} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{member}}{ A read/write active binding.
#'          Restricts member target class to VocabularyEntry
#'
#'                              \code{myDDI4_VocabularyEntryIndicator$member <- c(myVocabularyEntryURN)} sets the association 'member' from 'myDDI4_VocabularyEntryIndicator' to a character vector of DdiUrns for R6 VocabularyEntry class instances
#'
#'                              \code{myDDI4_VocabularyEntryIndicator$member} returns a character vector of of DdiUrns for R6  VocabularyEntry class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_VocabularyEntryIndicator$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_VocabularyEntryIndicator <- R6Class(classname="DDI4_VocabularyEntryIndicator",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA,    member=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_VocabularyEntryIndicator may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(member)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_VocabularyEntry", proposedUrnVector=member , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.index
      }
    },


# There are 1 class association(s):
    member = function(val){
      if (!missing(val)){
        private$.member <- returnUrnVectorIfValid(validClass="DDI4_VocabularyEntry", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.member, referencingAssociationName="member",  referencingUrn=thisUrn)  
       # invisible( private$.member)
      } else {
          private$.member
      }
    }

  ),

  private = list(
    .index = NULL,
    .member = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        index = c(
          min="0", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        member = c(
          min="1", 
          max="1", 
          datatype = "DDI4_VocabularyEntry", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_VocabularyEntryIndicator


#' @title DDI4_VocabularyEntryRelation  Relation of Vocabulary Entries as defined using RelationSpecification 
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/VocabularyEntryRelation}{VocabularyEntryRelation} 

#'
#'   Definition
#'
#'   ============
#'
#'   Relation of Vocabulary Entries as defined using RelationSpecification 
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_VocabularyEntryRelation$DdiUrn} returns the DDI URN of myDDI4_VocabularyEntryRelation.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an enumeration list
#'
#'                              \code{myDDI4_VocabularyEntryRelation$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_VocabularyEntryRelation' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_VocabularyEntryRelation$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provide a semantic that provides a context for the relationship using and External Controlled Vocabulary
#'
#'                              \code{myDDI4_VocabularyEntryRelation$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_VocabularyEntryRelation' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_VocabularyEntryRelation$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of its totality using an enumeration list.
#'
#'                              \code{myDDI4_VocabularyEntryRelation$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_VocabularyEntryRelation' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_VocabularyEntryRelation$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{source}}{ A read/write active binding.
#'          Restricts source to VocabularyEntry
#'
#'                              \code{myDDI4_VocabularyEntryRelation$source <- c(myVocabularyEntryURN)} sets the association 'source' from 'myDDI4_VocabularyEntryRelation' to a character vector of DdiUrns for R6 VocabularyEntry class instances
#'
#'                              \code{myDDI4_VocabularyEntryRelation$source} returns a character vector of of DdiUrns for R6  VocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{target}}{ A read/write active binding.
#'          Restricts target to VocabularyEntry and cardinality.
#'
#'                              \code{myDDI4_VocabularyEntryRelation$target <- c(myVocabularyEntryURN)} sets the association 'target' from 'myDDI4_VocabularyEntryRelation' to a character vector of DdiUrns for R6 VocabularyEntry class instances
#'
#'                              \code{myDDI4_VocabularyEntryRelation$target} returns a character vector of of DdiUrns for R6  VocabularyEntry class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_VocabularyEntryRelation$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_VocabularyEntryRelation <- R6Class(classname="DDI4_VocabularyEntryRelation",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, hasRelationSpecification=NA, semantic=NA, totality=NA,    source=NA,target=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_VocabularyEntryRelation may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=1, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(source)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_VocabularyEntry", proposedUrnVector=source , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
      }
      if(!missing(target)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_VocabularyEntry", proposedUrnVector=target , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },


# There are 2 class association(s):
    source = function(val){
      if (!missing(val)){
        private$.source <- returnUrnVectorIfValid(validClass="DDI4_VocabularyEntry", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.source, referencingAssociationName="source",  referencingUrn=thisUrn)  
       # invisible( private$.source)
      } else {
          private$.source
      }
    },

    target = function(val){
      if (!missing(val)){
        private$.target <- returnUrnVectorIfValid(validClass="DDI4_VocabularyEntry", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.target, referencingAssociationName="target",  referencingUrn=thisUrn)  
       # invisible( private$.target)
      } else {
          private$.target
      }
    }

  ),

  private = list(
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .source = NULL,
    .target = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        hasRelationSpecification = c(
          min="1", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        source = c(
          min="1", 
          max="1", 
          datatype = "DDI4_VocabularyEntry", 
          structure = "DdiUrn vector"
        ),
        target = c(
          min="1", 
          max="1", 
          datatype = "DDI4_VocabularyEntry", 
          structure = "DdiUrn vector"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_VocabularyEntryRelation


#' @title DDI4_VocabularyRelationStructure  Contains the Vocabulary Relations used to structure the ControlledVocabulary
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/VocabularyRelationStructure}{VocabularyRelationStructure} 

#'
#'   Definition
#'
#'   ============
#'
#'   Contains the Vocabulary Relations used to structure the ControlledVocabulary
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_VocabularyRelationStructure$DdiUrn} returns the DDI URN of myDDI4_VocabularyRelationStructure.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{criteria}}{ A read/write active binding.
#'        Intentional definition of the order criteria (e.g. alphabetical, numerical, increasing, decreasing, etc.)
#'
#'                              \code{myDDI4_VocabularyRelationStructure$criteria <- list(myInternationalStructuredString)} sets the 'criteria' property of 'myDDI4_VocabularyRelationStructure' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_VocabularyRelationStructure$criteria} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A display label for the OrderRelation. May be expressed in multiple languages. Repeat for labels with different content, for example, labels with differing length limitations.
#'
#'                              \code{myDDI4_VocabularyRelationStructure$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_VocabularyRelationStructure' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_VocabularyRelationStructure$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#'   \describe{
#'     \item{\code{usage}}{ A read/write active binding.
#'        Explanation of the ways in which some decision or object is employed. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_VocabularyRelationStructure$usage <- list(myInternationalStructuredString)} sets the 'usage' property of 'myDDI4_VocabularyRelationStructure' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_VocabularyRelationStructure$usage} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{hasRelationSpecification}}{ A read/write active binding.
#'        Provides information on reflexivity, transitivity, and symmetry of relationship using a descriptive term from an enumeration list.  Use if all relations within this relation structure are of the same specification.
#'
#'                              \code{myDDI4_VocabularyRelationStructure$hasRelationSpecification <- list(myRelationSpecification)} sets the 'hasRelationSpecification' property of 'myDDI4_VocabularyRelationStructure' to a list of R6 RelationSpecification class instances
#'
#'                              \code{myDDI4_VocabularyRelationStructure$hasRelationSpecification} returns a list of of R6 : RelationSpecification class instances}
#'   }
#'   \describe{
#'     \item{\code{semantic}}{ A read/write active binding.
#'        Provides semantic context for the relationship
#'
#'                              \code{myDDI4_VocabularyRelationStructure$semantic <- list(myExternalControlledVocabularyEntry)} sets the 'semantic' property of 'myDDI4_VocabularyRelationStructure' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_VocabularyRelationStructure$semantic} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{totality}}{ A read/write active binding.
#'        Type of relation in terms of totality with respect to an associated collection. 
#'
#'                              \code{myDDI4_VocabularyRelationStructure$totality <- list(myTotalityType)} sets the 'totality' property of 'myDDI4_VocabularyRelationStructure' to a list of R6 TotalityType class instances
#'
#'                              \code{myDDI4_VocabularyRelationStructure$totality} returns a list of of R6 : TotalityType class instances}
#'   }
#'   \describe{
#'     \item{\code{hasMemberRelation}}{ A read/write active binding.
#'        Relation between VocabularyEntries
#'
#'                              \code{myDDI4_VocabularyRelationStructure$hasMemberRelation <- list(myVocabularyEntryRelation)} sets the 'hasMemberRelation' property of 'myDDI4_VocabularyRelationStructure' to a list of R6 VocabularyEntryRelation class instances
#'
#'                              \code{myDDI4_VocabularyRelationStructure$hasMemberRelation} returns a list of of R6 : VocabularyEntryRelation class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_VocabularyRelationStructure$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, criteria=NA, displayLabel=NA, usage=NA, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_VocabularyRelationStructure <- R6Class(classname="DDI4_VocabularyRelationStructure",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, criteria=NA, displayLabel=NA, usage=NA, hasRelationSpecification=NA, semantic=NA, totality=NA, hasMemberRelation=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_VocabularyRelationStructure may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(criteria)){
        private$.criteria <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=criteria , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(usage)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=usage , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasRelationSpecification)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=hasRelationSpecification , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(semantic)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=semantic , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(totality)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=totality , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasMemberRelation)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_VocabularyEntryRelation", proposedObjectList=hasMemberRelation , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 7 properties: 
    criteria = function(val){
      if (!missing(val)){
        private$.criteria <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.criteria
      }
    },

    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },

    hasMemberRelation = function(val){
      if (!missing(val)){
        private$.hasMemberRelation <- returnR6ObjectIfValid(validClass="DDI4_VocabularyEntryRelation", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasMemberRelation
      }
    },

    hasRelationSpecification = function(val){
      if (!missing(val)){
        private$.hasRelationSpecification <- returnEnumeratedValues(enumerationObject=DDI4_RelationSpecification$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.hasRelationSpecification
      }
    },

    semantic = function(val){
      if (!missing(val)){
        private$.semantic <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.semantic
      }
    },

    totality = function(val){
      if (!missing(val)){
        private$.totality <- returnEnumeratedValues(enumerationObject=DDI4_TotalityType$new(registerThisObject=FALSE), proposedCharacterVector=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.totality
      }
    },

    usage = function(val){
      if (!missing(val)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.usage
      }
    },


# There are 0 class association(s):
    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .criteria = NULL,
    .displayLabel = NULL,
    .usage = NULL,
    .hasRelationSpecification = NULL,
    .semantic = NULL,
    .totality = NULL,
    .hasMemberRelation = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        criteria = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasMemberRelation = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_VocabularyEntryRelation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasRelationSpecification = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RelationSpecification", 
          structure = "Enumeration vector"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        semantic = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        totality = c(
          min="0", 
          max="1", 
          datatype = "DDI4_TotalityType", 
          structure = "Enumeration vector"
        ),
        usage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_VocabularyRelationStructure


#' @title DDI4_WebLink  A web site (normally a URL) with information on type of site, privacy flag, and effective dates
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/WebLink}{WebLink} 

#'
#'   Definition
#'
#'   ============
#'
#'   A web site (normally a URL) with information on type of site, privacy flag, and effective dates.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   a:URLType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_WebLink$DdiUrn} returns the DDI URN of myDDI4_WebLink.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{isPreferred}}{ A read/write active binding.
#'        Set to "true" if this is the preferred URL.
#'
#'                              \code{myDDI4_WebLink$isPreferred <- mylogical} sets the isPreferred property of 'myDDI4_WebLink' to the vector of datatype:logical
#'
#'                              \code{myDDI4_WebLink$isPreferred} returns a vector of datatype: logical}
#'   }
#'   \describe{
#'     \item{\code{uri}}{ A read/write active binding.
#'        Normally a URL
#'
#'                              \code{myDDI4_WebLink$uri <- mycharacter} sets the uri property of 'myDDI4_WebLink' to the vector of datatype:character
#'
#'                              \code{myDDI4_WebLink$uri} returns a vector of datatype: character}
#'   }
#'   \describe{
#'     \item{\code{typeOfWebsite}}{ A read/write active binding.
#'        The type of URL for example personal, project, organization, division, etc.
#'
#'                              \code{myDDI4_WebLink$typeOfWebsite <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfWebsite' property of 'myDDI4_WebLink' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_WebLink$typeOfWebsite} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{effectiveDates}}{ A read/write active binding.
#'        The period for which this URL is valid.
#'
#'                              \code{myDDI4_WebLink$effectiveDates <- list(myDateRange)} sets the 'effectiveDates' property of 'myDDI4_WebLink' to a list of R6 DateRange class instances
#'
#'                              \code{myDDI4_WebLink$effectiveDates} returns a list of of R6 : DateRange class instances}
#'   }
#'   \describe{
#'     \item{\code{privacy}}{ A read/write active binding.
#'        Indicates the privacy level of this URL
#'
#'                              \code{myDDI4_WebLink$privacy <- list(myExternalControlledVocabularyEntry)} sets the 'privacy' property of 'myDDI4_WebLink' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_WebLink$privacy} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_WebLink$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, isPreferred=NA, uri=NA, typeOfWebsite=NA, effectiveDates=NA, privacy=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_WebLink <- R6Class(classname="DDI4_WebLink",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, isPreferred=NA, uri=NA, typeOfWebsite=NA, effectiveDates=NA, privacy=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_WebLink may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(isPreferred)){
        private$.isPreferred <- returnVectorIfValid(validVectorClass="logical", proposedVector=isPreferred , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(uri)){
        private$.uri <- returnVectorIfValid(validVectorClass="character", proposedVector=uri , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(typeOfWebsite)){
        private$.typeOfWebsite <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfWebsite , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(effectiveDates)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=effectiveDates , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(privacy)){
        private$.privacy <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=privacy , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 5 properties: 
    effectiveDates = function(val){
      if (!missing(val)){
        private$.effectiveDates <- returnR6ObjectIfValid(validClass="DDI4_DateRange", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.effectiveDates
      }
    },

    isPreferred = function(val){
      if (!missing(val)){
        private$.isPreferred <- returnVectorIfValid(validVectorClass="logical", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.isPreferred
      }
    },

    privacy = function(val){
      if (!missing(val)){
        private$.privacy <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.privacy
      }
    },

    typeOfWebsite = function(val){
      if (!missing(val)){
        private$.typeOfWebsite <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.typeOfWebsite
      }
    },

    uri = function(val){
      if (!missing(val)){
        private$.uri <- returnVectorIfValid(validVectorClass="character", proposedVector=val, minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.uri
      }
    }


  ),

  private = list(
    .isPreferred = NULL,
    .uri = NULL,
    .typeOfWebsite = NULL,
    .effectiveDates = NULL,
    .privacy = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        effectiveDates = c(
          min="0", 
          max="1", 
          datatype = "DDI4_DateRange", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        isPreferred = c(
          min="0", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        privacy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        typeOfWebsite = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        uri = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_WebLink


#' @title DDI4_WorkflowControlStep  A subtype of WorkflowStep which controls the ordering of Workflow Steps within a Process
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/WorkflowControlStep}{WorkflowControlStep} 

#'
#'   Definition
#'
#'   ============
#'
#'   A subtype of WorkflowStep which controls the ordering of Workflow Steps within a Process. Abstract base for specialized ordering conditions that have specific requirements to relay ordering information.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A WorkflowStepSequence is a subtype of WorkflowControlStep which defines a sequential order for included steps.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_WorkflowStep}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{executes}}{ A read/write active binding.
#'          The WorkflowSteps executed by the control mechanism
#'
#'                              \code{myDDI4_WorkflowControlStep$executes <- c(myWorkflowStepURN)} sets the association 'executes' from 'myDDI4_WorkflowControlStep' to a character vector of DdiUrns for R6 WorkflowStep class instances
#'
#'                              \code{myDDI4_WorkflowControlStep$executes} returns a character vector of of DdiUrns for R6  WorkflowStep class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_WorkflowControlStep$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, , ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_WorkflowControlStep <- R6Class(classname="DDI4_WorkflowControlStep",
  inherit = DDI4_WorkflowStep,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, executes=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_WorkflowControlStep may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)



       if(!missing(executes)){
        private$.executes <- returnUrnVectorIfValid(validClass="DDI4_WorkflowStep", proposedUrnVector=executes , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.executes, referencingAssociationName="executes",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 class association(s):
    executes = function(val){
      if (!missing(val)){
        private$.executes <- returnUrnVectorIfValid(validClass="DDI4_WorkflowStep", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.executes, referencingAssociationName="executes",  referencingUrn=thisUrn)  
       # invisible( private$.executes)
      } else {
          private$.executes
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .executes = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        executes = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_WorkflowStep", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInformationFlow = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPerformedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowService", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_WorkflowControlStep


#' @title DDI4_WorkflowProcess  A Workflow Process is a realization of Process which identifies the Workflow Step Sequence which contains the WorkflowSteps and their order
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/WorkflowProcess}{WorkflowProcess} 

#'
#'   Definition
#'
#'   ============
#'
#'   A Workflow Process is a realization of Process which identifies the Workflow Step Sequence which contains the WorkflowSteps and their order. It may identify the algorithm that it implements.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   Overall description of steps taken when ingesting a dataset into an archive; A Sampling Process; A step or sub-step of the Generic Longitudinal Business Process Model (GLBPM).
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_WorkflowProcess$DdiUrn} returns the DDI URN of myDDI4_WorkflowProcess.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{overview}}{ A read/write active binding.
#'        Provides a high level overview or summary of the class. Can be used to inform end-users or as part of an executive summary. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_WorkflowProcess$overview <- list(myInternationalStructuredString)} sets the 'overview' property of 'myDDI4_WorkflowProcess' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_WorkflowProcess$overview} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        Whether the collection is a bag or a set: a bag is a collection with duplicates allowed, a set is a collection without duplicates
#'
#'                              \code{myDDI4_WorkflowProcess$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_WorkflowProcess' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_WorkflowProcess$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_WorkflowProcess$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_WorkflowProcess' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_WorkflowProcess$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{usage}}{ A read/write active binding.
#'        Explanation of the ways in which the pattern is employed. Supports  the use of multiple languages and  structured text.  
#'
#'                              \code{myDDI4_WorkflowProcess$usage <- list(myInternationalStructuredString)} sets the 'usage' property of 'myDDI4_WorkflowProcess' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_WorkflowProcess$usage} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{implementsAlgorithm}}{ A read/write active binding.
#'          The algorithms implemented by the process. May be expressed by AlgorithmOverview or any subclass
#'
#'                              \code{myDDI4_WorkflowProcess$implementsAlgorithm <- c(myAlgorithmOverviewURN)} sets the association 'implementsAlgorithm' from 'myDDI4_WorkflowProcess' to a character vector of DdiUrns for R6 AlgorithmOverview class instances
#'
#'                              \code{myDDI4_WorkflowProcess$implementsAlgorithm} returns a character vector of of DdiUrns for R6  AlgorithmOverview class instances}
#'   }
#'   \describe{
#'     \item{\code{isSpecifiedBy}}{ A read/write active binding.
#'          Points to the WorkflowControlStep subtype which defines the starting point for initiating the set of process steps contained in the process.
#'
#'                              \code{myDDI4_WorkflowProcess$isSpecifiedBy <- c(myWorkflowControlStepURN)} sets the association 'isSpecifiedBy' from 'myDDI4_WorkflowProcess' to a character vector of DdiUrns for R6 WorkflowControlStep class instances
#'
#'                              \code{myDDI4_WorkflowProcess$isSpecifiedBy} returns a character vector of of DdiUrns for R6  WorkflowControlStep class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_WorkflowProcess$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, overview=NA, name=NA, purpose=NA, usage=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_WorkflowProcess <- R6Class(classname="DDI4_WorkflowProcess",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, overview=NA, name=NA, purpose=NA, usage=NA,    implementsAlgorithm=NA,isSpecifiedBy=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_WorkflowProcess may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(overview)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=overview , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(usage)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=usage , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(implementsAlgorithm)){
        private$.implementsAlgorithm <- returnUrnVectorIfValid(validClass="DDI4_AlgorithmOverview", proposedUrnVector=implementsAlgorithm , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.implementsAlgorithm, referencingAssociationName="implementsAlgorithm",  referencingUrn=thisUrn)  
      }
      if(!missing(isSpecifiedBy)){
        private$.isSpecifiedBy <- returnUrnVectorIfValid(validClass="DDI4_WorkflowControlStep", proposedUrnVector=isSpecifiedBy , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isSpecifiedBy, referencingAssociationName="isSpecifiedBy",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 4 properties: 
    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    overview = function(val){
      if (!missing(val)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overview
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    usage = function(val){
      if (!missing(val)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.usage
      }
    },


# There are 2 class association(s):
    implementsAlgorithm = function(val){
      if (!missing(val)){
        private$.implementsAlgorithm <- returnUrnVectorIfValid(validClass="DDI4_AlgorithmOverview", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.implementsAlgorithm, referencingAssociationName="implementsAlgorithm",  referencingUrn=thisUrn)  
       # invisible( private$.implementsAlgorithm)
      } else {
          private$.implementsAlgorithm
      }
    },

    isSpecifiedBy = function(val){
      if (!missing(val)){
        private$.isSpecifiedBy <- returnUrnVectorIfValid(validClass="DDI4_WorkflowControlStep", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isSpecifiedBy, referencingAssociationName="isSpecifiedBy",  referencingUrn=thisUrn)  
       # invisible( private$.isSpecifiedBy)
      } else {
          private$.isSpecifiedBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .overview = NULL,
    .name = NULL,
    .purpose = NULL,
    .usage = NULL,
    .implementsAlgorithm = NULL,
    .isSpecifiedBy = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        implementsAlgorithm = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_AlgorithmOverview", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isSpecifiedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowControlStep", 
          structure = "DdiUrn vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usage = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_WorkflowProcess


#' @title DDI4_WorkflowService  A means of performing a Workflow Step as part of a concrete implementation of a Business Function  (an ability that an organization possesses, typically expressed in general and high level terms and requiring a combination of organization, people, processes and technology to achieve)
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/WorkflowService}{WorkflowService} 

#'
#'   Definition
#'
#'   ============
#'
#'   A means of performing a Workflow Step as part of a concrete implementation of a Business Function  (an ability that an organization possesses, typically expressed in general and high level terms and requiring a combination of organization, people, processes and technology to achieve).
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A specification for an air flow monitor used to capture a measure for an ImplementedMeasure (a subtype of Act).
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_WorkflowService$DdiUrn} returns the DDI URN of myDDI4_WorkflowService.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{serviceInterface}}{ A read/write active binding.
#'        Specifies how to communicate with the service.
#'
#'                              \code{myDDI4_WorkflowService$serviceInterface <- list(myExternalControlledVocabularyEntry)} sets the 'serviceInterface' property of 'myDDI4_WorkflowService' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_WorkflowService$serviceInterface} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{serviceLocation}}{ A read/write active binding.
#'        Specifies where the service can be accessed.
#'
#'                              \code{myDDI4_WorkflowService$serviceLocation <- list(myExternalControlledVocabularyEntry)} sets the 'serviceLocation' property of 'myDDI4_WorkflowService' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_WorkflowService$serviceLocation} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{estimatedDuration}}{ A read/write active binding.
#'        The estimated time period associated with the operation of the Service. This may be expressed as a time, date-time, or duration.
#'
#'                              \code{myDDI4_WorkflowService$estimatedDuration <- list(myDate)} sets the 'estimatedDuration' property of 'myDDI4_WorkflowService' to a list of R6 Date class instances
#'
#'                              \code{myDDI4_WorkflowService$estimatedDuration} returns a list of of R6 : Date class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{hasAgent}}{ A read/write active binding.
#'          Actor that performs a role in the service,
#'
#'                              \code{myDDI4_WorkflowService$hasAgent <- c(myAgentURN)} sets the association 'hasAgent' from 'myDDI4_WorkflowService' to a character vector of DdiUrns for R6 Agent class instances
#'
#'                              \code{myDDI4_WorkflowService$hasAgent} returns a character vector of of DdiUrns for R6  Agent class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_WorkflowService$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, serviceInterface=NA, serviceLocation=NA, estimatedDuration=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_WorkflowService <- R6Class(classname="DDI4_WorkflowService",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, serviceInterface=NA, serviceLocation=NA, estimatedDuration=NA,    hasAgent=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_WorkflowService may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(serviceInterface)){
        private$.serviceInterface <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=serviceInterface , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(serviceLocation)){
        private$.serviceLocation <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=serviceLocation , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(estimatedDuration)){
        private$.estimatedDuration <- returnR6ObjectIfValid(validClass="DDI4_Date", proposedObjectList=estimatedDuration , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(hasAgent)){
        private$.hasAgent <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=hasAgent , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasAgent, referencingAssociationName="hasAgent",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 3 properties: 
    estimatedDuration = function(val){
      if (!missing(val)){
        private$.estimatedDuration <- returnR6ObjectIfValid(validClass="DDI4_Date", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.estimatedDuration
      }
    },

    serviceInterface = function(val){
      if (!missing(val)){
        private$.serviceInterface <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.serviceInterface
      }
    },

    serviceLocation = function(val){
      if (!missing(val)){
        private$.serviceLocation <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.serviceLocation
      }
    },


# There are 1 class association(s):
    hasAgent = function(val){
      if (!missing(val)){
        private$.hasAgent <- returnUrnVectorIfValid(validClass="DDI4_Agent", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasAgent, referencingAssociationName="hasAgent",  referencingUrn=thisUrn)  
       # invisible( private$.hasAgent)
      } else {
          private$.hasAgent
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .serviceInterface = NULL,
    .serviceLocation = NULL,
    .estimatedDuration = NULL,
    .hasAgent = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        estimatedDuration = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Date", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAgent = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Agent", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        serviceInterface = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        serviceLocation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_WorkflowService


#' @title DDI4_WorkflowStep  One of the constituents of a Workflow
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/WorkflowStep}{WorkflowStep} 

#'
#'   Definition
#'
#'   ============
#'
#'   One of the constituents of a Workflow. It can be a composition or atomic and might be performed by a Service.
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A ControlConstruct that orchestrates steps and substeps in a Workflow is a WorkflowStep. An Act is a WorkflowStep.
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   An atomic Workflow Step has no Control Constructs -- it's an Act. A composition consists of a tree of Control Constructs. In this tree Acts are associated with the leaf nodes.
#'
#'   
#'
#'   Furthermore, a composition might be a glass box or a black box from the perspective of a service. If a service performs a WorkflowStep, it might invoke just the topmost ControlConstruct -- for example a WorkflowSequence. It may know nothing about the internal workings of the sequence. In this case the WorkflowStep is a black box.
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   Process Step
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_WorkflowStep$DdiUrn} returns the DDI URN of myDDI4_WorkflowStep.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{displayLabel}}{ A read/write active binding.
#'        A structured display label providing a fully human readable display for the identification of the object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_WorkflowStep$displayLabel <- list(myLabelForDisplay)} sets the 'displayLabel' property of 'myDDI4_WorkflowStep' to a list of R6 LabelForDisplay class instances
#'
#'                              \code{myDDI4_WorkflowStep$displayLabel} returns a list of of R6 : LabelForDisplay class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage. 
#'
#'                              \code{myDDI4_WorkflowStep$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_WorkflowStep' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_WorkflowStep$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_WorkflowStep$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_WorkflowStep' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_WorkflowStep$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{usage}}{ A read/write active binding.
#'        Explanation of the ways in which some decision or object is employed. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_WorkflowStep$usage <- list(myInternationalStructuredString)} sets the 'usage' property of 'myDDI4_WorkflowStep' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_WorkflowStep$usage} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{overview}}{ A read/write active binding.
#'        Short natural language account of the information obtained from the combination of properties and relationships associated with an object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_WorkflowStep$overview <- list(myInternationalStructuredString)} sets the 'overview' property of 'myDDI4_WorkflowStep' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_WorkflowStep$overview} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#'   \describe{
#'     \item{\code{hasInformationFlow}}{ A read/write active binding.
#'        Mapping of specific information flow within the workflow
#'
#'                              \code{myDDI4_WorkflowStep$hasInformationFlow <- list(myBinding)} sets the 'hasInformationFlow' property of 'myDDI4_WorkflowStep' to a list of R6 Binding class instances
#'
#'                              \code{myDDI4_WorkflowStep$hasInformationFlow} returns a list of of R6 : Binding class instances}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{isPerformedBy}}{ A read/write active binding.
#'          Identifies the Service Implementation which performs the Workflow Step. Specialization of isPerformedBy in Process Step for Service Implementations.
#'
#'                              \code{myDDI4_WorkflowStep$isPerformedBy <- c(myWorkflowServiceURN)} sets the association 'isPerformedBy' from 'myDDI4_WorkflowStep' to a character vector of DdiUrns for R6 WorkflowService class instances
#'
#'                              \code{myDDI4_WorkflowStep$isPerformedBy} returns a character vector of of DdiUrns for R6  WorkflowService class instances}
#'   }
#'   \describe{
#'     \item{\code{hasInputParameter}}{ A read/write active binding.
#'          An identification for a Parameter related to the WorkflowStep and referred to by a Binding indicating the Input to the WorkflowStep 
#'
#'                              \code{myDDI4_WorkflowStep$hasInputParameter <- c(myParameterURN)} sets the association 'hasInputParameter' from 'myDDI4_WorkflowStep' to a character vector of DdiUrns for R6 Parameter class instances
#'
#'                              \code{myDDI4_WorkflowStep$hasInputParameter} returns a character vector of of DdiUrns for R6  Parameter class instances}
#'   }
#'   \describe{
#'     \item{\code{hasOutputParameter}}{ A read/write active binding.
#'          An identification for a Parameter related to the WorkflowStep and referred to by a Binding indicating the Output from the WorkflowStep
#'
#'                              \code{myDDI4_WorkflowStep$hasOutputParameter <- c(myParameterURN)} sets the association 'hasOutputParameter' from 'myDDI4_WorkflowStep' to a character vector of DdiUrns for R6 Parameter class instances
#'
#'                              \code{myDDI4_WorkflowStep$hasOutputParameter} returns a character vector of of DdiUrns for R6  Parameter class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_WorkflowStep$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, displayLabel=NA, name=NA, purpose=NA, usage=NA, overview=NA, hasInformationFlow=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_WorkflowStep <- R6Class(classname="DDI4_WorkflowStep",
  inherit = DDI4_AnnotatedIdentifiable,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, displayLabel=NA, name=NA, purpose=NA, usage=NA, overview=NA, hasInformationFlow=NA,    isPerformedBy=NA,hasInputParameter=NA,hasOutputParameter=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_WorkflowStep may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(displayLabel)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=displayLabel , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(usage)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=usage , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(overview)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=overview , minimumCardinality=0, maximumCardinality=1)  
      }
      if(!missing(hasInformationFlow)){
        private$.hasInformationFlow <- returnR6ObjectIfValid(validClass="DDI4_Binding", proposedObjectList=hasInformationFlow , minimumCardinality=0, maximumCardinality=Inf)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(isPerformedBy)){
        private$.isPerformedBy <- returnUrnVectorIfValid(validClass="DDI4_WorkflowService", proposedUrnVector=isPerformedBy , minimumCardinality=0, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isPerformedBy, referencingAssociationName="isPerformedBy",  referencingUrn=thisUrn)  
      }
      if(!missing(hasInputParameter)){
        private$.hasInputParameter <- returnUrnVectorIfValid(validClass="DDI4_Parameter", proposedUrnVector=hasInputParameter , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasInputParameter, referencingAssociationName="hasInputParameter",  referencingUrn=thisUrn)  
      }
      if(!missing(hasOutputParameter)){
        private$.hasOutputParameter <- returnUrnVectorIfValid(validClass="DDI4_Parameter", proposedUrnVector=hasOutputParameter , minimumCardinality=0, maximumCardinality=Inf, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasOutputParameter, referencingAssociationName="hasOutputParameter",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 6 properties: 
    displayLabel = function(val){
      if (!missing(val)){
        private$.displayLabel <- returnR6ObjectIfValid(validClass="DDI4_LabelForDisplay", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.displayLabel
      }
    },

    hasInformationFlow = function(val){
      if (!missing(val)){
        private$.hasInformationFlow <- returnR6ObjectIfValid(validClass="DDI4_Binding", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.hasInformationFlow
      }
    },

    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    overview = function(val){
      if (!missing(val)){
        private$.overview <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.overview
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    usage = function(val){
      if (!missing(val)){
        private$.usage <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.usage
      }
    },


# There are 3 class association(s):
    hasInputParameter = function(val){
      if (!missing(val)){
        private$.hasInputParameter <- returnUrnVectorIfValid(validClass="DDI4_Parameter", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasInputParameter, referencingAssociationName="hasInputParameter",  referencingUrn=thisUrn)  
       # invisible( private$.hasInputParameter)
      } else {
          private$.hasInputParameter
      }
    },

    hasOutputParameter = function(val){
      if (!missing(val)){
        private$.hasOutputParameter <- returnUrnVectorIfValid(validClass="DDI4_Parameter", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=Inf, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.hasOutputParameter, referencingAssociationName="hasOutputParameter",  referencingUrn=thisUrn)  
       # invisible( private$.hasOutputParameter)
      } else {
          private$.hasOutputParameter
      }
    },

    isPerformedBy = function(val){
      if (!missing(val)){
        private$.isPerformedBy <- returnUrnVectorIfValid(validClass="DDI4_WorkflowService", proposedUrnVector=val , minimumCardinality=0, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.isPerformedBy, referencingAssociationName="isPerformedBy",  referencingUrn=thisUrn)  
       # invisible( private$.isPerformedBy)
      } else {
          private$.isPerformedBy
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .displayLabel = NULL,
    .name = NULL,
    .purpose = NULL,
    .usage = NULL,
    .overview = NULL,
    .hasInformationFlow = NULL,
    .isPerformedBy = NULL,
    .hasInputParameter = NULL,
    .hasOutputParameter = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInformationFlow = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPerformedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowService", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        usage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = TRUE
  )
)  # ends class definition for DDI4_WorkflowStep


#' @title DDI4_WorkflowStepOrder  Associates an sequence ordering number with any WorflowStep or sub-type,
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/WorkflowStepOrder}{WorkflowStepOrder} 

#'
#'   Definition
#'
#'   ============
#'
#'   Associates an sequence ordering number with any WorflowStep or sub-type,
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_WorkflowStepOrder$DdiUrn} returns the DDI URN of myDDI4_WorkflowStepOrder.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{index}}{ A read/write active binding.
#'        Index value of WorkflowStep in an ordered array
#'
#'                              \code{myDDI4_WorkflowStepOrder$index <- myinteger} sets the index property of 'myDDI4_WorkflowStepOrder' to the vector of datatype:integer
#'
#'                              \code{myDDI4_WorkflowStepOrder$index} returns a vector of datatype: integer}
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Relationships:}{
#'   \describe{
#'     \item{\code{indexedWorkflowStep}}{ A read/write active binding.
#'          The subtype of a WorkflowStep occuring at the indexed position within the array. 
#'
#'                              \code{myDDI4_WorkflowStepOrder$indexedWorkflowStep <- c(myWorkflowStepURN)} sets the association 'indexedWorkflowStep' from 'myDDI4_WorkflowStepOrder' to a character vector of DdiUrns for R6 WorkflowStep class instances
#'
#'                              \code{myDDI4_WorkflowStepOrder$indexedWorkflowStep} returns a character vector of of DdiUrns for R6  WorkflowStep class instances}
#'   }
#' }


#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_WorkflowStepOrder$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, index=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_WorkflowStepOrder <- R6Class(classname="DDI4_WorkflowStepOrder",
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, index=NA,    indexedWorkflowStep=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_WorkflowStepOrder may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

      if(!missing(index)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=index , minimumCardinality=1, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


 

       if(!missing(indexedWorkflowStep)){
        private$.indexedWorkflowStep <- returnUrnVectorIfValid(validClass="DDI4_WorkflowStep", proposedUrnVector=indexedWorkflowStep , minimumCardinality=1, maximumCardinality=1, silent=silentState)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.indexedWorkflowStep, referencingAssociationName="indexedWorkflowStep",  referencingUrn=thisUrn)  
      }
    }
  ),

  active = list(

# There are 1 properties: 
    index = function(val){
      if (!missing(val)){
        private$.index <- returnVectorIfValid(validVectorClass="integer", proposedVector=val, minimumCardinality=1, maximumCardinality=1)  
      } else {
        private$.index
      }
    },


# There are 1 class association(s):
    indexedWorkflowStep = function(val){
      if (!missing(val)){
        private$.indexedWorkflowStep <- returnUrnVectorIfValid(validClass="DDI4_WorkflowStep", proposedUrnVector=val , minimumCardinality=1, maximumCardinality=1, silent= private$isSilent)  
        thisUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")
        result <- addToReferencedUrn(referencedUrn=private$.indexedWorkflowStep, referencingAssociationName="indexedWorkflowStep",  referencingUrn=thisUrn)  
       # invisible( private$.indexedWorkflowStep)
      } else {
          private$.indexedWorkflowStep
      }
    }

  ),

  private = list(
    .index = NULL,
    .indexedWorkflowStep = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        index = c(
          min="1", 
          max="1", 
          datatype = "integer", 
          structure = "vector"
        ),
        indexedWorkflowStep = c(
          min="1", 
          max="1", 
          datatype = "DDI4_WorkflowStep", 
          structure = "DdiUrn vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("structuredDatatype"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_WorkflowStepOrder


#' @title DDI4_WorkflowStepSequence  A WorkflowStepSequence controls the order of WorkflowSteps by defining a simple sequence
#' 
#' @docType class 
#' @import R6    
#                R6Class 
#' @keywords content 
#' @return Content carrying object of class \code{\link{R6Class}}. 
#' @format \code{\link{R6Class}} object. 
#' @description
#'   Implementation of the DDI4 class \href{https://lion.ddialliance.org/ddiobjects/WorkflowStepSequence}{WorkflowStepSequence} 

#'
#'   Definition
#'
#'   ============
#'
#'   A WorkflowStepSequence controls the order of WorkflowSteps by defining a simple sequence.
#'
#'   
#'
#'   WorkflowSteps that may be used in a WorkflowStepSequence may be of many subtypes, covering simple or computational acts, conditional steps such as an IfThenElse, temporally defined ordering controls such as Split and SplitJoin, or others designed for various specializations of the WorkflowProcess. .
#'
#'   
#'
#'   WorkflowSteps in a WorkflowStepSequence exchange variables and values by way of output parameters, input parameters and their bindings.
#'
#'   
#'
#'   
#'
#'   Examples
#'
#'   ==========
#'
#'   A WorkflowStepSequence can be used to describe the flow of questions in a questionnaire. 
#'
#'   
#'
#'   A WorkflowStepSequence can be used to decompose a BusinessProcess like data integration or anonymization into a series of steps.
#'
#'   
#'
#'   
#'
#'   
#'
#'   Explanatory notes
#'
#'   ===================
#'
#'   If more complex temporal or graph ordering is required use the subtype StructuredWorkflowSteps.
#'
#'   The relationship "executes" in this instance of a WorkflowControlStep is an optional means of noting the set of WorkflowSteps managed by the sequence. Sequencing is done using orderedStep. This is currently an anomoly that should be resolved in future discussion application of transformation rules from UML model. 
#'
#'   
#'
#'   
#'
#'   Synonyms
#'
#'   ==========
#'
#'   
#'
#'   
#'
#'   DDI 3.2 mapping
#'
#'   =================
#'
#'   d:SequenceType
#'
#'   
#'
#'   RDF mapping
#'
#'   =============
#'
#'   
#'
#'   
#'
#'   GSIM mapping
#'
#'   ==============
#'
#'   
#'
#'   									


#' @section Ancestry:
#' Inherits from (direct parent listed first):
#'  \itemize{
#'    \item{\code{\link{DDI4_WorkflowControlStep}}}
#'    \item{\code{\link{DDI4_WorkflowStep}}}
#'    \item{\code{\link{DDI4_AnnotatedIdentifiable}}}
#'    \item{\code{\link{DDI4_Identifiable}}}
#'  }
#'
#' @section Active Bindings:
#'   Active Bindings to get or set properties or associations. Only values of the proper type and number are allowed.

#' \subsection{Common Active Bindings:}{
#'   \describe{
#'     \item{\code{DdiUrn}}{ read only - \code{myDDI4_WorkflowStepSequence$DdiUrn} returns the DDI URN of myDDI4_WorkflowStepSequence.} 
#'   }
#' }

#' \subsection{Active Bindings for DDI4 Properties:}{
#'   \describe{
#'     \item{\code{typeOfWorkflowStepSequence}}{ A read/write active binding.
#'        Provides the ability to "type" a sequence for classification or processing purposes. Supports the use of an external controlled vocabulary.
#'
#'                              \code{myDDI4_WorkflowStepSequence$typeOfWorkflowStepSequence <- list(myExternalControlledVocabularyEntry)} sets the 'typeOfWorkflowStepSequence' property of 'myDDI4_WorkflowStepSequence' to a list of R6 ExternalControlledVocabularyEntry class instances
#'
#'                              \code{myDDI4_WorkflowStepSequence$typeOfWorkflowStepSequence} returns a list of of R6 : ExternalControlledVocabularyEntry class instances}
#'   }
#'   \describe{
#'     \item{\code{orderedStep}}{ A read/write active binding.
#'        The Workflow Steps included in the Collection with an index for position within ordered array. 
#'
#'                              \code{myDDI4_WorkflowStepSequence$orderedStep <- list(myWorkflowStepOrder)} sets the 'orderedStep' property of 'myDDI4_WorkflowStepSequence' to a list of R6 WorkflowStepOrder class instances
#'
#'                              \code{myDDI4_WorkflowStepSequence$orderedStep} returns a list of of R6 : WorkflowStepOrder class instances}
#'   }
#'   \describe{
#'     \item{\code{name}}{ A read/write active binding.
#'        A linguistic signifier. Human understandable name (word, phrase, or mnemonic) that reflects the ISO/IEC 11179-5 naming principles. If more than one name is provided provide a context to differentiate usage.
#'
#'                              \code{myDDI4_WorkflowStepSequence$name <- list(myObjectName)} sets the 'name' property of 'myDDI4_WorkflowStepSequence' to a list of R6 ObjectName class instances
#'
#'                              \code{myDDI4_WorkflowStepSequence$name} returns a list of of R6 : ObjectName class instances}
#'   }
#'   \describe{
#'     \item{\code{purpose}}{ A read/write active binding.
#'        Explanation of the intent of some decision or object. Supports the use of multiple languages and structured text.
#'
#'                              \code{myDDI4_WorkflowStepSequence$purpose <- list(myInternationalStructuredString)} sets the 'purpose' property of 'myDDI4_WorkflowStepSequence' to a list of R6 InternationalStructuredString class instances
#'
#'                              \code{myDDI4_WorkflowStepSequence$purpose} returns a list of of R6 : InternationalStructuredString class instances}
#'   }
#' }
#' \subsection{Other methods:}{
#'   \describe{
#'       \item{\code{describe_initializeArguments()}}{This returns a named list of character vectors. Each describes one argument of the initialize function (i.e. an argument of DDI4_WorkflowStepSequence$new()). A DDI 4 property or association is the name of a description vector. The description vector contains two elements, the cardinality of the initialize argument, and the datatype of that argument. An argument value is a vector or a list. The cardinality shows the minimum and maximum number of values in that vector or list. }

#'       \item{\code{setSilent()}}{This sets the object to silent mode. Validation notes will not be displayed These include messages about the object not being registered, or not found through the XML Catalog. Silent mode is useful when importing a set of objects that may not be in any particular order }

#'       \item{\code{setNotSilent()}}{This sets the object to verbose mode. Validation notes will be displayed These include messages about the object not being registered, or not found through the XML Catalog. Verbose mode might be useful when writing scripts to create objects where the order may be controlled. }

#'       \item{\code{getIsSilent()}}{This returns the current silent or verbose mode (silent is TRUE or FALSE.}

#'       \item{\code{get_ClassGroup()}}{This returns the type of class: enumeration, regex, identifiable, or structuredDatatype.}

#'       \item{\code{print()}}{prints the object with a layout specific to a DDI 4 R6 object. }

#'       \item{\code{new(silentState=FALSE, typeOfWorkflowStepSequence=NA, orderedStep=NA, name=NA, purpose=NA, ...)}}{The \code{new()} function creates an object of this class. All classes that inherit from the "Identifiable" class also have arguments of "agency", "id", and "version", that together comprise a globally unique identifier. Defaults for these will be generated if they are not entered. The resulting global identifier can be retrieved with the \code{get_DdiUrn()} function.}
#'    }
#' }


#' @export  
DDI4_WorkflowStepSequence <- R6Class(classname="DDI4_WorkflowStepSequence",
  inherit = DDI4_WorkflowControlStep,
  public = list(
    print = function(...) {
      cat(ddiObjectFormat(self), sep="\n") 
    },	

    describe_initializeArguments = function(){
      private$.initializeFields
    },

    setSilent = function(){
      private$isSilent <- TRUE
    },

    setNotSilent = function(){
      private$isSilent <- FALSE
    },

   getIsSilent = function(){
      private$isSilent
    },
    get_ClassGroup = function(){
      private$classGroup 
    },


    initialize = function(registerThisObject=TRUE, silentState=FALSE, typeOfWorkflowStepSequence=NA, orderedStep=NA, name=NA, purpose=NA, ...){
      if (private$isAbstract){
        stop("The abstract class DDI4_WorkflowStepSequence may not be instantiated")
      }

      if(!missing(silentState)){
        private$isSilent=silentState
      }

        #  pass through the register and silent parameters
      super$initialize(registerThisObject=registerThisObject, silentState=silentState, ...)

      if(!missing(typeOfWorkflowStepSequence)){
        private$.typeOfWorkflowStepSequence <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=typeOfWorkflowStepSequence , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(orderedStep)){
        private$.orderedStep <- returnR6ObjectIfValid(validClass="DDI4_WorkflowStepOrder", proposedObjectList=orderedStep , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(name)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=name , minimumCardinality=0, maximumCardinality=Inf)  
      }
      if(!missing(purpose)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=purpose , minimumCardinality=0, maximumCardinality=1)  
      }

      private$.DdiUrn <- paste("urn","ddi",private$.agency,private$.id,private$.version,sep=":")


     }
  ),

  active = list(

# There are 4 properties: 
    name = function(val){
      if (!missing(val)){
        private$.name <- returnR6ObjectIfValid(validClass="DDI4_ObjectName", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.name
      }
    },

    orderedStep = function(val){
      if (!missing(val)){
        private$.orderedStep <- returnR6ObjectIfValid(validClass="DDI4_WorkflowStepOrder", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.orderedStep
      }
    },

    purpose = function(val){
      if (!missing(val)){
        private$.purpose <- returnR6ObjectIfValid(validClass="DDI4_InternationalStructuredString", proposedObjectList=val , minimumCardinality=0, maximumCardinality=1)  
      } else {
        private$.purpose
      }
    },

    typeOfWorkflowStepSequence = function(val){
      if (!missing(val)){
        private$.typeOfWorkflowStepSequence <- returnR6ObjectIfValid(validClass="DDI4_ExternalControlledVocabularyEntry", proposedObjectList=val , minimumCardinality=0, maximumCardinality=Inf)  
      } else {
        private$.typeOfWorkflowStepSequence
      }
    },

    DdiUrn = function(val){
      if (missing(val)){
        private$.DdiUrn  
      } else {
          stop(paste('DdiUrn is read only, to change a DDI identifier, assign the value of agency, id, or version independently.'))
      }
    }

  ),

  private = list(
    .typeOfWorkflowStepSequence = NULL,
    .orderedStep = NULL,
    .name = NULL,
    .purpose = NULL,

    .DdiUrn = NULL,

    .initializeFields = list(
        agency = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        basedOnObject = c(
          min="0", 
          max="1", 
          datatype = "DDI4_BasedOnObjectInformation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        displayLabel = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LabelForDisplay", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        executes = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_WorkflowStep", 
          structure = "DdiUrn vector"
        ),
        hasAnnotation = c(
          min="0", 
          max="1", 
          datatype = "DDI4_Annotation", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInformationFlow = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Binding", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasInputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        hasOutputParameter = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_Parameter", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        id = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        isPerformedBy = c(
          min="0", 
          max="1", 
          datatype = "DDI4_WorkflowService", 
          structure = "DdiUrn vector"
        ),
        isPersistent = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        isUniversallyUnique = c(
          min="1", 
          max="1", 
          datatype = "logical", 
          structure = "vector"
        ),
        localId = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_LocalIdFormat", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        name = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ObjectName", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        orderedStep = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_WorkflowStepOrder", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        overview = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        purpose = c(
          min="0", 
          max="1", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        relatedMaterial = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalMaterial", 
          structure = "DdiUrn vector"
        ),
        typeOfWorkflowStepSequence = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_ExternalControlledVocabularyEntry", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        usage = c(
          min="0", 
          max="Inf", 
          datatype = "DDI4_InternationalStructuredString", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        version = c(
          min="1", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        ),
        versionDate = c(
          min="0", 
          max="1", 
          datatype = "DDI4_IsoDateType", 
          structure = "Regex vector"
        ),
        versionRationale = c(
          min="0", 
          max="1", 
          datatype = "DDI4_RationaleDefinition", 
          isAbstractDatatype = FALSE,
          structure = "StructuredDatatype list"
        ),
        versionResponsibility = c(
          min="0", 
          max="1", 
          datatype = "character", 
          structure = "vector"
        )
    ),

     isSilent = FALSE,
     classGroup = c("identifiable"),

     isAbstract = FALSE
  )
)  # ends class definition for DDI4_WorkflowStepSequence
